// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap;

import com.bbn.openmap.Layer;
//import com.bbn.openmap.LayerHandler;
import fr.unicaen.openmap.layer.dataDisplayLayer.SimpleDataDisplayLayer;
import java.util.HashMap;
import java.util.Properties;

/**
 *LayerHandler used only for the atlas mode
 * @author jchauvea
 */
public class AtlasLayerHandler extends LayerHandler {

    protected HashMap<Layer, LayerNameAndProps> loadableLayers = new HashMap<Layer, LayerNameAndProps>();

    public void addLoadableLayer(Layer layer, String name, Properties props) {
        loadableLayers.put(layer, new LayerNameAndProps(name, props));
    }


    public void loadLayer(Layer l){
	// System.out.println("loadLayer "+loadableLayers.get(l).getName() + " with: "+ loadableLayers.get(l).getProperties());
        l.setProperties(loadableLayers.get(l).getName(), loadableLayers.get(l).getProperties());
        l.setVisible(true);      
        addLayer(l);
        setLayers();

    }

    public void unloadLayer(Layer l){
        removeLayer(l);
    }

    @Override
	public void removeLayer(Layer l){
	super.removeLayer(l);
	if(l instanceof SimpleDataDisplayLayer)
	    ((SimpleDataDisplayLayer)l).purge();
    }

    @Override
	public boolean turnLayerOn(boolean b, Layer l){
        if(b)
            loadLayer(l);
        return true;
    }


    class LayerNameAndProps{
        private String name;
        private String prettyName;	
        private Properties properties;

	public LayerNameAndProps(String n, Properties p){
	    this.name = n;
	    this.properties = p;
	    this.prettyName = p.getProperty(name + ";" + Layer.PrettyNameProperty);
	}

	public String getName(){
	    return name;
	}

	public String getPrettyName(){
	    return prettyName;
	}

	public Properties getProperties(){
	    return properties;
	}
    }


}
