// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>

package fr.unicaen.openmap;

import com.bbn.openmap.Layer;
import fr.unicaen.openmap.layer.legendLayer.LegendLayer;
import fr.unicaen.openmap.dataStorage.AnimationDSEventManager;
import fr.unicaen.openmap.layer.dataDisplayLayer.SimpleDataDisplayLayer;

/**
 * This LayerHandler:
 * makes LegendLayer to appear only as a top layer
 *warns the MetaAnimator that a layer is removed
 * @author jchauvea
 */
public class LayerHandler extends com.bbn.openmap.LayerHandler {

    private boolean legended = false;

    @Override
    public void init(Layer[] layers) {
        for (Layer l : layers) {
            if (l instanceof LegendLayer) {
                legended = true;
            }
        }
        super.init(layers);
    }

    @Override
    public void addLayer(Layer layer, int position) {
        if(legended && position == 0)
            position = 1;
        super.addLayer(layer, position);
        if (layer instanceof LegendLayer) {
            legended = true;
        }
    }

    // only need to override removeLayer(Layer[] currentLayers, int index) as it is the only one that does the job
    @Override
    protected void removeLayer(Layer[] currentLayers, int index) {
        Layer rLayer = currentLayers[index]; 
	// System.out.println("== A remove layer "+ rLayer);

	if (rLayer instanceof LegendLayer) 
	    legended = false;
	super.removeLayer(currentLayers, index);
	// As it appear pushed event has no effect, 
	// I warn directly any present MetaAnimator through the AnimationDSEventManager
	if (rLayer instanceof SimpleDataDisplayLayer) 
	    AnimationDSEventManager.fireRemoveLayer((SimpleDataDisplayLayer)rLayer);
   }

    public boolean isLegended() {
        return legended;
    }
}
