// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.animation;

import com.bbn.openmap.Layer;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.animation.gui.AnimationDSChainPanel;
import fr.unicaen.openmap.dataStorage.MonoValuedFilterDataStorage;
import fr.unicaen.openmap.dataStorage.AnimationDataStorageListener;
import fr.unicaen.openmap.dataStorage.DataStorageIteratorAdapter;
import fr.unicaen.openmap.dataStorage.SortDataStorage;
import fr.unicaen.openmap.dataStorage.gui.AdaptorButtonArray;
import fr.unicaen.openmap.layer.dataDisplayLayer.SimpleDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.util.ValueComparatorFactory;
import java.awt.event.ActionEvent;
import java.lang.reflect.InvocationTargetException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Set;
import java.util.TreeSet;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.Timer;

/**
 *
 * @author jchauvea
 */
public class AnimationDSChain implements AnimationDataStorageListener, AnimationDSChainListener {

    private ArrayList<MonoValuedFilterDataStorage> animationDatastorages;
    private AnimationDSChainPanel gui;
    private String _values[];
    private SortDataStorage.Format sortingFormat = SortDataStorage.Format.STRING;
    private boolean ascSort = true;
    protected java.util.Comparator<String> comparator; 
    private MetaAnimator metaAnimator;
    private Timer timer;
    private AbstractAction nextValueAction;

    private int currentIndex = 0;
    private String name;
    private AnimationDSChain slave;
    private AnimationDSChain master;
    private HashMap<String, String> animationsToAdd = new HashMap<String, String>();
    public static String listAnimationsPropertyString = "listAnimations";
    public static String layersList = "layersList";
    public static String delayPropertyString = "delay";
    public static String isSlavePropertyString = "isSlave";

    boolean mustDoNextStep = true; // used when is master

    public AnimationDSChain(MetaAnimator ma, String name) {
        this.metaAnimator = ma;
        this.name = name;
        animationDatastorages = new ArrayList<MonoValuedFilterDataStorage>();

        // Action triggered by the timer or the next buttons
        nextValueAction = new AbstractAction() {

		public void actionPerformed(ActionEvent e) {
		    nextValue();
		}
	    };
        timer = new Timer(0, getNextValueAction());
    }

    private void resetAnimationDatastorages() {
	for (MonoValuedFilterDataStorage ads : animationDatastorages) {
	    ads.setFilterColumnValue(MonoValuedFilterDataStorage.noFilter);
	}	
	currentIndex = getValues() == null || getValues().length == 0 ? -1 
	    : getIndexOfValue(MonoValuedFilterDataStorage.noFilter);
	getMetaAnimator().refreshLegendLayer();
	if (gui != null)
	    gui.updateSelectedValue(MonoValuedFilterDataStorage.noFilter);
    }

    public void addAnimationDataStorage(MonoValuedFilterDataStorage ads) {
	Debug.message("Animation", "** AnimationDSchain "+this+" addAnimationDataStorage 0 "+ads);
        animationDatastorages.add(ads);
        metaAnimator.fireAnimationDSSelected(this, ads);

	refreshValues();
        if (gui != null) {
	    Debug.message("Animation", "** AnimationDSchain "+this+" refresh gui ");
            gui.refresh();
	}
    }

    public void removeAnimationDataStorage(MonoValuedFilterDataStorage ads) {
        animationDatastorages.remove(ads);
        metaAnimator.fireAnimationDSUnselected(this, ads);
	// <removeButton>
	fireRefreshRemoveButtons();
	// </removeButton>
	refreshValues();
        if (gui != null)
            gui.refresh();

    }

    /**
     * @return the animationDatastorages
     */
    public ArrayList<MonoValuedFilterDataStorage> getAnimationDatastorages() {
        return animationDatastorages;
    }

    public boolean containsAnimationDatastorage(MonoValuedFilterDataStorage ads) {
	if (animationDatastorages.contains(ads) )
	    return true;
	Debug.message("Animation", "[AnimationDSchain] searching for "+ads+ " in animationsToAdd: "+animationsToAdd);
	return isAnimationToAdd(ads);
    }

    public AnimationDSChainPanel getGUI() {
        if (gui == null) {
            gui = new AnimationDSChainPanel(this);
	    gui.sort();
        }
        return gui;
    }

    public JComponent getToolGUI() {
	return getGUI().getToolGUI();
    }

    public String[] getValues() { return _values; }
    protected String getValueAt(int i) { return _values[i]; }
    protected int getIndexOfValue(String v) {
	// dichotomic search
	int low = 0;
	int hight = _values.length - 1;
	while (low < hight) {
	    int m = (low + hight) / 2;
	    if (comparator.compare(_values[m], v) < 0)
		low = m + 1;
	    else 
		hight = m;
	}
	return comparator.compare(_values[hight], v) == 0 ? hight : -1; 
    }

    public static boolean isValid(MonoValuedFilterDataStorage ads) {
	return ads.getFilterColumnName() != null
	    && ! ads.getFilterColumnName().equals("")
	    && ads.getKeyValues() != null;
    }

    void refreshValues() {
	// Logger.getLogger("ValueComparator").log(Level.INFO, "[AnimationDSChain] REFRESH VALUES");
	comparator =  ValueComparatorFactory.createComparator(sortingFormat, ascSort);
	TreeSet<String> values = new TreeSet<String>(comparator);

	// find the first non null ads
	MonoValuedFilterDataStorage ads0 = null;
        for (MonoValuedFilterDataStorage ads : animationDatastorages) {
	    if (ads.getKeyValues() == null) continue;
	    ads0 = ads;
	    break;
	}
	
	if (ads0 != null) {	    
	    // compute the intersection of key sets
	    for (String val : ads0.getKeyValues()) {	
		// // skip noFilter
		// if (MonoValuedFilterDataStorage.noFilter.equals(val)) continue;
		// check if val is contained in all the other filters key set
		boolean toAdd = true;
		for (MonoValuedFilterDataStorage ads : animationDatastorages) {
		    if (ads == ads0) continue;
		    if (ads.getKeyValues() == null ) continue;
		    if (ads.getIndexOfKeyValue(val) < 0) {
			toAdd = false;
			break;
		    }
		}
		if (toAdd) 
		    values.add(val);
	    }
	}

	_values = values.toArray(new String[values.size()]);
        
        // we should use an event...
        getToolGUI().setVisible(true);
    	// Logger.getLogger("ValueComparator").log(Level.INFO, "[AnimationDSChain] VALUES SIZE " + getValues().length);
    }

    public void sortValues() {
	if (_values == null)
	    return;
	comparator =  ValueComparatorFactory.createComparator(sortingFormat, ascSort);
	TreeSet<String> values = new TreeSet<String>(comparator);
	for (String v : _values)
	    values.add(v);
	_values =  values.toArray(new String[values.size()]);
	// sync filterColumnValue
	resetAnimationDatastorages();
    }

    public void animationRemoved(MonoValuedFilterDataStorage ads) {
        animationDatastorages.remove(ads);
	refreshValues();
        if (gui != null) {
            gui.refresh();
            gui.removeAnimationFromGUI(ads);
        }
	// <removeButton>
	fireRefreshRemoveButtons();
	// </removeButton>
        if (animationDatastorages.isEmpty() && metaAnimator.getNbChain() > 1) {
            metaAnimator.removeChain(this);
        }
    }

    public void animationAdded(MonoValuedFilterDataStorage ads) {
        Debug.message("Animation", "=== AnimationDSChain.animationAdded("+ads + " to gui @" + (gui == null ? 0 : gui.hashCode()));
        if (gui != null)
            gui.addAnimationGUI(ads);
    }

    public void animationChanged(MonoValuedFilterDataStorage ads) {
	refreshValues();
        if (gui != null)
            gui.refresh();
    }

    /**
     * @return the metaAnimator
     */
    public MetaAnimator getMetaAnimator() {
        return metaAnimator;
    }

    /**
     * @return the delay
     */
    public int getDelay() {
        return timer.getDelay() / 1000;
    }

    /**
     * @param delay the delay to set
     */
    public void setDelay(int delay) {
        timer.setDelay(delay * 1000);
    }

    public void rewind() {
	currentIndex = 0; 
    }

    public void start() {
        // rewind();
        if (isSlave()) {
	    System.err.println("[Animation] can't start a slave");
            return;
	}
        timer.start();
    }

    public void stop() {
        timer.stop();
    }

    /** 
     * perform a nextStep taking slave into account
     * return false when at end of iteration
     */
    public boolean nextValue() {
	if (getValues() == null ||  getValues().length == 0)
	    return false;

	if (getSlave() == null)
	    return nextStep();
	
	// we have a slave
	if (mustDoNextStep || MonoValuedFilterDataStorage.noFilter.equals(getValueAt(currentIndex)))
	    nextStep();
	
	turnCascadeIcon();
	mustDoNextStep = ! getSlave().nextValue();
	if (gui != null)
	    gui.refresh();
	
	// we are at end if: mustDoNextStep && currentIndex == getValues().length - 1;
	return ! mustDoNextStep || currentIndex < getValues().length - 1;
    }

    private boolean nextStep() {
	if (++currentIndex >= getValues().length) 
	    currentIndex = 0;
		    
	String val = getValueAt(currentIndex);
	if (MonoValuedFilterDataStorage.noFilter.equals(val)) {
	    if (++currentIndex >= getValues().length) 
		currentIndex = 0;
	    val = getValueAt(currentIndex);
	}
	//	System.out.println("**Animation nextStep "+AnimationDSChain.this + " " +currentIndex+ " val: "+val);
	if (gui != null)
	    gui.updateSelectedValue(val);
	nextStep(val);

	if (currentIndex == getValues().length - 1)
	    return false;  // loop(export) is finished
	return true;
    }
    
    /** used by the manual selection of an item from a combo box of values */
    public void nextStep(String value) {
	currentIndex = getIndexOfValue(value);
	//System.out.println("NEXT STEP------------>" + this + " "+ currentIndex + " val:" + value);
	// forward to every filter of the chain
        for (MonoValuedFilterDataStorage ads : animationDatastorages)
            ads.setFilterColumnValue(value);
	getMetaAnimator().refreshLegendLayer();
    }

    public String getCurrentValue() {
	if (_values == null || _values.length == 0 || currentIndex < 0 || currentIndex >= _values.length)
	    return null;
	return getValueAt(currentIndex);
    }
    
    /**
     * @return the nextValueAction
     */
    public AbstractAction getNextValueAction() {
	return nextValueAction;
    }

    public void delete() {
	for (int i = 0; i < animationDatastorages.size(); i++)
	    removeAnimationDataStorage(animationDatastorages.get(i));
	metaAnimator.removeChain(this);
    }

    public String getName() {
	return this.name;
    }

    public MonoValuedFilterDataStorage addAnimationToLayer(Layer l) {
	if (l == null)
	    return null;
	try {
	    ThematicDataDisplayLayer layer = null;
	    try {
		layer = (ThematicDataDisplayLayer) l;
		//System.out.println("addAnimationToLayer "+ l + "," +layer.getDataStorageIterator().getDataStorage());
		MonoValuedFilterDataStorage ds = (MonoValuedFilterDataStorage) AdaptorButtonArray.getDataStorageConstructor(MonoValuedFilterDataStorage.class.getName()).newInstance(
																						     new Object[]{l, layer.getDataStorageIterator().getDataStorage()});
		layer.setDataStorageIterator(new DataStorageIteratorAdapter(ds));
		layer.dataStorageEvent(ds);
		return ds;
	    } catch (ClassCastException e) {
		JOptionPane.showMessageDialog(gui, "Impossible to add an animation to " + l.getName(), "Bad layer", JOptionPane.ERROR_MESSAGE);
	    }
	} catch (InstantiationException ex) {
	    Logger.getLogger(AnimationDSChain.class.getName()).log(Level.SEVERE, null, ex);
	} catch (IllegalAccessException ex) {
	    Logger.getLogger(AnimationDSChain.class.getName()).log(Level.SEVERE, null, ex);
	} catch (IllegalArgumentException ex) {
	    Logger.getLogger(AnimationDSChain.class.getName()).log(Level.SEVERE, null, ex);
	} catch (InvocationTargetException ex) {
	    Logger.getLogger(AnimationDSChain.class.getName()).log(Level.SEVERE, null, ex);
	    System.err.println("Caused by: ");
	    ex.getCause().printStackTrace();
	}
	return null;
    }

    public int getSize() {
	return animationDatastorages.size();
    }

    public void animationDSSelected(AnimationDSChain source, MonoValuedFilterDataStorage ads) {
	resetAnimationDatastorages();
	if (gui != null) {
	    if (this != source)
		gui.removeAnimationFromGUI(ads);
	    gui.updateSelectedValue(MonoValuedFilterDataStorage.noFilter);
	}
    }

    public void animationDSUnselected(AnimationDSChain source, MonoValuedFilterDataStorage ads) {
	if (gui != null && this != source)
	    gui.addAnimationGUI(ads);
    }

    /**
     * @return the slave
     */
    public AnimationDSChain getSlave() {
	return slave;
    }

    /**
     * @param slave the slave to set
     */
    public void setSlave(AnimationDSChain slave) {
	this.slave = slave;
    }

    /**
     * @return the master
     */
    public AnimationDSChain getMaster() {
	return master;
    }

    /**
     * @param master the master to set
     */
    public void setMaster(AnimationDSChain master) {
	this.master = master;
	// Don't update gui as we may not have all the info yet...
	if (gui != null)
	    gui.setEnabledParametersGUI(true);
    }

    public boolean isSlave() {
	return master != null;
    }

    public boolean isRunning() {
	return timer.isRunning();
    }

    public void setProperties(String prefix, Properties props) {
	prefix = PropUtils.getScopedPropertyPrefix(prefix);
	timer.setDelay(Integer.parseInt(props.getProperty(prefix + delayPropertyString, "0")));
	String layersString = props.getProperty(prefix + layersList, "");
	animationsToAdd.clear();
	if (layersString == null)
	    return;
	String[] layers = layersString.split(" ");
	for (String l : layers) {
	    if (l.equals(""))
		continue;
	    String ranks = props.getProperty(prefix + l, "");
	    // System.out.println("##AnimationDSChain setProperties "+l +" "+props.getProperty(prefix + l, ranks)+" to "+this);
	    animationsToAdd.put(l, props.getProperty(prefix + l, ranks));
	}
	Debug.message("Animation", "[AnimationDSChain] setProperties, animationsToAdd= "+animationsToAdd+" =");
    }

    public void addAnimationsToAddAndSetToolGUIVisibility() {
	if (addAnimationsToAdd()) {
	    setToolGuiVisibleIfNecessary();
	}
    }

    protected boolean addAnimationsToAdd() {
	if (animationsToAdd.isEmpty())
	    return true;
	for (String l : animationsToAdd.keySet()) {
	    String[] ranks = animationsToAdd.get(l).split(" ");
	    for (String r : ranks) {
		try {
		    MonoValuedFilterDataStorage anim = metaAnimator.findAnimationDataStorage(l, Integer.parseInt(r));
		    Debug.message("Animation", "[AnimationDSChain]  adding "+ l + "." + r + " : "+ anim);
		    if (anim == null)
			return false;
		    addAnimationDataStorage(anim);
		} catch (NumberFormatException ex) {
		    System.err.println("***Error adding filter to anim chain " + l + " " + r);
		    return false;
		}
	    }
	}
	animationsToAdd.clear();
	return true;
    }

    protected boolean isAnimationToAdd(MonoValuedFilterDataStorage ads) {
	if (animationsToAdd.isEmpty())
	    return false;
	for (String l : animationsToAdd.keySet()) {
	    String[] ranks = animationsToAdd.get(l).split(" ");
	    for (String r : ranks) {
		try {
		    MonoValuedFilterDataStorage anim = metaAnimator.findAnimationDataStorage(l, Integer.parseInt(r));
		    Debug.message("Animation", "[AnimationDSChain]  testing "+ l + "." + r + " : "+ anim);
		    if (anim == null)
			continue;
		    
		    if (anim == ads) {
			return true;
		    }
		} catch (NumberFormatException ex) {
		    System.err.println("***Error  testing anim chain " + l + " " + r);
		    return false;
		}
	    }
	}
	return false;
    }

    protected void setToolGuiVisibleIfNecessary() {
	Debug.message("Animation", "** " + this + "setToolVisible " + (animationDatastorages.size() > 0) + "  && "+(_values != null));
	getToolGUI().setVisible(animationDatastorages.size() > 0 && _values != null);
    }

    public Properties getProperties(String prefix, Properties props) {
	prefix = PropUtils.getScopedPropertyPrefix(prefix);
	HashMap<String, String> animationPerLayers = new HashMap<String, String>();
	props.put(prefix + delayPropertyString, String.valueOf(timer.getDelay()));
	// Warning: isSlave property is put in AnimationDSChain and used in MetaAnimator
	props.put(prefix + isSlavePropertyString, String.valueOf(isSlave()));
	for (MonoValuedFilterDataStorage ads : animationDatastorages) {
	    ThematicDataDisplayLayer layer = ads.getLayer();
	    // get & put in 2 steps to avoid conflict:
	    String indexes = animationPerLayers.get(layer.getName());
	    if (indexes == null)
		indexes = "";
	    else
		indexes += " ";
	    indexes += String.valueOf(layer.getDataStorageIterator().getDataStorage().getAnimationDataStorageList().indexOf(ads));
	    animationPerLayers.put(layer.getName(), indexes);
	}
	String animatedLayers = "";
	for (String key : animationPerLayers.keySet()) {
	    String ranks = animationPerLayers.get(key);
	    if (ranks == null)
		continue;
	    animatedLayers += " " + key;
	    props.put(prefix + key, ranks);
	}
	props.put(prefix + layersList, animatedLayers);
	
	return props;
    }

    public String getVariablesValuesAsString() {
	StringBuffer ret = new StringBuffer();
	for (MonoValuedFilterDataStorage filterDS : animationDatastorages) {
	    ret.append(filterDS.getVariableValueAsString());
	    ret.append("_");
	}
	return ret.toString();
    }

    public ArrayList<String> getVariablesValuesAsArrayOfString(String sep) {
	ArrayList<String> ret = new ArrayList<String>();
	for (MonoValuedFilterDataStorage filterDS : animationDatastorages)
	    ret.add(filterDS.getVariableValueAsString(sep));
	return ret;
    }

    public SortDataStorage.Format getSortingFormat() {
	return sortingFormat;
    }

    public void setSortingFormat(SortDataStorage.Format f) {
	sortingFormat = f;
    }

    public boolean getAcSort() {
	return ascSort;
    }

    public void setAscSort(boolean a) {
	ascSort = a;
    }

    // <removeButton>
    public void fireRefreshRemoveButtons() {
    	metaAnimator.fireRefreshRemoveButtons();
    }
    
    public void refreshRemoveButtons() {
    	if (gui != null)
    	    gui.refreshRemoveButtons();
    }
    // </removeButton>

    private AnimationDSChainCascade cascade = null;
    void setCascade(AnimationDSChainCascade cascade) {
	this.cascade = cascade;
    }
    
    void turnCascadeIcon() {
	if (cascade != null)
	    cascade.turnIcon();
    }

    //todo: revoir le test
    public boolean isReady() {
        return getValues() != null;
    }
}
