// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.animation;

import fr.unicaen.openmap.*;
import com.bbn.openmap.Layer;
import com.bbn.openmap.gui.OMToolComponent;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.dataStorage.AnimationDSEventManager;
import fr.unicaen.openmap.dataStorage.DataStorage;
import fr.unicaen.openmap.dataStorage.DataStorageIterator;
import fr.unicaen.openmap.dataStorage.MonoValuedFilterDataStorage;
import fr.unicaen.openmap.dataStorage.AnimationDataStorageListener;
import fr.unicaen.openmap.gui.ThemaMapFileChooserManager;
import fr.unicaen.openmap.layer.dataDisplayLayer.SimpleDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.legendLayer.LegendLayer;
import fr.unicaen.openmap.util.ImageExporter;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.SwingUtilities;

/**
 * Tool to manage Animations.
 *
 * @author jchauvea
 */
public class MetaAnimator extends OMToolComponent implements AnimationDataStorageListener {
    
    private JComponent gui;
    private Box chainBoxList;
    private List<MonoValuedFilterDataStorage> availableDSList;
    private List<MonoValuedFilterDataStorage> usedDSList;
    private LayerHandler layerHanlder;
    private JButton bShortCut;
    private JButton bNewChain;
    private JPanel toolGUI;
    private JTextField fieldPath;
    private int nbChain = 0;
    private List<AnimationDSChain> chains;
    private JFrame dialog;
    private JPanel toolBar;
    private ArrayList<AnimationDSChainListener> listeners;
    private ArrayList<AnimationDSChainCascade> cascades;
    public static String nbChainsPropertyString = "nbChains";
    public static String chainPropertyString = "chain";
    private boolean typePNG = true;
    
    public void setTypePNG(boolean b) {
        typePNG = b;
    }
    
    public boolean getTypePNG() {
        return typePNG;
    }
    
    public MetaAnimator() {
        super();
	init();
    }
    
    public void init() {
        listeners = new ArrayList<AnimationDSChainListener>();
        availableDSList = new ArrayList<MonoValuedFilterDataStorage>();
        usedDSList = new ArrayList<MonoValuedFilterDataStorage>();
        cascades = new ArrayList<AnimationDSChainCascade>();
        chains = new ArrayList<AnimationDSChain>();
        bShortCut = new JButton(Util.loadIcon("Movie24.png"));
        AnimationDSEventManager.addAnimationDataStorageListener(this);
        bShortCut.addActionListener(new ActionListener() {
            
            public void actionPerformed(ActionEvent e) {
                popup();
            }
        });
        ComponentSizer.fixComponentSize(bShortCut, new Dimension(28, 28));
        bShortCut.setBackground(Color.GRAY);
        bShortCut.setOpaque(false);
	initToolGUI();
        bNewChain = new JButton(Util.loadIcon("newAnim.png"));
        bNewChain.addActionListener(new ActionListener() {
            
            public void actionPerformed(ActionEvent arg0) {
                addChainGUI(initChain(), getNbChain());
            }
        });
        bNewChain.setToolTipText(i18n.get(this, "addAnimation", "Add an animation"));
        ComponentSizer.fixComponentSize(bNewChain, new Dimension(35, 35));
        bNewChain.setBackground(Color.GRAY);
        bNewChain.setOpaque(false);
        bNewChain.setVisible(false);
    }

    public void initToolGUI() {
	if (toolGUI == null)
	    toolGUI = new JPanel();
	toolGUI.removeAll();
        toolGUI.add(bShortCut);
        toolGUI.setBackground(Color.GRAY);
        toolGUI.setOpaque(false);
    }

    public void popup() {
        if (dialog == null) {
            dialog = new JFrame(i18n.get(this, "metaAnimator", "Meta Animator"));
            dialog.getContentPane().add(getGUI());
            dialog.pack();
            dialog.setSize(700, 600);
        }
        dialog.setLocationRelativeTo(bShortCut);
        dialog.setVisible(true);
        dialog.toFront();
        bNewChain.setVisible(true);
        bNewChain.requestFocusInWindow();
    }
    
    @Override
    public void findAndInit(Object someObj) {
        if (someObj instanceof LayerHandler) {
            layerHanlder = (LayerHandler) someObj;
        }
    }

    /**
     * Collect all animations (AnimationDataStorage) by browsing layerhandler's layers
     * @return the number of collected animations
     */
    private int collectAnimationDataStorages() {
        availableDSList.clear();
        List<MonoValuedFilterDataStorage> tmpList = new ArrayList<MonoValuedFilterDataStorage>();
        for (Layer l : layerHanlder.getLayers()) {
            if (l instanceof SimpleDataDisplayLayer) {
                SimpleDataDisplayLayer sddl = (SimpleDataDisplayLayer) l;
                
                tmpList.addAll(sddl.getDataStorageIterator().getDataStorage().getAnimationDataStorageList());
            }
        }
        for (MonoValuedFilterDataStorage ads : tmpList) {
            if (!availableDSList.contains(ads) && !usedDSList.contains(ads)) {
                availableDSList.add(ads);
            }
        }
        return availableDSList.size();
    }
    
    public Component getGUI() {
        if (gui == null) {
            initGUI();
        }
        return gui;
    }
    
    private void initGUI() {
        gui = new JPanel(new BorderLayout());
	reInitGUI();
    }
    
    public void reInitGUI() {   
	if (gui == null) {
	    initGUI();
	    return;
	}

	gui.removeAll();
        collectAnimationDataStorages();
        JPanel wrap = new JPanel(new BorderLayout());
        gui.add(wrap, BorderLayout.NORTH);
        toolBar = new JPanel(new BorderLayout());
        Box box = Box.createVerticalBox();
        final JPanel wrapAnimExport = new JPanel();
        wrapAnimExport.add(bNewChain);
        toolBar.add(wrapAnimExport, BorderLayout.WEST);
        //ComponentSizer.fixComponentSize(wrapAnimExport, new Dimension(40, 40));
        final JToggleButton exportButton = new JToggleButton(Util.loadIcon("Export16.png"));
        wrapAnimExport.add(exportButton);
        
        
        final JPanel panelExport = new JPanel();
        fieldPath = new JTextField();
        fieldPath.setEditable(false);
        JButton bBrowse = new JButton("browse");
        
        Box boxTypePNG = Box.createVerticalBox();
        ButtonGroup imageTypeButtonGroup = new ButtonGroup();
        final JRadioButton bPNG = new JRadioButton("PNG");
        bPNG.setSelected(true);
        JRadioButton bSVG = new JRadioButton("SVG");
        imageTypeButtonGroup.add(bPNG);
        imageTypeButtonGroup.add(bSVG);
        ItemListener pngSvgListener = new ItemListener() {
            
            public void itemStateChanged(ItemEvent arg0) {
                setTypePNG(bPNG.isSelected());
            }
        };
        bPNG.addItemListener(pngSvgListener);
        bSVG.addItemListener(pngSvgListener);
        ComponentSizer.fixComponentSize(bPNG, new Dimension(16 * 4, 16));
        ComponentSizer.fixComponentSize(bSVG, new Dimension(16 * 4, 16));
        boxTypePNG.add(bPNG);
        boxTypePNG.add(bSVG);
        panelExport.add(boxTypePNG);
        
        JButton bExport = new JButton("Export");
        exportButton.setBackground(Color.GRAY);
        exportButton.setOpaque(false);
        ComponentSizer.fixComponentSize(exportButton, new Dimension(35, 35));
        fieldPath.setText(Environment.get(ThemaMapFileChooserManager.DATA_SAVE_PATH,
                System.getProperty("user.dir")));
        bBrowse.addActionListener(new ActionListener() {
            
            public void actionPerformed(ActionEvent ae) {
                ThemaMapFileChooserManager.dataLoadFileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                fieldPath.setText(ThemaMapFileChooserManager.dataLoadDialog("", null)
                        + System.getProperty("file.separator"));
            }
        });
        
        bExport.addActionListener(new ActionListener() {
            
            public void actionPerformed(ActionEvent arg0) {
                SwingUtilities.invokeLater(new Runnable() {
                    
                    public void run() {
                        new Exporter(fieldPath.getText(),
                                getTypePNG() ? ImageExporter.ImageType.PNG
                                : ImageExporter.ImageType.SVG).run();
                    }
                });
            }
        });
        
        panelExport.add(new JLabel(i18n.get(this, "exportDir", "Export Dir:")));
        panelExport.add(fieldPath);
        panelExport.add(bBrowse);
        panelExport.add(bExport);
        exportButton.addActionListener(new ActionListener() {
            
            public void actionPerformed(ActionEvent arg0) {
                panelExport.setVisible(exportButton.isSelected());
                //wrapAnimExport.updateUI();
            }
        });
        wrapAnimExport.add(panelExport);
        panelExport.setVisible(false);
        
        toolBar.setBorder(BorderFactory.createMatteBorder(1, 1, 0, 1, Color.gray));
        wrap.add(toolBar, BorderLayout.NORTH);
        wrap.add(new JScrollPane(box), BorderLayout.CENTER);
        
        chainBoxList = Box.createVerticalBox();
        box.add(chainBoxList);
        for (AnimationDSChain ch : chains) {
            ch.addAnimationsToAdd();
        }
        initChainsGUI();
    }
    
    synchronized public void fireAnimationDSSelected(AnimationDSChain source, MonoValuedFilterDataStorage ads) {
	if (getUsedDSList().contains(ads)) {
	    Debug.message("Animation", "--++ "+ads+" already selected "+"Available : " + getAvailableDSList()+" Used :" + getUsedDSList());
	    //  no return here we must also forward the event
	} else {
	    getUsedDSList().add(ads);
	    getAvailableDSList().remove(ads);
	}
        for (AnimationDSChainListener adscl : listeners) {
            adscl.animationDSSelected(source, ads);
        }
	Debug.message("Animation" , "++ "+ads+" selected "+"Available : " + getAvailableDSList()+" Used :" + getUsedDSList());
	//	if (Debug.debugging("Animation")) { try{ throw(new Exception("selected ")); } catch(Exception e) { e.printStackTrace(); }}
    }
    
    synchronized public void fireAnimationDSUnselected(AnimationDSChain source, MonoValuedFilterDataStorage ads) {
	if (getAvailableDSList().contains(ads)) {
	    Debug.message("Animation", "---- "+ads+" already unselected\n"+"Available : " + getAvailableDSList()+"\nUsed :" + getUsedDSList());
	    return;
	}
	getAvailableDSList().add(ads);
	getUsedDSList().remove(ads); 
	for (AnimationDSChainListener adscl : listeners) {
            adscl.animationDSUnselected(source, ads);
        }
	Debug.message("Animation", "-- "+ads+" unselected "+"Available : " + getAvailableDSList()+" Used :" + getUsedDSList());
	//	if (Debug.debugging("Animation")) { try{ throw(new Exception("unselected but still alive")); } catch(Exception e) { e.printStackTrace(); }}
    }
    
    @Override
    public Container getFace() {
        initChainsGUI();
        return toolGUI;
    }
    
    @Override
    public boolean getUseAsTool() {
        return true;
    }
    
    private AnimationDSChain initChain() {
        AnimationDSChain newChain = new AnimationDSChain(this, i18n.get(MetaAnimator.class, "animation", "Animation ") + (getNbChain() + 1));
        chains.add(newChain);
        listeners.add(newChain);
        return newChain;
    }
    
    public void initChainsGUI() {
        //toolGUI.removeAll();
        if (gui == null) {
            initGUI();
        }
        int i = 1;
        // System.out.println("chains....:" + chains.size());
        for (AnimationDSChain chain : chains) {
            addChainGUI(chain, i++);
        }
    }
    
    private void addChainGUI(AnimationDSChain chain, int index) {
	Debug.message("Animation", "=== MetaAnimator.addChainGUI " + index);
        if (index > 1) {
            AnimationDSChainCascade cascade =
                    new AnimationDSChainCascade(chains.get(index - 2), chains.get(index - 1));
            chainBoxList.add(cascade.getGUI());
            cascades.add(cascade);
            // System.out.println("=== MetaAnimator.addChainGUI " + cascade);
        }
        JComponent chainPanel = chain.getGUI();
        chainPanel.setBorder(BorderFactory.createTitledBorder(chain.getName()));
        chainBoxList.add(chainPanel);
        JComponent cp = chain.getToolGUI();
        toolGUI.add(cp);
        cp.setVisible(chain.isReady());
        
        toolGUI.updateUI();
        
        gui.setVisible(false);
        gui.setVisible(true);
        
    }

    /**
     * @return the availableDSList
     */
    public List<MonoValuedFilterDataStorage> getAvailableDSList() {
        return availableDSList;
    }

    /**
     * @param availableDSList the availableDSList to set
     */
    public void setAvailableDSList(List<MonoValuedFilterDataStorage> availableDSList) {
        this.availableDSList = availableDSList;
    }

    /**
     * @return the usedDSList
     */
    private List<MonoValuedFilterDataStorage> getUsedDSList() {
        return usedDSList;
    }
    
    public void removeChain(AnimationDSChain chain) {
        chain.stop();
        listeners.remove(chain);
        chains.remove(chain);
        // remove the chain from the master
        if (chain.getMaster() != null) {
            chain.getMaster().setSlave(null);
        }
        // remove the chain from the slave
        if (chain.getSlave() != null) {
            chain.getSlave().setMaster(null);
        }
        // remove the GUI
        if (chainBoxList != null) {
            chainBoxList.remove(chain.getGUI());
        }

        //remove cascadePanel
        if (!chains.isEmpty() && cascades.size() > 0) {
            if (chainBoxList != null) {
                chainBoxList.remove(cascades.get(cascades.size() - 1).getGUI());
            }
        }
        toolGUI.remove(chain.getToolGUI());
        toolGUI.updateUI();
        if (chainBoxList != null) {
            chainBoxList.setVisible(false);
            chainBoxList.setVisible(true);
        }
    }
    
    public void animationRemoved(MonoValuedFilterDataStorage ads) {
        Debug.message("Animation", "[MetaAnimator].animationRemoved("+ads+") "+availableDSList);
        availableDSList.remove(ads);
        usedDSList.remove(ads);
        for (int i = 0; i < chains.size(); i++) {
            chains.get(i).animationRemoved(ads);
        }
    }
    
    public void animationAdded(MonoValuedFilterDataStorage ads) {
        Debug.message("Animation", "[MetaAnimator].animationAdded("+ads+") "+availableDSList);
        availableDSList.add(ads);
        ads.setMetaAnimator(this);
        for (AnimationDSChain adsc : chains) {
            adsc.animationAdded(ads);
        }
	Debug.message("Animation" , "[MetaAnimator] -- animationAdded "+ads+" selected  "+"Available : " + getAvailableDSList()+"  Used :" + getUsedDSList());

    }
    
    public void animationChanged(MonoValuedFilterDataStorage ads) {
        for (AnimationDSChain adsc : chains) {
            adsc.animationChanged(ads);
        }
    }
    
    public LayerHandler getLayerHandler() {
        return layerHanlder;
    }
    
    public Layer[] getLayers() {
        return layerHanlder.getLayers();
    }
    
    public LegendLayer getLegendLayer() {
        for (Layer l : getLayers()) {
            if (l instanceof LegendLayer) {
                return (LegendLayer) l;
            }
        }
        return null;
    }
    
    public void refreshLegendLayer() {
        LegendLayer l = getLegendLayer();
        if (l != null) {
            l.refreshGraphicsAndGUI();
        }
    }
    
    public int getNbChain() {
        return chains.size();
    }
    
    @Override
    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        nbChain = Integer.parseInt(properties.getProperty(prefix + nbChainsPropertyString, "1"));
        for (int i = 0; i < nbChain; i++) {
            initChain();
        }
        for (int i = 0; i < chains.size(); i++) {
            String chainPrefix = prefix + chainPropertyString + "." + String.valueOf(i) + ".";
            chains.get(i).setProperties(chainPrefix, properties);
	    // System.out.println("=== MetaAnimator chain " + i + " "+chainPrefix+"  " + properties.get(chainPrefix + AnimationDSChain.isSlavePropertyString));
            if (PropUtils.booleanFromProperties(properties, chainPrefix + AnimationDSChain.isSlavePropertyString, false)) {
                if (i == 0) {
                    Debug.error("**Error in properties: MetaAnimator 1st AnimationDSChain is slave");
                } else {
                    // System.out.println("=== MetaAnimator slave chain " + i);
                    chains.get(i - 1).setSlave(chains.get(i));
                    chains.get(i).setMaster(chains.get(i - 1));
                }
            }
        }
    }
    
    @Override
    public Properties getProperties(Properties properties) {
        properties = super.getProperties(properties);
        String prefix = PropUtils.getScopedPropertyPrefix(this);
        properties.put(prefix + nbChainsPropertyString, String.valueOf(chains.size()));
        for (int i = 0; i < chains.size(); i++) {
            String chainPrefix = prefix + chainPropertyString + "." + String.valueOf(i) + ".";
            chains.get(i).getProperties(chainPrefix, properties);
        }
        return properties;
    }

    /**
     * returns ads.getLayer().getDataStorageIterator().getDataStorage().getAnimationDataStorageList().indexOf(ads)
     * testing for null values.
     * Returns -1 in that later case
    */
    private int indexOf(MonoValuedFilterDataStorage ads) {
	DataStorageIterator dsi = ads.getLayer().getDataStorageIterator();
	if (dsi == null)
	    return -1;
	DataStorage ds = dsi.getDataStorage();
	if (ds == null)
	    return -1;
	return ds.getAnimationDataStorageList() == null ? -1 
	    : ds.getAnimationDataStorageList().indexOf(ads);
    }

    public MonoValuedFilterDataStorage findAnimationDataStorage(String layerName, int index) {
	Debug.message("Animation", "[MetaAnimator].findAnimationDataStorage("+layerName+" "+index+") in "+availableDSList+ " " + usedDSList);
        for (MonoValuedFilterDataStorage ads : availableDSList) {
	    if (Debug.debugging("Animation"))  System.out.print(" " + ads + "/" + ads.getLayer()); //
            if (ads.getLayer().getName().equals(layerName) && indexOf(ads) == index) {
		if (Debug.debugging("Animation")) System.out.println("*"); //
                return ads;
            }
        }
        for (MonoValuedFilterDataStorage ads : usedDSList) {
	    if (Debug.debugging("Animation"))  System.out.print(" " + ads + "/" + ads.getLayer()); //
            if (ads.getLayer().getName().equals(layerName) && indexOf(ads) == index) {
		if (Debug.debugging("Animation")) System.out.println("*"); //
                return ads;
            }
        }
	System.out.println("-");
        return null;
    }
    
    public String getVariablesValuesAsString() {
        StringBuffer ret = new StringBuffer();
        for (AnimationDSChain anim : chains) {
            String s = anim.getVariablesValuesAsString();
            if (s != null && s.length() > 0) {
                ret.append(s);
                ret.append("_");
            }
        }
        return ret.toString();
    }
    
    public ArrayList<String> getVariablesValuesAsArrayOfString() {
        ArrayList<String> ret = new ArrayList<String>();
        for (AnimationDSChain anim : chains) {
            ArrayList<String> s = anim.getVariablesValuesAsArrayOfString(" ");
            if (s != null && s.size() > 0) {
                ret.addAll(s);
            }
        }
        return ret;
    }
    
    
    // <removeButton>
     public void fireRefreshRemoveButtons() {
         for (AnimationDSChain chain : chains) {
             chain.refreshRemoveButtons();
         }
     }
    // </removeButton>

    public void setToolGuiVisibleIfNecessary() {
	for (AnimationDSChain chain : chains)
	    chain.addAnimationsToAddAndSetToolGUIVisibility();
    }

    class Exporter extends Thread {
        
        private String fileName = "Map-";
        private String folder;
        private ImageExporter.ImageType type;
        
        public Exporter(String folder, ImageExporter.ImageType type) {
            this.folder = folder;
            if (!folder.endsWith("/")) {
                this.folder = folder + System.getProperty("file.separator");
            }
            this.type = type;
            // rewind all the animation chains
            for (AnimationDSChain ch : chains) {
                ch.rewind();
            }
        }
        
        @Override
        public void run() {
            dialog.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            AnimationDSChain chain = chains.get(0);
	    chain.rewind();
	    boolean go = true;
            while (go) {
		go = chain.nextValue();
                String varsVals = getVariablesValuesAsString();
                varsVals = ImageExporter.removeSlashesFrom(varsVals);
                ImageExporter.exportInFile(folder + fileName + varsVals
                        + "." + type.toString().toLowerCase(), type); 
            }
            dialog.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        }
    }
}
