// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>

package fr.unicaen.openmap.animation.gui;

import com.bbn.openmap.I18n;
import com.bbn.openmap.Layer;
import com.bbn.openmap.LayerHandler;
import com.bbn.openmap.event.LayerEvent;
import com.bbn.openmap.event.LayerListener;
import com.bbn.openmap.util.Debug;

import fr.unicaen.openmap.Environment;
import fr.unicaen.openmap.animation.AnimationDSChain;
import fr.unicaen.openmap.dataStorage.gui.AdaptorButtonArray;
import fr.unicaen.openmap.dataStorage.DataStorage;
import fr.unicaen.openmap.dataStorage.MonoValuedFilterDataStorage;
import fr.unicaen.openmap.dataStorage.SortDataStorage;
import fr.unicaen.openmap.layer.dataDisplayLayer.SimpleDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


/**
 *
 * @author dje
 */
public class AnimationDSChainPanel extends JPanel implements LayerListener {

    private AnimationDSChain dsChain;
    private JPanel toolGUI;
    private JToolBar toolButtons;
    private JComboBox cbValues, cbValuesTool;
    private JSpinner spinnerDelay;
    private JToggleButton bRun, bRunTool;
    private JButton nextButton, nextButtonTool;
    private JButton bRemoveChain;
    private JButton bNewAnim;
    private JButton bShowTool;
    private JPanel pAddAnim;
    private JComboBox comboboxLayers, boxFormat;
    private ActionListener manualActionListener;
    protected I18n i18n = Environment.getI18n();
    private Box listBox;
    private JRadioButton bAsc, bDesc;
    private HashMap<MonoValuedFilterDataStorage, JComponent> animationsGUI;
    private boolean activeListener;
    // <removeButton>
    private java.util.Map<MonoValuedFilterDataStorage, JButton> removeButtonMap 
    	= new java.util.HashMap<MonoValuedFilterDataStorage, JButton>();
    // </removeButton>
    private java.util.Map<MonoValuedFilterDataStorage, JCheckBox> checkBoxMap 
	= new java.util.HashMap<MonoValuedFilterDataStorage, JCheckBox>();
    public AnimationDSChainPanel(AnimationDSChain dsc) {
        super(new BorderLayout());
        this.dsChain = dsc;
        activeListener = true;
        animationsGUI = new HashMap<MonoValuedFilterDataStorage, JComponent>();
        manualActionListener = new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if(!activeListener)
                    return;
                JComboBox source = (JComboBox) e.getSource();
                if (source.getSelectedItem() != null) {
                    String item = source.getSelectedItem().toString();
                    dsChain.nextStep(item);

		    updateSelectedValue(item); // intead of the folowing ??
                    // activeListener = false;
                    // if (source == cbValues) {
                    //     if (cbValuesTool != null)
		    // 	    cbValuesTool.setSelectedItem(item);
                    // } else {
		    // 	if (cbValues != null)
		    // 	    cbValues.setSelectedItem(item);
		    // }
                    // activeListener = true;

                }
            }
        };

        cbValues = new JComboBox();// dsc.computeUnionValues().toArray());
        cbValues.addActionListener(manualActionListener);
        ComponentSizer.fixComponentSize(cbValues, new Dimension(120, 20));

        listBox = Box.createVerticalBox();
	//System.out.println("AnimationDSChainPanel AnimationDataStorages "+dsChain.getAnimationDatastorages());
	//System.out.println("AnimationDSChainPanel AvailableDSList "+dsChain.getMetaAnimator().getAvailableDSList());

        for(MonoValuedFilterDataStorage ads : dsChain.getAnimationDatastorages())
            addAnimationGUI(ads);
        for (MonoValuedFilterDataStorage ads : dsChain.getMetaAnimator().getAvailableDSList()) 
            addAnimationGUI(ads);

        add(listBox, BorderLayout.WEST);
        listBox.setBorder(BorderFactory.createTitledBorder(i18n.get(AnimationDSChainPanel.class, "filters", "Filters")));
        Box panParameters = Box.createVerticalBox();


        ButtonGroup bg = new ButtonGroup();
        bAsc = new JRadioButton();
        bDesc = new JRadioButton();
        bg.add(bAsc);
        bg.add(bDesc);
        bAsc.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent arg0) {
                boolean asc = bAsc.isSelected();
                bAsc.setIcon(asc ? Util.loadIcon("Up16.png") : Util.loadIcon("Up16-2.png"));
                bDesc.setIcon(asc ? Util.loadIcon("Down16-2.png") : Util.loadIcon("Down16.png"));
		dsChain.setAscSort(asc);
                sort();
            }
	    });
	// no need to ass an action listener to bDesc as bAsc is always modified thanks to the button group.

        Box wrapSort = Box.createVerticalBox();
        JPanel panSort = new JPanel();
        JPanel wrapCustom = new JPanel();
        wrapSort.add(panSort);

        wrapSort.add(wrapCustom);

        boxFormat = new JComboBox(SortDataStorage.Format.values());
        boxFormat.setSelectedItem(dsChain.getSortingFormat());
        JPanel panFormat = new JPanel();
        panFormat.add(boxFormat);
        final JLabel labCustom = new JLabel(i18n.get(AnimationDSChainPanel.class, "customFormat", "Custom Format "));
        wrapCustom.add(labCustom);
        final JTextField fieldCustomFormat = new JTextField("", 15);
        wrapCustom.add(fieldCustomFormat);
        labCustom.setVisible(false);
        fieldCustomFormat.setVisible(false);
        //ComponentSizer.fixComponentSize(fieldCustomFormat, new Dimension(100, 20));
        //ComponentSizer.fixComponentSize(boxFormat, new Dimension(100, 20));
        fieldCustomFormat.addKeyListener(new KeyAdapter() {
            public void keyReleased(KeyEvent evt) {
                dsChain.getSortingFormat().setInnerFormat(fieldCustomFormat.getText());
            }
        });
        boxFormat.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
		SortDataStorage.Format selectedFormat = 
		    (SortDataStorage.Format) ((JComboBox) e.getSource()).getSelectedItem();
                dsChain.setSortingFormat(selectedFormat);
                labCustom.setVisible(selectedFormat == SortDataStorage.Format.DATE_CUSTOM);
                fieldCustomFormat.setVisible(selectedFormat == SortDataStorage.Format.DATE_CUSTOM);
		//                if(bAsc.isSelected() || bDesc.isSelected())
		sort();
            }
        });

        bAsc.setIcon(Util.loadIcon("Up16-2.png"));
        bDesc.setIcon(Util.loadIcon("Down16-2.png"));
        panFormat.add(bAsc);
        panFormat.add(bDesc);

        panSort.add(new JLabel(i18n.get(AnimationDSChainPanel.class, "sort", "Sort")));
        panSort.add(panFormat);


        JPanel panValues = new JPanel();
        panValues.add(new JLabel(i18n.get(AnimationDSChainPanel.class, "values", "Values")));
        nextButton = newNextButton();
        panParameters.add(panValues);
        JPanel panTimer = new JPanel();
        spinnerDelay = new JSpinner(new SpinnerNumberModel(dsChain.getDelay(), 0, 60, 1));
        spinnerDelay.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                int del = Integer.parseInt(spinnerDelay.getValue().toString());
                dsChain.setDelay(del);
            }
        });
        bRun = newRunButton();
        panValues.add(cbValues);
        panValues.add(nextButton);
        panValues.add(panTimer);
        panTimer.add(bRun);
        panTimer.add(new JLabel(i18n.get(AnimationDSChainPanel.class, "delay", "Delay")));
        panTimer.setBorder(BorderFactory.createLineBorder(Color.gray));
        ComponentSizer.fixComponentSize(bRun, new Dimension(30, 20));

        panTimer.add(spinnerDelay);
        panParameters.add(wrapSort);
        add(panParameters, BorderLayout.CENTER);
        panParameters.setBorder(
                BorderFactory.createTitledBorder(""/*
                i18n.get(AnimationDSChainPanel.class, "chainParameters", "Chain parameters")*/));
        bRemoveChain = new JButton(Util.loadIcon("Delete16.png"));
        bRemoveChain.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                dsChain.delete();
            }
        });
        bRemoveChain.setToolTipText(i18n.get(AnimationDSChainPanel.class, "delete", "Delete the Chain"));

        bNewAnim = new JButton(Util.loadIcon("addanim.png"));
        bNewAnim.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                addAnimationCreationGUI();
                bNewAnim.setEnabled(false);
            }
        });
        bNewAnim.setToolTipText(i18n.get(AnimationDSChainPanel.class, "addAnimationToLayer", 
					 "Add a new filter to a layer"));

        bShowTool = new JButton(Util.loadIcon("enabled.png"));
        bShowTool.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                boolean visibility = !dsChain.getToolGUI().isVisible();
                dsChain.getToolGUI().setVisible(visibility);
                bShowTool.setIcon(visibility ? Util.loadIcon("enabled.png") : Util.loadIcon("disabled.png"));
            }
        });
        bShowTool.setToolTipText(i18n.get(AnimationDSChainPanel.class, "showAnimationTool", 
					  "Show/Hide animation tool in the map panel tool bar"));


        bRemoveChain.setBackground(Color.gray);
        bRemoveChain.setOpaque(false);
        bNewAnim.setBackground(Color.gray);
        bNewAnim.setOpaque(false);
	bShowTool.setBackground(Color.gray);
        bShowTool.setOpaque(false);

        toolButtons = new JToolBar();
        toolButtons.setFloatable(false);
        toolButtons.add(bNewAnim);
        toolButtons.add(bShowTool);
        toolButtons.add(Box.createRigidArea(new Dimension(20, 20)));
        toolButtons.add(bRemoveChain);
        //bRemoveChain.setBorder(BorderFactory.createEmptyBorder(0, 15, 0, 0));
        ComponentSizer.fixComponentSize(bNewAnim, new Dimension(35, 35));
        ComponentSizer.fixComponentSize(bRemoveChain, new Dimension(35, 35));
        ComponentSizer.fixComponentSize(bShowTool, new Dimension(35, 35));
        add(toolButtons, BorderLayout.NORTH);

        //ComponentSizer.fixComponentSize(wrapTop, new Dimension(680, 30));
        toolGUI = buildToolGUI();
        //ComponentSizer.fixComponentSize(this, new Dimension(680, 200));

	refresh(); ///// !!! true);
        setEnabledSortGUI(false);
        setEnabledParametersGUI(dsChain.getAnimationDatastorages().size() > 0);

	bAsc.setSelected(true);
	// we can't sort the values now as the panel is not fully constructed yet
	// should provide an init method that could perform the sort....
	// but we will call directly in AnimationDSChain
	// sort();
    }

    private JButton newNextButton() {
	JButton b = new  JButton(Util.loadIcon("FastForward16.png"));
	b.setToolTipText(i18n.get(AnimationDSChainPanel.class, "next", "Next value"));
	b.addActionListener(dsChain.getNextValueAction());
	b.setBackground(Color.gray);
	b.setOpaque(false);
	
	return b;
    }

    private JToggleButton newRunButton() {
	JToggleButton b = new  JToggleButton(Util.loadIcon("Play16.png"));
	b.setToolTipText(i18n.get(AnimationDSChainPanel.class, "start", "Start"));
        b.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
		if (! (e.getSource() instanceof JToggleButton))
		    return;
		JToggleButton b = (JToggleButton) e.getSource();
                if (dsChain.isRunning()) {
                    dsChain.stop();
                    bRun.setIcon(Util.loadIcon("Play16.png"));
                    bRunTool.setIcon(Util.loadIcon("Play16.png"));
                } else {
                    dsChain.start();
                    bRun.setIcon(Util.loadIcon("Pause16.png"));
                    bRunTool.setIcon(Util.loadIcon("Pause16.png"));
                }
                b.updateUI();
            }
        });
	b.setBackground(Color.gray);
	b.setOpaque(false);
	
	return b;
    }

    public void sort(){
	if (dsChain == null)
	    return;
        try{
            dsChain.sortValues();
	    refresh();
        } catch(java.lang.IllegalArgumentException ex){ // in case of bad format thrown exception
	    ex.printStackTrace();
            JOptionPane.showMessageDialog(
                    this,
                    i18n.get(this, "sortWarning", "Your data will not be sorted"),
                    i18n.get(this, "sortWarningTitle", "Format error"),
                    JOptionPane.WARNING_MESSAGE);
        }
    }

    private JPanel buildToolGUI() {
        JPanel tool = new JPanel();
        tool.setBackground(Color.GRAY);
        tool.setOpaque(false);
        nextButtonTool = newNextButton();
        bRunTool = newRunButton();

        cbValuesTool = new JComboBox(); //dsChain.computeUnionValues().toArray());
        //cbValuesTool.setSelectedItem(cbValues.getSelectedItem());
        cbValuesTool.addActionListener(manualActionListener);
        // ComponentSizer.fixComponentSize(cbValuesTool, new Dimension(120, 25));

        tool.add(cbValuesTool);
        tool.add(nextButtonTool);
        tool.add(bRunTool);

        return tool;
    }

    public void setEnabledRunButton(boolean b){
        bRun.setEnabled(b);
        bRunTool.setEnabled(b);
    }

    public JComponent getToolGUI() {
        return toolGUI;
    }

    static final int MAX_DIFFERENT_VALUES = 500;
 
    public void refresh() {
        activeListener = false;
	refreshCheckBoxes();
        cbValues.removeAllItems();
        cbValuesTool.removeAllItems();
	int selectedIndex = -1; 
	if (dsChain.getValues() == null)
	    return; // no updateUI needed ???
	if (dsChain.getValues().length > MAX_DIFFERENT_VALUES) {
	    System.err.println("[Animation] ***WARNING*** too many different values for the filter"); 
	    cbValues.addItem("*too many values*");
            cbValuesTool.addItem("*too many values*");
	    return;
	}
	int index = 0;
        for (Object o : dsChain.getValues()) {
            // System.out.println("ADD VALUE " + o);
            cbValues.addItem(o);
            cbValuesTool.addItem(o);
	    if (dsChain.getCurrentValue() != null && dsChain.getCurrentValue().equals(o))
		selectedIndex = index;
	    index++;
        }

	// update selected values
	cbValues.setSelectedIndex(selectedIndex);
	cbValuesTool.setSelectedIndex(selectedIndex);

        activeListener = true;
        cbValues.updateUI();
        cbValuesTool.updateUI();
    }

    public void removeAnimationFromGUI(MonoValuedFilterDataStorage ads) {
        if (animationsGUI.get(ads) != null) {
            listBox.remove(animationsGUI.get(ads));
            listBox.updateUI();
            animationsGUI.remove(ads);
	    // <removeButton>
	    removeButtonMap.remove(ads);
	    // </removeButton>
	    checkBoxMap.remove(ads);
        }
	fireRefreshRemoveButtons();
        updateUI();
    }

    private void addAnimationCreationGUI() {
        if (pAddAnim == null) {
            pAddAnim = new JPanel();
            JLabel labNewAnimation = new JLabel(
                    i18n.get(AnimationDSChainPanel.class, "addAnimationTo", "Add an annimation to: "));

            LayerHandler layerHandler = dsChain.getMetaAnimator().getLayerHandler();
            comboboxLayers = new JComboBox();
            layerHandler.addLayerListener(this);
            JButton bOk = new JButton(i18n.get(AnimationDSChainPanel.class, "ok", "Ok"));
            JButton bCancel = new JButton(i18n.get(AnimationDSChainPanel.class, "cancel", "Cancel"));
            pAddAnim.add(labNewAnimation);
            pAddAnim.add(comboboxLayers);
            pAddAnim.add(bOk);
            pAddAnim.add(bCancel);
            toolButtons.add(pAddAnim);
            toolButtons.updateUI();
            toolButtons.add(pAddAnim);
            ComponentSizer.fixComponentSize(comboboxLayers, new Dimension(120, 20));
            bOk.setBorder(null);
            bCancel.setBorder(null);
            bOk.setBackground(Color.LIGHT_GRAY);
            bCancel.setBackground(Color.LIGHT_GRAY);
            //mponentSizer.fixComponentSize(bOk, new Dimension(45, 25));
            ComponentSizer.fixComponentSize(pAddAnim, new Dimension(450, 25));
            pAddAnim.setBackground(Color.gray);
            pAddAnim.setOpaque(false);
            bOk.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent ae) {
                    pAddAnim.setVisible(false);
                    bNewAnim.setEnabled(true);
                    dsChain.addAnimationToLayer((Layer)comboboxLayers.getSelectedItem());
                }
            });
            bCancel.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent arg0) {
                    pAddAnim.setVisible(false);
                    bNewAnim.setEnabled(true);
                }
            });
        }
        pAddAnim.setVisible(true);
    }

    /**
     *
     */
    public void addAnimationGUI(final MonoValuedFilterDataStorage ads) {
	// System.out.println("### addAnimationGUI ### "+ ads);
        listBox.setVisible(true);
        final JPanel panAnim = new JPanel();
	// <removeButton>
	final JButton removeButton = new JButton(Util.loadIcon("Delete10.png"));
	panAnim.add(removeButton);
	// </removeButton>
        final JCheckBox checkBox = new JCheckBox(ads.getFullName());
	checkBoxMap.put(ads, checkBox);
	panAnim.add(checkBox);
	JComboBox adsGUI = (JComboBox) ads.getMetaGUI();
        panAnim.add(adsGUI);
	ads.updateMetaGUI(null);

        animationsGUI.put(ads, panAnim);
	// <removeButton>
        ComponentSizer.fixComponentSize(removeButton, new Dimension(13, 13));
	// </removeButton>
        ComponentSizer.fixComponentSize(checkBox, new Dimension(130, 25));
        ComponentSizer.fixComponentSize(adsGUI, new Dimension(100, 25));
        ComponentSizer.fixComponentSize(panAnim, new Dimension(280, 30));
        listBox.add(panAnim);
	// <removeButton>
        removeButton.setBackground(Color.gray);
        removeButton.setOpaque(false);
	removeButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    dsChain.removeAnimationDataStorage(ads);
		    if (!(ads.isOnTop())) { // should never happen
			JOptionPane.showMessageDialog(removeButton,
						      i18n.get(AdaptorButtonArray.class,
							       "cantRemove",
							       "Can't remove an adaptor not on top"));
			return;
		    }
		    String mess = String.format(i18n.get(AdaptorButtonArray.class,
							 "removeAdaptorConfirmation",
							 I18n.TOOLTIP,
							 "Do you really want to remove the adapter %s?"),
						ads.getName());
		    int rep = JOptionPane.showConfirmDialog(removeButton,
							    mess,
							    i18n.get(AdaptorButtonArray.class, "removeAdaptorConfirmation",
								     "Confirmation"),
							    JOptionPane.YES_NO_OPTION);
		    if (rep == JOptionPane.YES_OPTION)
			((SimpleDataDisplayLayer)ads.getDataStorageListener()).removeTopAdaptorDataStorage();
		}
	    });
	removeButtonMap.put(ads, removeButton);
	fireRefreshRemoveButtons();
	// </removeButton>
        checkBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                if (checkBox.isSelected()) {
                    dsChain.addAnimationDataStorage(ads);
                    checkBox.setToolTipText(i18n.get(AnimationDSChainPanel.this,"rmFilter","Remove filter from animation"));
		    JComponent adsGUI = ads.getMetaGUI();
		    panAnim.add(adsGUI); // the gui is removed automagically from the panel it was in.
		    adsGUI.setVisible(true);
                } else {
                    dsChain.removeAnimationDataStorage(ads);
                    checkBox.setToolTipText(i18n.get(AnimationDSChainPanel.this,"addFilter","Add filter to animation"));
                }
                // if (panAnim.getComponentCount() == 1) panAnim.add(adsGUI);

                panAnim.updateUI();
                setEnabledParametersGUI(dsChain.getAnimationDatastorages().size() > 0);
            }
        });
        boolean contains = dsChain.containsAnimationDatastorage(ads);
	Debug.message("Animation", "[AnimationDSChainPanel] "+ads + " is contained in [" + dsChain.getAnimationDatastorages() + "] " + contains);
        checkBox.setSelected(contains);
        checkBox.setToolTipText(checkBox.isSelected() ?
            i18n.get(AnimationDSChainPanel.this,"rmFilter","Remove filter from animation") :
            i18n.get(AnimationDSChainPanel.this,"addFilter","Add filter to animation"));
        if(contains){
            cbValues.setSelectedItem(ads.getFilterColumnValue());      
	    adsGUI.setVisible(true);    
        }
        listBox.updateUI();
    }

    private void fireRefreshRemoveButtons() {
   	dsChain.fireRefreshRemoveButtons();
    }
   
    public void refreshRemoveButtons() {
    	for (java.util.Map.Entry<MonoValuedFilterDataStorage, JButton> entry : removeButtonMap.entrySet()) {
    	    // System.out.println("refreshRemoveButtons "+ entry.getKey() +" is on top: " +entry.getKey().isOnTop());
    	    entry.getValue().setEnabled(entry.getKey().isOnTop());
    	}
    }

    public void refreshCheckBoxes() {
	for (java.util.Map.Entry<MonoValuedFilterDataStorage, JCheckBox> entry : checkBoxMap.entrySet()) {
	    boolean contained = dsChain.getAnimationDatastorages().contains(entry.getKey());
	    JCheckBox cb = entry.getValue();
	    // if ((contained && ! cb.isSelected()) || (!contained && cb.isSelected())) {
	    // 	cb.doClick();
	    // }
	    if (contained && ! cb.isSelected()) {
		// System.out.println("refreshCheckBoxes "+ entry.getKey() +" " + contained + " et " + cb.isSelected());
		cb.setSelected(true);
	    } else if (!contained && cb.isSelected()) {
		// System.out.println("refreshCheckBoxes "+ entry.getKey() +" " + contained + " et " + cb.isSelected());
		cb.setSelected(false);		
	    }
	}
    }

    public void updateSelectedValue(String value) {
	activeListener = false;
        cbValues.setSelectedItem(value);
        cbValues.updateUI();
        cbValuesTool.setSelectedItem(value);
        cbValuesTool.updateUI();
	activeListener = true;
    }

    private void updateComboLayers(Layer[] layers) {
        ArrayList<Layer> lNames = new ArrayList<Layer>();
        for (Layer l : layers){
            if(l instanceof ThematicDataDisplayLayer){
                lNames.add(l);
            }
        }
        comboboxLayers.removeAllItems();
        for (Layer l : lNames)
            comboboxLayers.addItem(l);
    }

    public void setLayers(LayerEvent le) {
	// System.out.println("== Animation setLayers: "+le.getType() + " " + java.util.Arrays.asList(le.getLayers()));
        updateComboLayers(le.getLayers());
    }

    public void setEnabledParametersGUI(boolean b) {
        if(cbValues == null)
	    return;
        cbValues.setEnabled(b);
        bRun.setEnabled(b);
        nextButton.setEnabled(b);
        cbValuesTool.setEnabled(b);
        bRunTool.setEnabled(b);
        setEnabledSortGUI(b);
	// always disable timer gui if master is not null
	setEnabledTimerGUI(dsChain.getMaster() != null ? false : b);
    }

    public void setEnabledTimerGUI(boolean b) {
	bRunTool.setEnabled(b);
	bRun.setEnabled(b);
        spinnerDelay.setEnabled(b);
    }

    public void setEnabledSortGUI(boolean b) {
        boxFormat.setEnabled(b);
        bAsc.setEnabled(b);
        bDesc.setEnabled(b);
    }
}
