// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * AggregationDataStorage.java
 *
 * Created on 9 April 2010
 *
 * @author Jacques.Madelaine@info.unicaen.fr
 *
 */
package fr.unicaen.openmap.dataStorage;

import com.bbn.openmap.util.Debug;

import com.bbn.openmap.util.PropUtils;

import javax.swing.*;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.*;

import javax.swing.event.*;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import java.util.*;

import fr.unicaen.openmap.util.*;
import fr.unicaen.openmap.dataStorage.PivotDataStorage.AggregateFunctionComboBox;
import fr.unicaen.openmap.dataStorage.PivotDataStorage.AttributeType;
import fr.unicaen.openmap.dataStorage.PivotDataStorage.RBAction;
import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import fr.unicaen.openmap.dataStorage.gui.DataTableStoragePanel;
import fr.unicaen.openmap.dataStorage.gui.AggregateDataStoragePanel;
import fr.unicaen.openmap.dataStorage.DataTable;
import fr.unicaen.openmap.dataStorage.expr.SpatialFunction;
import fr.unicaen.openmap.gui.component.JComponentCellEditor;
import fr.unicaen.openmap.gui.component.JComponentCellRenderer;
import fr.unicaen.openmap.gui.component.JComponentTable;
import fr.unicaen.openmap.gui.component.JComponentTableModel;


import com.bbn.openmap.I18n;
import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMRect;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.omGraphics.OMPoint;
import com.bbn.openmap.geo.Geo;

/**
 * Creates aggregates group by a set of criterion attributes values.
 * Aggregate operations over non geometry data are the same as for Pivot or Grid. 
 * Aggregate operations over geometry are : ConvexHull, Union, Intersection, ... 
 */
public class AggregateDataStorage extends PivotDataStorage {
    /* --------------------------  Begin   Properties  ---------------------*/

    public static final String geometryAttributesListPropertyString = "geometryAttributes";
    public static final String parameterColumnNamePropertyString = "parameterColumnName";
    /* ---------------------------  End     Properties  ---------------------*/
    /** geometryAttributesHashMap is used by the GUI to edit the list geometryAttributesList 
     */
    protected HashMap<String, AttributeType> geometryAttributesHashMap = null;
    private ArrayList<String> geometryAttributesList = new ArrayList<String>();

    protected ArrayList<String> getGeometryAttributesList() {
        return geometryAttributesList;
    }
    public static enum OMGAttributeType {
	GEOMETRYLIST, PARAMETERLIST
    }
    public AggregateDataStorage(DataStorageListener dsl) {
        super(dsl, null);
    }

    public AggregateDataStorage(DataStorageListener dsl, DataStorage dataStorage) {
        super(dsl, dataStorage);
    }

    public void setProperties(String prefix, java.util.Properties properties) {
        setInitialized(false);
        super.setProperties(prefix, properties);

        prefix = PropUtils.getScopedPropertyPrefix(prefix) + pivotDSPropertyString + ".";
        geometryAttributesList = UnicaenPropUtils.arrayListFromProperties(properties, prefix + geometryAttributesListPropertyString, listDelim);
        for (String attribute : geometryAttributesList) {
	    GeometryAggregateFunction geometryAggregateFunction = 
		Enum.valueOf(GeometryAggregateFunction.class, 
			     properties.getProperty(prefix + aggregateFunctionPropertyString + "." + attribute, "CENTROID"));
            setGeometryAggregateFunction(attribute, geometryAggregateFunction);
	    if (geometryAggregateFunction == GeometryAggregateFunction.BARYCENTER)
		setParameterColumnName(attribute,
				       properties.getProperty(prefix + parameterColumnNamePropertyString + "." + attribute, "data"));
        }
        initialize(); // in case we received a DataStorageEvent meanwhile...
    }

    public java.util.Properties getProperties(String prefix, java.util.Properties properties) {
        properties = super.getProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + pivotDSPropertyString + ".";
        UnicaenPropUtils.arrayListToPropertyString(properties, prefix + geometryAttributesListPropertyString, geometryAttributesList, listDelim);
        for (String attribute : geometryAttributesList) {
            properties.put(prefix + aggregateFunctionPropertyString + "." + attribute, 
			   getGeometryAggregateFunction(attribute).name());
	    if (getGeometryAggregateFunction(attribute) == GeometryAggregateFunction.BARYCENTER)
		properties.put(prefix + parameterColumnNamePropertyString + "." + attribute, 
			       getParameterColumnName(attribute));     
        }

        return properties;
    }
    private LonLatNames lonLatNames;    // suffixes of column names starting with longitude or latitude

    static public enum GeometryAggregateFunction {

        BARYCENTER(i18n.get(PivotDataStorage.class, "BARYCENTER", "Barycenter")),
	    CENTROID(i18n.get(PivotDataStorage.class, "CENTROID", "Centroid")),
	    CONVEXHULL(i18n.get(PivotDataStorage.class, "CONVEXHULL", "Convex Hull")), 
	    UNION(i18n.get(PivotDataStorage.class, "UNION", "Union")),
	    INTERSECTION(i18n.get(PivotDataStorage.class, "INTERSECTION", "Intersection")),
	    FIRST(i18n.get(PivotDataStorage.class, "FIRST", "First")),
	    LAST(i18n.get(PivotDataStorage.class, "LAST", "Last")),
	    //CONTOURS(i18n.get(PivotDataStorage.class, "CONTOURS", "Contours")),
	    ;
	private final String name;
	// public String getName() { return name; }
	public String toString() { return name; }
	GeometryAggregateFunction(String name) {
	    this.name = name;
	}
	public boolean needsSecondParameter() {
	    return this == BARYCENTER;
	}
    }
    
    /** gives the geometry aggregate function for each geometry attribute */ 
    private HashMap<String, GeometryAggregateFunction> geometryAggregateFunctionMap =
            new HashMap<String, GeometryAggregateFunction>();

    protected GeometryAggregateFunction getGeometryAggregateFunction(String attribute) {
        GeometryAggregateFunction geometryAggregateFunction =
                geometryAggregateFunctionMap.get(attribute);
        return geometryAggregateFunction != null ? geometryAggregateFunction : GeometryAggregateFunction.CENTROID;
    }
    protected void setGeometryAggregateFunction(String attribute, GeometryAggregateFunction a) {
        geometryAggregateFunctionMap.put(attribute, a);
    }

    /** gives the attribute used as parameter in case of Barycenter on a geometry */ 
    private HashMap<String, String> parameterColumnNameMap =
            new HashMap<String, String>();

    protected String getParameterColumnName(String attribute) {
	return parameterColumnNameMap.get(attribute);
    }
    protected void setParameterColumnName(String geometryAttribute, String parameterColumnName) {
	parameterColumnNameMap.put(geometryAttribute, parameterColumnName);
    }

    /** operates the geometryAggregateFunction over value and data. Value turns to be in fact a AggregateResult.
    Returns a OMDataLocation hiding the AggregateResult.
     */
    @SuppressWarnings("unchecked")
	protected OMDataLocation geometryOperate(OMDataLocation value, OMDataLocation data, GeometryAggregateFunction geometryAggregateFunction, Object parameter) {
        if (value == null) {
	    List<Object> b = null; // parameter list for BARYCENTER
            switch (geometryAggregateFunction) {
	    case BARYCENTER:
		b = new ArrayList<Object>();
		b.add(parameter);
		// no break here!
	    case CENTROID:
	    case CONVEXHULL:
	    case UNION:
	    case INTERSECTION: 
	    case FIRST:
	    case LAST:
		{
		    List<OMDataLocation> a = new ArrayList<OMDataLocation>();
		    a.add(data);
		    OMGraphic omg = new OMGraphicList(0);
		    omg.putAttribute(OMGAttributeType.GEOMETRYLIST, a);
		    if (b != null)
			omg.putAttribute(OMGAttributeType.PARAMETERLIST, b);
		    return new OMDataLocation(omg);
		}
            }
            return null;
        }
        OMGraphic omg = value.getOMGraphic();
        if (omg == null)
            return data;
        Object geometryList = omg.getAttribute(OMGAttributeType.GEOMETRYLIST);
        if (geometryList instanceof ArrayList) {
 	    List<Object> b = null; // parameter list for BARYCENTER
	    switch (geometryAggregateFunction) {
	    case BARYCENTER:
		b = (List<Object>)omg.getAttribute(OMGAttributeType.PARAMETERLIST);
		b.add(parameter); 
	    case CENTROID:
	    case CONVEXHULL:
	    case UNION:
	    case INTERSECTION:
		((List<OMDataLocation>)geometryList).add(data);
		return value;
	    case FIRST:
		return value;
	    case LAST:
		// replace graphic with current graphic in value
		((List<OMDataLocation>)geometryList).set(0, data);
		return value;
            }
            Debug.error("[AggregateDataStorage] *** Error in aggregate for (" + value + ", " + data + ")");
            return null;
        }
        return data;
    }

    @SuppressWarnings("unchecked")
	protected void replaceAggregateResultByRealResult(DataObject data, ArrayList<String> dataColumnNames) {
        super.replaceAggregateResultByRealResult(data, dataColumnNames);

        Map<String, OMDataLocation> omdls = data.getOmdls();
        for (Map.Entry<String, OMDataLocation> entry : omdls.entrySet()) {
            OMDataLocation omdl = entry.getValue();
            if (omdl == null)
                continue;
            OMGraphic omg = omdl.getOMGraphic();
            if (omg == null)
                continue;
            Object geometryList = omg.getAttribute(OMGAttributeType.GEOMETRYLIST);
            if (geometryList instanceof ArrayList) {
                List<OMDataLocation> a = (List<OMDataLocation>) geometryList;
		OMDataLocation entryValue = null;
                switch (getGeometryAggregateFunction(entry.getKey())) {
		case FIRST:
		case LAST:
		    entryValue = a.get(0);
		    break;
		case BARYCENTER: 
		    {
			List<Object> b = 
			    (List<Object>)omg.getAttribute(OMGAttributeType.PARAMETERLIST);
			entryValue = SpatialFunction.barycenter(a, b); 
		    }
		    break;
		case CENTROID:
		    entryValue = SpatialFunction.centroid(a);
		    break;
		case CONVEXHULL:
		    entryValue = SpatialFunction.convexHull(a);
		    break;
		case UNION:
		    entryValue = SpatialFunction.union(a);
		    break;
		case INTERSECTION:
		    entryValue = SpatialFunction.intersection(a);
		    break;
                }  
		if (entryValue == null)
		    entryValue = new OMDataLocation(null, null, null);
		entry.setValue(entryValue);
            }
        }
    }

    public void initialize() {
        if (getDataStorage() == null || !getDataStorage().isReady()) {
            Debug.error("[AggregateDataStorage] can't initialize because dataStorage is not ready !!!!");
            return;
        }

        DataTableModel dtm = getDataStorage().getDefaultDataTableModel();

        Debug.message("DataStorage", "[AggregateDataStorage] start initialize");

        lonLatNames = new LonLatNames();

	// new data column names
        // merge stat and criteria column names
        ArrayList<String> dataColumnNames = new ArrayList<String>(getCriteriaAttributesList());
        for (String a : getStatisticsAttributesList())
            dataColumnNames.add(a);

        ArrayList<String> dataColumnAliases = null; // should be updated --TBD

        // we need a temporary map in order to store
        // for each < DataObject holding criteria values > a DataObject
        HashMap<DataObject, DataObject> index = new HashMap<DataObject, DataObject>();

        // first pass: aggregate the data
        for (int i = 0; i < getDataStorage().getCount(); i++) {
            DataObject currentDataObject = getDataStorage().getElementAt(i);

            // collect criteria attributes values
            DataObject criteriaDataObject = new DataObject();
            for (String criterion : getCriteriaAttributesList())
                criteriaDataObject.addData(criterion, currentDataObject.getData(criterion));

            // retrieve the DataObject for the criteria value (or make a new one)
            DataObject dataForCriteria = index.get(criteriaDataObject);
            if (dataForCriteria == null) {
                dataForCriteria = new DataObject();
                // System.out.println("new data  for " + criteriaDataObject);
            }

            for (String name : getStatisticsAttributesList()) {
                // get and add data according to the value of  columns
                Object previous = dataForCriteria.getData(name);
                Object current = currentDataObject.getData(name);
                dataForCriteria.addData(name, operate(previous, current, getAggregateFunction(name)));
            }
            for (String name : getGeometryAttributesList()) {
                // get and add geometry data according to the value of  columns
                OMDataLocation previous = dataForCriteria.getOMDataLocation(name);
                OMDataLocation current = currentDataObject.getOMDataLocation(name);
		GeometryAggregateFunction geometryAggregateFunction = getGeometryAggregateFunction(name);
		Object parameter = null;
		if (geometryAggregateFunction == GeometryAggregateFunction.BARYCENTER)
		    parameter = currentDataObject.getData(getParameterColumnName(name));
                dataForCriteria.addOMDataLocation(name, geometryOperate(previous, current, geometryAggregateFunction, parameter));
            }

            index.put(criteriaDataObject, dataForCriteria);
        }

        /// for (String attr : getCriteriaAttributesList()) // Is it useful ???
        ///     lonLatNames.getLonLatType(attr);
        ///
        boolean buildCentroid = lonLatNames.getLonLatNamesSuffixes().size() > 0;

        if (buildCentroid)
            Debug.message("DataStorage", "[GridDataStorage] build centroid " + lonLatNames);

        ////////

        // second pass: gather all data
        ArrayList<DataObject> aggregateDataObjects = new ArrayList<DataObject>();
        for (Map.Entry<DataObject, DataObject> entry : index.entrySet()) {
            DataObject data = entry.getValue();
            // 1- complete the computation of aggregate values
            replaceAggregateResultByRealResult(data, dataColumnNames);
            // 2- merge the criteria values
            data.putAll(entry.getKey());
            // 3- built lon/lat centroïd
            if (buildCentroid) {
                for (String suffix : lonLatNames.getLonLatNamesSuffixes()) {
                    try {
                        lonLatNames.putLon(suffix,
                                data.getData(lonLatNames.getLonName(suffix)));
                        lonLatNames.putLat(suffix,
                                data.getData(lonLatNames.getLatName(suffix)));
                    } catch (java.lang.NullPointerException exception) {
                        // one lat or lon does not exist...
                    } catch (java.lang.Exception exception) {
                        Debug.error("*** AggregateDataStorage bad cast to Double building centroid for \"" + data.getData(lonLatNames.getLonName(suffix)) + "\", \"" + data.getData(lonLatNames.getLatName(suffix)) + "\"");
                    }
                }
                lonLatNames.putCentroids(data);
            }
            aggregateDataObjects.add(data);
        }

        // add omColumnNames
        ArrayList<String> omColumnNames = lonLatNames.getOMGraphicNames();
        for (String a : getGeometryAttributesList())
            omColumnNames.add(a);

	// store the data
        dataTable.setTable(aggregateDataObjects, dataColumnNames, omColumnNames, dataColumnAliases);

        // data are stored, we can create dataTableModel and panel to view them.
        dataTableModel = new DataTableModel(dataTable);
        resetDataTableStoragePanel(dataTableModel);
        setInitialized(true);
        Debug.message("DataStorage", "[AggregateDataStorage] (re)initialized");

        // tell the listener we're ready
        fireDataStorageEvent();
    }

    public class GeometryAggregateFunctionComboBox extends JComboBox implements ItemListener {

        private String attribute;
	private JComboBox parameterComboBox;
        public GeometryAggregateFunctionComboBox(String attribute, JComboBox parameterComboBox) {
            super(GeometryAggregateFunction.values());
            this.attribute = attribute;
	    this.parameterComboBox = parameterComboBox;
	    parameterComboBox.setVisible(geometryAggregateFunctionNeedsSecondParameter(attribute));
            setSelectedIndex(getGeometryAggregateFunction(attribute).ordinal());
            addItemListener(this);
        }

        public void itemStateChanged(ItemEvent e) {
            int index = ((JComboBox) e.getSource()).getSelectedIndex();
            setGeometryAggregateFunction(attribute, GeometryAggregateFunction.values()[index]);
	    parameterComboBox.setVisible(geometryAggregateFunctionNeedsSecondParameter(attribute));
	    parameterComboBox.setEnabled(geometryAggregateFunctionNeedsSecondParameter(attribute));
            getParent().repaint(); // to update the UI and really enable the parameterComboBox
        }

	public void setEnabled(boolean b) {
	    super.setEnabled(b);
	    parameterComboBox.setEnabled(b && geometryAggregateFunctionNeedsSecondParameter(attribute));
	}
    }

    public class ParameterComboBox extends JComboBox implements ItemListener {

        private String attribute;

        public ParameterComboBox(String attribute) {
            super(getDataStorage().getDefaultDataTableModel().getDataColumnNames());
            this.attribute = attribute;
	    setSelectedItem(getParameterColumnName(attribute));
            addItemListener(this);
        }

        public void itemStateChanged(ItemEvent e) {
	    setParameterColumnName(attribute,
				   ((JComboBox) e.getSource()).getSelectedItem().toString());
        }
    }

    public JComponent getGUI() {
        return new AggregateDataStoragePanel(this);
    }

    public boolean hasGeometryAttributeType(String n, AttributeType att) {
        return geometryAttributesHashMap.get(n) == att;
    }

    public boolean geometryAggregateFunctionNeedsSecondParameter(String name) {
	return getGeometryAggregateFunction(name).needsSecondParameter();
    }

    public void initGeometryAttributesHashMapFromLists(String[] attributesNames) {
        geometryAttributesHashMap = new HashMap<String, AttributeType>(attributesNames.length);
        for (String name : attributesNames)
            geometryAttributesHashMap.put(name, AttributeType.DISCARD);
        // update map with lists
        // AttributeType.STATISTIC in place of GEOMETRY
        if (geometryAttributesList != null)
            for (String name : geometryAttributesList)
                geometryAttributesHashMap.put(name, AttributeType.STATISTIC);
    }

    /** AttributesLists are made of RealColumnNames */
    public void initGeometryAttributesListsFromHashMap() {
        geometryAttributesList = new ArrayList<String>();

        for (Map.Entry<String, AttributeType> e : geometryAttributesHashMap.entrySet())
            switch (e.getValue()) {
                case DISCARD:
                    break;
                case STATISTIC:
                    geometryAttributesList.add(e.getKey());
                    break;
                default:
                    break;
            }
        Debug.message("DataStorage", "[AggregateDataStorage] Geometry: " + geometryAttributesList);
    }

    public class RBGeometryAction extends RBAction {

        public RBGeometryAction(String name, AttributeType type, JComboBox aggregateFunctionComboBox, JTable table) {
            super(name, type, aggregateFunctionComboBox, table);
        }

        public void actionPerformed(ActionEvent e) {
            geometryAttributesHashMap.put(name, type);
            if (aggregateFunctionComboBox != null) // enable the aggregateFunctionComboBox
            {
                aggregateFunctionComboBox.setEnabled(type == AttributeType.STATISTIC);
            }
            table.updateUI();
        }
    }
}
