// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
// **********************************************************************
//  CERTIC, CIT
//  17, rue claude Bloch
//  Universitïé de Caen-Basse-Normandie
//  14032 Caen Cedex
//  +33(0)231-566-900
// **********************************************************************
//
// $Date: 2005/09/19 15:35:21 $
// $Author: christophe.turbout@info.unicaen.fr $
//
//
// 2006-05-17: Jacques replace Vector by ArrayList (and elementAt calls by get)
// ****************************************************************************

package fr.unicaen.openmap.dataStorage;
import java.util.ArrayList;
import com.bbn.openmap.util.Debug;

/**
 * DataTable store each row of Data.
 *
 * This class store real data from any type of data source.
 */

public class DataTable {
    
    /** Stored data and ompgraphics values
     */
    private ArrayList<DataObject> dataObjects;
    
    /** Stored data columns names.
     */
    private ArrayList<String> dataColumnNames;
    
    /** Stored omgraphics columns names.
     */
    private ArrayList<String> omColumnNames;
    
    /** Stored data columns names aliases.
     */
    private ArrayList<String> dataColumnAliases;
    
    /** Default constructor
     */
    public DataTable(){
        dataObjects = new ArrayList<DataObject>();
        dataColumnNames = new ArrayList<String>();
        omColumnNames = new ArrayList<String>();
        dataColumnAliases = new ArrayList<String>();
    };
    
    public DataTable(ArrayList<DataObject> dataObjects, ArrayList<String> dataColumnNames, ArrayList<String> omColumnNames, ArrayList<String> dataColumnAliases) {
        this.dataObjects = dataObjects;
        this.dataColumnNames = dataColumnNames;
        this.omColumnNames = omColumnNames;
        this.dataColumnAliases =  dataColumnAliases;
    }
        
    /** (re)init method
     * (should deprecated the other setTable method
     */
    public void setTable(ArrayList<DataObject> dataObjects, ArrayList<String> dataColumnNames, ArrayList<String> omColumnNames, ArrayList<String> dataColumnAliases) {
        this.dataObjects = dataObjects;
        this.dataColumnNames = dataColumnNames;
        this.omColumnNames = omColumnNames;
        this.dataColumnAliases =  dataColumnAliases;
        if (dataColumnAliases != null && ! dataColumnAliases.isEmpty() &&  dataColumnAliases.size() != dataColumnNames.size())
            Debug.error("[DataTable]setTable() **** Error **** Aliases list has a different size than column names list");
        Debug.message("DataStorage", "[DataTable]setTable() "+ dataColumnNames + " " + omColumnNames + " " + dataColumnAliases + " => " + dataObjects.size() + " objects loaded");
        
    }
    
    public void setTable(ArrayList<DataObject> dObjs){
    	Debug.message("DataStorage", "[DataTable]setTable() "+dObjs.size()+" "+dataColumnNames+" "+omColumnNames+" "+dataColumnAliases);
    	setTable(dObjs, dataColumnNames, omColumnNames, dataColumnAliases);
    }
    
    public DataObject getDataObjectAt(int row) { 
    	return dataObjects == null ? null : dataObjects.get(row); }
    
    /**
     * Give us the ability to access informations implementing an {@link AbstractTableModel}.
     * @param row index in DataTable
     * @param column index in dataTable
     * @return {@link Object} which is the content of the cell at (row,col) int the data part.
     */
    public Object getDataAt(int row, int column) {
    	try{
        if (dataObjects == null) return null;
        DataObject dataObject =  dataObjects.get(row);
        if (dataObject == null) return null;
        Debug.message("DataTable", "[DataTable]getDataAt(row = "+ row+" , col = "+column+")  columns size = "+dataColumnNames.size());
        String key = dataColumnNames.get(column);
        return dataObject.getData(key);
    	}
    	catch( IndexOutOfBoundsException e){
    		return null;}
    	
    }

    public void setDataAt(int row, int col, Object val, DataTableListener dtl){
        DataObject dObj = getDataObjectAt(row);
        String colName = getDataColumnName(col);
        dObj.addData(colName, val);
	dtl.updateEvent(dObj, colName, val);
    }

    /**
     * Give the number of row stored in the DataTable
     * @return int
     */
    public int getRowCount() { return dataObjects == null ? 0 : dataObjects.size(); }
    
    /**
     * @return int, number of column in the DataTable.
     */
    public int getColumnCount() { return dataColumnNames == null ? 0 : dataColumnNames.size(); }
    
    /**
     * @return String, name of data column i in the DataTable.
     */
    public String getDataColumnName(int i) {
	//        Debug.message("DataStorage", "[DataTable]getDataColumnName() i = "+ i+" nbColumn =  "+dataColumnNames.size());
        if (dataColumnAliases != null && dataColumnAliases.size() > i) return dataColumnAliases.get(i);
        return getRealColumnName(i);
    }
    
   /**
     * @return String, name of data column i in the DataTable.
     */
    public String getRealColumnName(int i) {
	//        Debug.message("DataStorage", "[DataTable]getRealColumnName() i = "+ i+" nbColumn =  "+dataColumnNames.size());
        if (dataColumnNames != null && dataColumnNames.size() > i) return dataColumnNames.get(i);
        return "Not defined";
    }
   
    /**
     * @return int, the index of column.
     **/
    public int getColumnIndex(String columnName){
        return dataColumnNames.indexOf(columnName);
    }

    /**
     * @return String, name of OMGraphic column i in the DataTable.
     **/
    public String getOMColumnName(int i){
	if (i <0 || omColumnNames == null || omColumnNames.size() <= i)
	    return "Not defined";
	 return omColumnNames.get(i);
    }

    public int getOMColumnCount() { return omColumnNames.size(); }

    public ArrayList<DataObject> getDataObjects(){return  dataObjects;}
    
//    public void sortDataObjects(String sortParameter){
//    	DataObject.sortParameter = sortParameter;
//		ArrayList<DataObject> dObjsTmp = new ArrayList<DataObject>();
//		for(DataObject dObj: dataObjects){
//			if(dObjsTmp.isEmpty())
//				dObjsTmp.add(dObj);
//			else{
//				for(DataObject dObjTmp : dObjsTmp){
//					if(dObjTable.compareTo(dObjTableTmp) < 0){
//						keyValuesTmp.add(i, s);
//						added = true;
//						break;
//					}
//				}
//				if(!added)
//					keyValuesTmp.add(s);
//			}
//		}
//    	System.out.println("SET TABLLLLLLLLLLLLLLLLLLLLLLE");
//    	Collections.sort(dataObjects);
//    	setTable(dataObjects, dataColumnNames, omColumnNames, dataColumnAliases);
//    	
//    }


}
