// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>


package fr.unicaen.openmap.dataStorage;

/**
 *
 * @author jchauvea
 */
import fr.unicaen.openmap.dataStorage.SortDataStorage.Format;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;

public class DateDataObjectComparator extends DataObjectComparator {
    SimpleDateFormat dateFormat;

    public DateDataObjectComparator(String sortParameter, Format format) {
	super(sortParameter, format);
	dateFormat= new SimpleDateFormat(format.getInnerFormat());
    }
    
    @Override
	public int compare(DataObject d1, DataObject d2) {
	if (sortParameter == null)
	    return 0;
        Object data1 = d1.getData(sortParameter);
	Object data2 = d2.getData(sortParameter);
	if (data1 == null) 
	    return data2 == null ? 0 : -1;
	
        try {
            Date date1 = dateFormat.parse(data1.toString());
            Date date2 = dateFormat.parse(data2.toString());
            Long time1 = date1.getTime();
            Long time2 = date2.getTime();
            return time1.compareTo(time2);

        } catch (ParseException ex) {
            Logger.getLogger(DateDataObjectComparator.class.getName()).logp(Level.SEVERE, "SortDataStorage", "Bad format ", data1.toString());
        }

        return 0;
    }

}
