// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * DualDataStorage.java
 *
 * Created on 25 october 2007
 *
 * @author Jacques.Madelaine@info.unicaen.fr
 *
 */

package fr.unicaen.openmap.dataStorage;


import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import com.bbn.openmap.I18n;
import com.bbn.openmap.BasicI18n;

import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.dataStorage.gui.MultiDataTableModel;
import fr.unicaen.openmap.dataStorage.gui.DataTableStoragePanel;
import fr.unicaen.openmap.dataStorage.gui.DataTableModel;

import javax.swing.table.TableModel;
import java.awt.event.*;
import javax.swing.*;
import java.awt.Color;
import java.util.HashMap;
import java.util.Map;

/** DualDataStorage is the result of the natural join of 2 DataSorages.

*/
public class DualDataStorage extends RemovableDataStorageAdaptor {
    /* --------------------------  Begin   Properties  ---------------------*/
    public static final String ds0PropertyString = "ds0";
    public static final String ds1PropertyString = "ds1";
    public static final String dataStorageClassPropertyString = "dataStorageClass";
    public static final String joinColumnNamePropertyString = "joinColumnName";        
    /* --------------------------  End     Properties    -------------------*/
    /** suffixe added in case of attribute name clash */
    private static final String  DUPLICATE_SUFFIXE = "_2";
 
    private String joinColumnName;
    private HashedDataStorage ds1;
    
    public DualDataStorage(DataStorageListener dsl) {
	super(dsl, null);
    }

    public DualDataStorage(DataStorageListener dsl, DataStorage ds0, String joinColumnName0, DataStorage ds1, String joinColumnName1) {
	super(dsl, ds0);
	this.joinColumnName = joinColumnName0;
	this.ds1 = new HashedDataStorage(this, ds1, joinColumnName1);
	ds0.setDataStorageListener(this);
    }

    /** The new DualDataStorage Listener will be the one of ds0 */
    public DualDataStorage(DataStorage ds0, String joinColumnName0, DataStorage ds1, String joinColumnName1) {
	this(ds0.getDataStorageListener(), ds0, joinColumnName0, ds1, joinColumnName1);
    }

    public DualDataStorage(DataStorageListener dsl, DataStorage ds0, String joinColumnName0, HashedDataStorage ds1) {
	super(dsl, ds0);
	this.joinColumnName = joinColumnName0;
	this.ds1 = ds1;
	ds0.setDataStorageListener(this);
	ds1.setDataStorageListener(this);
    }

    /**
     * Tells if the dataStorage has its data.
     * <p>DualDataStorage is ready as soon as all its components are ready.
     */
    public boolean isReady() {
        boolean ret = getDS0() != null && getDS0().isReady() && getDS1() != null && getDS1().isReady();
	if (ret) // in case ds0 and ds1 are ready but the data table model is not ready. 
	    getDefaultDataTableModel();
	return ret;
    }
    
    public void initialize() {
	if (isReady()) {
	    Debug.message("DataStorage", "[DualDataStorage] data ready ***");
	    
	    // We can now (re)build the DataTableModel
	    dataTableModel = newMultiDataTableModel();
	    if (dtsp == null)
		dtsp = new DataTableStoragePanel(this, dataTableModel);
	    else
		((DataTableStoragePanel)dtsp).setModel(dataTableModel);
	    // tell the listener we're ready
	    fireDataStorageEvent();

	    // refresh GUI
	}
    }

    public DataStorage getSecondDataStorage() { return getDS1() == null ? null : getDS1().getDataStorage(); }

    private HashMap<String, String> dataColumnNamesMapping;
    private HashMap<String, String> omColumnNamesMapping;

    private DataTableModel newMultiDataTableModel() {
	
	// Built dataColumnNames and OMColumnNames arrays from the DataStorages DataTableModel
	// Shall we remove the join column ?
 
	DataTableModel[] tableModels = new DataTableModel[2];
	tableModels[0] = getDS0().getDefaultDataTableModel();
	tableModels[1] = getDS1().getDefaultDataTableModel();
	if (tableModels[0] == null || tableModels[1] == null)
	    return null;
	int nbDataColumns = 0, nbOMColumns = 0;
	for (int i = 0; i < 2; i++) {
	    nbDataColumns += tableModels[i].getColumnCount();
	    nbOMColumns += tableModels[i].getOMColumnCount();
	    // Debug.message("DataStorage", "nbDataColumns = " + nbDataColumns + " nbOMColumns = " + nbOMColumns);
	}
	// Debug.message("DataStorage", "nbDataColumns = " + nbDataColumns + " nbOMColumns = " + nbOMColumns);
	String[] dataColumnNames = new String[nbDataColumns];
	dataColumnNamesMapping = new HashMap<String, String>();
	{
	    int x = 0;
	    for (int i = 0 ; i < 2; i++)
		for (int j = 0; j < tableModels[i].getColumnCount(); j++) {
		    // System.out.println(i + " " + j + " " + x + " " +  tableModels[i].getColumnName(j));
		    String dataColumnName = tableModels[i].getColumnName(j);
		    String mappedDataColumnName = dataColumnName;
		    if (i == 1) {
			if (contains(dataColumnNames, x, dataColumnName)) 
			    mappedDataColumnName += DUPLICATE_SUFFIXE; // rename in case of name clash...
			dataColumnNamesMapping.put(dataColumnName, mappedDataColumnName);
		    }
		    dataColumnNames[x++] = mappedDataColumnName;
		}
	}
	String[] omColumnNames = new String[nbOMColumns];
	omColumnNamesMapping = new HashMap<String, String>();
	{
	    int x = 0;
	    for (int i = 0 ; i < 2; i++)
		for (int j = 0; j < tableModels[i].getOMColumnCount(); j++) {
		    String omColumnName = tableModels[i].getOMColumnName(j);
		    String mappedOmColumnName = omColumnName;
		    if (i == 1) {
			if (contains(omColumnNames, x, omColumnName)) // rename in case of name clash...
			    mappedOmColumnName += DUPLICATE_SUFFIXE;
			omColumnNamesMapping.put(omColumnName, mappedOmColumnName);
		    }
		    omColumnNames[x++] = mappedOmColumnName;
		}
	}
	return new MultiDataTableModel(this, dataColumnNames, omColumnNames);
    }
	
    private static boolean contains(String[] table, int n, String name) {
	for (int i = 0; i < n; i++)
	    if (name.equals(table[i]))
		return true;
	return false;
    }
    public DataTableModel getDefaultDataTableModel() {	    
	if (dataTableModel == null) {
	    Debug.message("DataStorage", "[DualDataStorage.getDefaultDataTableModel()] is null "); 
	    dataTableModel = newMultiDataTableModel();
	}
	
	return dataTableModel;
    }

    // <joinColumnName ComboBox management>
    private JComboBox joinColumnComboBox = new JComboBox();;

    // prevent JComboBox to set selected the added item
    private boolean _inhibitSetJoinColumnName = false;
    protected boolean isSetJoinColumnNameInhibited() { return _inhibitSetJoinColumnName; }

    private void resetJoinColumnComboBox(String[] names) {
	joinColumnName = "";
	setJoinColumnComboBox(names);
    }

    private void setJoinColumnComboBox(String[] names) {
	_inhibitSetJoinColumnName = true;
	joinColumnComboBox.removeAllItems();
	for (String s : names)
	    joinColumnComboBox.addItem(s);
	_inhibitSetJoinColumnName = false;
	joinColumnComboBox.setSelectedIndex(-1);
    }
    // </joinColumnName ComboBox management>


    // <Warning message>
    private JLabel warningMessage = new JLabel();
    // </Warning message>
    
    public void refreshGUI() { 
	Debug.message("DataStorage", "DualDataStorage.refreshGUI() not implemented !!!");
    }
    public JComponent getGUI() {
	Debug.message("DataStorage", "DualDataStorage.getGUI()");
	JPanel palette = new JPanel();	
	palette.setLayout(new BoxLayout(palette, BoxLayout.Y_AXIS));
	JPanel box;

	box = new JPanel();
	box.setLayout(new BoxLayout(box, BoxLayout.Y_AXIS));
	// box.setBorder(BorderFactory.createLineBorder(Color.black));
	box.setBorder(BorderFactory.createTitledBorder(""));

	if (getDS1() == null) {
	    warningMessage.setText("No 2d DataStorage");
	    return palette;
	}
	box.add(getDS1().getGUI());

	palette.add(box);
	palette.add(new JLabel(Util.loadIcon("vertical-equal.png")));
	box = new JPanel();
	box.setLayout(new BoxLayout(box, BoxLayout.Y_AXIS));
	// box.setBorder(BorderFactory.createLineBorder(Color.black));

	if (getDS0() == null) {
	    warningMessage.setText("No 1rst DataStorage");
	    return palette;
	}

	box.setBorder(BorderFactory.createTitledBorder(""));
	JPanel joinColumnNamePanel = new JPanel();
	joinColumnNamePanel.add(new JLabel(i18n.get(this, "joinAttribute", "Join attribute ")));
	if (getDS0().getDefaultDataTableModel() != null) {
	    setJoinColumnComboBox(getDS0().getDefaultDataTableModel().getDataColumnNames());
	    Debug.message("DataStorage", "[DualDataStorage:] set joinColumnComboBox to '"+joinColumnName+"'");
	    joinColumnComboBox.setSelectedIndex(getDS0().getDefaultDataTableModel().getIndexForColumnName(joinColumnName));
	}
	joinColumnComboBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    if (getDS0().getDefaultDataTableModel() != null) {
			int index = ((JComboBox)e.getSource()).getSelectedIndex();
			if (index >= 0 && ! isSetJoinColumnNameInhibited()) {
			    setJoinColumnName(getDefaultDataTableModel().getRealColumnName(index));
			    // <Warning message>
			    warningMessage.setText("");
			    // </Warning message>
			}
		    }}});
	joinColumnNamePanel.add(joinColumnComboBox);
	
	box.add(joinColumnNamePanel);
	
	box.add(getDS0().getGUI());

	palette.add(box);

	return palette;
    }
    
    public JComponent getDataGUI(){
	if (dtsp == null) {
	    dataTableModel = getDefaultDataTableModel();
	    // and the data panel
	    dtsp = new DataTableStoragePanel(this, dataTableModel);
	}
	return dtsp;
    }
        
    public void setProperties(String prefix, java.util.Properties properties) {
	super.setProperties(prefix, properties);
	prefix = PropUtils.getScopedPropertyPrefix(prefix);

	String ds0Prefix = prefix + ds0PropertyString + ".";
	String ds0ClassName = properties.getProperty(ds0Prefix + dataStorageClassPropertyString);
	setDataStorage(Util.loadDataStorage(ds0ClassName, ds0Prefix, properties, this));
	joinColumnName = properties.getProperty(ds0Prefix + joinColumnNamePropertyString, "");	

	String ds1Prefix = prefix + ds1PropertyString + ".";
	String ds1ClassName = properties.getProperty(ds1Prefix + dataStorageClassPropertyString);
	ds1 = (HashedDataStorage)Util.loadDataStorage(ds1ClassName, ds1Prefix, properties, this);	
    }

    public java.util.Properties getProperties(java.util.Properties properties) {
	return getProperties("", properties);
  }


    public java.util.Properties getProperties(String prefix, java.util.Properties properties) {
	properties = super.getProperties(prefix, properties);
	if (getDS0() != null) { 
	    String ds0Prefix = prefix + ds0PropertyString + ".";
	    properties.put(ds0Prefix + dataStorageClassPropertyString, getDS0().getClass().getCanonicalName());
	    properties = getDS0().getProperties(ds0Prefix, properties);
	    properties.put(ds0Prefix + joinColumnNamePropertyString, joinColumnName);	
	}
	if (getDS1() != null) { 
	    String ds1Prefix = prefix + ds1PropertyString + ".";
	    properties.put(ds1Prefix + dataStorageClassPropertyString, getDS1().getClass().getCanonicalName());
	    properties = getDS1().getProperties(ds1Prefix, properties);
	}
	return properties;
    }


    public DataObject getElementAt(int index) {
	DataObject obj = getDS0().getElementAt(index);
	Object joinColumnObjectValue = obj.getData(joinColumnName);
	if (joinColumnObjectValue == null)
	    return obj;
	String joinColumnValue = obj.getData(joinColumnName).toString();
	// System.out.print("DualDataStorage.getElementAt("+index+") '"+joinColumnValue+"' =>");

	DataObject d = getDS1().getElementForJoinValue(joinColumnValue);
	if (d == null) 
	    return obj;
	// System.out.print(" + "+d);

	if(dataColumnNamesMapping != null && omColumnNamesMapping != null){
		for (Map.Entry<String, String> e : dataColumnNamesMapping.entrySet())
		    obj.addData(e.getValue(), d.getData(e.getKey()));
		for (Map.Entry<String, String> e : omColumnNamesMapping.entrySet())
		    obj.addOMDataLocation(e.getValue(), d.getOMDataLocation(e.getKey()));
	}

	// System.out.println(" => "+obj);
	return obj;
    }
    private int countWarning = -1; // used to control warning message output

    public int getCount() {
	if (getDS0() == null) {
	    Debug.error("[DualDataStorage] ###Error null DataStorage");
	    return 0;
	}
	int count = getDS0().getCount();

	if (Debug.debugging("DualDataStorage")) {
	    if (getDS1() == null) {
		Debug.error("[DualDataStorage] ###Error null 2nd DataStorage");
		return 0;
	    }
	    if (count != getDS1().getCount() && countWarning != count) {
		countWarning = count;
		System.err.println(String.format("## Warning ##: [DualDataStorage] different row counts for 2nd data storage, founded: %d, expecting: %d", getDS1().getCount(), count));
	    }
	}

	return count;
    }

    public void refresh() { this.dataStorageEvent(this); }

    public void dataStorageEvent(DataStorage source) {
        Debug.message("DataStorage", classNameForDebug + " dataStorageEvent <== " + source);
        if (getDataStorage() != null && getDataStorage().isReady() && ds1 != null && ds1.isReady()) {
            Debug.message("DataStorage", classNameForDebug + " data ready ***");
            initialize();
        }
    }

    protected DataStorage getDS0() { return getDataStorage(); }
    protected HashedDataStorage getDS1() { return ds1; }
    protected String getJoinColumnName() { return joinColumnName; }
    protected void setJoinColumnName(String s) { 
	joinColumnName = s; 
	Debug.message("DataStorage", "[DualDataStorage] setting joinColumnName to:" + s);
    }
    
    public void setWarning(boolean b) { if (getDS1() != null) getDS1().setWarning(b); }
}
