// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage;
import com.bbn.openmap.omGraphics.EditableOMCircle;
import com.bbn.openmap.omGraphics.EditableOMGraphic;
import com.bbn.openmap.omGraphics.EditableOMGraphicList;
import com.bbn.openmap.omGraphics.EditableOMLine;
import com.bbn.openmap.omGraphics.EditableOMPoint;
import com.bbn.openmap.omGraphics.EditableOMPoly;
import com.bbn.openmap.omGraphics.OMAction;
import com.bbn.openmap.omGraphics.OMCircle;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMLine;
import com.bbn.openmap.omGraphics.OMPoint;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.tools.drawing.DrawingToolRequestor;
import com.bbn.openmap.tools.drawing.OMLineLoader;
import com.bbn.openmap.tools.drawing.OMPointLoader;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.dataStorage.gml.out.GMLWriter;
import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import fr.unicaen.openmap.dataStorage.gui.DataTableStoragePanel;
import fr.unicaen.openmap.dataStorage.gui.EditableDataTableModel;
import fr.unicaen.openmap.dataStorage.gui.EditableDataTableStoragePanel;
import fr.unicaen.openmap.dataStorage.out.CSVWriter;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.tools.drawing.TMDrawingTool;
import fr.unicaen.openmap.tools.drawing.TMDrawingToolLauncher;
import fr.unicaen.openmap.tools.drawing.TMPolyLineLoader;
import fr.unicaen.openmap.tools.drawing.TMPolygonLoader;
import fr.unicaen.openmap.util.LonLatNames;
import fr.unicaen.openmap.util.Util;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

/**
 * @author jchauvea
 */
public class EditableDataStorage extends RemovableInformationDataStorage implements DrawingToolRequestor, DataTableListener {

    public static final String OM_DATA_LOCATION = "OMDataLocation";

    private TMDrawingTool drawingTool;
    private TMDrawingToolLauncher drawingLauncher;
    private DrawingToolRequestor requestor;
    private Box gui;
    protected ArrayList<DataObject> dataObjects;
    private String dataColumn;
    private String omColumn;

    public EditableDataStorage(DataStorageListener dsl) {
        this(dsl, null);
    }

    public EditableDataStorage(DataStorageListener dsl, DataStorage dataStorage) {
	super(dsl, dataStorage);
        dataTableModel = new EditableDataTableModel(this, dataTable);
        dtsp = new EditableDataTableStoragePanel(this, dataTableModel);
        drawingTool = new TMDrawingTool();
        drawingTool.addLoader(new OMPointLoader());
        // drawingTool.addLoader(new OMLineLoader()); don't seem to work!!!
	drawingTool.addLoader(new TMPolyLineLoader());
        drawingTool.addLoader(new TMPolygonLoader());
        drawingLauncher = new TMDrawingToolLauncher();
        drawingLauncher.setDrawingTool(drawingTool);
	drawingLauncher.setCurrentRequestor(this);
        initialize();
    }


    @Override
    public boolean isReady() {
        return true;
    }

    public JComponent getGUI() {
        if (gui == null) {
            gui = Box.createVerticalBox();
	    JPanel wrapDrawingLauncher = new JPanel();
	    wrapDrawingLauncher.add(new JLabel(i18n.get(this, "guiLabel", "Use right clic on the map to create/edit shapes")));
	    gui.add(wrapDrawingLauncher);
	    
            Box panCombos = Box.createVerticalBox();
            JPanel wrapComboDataCols = new JPanel();
            JPanel wrapComboOMCols = new JPanel();
	    if (getDataStorage().getDefaultDataTableModel() == null)
		return null;
            final JComboBox comboDataCols = new JComboBox(getDataStorage().getDefaultDataTableModel().getDataColumnNames());
            final JComboBox comboOMCols = new JComboBox(getDataStorage().getDefaultDataTableModel().getOMColumnNames());
            comboDataCols.addItemListener(new ItemListener() {
                public void itemStateChanged(ItemEvent arg0) {
                    dataColumn = comboDataCols.getSelectedItem().toString();
                }
            });
            comboOMCols.addItemListener(new ItemListener() {
                public void itemStateChanged(ItemEvent arg0) {
                    omColumn = comboOMCols.getSelectedItem().toString();
                }
            });

            if(omColumn == null) {
		if (comboOMCols.getSelectedItem() != null)
		    omColumn = comboOMCols.getSelectedItem().toString();
		else 
		    ; // warning : unable to create graphic...
            } else
                comboOMCols.setSelectedItem(omColumn);
            if(dataColumn == null)
                dataColumn = comboDataCols.getSelectedItem().toString();
            else
                comboDataCols.setSelectedItem(dataColumn);
            
            panCombos.add(wrapComboDataCols);
            panCombos.add(wrapComboOMCols);
            wrapComboDataCols.add(new JLabel(i18n.get(this, "indexCol", "Index column")));
            wrapComboDataCols.add(comboDataCols);
            wrapComboOMCols.add(new JLabel(i18n.get(this,"graphicCol","Graphic column")));
            wrapComboOMCols.add(comboOMCols);
            gui.add(panCombos);
            gui.add(getDataStorage().getGUI());
            gui.setBorder(BorderFactory.createTitledBorder(i18n.get(this,"title","Edition/Selection/Drawing tool")));
        }
        return gui;
    }

    private boolean drawingToolCanvasIsSet = false;
    private void setDrawingToolCanvas() {
	if (drawingToolCanvasIsSet)
	    return;
	if (getDataStorageListener()  instanceof ThematicDataDisplayLayer) {
	    // System.out.println("canvas set to mapbean for " + drawingTool.hashCode());
	    drawingTool.setCanvas((JComponent) ((ThematicDataDisplayLayer)	getDataStorageListener()).getMapBean());
	}
	drawingToolCanvasIsSet = true;
    }

    public List<Component> getItemsForOMGraphicMenu(final OMGraphic omg) {
	setDrawingToolCanvas();
	// inhibit OMGraphic menu when editing an OmGraphic
	if (drawingTool.getCurrentEditable() != null)       
	    return null;
        ArrayList<Component> components = new ArrayList<Component>();
        JMenuItem item;

	item = new JMenuItem(i18n.get(ThematicDataDisplayLayer.class, "delete", "Delete Graphic"));
	components.add(item);
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                if (JOptionPane.showConfirmDialog(null,
                        i18n.get(ThematicDataDisplayLayer.class, "deleteConfirm", "Do you really want to delete this graphic?"),
                        i18n.get(ThematicDataDisplayLayer.class, "deleting", "deleting a graphic"),
                        JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
                    Object o;
                    if ((o = omg.getAttribute(ThematicDataDisplayLayer.INDEX_IN_DS)) != null) {
                        if (removeOMGraphicAt((Integer) o, omg)) {
			    fireDataStorageEvent(); // refresh the layer
                        }
                    }
                }
            }
        });

	item = new JMenuItem(i18n.get(ThematicDataDisplayLayer.class, "deleteAssociated", "Delete Graphic and associated data"));
	components.add(item);
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                if (JOptionPane.showConfirmDialog(null,
                        i18n.get(ThematicDataDisplayLayer.class, "deleteConfirm", "Do you really want to delete this graphic?"),
                        i18n.get(ThematicDataDisplayLayer.class, "deleting", "deleting a graphic"),
                        JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
                    Object o;
                    if ((o = omg.getAttribute(ThematicDataDisplayLayer.INDEX_IN_DS)) != null) {
                        removeDataObjectAt((Integer) o);
                    }
                }
            }
        });

        item = new JMenuItem(i18n.get(ThematicDataDisplayLayer.class, "edit", "Edit Graphic"));
	components.add(item);
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                Object o;
                if ((o = omg.getAttribute(ThematicDataDisplayLayer.INDEX_IN_DS)) != null) {
                    editOMGraphicAt((Integer) o, omg);
		    fireDataStorageEvent(); // refresh the layer
                }
            }
        });

        return components;
    }

    public List<Component> getItemsForMapMenu(java.awt.event.MouseEvent me) {
	setDrawingToolCanvas();
	// inhibit map menu when editing an OMGraphic
	if (drawingTool.getCurrentEditable() != null)       
	    return null;
	return drawingLauncher.getContextualMenu();
    }

    /* used to sync lon/lat geom with data */
    private String lonLatName = null;
    private DataObject lonLatDataObject = null;

    public void drawingComplete(OMGraphic omg, OMAction action) {
	// System.out.println("** drawingComplete() " + omg + " "+ action);
	if (action.isMask(OMGraphic.UPDATE_GRAPHIC_MASK)) {
	    if (lonLatName != null) {
		// update lon lat data columns accordingly
		String[] dataLonLatNames = lonLatName.split("/");
		boolean blon = lonLatDataObject.addData(dataLonLatNames[0], ((OMPoint)omg).getLon());
		boolean blat = lonLatDataObject.addData(dataLonLatNames[1], ((OMPoint)omg).getLat());
		// System.out.println(blon+" " +blat+ " "+lonLatDataObject);
	}

	    fireDataStorageEvent(); // refresh the layer
	    return;
	}
	// System.out.println("** drawingComplete() " + omg + " "+ action);

        DataObject dObj = new DataObject();
        OMDataLocation omdl = new OMDataLocation(omg);
        dObj.addData(dataColumn, getCount() + 1);
        dObj.addOMDataLocation(omColumn, omdl);
        dataObjects.add(dObj);
	Debug.message("DataStorage", "+++ Adding dataObject "+ dObj);
	setDirty(true);
        setTable(); // will send a dataStorageEvent
    }

    public void updateEvent(DataObject dataObject, String colName, Object value) {
	// System.out.println("Update "+ colName + " with " + value);
	String lonLatName = LonLatNames.getLonLatName(colName, 
						      getDefaultDataTableModel().getDataColumnNames(), 
						      getDefaultDataTableModel().getOMColumnNames());	   
	if (lonLatName != null) {    // update the geometry from lon/lat data
	    //  get the OMDataLocation holding the omg to nullify 
	    // the centroid and the bounding box that will become false
	    OMDataLocation omdl = dataObject.getOMDataLocation(lonLatName);
	    omdl.setCentroid(null);
	    omdl.setBoundingBox(null);
	    OMPoint omg = (OMPoint) omdl.getOMGraphic();
	    // omg.putAttribute(OM_DATA_LOCATION, omdl);
	    // System.out.println("Update "+ omg + " with " + lonLatName);

	    String[] dataLonLatNames = lonLatName.split("/");
	    omg.setLon(Util.parseDouble(dataObject.getData(dataLonLatNames[0])));
	    omg.setLat(Util.parseDouble(dataObject.getData(dataLonLatNames[1])));

	    setTable(); // will send a dataStorageEvent
	}
    }


    private boolean _dirty = false;
    public boolean isDirty() { return _dirty; }
    private void setDirty(boolean b) { _dirty = b; }

    public void write(CSVWriter writer) {
	super.write(writer);
	setDirty(false);
    }

    public void write(GMLWriter writer) {
	super.write(writer);
	setDirty(false);
    }

    /** returns the original underlying DataStorage and update the listeners */
    public DataStorage removeItSelf() {
	if (isDirty() && JOptionPane.showConfirmDialog(null,
						       i18n.get(this, "exitWithoutSaving", "Geometries added. Do you want to exit without exporting your editions?"),
						       i18n.get(fr.unicaen.openmap.dataStorage.gui.AdaptorButtonArray.class, 
								"removeAdaptorConfirmation", "Confirmation"),
						       JOptionPane.YES_NO_OPTION) != JOptionPane.YES_OPTION)
	    return this;
	return super.removeItSelf();	
    }

    public void initialize() {
	if (getDataStorage() == null)
	    return;
        dataObjects = new ArrayList<DataObject>(getDataStorage().getCount());
        for (int i = 0; i < getDataStorage().getCount(); i++) {
            dataObjects.add(getDataStorage().getElementAt(i));
        }
        setTable();
    }

    protected boolean setTable() {

        if (dataObjects == null) {
            return false;
        }
	DataTableModel _defaultDataTableModel = getDataStorage().getDefaultDataTableModel();
	if (_defaultDataTableModel == null)
	    return false;
        dataTable.setTable(dataObjects,
                new ArrayList<String>(Arrays.asList(_defaultDataTableModel.getDataColumnNames())),
                new ArrayList<String>(Arrays.asList(_defaultDataTableModel.getOMColumnNames())),
                getDataStorage().getDataAttributesNamesAliases());
        ((DataTableStoragePanel) dtsp).fireStructureChanged();
        fireDataStorageEvent();
        return true;
    }

    public boolean removeOMGraphicAt(int index, OMGraphic omg) {
        DataObject dObj = getElementAt(index);
        for (String s : getDefaultDataTableModel().getOMColumnNames()) {
            OMGraphic tmpOmg = dObj.getOMDataLocation(s).getOMGraphic();
            if (tmpOmg == null) continue;
            if (tmpOmg instanceof OMGraphicList) {
                OMGraphicList l = (OMGraphicList) tmpOmg;
                if (l.contains(omg)) {
                    l.remove(omg);
                    return true;
                }
            }
            if (tmpOmg == omg) {
                dObj.addOMDataLocation(s, new OMDataLocation());
                return true;
            }
        }
        return false;
    }

    public void removeDataObjectAt(int index) {
        dataObjects.remove(index);
        setTable();
    }
    
    public void editOMGraphicAt(int index, OMGraphic omg) {
	Debug.message("DataStorage", "## editOMGraphicAt "+index + " " + omg);
        if (omg instanceof OMGraphicList) {
	    // the selected OMGraphic is wrapped in a list
	    editOMGraphicAt(index, ((OMGraphicList) omg).getOMGraphicAt(0));
            return;
        }
	//  get the OMDataLocation holding the omg to nullify 
	// the centroid and the bounding box that will become false
	DataObject dataObject = getElementAt(index);
 	OMDataLocation omdl = null;
 	String name = null;
	for (String omColumnName : getDefaultDataTableModel().getOMColumnNames()) {
	    // System.out.println("-- " + omColumnName + " " + omg);
	    omdl = dataObject.getOMDataLocation(omColumnName);
            if (omdl == null)
                continue;
            OMGraphic tmpOmg = omdl.getOMGraphic();
	    // System.out.println("== " + omColumnName + " " + tmpOmg + " " + omdl);
            if (tmpOmg == null) 
		continue;
            if (tmpOmg == omg || 
		(tmpOmg instanceof OMGraphicList && ((OMGraphicList) tmpOmg).contains(omg))) {
		name = omColumnName;
		break;
	    }
	}
	if (name != null) {
	    // System.out.println("## editOMGraphicAt "+index + " " + omg + " " + omdl);
	    omdl.setCentroid(null);
	    omdl.setBoundingBox(null);
	    omg.putAttribute(OM_DATA_LOCATION, omdl);
	} else
	    Debug.error("No OMDataLocation for " + omg + " at index " + index);
	
 	lonLatName = getLonLatGeometryColumnName(dataObject, omg);
	if (lonLatName != null) {
	    // drawingComplete will sync lon/lat data with the OMPoint new position
	    lonLatDataObject = dataObject;
	}

        EditableOMGraphic eomg = drawingTool.getEditableGraphic(omg);
	eomg.setActionMask(OMGraphic.UPDATE_GRAPHIC_MASK);
        drawingTool.setCurrentEditable(null);
        OMGraphic newOMG = drawingTool.edit(eomg, this);
	// System.out.println("## "+(newOMG == omg) + " " + newOMG + " " + omg);
    }
    
    /** returns the geometryColumnName of the omg if it is a builtin centroid from lon/lat data, null otherwise */
    private String getLonLatGeometryColumnName(DataObject dataObject, OMGraphic omg) {
	if (! (omg instanceof OMPoint))
	    return null;
    	String name = dataObject.getGeometryColumnName(omg);
    	return LonLatNames.isAGeometryLonLatName(name) ? name : null;
    }
    
  
    public static EditableOMGraphic makeEditableOMGraphicfor(OMGraphic omg) {
        if (omg instanceof OMPoly)
            return new EditableOMPoly((OMPoly) omg);
        if (omg instanceof OMPoint)
            return new EditableOMPoint((OMPoint) omg); 
        if (omg instanceof OMCircle)
            return new EditableOMCircle((OMCircle) omg);
        if (omg instanceof OMLine)
            return new EditableOMLine((OMLine) omg);
        if (omg instanceof OMGraphicList) {
            EditableOMGraphicList eomgl = new EditableOMGraphicList();
            for (OMGraphic o : (OMGraphicList) omg) {
                eomgl.add(makeEditableOMGraphicfor(o));
            }
            return eomgl;
        }
        return null;
    }
    public static final String editDSPropertyString = "editDatastorage";
    public static final String dataStorageClassPropertyString = "dataStorageClass";
    public static final String dataColumnPropertyString = "dataColumn";
    public static final String omColumnPropertyString = "omColumn";


    public void setProperties(String prefix, Properties properties) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        super.setProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + editDSPropertyString + ".";
        String dsClassName = properties.getProperty(prefix + dataStorageClassPropertyString, "");
        dataColumn = properties.getProperty(prefix + dataColumnPropertyString, "");
        omColumn = properties.getProperty(prefix + omColumnPropertyString, "");
        if ("".equals(dsClassName)) {
            return;
        }
        setDataStorage(Util.loadDataStorage(dsClassName, prefix, properties, this));
        initialize();
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + editDSPropertyString + ".";
        properties.put(prefix + dataColumnPropertyString, dataColumn);
        properties.put(prefix + omColumnPropertyString, omColumn);
        if (getDataStorage() != null) {
            properties.put(prefix + dataStorageClassPropertyString, getDataStorage().getClass().getCanonicalName());
            // Debug.message("DataStorage", "PivotDataStorage.getProperties() - DataStorage member is a: "+dataStorage.getClass().getCanonicalName());
            properties = getDataStorage().getProperties(prefix, properties);
        }
        return properties;

    }

}
