// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * MonoValuedFilterDataStorage.java
 *
 * Created on 27 february 2008
 *
 * @author Jacques.Madelaine@info.unicaen.fr
 *
 */
package fr.unicaen.openmap.dataStorage;

import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;

import fr.unicaen.openmap.animation.MetaAnimator;

import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import fr.unicaen.openmap.layer.dataDisplayLayer.SimpleDataDisplayLayer;
import fr.unicaen.openmap.util.ImageExporter;
import fr.unicaen.openmap.util.Util;
import java.util.ArrayList;
import java.util.Map;
import java.util.TreeMap;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

/**
 * A FilterDataStorage is a DataStorage made of :
 * <ol>
 * <li>a DataStorage of any type;</li>
 * <li>a column name used for filtering;</li>
 * <li>a value.</li>
 * </ol>
 * Its data is the restriction of the DataStorage rows for which the column
 * value is equal to the given value.
 */
public class MonoValuedFilterDataStorage extends FilterDataStorage {

    public static final String filterColumnNamePropertyString = "filterColumnName";
    public static final String filterColumnValuePropertyString = "filterColumnValue";
    public static final String modePropertyString = "mode";
    public static final String dataStorageClassPropertyString = "dataStorageClass";
    public static final String animationDSPropertyString = "animationDS";
    /** get the filtered DataTable given a value of the filter column */
    protected Map<String, ArrayList<DataObject>> keyMap;
    public static final String noFilter =
            i18n.get(MonoValuedFilterDataStorage.class, "noFilter", "-- no Filter --");
    protected Box animationGUI;
    // <export>
    private Exporter exporter;
    // </export>
    private boolean exportMode;
    private JLabel columnLabel;
    private JLabel valueLabel;
    private JComboBox metaGUI;
    private MetaAnimator metaAnimator;

    private String filterColumnValue = "";
    private int filterColumnValueIndex = -1;

    public MonoValuedFilterDataStorage(DataStorageListener dsl) {
        super(dsl);
    }

    public MonoValuedFilterDataStorage(DataStorageListener dsl, DataStorage dataStorage) {
        this(dsl, dataStorage, null);	
    }

    public MonoValuedFilterDataStorage(DataStorageListener dsl, DataStorage dataStorage,
            String filterColumnName) {
        super(dsl, dataStorage, filterColumnName);	
        initialize();
    }

    /**
     * @param metaAnimator the metaAnimator to set
     */
    public void setMetaAnimator(MetaAnimator metaAnimator) {
        this.metaAnimator = metaAnimator;

    }

    public void setExportMode(boolean b) {
        exportMode = b;
        initFilterExpressionPanel();
        filterExpressionPanel.updateUI();
    }

    public String getFilterColumnName() {
        return filterColumnName;
    }

    protected void setFilterColumnName(String s) {
        filterColumnName = s;
        Debug.message("Animation", "[MonoValuedFilterDataStorage] (re)init index for filterColumnName: " + s);
        createKeyMap(getDataStorage());     
	Debug.message("Animation", "[MonoValuedFilterDataStorage] (re)init index for filterColumnName: " + s + " done");
	// setFilterColumnValue(noFilter);
	updateColumnLabel();
    }

    protected ArrayList<DataObject> getFilteredData() {
        if (keyMap == null)
            return new ArrayList<DataObject>();
	ArrayList<DataObject> ret = keyMap.get(filterColumnValue);
	// Debug.message("DataStorage", "[MonoValuedFilterDataStorage]##" + this + " val " + filterColumnValue + " -> "+ (ret == null ? -1 : ret.size()));
        return keyMap.get(filterColumnValue);
    }

    protected boolean createKeyMap(DataStorage dataStorage) {
        if (dataStorage == null || !dataStorage.isReady()) {
            Debug.message("DataStorage", "[MonoValuedFilterDataStorage] dataStorage " + dataStorage + "not ready, skip keyMap creation");
            return false;
        }

        // setColumnNames();

        if (filterColumnName == null || "".equals(filterColumnName)) {
            Debug.message("DataStorage", "[MonoValuedFilterDataStorage] filterColumnName unset, skip keyMap creation");
            return false;
        }

        keyMap = new TreeMap<String, ArrayList<DataObject>>(comparator);
        // step 1: collect the different filterColumnName values, associating a
        // null ArrayList (to prevent several redondant creation)
        for (int i = 0; i < dataStorage.getCount(); i++) {
            Object d = dataStorage.getElementAt(i).getData(filterColumnName);
            keyMap.put(d == null ? "" : d.toString(), null);
        }
        // step 2: build the keyValues table
	// add room for the "nofilter" index
        keyValues = keyMap.keySet().toArray(new String[keyMap.keySet().size()+1]);
        keyValues[keyValues.length-1] = noFilter;

        // step 3: create the ArrayLists
        for (String value : keyValues) {
            keyMap.put(value, new ArrayList<DataObject>());
        }

        ArrayList<DataObject> fullData = keyMap.get(noFilter);
        keyMap.put(noFilter, fullData);
        for (int i = 0; i < dataStorage.getCount(); i++) {
            DataObject data = dataStorage.getElementAt(i);
            Object d = data.getData(filterColumnName);
            String value = (d == null ? "" : d.toString());
            keyMap.get(value).add(data);
	    fullData.add(data);
	}

        Debug.message("DataStorage", "[MonoValuedFilterDataStorage] keyMap created");
        return true;
    }

    public String getFilterColumnValue() {
        return filterColumnValue;
    }

    protected int getFilterColumnValueIndex() {
        return filterColumnValueIndex;
    }

    public void setFilterColumnValue(String value) {
        if (value == null || keyValues == null) {
            return;
            //value = "";
        }
        filterColumnValue = value;
        filterColumnValueIndex = getIndexOfKeyValue(filterColumnValue);
        setTable();
	updateValueLabel();
    }

    protected void setFilterColumnValueIndex(int index) {
        filterColumnValueIndex = index;
        try {
            filterColumnValue = getKeyValueAt(index);
       } catch (IndexOutOfBoundsException e) {
            filterColumnValue = "";
            filterColumnValueIndex = -1;
        }

        setTable();
    }

    protected int incrFilterColumnValueIndex() {
        return incrFilterColumnValueIndex(true);
    }

    protected int incrFilterColumnValueIndex(boolean setFilter) {
        AnimationDSEventManager.fireChangedAnimation(this);
        int index = filterColumnValueIndex + 1;
        if (index == 0) // 0 is for no-filter
        {
            index = 1;
        }
        if (index >= keyValues.length) {
            index = 1;
        }
        if (setFilter) {
            setFilterColumnValueIndex(index);
        }
        return index;
    }

    /** returns the original DataStorage */
    public DataStorage removeItSelf() {
	DataStorage ds = super.removeItSelf();
        AnimationDSEventManager.fireRemoveAnimation(this);
        return ds;
    }

    @Override
    public void setProperties(String prefix, java.util.Properties properties) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        super.setProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + animationDSPropertyString + ".";
        String dsClassName = properties.getProperty(prefix + dataStorageClassPropertyString, "");
        if ("".equals(dsClassName)) {
            return;
        }
        setDataStorage(Util.loadDataStorage(dsClassName, prefix, properties, this));
        filterColumnName = properties.getProperty(prefix + filterColumnNamePropertyString, "");
        filterColumnValue = properties.getProperty(prefix + filterColumnValuePropertyString, "");
        initialize();
    }

    private boolean addAnimationFired = false;

    public void initialize() {
        if (getDataStorage() == null || ! getDataStorage().isReady()) {
            Debug.message("DataStorage", "[MonoValuedFilterDataStorage] can't initialize because dataStorage is not ready !!!!");
            return;
        }
        setColumnNames();
        dataTableModel = new DataTableModel(dataTable);
        resetDataTableStoragePanel(dataTableModel);
        setFilterColumnName(filterColumnName);
	setFilterColumnValue(filterColumnValue);

	if (! addAnimationFired) {
	    addAnimationFired = true;
	    AnimationDSEventManager.fireAddAnimation(this);
	} 
	// else System.out.println("*** MonoValuedFilterDataStorage double initialize ***"+ this);
    }

    @Override
    public java.util.Properties getProperties(java.util.Properties properties) {
        return getProperties("", properties);
    }

    @Override
    public java.util.Properties getProperties(String prefix,
            java.util.Properties properties) {
        properties = super.getProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + animationDSPropertyString + ".";
        if (getDataStorage() != null) {
            properties.put(prefix + dataStorageClassPropertyString, getDataStorage().getClass().getCanonicalName());
            properties = getDataStorage().getProperties(prefix, properties);
        }
        String fcol = getFilterColumnName() == null ? "" : getFilterColumnName();
        properties.put(prefix + filterColumnNamePropertyString,
                fcol);
        properties.put(prefix + filterColumnValuePropertyString,
                getFilterColumnValue());
        //properties.put(prefix + modePropertyString, selectedMode.toString());
        return properties;
    }

    
    public boolean isReady() {
        return getDataStorage() != null && getDataStorage().isReady() /*&& keyMap != null*/;
    }
    private JButton toolNextButton;

    public void initFilterExpressionPanel() {

        if (filterExpressionPanel == null) {
            filterExpressionPanel = new JPanel(new BorderLayout());
        }
        filterExpressionPanel.removeAll();
        JPanel gridPanel = new JPanel(new GridLayout(2, 2));
        filterExpressionPanel.setBorder(
                BorderFactory.createTitledBorder(i18n.get(this, "filter", "Filter")));
        filterExpressionPanel.add(gridPanel, BorderLayout.CENTER);
        if (getDataStorage() == null) {
            warningMessage.setText(i18n.get(MonoValuedFilterDataStorage.class,
                    "noDSWarning",
                    "<html><font color='red'>No data storage</font></html>"));
            return;
        }


        gridPanel.add(new JLabel(i18n.get(MonoValuedFilterDataStorage.class,
                "filterAttribute", "Filter attribute ")));
        columnLabel = new JLabel(filterColumnName);
        gridPanel.add(columnLabel);

        gridPanel.add(new JLabel(i18n.get(this, "filtervalue", "Filter value")));
        if (filterColumnValue == null || "".equals(filterColumnValue)) {
            valueLabel = new JLabel(i18n.get(MonoValuedFilterDataStorage.class,
                "notConfigured", "Not configured"));
            valueLabel.setFont(new Font("Arial", Font.ITALIC, 12));
        }
        valueLabel = new JLabel(filterColumnValue);
        gridPanel.add(valueLabel);

        if (metaAnimator != null) {
            JButton confFilterButton = new JButton(
                    i18n.get(MonoValuedFilterDataStorage.class,"configure","Configure"));
            JPanel wrapConf = new JPanel();
            confFilterButton.setBackground(Color.gray);
            confFilterButton.setOpaque(false);
            wrapConf.add(confFilterButton);
            filterExpressionPanel.add(
                    wrapConf,
                    BorderLayout.SOUTH);
            confFilterButton.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent arg0) {
                    metaAnimator.popup();
                }
            });

        }

        updateMetaGUI(filterColumnName);
    }

    public JComponent getGUI() {
        if (animationGUI == null) {
            animationGUI = Box.createVerticalBox();// AnimationDataStoragePanel(this);
            initFilterExpressionPanel();
            animationGUI.add(getFilterExpressionPanel());
            animationGUI.add(getDataStorage().getGUI());
        }
        return animationGUI;
    }

    boolean metaGUIIsActive = false;
    public JComponent getMetaGUI() {
        Debug.message("Animaton", "[MonoValuedFilterDataStorage] GET METAGUI " + dataColumnNames.size());
        if (metaGUI == null) {
	    String dataColumnNames[] = getDataStorage().getDefaultDataTableModel().getDataColumnNames();
            metaGUI = new JComboBox(dataColumnNames);
	    // force an empty selection in case of an invalid filterColumnName
	    metaGUI.setSelectedItem(java.util.Arrays.asList(dataColumnNames).contains(filterColumnName) ? filterColumnName : null);

            metaGUI.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
		    if (! metaGUIIsActive) {
			// System.out.println("======== MetaGUI inactive");
			return;
		    }
                    if (getDataStorage() != null) {
                        int index = ((JComboBox) e.getSource()).getSelectedIndex();
                        if (index >= 0) {
                            setFilterColumnName(getDataStorage().getDefaultDataTableModel().getRealColumnName(
                                    index));
                        }
                        AnimationDSEventManager.fireChangedAnimation(MonoValuedFilterDataStorage.this);
                    }
                }
            });
	    	    
	    Debug.message("Animation", "[MonoValuedFilterDataStorage] GET METAGUI \""+ filterColumnName+"\" "+java.util.Arrays.asList(dataColumnNames).contains(filterColumnName) );
	    metaGUIIsActive = true;
        }
        return metaGUI;
    }

    public void updateMetaGUI(String filter) {
        if(metaGUI == null)
            return;
	metaGUIIsActive = false;
        metaGUI.removeAllItems();
	int index = -1; // to store index of filterColumnName
	int i = 0;
        for (String s : dataColumnNames) {
            metaGUI.addItem(s);
	    if (s.equals(filterColumnName))
		index = i;
	    i++;
        }
	
	// if filter is null use filterColumnName unless it is invalid
	// use of setSelectedIndex as the Item seems to change??
	if (filter == null) 
	     metaGUI.setSelectedIndex(index);
	else
	    metaGUI.setSelectedItem("".equals(filter) ? null: filter);
	metaGUIIsActive = true;
	Debug.message("Animation", "[MonoValuedFilterDataStorage] UPDATE METAGUI \""+ filter+"\" \""+ filterColumnName + "\", index: "+index);
    }

    public boolean isOnTop() {
	return getDataStorageListener() instanceof SimpleDataDisplayLayer;
    }

    public JComponent getFilterExpressionPanel() {
        return filterExpressionPanel;
    }


    //<export>
     public void export(String folder, ImageExporter.ImageType type) {
         //setNbStepAnimation(0);
         if (exporter == null) {
             //exporter = new Exporter(folder, getNextValueAction(), type);
         }
         exporter.start();
     }
 
     public void stopExport() {
         exporter.stopThread();
     }
    // </export>

   public String getFullName() {
        return getLayer().getName();
    }

    public String getName() {
        return super.getName()+ " [" + filterColumnName+"]";
    }

    public boolean isSet() {
        return getFilterColumnName() != null && !"".equals(getFilterColumnName());
    }

    public void updateValueLabel() {
	if (valueLabel != null)
	    valueLabel.setText(filterColumnValue);
    }

    public void updateColumnLabel() {
	if (columnLabel != null)
        columnLabel.setText(filterColumnName);
    }

    public String getVariableValueAsString() {
	return getVariableValueAsString("=");
    }

    public String getVariableValueAsString(String sep) {
	return filterColumnName + sep + filterColumnValue;
    }

}

    // <export>
 class Exporter extends Thread {
 
     private String fileName = "AnimationExport-";
     private int index = 0;
     private Action action;
     private String folder;
     private boolean end;
     private ImageExporter.ImageType type;
 
     public Exporter(String folder, Action action, ImageExporter.ImageType type) {
         this.action = action;
         this.folder = folder;
         this.type = type;
     }
 
     public void run() {
         while (!end) {
             ImageExporter.exportInFile(folder + fileName + index + "." + type.toString().toLowerCase(),
                     type);
             index++;
             action.actionPerformed(null);
         }
     }
 
     public synchronized void stopThread() {
         end = true;
     }
 }
// </export>
