// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/**
 * MultiValueFilterDataStorage
 * 
 * Created on 14 april 2008
 *
 * @author  Jacques.Madelaine@info.unicaen.fr
 *
 */
package fr.unicaen.openmap.dataStorage;

import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;

import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import fr.unicaen.openmap.gui.component.CheckListItem;
import fr.unicaen.openmap.gui.component.CheckListRenderer;
import fr.unicaen.openmap.util.UnicaenPropUtils;
import fr.unicaen.openmap.util.Util;

import java.awt.BorderLayout;
import java.util.*;
import java.awt.event.*;
import java.awt.Component;

import javax.swing.*;

/**
 * A MultiValueFilterDataStorage is a DataStorage made of :
 * <ol><li>a DataStorage of any type;</li>
 *     <li>a column name used for filtering;</li>
 *     <li>a set of values.</li>
 * </ol>
 * Its data is the restriction of the DataStorage rows for which the column value is equal to one value of the set of values. 
 */
public class MultiValuedFilterDataStorage extends FilterDataStorage implements ItemListener {

    public static final String dataStorageClassPropertyString = "dataStorageClass";
    public static final String multiFilterDSPropertyString = "multiFilterDS";
    public static final String filterColumnNamePropertyString = "filterColumnName";
    public static final String filterColumnValuesPropertyString = "filterColumnValues";

    	
    public MultiValuedFilterDataStorage(DataStorageListener dsl) {
        this(dsl, null);
    }

    public MultiValuedFilterDataStorage(DataStorageListener dsl, DataStorage dataStorage) {
        this(dsl, dataStorage, null);
    }

    public MultiValuedFilterDataStorage(DataStorageListener dsl, DataStorage dataStorage, String filterColumnName) {
        super(dsl, dataStorage, filterColumnName);
    }

    /** Set the filter column name. Init the key values list */
    protected void setFilterColumnName(String s) { 
	Debug.message("DataStorage", "[MultiValueFilterDataStorage] change filterColumnName to:" + s);
	if (s == null)
	    return;
	filterColumnName = s; 
	// init keyValues
	Set<String> keyValuesSet = new TreeSet<String>(comparator);
	for (int i = 0; i < getDataStorage().getCount(); i++) {
	    Object data = getDataStorage().getElementAt(i).getData(filterColumnName);
	    keyValuesSet.add(data == null ? "" : data.toString());
	}
	keyValues = keyValuesSet.toArray(new String[keyValuesSet.size()]);

	if(filterPanel != null){ 	//bad code ...but it works (hardcore refresh)
	    filterPanel.setVisible(false);
	    filterPanel.setVisible(true);
	}
	Debug.message("MultiValueFilterDataStorage", "[MultiValueFilterDataStorage] Key Values Set Cardinal is : " + keyValues.length); 
    }
    private ArrayList<String> filterColumnValues = new ArrayList<String>();

    public ArrayList<String> getFilterColumnValues() {
        return filterColumnValues;
    }
    //    private ArrayList<Integer> filterColumnValuesIndexes = new ArrayList<Integer>();
    //    protected ArrayList<Integer> getFilterColumnValuesIndexes() { return filterColumnValuesIndexes; }

    protected void setFilterColumnValues(ArrayList<String> values) {
        if (values == null) {
            values = new ArrayList<String>();
        }
        filterColumnValues = values;
        setTable();
    }

    private boolean isFiltered(DataObject data) {
	Object d = data.getData(filterColumnName);
	String dString = d == null ? null : d.toString();
        for (String v : getFilterColumnValues()) {
	    if (v == null)
		return d == null || "".equals(dString);
	    if (d == null)
		return false;
            if (v.equals(dString))
                return true;       
        }
        return false;
    }

    protected ArrayList<DataObject> getFilteredData() {
        ArrayList<DataObject> datas = new ArrayList<DataObject>();

        DataStorageIterator dsi = new DataStorageIteratorAdapter(getDataStorage());
        while (dsi.hasNext()) {
            DataObject data = dsi.next();
            if (isFiltered(data)) {
                datas.add(data);
            }
        }
        return datas;
    }

    public boolean isReady() {
        return getDataStorage() != null && getDataStorage().isReady();
    }

    protected void initFilterExpressionPanel() {
        if (filterExpressionPanel == null) {
            filterExpressionPanel = new JPanel();
            filterExpressionPanel.setLayout(new BorderLayout()/*new BoxLayout(filterExpressionPanel, BoxLayout.Y_AXIS)*/);
        }
        filterExpressionPanel.removeAll();
        if (getDataStorage() == null) {
            warningMessage.setText(i18n.get(this, "noDSWarning", "<html><font color='red'>No data storage</font></html>"));
            return;
        }

        Box panTopBox = Box.createVerticalBox();
        panTopBox.add(new JLabel(i18n.get(this, "filterAttribute", "Filter attribute ")));
        JComboBox filterColumnNameComboBox = new JComboBox(dataColumnNames.toArray());
        filterColumnNameComboBox.setSelectedIndex(dataColumnNames.indexOf(filterColumnName));

        filterColumnNameComboBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (getDataStorage() != null) {
                    int index = ((JComboBox) e.getSource()).getSelectedIndex();
                    if (index >= 0) {
                        setFilterColumnName(getDataStorage().getDefaultDataTableModel().getRealColumnName(index));
                        // <Warning message>
                        warningMessage.setText(i18n.get(MonoValuedFilterDataStorage.class, "warning3", "Select a column value"));
                        // </Warning message>
                        initFilterExpressionPanel();
                    }
                }
            }
        });
        panTopBox.add(filterColumnNameComboBox);
        filterExpressionPanel.add(panTopBox, BorderLayout.NORTH);
        if (keyValues == null) {
            if(filterColumnName != null & ! "".equals(filterColumnName))
                setFilterColumnName(filterColumnName);
            else
                return;
        }

        cbPanel = new JPanel(new BorderLayout());
	CheckListItem[] checkListItem = new CheckListItem[keyValues.length];
	{
	    int index = 0;
	    for (String v : keyValues) {
		checkListItem[index] = new CheckListItem(v);
		checkListItem[index].setSelected(filterColumnValues.contains(v));
		index++;
	    }
	}

        final JList list = new JList(checkListItem);
        list.setCellRenderer(new CheckListRenderer());
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        list.addMouseListener(new MouseAdapter() {

            public void mouseClicked(MouseEvent event) {
                filterColumnValues.clear();
                JList list = (JList) event.getSource();
                // Get index of item clicked
                int index = list.locationToIndex(event.getPoint());
                CheckListItem item = (CheckListItem) list.getModel().getElementAt(index);
                // Toggle selected state
                item.setSelected(!item.isSelected());
                // Repaint cell
                list.repaint(list.getCellBounds(index, index));

                for (int i = 0; i < list.getModel().getSize(); i++) {
                    CheckListItem cItem = (CheckListItem) list.getModel().getElementAt(i);
                    if (cItem.isSelected()) {
                        filterColumnValues.add(getKeyValueAt(i));
                    }
                }

                setFilterColumnValues(filterColumnValues);
            }
        });
        
        JScrollPane sp = new JScrollPane(list);
        //sp.setMaximumSize(new Dimension(280, 250));
        //cbPanel.setPreferredSize(new Dimension(290, 250));
        final JCheckBox checkAll = new JCheckBox(i18n.get(this, "selectall", "Select all"));
        checkAll.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                filterColumnValues.clear();
                for (int i = 0; i < list.getModel().getSize(); i++) {
                    CheckListItem cItem = (CheckListItem) list.getModel().getElementAt(i);
                    cItem.setSelected(checkAll.isSelected());
                    if (checkAll.isSelected()) {
                        filterColumnValues.add(getKeyValueAt(i));
                    }
                }
                list.repaint();

                setFilterColumnValues(filterColumnValues);
            }
        });
        cbPanel.add(checkAll, BorderLayout.NORTH);
        cbPanel.add(sp, BorderLayout.CENTER);
        filterExpressionPanel.add(cbPanel, BorderLayout.CENTER);
        filterExpressionPanel.updateUI();
    }
    protected JPanel cbPanel;

    public void itemStateChanged(ItemEvent e) {
        //ArrayList<String> filterColumnValues = new ArrayList<String>();
        filterColumnValues.clear();
        for (int i = 0; i < cbPanel.getComponentCount(); i++) {
            Component c = cbPanel.getComponent(i);
            if (c instanceof JCheckBox) {
                if (((JCheckBox) c).isSelected()) {
                    // System.out.println("MultiValueFilterDataStorage value " + i + " " + keyValues.get(i) +" selected");
                    filterColumnValues.add(getKeyValueAt(i));
                }
            }
        }
        setFilterColumnValues(filterColumnValues);
    }

    public void setProperties(String prefix, java.util.Properties properties) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        super.setProperties(prefix, properties);

        prefix = PropUtils.getScopedPropertyPrefix(prefix) + multiFilterDSPropertyString + ".";
        String dsClassName = properties.getProperty(prefix + dataStorageClassPropertyString, "");
        if ("".equals(dsClassName)) {
            return;
        }
        setDataStorage(Util.loadDataStorage(dsClassName, prefix, properties, this));
        //System.out.println("------------DATASTORAGE " + dataStorage);
        filterColumnName = properties.getProperty(prefix + filterColumnNamePropertyString, "");
        filterColumnValues = UnicaenPropUtils.arrayListFromProperties(properties, prefix + filterColumnValuesPropertyString, listDelim);
        initialize();
    }

    public void initialize() {
        if (getDataStorage() == null || !getDataStorage().isReady()) {
            Debug.message("DataStorage", "[MultiFilterDS] can't initialize because dataStorage is not ready !!!!");
            return;
        }
        setColumnNames();
        dataTableModel = new DataTableModel(dataTable);
        resetDataTableStoragePanel(dataTableModel);
        setTable();
        setFilterColumnName(filterColumnName);
        setFilterColumnValues(filterColumnValues);
	//initFilterExpressionPanel();
    }

    public java.util.Properties getProperties(String prefix, java.util.Properties properties) {
        properties = super.getProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + multiFilterDSPropertyString + ".";
        if (getDataStorage() != null) {
            properties.put(prefix + dataStorageClassPropertyString, getDataStorage().getClass().getCanonicalName());
            // Debug.message("DataStorage", "PivotDataStorage.getProperties() - DataStorage member is a: "+dataStorage.getClass().getCanonicalName());
            properties = getDataStorage().getProperties(prefix, properties);
        }
        properties.put(prefix + filterColumnNamePropertyString, filterColumnName);
        UnicaenPropUtils.arrayListToPropertyString(properties, prefix + filterColumnValuesPropertyString, filterColumnValues, listDelim);

        return properties;
    }
}
