// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * DualDataStorage.java
 *
 * Created on 25 october 2007
 *
 * @author Jacques.Madelaine@info.unicaen.fr
 *
 */

package fr.unicaen.openmap.dataStorage;


import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import com.bbn.openmap.I18n;
import com.bbn.openmap.BasicI18n;

import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.UnicaenPropUtils;
import fr.unicaen.openmap.dataStorage.gui.MultiDataTableModel;
import fr.unicaen.openmap.dataStorage.gui.DataTableStoragePanel;
import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import fr.unicaen.openmap.gui.SelectAndReorderFromList;
import fr.unicaen.openmap.util.gui.ComponentSizer;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import javax.swing.*;
import javax.swing.DefaultListModel;
import javax.swing.table.TableModel;

/** 
 * SelectColumnDataStorage is an adaptor that can delete columns (the restriction data base operation).
*/
public class SelectColumnDataStorage extends RemovableDataStorageAdaptor {
    /* --------------------------  Begin   Properties  ---------------------*/
    public static final String dsPropertyString = "ds";
    public static final String dataStorageClassPropertyString = "dataStorageClass";
    public static final String dataColumnNamesPropertyString = "dataColumnNames";        
    public static final String omColumnNamesPropertyString = "omColumnNames";        
    /* --------------------------  End     Properties    -------------------*/
 
    private ArrayList<String> dataColumnNames;
    private ArrayList<String> omColumnNames;
    
    public SelectColumnDataStorage(DataStorageListener dsl) {
	super(dsl, null);
    }

    public SelectColumnDataStorage(DataStorageListener dsl, DataStorage ds) {
        super(dsl, ds);

	this.dataColumnNames = new ArrayList<String>(Arrays.asList(ds.getDefaultDataTableModel().getDataColumnNames()));
	this.omColumnNames = new ArrayList<String>(Arrays.asList(ds.getDefaultDataTableModel().getOMColumnNames()));
    }

    public SelectColumnDataStorage(DataStorageListener dsl, DataStorage dataStorage, ArrayList<String>  dataColumnNames, ArrayList<String>  omColumnNames) {
	super(dsl, dataStorage);
	this.dataColumnNames = dataColumnNames;
	this.omColumnNames = omColumnNames;
    }

    /** The new SelectColumnDataStorage Listener will be the one of ds */
    public SelectColumnDataStorage(DataStorage ds, ArrayList<String>  dataColumnNames, ArrayList<String>  omColumnNames) {
	this(ds.getDataStorageListener(), ds, dataColumnNames, omColumnNames);
    }

    /**
     * Tells if the dataStorage has its data.
     * May also bu used to reset the data storage in case dataTableModel is null.
     */
    public boolean isReady() {
        boolean ret = getDataStorage() != null && getDataStorage().isReady();
	if (ret) // in case ds is ready but the data table model is not ready. 
	    getDefaultDataTableModel();
	return ret;
    }

    public void initialize() {
	dataTableModel = null;
	if (isReady()) {
	    refreshGUI();
	    fireDataStorageEvent();
	}
    }

    public DataTableModel getDefaultDataTableModel() {	    
	if (dataTableModel == null) {
	    Debug.message("DataStorage", "[SelectColumnDataStorage.getDefaultDataTableModel()] is null "+this); 
	    dataTableModel = 	 new SelectColumnDataTableModel(getDataStorage().getDefaultDataTableModel(), dataColumnNames, omColumnNames);
	    if (dtsp == null)
		dtsp = new DataTableStoragePanel(this, dataTableModel);
	    else
		((DataTableStoragePanel)dtsp).setModel(dataTableModel);
	}
	
	return dataTableModel;
    }
    
    public void refreshGUI() { 
	Debug.message("DataStorage", "SelectColumnDataStorage.refreshGUI()");
	updateListsPanel();
    }

    private JComponent listsPanel;

    private JComponent createSelectPanel(List<String> allNames, final List<String> selectedNames) {

	DefaultListModel originalList = new DefaultListModel();

	for(String a : allNames) 
	    originalList.addElement(a);
	final DefaultListModel attributes = new DefaultListModel();
	// check if selected names are still existant in the original
	for (String a : selectedNames)
	    if (allNames.contains(a))
		attributes.addElement(a);
	ActionListener listener = new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    selectedNames.clear();
		    for(java.util.Enumeration e = attributes.elements(); e.hasMoreElements(); )
			selectedNames.add(e.nextElement().toString()); 		    
		    dataTableModel = null; // in order to force an initialize
		    SelectColumnDataStorage.this.fireDataStorageEvent();
		}
	    };
	return new SelectAndReorderFromList(originalList, attributes, 
					    i18n.get(this, "original", "Original"), 
					    i18n.get(this, "selected", "Selected"),
					    listener, false);
    }

    private void updateListsPanel() {
	listsPanel.removeAll();
	listsPanel.add(new JLabel(i18n.get(fr.unicaen.openmap.dataStorage.expr.gui.AddColumnPanel.class, "dataColumns", "Data columns")));
	listsPanel.add(createSelectPanel(Arrays.asList(getDataStorage().getDefaultDataTableModel().getDataColumnNames()), dataColumnNames));
	listsPanel.add(new JLabel(i18n.get(fr.unicaen.openmap.dataStorage.expr.gui.AddColumnPanel.class, "geometryColumns", "Geometry columns")));
	listsPanel.add(createSelectPanel(Arrays.asList(getDataStorage().getDefaultDataTableModel().getOMColumnNames()), omColumnNames));
	listsPanel.updateUI();
    }

    private JComponent selectColumnPanel = null;

    public JComponent getGUI() {
        if (selectColumnPanel != null) {
            return selectColumnPanel;
        }
	selectColumnPanel =  Box.createVerticalBox();
	listsPanel = Box.createVerticalBox();

	selectColumnPanel.add(listsPanel);
	updateListsPanel();

	Debug.message("DataStorage", "SelectColumnDataStorage.getGUI() , ds = "+ getDataStorage());

        selectColumnPanel.add(getDataStorage().getGUI());
        selectColumnPanel.setBorder(BorderFactory.createTitledBorder(i18n.get(this, "name", "Select Columns")));
        return selectColumnPanel;
    }

    public JComponent getDataGUI(){
	if (dtsp == null) {
	    dataTableModel = getDefaultDataTableModel();
	    // and the data panel
	    dtsp = new DataTableStoragePanel(this, dataTableModel);
	}
	return dtsp;
    }
        
    public void setProperties(String prefix, java.util.Properties properties) {
	super.setProperties(prefix, properties);
	prefix = PropUtils.getScopedPropertyPrefix(prefix);

	String dsPrefix = prefix + dsPropertyString + ".";
	String dsClassName = properties.getProperty(dsPrefix + dataStorageClassPropertyString);
	setDataStorage(Util.loadDataStorage(dsClassName, dsPrefix, properties, this));

	dataColumnNames = UnicaenPropUtils.arrayListFromProperties(properties, prefix + dataColumnNamesPropertyString, listDelim);
	omColumnNames = UnicaenPropUtils.arrayListFromProperties(properties, prefix + omColumnNamesPropertyString, listDelim);
    }

    public java.util.Properties getProperties(java.util.Properties properties) {
	return getProperties("", properties);
  }


    public java.util.Properties getProperties(String prefix, java.util.Properties properties) {
	properties = super.getProperties(prefix, properties);
	if (getDataStorage() != null) { 
	    String dsPrefix = prefix + dsPropertyString + ".";
	    properties.put(dsPrefix + dataStorageClassPropertyString, getDataStorage().getClass().getCanonicalName());
	    properties = getDataStorage().getProperties(dsPrefix, properties);
	}        
        UnicaenPropUtils.arrayListToPropertyString(properties, prefix + dataColumnNamesPropertyString, dataColumnNames, listDelim);
        UnicaenPropUtils.arrayListToPropertyString(properties, prefix + omColumnNamesPropertyString, omColumnNames, listDelim);
	return properties;
    }


    public DataObject getElementAt(int index) {
	return getDataStorage().getElementAt(index);
    }
   

    public int getCount() {
	if (getDataStorage() == null) {
	    Debug.error("[SelectColumnDataStorage] ###Error null DataStorage");
	    return 0;
	}
	return getDataStorage().getCount();
    }

    public void refresh() { this.dataStorageEvent(this); }

}

/**
 * forward all request to the DataTableModel except for the list of column
 */
class SelectColumnDataTableModel extends DataTableModel {

    private DataTableModel dtm;
    private ArrayList<String> dataColumnNames;
    private ArrayList<String> omColumnNames;
    
    public SelectColumnDataTableModel(DataTableModel dtm, ArrayList<String> dataColumnNames, ArrayList<String> omColumnNames) {
	super(null) ; // we override all methods that use the table attribute
        this.dtm = dtm;
	this.dataColumnNames = dataColumnNames;
	this.omColumnNames = omColumnNames;
	checkColumnNames(dtm.getDataColumnNames(), dataColumnNames);
	checkColumnNames(dtm.getOMColumnNames(), omColumnNames);
	Debug.message("DataStorage","** SelectColumnDataTableModel "+ dataColumnNames + " " + omColumnNames);
    }
    
    private void checkColumnNames(String[] names, ArrayList<String> selectedNames) {
	ArrayList<String> selectedNamesCopy = new ArrayList<String>(selectedNames);
	for(String n : selectedNamesCopy) {
	    boolean remove = true;
	    for (String nn : names)
		if (nn.equals(n)) {
		    remove = false;
		    break;
		}
	    if (remove) {
		Debug.message("DataStorage", "[SelectColumnDataStorage : removing column "+n);
		selectedNames.remove(n);
	    }
	}
    } 

    public int getRowCount() {
        return dtm.getRowCount();
    }

    public int getColumnCount() {
        return dataColumnNames.size();
    }

    public Object getValueAt(int row, int col) {
	// convert colnumber to columnName
	String colName = dataColumnNames.get(col);
	// convert colName into col number for the dtm model
	col = getDtmIndexForColumnName(colName);
        return dtm.getValueAt(row, col);
    }

    public String getColumnName(int column) {
        return dataColumnNames.get(column);
    }

    public String getRealColumnName(int column) {
        return dataColumnNames.get(column);
    }

    public boolean isCellEditable(int row, int col) {
        return false;
    }

    public Class getColumnClass(int c) {
        try {
            return getValueAt(0, c).getClass();
        } catch (java.lang.NullPointerException e) {
            return "".getClass();
        }
    }

    public int getOMColumnCount() {
        return  omColumnNames.size();
    }

    public String getOMColumnName(int c) {
        return omColumnNames.get(c);
    }

    public String getRealOMColumnName(int c) {
        return omColumnNames.get(c);
    }

    public String[] getDataColumnNames() {
        return dataColumnNames.toArray(new String[0]);
    }

    public String[] getOMColumnNames() {
        return omColumnNames.toArray(new String[0]);
    }

    public int getDtmIndexForColumnName(String name) {
        if (name == null) {
            return -1;
        }
        for (int i = 0; i < dtm.getColumnCount(); i++) {
            if (name.equals(dtm.getColumnName(i))) {
                return i;
            }
        }
        return -1;
    }

    public int getDtmIndexForOMColumnName(String name) {
        if (name == null) {
            return -1;
        }
        for (int i = 0; i < dtm.getOMColumnCount(); i++) {
            if (name.equals(dtm.getOMColumnName(i))) {
                return i;
            }
        }
        return -1;
    }

    public DataTable getTable() {
        return dtm.getTable();
    }

    public Object[] getValues(String colName){
        int indexCol = getIndexForColumnName(colName);
        Object[] values = new Object[getRowCount()];
        for(int i = 0; i < getRowCount(); i++){
            values[i] = getValueAt(i, indexCol);
        }
        return values;

    }

    public DataObject getDataObjectAt(int row){
        return dtm.getDataObjectAt(row);
    }



}
