// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * SelectionMatrix.java
 *
 * Created on 6 octobre 2005, 15:45
 *
 * @author Christophe.Turbout@info.unicaen.fr
 */
package fr.unicaen.openmap.dataStorage;

import com.bbn.openmap.util.ColorFactory;
import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.layer.isochroneLayer.IsochroneDataObject;
import fr.unicaen.openmap.util.Util;
import java.awt.Color;
import java.util.ArrayList;
import java.util.Vector;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 *
 * @author Christophe.Turbout@info.unicaen.fr
 *
 * <p>This Vector stores the selection that matches a column in the matrix representation.
 */
public class SelectionMatrix extends Object {
    /**
     * default value is set to 100.
     **/
    private int nbRows = 100;
    
    private int idCreation = -1;
    /**
     * The name of the column used in GUI JTabPane
     **/
    private ArrayList<String> columnNames = null;
    /**
     * This variable tell us if the item is used in one or many selection.
     **/
    private Vector<Boolean> composedSelection = null;
    private String defaultString = null;
    private int currentSubSelection = -1;
    private int nbColumnCreated=0;
    private ArrayList<Vector<Boolean>> columnStorage = null;
    private boolean multiSelectionMode = false;
    private  Vector<Integer> res = null; // store the last modification on a row.
    // les données nécéssaires à la création de la zone si définie par isochrone.
    private IsochroneDataObject isochroneDataObject = null; 
    
    /**
     * Creates a new instance of SelectionMatrix
     *
     * Matrix is initialized with a vector (a column) witch match default selection (unselected) if boolean toInit is set.
     *
     * We take also a parameter multiSelectionMode which tell us if a row is selected only in an unique subselection or can be in many.
     *
     * id is the unique identifiant set for each SelectionMatrix created.
     */
    public SelectionMatrix(int nbRow,String defaultString, Boolean toInit, boolean multimode, int id) {
        Debug.message(Util.className(this), " Constructeur 1");
        this.nbRows = nbRow;
        this.defaultString=defaultString;
        this.multiSelectionMode=multimode;
        this.idCreation = id;
        
        try {
            columnStorage=new ArrayList<Vector<Boolean>>();
            columnNames = new ArrayList<String>();
            composedSelection = new Vector<Boolean>(nbRow);
            for (int i=0;i<nbRow;i++) composedSelection.add(false);
            if (toInit) addColumn(false, defaultString, true);
        } catch (Exception e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
    }
    
    /**
     * Take in parameter a nodeset which store informations.
     **/
    public SelectionMatrix(int nbRow,String defaultString, Element selection, boolean multimode, int id) {
        Debug.message(Util.className(this), " Constructeur 2");
        this.nbRows = nbRow;
        this.defaultString=defaultString;
        this.idCreation = id;
        
        try {
            columnStorage=new ArrayList<Vector<Boolean>>();
            columnNames = new ArrayList<String>();
            composedSelection = new Vector<Boolean>(nbRow);
            for (int i=0;i<nbRow;i++) composedSelection.add(false);
            // now parse the nodeset to add columns
            
            // get the subselections
            NodeList subSelections = selection.getElementsByTagName("subselection");
            // need to treat each Subselection
            for (int k=0;k<subSelections.getLength();k++) {
                Vector<Boolean> column = new Vector<Boolean>();
                for (int j=0;j<nbRow;j++) column.add(false);
                
                Element element = (Element) subSelections.item(k);
                Debug.message(Util.className(this), "nom : "+element.getNodeName());
                // find subname
                NodeList nameList = element.getElementsByTagName("name");
                String name = nameList.item(0).getFirstChild().getNodeValue();
                Debug.message(Util.className(this), "Nom de la sous sélection : "+name);
                columnNames.add(name);
                NodeList idCreationList = element.getElementsByTagName("id");
                if (idCreationList.getLength()==0) idCreation=-1;
                else idCreation = Integer.parseInt(idCreationList.item(0).getFirstChild().getNodeValue());
                // get values
                NodeList valueList = element.getElementsByTagName("value");
                if (valueList.getLength()>0 && valueList.item(0).getFirstChild()!=null){
                    String values = valueList.item(0).getFirstChild().getNodeValue();
                    Debug.message(Util.className(this), "Les valeurs sont : "+values);
                    // split results
                    String[] splitted = values.split(",");
                    // set line to true
                    for (int m=0;m<splitted.length;m++){
                        Integer line = new Integer(splitted[m]);
                        // Debug.message(Util.className(this), this + "Ligne allumée : "+line);
                        column.set(line,true);
                        composedSelection.set(line,true);
                    }
                }
                columnStorage.add(column);
            }
            
            // get informations if the exist
            NodeList isochrone = selection.getElementsByTagName("isochroneDataObject");
            if (isochrone.getLength()>0){
                isochroneDataObject = new IsochroneDataObject();
                isochroneDataObject.loadFromXML((Element)isochrone.item(0));
            } 
            
            // get the subselections
            NodeList nbcreated = selection.getElementsByTagName("nbColumnCreated");
            if (nbcreated!=null && nbcreated.getLength()>0){
                Debug.message(Util.className(this), " le nombre de sous-selections : "+nbcreated);
                nbColumnCreated=new Integer(nbcreated.item(0).getFirstChild().getNodeValue());
            }
            else {
              nbColumnCreated=columnStorage.size();  
            }
        } catch (Exception e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
    }
    
    /*
     * return the number of coulmns int the selectionMatrix
     **/
    public int getNbCols() { return columnStorage.size();}
    
    /*
     * return the id Creation.
     */ 
    public int getCreationId(){
        return idCreation;
    }
    
    /*
     * return the number of rows int the selectionMatrix
     **/
    public int getNbRows() { return nbRows; }
    
    /**
     * Return the numbers of rows selected in the selectionMatrix.
     *
     * Useful to implement a view which show selected rows ...
     **/
    public int getNbBoolRows(int currentSelection){
        int nbtrue = 0;
        if (currentSelection<columnStorage.size()){
            try {
                Vector<Boolean> column = (Vector<Boolean>)columnStorage.get(currentSelection);
                for (int i=0;i<column.size();i++)
                    if (column.get(i)) nbtrue++;
            } catch (Exception e){
                System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
                e.printStackTrace();
            }
        }
        return nbtrue;
    }
    
    /**
     * Return the number column selected at rowindex.
     **/
    public int getBoolRows(int currentSelection, int rowindex){
        int nbtrue = -1;
        int res =-1;
        int i=-1;
        try {
            if (currentSelection<columnStorage.size()) {
                Vector<Boolean> column = (Vector<Boolean>)columnStorage.get(currentSelection);
                while (nbtrue<rowindex){
                    i++;
                    if (column.get(i)) nbtrue++;
                }
                if (nbtrue==rowindex) res=i;
            }
        }catch (Exception e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
        return res;
    }
    
    /**
     * return the name of the i column.
     **/
    public String getColumnName(int i){
        if (i<columnStorage.size())
            try {
                return columnNames.get(i);
            } catch (Exception e){
                System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
                e.printStackTrace();
            }
        return null;
    }
    
    /**
     * return the sub selection names.
     **/
    public ArrayList<String> getColumnNames(){
        return columnNames;
    }
    
    
    /**
     * Return Class of the column.
     **/
    public Class getColumnClass(int column) { return getValueAt(0,column).getClass(); }
    
    /**
     * Return the value at row, col.
     **/
    public Object getValueAt(int row, int col){
        try {
            return (Boolean) ((Vector<Boolean>)columnStorage.get(col)).get(row);
        } /*catch (ArrayIndexOutOfBoundsException e){
            System.out.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
            return null;
        }*/
        catch (Exception e){return null;}
    }
    
    
    /**
     * store the object at row, col.
     **/
    public void setValueAt(Object object, int row, int col){
        res = new Vector<Integer>();
        for (int k=0;k<columnStorage.size();k++) res.add(0);
        try {
            // if multiSelectionMode is set, normal use otherwise we have to unselect all others
            Vector<Boolean> column = null;
            if (!multiSelectionMode && (Boolean)object){
                for (int i=0;i<columnStorage.size();i++){
                    column = (Vector<Boolean>)columnStorage.get(i);
                    // deselect all and note those wich are on
                    if (column.get(row))  res.set(i,-1);
                    column.set(row,false);
                    // otherwise, nothing to do.
                }
                // if the column is to select => we add it.
                if ((Boolean)object) res.set(col,1);
                ((Vector<Boolean>)columnStorage.get(col)).set(row, (Boolean)object);
                composedSelection.set(row, (Boolean)object);
            } else {
                if (((Vector<Boolean>)columnStorage.get(col)).get(row)) res.set(col,1);
                else res.set(col,-1);
                ((Vector<Boolean>)columnStorage.get(col)).set(row, (Boolean)object);
                composedSelection.set(row, getComposedValue(row));
            }
        } catch (ArrayIndexOutOfBoundsException e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
    }
    
    /**
     * set all column value to false.
     **/
    private void deselectAllColumns(int row){
        for (int i=0;i<columnStorage.size();i++){
            ((Vector<Boolean>)columnStorage.get(i)).set(row, false);
        }
    }
    
    /**
     * return true if one of the column is selected.
     **/
    public boolean getComposedValue(int rowindex){
        boolean value = false;
        for (int i=0;i<columnStorage.size();i++){
            Vector<Boolean> therow = (Vector<Boolean>) columnStorage.get(i);
            value = value || (Boolean) therow.get(rowindex);
        }
        return value;
    }
    
    /**
     * Add a column in the matrix with bool default value. This column's name is name.
     * Return the new name based on Name and the current subselection.
     **/
    public String addColumn(boolean bool, String name, boolean autoindent){
        nbColumnCreated++;
        currentSubSelection=columnStorage.size();
        String subname = null;
        if (autoindent) subname=new String(name+" "+nbColumnCreated);
        else subname = new String(name);
        Debug.message(Util.className(this), "la sous selection "+currentSubSelection+" insérée : "+subname);
        columnNames.add(currentSubSelection,subname);
        Vector<Boolean> column = new Vector<Boolean>(nbRows);
        for (int i=0;i<nbRows;i++) column.add(bool);
        columnStorage.add(column);
        return subname;
    }
    
    public void deleteCurrentSelection(){
        columnStorage.remove(currentSubSelection);
        columnNames.remove(currentSubSelection);
        currentSubSelection=columnStorage.size()-1;
        for (int i=0;i<nbRows;i++) {
            composedSelection.set(i, getComposedValue(i));
        }
    }
    
    /**
     * Select the rowindex in the current selection.
     **/
    public void selectRow(int rowindex){
        setValueAt(true, rowindex,currentSubSelection);
    }
    
    /**
     * Select the rowindex in the current selection.
     **/
    public void selectRowInSubSelection(int rowindex, int subSelection){
        setValueAt(false, rowindex,currentSubSelection);
        setValueAt(true, rowindex,subSelection);
    }
    
    /**
     * Deselect the rowindex in the current selection.
     **/
    public void deselectRow(int index){
        setValueAt(false, index,currentSubSelection);
    }
    
    /**
     * Return true if the rowindex row is selected in the current selection.
     **/
    public boolean isSelected(int rowindex) {
        return composedSelection.get(rowindex);
    }
    
    /**
     * row index is the row selected. model is used to find the selected column.
     * we return the first column which have the row selected.
     **/
    public int getSelectedColumn(int rowindex){
        for (int i=0;i<columnStorage.size();i++) if ((Boolean)getValueAt(rowindex, i)) return i;
        return -1;
    }
    
    /**
     * Select or deselect all the row in the current selection.
     **/
    public void setColumnState(boolean state){
        for (int i=0;i<nbRows;i++){
            setValueAt(state,i,currentSubSelection);
        }
    }
    
    /**
     * return true if the currentSubSelection of row index is selected
     **/
    public boolean isSelectedCurrentSubSelection(int rowIndex){
        return (Boolean) getValueAt(rowIndex,currentSubSelection);
    }
    
    /**
     * return the current subselection.
     **/
    public int getCurrentSubSelection(){
        return currentSubSelection;
    }
    
    /**
     * return the current subselection.
     **/
    public int getSubSelectionsSize(){
        return columnStorage.size();
    }
    
    /**
     * return the current subselection.
     **/
    public void setCurrentSubSelection(int subSelection){
        currentSubSelection=subSelection;
    }
    
    /**
     * set the subselection index name.
     **/
    public void setSubSelectionName(int index, String name){
        columnNames.set(index,name);
    }
    
    /**
     * Add a column in the matrix with bool default value. This column's name is name.
     * Return the new name based on Name and the current subselection.
     **/
    private void addColumn(Vector<Boolean> columnToCopy){
        Vector<Boolean> column = new Vector<Boolean>(columnToCopy.size());
        // initialization and update for composed column
        columnStorage.add(column);
        for (int i=0;i<columnToCopy.size();i++) {
            column.add(columnToCopy.get(i));
            composedSelection.set(i, getComposedValue(i));
        }
    }
    
    /**
     * The method return a copy of the current matrix.
     **/
    public SelectionMatrix getCopy(int id){
        // create a new empty matrix
        SelectionMatrix newmatrix = new SelectionMatrix(nbRows,defaultString,false,multiSelectionMode,id);
        // add the all the columns of the current matrix to the new.
        ArrayList<String> names = new ArrayList<String>();
        for (int i=0;i<columnStorage.size();i++){
            newmatrix.addColumn(columnStorage.get(i));
            names.add(columnNames.get(i));
        }
        newmatrix.setColumnNames(names);
        newmatrix.setNbColumnCreated(nbColumnCreated);
        newmatrix.setCurrentSubSelection(0);
        return newmatrix;
    }
    
    /**
     * set the column names;
     **/
    private void setColumnNames(ArrayList<String> names){
        columnNames=names;
    }
    
    /**
     * set the number of column already created
     **/
    private void setNbColumnCreated(int number){
        nbColumnCreated=number;
    }
    
    /**
     * Save the selection matrix in an XML form, store only
     **/
    
    String saveToXML(String name, ArrayList<Color> colors){
        StringBuffer buffer = new StringBuffer();
        buffer.append("<selection><name>"+name+"</name><id>"+idCreation+"</id>");
        buffer.append("<nbColumnCreated>"+nbColumnCreated+"</nbColumnCreated>");
        // sauvegarde des informations si elles existent.
        if (isochroneDataObject!=null) buffer.append(isochroneDataObject.savetoXML());
        for (int i=0;i<columnStorage.size();i++){
            Vector<Boolean> column = (Vector<Boolean>)columnStorage.get(i);
            buffer.append("<subselection>");
            buffer.append("<name>"+columnNames.get(i)+"</name>");
            buffer.append("<color>"+ColorFactory.getHexColorString(colors.get(i))+"</color>");
            buffer.append("<value>");
            boolean first=true;
            for (int j=0;j<column.size();j++){
                if (column.get(j)) {
                    if (first) first=false;
                    else buffer.append(",");
                    buffer.append(j);
                }
            }
            buffer.append("</value>");
            buffer.append("</subselection>");
        }
        buffer.append("</selection>");
        return buffer.toString();
    }
    
    /**
     * Return the number of column created. Whit this mecanism we are sure that automatic name is unique.
     **/
    public int getNbColumnCreated(){
        return nbColumnCreated;
    }
    
    public Vector<Integer> getLastModifiedRow(){
        return res;
    }
    
    public void setIsochroneDataObject(IsochroneDataObject object){
        isochroneDataObject=object;
    }
    
    public IsochroneDataObject getIsochroneDataObject(){
        return isochroneDataObject;
    }
}
