// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.bbn.openmap.Environment;
import com.bbn.openmap.util.Debug;



import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import fr.unicaen.openmap.dataStorage.gui.DataTableStoragePanel;
import fr.unicaen.openmap.gui.RefreshableStringComboBox;
import fr.unicaen.openmap.util.Util;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.Properties;
import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

public class SortDataStorage extends RemovableInformationDataStorage {

    public static final String dataStorageClassPropertyString = "dataStorageClass";
    public static final String sortDSPropertyString = "sortDataStorage";
    public static final String ascOrderPropertyString = "ascOrder";
    public static final String sortParameterPropertyString = "sortParameter";
    public static final String selectedFormatPropertyString = "selectedFormat";
    protected JPanel sortPanel;
    protected ArrayList<DataObject> dataObjects;
    protected DataObjectComparator dataObjectCpmparator;
    // private String customFormat;
    private boolean asc;

    public DataObjectComparator getDataObjectCpmparator() {
        return dataObjectCpmparator;
    }

    public enum Format {

        STRING("string", "string"),
        NUMERIC("numeric", "numeric"),
        DATE_1("date_yyyy.MM.dd", "yyyy.MM.dd"),
        DATE_2("date_dd-MM-yyyy", "dd-MM-yyyy"),
        DATE_3("date_yyyy/MM/dd", "yyyy/MM/dd"),
        DATE_CUSTOM("date_custom", "");
        private final String label;
        private String innerFormat;
        private String propertyString;

        Format(String label, String innerFormat) {
            this.label = Environment.getI18n().get(SortDataStorage.class, label, label);
            this.innerFormat = innerFormat;
            this.propertyString = label;
        }

        public String toString() {
            return label;
        }

        public String toSymbol() {
            return super.toString();
        }

        public String getPropertyString(){
            return propertyString;
        }

        public String getInnerFormat() {
            return innerFormat;
        }

        public void setInnerFormat(String format) {
            this.innerFormat = format;
        }
    }
    private String sortParameter;
    private Format selectedFormat = Format.STRING;

    public SortDataStorage(DataStorageListener dsl) {
        this(dsl, null);
    }

    public SortDataStorage(DataStorageListener dsl, DataStorage dataStorage) {
        super(dsl, dataStorage);
        asc = true;
    }

    @Override
    public boolean isReady() {
        return getDataStorage() != null && getDataStorage().isReady() && sortParameter != null;
    }

    public void initialize() {
        if (getDataStorage() == null  || !getDataStorage().isReady()) {
            Debug.error("[SortDataStorage] can't initialize because dataStorage is not ready !!!!");
            return;
        }
        dataObjects = new ArrayList<DataObject>(getDataStorage().getCount());
        for (int i = 0; i < getDataStorage().getCount(); i++) {
            dataObjects.add(getDataStorage().getElementAt(i));
        }
        Collections.sort(dataObjects, DataObjectComparatorFactory.buildComparator(sortParameter, selectedFormat));
        if (!asc)//reverse sort
            Collections.reverse(dataObjects);

        setTable();
	refreshGUI();
    }

    protected boolean setTable() {

        if (dataObjects == null)
            return false;

        dataTable.setTable(dataObjects,
                new ArrayList<String>(Arrays.asList(getDataStorage().getDefaultDataTableModel().getDataColumnNames())),
                new ArrayList<String>(Arrays.asList(getDataStorage().getDefaultDataTableModel().getOMColumnNames())),
                getDataStorage().getDataAttributesNamesAliases());
        ((DataTableStoragePanel) dtsp).fireStructureChanged();
        fireDataStorageEvent();
        return true;
    }

    public void setCustomFormat(String f) {
	if (selectedFormat == Format.DATE_CUSTOM)
	    selectedFormat.setInnerFormat(f);
    }

    private RefreshableStringComboBox comboColumns;

    public void refreshGUI() {
	if (comboColumns != null)
	    comboColumns.init(getDataStorage().getDefaultDataTableModel().getDataColumnNames(), sortParameter);
    }

    public JComponent getGUI() {
        if (sortPanel != null) {
            return sortPanel;
        }
	
        sortPanel = new JPanel();
        ActionListener comboColumnsListener = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    sortParameter = (String) ((JComboBox) e.getSource()).getSelectedItem();
		    //dataTable.sortData((String)((JComboBox) e.getSource()).getSelectedItem(), selectedFormat);
		}
	    };
	comboColumns = new RefreshableStringComboBox(comboColumnsListener, getDataStorage().getDefaultDataTableModel().getDataColumnNames(), sortParameter);
        JButton validateButton = new JButton(i18n.get(this, "validate", "Validate"));

        validateButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                initialize();
            }
        });
        JPanel comboPanel = new JPanel(new GridLayout(4, 2));

        ButtonGroup bg = new ButtonGroup();
        final JRadioButton bAsc = new JRadioButton(i18n.get(this, "ascOrder", "Ascending Order"));
        JRadioButton bDesc = new JRadioButton(i18n.get(this, "descOrder", "Descending Order"));
        bAsc.setSelected(asc);
        bDesc.setSelected(!asc);
        bg.add(bAsc);
        bg.add(bDesc);
        bAsc.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent arg0) {
                asc = bAsc.isSelected();
            }
        });
        comboPanel.add(bAsc);
        comboPanel.add(bDesc);
        JComboBox boxFormat = new JComboBox(Format.values());
        boxFormat.setSelectedItem(selectedFormat);
        //System.out.println("SELECTED FORMAT = " + selectedFormat);
        comboPanel.add(new JLabel(i18n.get(this, "sortColumn", "Sort column ")));
        comboPanel.add(comboColumns);
        comboPanel.add(new JLabel(i18n.get(this, "columnFormat", "Column Format ")));
        comboPanel.add(boxFormat);
        final JLabel labCustom = new JLabel(i18n.get(this, "customFormat", "Custom Format "));
        comboPanel.add(labCustom);
        final JTextField fieldCustomFormat = new JTextField("");
        comboPanel.add(fieldCustomFormat);
        labCustom.setVisible(false);
        fieldCustomFormat.setVisible(false);

        fieldCustomFormat.addKeyListener(new KeyAdapter() {

            public void keyReleased(KeyEvent evt) {
                setCustomFormat(fieldCustomFormat.getText());
            }
        });
        boxFormat.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                selectedFormat = (Format) ((JComboBox) e.getSource()).getSelectedItem();
                labCustom.setVisible(selectedFormat == Format.DATE_CUSTOM);
                fieldCustomFormat.setVisible(selectedFormat == Format.DATE_CUSTOM);
            }
        });


        JPanel validatePanel = new JPanel();
        validatePanel.add(validateButton);
        sortPanel.setLayout(new BoxLayout(sortPanel, BoxLayout.Y_AXIS));
        sortPanel.add(comboPanel);
        sortPanel.add(validatePanel);
        sortPanel.add(getDataStorage().getGUI());
        sortPanel.setBorder(BorderFactory.createTitledBorder(i18n.get(this, "sort", "Sort")));
        if (sortParameter == null || sortParameter.equals("")) {
            sortParameter = getDataStorage().getDefaultDataTableModel().getDataColumnNames()[0];
        }
        return sortPanel;
    }

    public void setProperties(String prefix, java.util.Properties properties) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        super.setProperties(prefix, properties);

        prefix = PropUtils.getScopedPropertyPrefix(prefix) +sortDSPropertyString + ".";
        String dsClassName = properties.getProperty(prefix + dataStorageClassPropertyString, "");
        if ("".equals(dsClassName)) {
            return;
        }
        setDataStorage(Util.loadDataStorage(dsClassName, prefix, properties, this));
        asc = PropUtils.booleanFromProperties(properties, prefix + ascOrderPropertyString, true);
        sortParameter = properties.getProperty(prefix + sortParameterPropertyString, "");
        //System.out.println("SET PROPERTIES = " + properties.getProperty(prefix + selectedFormatPropertyString,"STRING"));
	String innerFormat = properties.getProperty(prefix + selectedFormatPropertyString, "string");
	selectedFormat = Format.DATE_CUSTOM;
	for (Format f : Format.values()) {
	    if (f.getInnerFormat().equals(innerFormat))
		selectedFormat = f;
	}
	if (selectedFormat == Format.DATE_CUSTOM)
	    selectedFormat.setInnerFormat(innerFormat);
        initialize();
    }



    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix) + sortDSPropertyString + ".";
        properties.put(prefix + sortParameterPropertyString, sortParameter);
        properties.put(prefix + ascOrderPropertyString, String.valueOf(asc));
        properties.put(prefix + selectedFormatPropertyString, selectedFormat.getInnerFormat());
        if (getDataStorage() != null) {
            properties.put(prefix + dataStorageClassPropertyString, getDataStorage().getClass().getCanonicalName());
            // Debug.message("DataStorage", "PivotDataStorage.getProperties() - DataStorage member is a: "+getDataStorage().getClass().getCanonicalName());
            properties = getDataStorage().getProperties(prefix, properties);
        }
        return properties;

    }
}
