// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.expr;

import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;


import fr.unicaen.openmap.app.ThemaMap;
import fr.unicaen.openmap.dataStorage.DataStorage;
import fr.unicaen.openmap.dataStorage.DataStorageListener;
import fr.unicaen.openmap.dataStorage.InformationDataStorage;
import fr.unicaen.openmap.dataStorage.RemovableDataStorageAdaptor;
import fr.unicaen.openmap.dataStorage.expr.gui.AddColumnPanel;
import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import fr.unicaen.openmap.dataStorage.gui.MultiDataTableModel;
import fr.unicaen.openmap.dataStorage.gui.DataTableStoragePanel;
import org.antlr.runtime.*;
import org.antlr.runtime.tree.*;
import java.util.List;
import java.util.HashMap;
import javax.swing.*;

public abstract class ExprDataStorage extends RemovableDataStorageAdaptor {

    public static final String exprPropertyString = "expr";
    public static final String exprDSPropertyString = "exprDS";

    protected String expr = null;
    protected CommonTree treeExpr = null;
 
    public ExprDataStorage(DataStorageListener dsl) {
        this(dsl, null);
    }

    public ExprDataStorage(DataStorageListener dsl, DataStorage dataStorage) {
        super(dsl, dataStorage);
    }

    public boolean isReady() {
	return getDataStorage() != null && getDataStorage().isReady() && treeExpr != null;
    }

    /** envoroinment for function such as Classif, min and max */
    private HashMap<Object, Object> _exprEnviron;
    protected void reinitEnviron() { 
	_exprEnviron = new HashMap<Object, Object>();
    }
    public Object getEnviron(Object key) {
	return _exprEnviron.get(key);
    }
    public void putEnviron(Object key, Object value) {
	_exprEnviron.put(key, value);
    }
    
    private boolean _exprIsOK = false;
    public boolean exprIsOK() { return _exprIsOK; }

    public String getExpr() { return expr; }

    public boolean setExpr(String _expr) {
        // System.out.println("EXPRESSION = " + _expr);
	Debug.message("DataStorage", "[ExprDataStorage.setExpr to: " + _expr);
	if (_expr == null)
	    _expr = "";
	expr = _expr;
	
	reinitEnviron();
	// parse expr...
        MyExprGrammarParser parser = 
	    new MyExprGrammarParser(new CommonTokenStream(
							  new ExprGrammarLexer(new ANTLRStringStream(_expr), 
									       getDefaultDataTableModel())));
	try {   
	    treeExpr  = (CommonTree) parser.arg().getTree();
	} catch (org.antlr.runtime.RecognitionException e) {
	    System.err.println("Expr recognition error: " + e); 
	    treeExpr = null;
	    return false;
	}
	_exprIsOK = parser.getErrMsg() == null;
	Debug.message("DataStorage", "[ExprDataStorage] errmsg: "+parser.getErrMsg()+", ok: " + _exprIsOK+", expr: "+ _expr + ", tree: " + treeExpr.toStringTree() + " type: "+treeExpr.getType());
	errorLine = parser.getErrorLine();
	errorCharPosInLine = parser.getErrorCharPosInLine();
	return _exprIsOK;
    }
    private int errorLine;
    private int errorCharPosInLine;
    public int getErrorLine() { return errorLine;}
    public int getErrorCharPosInLine() { return errorCharPosInLine;}
    
    abstract public void initialize();

    public DataTableModel getDefaultDataTableModel() {	    
	if (dataTableModel == null) {
	    Debug.message("DataStorage", "[ExprDataStorage.getDefaultDataTableModel()] is null ");
	    dataTableModel = newMultiDataTableModel();
	}
	
	return dataTableModel;
    }

    abstract protected boolean setColumnNames();

    protected List<String> dataColumnNames = null;
    protected List<String> omColumnNames = null;

    protected DataTableModel newMultiDataTableModel() {
	if (! setColumnNames())
	    return null;
	return new MultiDataTableModel(this, 
				       dataColumnNames.toArray(new String[dataColumnNames.size()]), 
				       omColumnNames.toArray(new String[omColumnNames.size()]));
    }

    public JComponent getDataGUI(){
	if (dtsp == null) {
	    dataTableModel = getDefaultDataTableModel();
	    // and the data panel
	    dtsp = new DataTableStoragePanel(this, dataTableModel);
	}
	return dtsp;
    }

    public java.util.Properties getProperties(String prefix, java.util.Properties properties) {
       properties = super.getProperties(prefix, properties);
       prefix = PropUtils.getScopedPropertyPrefix(prefix) + exprDSPropertyString + ".";
       if (getExpr() != null)
	   properties.put(prefix + exprPropertyString, getExpr().replaceAll("\n", "\\\\n"));
       return properties;
    }

    public void setProperties(String prefix, java.util.Properties properties) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix + exprDSPropertyString + ".");
        super.setProperties(prefix, properties);
        //System.out.println("expression pour "+prefix + exprPropertyString+"= " + properties.getProperty(prefix + exprPropertyString));
	setExpr(properties.getProperty(prefix + exprPropertyString, ""));
    }
   
}

class MyExprGrammarParser extends ExprGrammarParser {
    
    public MyExprGrammarParser(TokenStream input) {
	super(input);
    }
    // public MyExprGrammarParser(TokenStream input, RecognizerSharedState state) {
    // 	super(input, state);      
    // }
    private String errmsg = null;
    public String getErrMsg() { return errmsg; }
    public void emitErrorMessage(String msg) {
	errmsg = msg;
	System.err.println(msg);
    }
    private int errorLine;
    private int errorCharPosInLine;
    public int getErrorLine() { return errorLine;}
    public int getErrorCharPosInLine() { return errorCharPosInLine;}

    public String getErrorHeader(RecognitionException e) {
	errorLine = e.line;
	errorCharPosInLine = e.charPositionInLine;
	return super.getErrorHeader(e);
    }
}
