// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
grammar ExprGrammar;
// Grammar for ExprDataStorage
// compile with:  java org.antlr.Tool ExprGrammar.g
// having antlr.jar in your CLASSPATH ( export CLASSPATH=$CLASSPATH:/usr/local/java/antlrworks.jar )
// Or in a single command line:
// CLASSPATH=$CLASSPATH:/usr/local/java/antlrworks.jar java org.antlr.Tool ExprGrammar.g

options {
    output=AST;
	language = Java ;
    ASTLabelType=CommonTree; 
}

tokens {
	LBRACE		= '{' ;
	RBRACE		= '}' ;
	LPAREN		= '(' ;
	RPAREN		= ')' ;
	DOT		        = '.' ;
	SEMIC		    = ';' ;
	COMMA		= ',' ;
	LT		        = '<' ;
	GT		        = '>' ;
	LTE		        = '<=' ;
	GTE		        = '>=' ;
	EQ		        = '=' ;
	NEQ		        = '<>' ;
    NOTEQ         = '!=';
	ADD		    = '+' ;
	SUB		        = '-' ;
	MUL		        = '*' ;
	MOD		    = '%' ;
	AND		    = '&' ;
	OR		        = '|' ;
	XOR		        = '^' ;
	INV		        = '~' ;
    // ASSIGN		= '=' ;
	DIV		        = '/' ;
    ARGS;
    LIST;
    NEG;
    POS;

    DATACOLUMNNAME;
    OMCOLUMNNAME;
}


// applies only to the parser:
@header {
    package  fr.unicaen.openmap.dataStorage.expr;
}

// applies only to the lexer:
@lexer::header {
    package  fr.unicaen.openmap.dataStorage.expr;
    import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
}
@lexer::members {
    protected DataTableModel dtm;
    public ExprGrammarLexer(CharStream input, DataTableModel dtm) {
            this(input);
            this.dtm = dtm;
    }
    public boolean isDataColumName(String s) {
        if (dtm == null)
            return false;
        for(String name : dtm.getDataColumnNames())
            if (name.equals(s)) 
                return true;
        return false;
    }  
    public boolean isOMColumName(String s) {
        if (dtm == null)
            return false;
        for(String name : dtm.getOMColumnNames())
            if (name.equals(s)) 
                return true;
        return false;
    }
}

arg 
    : value
    ;

args 
    : arg (COMMA! arg)*
    ;

value
    : arithmeticExpression
    | geometry
    | list
    ;
list 
    : LBRACE list_content? RBRACE -> ^(LIST list_content)
    ;
list_content
    : value (COMMA! value)*
    ;

 
primaryExpression
    : TRUE
    | FALSE
    | DATACOLUMNNAME
    | INT | FLOAT
    | arithmeticFunctionCall
    | LPAREN! arithmeticExpression RPAREN!
    | stringExpr
    | classificationCall
    ;

classificationCall 
    : FUNCTION_CLASSIF_NO_CLASS_NUMBER^  LPAREN! DATACOLUMNNAME (COMMA! arithmeticExpression)? RPAREN! 
    | FUNCTION_CLASSIF^  LPAREN! DATACOLUMNNAME COMMA! INT  (COMMA! arithmeticExpression)? RPAREN! 
    | FUNCTION_CLASSIF_CUSTOM^  LPAREN! DATACOLUMNNAME (COMMA! arithmeticExpression)+ RPAREN! 
    ;
arithmeticFunctionCall
    : FUNCTION_N_TO_N^ LPAREN! arithmeticExpression RPAREN! 
    | FUNCTION_N1OR2_TO_N^ LPAREN! arithmeticExpression (COMMA! arithmeticExpression)? RPAREN! 
    | FUNCTION_G_TO_N^ LPAREN! geometry RPAREN!
    | FUNCTION_G_G_TO_N^ LPAREN! geometry COMMA! geometry RPAREN!
    | FUNCTION_S_TO_N^ LPAREN! arithmeticExpression RPAREN! 
    | FUNCTION_S_S_TO_N^ LPAREN! arithmeticExpression COMMA! arithmeticExpression RPAREN! 
    | IF^ LPAREN! logicalORExpression COMMA! arithmeticExpression COMMA! arithmeticExpression RPAREN!
    ;    

geometry 
    : OMCOLUMNNAME
    | geometryFunctionCall
    ;

geometryFunctionCall
    : FUNCTION_N_N_TO_G^ LPAREN! arithmeticExpression COMMA! arithmeticExpression RPAREN! 
    | FUNCTION_G_TO_G^ LPAREN! geometry RPAREN! 
    | FUNCTION_G_N_TO_G^ LPAREN! geometry COMMA! arithmeticExpression RPAREN! 
    | FUNCTION_G_N_N_TO_G^ LPAREN! geometry COMMA! arithmeticExpression COMMA! arithmeticExpression RPAREN! 
    | FUNCTION_G_G_TO_G^ LPAREN! geometry COMMA! geometry RPAREN! 
;

stringExpr
    : STRING
    | FUNCTION_S_SPLUS_TO_S^ LPAREN! arithmeticExpression (COMMA! arithmeticExpression)+ RPAREN! 
    | FUNCTION_N_SPLUS_TO_S^ LPAREN! arithmeticExpression (COMMA! arithmeticExpression)+ RPAREN! 
    | FUNCTION_S_N1OR2_TO_S^ LPAREN! arithmeticExpression COMMA! arithmeticExpression (COMMA! arithmeticExpression)? RPAREN!
    | FUNCTION_S_TO_S^ LPAREN! arithmeticExpression RPAREN! 
    ;
unaryExpression
	: primaryExpression
	| unaryOperator^ unaryExpression
	;
	
unaryOperator
	: NOT
    | op=ADD { $op.setType(POS); }
	| op=SUB { $op.setType(NEG); }
	;

multiplicativeExpression
	: unaryExpression ( ( MUL | DIV | MOD )^ unaryExpression )*
	;

additiveExpression
	: multiplicativeExpression ( ( ADD | SUB )^ multiplicativeExpression )*
	;

relationalExpression
	: additiveExpression ( ( LT | GT | LTE | GTE  )^ additiveExpression )*
	;

equalityExpression
	: relationalExpression ( ( EQ | NEQ )^ relationalExpression )*
	;

logicalANDExpression
	: equalityExpression ( LAND^ equalityExpression )*
	;
	
logicalORExpression
	: logicalANDExpression ( LOR^ logicalANDExpression )*
	;
	
arithmeticExpression /* conditionalExpression */
	: logicalORExpression
	;


// lexer

TRUE : 'TRUE' | 'true';

FALSE : 'FALSE' | 'false';

// We reserve few names inorder to suppress ANTLR warning (see ID rule below)
FUNCTION_N_TO_N : 'COS';
FUNCTION_N1OR2_TO_N : 'ROUND';
FUNCTION_G_TO_N : 'ST_AREA';
FUNCTION_G_G_TO_N : 'CENTROIDDISTANCE';
FUNCTION_N_N_TO_G : 'MAKEPOINT';
FUNCTION_G_TO_G : 'CENTROID';
FUNCTION_G_N_TO_G : 'MAKEDISC';
FUNCTION_G_N_N_TO_G : 'MAKERECT';
FUNCTION_G_G_TO_G : 'ST_UNION';
FUNCTION_S_TO_N : 'LENGTH';
FUNCTION_S_TO_S : 'TRIM';
FUNCTION_S_SPLUS_TO_S : 'CONCAT';
FUNCTION_N_SPLUS_TO_S : 'CHOOSE';
FUNCTION_S_N1OR2_TO_S : 'SUBSTRING';
FUNCTION_S_S_TO_N : 'MATCH';
FUNCTION_CLASSIF_NO_CLASS_NUMBER : 'CL_DISCRETE';
FUNCTION_CLASSIF : 'CL_QUANTILE';
FUNCTION_CLASSIF_CUSTOM : 'CL_CUSTOM';
OMCOLUMNNAME : 'SHAPE';
DATACOLUMNNAME : 'DATA';
IF : ('if'|'IF');
NOT : ('!'|'not'|'NOT');
LAND : ('&&'|'and'|'AND');
LOR : ('||'|'or'|'OR');

ID  :	((Letter (Letter|JavaIDDigit|'/')*) | ( '`' .* '`'))
        { 
            // strip backquotes if exists
            String text = $text;
            if (text.length() > 2 && text.charAt(0) == '`')
                text = text.substring(1, text.length()-1);
            /* change $type from ID to the type of the list the token belongs to */
			/* allows to have case insentitive keywords */
            if (isDataColumName(text)) $type = DATACOLUMNNAME;
            else if (isOMColumName(text)) $type = OMCOLUMNNAME;
            else if (Function.isFunction_n_to_n($text)) $type = FUNCTION_N_TO_N; 
            else if (Function.isFunction_n1or2_to_n($text)) $type = FUNCTION_N1OR2_TO_N; 
            else if (Function.isFunction_g_to_n($text)) $type = FUNCTION_G_TO_N; 
            else if (Function.isFunction_g_g_to_n($text)) $type = FUNCTION_G_G_TO_N; 
            else if (Function.isFunction_n_n_to_g($text)) $type = FUNCTION_N_N_TO_G; 
            else if (Function.isFunction_g_to_g($text)) $type = FUNCTION_G_TO_G; 
            else if (Function.isFunction_g_n_to_g($text)) $type = FUNCTION_G_N_TO_G; 
            else if (Function.isFunction_g_n_n_to_g($text)) $type = FUNCTION_G_N_N_TO_G; 
            else if (Function.isFunction_g_g_to_g($text)) $type = FUNCTION_G_G_TO_G; 
            else if (Function.isFunction_s_to_n($text)) $type = FUNCTION_S_TO_N; 
            else if (Function.isFunction_s_to_s($text)) $type = FUNCTION_S_TO_S; 
            else if (Function.isFunction_s_splus_to_s($text)) $type = FUNCTION_S_SPLUS_TO_S; 
            else if (Function.isFunction_n_splus_to_s($text)) $type = FUNCTION_N_SPLUS_TO_S; 
            else if (Function.isFunction_s_n1or2_to_s($text)) $type = FUNCTION_S_N1OR2_TO_S; 
            else if (Function.isFunction_s_s_to_n($text)) $type = FUNCTION_S_S_TO_N; 
            else if (Function.isFunction_Classif($text)) $type = FUNCTION_CLASSIF;
            else if (Function.isFunction_Classif_No_Class_Number($text)) $type = FUNCTION_CLASSIF_NO_CLASS_NUMBER;
            else if (Function.isFunction_Classif_Custom($text)) $type = FUNCTION_CLASSIF_CUSTOM;
            // System.out.println("******** ID : >"+ $text + "< -> type " + $type);
        }
    ;

INT :	'0'..'9'+
    ;

FLOAT
    :   ('0'..'9')+ '.' ('0'..'9')* EXPONENT?
    |   '.' ('0'..'9')+ EXPONENT?
    |   ('0'..'9')+ EXPONENT
    ;

COMMENT
    :   '/*' ( options {greedy=false;} : . )* '*/' {$channel=HIDDEN;}
    ;

WS  :   ( ' '
        | '\t'
        | '\r'
        | '\n'
        ) {$channel=HIDDEN;}
    ;

STRING
    :  '"' ( ESC_SEQ | ~('\\'|'"') )* '"'
    ;

fragment
EXPONENT : ('e'|'E') ('+'|'-')? ('0'..'9')+ ;

fragment
HEX_DIGIT : ('0'..'9'|'a'..'f'|'A'..'F') ;

fragment
ESC_SEQ
    :   '\\' ('b'|'t'|'n'|'f'|'r'|'\"'|'\''|'\\')
    |   UNICODE_ESC
    |   OCTAL_ESC
    ;

fragment
OCTAL_ESC
    :   '\\' ('0'..'3') ('0'..'7') ('0'..'7')
    |   '\\' ('0'..'7') ('0'..'7')
    |   '\\' ('0'..'7')
    ;

fragment
UNICODE_ESC
    :   '\\' 'u' HEX_DIGIT HEX_DIGIT HEX_DIGIT HEX_DIGIT
    ;

fragment
Letter
    :  '\u0024' |
       '\u0041'..'\u005a' |
       '\u005f' |
       '\u0061'..'\u007a' |
       '\u00c0'..'\u00d6' |
       '\u00d8'..'\u00f6' |
       '\u00f8'..'\u00ff' |
       '\u0100'..'\u1fff' |
       '\u3040'..'\u318f' |
       '\u3300'..'\u337f' |
       '\u3400'..'\u3d2d' |
       '\u4e00'..'\u9fff' |
       '\uf900'..'\ufaff'
    ;

fragment
JavaIDDigit
    :  '\u0030'..'\u0039' |
       '\u0660'..'\u0669' |
       '\u06f0'..'\u06f9' |
       '\u0966'..'\u096f' |
       '\u09e6'..'\u09ef' |
       '\u0a66'..'\u0a6f' |
       '\u0ae6'..'\u0aef' |
       '\u0b66'..'\u0b6f' |
       '\u0be7'..'\u0bef' |
       '\u0c66'..'\u0c6f' |
       '\u0ce6'..'\u0cef' |
       '\u0d66'..'\u0d6f' |
       '\u0e50'..'\u0e59' |
       '\u0ed0'..'\u0ed9' |
       '\u1040'..'\u1049'
   ;
