// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.expr;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.lang.reflect.Constructor;

import org.antlr.runtime.tree.*;
import fr.unicaen.openmap.dataStorage.DataObject;
import fr.unicaen.openmap.dataStorage.DataStorage;
import fr.unicaen.openmap.dataStorage.DataStorageIteratorAdapter;
import fr.unicaen.openmap.dataStorage.DataStorageIterator;
import fr.unicaen.openmap.dataStorage.Centroid;
import fr.unicaen.openmap.dataStorage.BoundingBox;
import fr.unicaen.openmap.dataStorage.OMDataLocation;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.classification.Classification;
import fr.unicaen.openmap.util.classification.ClassificationOfDouble;
import com.bbn.openmap.geo.Geo;
import com.bbn.openmap.omGraphics.OMPoint;
import com.bbn.openmap.omGraphics.OMCircle;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.omGraphics.OMRect;
import com.bbn.openmap.proj.Length;
import com.bbn.openmap.util.Debug;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;

public class Function {

    static protected I18n i18n = Environment.getI18n();
    static protected Function f = new Function(null, null); // for use with i18n

    public enum FunctionRelationnalAndLogical {
	LT("<", i18n.get(f, "lt", "x < y true if x is less than y")),
	    GT(">", i18n.get(f, "gt", "x < y true if x is greater than y")),
	    LTE("<=", i18n.get(f, "lte", "x < y true if x is less than or equal to y")),
	    GTE(">=", i18n.get(f, "gte", "x >= y true if x is greater or equal to y")),
	    EQ("=", i18n.get(f, "eq", "x = y true if x equals y")),
	    NEQ("<>", i18n.get(f, "neq", "x <> y, is true if x and y differs")),
	    LAND("and", i18n.get(f, "land", "a and b, is true if both a and b are true")),
	    LOR("or", i18n.get(f, "lor", "a or b,  true if a is true or b is true"));
	
        private String sign;
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

        FunctionRelationnalAndLogical(String s, String doc){
            this.sign = s;
	    this.doc = doc;
        }

        @Override
	    public String toString(){return this.sign;}

        public static String getPattern(){return "";}
    }

    public enum Function_n_n_to_n {
        ADD("+", i18n.get(f, "add", "x+y, computes the sum of the two numbers")),
	    MUL("*", i18n.get(f, "mul", "x*y, computes the product of the two numbers")),
	    SUB("-", i18n.get(f, "sub", "x-y, computes the difference of the two numbers")),
	    DIV(" / ", i18n.get(f, "div", "x/y, computes the quotient of the two numbers<br>Beware: put space around / in case of confusion with a LAT/LON geometry column name")),
	    MOD("%", i18n.get(f, "mod", "x%y, computes the remainder of the two numbers")),
	    ;
	
        private String sign;
        private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

        Function_n_n_to_n(String s, String doc){
            this.sign = s;
	    this.doc = doc;
        }

        @Override
	    public String toString(){return this.sign;}

        public static String getPattern(){return "";}
    }

    public enum Function_n_to_n {
	ABS(i18n.get(f, "abs", "abs(x), returns the the absolute value of x")),
	    FLOOR(i18n.get(f, "floor", "floor(x), returns the largest integer  that is less than or equal to x")),
	    INT(i18n.get(f, "int", "int(x), returns the largest integer  that is less than or equal to x")),
	    CEIL(i18n.get(f, "ceil", "ceil(x), returns the smallest integer  that is greater than or equal to x")),
	    LOG(i18n.get(f, "log", "log(x), computes the natural logarithm (base e) of x")),
	    LOG10(i18n.get(f, "log10", "log10(x), computes the base 10 logarithm of x")),
	    // in the geom tab
	    DEGTOKM(i18n.get(f, "degtokm", "degtokm(x), computes the length in km of an x degrees arc on the surface of the Earth. Beware, it uses a rather crude method")),
	    KMTODEG(i18n.get(f, "kmtodeg", "kmtodeg(x), give the angle of an arc of x km long. Beware, it uses the same crude method as degtokm")),
	    // in the trigo tab
	    SIN(i18n.get(f, "sin", "sin(x), computes the trigonometric sine of an angle x")),
	    COS(i18n.get(f, "cos", "cos(x), computes the trigonometric cosine of an angle.")),
	    TAN(i18n.get(f, "tan", "tan(x), computes the trigonometric tangent of an angle x")),
	    NZ(i18n.get(f, "nz", "nz(x), returns 0 if x is null, x otherwise"));

        public static String getPattern(){return "()";}
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

	Function_n_to_n(String doc){
	    this.doc = doc;
        }
        public static double eval(Function_n_to_n f, double d) {
            switch (f) {
	    case ABS:
		return Math.abs(d);
	    case FLOOR:
	    case INT:
		return Math.floor(d);
	    case CEIL:
		return Math.ceil(d);
	    case LOG:
		return Math.log(d);
	    case LOG10:
		return Math.log10(d);
	    case DEGTOKM:
		return Length.KM.fromRadians(Length.DECIMAL_DEGREE.toRadians(d));
	    case KMTODEG:
		return Length.DECIMAL_DEGREE.fromRadians(Length.KM.toRadians(d));
	    case SIN:
		return Math.sin(d);
	    case COS:
		return Math.cos(d);
	    case TAN:
		return Math.tan(d);
	    case NZ:
		return d;
            }
            return 0;
        }

    }

    static public boolean isFunction_n_to_n(String f) {
        for (Function_n_to_n ff : Function_n_to_n.values()) {
            if (ff.toString().equals(f.toUpperCase())) {
                return true;
            }
        }
        return false;
    }

    public enum Function_n1or2_to_n {
	MOD(i18n.get(f, "mod", "mod(x, y), returns x modulo y")),
	ROUND(i18n.get(f, "round", "round(x), round x to the closest integer"));
	
        public String getPattern(){ return this == MOD ? "(,)" : "()"; }
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

	Function_n1or2_to_n(String doc){
	    this.doc = doc;
        }
        public static double eval(Function_n1or2_to_n f, Object args[]) {
            switch (f) {
	    case MOD:
		if (args.length == 1)
		    return (Util.parseDouble(args[0]));
		return Util.parseDouble(args[0]) % Util.parseDouble(args[1]);
	    case ROUND:
		if (args.length == 1)
		    return Math.round(Util.parseDouble(args[0]));
		{
		    double power = Math.pow(10, -Util.parseDouble(args[1]));
		    return Math.round(Util.parseDouble(args[0])/power)*power;
		}
            }
            return 0;
        }

    }

    static public boolean isFunction_n1or2_to_n(String f) {
        for (Function_n1or2_to_n ff : Function_n1or2_to_n.values()) {
            if (ff.toString().equals(f.toUpperCase())) {
                return true;
            }
        }
        return false;
    }

    public enum Function_g_to_n {

        ST_LATITUDE(i18n.get(f, "st_latitude", "st_latitude(geom) returns the latitude of the centroid of the geom")),
	    ST_LONGITUDE(i18n.get(f, "st_longitude", "st_longitude(geom) returns the longitude of the centroid of the geom")),
	    ST_AREA(i18n.get(f, "st_area", "st_area(geom) returns the area of the geom")),
	    ST_LENGTH(i18n.get(f, "st_length", "st_length(geom) returns the length (or perimeter) of the geom")),
	    //JST_AREA(i18n.get(f, "jst_area", "jst_area(geom) returns the area of the geom")),
	    //JST_LENGTH(i18n.get(f, "jst_area", "jst_area(geom) returns the area of the geom")),
	    ;


        public static String getPattern(){return "()";}
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

	Function_g_to_n(String doc){
	    this.doc = doc;
        }

        public static double eval(Function_g_to_n f, OMDataLocation om) {
	    switch (f) {
	    case ST_LONGITUDE:
		if (om == null || om.getCentroid() == null)
		    return 0;
		return om.getCentroid().getLon();
	    case ST_LATITUDE:
		if (om == null || om.getCentroid() == null)
		    return 0;
		return om.getCentroid().getLat();
	    case ST_AREA:
		if (om == null) return 0;
		if (om.getOMGraphic() == null) return 0;
		return SpatialFunction.areaInSQKM(om.getOMGraphic());
	    case ST_LENGTH:
		if (om == null) return 0;
		if (om.getOMGraphic() == null) return 0;
		return SpatialFunction.lengthInKM(om.getOMGraphic());
		//case JST_AREA:
		//	if (om == null) return 0;
		//	if (om.getOMGraphic() == null) return 0;
		//	return SpatialFunction.jst_area(om.getOMGraphic());
		//case JST_LENGTH:
		//	if (om == null) return 0;
		//	if (om.getOMGraphic() == null) return 0;
		//	return SpatialFunction.jst_length(om.getOMGraphic());
            }
            return 0;
        }
    }

    static public boolean isFunction_g_to_n(String f) {
        for (Function_g_to_n ff : Function_g_to_n.values())
            if (ff.toString().equals(f.toUpperCase()))
                return true;
        return false;
    }

    public enum Function_g_g_to_n {

        CENTROIDDISTANCE(i18n.get(f, "centroidDistance", "centroidDistance(geom1, geom2) returns the distance between the centroids of geom1 and geom2")),
	    ST_DISTANCE(i18n.get(f, "st_distance", "st_distance(geom1, geom2) returns the shortest distance between the 2 geometries geom1 and geom2")),
	    ST_INTERSECTS(i18n.get(f, "st_intersects", "st_intersects(geom1, geom2) returns true if the two geometries intersects")),
	    ;

        public static double eval(Function_g_g_to_n f, OMDataLocation om1, OMDataLocation om2) {
	    if (om1 == null || om2 == null)
		return 0;
            switch (f) {
	    case CENTROIDDISTANCE:
		if (om1.getCentroid() == null || om2.getCentroid() == null)
		    return 0;
		return Geo.distanceKM(om1.getCentroid().getLat(), om1.getCentroid().getLon(),
				      om2.getCentroid().getLat(), om2.getCentroid().getLon());
	    case ST_DISTANCE:
		if (om1 == null || om2 == null) return 0;
		return SpatialFunction.distance(om1, om2);		
	    case ST_INTERSECTS:
		if (om1 == null || om2 == null) return 0;
		return SpatialFunction.intersects(om1, om2) ? 1 : 0;		
            }
            return 0;
        }

        public static String getPattern(){return "(,)";}
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

	Function_g_g_to_n(String doc){
	    this.doc = doc;
        }
    }

    static public boolean isFunction_g_g_to_n(String f) {
        for (Function_g_g_to_n ff : Function_g_g_to_n.values())
            if (ff.toString().equals(f.toUpperCase()))
                return true;
        return false;
    }

    public enum Function_g_g_to_g {

	ST_INTERSECTION(i18n.get(f, "st_intersection", "st_intersection(geom1, geom2) computes the set intersection of the two geometries")),
	    ST_UNION(i18n.get(f, "st_union", "st_union(geom1, geom2) computes the set union of the two geometries"));

        public static OMDataLocation eval(Function_g_g_to_g f, OMDataLocation om1, OMDataLocation om2) {
	    if (om1 == null || om2 == null)
		return null;
            switch (f) {
	    case ST_UNION:
		if (om1.getOMGraphic() == null || om2.getOMGraphic() == null)
		    return null;
		return SpatialFunction.union(om1, om2);
	    case ST_INTERSECTION:
		if (om1.getOMGraphic() == null || om2.getOMGraphic() == null)
		    return null;
		return SpatialFunction.intersection(om1, om2);
            }
            return null;
        }

        public static String getPattern(){return "(,)";}
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

	Function_g_g_to_g(String doc){
	    this.doc = doc;
        }
    }

    static public boolean isFunction_g_g_to_g(String f) {
        for (Function_g_g_to_g ff : Function_g_g_to_g.values())
            if (ff.toString().equals(f.toUpperCase()))
                return true;
        return false;
    }

    public enum Function_n_n_to_g {

        MAKEPOINT(i18n.get(f, "makepoint", "makePoint(lat, lon) makes a point geometry given a latitude and a longitude"));

        public static OMDataLocation eval(Function_n_n_to_g f, double d1, double d2) {
            switch (f) {
	    case MAKEPOINT:
		return new OMDataLocation(
					  new OMPoint(d1, d2),
					  new Centroid(d2, d1),
					  new BoundingBox(d2, d1, d2, d1));
            }
            return null;
        }

        public static String getPattern(){return "(,)";}
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

	Function_n_n_to_g(String doc){
	    this.doc = doc;
        }
    }

    static public boolean isFunction_n_n_to_g(String f) {
        for (Function_n_n_to_g ff : Function_n_n_to_g.values())
            if (ff.toString().equals(f.toUpperCase()))
                return true;
        return false;
    }

    public enum Function_g_to_g {

        CENTROID(i18n.get(f, "centroid", "centroid(geom) returns a point centroid of the geometry")),
        ST_CONVEXHULL(i18n.get(f, "st_convexhull", "convexhull(geom) returns a convex hull of the geometry")),
	    // TESTJTS(i18n.get(f, "testjts", "testjts(geom) returns the geomfromjts(geomtojts(geom))")),
	    ;

        public static OMDataLocation eval(Function_g_to_g f, OMDataLocation om) {
            switch (f) {
	    case CENTROID:
		return new OMDataLocation(
					  null,
					  om.getCentroid() != null ? om.getCentroid()
					  : SpatialFunction.centroid(om.getOMGraphic()),
					  null);
	    case ST_CONVEXHULL:
		{
		    ArrayList<OMDataLocation> l = new ArrayList<OMDataLocation>(1);
		    l.add(om);
		    return SpatialFunction.convexHull(l);
		}
		//     case TESTJTS:
		// 	return fr.unicaen.openmap.jts.JTSFactory.testJTS(om);
            }
            return null;
        }

        public static String getPattern(){return "()";}
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

	Function_g_to_g(String doc){
	    this.doc = doc;
        }
    }

    static public boolean isFunction_g_to_g(String f) {
        for (Function_g_to_g ff : Function_g_to_g.values())
            if (ff.toString().equals(f.toUpperCase()))
                return true;
        return false;
    }

    public enum Function_g_n_to_g {
	ST_BUFFER(i18n.get(f, "st_buffer", "st_buffer(geom, d) makes a geometry that is the buffer of the geom at distance d")),
	    MAKEDISC(i18n.get(f, "makeDisc", "makeDisc(geom, r) makes a disk geometry centered on the centroid of the geom and with the radius r")),
	    SIMPLIFY(i18n.get(f, "simplify", "simplify(geom, t) simplify the geom using the distance tolerance t")),
	    SPLINE(i18n.get(f, "spline", "spline(geom, nbsegs) constructs a spline using nbsegs for each line"));
        public static OMDataLocation eval(Function_g_n_to_g f, OMDataLocation om, double d) {
	    if (om == null)
		return null;
            switch (f) {
	    case ST_BUFFER:
		{
		    if (om.getOMGraphic() == null)
			return null;
		    double lat = om.getCentroid().getLat();
		    double lon = om.getCentroid().getLon();
		    double dInRadians = Length.KM.toRadians(d);
		    return new OMDataLocation(
					      fr.unicaen.openmap.jts.JTS.buffer(om.getOMGraphic(), d),
					      om.getCentroid(),
					      new BoundingBox(lon - dInRadians, lat - dInRadians,
							      lon + dInRadians, lat + dInRadians));
		}
	    case MAKEDISC: 
		{
		    if (om.getCentroid() == null)
			return null;
		    double lat = om.getCentroid().getLat();
		    double lon = om.getCentroid().getLon();
		    double dInRadians = Length.KM.toRadians(d);
		    double dInDegrees = Length.DECIMAL_DEGREE.fromRadians(dInRadians);
		    return new OMDataLocation((f == Function_g_n_to_g .MAKEDISC 
					      ? new OMCircle(lat, lon, dInRadians, Length.RADIAN)
					      : new OMRect(
							 lat + dInDegrees, lon - dInDegrees,
							 lat - dInDegrees, lon + dInDegrees,
							 OMGraphic.LINETYPE_STRAIGHT)),
					      om.getCentroid(),
					      new BoundingBox(lon - dInDegrees, lat + dInDegrees,
							      lon + dInDegrees, lat - dInDegrees));
		}
	    case SIMPLIFY:
		{
		    if (om.getOMGraphic() == null)
			return null;
		    return SpatialFunction.simplify(om, d);
		}
	    case SPLINE:
		{
		    if (om.getOMGraphic() == null)
			return null;
		    return SpatialFunction.spline(om, (int)d);
		}
	    }
	    return null;
	}

        public static String getPattern(){return "(,)";}
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

	Function_g_n_to_g(String doc){
	    this.doc = doc;
        }
    }

    static public boolean isFunction_g_n_to_g(String f) {
        for (Function_g_n_to_g ff : Function_g_n_to_g.values())
            if (ff.toString().equals(f.toUpperCase()))
                return true;
        return false;
    }


    public enum Function_g_n_n_to_g {
	MAKERECT(i18n.get(f, "makeRect", "makeRect(geom, l, h) makes a rectangle centered on the centroid of the geom and with the length l and height h in km")),
	    ST_TRANSLATE(i18n.get(f, "translate", "translate(geom, dlon, dlat) translates the geom of dlat km in latitude and dlon km in longitude")),
	    ;

        public static OMDataLocation eval(Function_g_n_n_to_g f, OMDataLocation om, Double d1, Double d2) {
            switch (f) {
	    case MAKERECT: 
		{
		    if (om.getCentroid() == null)
			return null;
		    double lat = om.getCentroid().getLat();
		    double lon = om.getCentroid().getLon();
		    double dlonInRadians = Length.KM.toRadians(d1);
		    double rlonInDegrees = Length.DECIMAL_DEGREE.fromRadians(dlonInRadians)/2;
		    double dlatInRadians = Length.KM.toRadians(d2);
		    double rlatInDegrees = Length.DECIMAL_DEGREE.fromRadians(dlatInRadians)/2;
		    return new OMDataLocation(new OMRect(
							 lat + rlatInDegrees, lon - rlatInDegrees,
							 lat - rlonInDegrees, lon + rlonInDegrees,
							 OMGraphic.LINETYPE_STRAIGHT),
					      om.getCentroid(),
					      new BoundingBox(lon - rlonInDegrees, lat + rlatInDegrees,
							      lon + rlonInDegrees, lat - rlatInDegrees));
		}
	case ST_TRANSLATE:
		return SpatialFunction.translate(om, d1, d2);
            }
            return null;
        }

        public static String getPattern(){return "(,,)";}
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

	Function_g_n_n_to_g(String doc){
	    this.doc = doc;
        }
    }

    static public boolean isFunction_g_n_n_to_g(String f) {
        for (Function_g_n_n_to_g ff : Function_g_n_n_to_g.values())
            if (ff.toString().equals(f.toUpperCase()))
                return true;
        return false;
    }

    public enum Function_s_to_n {

	LENGTH(i18n.get(f, "length", "length(s), returns the number of character of the string s"));

        public static String getPattern(){return "()";}
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

	Function_s_to_n(String doc){
	    this.doc = doc;
        }
        public static Object eval(Function_s_to_n f, Object o) {
	    if (o == null)
		return null;
	    String s = o.toString();
            switch (f) {
	    case LENGTH:
		return new Double(s.length());
            }
            return null;
        }

    }

    static public boolean isFunction_s_to_n(String f) {
        for (Function_s_to_n ff : Function_s_to_n.values()) {
            if (ff.toString().equals(f.toUpperCase())) {
                return true;
            }
        }
        return false;
    }

    public enum Function_s_to_s {

	TRIM(i18n.get(f, "trim", "trim(s), trims white spaces at both ends of the string s")),
	    ORD(i18n.get(f, "ord", "ord(s), returns the code of the first character of the string s")),
	    CHAR(i18n.get(f, "char", "char(n), returns a character whose code is n"));

        public static String getPattern(){return "()";}
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

	Function_s_to_s(String doc){
	    this.doc = doc;
        }
        public static Object eval(Function_s_to_s f, Object o) {
	    if (o == null)
		return null;
	    String s = o.toString();
            switch (f) {
	    case TRIM:
		return s.trim();
	    case ORD:
		return s.length() > 0 ? s.codePointAt(0) : 0;
	    case CHAR:
		return new String(Character.toChars((int)parseDouble(s)));
            }
            return null;
        }

    }

    static public boolean isFunction_s_to_s(String f) {
        for (Function_s_to_s ff : Function_s_to_s.values()) {
            if (ff.toString().equals(f.toUpperCase())) {
                return true;
            }
        }
        return false;
    }


    public enum Function_s_splus_to_s {

	CONCAT(i18n.get(f, "concat", "concat(s1, s2, ...), catenate the strings s1, s2, ..."));

        public static String getPattern(){return "(,)";}
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

	Function_s_splus_to_s(String doc){
	    this.doc = doc;
        }
        public static Object eval(Function_s_splus_to_s f, Object ... args) {
            switch (f) {
	    case CONCAT:
		{
		    StringBuffer buffer = new StringBuffer();
		    for (Object o : args)
			if (o != null)
			    buffer.append(o.toString());
		    return buffer.toString();
		}
            }
            return 0;
        }

    }

    static public boolean isFunction_s_splus_to_s(String f) {
        for (Function_s_splus_to_s ff : Function_s_splus_to_s.values()) {
            if (ff.toString().equals(f.toUpperCase())) {
                return true;
            }
        }	
        return false;
    }

    public enum Function_n_splus_to_s {

	CHOOSE(i18n.get(f, "choose", "choose(n, s0, s1, s2, ...), chooses the nth string expression argument sn,"));

        public static String getPattern(){return "(,)";}
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

	Function_n_splus_to_s(String doc){
	    this.doc = doc;
        }
        public static Object eval(Function_n_splus_to_s f, double d, Object ... args) {
            switch (f) {
	    case CHOOSE:
		{ 
		    int index = (int)d;
		    return index < 0 || index >= args.length ? null : args[(int)d];
		}
	    }
            return null;
        }

    }

    static public boolean isFunction_n_splus_to_s(String f) {
        for (Function_n_splus_to_s ff : Function_n_splus_to_s.values()) {
            if (ff.toString().equals(f.toUpperCase())) {
                return true;
            }
        }	
        return false;
    }
    
    public enum Function_s_n1or2_to_s {

	SUBSTRING(i18n.get(f, "substring", "substring(s, i[, j]), extract a substring from s starting at index i and extending to character at index j or at the end of the string s if j is omitted. Indexes start at 0."));

        public static String getPattern(){return "(,)";}
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

	Function_s_n1or2_to_s(String doc){
	    this.doc = doc;
        }
        public static Object eval(Function_s_n1or2_to_s f, Object o, Object... args) {
	    if (o == null)
		return null;
	    String s = o.toString();
	    try {
		switch (f) {
		case SUBSTRING:
		    if (args.length == 1)
			return s.substring((int)Math.round(Util.parseDouble(args[0])));
		    return s.substring((int)Math.round(Util.parseDouble(args[0])), 
				       (int)Math.round(Util.parseDouble(args[1])));
		}
	    } catch(Exception e) {
		Debug.error("Error for "+f+"("+s+java.util.Arrays.asList(args)+")");
	    }
            return null;
        }

    }

    static public boolean isFunction_s_n1or2_to_s(String f) {
        for (Function_s_n1or2_to_s ff : Function_s_n1or2_to_s.values()) {
            if (ff.toString().equals(f.toUpperCase())) {
                return true;
            }
        }
        return false;
    }

    public enum Function_s_s_to_n {

	MATCH(i18n.get(f, "match", "match(s, r), returns true if the regexp r matches the string s"));

        public static String getPattern(){return "(,)";}
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

	Function_s_s_to_n(String doc){
	    this.doc = doc;
        }
        public static Object eval(Function_s_s_to_n f, Object o1, Object o2) {
	    
	    switch (f) {
	    case MATCH:
		return o1 == null || o2 == null ? null  
		    : o1.toString().matches(o2.toString());
	    }
            return 0;
        }

    }

    static public boolean isFunction_s_s_to_n(String f) {
        for (Function_s_s_to_n ff : Function_s_s_to_n.values()) {
            if (ff.toString().equals(f.toUpperCase())) {
                return true;
            }
        }
        return false;
    }

    static final String NULL = "null";
    
    @SuppressWarnings("unchecked")
	static public Map<Object, Classification> initClassif(String classificationClassName, String columnName, int nbClasses, CommonTree groupbyValueTree, ExprDataStorage dataStorage, Function function) {
	try {
	    Map<Object, Classification> ret = new HashMap<Object, Classification>();

	    Class cls = Class.forName(classificationClassName);
	    Constructor constructor = null;
	    Constructor[] constructors = cls.getConstructors();
	    for (Constructor c : cls.getConstructors()) {
		java.lang.reflect.Type[] params = c.getGenericParameterTypes();
		if (params.length == 2) { // && params[1] == Class.forName("java.lang.Number")) {
		    constructor = c;
		    break;
		}
	    }
	    if (constructor == null) {
		Debug.error("[Function] *** construction failure for " + classificationClassName + " No constructor");
		return null;
	    }

	    Class clsClassificationOfDouble = Class.forName("fr.unicaen.openmap.util.classification.ClassificationOfDouble");
	    if (! clsClassificationOfDouble.isAssignableFrom(cls)) {
		// work with String
		// build one array of data per groupbyValue
		Map<Object, ArrayList<String>> dataMap = new HashMap<Object, ArrayList<String>>();
		DataStorageIterator dsi = new DataStorageIteratorAdapter(dataStorage.getDataStorage());
		while (dsi.hasNext()) {
		    DataObject dataObject = dsi.next();
		    Object data = dataObject.getData(columnName);
		    String value = data == null ? NULL : data.toString();
		    Object groupbyValue = groupbyValueTree == null ? NULL : function.eval(groupbyValueTree, dataObject);
		    ArrayList<String> dataGroupbyValue = dataMap.get(groupbyValue);
		    if (dataGroupbyValue == null) 
			dataGroupbyValue = new ArrayList<String>();
		    dataGroupbyValue.add(value);
		    dataMap.put(groupbyValue, dataGroupbyValue);
		}
		//
		for (Object groupbyValue : dataMap.keySet()) {
		    fr.unicaen.openmap.util.classification.DiscreteOfString classif = 
			(fr.unicaen.openmap.util.classification.DiscreteOfString) 
			constructor.newInstance(new Object[]{dataMap.get(groupbyValue), nbClasses});		
		    classif.reInit(dataMap.get(groupbyValue));
		    ret.put(groupbyValue, classif);
		}
	    } else { // work with Double
		// build one array of data per groupbyValue
		Map<Object, ArrayList<Double>> dataMap = new HashMap<Object, ArrayList<Double>>();
		DataStorageIterator dsi = new DataStorageIteratorAdapter(dataStorage.getDataStorage());
		while (dsi.hasNext()) {
		    DataObject dataObject = dsi.next();
		    Object d = dataObject.getData(columnName);
		    if (d == null)
			continue;
		    if ("".equals(d.toString()))
			continue;
		    double value = Util.parseDouble(d);
		    if (Double.isNaN(value) || value == Double.NEGATIVE_INFINITY || value == Double.POSITIVE_INFINITY)
			continue;
		    Object groupbyValue = groupbyValueTree == null ? NULL : function.eval(groupbyValueTree, dataObject);
		    ArrayList<Double> dataGroupbyValue = dataMap.get(groupbyValue);
		    if (dataGroupbyValue == null) 
			dataGroupbyValue = new ArrayList<Double>();
		    dataGroupbyValue.add(value);
		    dataMap.put(groupbyValue, dataGroupbyValue);
		}
		//
		for (Object groupbyValue : dataMap.keySet()) {
		    ClassificationOfDouble classif = (ClassificationOfDouble) constructor.newInstance(new Object[]{dataMap.get(groupbyValue), nbClasses});		
		    classif.reInit(dataMap.get(groupbyValue));
		    ret.put(groupbyValue, classif);
		}
	    }
	    return ret;
	} catch (Exception e) {
	    Debug.error("[Function] *** construction failure for " + classificationClassName);
	    e.printStackTrace();
	}
	return null;
    }

    static public Map<Object, Double> initPercentile(String columnName, int nth, CommonTree groupbyValueTree, ExprDataStorage dataStorage, Function function) {
	Map<Object, Classification> classifMap = 
	    initClassif("fr.unicaen.openmap.util.classification.Quantile", columnName, 100, groupbyValueTree, dataStorage, function);
	Map<Object, Double> ret = new HashMap<Object, Double>();
	for (Object groupbyValue : classifMap.keySet())
	    ret.put(groupbyValue, ((ClassificationOfDouble)classifMap.get(groupbyValue)).getNthBreak(nth));
	// System.out.println("*** Percentile result "+ret);
	return ret;
    }

    static public Map<Object, Double> initMinOrMax(Function_Classif_No_Class_Number f, String columnName, CommonTree groupbyValueTree, ExprDataStorage dataStorage, Function function) {	
	double initValue = (f == Function_Classif_No_Class_Number.MIN ? Double.MAX_VALUE : Double.MIN_VALUE);
	Map<Object, Double>  ret = new HashMap<Object, Double>();
	DataStorageIterator dsi = new DataStorageIteratorAdapter(dataStorage.getDataStorage());
	while (dsi.hasNext()) {
	    DataObject dataObject = dsi.next();
	    double value = Util.parseDouble(dataObject.getData(columnName));
	    Object groupbyValue = groupbyValueTree == null ? NULL : function.eval(groupbyValueTree, dataObject);
	    Double retGroupbyValue = ret.get(groupbyValue);
	    if (retGroupbyValue == null) 
		retGroupbyValue = initValue;
	    if (f == Function_Classif_No_Class_Number.MIN) {
		if (value < retGroupbyValue)
		    ret.put(groupbyValue, value);
	    } else {
		if (value > retGroupbyValue)
		    ret.put(groupbyValue, value);
	    }
	}
	return ret;
    }
    /**
     * variance should use a more numerical stable algorithm
     */
    static public Map<Object, Double> initVariance(Function_Classif_No_Class_Number f, String columnName, CommonTree groupbyValueTree, ExprDataStorage dataStorage, Function function) {
	DataStorageIterator dsi = new DataStorageIteratorAdapter(dataStorage.getDataStorage());
	// we avoid the sum2 computation for these functions
	boolean needSum2 = f != Function_Classif_No_Class_Number.AVERAGE
	    && f != Function_Classif_No_Class_Number.SUM && f != Function_Classif_No_Class_Number.COUNT;

	// first step: compute the count, sum and sum square for each groupby value
	Map<Object, Integer>  nb = new HashMap<Object, Integer>();
	Map<Object, Double>  sum = new HashMap<Object, Double>();
	Map<Object, Double>  sum2 = new HashMap<Object, Double>();
	while (dsi.hasNext()) {
	    DataObject dataObject = dsi.next();
	    Object data = dataObject.getData(columnName);
	    // skip null values
	    if (data == null || (data instanceof String && "".equals((String)data))) 
		continue;
	    if (data instanceof Double) {
		Double d = (Double)data;
		if (d.isNaN() || d == Double.NEGATIVE_INFINITY || d == Double.POSITIVE_INFINITY)
		    continue;
	    }
	    double value = Util.parseDouble(data);
	    Object groupbyValue = groupbyValueTree == null ? NULL : function.eval(groupbyValueTree, dataObject);
	    Integer nbGroupbyValue = nb.get(groupbyValue);
	    Double sumGroupbyValue = sum.get(groupbyValue);
	    Double sum2GroupbyValue = sum2.get(groupbyValue);
	    if (nbGroupbyValue == null) {
		nbGroupbyValue = 0;
		sumGroupbyValue = 0.0;
		sum2GroupbyValue = 0.0;
	    }
	    nbGroupbyValue++;
	    nb.put(groupbyValue, nbGroupbyValue);
	    sumGroupbyValue += value;
	    sum.put(groupbyValue, sumGroupbyValue);
	    if (needSum2) {
		sum2GroupbyValue += value*value;
		sum2.put(groupbyValue, sum2GroupbyValue);
	    }
	}

	// second step: compute the function 
	Map<Object, Double>  ret = new HashMap<Object, Double>();
	for(Object groupbyValue : nb.keySet()) {
	    switch(f) {
	    case AVERAGE:
		ret.put(groupbyValue, sum.get(groupbyValue)/nb.get(groupbyValue));
		continue;
	    case SUM:
		ret.put(groupbyValue, sum.get(groupbyValue));
		continue;
	    case COUNT:
		ret.put(groupbyValue, new Double(nb.get(groupbyValue)));
		continue;
	    }
	    double sumGroupbyValue = sum.get(groupbyValue);
	    double sum2GroupbyValue = sum2.get(groupbyValue);
	    int nbGroupbyValue = nb.get(groupbyValue);
	    double var  = sum2GroupbyValue/nbGroupbyValue
		- (sumGroupbyValue*sumGroupbyValue)/(nbGroupbyValue*nbGroupbyValue);
	    switch(f) {
	    case VARP:
		ret.put(groupbyValue, var);
		continue;
	    case STDEVP:
		ret.put(groupbyValue, Math.sqrt(var));
		continue;
	    case VAR:
		ret.put(groupbyValue, var*nbGroupbyValue/(nbGroupbyValue-1));
		continue;
	    case STDEV:
		ret.put(groupbyValue, Math.sqrt(var*nbGroupbyValue/(nbGroupbyValue-1)));
		continue;
	    }
	}
	return ret;
    }
    
    public enum Function_Classif {
	
	CL_QUANTILE(i18n.get(f, "cl_quantile", "cl_quantile(column, nbclasses), returns the class number regarding the classification of the column"),
		    "fr.unicaen.openmap.util.classification.Quantile"),
	    CL_NESTEDMEANS(i18n.get(f, "cl_nestedmeans", "cl_nestedmeans(column, nbclasses), returns the class number regarding the classification of the column"),
			   "fr.unicaen.openmap.util.classification.NestedMeans"),
	    CL_STEPCLASSIFICATION(i18n.get(f, "cl_stepclassification", "cl_stepclassification(column, nbclasses), returns the class number regarding the classification of the column"),
				  "fr.unicaen.openmap.util.classification.StepClassification"),
	    CL_STANDARDIZATION(i18n.get(f, "cl_standardization", "cl_standardization(column, nbclasses), returns the class number regarding the classification of the column"),
			       "fr.unicaen.openmap.util.classification.Standardization"),
	    CL_BOXPLOT(i18n.get(f, "cl_boxplot", "cl_boxplot(column, nbclasses), returns the class number regarding the classification of the column"),
		       "fr.unicaen.openmap.util.classification.BoxPlot"),
	    PERCENTILE(i18n.get(f, "percentile", "percentile(column, n), returns the nth percentile regarding the distribution defined by the column"),
		       null),
	    ;	
        public static String getPattern(){return "(,)";}
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

	private String classificationClass;
	public String getClassificationClass() {
	    return classificationClass;
	}
	Function_Classif(String doc, String classificationClass){
	    this.doc = doc;
	    this.classificationClass = classificationClass;
        }

	@SuppressWarnings("unchecked")
	    private Object eval(String columnName, Object value, int nbClasses, Object groupbyValue, CommonTree groupbyValueTree, ExprDataStorage ds, CommonTree tree, Function function) {
	    switch(this) {
	    default:
		{
		    Map<Object, Classification> classifmap = (Map<Object, Classification>)ds.getEnviron(tree);
		    if (classifmap == null) {
			classifmap = initClassif(getClassificationClass(), columnName, nbClasses, groupbyValueTree, ds, function); 
			ds.putEnviron(tree, classifmap);
		    }
		    Classification classif = classifmap.get(groupbyValue == null ? NULL : groupbyValue);
		    if (classif == null) {
			return null;
		    }
		    return classif.getClass(value == null ? NULL : value.toString()); 
		}
	    case PERCENTILE:
		{
		    Map<Object, Double> result = (Map<Object, Double>)ds.getEnviron(tree);
		    if (result == null) {
			result = initPercentile(columnName, nbClasses, groupbyValueTree, ds, function);
			ds.putEnviron(tree, result);
		    }
		    return result.get(groupbyValue == null ? NULL : groupbyValue);
		}
	    }
	}
    }

    static public boolean isFunction_Classif(String f) {
        for (Function_Classif ff : Function_Classif.values()) {
            if (ff.toString().equals(f.toUpperCase())) {
                return true;
            }
        }
        return false;
    }

    /** Functions that take a column name as parameter and need to parse once the data storage */
    public enum Function_Classif_No_Class_Number {

	CL_DISCRETE(i18n.get(f, "cl_discrete", "cl_discrete(column), returns the class number regarding a discrete classification of the column"),
		    "fr.unicaen.openmap.util.classification.DiscreteOfString"),
	    MIN(i18n.get(f, "min", "min(column), return the minimun value of the given column"),
		null),
	    MAX(i18n.get(f, "max", "max(column), return the maximum value of the given column"),
		null),
	    COUNT(i18n.get(f, "count", "count(column), return the numbers of lines of the given column"),
		  null),
	    SUM(i18n.get(f, "sum", "sum(column), return the sum od the values of the given column"),
		null),
	    AVERAGE(i18n.get(f, "average", "average(column), return the average of the values of the given column"),
		    null),
	    MEDIAN(i18n.get(f, "median", "median(column), return the median value of the values of the given column (value separating the higher half, from the lower half)"),
		    null),
	    STDEVP(i18n.get(f, "stdevp", "stdevp(column), return the standard deviation of the population given by the values of the given column"),
		   null),
	    VARP(i18n.get(f, "varp", "varp(column), return the variance of the population given by the values of the given column"),
		 null),
	    STDEV(i18n.get(f, "stdev", "stdev(column), return the standard deviation of the sample given by the values of the given colum (also known as the N-1-standard deviationn)"),
		  null),
	    VAR(i18n.get(f, "var", "var(column), return the variance of the sample given by the values of the given column (also known as th N-1 variance)"),
		null),
	    ;	
        public static String getPattern(){return "()";}
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

	private String classificationClass;
	public String getClassificationClass() {
	    return classificationClass;
	}
	Function_Classif_No_Class_Number(String doc, String classificationClass){
	    this.doc = doc;
	    this.classificationClass = classificationClass;
        }
	/** evaluation is only computed once for each tree: the result is cached in the ds environment */
	@SuppressWarnings("unchecked")
	    public Object eval(String columnName, Object value, Object groupbyValue, CommonTree groupbyValueTree, ExprDataStorage ds, CommonTree  tree, Function function) {
	    switch(this) {
	    case MEDIAN:
		return Function_Classif.PERCENTILE.eval(columnName, value, 50, groupbyValue, groupbyValueTree, ds, tree, function);
	    case CL_DISCRETE:
		{
		    Map<Object, Classification> classif = (Map<Object, Classification>)ds.getEnviron(tree);
		    if (classif == null) {
			classif = initClassif(getClassificationClass(), columnName, 0, groupbyValueTree, ds, function); 
			ds.putEnviron(tree, classif);
		    }
		    Classification cls = classif.get(groupbyValue == null ? NULL : groupbyValue);
		    return cls == null ? NULL : cls.getClass(value == null ? NULL : value.toString()); 
		}
	    case MIN:
	    case MAX:
		{
		    Map<Object, Double> result = (Map<Object, Double>)ds.getEnviron(tree);
		    if (result == null) {
			result = initMinOrMax(this, columnName, groupbyValueTree, ds, function);
			ds.putEnviron(tree, result);
		    }
		    return result.get(groupbyValue == null ? NULL : groupbyValue);
		}
	    case COUNT:
	    case SUM:
	    case AVERAGE:
	    case STDEVP:
	    case VARP:
	    case STDEV:
	    case VAR:
		{
		    Map<Object, Double> result = (Map<Object, Double>)ds.getEnviron(tree);
		    if (result == null) {
			result = initVariance(this, columnName, groupbyValueTree, ds, function);
			ds.putEnviron(tree, result);
		    }
		    return result.get(groupbyValue == null ? NULL : groupbyValue);
		}
	    }
	    return 0;
	}
    }

    static public boolean isFunction_Classif_No_Class_Number(String f) {
        for (Function_Classif_No_Class_Number ff : Function_Classif_No_Class_Number.values()) {
            if (ff.toString().equals(f.toUpperCase())) {
                return true;
            }
        }
        return false;
    }


    public enum Function_Classif_Custom {

	CL_CUSTOM(i18n.get(f, "cl_custom", "cl_custom(column, v1, v2, vn-1), returns the class number regarding the breaks v1, v2, vn-1")),
	    ;	
        public static String getPattern(){return "(,)";}
	private String doc;
	public String getDoc() { return doc; }
	public FunctionDescription getFunctionDescription() {
	    return new FunctionDescription(toString(), getPattern(), doc);
	}

	Function_Classif_Custom(String doc){
	    this.doc = doc;
        }
	
	public Object eval(double value, double args[]) {
            switch (this) {
	    case CL_CUSTOM:
		{ 
		    int i;
		    for(i = 0; i < args.length; i++)
			if (value < args[i])
			    break;
		    return i;
		}
	    }
            return null;
        }

    }

    static public boolean isFunction_Classif_Custom(String f) {
        for (Function_Classif_Custom ff : Function_Classif_Custom.values()) {
            if (ff.toString().equals(f.toUpperCase())) {
                return true;
            }
        }
        return false;
    }

    public static enum TreeType {
	NUMERIC,
	    STRING,
	    GEOMETRY,
	    BOOLEAN,
	    UNKNOWN;
	public static TreeType getType(CommonTree tree) {
	    switch (tree.getType()) {
	    case ExprGrammarLexer.TRUE:
	    case ExprGrammarLexer.FALSE:
		return BOOLEAN;
	    case ExprGrammarLexer.INT:
	    case ExprGrammarLexer.FLOAT:
		return NUMERIC;
	    case ExprGrammarLexer.STRING:
		return STRING;
	    case ExprGrammarLexer.DATACOLUMNNAME:
		return UNKNOWN;
	    case ExprGrammarLexer.OMCOLUMNNAME:
		return GEOMETRY;
	    case ExprGrammarLexer.POS:
	    case ExprGrammarLexer.NEG:
	    case ExprGrammarLexer.MUL:
	    case ExprGrammarLexer.DIV:
	    case ExprGrammarLexer.MOD:
	    case ExprGrammarLexer.ADD:
	    case ExprGrammarLexer.SUB:
		return NUMERIC;
	    case ExprGrammarLexer.LT:
	    case ExprGrammarLexer.GT:
	    case ExprGrammarLexer.LTE:
	    case ExprGrammarLexer.GTE:
	    case ExprGrammarLexer.EQ:
	    case ExprGrammarLexer.NEQ:
	    case ExprGrammarLexer.LAND:
	    case ExprGrammarLexer.LOR:
		return BOOLEAN;
	    case ExprGrammarLexer.IF:
		return UNKNOWN;
	    case ExprGrammarLexer.FUNCTION_N_TO_N:
	    case ExprGrammarLexer.FUNCTION_N1OR2_TO_N:
	    case ExprGrammarLexer.FUNCTION_G_TO_N:
	    case ExprGrammarLexer.FUNCTION_G_G_TO_N:
		return NUMERIC;
	    case ExprGrammarLexer.FUNCTION_N_N_TO_G:
	    case ExprGrammarLexer.FUNCTION_G_TO_G:
	    case ExprGrammarLexer.FUNCTION_G_N_TO_G:
	    case ExprGrammarLexer.FUNCTION_G_N_N_TO_G:
	    case ExprGrammarLexer.FUNCTION_G_G_TO_G:
		return GEOMETRY;
	    case ExprGrammarLexer.FUNCTION_S_TO_N:
		return NUMERIC;
	    case ExprGrammarLexer.FUNCTION_S_TO_S:
	    case ExprGrammarLexer.FUNCTION_S_SPLUS_TO_S:
	    case ExprGrammarLexer.FUNCTION_N_SPLUS_TO_S:
	    case ExprGrammarLexer.FUNCTION_S_N1OR2_TO_S:
		return STRING;
	    case ExprGrammarLexer.FUNCTION_S_S_TO_N:
	    case ExprGrammarLexer.FUNCTION_CLASSIF_NO_CLASS_NUMBER:
	    case ExprGrammarLexer.FUNCTION_CLASSIF:
	    case ExprGrammarLexer.FUNCTION_CLASSIF_CUSTOM:
		return NUMERIC;
	    }
	    return UNKNOWN;
	}
    }

    protected CommonTree _tree;
    /** DataStorage is used by Function_Classif_* (for the moment...) */
    protected ExprDataStorage _dataStorage;
    Function(CommonTree tree, ExprDataStorage ds) {
	this._tree = tree;
	this._dataStorage = ds;
    }

    public Object eval(DataObject data) {
	return eval(_tree, data);
    }

    // Strip back tips if exists
    private static void stripBackTicks(CommonTree tree) {
	String text = tree.getText();
	if (text.length() < 2 || text.charAt(0) != '`')
	    return;
	tree.getToken().setText(text.substring(1, text.length()-1));
    }

    protected Object eval(CommonTree tree, DataObject data) {
	if (tree == null || data == null)
	    return null;

	// System.out.println("[Eval expr]  token type: "+tree.getType()+" for text: "+tree.getText());
	try {
	    Object child0 = null;
	    if (tree.getChildCount() > 0) {
		// Forall f ≠ nz, f(null) -> null
		child0 = eval((CommonTree)tree.getChild(0), data);
		// System.out.println("[Eval expr]  token type: "+tree.getType()+" for text: "+tree.getText()+ " with "+ child0);
		if (tree.getType() == ExprGrammarLexer.FUNCTION_N_TO_N && "NZ".equals(tree.getText().toUpperCase()))
		    return (child0 == null || "".equals(child0.toString()) || (child0 instanceof Double && ((Double)child0).isNaN()) ) ? 0.0 : child0;
		if(child0 == null || "".equals(child0.toString())) // cast numeric null to NaN
		    switch (TreeType.getType(tree)) {
		    case NUMERIC: return Double.NaN;
		    case BOOLEAN: return false;
		    default:
			return child0;
		    }
	    }
	    

	    switch (tree.getType()) {
	    case ExprGrammarLexer.TRUE:
		return true; // new Integer(1);
	    case ExprGrammarLexer.FALSE:
		return false; // new Integer(0);
	    case ExprGrammarLexer.INT:
		return Integer.parseInt(tree.getText());
	    case ExprGrammarLexer.FLOAT:
		return Double.parseDouble(tree.getText());
	    case ExprGrammarLexer.STRING:
		return tree.getText().substring(1, tree.getText().length()-1);
		// case ExprGrammarLexer.ID:                return tree.getText();

	    case ExprGrammarLexer.DATACOLUMNNAME:
		stripBackTicks(tree);
		return data.getData(tree.getText());
	    case ExprGrammarLexer.OMCOLUMNNAME:
		stripBackTicks(tree);
		return data.getOMDataLocation(tree.getText());

	    case ExprGrammarLexer.POS:
		return parseDouble(child0);
	    case ExprGrammarLexer.NEG:
		return -parseDouble(child0);
	    case ExprGrammarLexer.MUL:
		return parseDouble(child0) * parseDouble(eval((CommonTree)tree.getChild(1), data));
	    case ExprGrammarLexer.DIV:
		return parseDouble(child0) / parseDouble(eval((CommonTree)tree.getChild(1), data));
	    case ExprGrammarLexer.MOD:
		return parseDouble(child0) % parseDouble(eval((CommonTree)tree.getChild(1), data));
	    case ExprGrammarLexer.ADD:
		return parseDouble(child0) + parseDouble(eval((CommonTree)tree.getChild(1), data));
	    case ExprGrammarLexer.SUB:
		return parseDouble(child0) - parseDouble(eval((CommonTree)tree.getChild(1), data));

		// for relationnal operator, we should first test for the types of the args
		// Bool, Number, Strings, ...
	    case ExprGrammarLexer.LT:
	    case ExprGrammarLexer.GT:
	    case ExprGrammarLexer.LTE:
	    case ExprGrammarLexer.GTE:
	    case ExprGrammarLexer.EQ:
	    case ExprGrammarLexer.NEQ:
		return evalRelOp(tree.getType(), child0, eval((CommonTree)tree.getChild(1), data));

	    case ExprGrammarLexer.LAND:
		return toBoolean(child0) && toBoolean(eval((CommonTree)tree.getChild(1), data));
	    case ExprGrammarLexer.LOR:
		return toBoolean(child0) || toBoolean(eval((CommonTree)tree.getChild(1), data));
	    case ExprGrammarLexer.IF:
		return toBoolean(child0)
		    ? eval((CommonTree)tree.getChild(1), data)
		    : eval((CommonTree)tree.getChild(2), data);

	    case ExprGrammarLexer.FUNCTION_N_TO_N:
		return Function_n_to_n.eval(Function_n_to_n.valueOf(tree.getText().toUpperCase()),
					    parseDouble(child0));

	    case ExprGrammarLexer.FUNCTION_N1OR2_TO_N:
		{
		    Object[] args = new Object[tree.getChildCount()];
		    for (int i = 0; i < tree.getChildCount(); i++)
			args[i] = eval((CommonTree)tree.getChild(i), data);
		    return Function_n1or2_to_n.eval(Function_n1or2_to_n.valueOf(tree.getText().toUpperCase()), 
						    args);
		}

	    case ExprGrammarLexer.FUNCTION_G_TO_N:
		return Function_g_to_n.eval(Function_g_to_n.valueOf(tree.getText().toUpperCase()),
					    (OMDataLocation)child0);

	    case ExprGrammarLexer.FUNCTION_G_G_TO_N:
		return Function_g_g_to_n.eval(Function_g_g_to_n.valueOf(tree.getText().toUpperCase()),
					      (OMDataLocation)child0,
					      (OMDataLocation)eval((CommonTree)tree.getChild(1), data));
	    
	    case ExprGrammarLexer.FUNCTION_N_N_TO_G:
		return Function_n_n_to_g.eval(Function_n_n_to_g.valueOf(tree.getText().toUpperCase()),
					      parseDouble(child0),
					      parseDouble(eval((CommonTree)tree.getChild(1), data)));
	    case ExprGrammarLexer.FUNCTION_G_TO_G:
		return Function_g_to_g.eval(Function_g_to_g.valueOf(tree.getText().toUpperCase()),
					    (OMDataLocation)child0);
	    case ExprGrammarLexer.FUNCTION_G_N_TO_G:
		return Function_g_n_to_g.eval(Function_g_n_to_g.valueOf(tree.getText().toUpperCase()),
					      (OMDataLocation)child0,
					      parseDouble(eval((CommonTree)tree.getChild(1), data)));
	    case ExprGrammarLexer.FUNCTION_G_N_N_TO_G:
		return Function_g_n_n_to_g.eval(Function_g_n_n_to_g.valueOf(tree.getText().toUpperCase()),
						(OMDataLocation)child0,
						parseDouble(eval((CommonTree)tree.getChild(1), data)),
						parseDouble(eval((CommonTree)tree.getChild(2), data)));
	    case ExprGrammarLexer.FUNCTION_G_G_TO_G:
		return Function_g_g_to_n.eval(Function_g_g_to_n.valueOf(tree.getText().toUpperCase()),
					      (OMDataLocation)child0,
					      (OMDataLocation)eval((CommonTree)tree.getChild(1), data));
	    case ExprGrammarLexer.FUNCTION_S_TO_N:
		return Function_s_to_n.eval(Function_s_to_n.valueOf(tree.getText().toUpperCase()),
					    child0);
	    case ExprGrammarLexer.FUNCTION_S_TO_S:
		return Function_s_to_s.eval(Function_s_to_s.valueOf(tree.getText().toUpperCase()),
					    child0);
	    case ExprGrammarLexer.FUNCTION_S_SPLUS_TO_S:
		{
		    Object[] args = new Object[tree.getChildCount()];
		    for (int i = 0; i < tree.getChildCount(); i++)
			args[i] = eval((CommonTree)tree.getChild(i), data);
		    return Function_s_splus_to_s.eval(Function_s_splus_to_s.valueOf(tree.getText().toUpperCase()),
						      args);
		}
	    case ExprGrammarLexer.FUNCTION_N_SPLUS_TO_S:
		{
		    Object[] args = new Object[tree.getChildCount()-1];
		    for (int i = 1; i < tree.getChildCount(); i++)
			args[i-1] = eval((CommonTree)tree.getChild(i), data);
		    return Function_n_splus_to_s.eval(Function_n_splus_to_s.valueOf(tree.getText().toUpperCase()),
						      parseDouble(child0),
						      args);
		}
	    case ExprGrammarLexer.FUNCTION_S_N1OR2_TO_S:
		{
		    Object[] args = new Object[tree.getChildCount()-1];
		    for (int i = 1; i < tree.getChildCount(); i++)
			args[i-1] = eval((CommonTree)tree.getChild(i), data);
		    return Function_s_n1or2_to_s.eval(Function_s_n1or2_to_s.valueOf(tree.getText().toUpperCase()), 
						      child0, args);
		}
	    case ExprGrammarLexer.FUNCTION_S_S_TO_N:
		return Function_s_s_to_n.eval(Function_s_s_to_n.valueOf(tree.getText().toUpperCase()),
					      child0,
					      eval((CommonTree)tree.getChild(1), data));

		// the first arg of classif is a columnName. We pass both the name AND the current value	
	    case ExprGrammarLexer.FUNCTION_CLASSIF_NO_CLASS_NUMBER:
		// we need to pass the tree to the Function_Classif_....eval() in order to store global classif and value
		// and to eval the groupby values thanks to this
		return Function_Classif_No_Class_Number.valueOf(tree.getText().toUpperCase()).
		    eval(((CommonTree)tree.getChild(0)).getText(),
			 child0,
			 eval((CommonTree)tree.getChild(1), data),
			 (CommonTree)tree.getChild(1),
			 _dataStorage, tree, this);
	    case ExprGrammarLexer.FUNCTION_CLASSIF:
		return Function_Classif.valueOf(tree.getText().toUpperCase()).
		    eval(((CommonTree)tree.getChild(0)).getText(),
			 child0,
			 Integer.parseInt(((CommonTree)tree.getChild(1)).getText()),
			 eval((CommonTree)tree.getChild(2), data),
			 (CommonTree)tree.getChild(2),
			 _dataStorage, tree, this);
		
	    case ExprGrammarLexer.FUNCTION_CLASSIF_CUSTOM:
		// just some kind of CHOOSE
		{
		    double[] args = new double[tree.getChildCount()-1];
		    for (int i = 1; i < tree.getChildCount(); i++)
			args[i-1] = parseDouble(eval((CommonTree)tree.getChild(i), data));
		    return Function_Classif_Custom.valueOf(tree.getText().toUpperCase()).
			eval(parseDouble(child0), args);
		}
	    }
	    Debug.error("[Eval expr] invalid token type: "+tree.getType()+" for text: "+tree.getText());
	} catch(Exception e) {
	    Debug.error("[Eval expr] exception "+e+" for "+tree.getType());
	    e.printStackTrace();
	}
	return null;
    }

    private static Boolean evalRelOp(int op, Object child0, Object child1) { 
	if (child0 instanceof String && child1 instanceof String) {
	    int cmp = ((String)child0).compareTo((String)child1);	    
	    // System.out.println("Compare 2 strings: "+child0 +" "+child1+" -> "+cmp);
	    switch(op) {
            case ExprGrammarLexer.LT:
                return cmp < 0;
            case ExprGrammarLexer.GT:
                return cmp > 0;
            case ExprGrammarLexer.LTE:
                return cmp <= 0;
            case ExprGrammarLexer.GTE:
                return cmp >= 0;
            case ExprGrammarLexer.EQ:
                return cmp == 0;
            case ExprGrammarLexer.NEQ:
                return cmp != 0;
	    }
	    return false;
	}
	if (child0 instanceof Boolean && child1 instanceof Boolean) {
	    int bchild0 = ((Boolean)child0) ? 1 : 0;
	    int bchild1 = ((Boolean)child1) ? 1 : 0;	    
	    switch(op) {
            case ExprGrammarLexer.LT:
                return bchild0 < bchild1;
            case ExprGrammarLexer.GT:
                return bchild0 > bchild1;
            case ExprGrammarLexer.LTE:
                return bchild0 <= bchild1;
            case ExprGrammarLexer.GTE:
                return bchild0 >= bchild1;
            case ExprGrammarLexer.EQ:
                return bchild0 == bchild1;
            case ExprGrammarLexer.NEQ:
                return bchild0 != bchild1;
	    }
	    return false;
	}
	double dchild0 = parseDouble(child0);
	double dchild1 = parseDouble(child1);
	switch(op) {
	case ExprGrammarLexer.LT:
	    return dchild0 < dchild1;
	case ExprGrammarLexer.GT:
	    return dchild0 > dchild1;
	case ExprGrammarLexer.LTE:
	    return dchild0 <= dchild1;
	case ExprGrammarLexer.GTE:
	    return dchild0 >= dchild1;
	case ExprGrammarLexer.EQ:
	    return dchild0 == dchild1;
	case ExprGrammarLexer.NEQ:
	    return dchild0 != dchild1;
	}
	return false;
    }

    static private boolean toBoolean(Object o) {
        if (o instanceof Boolean)
            return (Boolean) o;
        
        if (o instanceof String)
            return ((String) o).matches("true");
        
        return parseDouble(o) != 0;
    }

    static double parseDouble(Object o) {
	if (o instanceof Boolean) 
	    return (Boolean)o ? 1 : 0;
	return Util.parseDouble(o);    
    }

    static public boolean isFunctionName(String name) {
	return  isFunction_n_to_n(name)
	    || isFunction_n1or2_to_n(name)
	    || isFunction_g_to_n(name)
	    || isFunction_g_g_to_n(name)
	    || isFunction_n_n_to_g(name)
	    || isFunction_g_to_g(name)
	    || isFunction_g_n_to_g(name);
    } 
    
    static public String getFunctionsDocs() {
	StringBuffer buffer = new StringBuffer();
	buffer.append("<h2>"+i18n.get(f, "unaryfunction","Numerical unary functions")+"</h2>");
	for (Function_n_to_n ff : Function_n_to_n.values())
	    buffer.append("<p>"+ff.getDoc()+"</p>");
	for (Function_n1or2_to_n ff : Function_n1or2_to_n.values())
	    buffer.append("<p>"+ff.getDoc()+"</p>");

	buffer.append("<h2>"+i18n.get(f, "binaryfunction","Numerical binary functions")+"</h2>");
	for (Function_n_n_to_n ff : Function_n_n_to_n.values())
	    buffer.append("<p>"+ff.getDoc()+"</p>");

	buffer.append("<h2>"+i18n.get(f, "g_to_n_function","Numerical functions from geometries")+"</h2>");
	for (Function_g_to_n ff : Function_g_to_n.values())
	    buffer.append("<p>"+ff.getDoc()+"</p>");

	buffer.append("<h2></h2>");
	// buffer.append("<h2>"+i18n.get(f, "g_g_to_n_function","Numerical functions from 2 geometries")+"</h2>");
	for (Function_g_g_to_n ff : Function_g_g_to_n.values())
	    buffer.append("<p>"+ff.getDoc()+"</p>");

	buffer.append("<h2>"+i18n.get(f, "to_g_function","Geometry constructors")+"</h2>");
	for (Function_n_n_to_g ff : Function_n_n_to_g.values())
	    buffer.append("<p>"+ff.getDoc()+"</p>");
     
	buffer.append("<h2></h2>");
	for (Function_g_to_g ff : Function_g_to_g.values())
	    buffer.append("<p>"+ff.getDoc()+"</p>");
     
	buffer.append("<h2></h2>");
	// buffer.append("<h2>"+i18n.get(f, "g_n_to_g_function","Geometry constructors with a geometry and a number")+"</h2>");
	for (Function_g_n_to_g ff : Function_g_n_to_g.values())
	    buffer.append("<p>"+ff.getDoc()+"</p>");

	for (Function_g_n_n_to_g ff : Function_g_n_n_to_g.values())
	    buffer.append("<p>"+ff.getDoc()+"</p>");

	for (Function_g_g_to_g ff : Function_g_g_to_g.values())
	    buffer.append("<p>"+ff.getDoc()+"</p>");

	buffer.append("<h2>"+i18n.get(f, "s_function","String manipulation functions")+"</h2>");
	for (Function_s_to_n ff : Function_s_to_n.values())
	    buffer.append("<p>"+ff.getDoc()+"</p>");

        for (Function_s_to_s ff : Function_s_to_s.values())
	    buffer.append("<p>"+ff.getDoc()+"</p>");

        for (Function_s_splus_to_s ff : Function_s_splus_to_s.values())
	    buffer.append("<p>"+ff.getDoc()+"</p>");

        for (Function_s_n1or2_to_s ff : Function_s_n1or2_to_s.values())
	    buffer.append("<p>"+ff.getDoc()+"</p>");
	
    	return "<html>"+buffer.toString().replaceAll("</?html>", "") + "</html>";
    }

    static public List<FunctionDescriptionList> getDescriptions() {
	List<FunctionDescriptionList> descriptions = new ArrayList<FunctionDescriptionList>();
	FunctionDescriptionList description;

	description = new FunctionDescriptionList(i18n.get(Function.class, "logic", "Logic"));
        for (FunctionRelationnalAndLogical ff : FunctionRelationnalAndLogical.values())
	    description.add(ff.getFunctionDescription());
	description.add(new FunctionDescription("if", "(,,)",
						i18n.get(f, "if", "if(b, x, y) gives x if b is true, y oterwise")));
        for (Function_n_splus_to_s ff : Function_n_splus_to_s.values())
	    description.add(ff.getFunctionDescription());
	descriptions.add(description);
	
	description = new FunctionDescriptionList(i18n.get(Function.class, "math", "Math"));
	for (Function_n_n_to_n ff : Function_n_n_to_n.values())
	    description.add(ff.getFunctionDescription());
	for (Function_n1or2_to_n ff : Function_n1or2_to_n.values()) {
	    description.add(ff.getFunctionDescription());
	}
	for (Function_n_to_n ff : Function_n_to_n.values()) {
	    // DEGTOKM and KMTODEG are in the geometry tab
	    if (ff == Function_n_to_n.DEGTOKM) continue;
	    if (ff == Function_n_to_n.KMTODEG)	continue;
	    description.add(ff.getFunctionDescription());
	}
	descriptions.add(description);

	//	description = new FunctionDescriptionList(i18n.get(Function.class, "trigo", "Trigo"));
	//	description.add(Function_n_to_n.SIN.getFunctionDescription());
	//	description.add(Function_n_to_n.COS.getFunctionDescription());
	//	description.add(Function_n_to_n.TAN.getFunctionDescription());
	//	descriptions.add(description);

	description = new FunctionDescriptionList(i18n.get(Function.class, "geometry", "Geom"));
	// DEGTOKM and KMTODEG from Function_n_to_n
	description.add(Function_n_to_n.DEGTOKM.getFunctionDescription());
       	description.add(Function_n_to_n.KMTODEG.getFunctionDescription());
	for (Function_g_to_n ff : Function_g_to_n.values())
	    description.add(ff.getFunctionDescription());
	for (Function_g_g_to_n ff : Function_g_g_to_n.values())
	    description.add(ff.getFunctionDescription());
	for (Function_n_n_to_g ff : Function_n_n_to_g.values())
	    description.add(ff.getFunctionDescription());
	for (Function_g_to_g ff : Function_g_to_g.values())
	    description.add(ff.getFunctionDescription());
	for (Function_g_n_to_g ff : Function_g_n_to_g.values())
	    description.add(ff.getFunctionDescription());
	for (Function_g_n_n_to_g ff : Function_g_n_n_to_g.values())
	    description.add(ff.getFunctionDescription());
	for (Function_g_g_to_g ff : Function_g_g_to_g.values())
	    description.add(ff.getFunctionDescription());
	descriptions.add(description);

	description = new FunctionDescriptionList(i18n.get(Function.class, "string", "String"));
	for (Function_s_to_n ff : Function_s_to_n.values())
	    description.add(ff.getFunctionDescription());
        for (Function_s_to_s ff : Function_s_to_s.values())
	    description.add(ff.getFunctionDescription());
        for (Function_s_splus_to_s ff : Function_s_splus_to_s.values())
	    description.add(ff.getFunctionDescription());
        for (Function_s_n1or2_to_s ff : Function_s_n1or2_to_s.values())
	    description.add(ff.getFunctionDescription());
	descriptions.add(description);

	description = new FunctionDescriptionList(i18n.get(Function.class, "stats", "Stats"));
        for (Function_Classif_No_Class_Number ff : Function_Classif_No_Class_Number.values()) {
	    if (ff == Function_Classif_No_Class_Number.CL_DISCRETE) continue; // in the classif tab
	    description.add(ff.getFunctionDescription());
	}
	description.add(Function_Classif.PERCENTILE.getFunctionDescription());
	descriptions.add(description);

	description = new FunctionDescriptionList(i18n.get(Function.class, "classif", "Classif"));
	description.add(Function_Classif_No_Class_Number.CL_DISCRETE.getFunctionDescription());
	for (Function_Classif ff : Function_Classif.values()) {
	    if (ff == Function_Classif.PERCENTILE) continue; // in the classif tab
	    description.add(ff.getFunctionDescription());
	}
	description.add(Function_Classif_Custom.CL_CUSTOM.getFunctionDescription());
	descriptions.add(description);

	return descriptions;
    }
}
