// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.expr;

import fr.unicaen.openmap.dataStorage.DataObject;
import fr.unicaen.openmap.dataStorage.DataStorage;
import fr.unicaen.openmap.dataStorage.DataStorageListener;
import fr.unicaen.openmap.dataStorage.expr.gui.RequestFilterDataStoragePanel;
import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import fr.unicaen.openmap.util.Util;
import java.util.ArrayList;
import java.util.Arrays;
import javax.swing.Box;
import javax.swing.JComponent;

/**
 *
 * @author jchauvea
 */
public class RequestFilterDataStorage extends AddColumnDataStorage {

    public ArrayList<Integer> filteredDataIndex = new ArrayList<Integer>();
    public static String ifBegin = "if(";
    public static String ifEnd = ",1,0)";
    protected String exprFilter;

    public enum requestElement {

        EQ("="),
        LTE("<="),
        GTE(">="),
        LT("<"),
        GT(">"),
        PARENTHESIS("( )"),
        AND("AND"),
        OR("OR"),
        NOT("NOT"),
        DIFF("<>");
        private String sign;

        private requestElement(String s) {
            this.sign = s;
        }

        @Override
        public String toString() {
            return sign;
        }
    }

    public RequestFilterDataStorage(DataStorageListener dsl) {
        super(dsl);
    }

    public RequestFilterDataStorage(DataStorageListener dsl, DataStorage dataStorage) {
        super(dsl, dataStorage);
    }

    /** Don't really compute a column but the filter index.
     *   Always returns null, so that no column name is added to the model.
     */
    @Override
    protected String computeColumn() {
        filteredDataIndex.clear();
        if (treeExpr == null) {
            return null;
        }
        DataStorage ds = getDataStorage();
        int count = ds.getCount();
 	Function function = new Function(treeExpr, this);
	if (function == null)
	    return null;
	Util.resetMaxNumberFormatExceptionOutput();
	for (int i = 0; i < count; i++) {
            DataObject data = ds.getElementAt(i);
	    Object res = function.eval(data);
            if (res != null && ((Integer)res) == 1) {
                filteredDataIndex.add(i);
            }
        }
	Util.resetMaxNumberFormatExceptionOutput();
	addedColumnName = null;
	return null; 
    }

    public String getName() {
        return name + " " + getExpr();
    }
    
    /** never add a column so set the name to "" */
    @Override
    public String setAddedColumnName(String s) {
        addedColumnName = "";
	return addedColumnName;
    }

    @Override
    public DataObject getElementAt(int i) {
        return getDataStorage().getElementAt(filteredDataIndex.get(i));
    }

    @Override
    public int getCount() {
        return filteredDataIndex.size(); 
    }

    RequestFilterDataStoragePanel gui;
    @Override
    public JComponent getGUI() {
        if (gui == null) {
            gui = new RequestFilterDataStoragePanel(this);
        }
	gui.setExpressionPanelState(exprIsOK());
        Box b = Box.createVerticalBox();
        b.add(gui);
        b.add(getDataStorage().getGUI());
	
        return b;
    }

    @Override
    public boolean setExpr(String _expr) {
        if (super.setExpr(_expr)) {
            exprFilter = _expr;
            return super.setExpr(ifBegin + _expr + ifEnd);
        } else
            return false;
    }

    @Override
    public String getExpr() {
        return exprFilter;
    }
}
