// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * AggregationDataStorage.java
 *
 * Created on 9 April 2010
 *
 * @author Jacques.Madelaine@info.unicaen.fr
 *
 */
package fr.unicaen.openmap.dataStorage.expr;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.swing.JOptionPane;

import fr.unicaen.openmap.dataStorage.OMDataLocation;
import fr.unicaen.openmap.dataStorage.Centroid;
import fr.unicaen.openmap.dataStorage.BoundingBox;
import fr.unicaen.openmap.util.Util;

import fr.unicaen.openmap.jts.JTSFactory;
import com.vividsolutions.jts.operation.distance.DistanceOp;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryCollection;
import com.vividsolutions.jts.simplify.TopologyPreservingSimplifier;
import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Polygon;
import com.vividsolutions.jts.algorithm.ConvexHull;

import com.vividsolutions.jts.operation.predicate.RectangleIntersects;
import com.vividsolutions.jts.geom.util .AffineTransformation;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.omGraphics.OMRect;
import com.bbn.openmap.omGraphics.OMSpline;
import com.bbn.openmap.omGraphics.OMPoint;
import com.bbn.openmap.omGraphics.OMLine;
import com.bbn.openmap.geo.Geo;
import com.bbn.openmap.proj.Length;
import com.bbn.openmap.util.Debug;

public class SpatialFunction {

    public final static double distanceTolerance = 0.001;

    public static double distance(OMDataLocation omdl1, OMDataLocation omdl2) {
	Geometry geom1 = null, geom2 = null;

	if (omdl1.getOMGraphic() == null) {
	    if (omdl1.getCentroid() == null)
		return 0;
	    geom1 = JTSFactory.LonLatToJTSGeometry(omdl1.getCentroid().getLon(), omdl1.getCentroid().getLat());
	} else
	    geom1 = JTSFactory.OMGraphicToJTSGeometry(omdl1.getOMGraphic());
	if (omdl2.getOMGraphic() == null) {
	    if (omdl2.getCentroid() == null)
		return 0;
	    geom2 = JTSFactory.LonLatToJTSGeometry(omdl2.getCentroid().getLon(), omdl2.getCentroid().getLat());
	} else
	    geom2 = JTSFactory.OMGraphicToJTSGeometry(omdl2.getOMGraphic());
	return DistanceOp.distance(geom1, geom2);
    }

    public static double distance(OMGraphic omg1, OMGraphic omg2) {
	return DistanceOp.distance(JTSFactory.OMGraphicToJTSGeometry(omg1),
				   JTSFactory.OMGraphicToJTSGeometry(omg2));
    }

    public static boolean intersects(OMDataLocation omdl1, OMDataLocation omdl2) {
	BoundingBox b1 = omdl1.getBoundingBox();
	BoundingBox b2 = omdl2.getBoundingBox();
	if (b1 != null && b2 != null && ! b1.intersects(b2))
	    return false;
	return intersects(omdl1.getOMGraphic(), omdl2.getOMGraphic());
    }

    public static boolean intersects(OMGraphic omg1, OMGraphic omg2) {
	if (omg1 == null || omg2 == null)
	    return false;

	// JTS intersects does not support collections
	if (omg1 instanceof OMGraphicList)
	    return intersects((OMGraphicList)omg1, omg2);

	if (omg2 instanceof OMGraphicList)
	    return intersects((OMGraphicList)omg2, omg1);

	// // OMPoint optimisation
	// if (omg1 instanceof OMPoint) 
	//     return intersects((OMPoint)omg1, omg2);
	// if (omg2 instanceof OMPoint) 
	//     return intersects((OMPoint)omg2, omg1);
	// 
	// // OMRect optimisation
	// if (omg1 instanceof OMRect)
	//     return intersects((OMRect)omg1, omg2);
	// if (omg2 instanceof OMRect) 
	//     return intersects((OMRect)omg2, omg1);

	return JTSFactory.OMGraphicToJTSGeometry(omg1).intersects(JTSFactory.OMGraphicToJTSGeometry(omg2));
 
	// // tests
	// Geometry g1 = JTSFactory.OMGraphicToJTSGeometry(omg1);
 	// Geometry g2 = JTSFactory.OMGraphicToJTSGeometry(omg2);
	// com.vividsolutions.jts.io.WKTWriter writer = new com.vividsolutions.jts.io.WKTWriter();
	// System.out.println(writer.write(g1) + "\n" + 	writer.write(g2));
	// System.out.println(g1.relate(g2) + " intersects " + g1.relate(g2).isIntersects());
 	// general case
 	// return g1.relate(g2).isIntersects();
    }

    /** We try the intersection with every geom list member as JTS doesn't support list for predicate */
    public static boolean intersects(OMGraphicList omglist, OMGraphic omg) {
	for (OMGraphic o : omglist) {
	    if (intersects(o, omg))
		return true;
	}
	return false;
    }

    public static OMDataLocation spline(OMDataLocation omdl, int arc_length) {
	return new OMDataLocation(spline(omdl.getOMGraphic(), arc_length), omdl.getCentroid(), omdl.getBoundingBox());
    }

    public static OMGraphic spline(OMGraphic omg, int arc_length) {
	if (! (omg instanceof OMPoly))
	    return omg;
	// OMPoly omp = (OMPoly)omg;
	//  return new OMSpline(omp.getLatLonArray(), omp.getUnits(), omp.getLineType(), arc_length/100);

	return spline(((OMPoly)omg), arc_length);
    }

    public static OMPoly spline(OMPoly omp, int nbarc) {
	double[] rawllpts = omp.getLatLonArray();
	int nbPts = rawllpts.length/2;
	double[] x = new double[nbPts];
        double[] y = new double[nbPts];  
	for(int i = 0; i < nbPts; i++) {
	    y[i] = rawllpts[i*2];
	    x[i] = rawllpts[i*2+1];
	}
	double[] xs = new double[nbarc*nbPts];
        double[] ys = new double[nbarc*nbPts];  
	fr.unicaen.openmap.layer.flowLayer.drawing.Spline.compute(nbPts, x, y, nbarc*nbPts, xs, ys);
	rawllpts = new double[2*nbarc*nbPts];
	for(int i = 0; i < nbarc*nbPts; i++) {
	    rawllpts[2*i] = ys[i];
	    rawllpts[2*i+1] = xs[i];
	}
	return new OMPoly(rawllpts,omp.getUnits(), omp.getUnits(), omp.getLineType());
    }

    public static OMDataLocation simplify(OMDataLocation omdl) {
	return simplify(omdl, distanceTolerance);
    }

    public static OMDataLocation simplify(OMDataLocation omdl, double distanceTolerance) {
	return new OMDataLocation(simplify(omdl.getOMGraphic(), distanceTolerance), omdl.getCentroid(), omdl.getBoundingBox());
    }

    public static OMGraphic simplify(OMGraphic omg) {
	return simplify(omg, distanceTolerance);
    }

    public static OMGraphic simplify(OMGraphic omg, double distanceTolerance) {
	Geometry geom =  JTSFactory.OMGraphicToJTSGeometry(omg);
	return JTSFactory.JTSGeometryToOMGraphic(
						 TopologyPreservingSimplifier.simplify(geom, distanceTolerance));
    }

    public static OMDataLocation union(OMDataLocation omdl1, OMDataLocation omdl2) {
	return JTSFactory.JTSGeometryToOMDataLocation(union(omdl1.getOMGraphic(), omdl2.getOMGraphic()));
    }

    public static Geometry union(OMGraphic omg1, OMGraphic omg2) {
	Geometry geom = JTSFactory.OMGraphicToJTSGeometry(omg1);
	return geom.union( JTSFactory.OMGraphicToJTSGeometry(omg2));
    }


    public static OMDataLocation intersection(OMDataLocation omdl1, OMDataLocation omdl2) {
	return JTSFactory.JTSGeometryToOMDataLocation(intersection(omdl1.getOMGraphic(), omdl2.getOMGraphic()));
    }

    public static Geometry intersection(OMGraphic omg1, OMGraphic omg2) {
	return intersection(JTSFactory.OMGraphicToJTSGeometry(omg1), JTSFactory.OMGraphicToJTSGeometry(omg2));
    }
    
    public static Geometry intersection(Geometry g1, Geometry g2) {
	if (g1 == null || g2 == null)
	    return null;

	// JTS intersection does not support collections
	if (g1 instanceof GeometryCollection)
	    return intersection((GeometryCollection)g1, g2);

	if (g2 instanceof GeometryCollection)
	    return intersection((GeometryCollection)g2, g1);

	return g1.intersection(g2);
    }

    public static Geometry intersection(GeometryCollection g1, Geometry g2) {
	Geometry res = null;
	for (int n = 0; n <  g1.getNumGeometries(); n++) {
	    Geometry g = intersection(g1.getGeometryN(n), g2);
	    if (res == null)
		res = g;
	    else
		res = res.union(g);
	}
	return res;
    }

    public static OMDataLocation union(List<OMDataLocation> l) {
	// The construction of one buffer is far more efficient than repeated unions

	OMGraphicList omgl = new OMGraphicList();
	for (OMDataLocation omdl : l) {
	    if (omdl == null)
		continue;
	    OMGraphic omg = omdl.getOMGraphic();
	    if (omg == null)
		continue;
	    omgl.add(omg);
	}
	Geometry res = JTSFactory.OMGraphicToJTSGeometry(omgl).buffer(1E-3);
	return JTSFactory.JTSGeometryToOMDataLocation(res);
    }

    public static OMDataLocation intersection(List<OMDataLocation> l) {
	Geometry inter = null;
	// we must flatten GeomeryCollection
	for (OMDataLocation omdl : l) {
	    if (omdl == null)
		continue;
	    OMGraphic omg = omdl.getOMGraphic();
	    if (omg == null)
		continue;
	    if (inter == null) {
		inter = JTSFactory.OMGraphicToJTSGeometry(omg);
		continue;
	    }
	    inter = intersection(inter, JTSFactory.OMGraphicToJTSGeometry(omg));
	}
	
	return JTSFactory.JTSGeometryToOMDataLocation(inter);
    }

    public static OMDataLocation convexHull(List<OMDataLocation> l) {
	OMGraphicList omgl = new OMGraphicList();
	for (OMDataLocation omdl : l) {
	    if (omdl == null)
		continue;
	    OMGraphic omg = omdl.getOMGraphic();
	    if (omg != null) {
		omgl.add(omg);
	    } else {
		Centroid c = omdl.getCentroid();
		if (c == null)
		    continue;
		// only take the centroid
		omgl.add(new OMPoint(c.getLat(), c.getLon()));
	    }
	}
	ConvexHull convexHull = 
	    new ConvexHull(JTSFactory.extractCoordinates(omgl), JTSFactory.defaultFactory);
	return JTSFactory.JTSGeometryToOMDataLocation(convexHull.getConvexHull());
    }

    public static OMDataLocation centroid(List<OMDataLocation> l) {
	return barycenter(l, null);
    }

    private static enum TypeGeo { ANY, POINT, POLY };
    public static OMDataLocation barycenter(List<OMDataLocation> l, List<Object> weightList) {
	boolean warning = false;
	TypeGeo typeGeo = TypeGeo.ANY;

	// used if typeGeo == TypeGeo.POLY because we need areas of polygons 
	// to compute their centroids
	List<CentroidAreaAndWeight> centroidAreaAndWeights = new ArrayList<CentroidAreaAndWeight>();

	// used if typeGeo == TypeGeo.POINT
	double latSum = 0, lonSum = 0;
	int nbPoint = 0;
	double totalWeight = 0;

	// weight parameter
	Iterator<Object> weightIter = null;
	if (weightList != null)
	    weightIter = weightList.iterator();
	for (OMDataLocation omdl : l) {
	    if (omdl == null)
		continue;
	    OMGraphic omg = omdl.getOMGraphic();
	    double weight = 1;
	    if (weightList != null) {
		if (weightIter.hasNext())
		    weight = Util.parseDouble(weightIter.next());
		else 
		    System.err.println("### Warning: Spatial function barycenter weightList too short");
	    }	
		
	    if (omg instanceof OMPoint) {
		if (typeGeo == TypeGeo.ANY)
		    typeGeo = TypeGeo.POINT;
		if (typeGeo != TypeGeo.POINT) {
		    warning = true;
		    continue;
		}
		nbPoint++;
		totalWeight += weight;
		latSum += weight * ((OMPoint)omg).getLat();
		lonSum += weight * ((OMPoint)omg).getLon();
	    } else if (omg instanceof OMPoly) {
		if (typeGeo == TypeGeo.ANY)
		    typeGeo = TypeGeo.POLY;
		if (typeGeo != TypeGeo.POLY) {
		    warning = true;
		    continue;
		}
		nbPoint++;
		double llPoints[] = ((OMPoly)omg).getRawllpts();
		double a = area(llPoints);
		Centroid c = centroid(llPoints, OMGraphic.RADIANS, a);
		centroidAreaAndWeights.add(new CentroidAreaAndWeight(c, a, weight));
	    } else {
		// only take the centroid
		if (typeGeo == TypeGeo.ANY)
		    typeGeo = TypeGeo.POINT;
		if (typeGeo != TypeGeo.POINT) {
		    warning = true;
		    continue;
		}
		Centroid c = omdl.getCentroid();
		if (c != null) {
		    nbPoint++;
		    totalWeight += weight;
		    latSum += weight * c.getLat();
		    lonSum += weight * c.getLon();
		}
	    }
	} // end for (OMDataLocation omdl : l) 
	if (warning) {
	    System.err.println("*** Centroid Warning: Only the centroid is used for geometry that are not polygons");
	    //	    JOptionPane.showMessageDialog(null, 
	    //					  "Only the centroid is used for geometry that are not polygons", 
	    //					  "Warning", JOptionPane.WARNING_MESSAGE);
	}
	if (nbPoint == 0)
	    return null;
	if (typeGeo == TypeGeo.POINT) {
	    double lon = lonSum/totalWeight;
	    double lat = latSum/totalWeight;
	    Centroid c = new Centroid(lon, lat);
	    Debug.message("DataStorage", "[expr] new centroid : " + c);
	    return new OMDataLocation(new OMPoint(lat, lon), 
				      c,
				      new BoundingBox(lon, lat, lon, lat));
	} else if (typeGeo == TypeGeo.POLY) {
	    if (centroidAreaAndWeights.size() == 0)
		return null;
	    totalWeight = 0;
	    for(CentroidAreaAndWeight caw: centroidAreaAndWeights) {
		double w = caw.weight*caw.area;
		totalWeight += w;
		latSum += w * caw.centroid.getLat();
		lonSum += w * caw.centroid.getLon();
	    }
	    double lon = lonSum / totalWeight;
	    double lat = latSum / totalWeight;
	    Centroid c = new Centroid(lon, lat);
	    Debug.message("DataStorage", "[expr] new centroid : " + c);
	    return new OMDataLocation(new OMPoint(lat, lon), 
				      c,
				      new BoundingBox(lon, lat, lon, lat));
	}	
	return null;
    }

    
    public static double areaInSQKM(OMGraphic omg) {
	return Length.KM.fromRadians(Length.KM.fromRadians(area(omg)));
    }

    public static double lengthInKM(OMGraphic omg) {
	return Length.KM.fromRadians(length(omg));
    }

    static public double[] getRawllpts(OMRect omr) {
	// decimal degrees !!!!
	double[] llpts = { 
	    omr.getNorthLat(),
	    omr.getEastLon(),
	    omr.getNorthLat(),
	    omr.getWestLon(),
	    omr.getSouthLat(),
	    omr.getWestLon(), 
	    omr.getSouthLat(),
	    omr.getEastLon(),
	    // close the LinearRing
	    omr.getNorthLat(),
	    omr.getEastLon(),
	};
	return llpts;
    }

    public static double area(OMGraphic omg) {
	if (omg instanceof OMPoly)
	    return area(((OMPoly)omg).getRawllpts());
	if (omg instanceof OMRect)
	    return area(getRawllpts((OMRect)omg));
	if (omg instanceof OMGraphicList) {
	    double a = 0;
	    for(OMGraphic om: (OMGraphicList)omg)
		a += area(om);
	    return a;
	}
	// System.err.println("*** Area of "+omg+" -> 0");
	return 0;
    }

    public static double area(double llPoints[]) {
	// A = \frac{1}{2} \sum_{i = 0}^{n - 1}( x_i y_{i + 1} - x_{i + 1} y_i)
	double a = 0;
	for(int i = 0; i <= llPoints.length/2-2; i++) 
	    a += llPoints[i*2]*llPoints[(i+1)*2+1] - llPoints[(i+1)*2]*llPoints[i*2+1]; 
	return Math.abs(a/2);
    }


   public static double jst_area(OMGraphic omg) {
       return (JTSFactory.OMGraphicToJTSGeometry(omg)).getArea();
   }


    public static double length(OMGraphic omg) {
	if (omg instanceof OMPoly)
	    return length(((OMPoly)omg).getRawllpts());
	if (omg instanceof OMRect)
	    return length(getRawllpts((OMRect)omg));
	if (omg instanceof OMLine)
	    return length(((OMLine)omg).getLL()); //	lat1, lon1, lat2, lon2.
	if (omg instanceof OMGraphicList) {
	    double p = 0;
	    for(OMGraphic om: (OMGraphicList)omg)
		p += length(om);
	    return p;
	}
	System.err.println("*** Length of "+omg+" -> 0");

	return 0;
    }

    public static double length(double llPoints[]) {
	// P = \frac{1}{2} \sum_{i = 0}^{n - 1}( length(x_i, y_i, x_{i+1} y_{i + 1})
	double p = 0;
	for(int i = 0; i <= llPoints.length/2-2; i++) 
	    p += length(llPoints[i*2], llPoints[i*2+1], llPoints[(i+1)*2], llPoints[(i+1)*2+1]);
	return p;
    }

    public static double length(double x1, double y1, double x2, double y2) {
	return Math.sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
    }

   public static double jst_length(OMGraphic omg) {
       return (JTSFactory.OMGraphicToJTSGeometry(omg)).getLength();
   }

    /** 
     * Compute a centroid of the OMGraphic
     * In case of a collection, keep the centroid of the largest area element
     */
    static public Centroid centroid(OMGraphic omg) {
	if (omg instanceof OMPoint)
		return new Centroid(((OMPoint)omg).getLon(), ((OMPoint)omg).getLat());
	if (omg instanceof OMRect)
	    return SpatialFunction.centroid((OMRect)omg); 
	if (omg instanceof OMPoly)
	    return centroid((OMPoly)omg);	
	if (omg instanceof OMLine) 
	    return centroid((OMLine)omg);
	
	if (omg instanceof OMGraphicList) {
	    // take the centroid of the wider member (area or perimeter for a line)
	    double maxsize = -1;
	    Centroid centroid = null;		   
	    for (OMGraphic g : (OMGraphicList)omg) {
		double s;
		if (g instanceof OMLine) 
		    s = length(g);
		else
		    s = area(g);
		if (s > maxsize) {
		    maxsize = s;
		    centroid = centroid(g);		    
		 }
	    }
	    if (centroid != null)
		return centroid;
	}
	return null;
    }

    /** 
     * Compute a centroid of the OMLine
     */
    public static Centroid centroid(OMLine oml) {
	double ll[] = oml.getLL(); //	lat1, lon1, lat2, lon2.
	return new Centroid((ll[1] + ll[3]) / 2, (ll[0] + ll[2]) / 2);
    }

    /** 
     * Compute a centroid of the OMPoly
     */
    public static Centroid centroid(OMPoly omp) {
	return centroid(omp.getRawllpts(), OMGraphic.RADIANS, null);
    }

    /** 
     * Compute a centroid of the OMRect
     */
    public static Centroid centroid(OMRect omr) {
	return new Centroid((omr.getEastLon() + omr.getWestLon()) / 2, 
		     (omr.getNorthLat() + omr.getSouthLat())/ 2);
    }

    /** 
     * Compute a centroid of the polygone given by llPoints in lat/lon order.
     * @parameter  lType is either OMGraphic.RADIANS or OMGraphic.DECIMAL_DEGREES
     */
    public static Centroid centroid(double llPoints[], int lType) {
	return centroid(llPoints, lType, null);
    }

    /** 
     * Compute a centroid of the polygone given by llPoints in lat/lon order.
     * @parameter  lType is either OMGraphic.RADIANS or OMGraphic.DECIMAL_DEGREES
     * @parameter _area can be given if known
     */
    public static Centroid centroid(double llPoints[], int lType, Double _area) {
	// x_G=\frac{1}{6 A} \sum_{i = 0}^{n - 1} (x_i + x_{i + 1}) (x_i y_{i + 1} - x_{i + 1} y_i)
	// y_G=\frac{1}{6 A} \sum_{i = 0}^{n - 1} (y_i + y_{i + 1}) (x_i y_{i + 1} - x_{i + 1} y_i)
	double lat = 0;
	double lon = 0;
	for(int i = 0; i <= llPoints.length/2-2; i++) {
	    lat +=  (llPoints[i*2] + llPoints[(i+1)*2]) * (llPoints[i*2]*llPoints[(i+1)*2+1] - llPoints[(i+1)*2]*llPoints[i*2+1]);
	    lon += (llPoints[i*2+1] + llPoints[(i+1)*2+1]) * (llPoints[i*2]*llPoints[(i+1)*2+1] - llPoints[(i+1)*2]*llPoints[i*2+1]);
	} 
	if (_area == null)
	    _area = area(llPoints);
	double a = _area*6;
	lat /= a;
	lon /= a;
	if (lType == OMGraphic.RADIANS) {
	    // Centroid is always in decimal degrees
	    lat = Geo.degrees(lat);
	    lon = Geo.degrees(lon);
	}
	return new Centroid(lon, lat);
    }

    /** angle of the vector in the  lat/lon plane, not the sphere */
    private static double angle(OMLine oml) {
	double coords[] = oml.getLL();
	double lat = coords[2]-coords[0];
	double lon = coords[3]-coords[1];
	// if ( oml.getAttribute(TAG) != FORWARDTAG) {
	//     lat = - lat;
	//     lon = - lon;
	// }
	return Math.atan2(lat, lon);
    }

    /** a1 and a2 are in the interval  [-PI, +PI] 
     *Returns a1 - a2 and normalise the result in order to be in the interval ]0, +2PI]
     */
    private static double subractAngle(double a1, double a2) {
	double angle = a1 - a2; // angle is in [-2 PI, 2PI]
	while ( angle < 1E-10 ) angle += 2*Math.PI;
	// now angle is in ]0, 2PI]
	return angle;
    }

    /** st_translate */
    public static OMDataLocation translate(OMDataLocation omdl, double lon, double lat) {
	return JTSFactory.JTSGeometryToOMDataLocation(translate(omdl.getOMGraphic(), lon, lat));
    }

    public static Geometry translate(OMGraphic omg, double lon, double lat) {
	Geometry geom = JTSFactory.OMGraphicToJTSGeometry(omg);
	return AffineTransformation.translationInstance(lon, lat).transform(geom);
    }

}

class CentroidAreaAndWeight {
    public Centroid centroid;
    public double area;
    public double weight;
    public CentroidAreaAndWeight(Centroid c, double a, double w) {
	centroid = c;
	area = a;
	weight = w;
    }
}
