// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.expr.gui;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;

import fr.unicaen.openmap.dataStorage.expr.AddColumnDataStorage;
import fr.unicaen.openmap.dataStorage.expr.Function;
import fr.unicaen.openmap.dataStorage.expr.FunctionDescription;
import fr.unicaen.openmap.dataStorage.expr.FunctionDescriptionList;
import fr.unicaen.openmap.exception.UnavailableColumnNameException;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import fr.unicaen.openmap.gui.RefreshableJList;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class AddColumnPanel extends JPanel {

    protected I18n i18n = Environment.getI18n();
    protected RefreshableJList fieldsList;
    protected RefreshableJList geoList;
    protected ExpressionPanel exprPanel;
    protected JTextField fieldColumnName;
    protected AddColumnDataStorage ds;
    public AddColumnPanel(AddColumnDataStorage _ds) {
	this.ds = _ds;
        setLayout(new BorderLayout());
        ListSelectionListener fieldsListListSelectionListener = new ListSelectionListener() {
		public void valueChanged(ListSelectionEvent lse) {
		    if (lse.getValueIsAdjusting()) {
			exprPanel.appendText(fieldsList.getSelectedValue().toString(), true);
			try {
			    fieldsList.setSelectedIndices(new int[]{});
			} catch (Exception e) {
			}
		    }
		}
	    };
        fieldsList = new RefreshableJList(fieldsListListSelectionListener, ds.getDataStorage().getDefaultDataTableModel().getDataColumnNames());
	ListSelectionListener geoListListSelectionListener = new ListSelectionListener() {
 		public void valueChanged(ListSelectionEvent lse) {
		    if (lse.getValueIsAdjusting()) {
			exprPanel.appendText(geoList.getSelectedValue().toString(), true);
			try {
			    geoList.setSelectedIndices(new int[]{});
			} catch (Exception e) {
			}
		    }
		}
	    }; 
	geoList = new RefreshableJList(geoListListSelectionListener, ds.getDataStorage().getDefaultDataTableModel().getOMColumnNames());
        JScrollPane sc = new JScrollPane(fieldsList);
        sc.setPreferredSize(new Dimension(200, 150));
        sc.setMaximumSize(new Dimension(200, 150));
        JScrollPane scGeo = new JScrollPane(geoList);
        scGeo.setPreferredSize(new Dimension(200, 150));
        scGeo.setMaximumSize(new Dimension(200, 150));


        /*JPanel panButtons = new JPanel(new GridLayout(8, 3));*/
        JTabbedPane panButtons = new JTabbedPane();

        for (FunctionDescriptionList l : Function.getDescriptions()) {
            JPanel panCat = new JPanel();
            ComponentSizer.fixComponentSize(panCat, new Dimension(380, 130));
            panButtons.addTab(l.getCategory(), panCat);
            for (final FunctionDescription f : l) {
                JButton b = new ExprButton(f.getName());
                b.addActionListener(new ActionListener() {

                    public void actionPerformed(ActionEvent arg0) {
                        exprPanel.appendText(" " + f.getName() + f.getPattern() + " ");
                    }
                });
                panCat.add(b);
                ComponentSizer.fixComponentSize(b, new Dimension(90, 20));
                b.setToolTipText(f.getDoc());
            }
        }

        JPanel panCenter = new JPanel();
        JPanel wrapFieldsList = new JPanel(new BorderLayout());
        wrapFieldsList.add(new JLabel(i18n.get(this, "dataColumns", "Data columns")), BorderLayout.NORTH);
        wrapFieldsList.add(sc, BorderLayout.CENTER);
        panCenter.add(wrapFieldsList);
        JPanel wrapGeoList = new JPanel(new BorderLayout());
        wrapGeoList.add(new JLabel(i18n.get(this, "geometryColumns", "Geometry columns")), BorderLayout.NORTH);
        wrapGeoList.add(scGeo, BorderLayout.CENTER);
        panCenter.add(wrapGeoList);
        panCenter.add(panButtons);
        Box panNewCol = Box.createVerticalBox();
        panNewCol.add(new JLabel(i18n.get(this, "nameOfTheNewColumn", "New Column")));
        fieldColumnName = new JTextField(10);
        panNewCol.add(fieldColumnName);
        fieldColumnName.setText(ds.getAddedColumnName());
	JButton showDocs = new JButton(i18n.get(this, "showDoc", "Show doc"));
	panNewCol.add(showDocs);
	showDocs.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
		javax.swing.JEditorPane docText = new javax.swing.JEditorPane("text/html", Function.getFunctionsDocs());
		docText.setEditable(false);
		javax.swing.JScrollPane docPanel = new javax.swing.JScrollPane(docText);
		fr.unicaen.openmap.util.gui.ComponentSizer.fixComponentSize(docPanel, new Dimension(800, 600));
		JOptionPane.showMessageDialog(AddColumnPanel.this,
					      docPanel,
					      i18n.get(this,"functionDoc","Functions Documentation"),
					      JOptionPane.INFORMATION_MESSAGE);
            }
        });

        add(panCenter, BorderLayout.CENTER);
        fieldColumnName.addKeyListener(new KeyListener() {

            public void keyTyped(KeyEvent arg0) {
            }

            public void keyPressed(KeyEvent arg0) {
            }

            public void keyReleased(KeyEvent arg0) {
                if (ds.isAnExistingColumnName(fieldColumnName.getText())) {
                    JOptionPane.showMessageDialog(null,
                            i18n.get(this, "alreadyUsedColumnName", "Column name already used"),
                            fieldColumnName.getText(), JOptionPane.ERROR_MESSAGE);
                } else {
		    int caretPos = fieldColumnName.getCaretPosition();
                    fieldColumnName.setText(ds.setAddedColumnName(fieldColumnName.getText()));
		    fieldColumnName.setCaretPosition(caretPos); // reset the cursor pos
                }
            }
        });

        exprPanel = new ExpressionPanel(ds, 3);
        exprPanel.add(panNewCol, 0);
        add(exprPanel, BorderLayout.SOUTH);
        setBorder(BorderFactory.createTitledBorder(i18n.get(this, "title", "Expression DS")));
    }

    /**
     * Use at init when expr is correct to set the exec button enabled and to light the green flag 
     */ 
    public void setExpressionPanelState(boolean b) {
	exprPanel.setState(b);
    } 

    public void refresh() {
        fieldsList.init(ds.getDataStorage().getDefaultDataTableModel().getDataColumnNames());
	geoList.init(ds.getDataStorage().getDefaultDataTableModel().getOMColumnNames());
	exprPanel.updateExpression();
    }
}
