// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.gml.in;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import fr.unicaen.openmap.dataStorage.DataObject;
import fr.unicaen.openmap.dataStorage.OMDataLocation;
import fr.unicaen.openmap.util.Util;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMLine;
import com.bbn.openmap.omGraphics.OMPoint;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.omGraphics.OMRect;
import com.bbn.openmap.proj.coords.LatLonPoint;
import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.dataStorage.BoundingBox;
import fr.unicaen.openmap.dataStorage.Centroid;

public class GMLParser extends DefaultHandler {

    private ArrayList<DataObject> dataObjects;
    private ArrayList<String> columnNames;
    private ArrayList<String> omGraphicNames;
    private StringBuffer tempStringBuffer;
    private OMDataLocation tempOMDataLocation;
    private DataObject tempDataObject;
    private String tempColumnName;
    private OMGraphicList tempOmgList;
    private pointType currentPoint;
    private LatLonPoint upperLeft, lowerRight;
    private int countShape = 0;
    private String currentTag = "NO_TAG";
    public static final int POLY_TYPE = 0;
    public static final int RECT_TYPE = 1;
    public static final int LINE_TYPE = 2;
    private int currentGmlType;

    public enum pointType {

        UPPER_LEFT, LOWER_RIGHT, CENTROID, GEO_POINT
    }

    public GMLParser(String fileName) {
	this(Util.open(fileName, true));
    }

    public GMLParser(InputStream in) {
        tempStringBuffer = new StringBuffer("");
        dataObjects = new ArrayList<DataObject>();
        columnNames = new ArrayList<String>();
        omGraphicNames = new ArrayList<String>();
        parse(in);

    }

    public ArrayList<DataObject> getDataObjects() {
        return dataObjects;
    }

    public ArrayList<String> getColumnNames() {
        return columnNames;
    }

    public ArrayList<String> getOMGraphicsNames() {
        return omGraphicNames;
    }

    public void parse(InputStream file) {

        SAXParserFactory factory = SAXParserFactory.newInstance();
        factory.setValidating(false);
        try {
            SAXParser saxParser = factory.newSAXParser();
            saxParser.parse(file, this);
        } catch (SAXParseException spe) {
            Debug.error("[GMLDataStorage] Parse Error, line " + spe.getLineNumber() + ": " + spe.getMessage());
        } catch (SAXException sxe) {
            Debug.error("[GMLDataStorage] " + sxe.getMessage());
        } catch (ParserConfigurationException pce) {
            Debug.error("[GMLDataStorage] " + pce.getMessage());
        } catch (IOException ioe) {
            Debug.error("[GMLDataStorage] " + ioe.getMessage());
        }
    }

    // private void debugElement(String qName, Attributes attrs) {
    //     System.err.println("Element: " + qName);
    //     for (int i = 0; i < attrs.getLength(); i++) {
    //         String qn = attrs.getQName(i);
    //         String type = attrs.getType(i);
    //         String value = attrs.getValue(i);
    //         System.err.println("qn: " + qn);
    //         System.err.println("type: " + type);
    //         System.err.println("value: " + value);
    //         System.err.println("");
    //     }
    // }

    @Override
    public void startElement(String uri, String localName, String qName, Attributes attrs) throws SAXException {
        currentTag = qName;

        if (qName.equals(GMLDtd.DS_FEATURE)) {
            tempDataObject = new DataObject();
            countShape = 0;
            return;
        }
        if (qName.equals(GMLDtd.GEOMETRY_PROPERTY) || qName.equals(GMLDtd.GEOMETRY_PROPERTY_OMPG)) {
            tempOMDataLocation = new OMDataLocation();
            countShape++;
            return;
        }

        if (qName.equals(GMLDtd.MUTLI_POLYGON)) {
            tempOmgList = new OMGraphicList();
            currentGmlType = POLY_TYPE;
            if (attrs.getValue("name") != null) {
                String col = attrs.getValue("name");
                if (!omGraphicNames.contains(col)) {
                    omGraphicNames.add(col);
                }
                tempDataObject.addOMDataLocation(col, tempOMDataLocation);
            }
            return;
        }

        if (qName.equals(GMLDtd.RECT)) {
            // no list used in this case, so no // tempOmgList = new OMGraphicList();
            currentGmlType = RECT_TYPE;
            if (attrs.getValue("name") != null) {
                String col = attrs.getValue("name");
                if (!omGraphicNames.contains(col)) {
                    omGraphicNames.add(col);
                }
                tempDataObject.addOMDataLocation(col, tempOMDataLocation);
            }
            return;
        }

        if (qName.equals(GMLDtd.LINE_STRING)) {
	    // no list used in this case, so no // tempOmgList = new OMGraphicList();
            currentGmlType = LINE_TYPE;
	    if (attrs.getValue("name") != null) {
                String col = attrs.getValue("name");
                if (!omGraphicNames.contains(col)) {
                    omGraphicNames.add(col);
                }
                tempDataObject.addOMDataLocation(col, tempOMDataLocation);
            }
            return;
        }

        if (qName.equals(GMLDtd.POINT)) {
            String col = attrs.getValue("name");
            //System.out.println("start elt POINT " + attrs.getValue("name"));
            if (col != null) {
                if (col.equals(GMLDtd.CENTROID)) {
                    currentPoint = pointType.CENTROID;
                    return;
                }
                if (col.equals(GMLDtd.UPPER_LEFT)) {
                    currentPoint = pointType.UPPER_LEFT;
                    return;
                }
                if (col.equals(GMLDtd.LOWER_RIGHT)) {
                    currentPoint = pointType.LOWER_RIGHT;
                    return;
                } else {
                    if (!omGraphicNames.contains(col)) {
                        omGraphicNames.add(col);
                    }
                    tempDataObject.addOMDataLocation(col, tempOMDataLocation);
                    currentPoint = pointType.GEO_POINT;
                }
            } else {
                System.err.println("Dropping point whithout name");
            }
        }

    }

    public void endElement(String uri, String localName, String qName) throws SAXException {
        try {
            if (qName.equals(GMLDtd.DATA_NAME) || qName.equals(GMLDtd.DATA_NAME_OMPG)) {
                if (!columnNames.contains(tempStringBuffer.toString().trim())) {
                    columnNames.add(tempStringBuffer.toString().trim());
                }
                tempColumnName = tempStringBuffer.toString().trim();
                tempStringBuffer = new StringBuffer("");
                return;
            }
            if (qName.equals(GMLDtd.COORDINATES)) {
                if (tempOMDataLocation != null) {
                    OMPoly om = new OMPoly(
                            getPointsFromString(tempStringBuffer.toString()),
                            OMPoly.DECIMAL_DEGREES,
                            OMPoly.LINETYPE_STRAIGHT);

                    om.setIsPolygon(true);
                    om.setRenderType(OMGraphic.RENDERTYPE_LATLON);
                    om.putAttribute(Util.OMGraphicIndex, dataObjects.size());
                    tempOmgList.add(om);
                }

                tempStringBuffer = new StringBuffer("");
                return;
            }

            if (qName.equals(GMLDtd.DATA_VALUE) || qName.equals(GMLDtd.DATA_VALUE_OMPG)) {
                if (tempDataObject != null && tempColumnName != null) {
                    tempDataObject.addData(tempColumnName, tempStringBuffer.toString().trim());
                }
                tempStringBuffer = new StringBuffer("");
                return;
            }
            if (qName.equals(GMLDtd.FEATURE_MEMBER)) {
                tempOMDataLocation = null;
                tempDataObject = null;
                tempStringBuffer = new StringBuffer("");
                return;

            }

            if (qName.equals(GMLDtd.DS_FEATURE)) {
                dataObjects.add(tempDataObject);
                tempDataObject = null;
                tempStringBuffer = new StringBuffer("");
                return;
            }


            if (qName.equals(GMLDtd.MUTLI_POLYGON)) {
                tempOMDataLocation.setOMGraphic(tempOmgList);
                try {
                    tempOMDataLocation.setBoundingBox(new BoundingBox(upperLeft, lowerRight));
                } catch (NullPointerException exc) {
                }
                tempStringBuffer = new StringBuffer("");
                return;
            }

            if (qName.equals(GMLDtd.POS)) {
                double point[] = getPointFromString(tempStringBuffer.toString().trim());

                switch (currentPoint) {
                    case CENTROID:
                        tempOMDataLocation.setCentroid(new Centroid(point[1], point[0]));
                        break;
                    case UPPER_LEFT:
                        upperLeft = new LatLonPoint.Double(point[0], point[1]);
                        break;
                    case LOWER_RIGHT:
                        lowerRight = new LatLonPoint.Double(point[0], point[1]);
                        break;
                    case GEO_POINT:
                        tempOMDataLocation.setOMGraphic(new OMPoint(point[0], point[1]));
                        tempOMDataLocation.setCentroid(new Centroid(point[1], point[0]));
                        return;


                }
                tempStringBuffer = new StringBuffer("");
                return;
            }

            if (qName.equals(GMLDtd.POS_LIST)) {
                switch (currentGmlType) {
                    case LINE_TYPE:
                        double point[] = getCoordsFromString(tempStringBuffer.toString().trim());
                        tempOMDataLocation.setOMGraphic(new OMLine(
								   point[0],
								   point[1],
								   point[2],
								   point[3],
								   OMLine.LINETYPE_STRAIGHT));
                        break;
                    case RECT_TYPE:
                        double pointRect[] = getRectCoordsFromString(tempStringBuffer.toString().trim());
                         tempOMDataLocation.setOMGraphic(new OMRect(
								    pointRect[0],
								    pointRect[1],
								    pointRect[2],
								    pointRect[3],
								    OMRect.LINETYPE_STRAIGHT));
			 break;
                }
                return;
            }

            tempStringBuffer = new StringBuffer("");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * This method will only concatenete buffers for
     */
    public void characters(char buf[], int offset, int len) throws SAXException {
        tempStringBuffer.append(buf, offset, len);
    }

    /**
     * Error handling is reported above.
     */
    public void error(SAXParseException e) throws SAXParseException {
        throw e;
    }

    /**
     * Warning is handled to stderr.
     */
    public void warning(SAXParseException err) throws SAXParseException {
        System.err.println("[GmlParser] Warning, line " + err.getLineNumber() + ", " + err.getSystemId());
        System.err.println("[GmlParser] " + err.getMessage());
    }

    private double[] getPointsFromString(String coords) {
        try {
            ArrayList<Double> pointsTmp = new ArrayList<Double>();
            String[] tab = coords.split(" ");
            for (int i = 0; i < tab.length; i++) {
                String[] latlon = tab[i].split(",");
                if (latlon.length == 2) {
                    pointsTmp.add(Double.valueOf(latlon[1]));
                    pointsTmp.add(Double.valueOf(latlon[0]));
                }

            }
            double[] points = new double[pointsTmp.size()];
            for (int j = 0; j < pointsTmp.size(); j++) {
                points[j] = pointsTmp.get(j).doubleValue();
            }
            return points;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    private double[] getCoordsFromString(String coords) {
        String[] stValues = coords.split(" ");
        double[] coordinates = new double[4];
        for (int i = 0; i < coordinates.length; i++) {
            coordinates[i] = Double.valueOf(stValues[i]);
        }
        return coordinates;
    }

    private double[] getPointFromString(String pointString) {
        double point[] = new double[2];
        String[] coords = pointString.split(" ");
        point[0] = Double.valueOf(coords[0]);
        point[1] = Double.valueOf(coords[1]);
        return point;
    }

    private double[] getRectCoordsFromString(String coords) {
        String[] stValues = coords.split(" ");
        double[] coordinates = new double[4];
        coordinates[0] = Double.valueOf(stValues[0]);
        coordinates[1] = Double.valueOf(stValues[1]);
        coordinates[2] = Double.valueOf(stValues[4]);
        coordinates[3] = Double.valueOf(stValues[5]);
        return coordinates;
    }
}
