// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.gml.out;

import com.bbn.openmap.omGraphics.OMCircle;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Iterator;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMLine;
import com.bbn.openmap.omGraphics.OMPoint;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.omGraphics.OMRect;
import com.vividsolutions.jts.io.gml2.GMLReader;

import fr.unicaen.openmap.dataStorage.Centroid;
import fr.unicaen.openmap.dataStorage.OMDataLocation;
import fr.unicaen.openmap.dataStorage.gml.GMLDataStorage;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.zip.GZIPOutputStream;

public class GMLWriter {

    private XMLStreamWriter writer;
    private FileWriter fWriter;
    private OutputStream outputStream;
    private String fileName;
    public final static String SPACE = " ";
    private String namespaceOMPG, namespaceGML;
    private boolean zip;

    public GMLWriter(String fileName, boolean zip) {
        this.fileName = fileName;
        this.zip = zip;
        namespaceOMPG = "http://themamap.info.unicaen.fr";
        namespaceGML = "http://www.opengis.net/gml";
    }

    public void startWrite(String dsType) {
        try {
            XMLOutputFactory output = XMLOutputFactory.newInstance();
            outputStream = null;
            fWriter = null;
            if (zip) {
                outputStream = new GZIPOutputStream(new FileOutputStream(fileName));
                writer = output.createXMLStreamWriter(outputStream);
            } else {
                fWriter = new FileWriter(fileName);
                writer = output.createXMLStreamWriter(fWriter);
            }
            writer.writeStartDocument();
            writer.writeCharacters("\n");
            writer.setPrefix("themamap", namespaceOMPG);
            writer.setPrefix("gml", namespaceGML);
            writer.setDefaultNamespace(namespaceOMPG);
            writer.writeStartElement(namespaceOMPG, "DataStorage");
            writer.writeNamespace("themamap", namespaceOMPG);
            writer.writeNamespace("gml", namespaceGML);
            writer.writeAttribute("type", dsType);
            writer.writeCharacters("\n");
            writer.writeStartElement(namespaceGML, "featureMembers");
            writer.writeCharacters("\n");
        } catch (XMLStreamException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void startOMPGFeature() {
        try {
            writer.writeCharacters("\n");
            writer.writeStartElement(namespaceGML, "DataStorageFeature");
            writer.writeCharacters("\n");
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }
    }

    public void endOMPGFeature() {
        try {
            writer.writeCharacters("\n");
            writer.writeEndElement();//fin featureMember
            writer.writeCharacters("\n");
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }
    }

    public void startGeometryProperty() {
        try {
            writer.writeStartElement(namespaceOMPG, "themamapGeometryProperty");
            writer.writeCharacters("\n");

        } catch (XMLStreamException e) {
            e.printStackTrace();
        }

    }

    public void endGeometryProperty() {
        try {
            writer.writeEndElement();//fin multipoly
            writer.writeCharacters("\n");
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }
    }

    public void startMultiPolygon(String name) {
        try {
            writer.writeStartElement(namespaceGML, "MultiPolygon");
            writer.writeAttribute("name", name);
            writer.writeCharacters("\n");
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }
    }

    public void endMultiPolygon() {
        try {
            writer.writeEndElement();//fin Multipoly
            writer.writeCharacters("\n");
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }
    }

    public void startPoint(String name) {
        try {
            writer.writeStartElement(namespaceGML, "Point");
            writer.writeAttribute("name", name);
            writer.writeCharacters("\n");
            writer.writeStartElement(namespaceGML, "pos");
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }
    }

    public void endPoint() {
        try {
            writer.writeEndElement();
            writer.writeCharacters("\n");
            writer.writeEndElement();
            writer.writeCharacters("\n");
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }
    }

    public void startLine(String name) {
        try {
            writer.writeStartElement(namespaceGML, "LineString");
            writer.writeAttribute("name", name);
            writer.writeCharacters("\n");
            writer.writeStartElement(namespaceGML, "posList");
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }
    }

    public void endLine() {
        endPoint();
    }

    public void startRect(String name) {
        try {
            writer.writeStartElement(namespaceGML, "Rectangle");
            writer.writeAttribute("name", name);
            writer.writeCharacters("\n");
            writer.writeStartElement(namespaceGML, "exterior");
            writer.writeCharacters("\n");
            writer.writeStartElement(namespaceGML, "posList");
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }

    }

    public void endRect() {
        try {
            writer.writeEndElement();
            writer.writeCharacters("\n");
            writer.writeEndElement();
            writer.writeCharacters("\n");
            writer.writeCharacters("\n");
            writer.writeEndElement();
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }

    }

    public void startPolygonMember() {
        try {
            writer.writeStartElement(namespaceGML, "polygonMember");
            writer.writeCharacters("\n");
            writer.writeStartElement(namespaceGML, "Polygon");
            writer.writeCharacters("\n");
            writer.writeStartElement(namespaceGML, "outerBoundaryIs");
            writer.writeCharacters("\n");
            writer.writeStartElement(namespaceGML, "LinearRing");
            writer.writeCharacters("\n");
            writer.writeStartElement(namespaceGML, "coordinates");
            writer.writeCharacters("\n");
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }
    }

    public void endPolygonMember() {
        try {
            writer.writeEndElement();//fin coordinates
            writer.writeCharacters("\n");
            writer.writeEndElement();//finlinearring
            writer.writeCharacters("\n");
            writer.writeEndElement();//fin outerbound
            writer.writeCharacters("\n");
            writer.writeEndElement();//fin polygon
            writer.writeCharacters("\n");
            writer.writeEndElement();//fin polygonMember
            writer.writeCharacters("\n");
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }
    }

    public void addData(String dataName, String dataValue, String type) {
        try {
            writer.writeStartElement(namespaceOMPG, "data");
            writer.writeCharacters("\n");
            writer.writeStartElement(namespaceOMPG, "dataName");
            writer.writeCharacters(dataName);
            writer.writeEndElement();//fin dataName
            writer.writeStartElement(namespaceOMPG, "dataValue");
            writer.writeAttribute("type", type);
            writer.writeCharacters(dataValue);
            writer.writeEndElement();//fin dataValue
            writer.writeEndElement();//fin data
            writer.writeCharacters("\n");
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }
    }

    public void closeDocument() {
        try {
            writer.writeCharacters("\n");
            writer.writeEndElement();//fin featureMembers
            writer.writeCharacters("\n");
            //writer.writeEndElement();//fin dataStorage
            writer.writeCharacters("\n");
            writer.writeEndDocument();
            if (zip) {
                outputStream.flush();
                outputStream.close();
            } else {
                fWriter.flush();
                fWriter.close();
            }
            writer.flush();
            writer.close();
        } catch (XMLStreamException e) {
            //e.printStackTrace();
        } catch (IOException e) {
            //e.printStackTrace();
        }

    }

    public void writeGMLElement(String name, OMGraphic omg) {
        try {
            if (omg instanceof OMGraphicList) {
                startMultiPolygon(name);
                writeGMLElementsFromList((OMGraphicList) omg);
                endMultiPolygon();
                return;
            }
            if (omg instanceof OMPoly) {
                GMLPoly gmlp = new GMLPoly((OMPoly) omg);
                startPolygonMember();
                writer.writeCharacters(gmlp.toString());
                writer.writeCharacters("\n");
                endPolygonMember();
                writer.writeCharacters("\n");
                return;
            }
            if (omg instanceof OMPoint) {
                GMLPoint gmlp = new GMLPoint((OMPoint) omg);
                startPoint(name);
                writer.writeCharacters(gmlp.toString());
                writer.writeCharacters("\n");
                endPoint();
                writer.writeCharacters("\n");
                return;
            }
            if (omg instanceof OMLine) {
                GMLLine gmll = new GMLLine((OMLine) omg);
                startLine(name);
                writer.writeCharacters(gmll.toString());
                writer.writeCharacters("\n");
                endLine();
                writer.writeCharacters("\n");
                return;
            }

            if (omg instanceof OMRect) {
                GMLRect rect = new GMLRect((OMRect) omg);
                startRect(name);
                writer.writeCharacters(rect.toString());
                writer.writeCharacters("\n");
                endRect();
                writer.writeCharacters("\n");

            }


        } catch (XMLStreamException e) {
            e.printStackTrace();
        }
    }

    public void writeCentroidAndBoundingBox(OMDataLocation omdl) {
        try {

            writeCentroid("centroid", omdl);
            //bbox
            if (omdl.getBoundingBox() == null) {
                return;
            }
            writer.writeStartElement(namespaceOMPG, "boundingbox");
            writer.writeStartElement(namespaceGML, "Point");
            writer.writeAttribute("name", "upperleft");
            writer.writeStartElement(namespaceGML, "pos");
            writer.writeCharacters(omdl.getBoundingBox().getUpperLeftLat() + " " + omdl.getBoundingBox().getUpperLeftLon());
            writer.writeEndElement();
            writer.writeEndElement();
            writer.writeCharacters("\n");
            writer.writeStartElement(namespaceGML, "Point");
            writer.writeAttribute("name", "lowerright");
            writer.writeStartElement(namespaceGML, "pos");
            writer.writeCharacters(omdl.getBoundingBox().getLowerRightLat() + " " + omdl.getBoundingBox().getLowerRightLon());
            writer.writeEndElement();
            writer.writeEndElement();
            writer.writeEndElement();
            writer.writeCharacters("\n");

        } catch (XMLStreamException e) {
            e.printStackTrace();
        }

    }

    public void writeCentroid(String name, OMDataLocation omdl) {
        try {
            //centroid
            if (omdl.getCentroid() == null) {
                return;
            }
            writer.writeStartElement(namespaceGML, "Point");
            writer.writeAttribute("name", name);
            writer.writeStartElement(namespaceGML, "pos");
            writer.writeCharacters(omdl.getCentroid().getLat() + " " + omdl.getCentroid().getLon());
            writer.writeEndElement();
            writer.writeEndElement();
            writer.writeCharacters("\n");
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }
    }

    public void writeGMLElementsFromList(OMGraphicList omgl) {
        //String gml = "";
        Iterator it = omgl.iterator();
        Object o = null;
        //Element g = document.createElementNS(kmlNS, "g");
        while (it.hasNext()) {
            o = it.next();
            if (o instanceof OMGraphicList) {
                writeGMLElementsFromList((OMGraphicList) o);
            } else if (o instanceof OMGraphic) {
                writeGMLElement("", (OMGraphic) o);
            }
        }
    }

    public void write(ArrayList<GMLDataStorage> list) {
        for (GMLDataStorage gmlds : list) {
            gmlds.write(this);
        }

    }

    public void write(GMLDataStorage gmlds) {
        gmlds.write(this);
    }

    private void indent(int n) {
        try {
            for (int i = 0; i < n; i++) {
                writer.writeCharacters("\t");
            }
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }
    }

    public void write(OMDataLocation omdl, String col) {
        startGeometryProperty();
        if (omdl.getOMGraphic() != null) {
            OMGraphic omg = omdl.getOMGraphic();
            writeCentroidAndBoundingBox(omdl);
            if (omg instanceof OMPoly) {
                //System.out.println("POLYGON...");
                startMultiPolygon(col);
                writeGMLElement("", omdl.getOMGraphic());
                endMultiPolygon();
            } else {
                //System.out.println(omg.getClass());
                writeGMLElement(col, omg);
            }
        } else {
            writeCentroid(col, omdl);
        }
        endGeometryProperty();
    }
}
