// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * AdaptorButtonArray.java
 *
 * Created on 27 novembre 2008
 *
 * @author Jacques.Madelaine@info.unicaen.fr
 *
 */
// *-* tab-width: 4; *-*
package fr.unicaen.openmap.dataStorage.gui;

import java.awt.event.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.*;

import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.layer.dataDisplayLayer.SimpleDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.util.*;

import com.bbn.openmap.I18n;
import com.bbn.openmap.Environment;
import com.bbn.openmap.util.Debug;
import java.lang.reflect.Constructor;

public class AdaptorButtonArray {

    AddDataStorageConfigurationDialog dialog;
    private JComponent[] buttons;
    private JButton removeAdaptorButton;

    public AdaptorButtonArray(final SimpleDataDisplayLayer layer) {

        final I18n i18n = Environment.getI18n();

        final JButton joinButton =
                new JButton(Util.loadIcon("testjoin.png"));
        joinButton.setToolTipText(i18n.get(this, "addDS",
                i18n.TOOLTIP,
                "<html><center>Join <br>a New<br>Data Storage</center></html>"));
        joinButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent event) {
                try {
                    dialog = new AddDataStorageConfigurationDialog(layer.getDataStorageIterator().getDataStorage(),
                            (JFrame) SwingUtilities.getAncestorOfClass(Class.forName("javax.swing.JFrame"), joinButton));
                    dialog.pack();
                    dialog.setVisible(true);
                } catch (Exception e) {
                    Debug.error("Error creating dialog to configure an adaptor");
                }
                DataStorage newds = dialog.getNewDS();
                if (newds != null) {
                    layer.setDataStorageIterator(new DataStorageIteratorAdapter(newds));
                }
            }
        });



        removeAdaptorButton =
                new JButton(Util.loadIcon("deleteAdapter.png"));

        removeAdaptorButton.setToolTipText(i18n.get(this, "removeDSAdaptor",
                I18n.TOOLTIP, "Remove the adaptor"));

        removeAdaptorButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent event) {
                DataStorage ds = layer.getDataStorageIterator().getDataStorage();
		if (!(ds instanceof RemovableDataStorage)) {
                    JOptionPane.showMessageDialog(removeAdaptorButton,
						  i18n.get(AdaptorButtonArray.class,
							   "nothingToRemove",
							   "Nothing to Remove"));
                    return;
                }
                String mess = String.format(i18n.get(AdaptorButtonArray.class,
						     "removeAdaptorConfirmation",
						     I18n.TOOLTIP,
						     "Do you really want to remove the adapter %s?"),
					    ds.getName());
		
                int rep = JOptionPane.showConfirmDialog(removeAdaptorButton,
							mess,
							i18n.get(AdaptorButtonArray.class, "removeAdaptorConfirmation",
								 "Confirmation"),
							JOptionPane.YES_NO_OPTION);
		if (rep == JOptionPane.YES_OPTION)
		    layer.removeTopAdaptorDataStorage();
	    }
	    });


        //buttons = new JButton[]{addSortButton, addMultiFilterButton, joinButton, addPivotButton, addGridButton, addDelaunayButton, addKernelGridButton, addAddColumnButton, reqButton, aggregateButton};

        buttons = new JComponent[]{
            buildAdaptorButton(
            "fr.unicaen.openmap.dataStorage.MonoValuedFilterDataStorage",
            "animated.png",
            i18n.get(this, "animDS", I18n.TOOLTIP, "animate the data"),
            layer),
            buildAdaptorButton(
            "fr.unicaen.openmap.dataStorage.SortDataStorage",
            "sort.png",
            i18n.get(this, "sortDS", I18n.TOOLTIP, "Sort the data"),
            layer),
            buildAdaptorButton("fr.unicaen.openmap.dataStorage.MultiValuedFilterDataStorage",
            "multivaluedFilter.png",
            i18n.get(this, "addFilterDS", I18n.TOOLTIP, "Add a Filter to the Data Source"),
            layer),
            buildAdaptorButton(
            "fr.unicaen.openmap.dataStorage.expr.RequestFilterDataStorage",
            "request.png",
            i18n.get(this, "requestFilterDS", I18n.TOOLTIP, "Add a filter defined with a logical expression"),
            layer),
	    new JToolBar.Separator(),
            buildAdaptorButton("fr.unicaen.openmap.dataStorage.SelectColumnDataStorage",
            "selectColumn.png",
            i18n.get(this, "addSelectColumnDS", I18n.TOOLTIP, "Adaptor that can discard columns"),
            layer),
            buildAdaptorButton(
            "fr.unicaen.openmap.dataStorage.expr.AddColumnDataStorage",
            "addcolumn2.png",
            i18n.get(this, "addColumnDS", I18n.TOOLTIP, "Add a column"),
            layer),
            buildAdaptorButton(
            "fr.unicaen.openmap.dataStorage.EditableDataStorage",
            "drawing.png",
            i18n.get(this, "EditableDS", I18n.TOOLTIP, "Editable Datastorage"),
            layer),
	    new JToolBar.Separator(),
            joinButton,
            buildAdaptorButton("fr.unicaen.openmap.dataStorage.PivotDataStorage",
            "testpivot.png",
            i18n.get(this, "pivotDS", I18n.TOOLTIP, "Pivot the Data Table"),
            layer),
            buildAdaptorButton("fr.unicaen.openmap.dataStorage.GridDataStorage",
            "testgrid.png",
            i18n.get(this, "gridDS", I18n.TOOLTIP, "Construct a regular grid"),
            layer),
            buildAdaptorButton(
            "fr.unicaen.openmap.dataStorage.KernelGridDataStorage",
            "kernelgrid.png",
            i18n.get(this, "kernelGridDS", I18n.TOOLTIP, "Construct a regular grid using kernel"),
            layer),
            buildAdaptorButton(
            "fr.unicaen.openmap.dataStorage.AggregateDataStorage",
            "agreggate.png",
            i18n.get(this, "aggregateDS", I18n.TOOLTIP, "Spatial aggregation"),
            layer),
            buildAdaptorButton(
            "fr.unicaen.openmap.dataStorage.DelaunayTriangulationDataStorage",
            "delaunay.png",
            i18n.get(this, "delaunayDS", I18n.TOOLTIP, "Construct a Delaunay triangular mesh"),
            layer),
            buildAdaptorButton(
            "fr.unicaen.openmap.dataStorage.Contours",
            "contours.png",
            i18n.get(this, "contours", I18n.TOOLTIP, "Construct isolines from a regular grid or a triangular mesh"),
            layer),
        };

    }

    public JComponent[] getButtons() {

        return buttons;

    }

    public JButton getRemoveAdaptorButton() {
        return removeAdaptorButton;
    }

    public void setVisible(boolean b) {
        for (int i = 0; i <
                buttons.length; i++) {
            buttons[i].setVisible(b);
        }

        removeAdaptorButton.setVisible(b);
    }

    public JButton buildAdaptorButton(
            final String className,
            String ic, String ttip, final SimpleDataDisplayLayer layer) {

        JButton button = new JButton(Util.loadIcon(ic));
        button.setToolTipText(ttip);
        button.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent event) {

                try {
                    DataStorage ds = (DataStorage) getDataStorageConstructor(className).newInstance(
                            new Object[]{layer, layer.getDataStorageIterator().getDataStorage()});
                    layer.setDataStorageIterator(new DataStorageIteratorAdapter(ds));
                    layer.dataStorageEvent(ds);
                } catch (Exception e) {
		    e.printStackTrace();
                }
            }
        });
        return button;
    }

    public static Constructor getDataStorageConstructor(String className) {
        try {
            Class cls = Class.forName(className);
            for (Constructor c : cls.getConstructors()) {
                java.lang.reflect.Type[] params = c.getGenericParameterTypes();
                if (params.length == 2) {
                    return c;
                }


            }

        } catch (Exception ex) {
            Logger.getLogger(AdaptorButtonArray.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }
}
