// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * AddDataStorageConfigurationDialog.java
 *
 * Created on 27 november 2007
 *
 * @author Jacques.Madelaine@info.unicaen.fr
 *
 */
package fr.unicaen.openmap.dataStorage.gui;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.propertyEditor.Inspector;

import fr.unicaen.openmap.util.propertyEditor.DataStoragePropertyEditor;
import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.Util;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Panel to configure a data storage added to a join data storage.
 *
 */
public class AddDataStorageConfigurationDialog extends JDialog implements DataStorageListener {

    private DataStorage jds;
    private int dialogStep = 1;
    protected I18n i18n = Environment.getI18n();

    /* <attributes to set> */
    private String firstJoinColumnName;

    protected void setFirstJoinColumnName(String s) {
        Debug.message("DataStorage", "### First Join Column Name: " + s);
        firstJoinColumnName = s;
    }
    private String secondJoinColumnName;

    protected void setSecondJoinColumnName(String s) {
        Debug.message("DataStorage", "### Second Join Column Name: " + s);
        secondJoinColumnName = s;
    }

    private DataStoragePropertyEditor.DataStorageClass dsClass =
	DataStoragePropertyEditor.DataStorageClass.SHAPE;

    protected void setDSClass(DataStoragePropertyEditor.DataStorageClass  dsClass) {
        Debug.message("DataStorage", "### New Data Storage Class Name: " + dsClass);
        this.dsClass = dsClass;
    }
    private DataStorage ds;

    protected DataStorage createDataStorage() {
        return ds = fr.unicaen.openmap.util.Util.createDataStorage(dsClass.getClassName(), this);
    }
    /* </attributes to set> */
    private DualDataStorage newDS = null;

    public DataStorage getNewDS() {
        return newDS;
    }

    public void dataStorageEvent(DataStorage source) {
        Debug.message("DataStorage", "[AddDataStorageConfigurationDialog:] dataStorageEvent <== " + source);
        if (source.isReady()) {
            nextButton.setEnabled(true);
        }
    }
    private JButton nextButton,  backButton;
    private JPanel navigationPanel = new JPanel();

    public AddDataStorageConfigurationDialog(DataStorage jds, JFrame frame) {
        super(frame, "Join a New Data Storage", true);
        setTitle(i18n.get(AdaptorButtonArray.class, "addDS", "Join a New Data Storage"));
        this.jds = jds;
        panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.PAGE_AXIS));
        setContentPane(panel);

        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {

            public void windowClosing(WindowEvent we) {
                cancelDialog();
            }
        });

        nextButton = new JButton(i18n.get(this, "next", "Next >"));
        nextButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                nextStep();
            }
        });
        backButton = new JButton(i18n.get(this, "back", "Back <"));
        backButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                previousStep();
            }
        });
        JButton cancelButton = new JButton(i18n.get(this, "cancel", "Cancel"));
        cancelButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                cancelDialog();
            }
        });
        navigationPanel.add(backButton);
        navigationPanel.add(nextButton);
        navigationPanel.add(cancelButton);

        refresh();
    }
    private JPanel panel;

    protected void addToDialogPanel(Component c) {
        panel.add(c);
    }
    //   protected void setDialogPanel(Component c) { panel.removeAll(); panel.add(c); }

    protected void whipeDialogPanel() {
        panel.removeAll();
    }

    protected void nextStep() {
        dialogStep++;
        refresh();
    }

    protected void previousStep() {
        if (dialogStep > 1) {
            dialogStep--;
        }
        refresh();
    }

    protected void cancelDialog() {
        dialogStep = 0;
        setVisible(false);
    }

    protected void finishDialog() {
        newDS = new DualDataStorage(jds, firstJoinColumnName, ds, secondJoinColumnName);
        newDS.refresh();
        cancelDialog();
    }

    /** Steps : <ul>
    <li> 1: set join column name
    <li> 2: choose new ds type
    <li> 3: configure new data storage
    <li> 4: set the join column name of the new data storage
    <li> add the ds
    </ul>
     */
    public void refresh() {
        Debug.message("DataStorage", "### JoinedDataStorageDialogPanel step " + dialogStep);
        whipeDialogPanel();
        ;
        if (dialogStep > 0) {
            JLabel stepLabel = new JLabel(i18n.get(this, "step" + dialogStep, "Step " + dialogStep));
            //stepLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
            addToDialogPanel(stepLabel);
        }

        backButton.setEnabled(dialogStep > 1);
        nextButton.setEnabled(false);
        getRootPane().setDefaultButton(nextButton);

        switch (dialogStep) {
	case 0:

	    return;
	case 1:
	    {
		JComboBox jcn1ComboBox = new JComboBox(jds.getDefaultDataTableModel().getDataColumnNames());
		jcn1ComboBox.setSelectedIndex(jds.getDefaultDataTableModel().getIndexForColumnName(firstJoinColumnName));
		nextButton.setEnabled(firstJoinColumnName != null);
		jcn1ComboBox.addActionListener(new ActionListener() {

                        public void actionPerformed(ActionEvent e) {
                            int index = ((JComboBox) e.getSource()).getSelectedIndex();
                            if (index >= 0) {
                                setFirstJoinColumnName(jds.getDefaultDataTableModel().getRealColumnName(index));
                                nextButton.setEnabled(true);
                            }
                        }
                    });
		JPanel panel = new JPanel();
		panel.add(jcn1ComboBox);
		addToDialogPanel(panel);
	    }
	    break;
	case 2:
	    {
		JComboBox dsClassNameComboBox = new JComboBox(DataStoragePropertyEditor.DataStorageClass.values());
		dsClassNameComboBox.setSelectedIndex(-1);
		dsClassNameComboBox.addActionListener(new ActionListener() {

                        public void actionPerformed(ActionEvent e) {
                            Object o = ((JComboBox) e.getSource()).getSelectedItem();
                            if (o != null) {
                                setDSClass(((DataStoragePropertyEditor.DataStorageClass)o));
                                nextButton.setEnabled(true);
                            }
                        }
                    });
		JPanel panel = new JPanel();
		panel.add(dsClassNameComboBox);
		addToDialogPanel(panel);
	    }
	    break;
		
	case 3:
	    {
		JComponent component = null;
		try {
		    component = createDataStorage().getGUI();
		} catch(Exception e) {
		    e.printStackTrace();
		    component = new JPanel();
		    component.add(new JLabel(dsClass+" "+i18n.get(DataStoragePropertyEditor.class, "notAvailable", "not available")));
		}
		addToDialogPanel(component);
	    }
	    break;
	case 4:
	    {
		JComboBox jcn2ComboBox = new JComboBox(ds.getDefaultDataTableModel().getDataColumnNames());
		jcn2ComboBox.setSelectedIndex(-1);

		jcn2ComboBox.addActionListener(new ActionListener() {

                        public void actionPerformed(ActionEvent e) {
                            int index = ((JComboBox) e.getSource()).getSelectedIndex();
                            if (index >= 0) {
                                setSecondJoinColumnName(ds.getDefaultDataTableModel().getRealColumnName(index));
                                nextButton.setEnabled(true);
                            }
                        }
                    });
		JPanel panel = new JPanel();
		panel.add(jcn2ComboBox);
		addToDialogPanel(panel);
	    }
	    break;
	case 5: {
	    JButton okButton = new JButton(i18n.get(this, "finish", "Finish"));
	    okButton.addActionListener(new ActionListener() {

                    public void actionPerformed(ActionEvent e) {
                        finishDialog();
                    }
                });
	    JPanel panel = new JPanel();
	    panel.add(okButton);
	    addToDialogPanel(panel);
	    getRootPane().setDefaultButton(okButton);
	}
        }

        addToDialogPanel(navigationPanel);

        pack();
    }
}
/// interface Settable { public void set(int index); }
/// 
/// /** to be used also for DualDataStorage... */
/// class CustomComboBoxPanel extends JPanel {
///     
///     private JComboBox comboBox = new JComboBox();
///     private JLabel warningMessage = new JLabel("You must set Join Column Name");
///     private Settable name;
///     CustomComboBoxPanel(String names[], int selectedIndex, Settable _name) {
/// 	this.name = _name;
/// 	this.add(new JLabel("Join attribute "));
/// 	this.add(comboBox);
/// 	this.add(warningMessage);
/// 	setComboBoxContent(names);
/// 	comboBox.setSelectedIndex(selectedIndex);
/// 
/// 	comboBox.addActionListener(new ActionListener() {
/// 		public void actionPerformed(ActionEvent e) {
/// 		    int index = ((JComboBox)e.getSource()).getSelectedIndex();
/// 		    if (index >= 0 && ! isSetNameInhibited()) {
/// 			name.set(index); // setJoinColumnName(getDefaultDataTableModel().getRealColumnName(index));
/// 			warningMessage.setText("");
/// 		    }
/// 		}});	
/// 	Debug.message("DataStorage", "### Combobox configured");
/// 
///     }
/// 
///     // prevent JComboBox to set selected the added item
///     private boolean _inhibitSetName = false;
///     protected boolean isSetNameInhibited() { return _inhibitSetName; }
/// 
///     public void resetComboBoxContent(String[] names) {
/// 	name.set(-1);
/// 	setComboBoxContent(names);
///     }
///     public void setComboBoxContent(String[] names) {
/// 	_inhibitSetName = true;
/// 	comboBox.removeAllItems();
/// 	for (String s : names)
/// 	    comboBox.addItem(s);
/// 	_inhibitSetName = false;
/// 	comboBox.setSelectedIndex(-1);
///     }
/// }
