// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.gui;
import fr.unicaen.openmap.dataStorage.SelectionMatrix;
import java.util.Vector;
import javax.swing.table.*;


/**
 * Link between graphical table display and the 'real' data.
 */
public class DataAndSelectionModel extends AbstractTableModel {
    /**
     * Associated datas.
     */
    private SelectionMatrix matrix = null;
    private TableModel datas = null;
    private String matrixName = null;
    
    public DataAndSelectionModel(TableModel datas, SelectionMatrix matrix, String matrixName) {
        this.matrix = matrix;
        this.datas = datas;
        this.matrixName=matrixName;
    }
    
    public int getColumnCount() { return matrix.getNbCols()+datas.getColumnCount(); }
    public int getRowCount() { return datas.getRowCount(); }
    public Object getValueAt(int row, int col) {
        if (datas.getColumnCount()>col)
            return datas.getValueAt(row,col);
        return matrix.getValueAt(row,col-datas.getColumnCount());
    }
    
    public String getColumnName(int col) {
        if (datas.getColumnCount()>col)
            return datas.getColumnName(col);
        return matrixName+"\n"+matrix.getColumnName(col-datas.getColumnCount());
    }
    public Class getColumnClass(int column) { 
        if (getValueAt(0,column)!=null) return getValueAt(0,column).getClass(); 
        else return Object.class;
    
    }
    
    public boolean isCellEditable(int row, int col) {
        return col >= datas.getColumnCount();
    }
    
    public void setValueAt(Object aValue, int row, int col) {       
        if (col>=datas.getColumnCount()){
            matrix.setValueAt(aValue,row,col-datas.getColumnCount());
            fireTableRowsUpdated(row, row);
       }
    }
    
    public int getDataColumnCount(){return datas.getColumnCount();}
    
    public void setMatrix(SelectionMatrix matrix, String matrixName){
      this.matrix=matrix;
      this.matrixName=matrixName;
      fireTableStructureChanged();
    }
    public void updateMatrixName(String name){
        matrixName=name;
    }
}
