// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * DataTableSelectionStoragePanel.java
 *
 * Created on 9 novembre 2005, 23:31
 *
 * @author Christophe.Turbout@info.unicaen.fr
 *
 */

package fr.unicaen.openmap.dataStorage.gui;
import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.dataStorage.SelectionDataStorage;
import fr.unicaen.openmap.dataStorage.SelectionDataStorageListener;
import fr.unicaen.openmap.dataStorage.SelectionMatrix;
import fr.unicaen.openmap.util.Util;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Vector;
import javax.swing.*;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;


/**
 * This class is used to display the content of a TableModel.
 *
 */
public class DataTableSelectionStoragePanel extends JPanel implements SelectionDataStorageListener, ActionListener, MouseListener, TableModelListener {
    /** String action command for delete a tab. */
    private final static String DELETE = "-1";
    
    private JTable tableView = null;
    private SelectionDataStorage selectionDataStorage = null;
    private boolean menuMode = true;
    
    public  DataTableSelectionStoragePanel(SelectionDataStorage selectionDataStorage) {
        this(selectionDataStorage, selectionDataStorage.getDataAndSelectionModel());
    }
    
    public  DataTableSelectionStoragePanel(SelectionDataStorage selectionDataStorage, TableModel model) {
        super();
        setDoubleBuffered(true);
        this.selectionDataStorage = selectionDataStorage;
        try {
            setLayout(new BorderLayout());
            tableView = new JTable(model){
                public String getToolTipText(MouseEvent e) {
                    Object tip = null;
                    java.awt.Point p = e.getPoint();
                    int rowIndex = rowAtPoint(p);
                    int colIndex = columnAtPoint(p);
                    int realColumnIndex = convertColumnIndexToModel(colIndex);
                    tip =  getValueAt(rowIndex, colIndex);
                    if (tip ==null) return "";
                    return tip.toString();
                }
            };
            
            // change default CellRenderer in default JTableHeader
            ColumnHeaderRenderer renderer = new ColumnHeaderRenderer(selectionDataStorage, selectionDataStorage.getDefaultDataTableModel().getColumnCount() );
            tableView.setAutoCreateColumnsFromModel(true);
            JTableHeader tableHeader = new JTableHeader(tableView.getColumnModel());
            tableHeader.setDefaultRenderer(renderer);
            tableHeader.setReorderingAllowed(false);
            tableView.setTableHeader(tableHeader);
            tableView.setShowHorizontalLines(true);
            tableView.setShowVerticalLines(true);
            tableView.setIntercellSpacing(new Dimension(2, 2));
            tableView.setRowSelectionAllowed(true);
            tableView.setRowHeight(25);            
            tableView.setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS); //JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS); // AUTO_RESIZE_OFF);
            tableView.addMouseListener(this);
            JScrollPane sp = new JScrollPane(tableView);
            add(sp);
        } catch (Exception e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
    }
    
    public void setColumnSize(int column, int size) {
        if (tableView!=null)
            tableView.getColumnModel().getColumn(column).setPreferredWidth(size);
    }
    
    /*
     * We have to update panel, data structure has changed. This method is used when the structure is not the same which
     * means a column are a row has been added or suppressed.
     *
     **/
    public void fireStructureChanged(){
        //selectionDataStorage.getDataAndSelectionModel().fireTableStructureChanged();
        ((DataAndSelectionModel)tableView.getModel()).fireTableStructureChanged();
    }
    
    
    /**
     * We have to update model thaht data has changed.
     **/
    public void fireTableDataChanged(){
        selectionDataStorage.getDataAndSelectionModel().fireTableDataChanged();
    }
    
    
    /**
     * return the JTable used in the panel.
     **/
    public JTable getJTable(){
        return tableView;
    }
    
    private JPopupMenu getPopup() {
        JPopupMenu pm = new JPopupMenu(); // construct a new JPopupMenu
        JMenuItem addNewMenu = new JMenuItem("Enlever la sélection");
        addNewMenu.setActionCommand(DELETE);
        addNewMenu.addActionListener(this);
        pm.add(addNewMenu);
        pm.addSeparator();
        SelectionColumnMatrixPanel selectionPanel = (SelectionColumnMatrixPanel)((JTabbedPane)getParent()).getParent();
        for (int i =0; i<selectionPanel.getSubSelectionsSize(); i++) {
            JMenuItem zoneItem = new JMenuItem("Envoyer vers la sous-zone "+(i+1));
            zoneItem.setActionCommand(""+i);
            zoneItem.addActionListener(this);
            pm.add(zoneItem);
        }
        return pm;
    }
    
    /**
     * Invoked when an action occurs.
     * @param e the action event
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().equals(DELETE)) {
            ListSelectionModel lsm = tableView.getSelectionModel();
            int minIndex = lsm.getMinSelectionIndex();
            int maxIndex = lsm.getMaxSelectionIndex();
            deleteRow(lsm,minIndex,maxIndex);
        } else if (e.getSource().getClass()==JMenuItem.class) {
            ListSelectionModel lsm = tableView.getSelectionModel();
            int minIndex = lsm.getMinSelectionIndex();
            int maxIndex = lsm.getMaxSelectionIndex();
            moveRowToSubZone(lsm,minIndex,maxIndex,Integer.parseInt(e.getActionCommand()));
        }
    }
    
    
    public void mouseClicked(MouseEvent e) { }
    public void mouseEntered(MouseEvent e) { }
    public void mouseExited(MouseEvent e) { }
    public void mouseReleased(MouseEvent e) { }
    public void mousePressed(MouseEvent e) {
        if (e.getID() == MouseEvent.MOUSE_PRESSED && e.getButton() == MouseEvent.BUTTON3 && menuMode) {
            ListSelectionModel lsm = tableView.getSelectionModel();
            if (! lsm.isSelectionEmpty() && lsm.getMinSelectionIndex()<(tableView.getRowCount()-2)) getPopup().show(tableView, e.getX(), e.getY());
        }
    }
    
    public void fireUpdateColors(){
        selectionDataStorage.getDataAndSelectionModel().fireTableStructureChanged();
    }
    
    /**
     * set the flag to draw or not menu used to deleted some rows.
     **/
    public void setMenuUsage(boolean state){
        menuMode=state;
    }
    
    /**
     * tell if we can select only one or many rows on the table. Default is set to many.
     **/
    public void setSelectionMode(int mode){
        tableView.setSelectionMode(mode);
    }
    
    /**
     * set the table model active.
     **/
    
    public void setTableModeListener(){
        tableView.getModel().addTableModelListener(this);
    }
    
    /**
     * give us the ability to update other views when table is modified.
     **/
    public void tableChanged(TableModelEvent e) {
        
        // refresh the layer.
        SelectionStoragePanel selectionPanel = (SelectionStoragePanel)((JTabbedPane)getParent()).getParent();
        if (!selectionPanel.isShowing()) {
            Debug.message(Util.className(this),"SelectionStoragePanel is not showing --> abort");
            return;
        }
        // if (selectionPanel.getUnicaenLayerListener() !=null) selectionPanel.getUnicaenLayerListener().refresh();
        // if (e.getFirstRow() == e.getLastRow() && e.getFirstRow()>1){
        if (e.getFirstRow() == e.getLastRow() && e.getFirstRow()!=TableModelEvent.HEADER_ROW){
            int firstRow = e.getFirstRow();
            Debug.message(Util.className(this),"firstRow = "+firstRow);
            // get the modified matrix
            SelectionMatrix matrix = selectionDataStorage.getSelectionMatrix(selectionDataStorage.getCurrentSelection());
            // the the modified rows
            Vector<Integer> modified = matrix.getLastModifiedRow();
            if (modified!=null && modified.size()!=-1) {
                Debug.message(Util.className(this),"selectionPanel zone "+selectionDataStorage.getCurrentSelection()+" = "+selectionPanel.toString());
                Debug.message(Util.className(this),"selectionPanel isShowing = "+selectionPanel.isShowing());
                Debug.message(Util.className(this),"selectionPanel width = "+selectionPanel.getWidth());
                // Utils.vectorDebug(modified);
                for (int i=0;i<modified.size();i++){
                    Debug.message(Util.className(this)," i = " + i + " --  currentSelection = "+selectionDataStorage.getCurrentSelection()+" -- e.getFirstRow() = "+e.getFirstRow());
                    if (modified.get(i)==1) selectionPanel.getSelections().fireDataChanged(selectionDataStorage.getCurrentSelection(),i,e.getFirstRow(),true);
                    if (modified.get(i)==-1) selectionPanel.getSelections().fireDataChanged(selectionDataStorage.getCurrentSelection(),i,e.getFirstRow(),false);
                }
            }
            if (selectionPanel.getUnicaenLayerListener() !=null) selectionPanel.getUnicaenLayerListener().refreshGraphics();
        }
    }
    
    private void deleteRow(ListSelectionModel lsm, int minIndex, int maxIndex) {
        Debug.message(Util.className(this),this+"indexmin : "+minIndex+", indexmax : "+maxIndex);
        SelectionColumnMatrixModel model = (SelectionColumnMatrixModel) tableView.getModel();
        // get the layer Listener
        JTabbedPane tabbedpane = (JTabbedPane) getParent();
        SelectionColumnMatrixPanel panel = (SelectionColumnMatrixPanel) tabbedpane.getParent();
        CustomTabbedPane custompane = (CustomTabbedPane) panel.getParent();
        SelectionMatrixPanel thepanel = (SelectionMatrixPanel) custompane.getParent();
        Vector<Integer> list = new Vector<Integer>();
        // check index to delete
        for (int i = minIndex; i <= maxIndex; i++)  if (lsm.isSelectedIndex(i) && i<model.getRealRowCount()) list.add(model.getRealRow(i));
        // delete them
        for (int j=0; j<list.size();j++) selectionDataStorage.deselectIndexInModel(list.get(j));
        if (thepanel.getUnicaenLayerListener()!=null) thepanel.getUnicaenLayerListener().refreshGraphics();
    }
    
    private void moveRowToSubZone(ListSelectionModel lsm, int minIndex, int maxIndex, int newZone) {
        SelectionColumnMatrixModel model = (SelectionColumnMatrixModel) tableView.getModel();
        JTabbedPane tabbedpane = (JTabbedPane) getParent();
        SelectionColumnMatrixPanel panel = (SelectionColumnMatrixPanel) tabbedpane.getParent();
        CustomTabbedPane custompane = (CustomTabbedPane) panel.getParent();
        SelectionMatrixPanel thepanel = (SelectionMatrixPanel) custompane.getParent();
        Vector<Integer> list = new Vector<Integer>();
        // check index to delete
        for (int i = minIndex; i <= maxIndex; i++)  if (lsm.isSelectedIndex(i) && i<model.getRealRowCount()) list.add(model.getRealRow(i));
        // delete them
        for (int j=0; j<list.size();j++) selectionDataStorage.moveIndexInModel(list.get(j),newZone);
        if (thepanel.getUnicaenLayerListener()!=null) thepanel.getUnicaenLayerListener().refreshGraphics();
    }

    public void fireSelectionChanged() {
        throw new UnsupportedOperationException("Not supported yet.");
    }
}
