// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.gui;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMColor;
import fr.unicaen.openmap.dataStorage.DataObject;
import fr.unicaen.openmap.dataStorage.OMDataLocation;
import java.awt.Color;
import java.awt.Paint;
import javax.swing.JTable;
import javax.swing.event.ListSelectionEvent;


/**
 *
 * @author jchauvea
 */
public class EditableDataStorageTable extends JTable {

    private DataTableStoragePanel editableDTSPanel;
    private OMGraphicList currentOmgs = new OMGraphicList();
    private Paint initialFillPaint = null;

    public EditableDataStorageTable(DataTableStoragePanel panel) {
        super(panel.getModel());
        this.editableDTSPanel = panel;
    }

    @Override
    public void valueChanged(ListSelectionEvent e) {
        super.valueChanged(e);
        deselectOMGraphic(currentOmgs);
        DataTableModel model = (DataTableModel) getModel();
        int index = getSelectedRow();//e.getLastIndex();
        if (index < 0)
            return;
        DataObject obj = model.getDataObjectAt(index);
        String[] oMcolNames = model.getOMColumnNames();
        currentOmgs.clear();
        for (String col : oMcolNames) {
            OMDataLocation omdl = obj.getOMDataLocation(col);
            if (omdl == null)
                continue;
            OMGraphic omg = omdl.getOMGraphic();
            if (omg != null) {
                currentOmgs.add(omg);
            }
        }
        selectOMGraphic(currentOmgs);
        editableDTSPanel.getDataStorage().getLayer().refreshGraphics();
    }

    public void selectOMGraphic(OMGraphic omg){
	// pb the graphic is wrap into a list that as no fill color
	initialFillPaint  = this.getFillPaint(omg);
        omg.setFillPaint(new Color(255, 255, 255, 255));
    }

    protected Paint getFillPaint(OMGraphic omg) {
	if (omg instanceof OMGraphicList &&  ((OMGraphicList) omg).size() > 0) {
	    OMGraphic g = ((OMGraphicList) omg).getOMGraphicAt(0);
	    if (g != null)
		return  g.getFillPaint();
	}
	return omg.getFillPaint();
    }
    
    public void deselectOMGraphic(OMGraphic omg){
        omg.setFillPaint(initialFillPaint);
    }
}
