// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.gui;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;
import fr.unicaen.openmap.dataStorage.GridDataStorage;
import fr.unicaen.openmap.dataStorage.PivotDataStorage;
import fr.unicaen.openmap.exception.UnavailableColumnNameException;
import fr.unicaen.openmap.gui.component.JComponentCellEditor;
import fr.unicaen.openmap.gui.component.JComponentCellRenderer;
import fr.unicaen.openmap.gui.component.JComponentTable;
import fr.unicaen.openmap.gui.component.JComponentTableModel;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

/**
 *
 * @author jchauvea
 */
public class GridDataStoragePanel extends JPanel {

    protected I18n i18n = Environment.getI18n();
    protected JPanel topPanel, panResolution;
    protected JComboBox geometryList;
    protected GridDataStorage gridDS;
    protected JTextField fieldName;
    public GridDataStoragePanel(GridDataStorage ds) {
	this(ds, true);
    }
    public GridDataStoragePanel(GridDataStorage ds, boolean withAggregateFunctionChooser) {
        this.gridDS = ds;
        Box gridPanel = Box.createVerticalBox();
        try {
            geometryList = new JComboBox(ds.getDataStorage().getDefaultDataTableModel().getOMColumnNames());
            if (!"".equals(ds.getGeometryColumnName()) && ds.getGeometryColumnName() != null) {
                geometryList.setSelectedIndex(
                        ds.getDataStorage().getDefaultDataTableModel().getIndexForOMColumnName(
                        ds.getGeometryColumnName()));
            } else {
                ds.setGeometryColumnName(ds.getDataStorage().getDefaultDataTableModel().getOMColumnNames()[0]);
            }
        } catch (NullPointerException ex) {
            System.err.println("Geometry Column Name List is unavailable");
            geometryList = new JComboBox();
        } catch (ArrayIndexOutOfBoundsException e) {
            System.err.println("Geometry Column Name List is empty");
            geometryList = new JComboBox();
        }

        geometryList.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                try {
                    int index = ((JComboBox) e.getSource()).getSelectedIndex();
                    gridDS.setGeometryColumnName(gridDS.getDataStorage().getDefaultDataTableModel().getRealOMColumnName(index));
                } catch (ArrayIndexOutOfBoundsException ex) {
                }
            }
        });       

        // panel for latitudeResolution and longitudeResolution settings
        panResolution = new JPanel();
        panResolution.add(new JLabel(i18n.get(GridDataStoragePanel.class, "latitudeResolution", "Latitude Resolution:")));
        final JSpinner latitudeResolutionField = new JSpinner(new SpinnerNumberModel(ds.getLatitudeResolution(), 0d, 180d, 1d));
        latitudeResolutionField.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                gridDS.setLatitudeResolution(((Double) latitudeResolutionField.getValue()).floatValue());
            }
        });
        latitudeResolutionField.setToolTipText(i18n.get(GridDataStoragePanel.class, "latitudeResolution", i18n.TOOLTIP, "Latitude Resolution"));
        panResolution.add(latitudeResolutionField);

        panResolution.add(new JLabel(i18n.get(GridDataStoragePanel.class, "longitudeResolution", "Longitude Resolution")));
        final JSpinner longitudeResolutionField = new JSpinner(new SpinnerNumberModel(ds.getLongitudeResolution(), 0d, 180d, 1d));
        longitudeResolutionField.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                gridDS.setLongitudeResolution(((Double) longitudeResolutionField.getValue()).floatValue());
            }
        });
        longitudeResolutionField.setToolTipText(i18n.get(this, "longitudeResolution", i18n.TOOLTIP, "Longitude resolution"));

        panResolution.add(longitudeResolutionField);
        gridPanel.add(buildTopPanel());
        //gridPanel.add(super.getGUI()); // should suppress alwaysPrefixPivotWithStatisticsName and Pivot radio buttons
        /***************************************/
        Box tablePanel = Box.createVerticalBox();
        String[] attributesNames = ds.getDataStorage().getDefaultDataTableModel().getDataColumnNames();
        ds.initAttributesHashMapFromLists(attributesNames);


        String criteriaToolTipText = i18n.get(PivotDataStorage.class, "criteria", I18n.TOOLTIP, "Lead to one row per criteria value");
        //String pivotToolTipText = i18n.get(this, "pivot", I18n.TOOLTIP, "Lead to one statistic attribute per pivot value");
        String statisticsToolTipText = i18n.get(PivotDataStorage.class, "statistics", I18n.TOOLTIP, "The statistics values aggregated at each mesh");
        String discardToolTipText = i18n.get(PivotDataStorage.class, "discard", I18n.TOOLTIP, "The discarded attributes");
        String aggregateFunctionToolTipText = i18n.get(PivotDataStorage.class, "aggregateFunction", I18n.TOOLTIP, "The aggregate function applied to the statistic attribute");

        String[] columnNamesWithAggregateFunction = {
            i18n.get(PivotDataStorage.class, "attributes", "Attributes"),
            i18n.get(PivotDataStorage.class, "discard", "Discard"),
            i18n.get(PivotDataStorage.class, "criteria", "Criteria"),
            i18n.get(PivotDataStorage.class, "statistics", "Statistics"),
            i18n.get(PivotDataStorage.class, "aggregateFunction", "Aggregate Function")};
        String[] columnNamesWithoutAggregateFunction = {
            i18n.get(PivotDataStorage.class, "attributes", "Attributes"),
            i18n.get(PivotDataStorage.class, "discard", "Discard"),
            i18n.get(PivotDataStorage.class, "criteria", "Criteria"),
            i18n.get(PivotDataStorage.class, "statistics", "Statistics")};
        Object[][] data = new Object[attributesNames.length][withAggregateFunctionChooser ? columnNamesWithAggregateFunction.length : columnNamesWithoutAggregateFunction.length];
        JComponentTableModel model = new JComponentTableModel(data, withAggregateFunctionChooser ? columnNamesWithAggregateFunction : columnNamesWithoutAggregateFunction);
        JComponentTable table = new JComponentTable(model);
        table.setRowHeight(25);
        table.setBackground(new Color(238, 238, 238));
        table.setDefaultRenderer(JComponent.class, new JComponentCellRenderer());
        DefaultTableCellRenderer renderer = new DefaultTableCellRenderer();
        renderer.setHorizontalAlignment(SwingConstants.CENTER);
        TableColumn col1 = table.getColumnModel().getColumn(0);
        col1.setCellRenderer(renderer);
        table.setDefaultEditor(JComponent.class, new JComponentCellEditor());

        int index = 0;
        for (String name : attributesNames) {
            table.setValueAt(name, index, 0);

	    JComboBox cb = ds.new AggregateFunctionComboBox(name);
	    cb.setToolTipText(aggregateFunctionToolTipText);
	    cb.setEnabled(ds.hasAttributeType(name, PivotDataStorage.AttributeType.STATISTIC));

	    if(withAggregateFunctionChooser) {		
		table.setValueAt(cb, index, 4);
	    }

            ButtonGroup attributeRBGroup = new ButtonGroup();
            JRadioButton rb1, rb2, rb3;

            rb1 = new JRadioButton(ds.new RBAction(name, PivotDataStorage.AttributeType.DISCARD, cb, table));
            rb1.setToolTipText(discardToolTipText);
            rb1.setVerticalAlignment(JRadioButton.CENTER);
            rb1.setSelected(ds.hasAttributeType(name, PivotDataStorage.AttributeType.DISCARD));
            attributeRBGroup.add(rb1);

            table.setValueAt(rb1, index, 1);


            rb2 = new JRadioButton(ds.new RBAction(name, PivotDataStorage.AttributeType.CRITERIA, cb, table));
            rb2.setToolTipText(criteriaToolTipText);
            rb2.setSelected(ds.hasAttributeType(name, PivotDataStorage.AttributeType.CRITERIA));
            attributeRBGroup.add(rb2);
            table.setValueAt(rb2, index, 2);

            rb3 = new JRadioButton(ds.new RBAction(name, PivotDataStorage.AttributeType.STATISTIC, cb, table));
            rb3.setToolTipText(statisticsToolTipText);
            rb3.setSelected(ds.hasAttributeType(name, PivotDataStorage.AttributeType.STATISTIC));
            attributeRBGroup.add(rb3);
            table.setValueAt(rb3, index, 3);

            index++;
        }



        JButton validateButton = new JButton(i18n.get(GridDataStoragePanel.class, "validate", "Validate"));
        // should be active only if at least a criteria is chosen -- TBD.
        validateButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                try{
                gridDS.setGridGeometryColumnName(fieldName.getText());
                gridDS.initAttributesListsFromHashMap();
                gridDS.initialize();
                }
                catch(UnavailableColumnNameException ex){
                    JOptionPane.showMessageDialog(null, ex.getMessage(), "", JOptionPane.ERROR_MESSAGE);
                }
            }
        });
        /*attributeAndValidatePanel*/
        JPanel validWrap = new JPanel();
        validWrap.add(validateButton);
        tablePanel.add(table.getTableHeader());
        tablePanel.add(table);
        tablePanel.add(validWrap);

        Box panel = Box.createVerticalBox();
        panel.add(tablePanel);
        // add the datastorage controls
        panel.add(ds.getDataStorage().getGUI());

        //table.setFillsViewportHeight(true);
        table.updateUI();
        //pivotPanel.setBorder(BorderFactory.createTitledBorder(i18n.get(this, "name", "Pivot")));
        /****************************************/
        gridPanel.add(panel);
        // wrap the panel in a flow layout panel in order to shrink the components
        add(gridPanel);
        setBorder(BorderFactory.createTitledBorder(i18n.get(GridDataStoragePanel.class, "name", "Grid")));

    }


    protected JComponent buildTopPanel(){
        topPanel = new JPanel();
        topPanel.add(new JLabel(i18n.get(GridDataStoragePanel.class, "centroid", "Centroid used:")));
        topPanel.add(geometryList);
        topPanel.add(panResolution);
        JPanel namePanel = new JPanel();
        namePanel.add(new JLabel(i18n.get(GridDataStoragePanel.class,"gridnamelabel","Name")));
        fieldName = new JTextField(gridDS.getGridGeometryColumnName(),15);
        fieldName.setToolTipText(i18n.get(GridDataStoragePanel.class, "gridnamelabel", i18n.TOOLTIP, "Name of the grid created geometry"));

        namePanel.add(fieldName);
        topPanel.add(namePanel);
        return topPanel;
    }
}
