// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.gui;

import java.sql.*;

import javax.swing.*;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.BorderLayout;

import fr.unicaen.openmap.util.propertyEditor.TextAreaPropertyEditor;

public class SQLEditor
{
	private final TextAreaPropertyEditor txt;
	private final JPanel panel;
	private final JList jl;
	private final DefaultListModel lm;
	private final Connection connection;

	public SQLEditor( Connection conn ) throws Exception
	{
		connection = conn;
		txt = new TextAreaPropertyEditor();
		panel = new JPanel( new BorderLayout() );
		lm = new DefaultListModel();
		jl = new JList(lm);
		JScrollPane jsp = new JScrollPane( jl, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );

		jl.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		jsp.setBorder(BorderFactory.createEtchedBorder(Color.gray, Color.white));
		jsp.setPreferredSize(new Dimension(300, 100));

		panel.add( jsp, BorderLayout.NORTH );
		panel.add( txt.getCustomEditor(), BorderLayout.SOUTH );

		jl.addListSelectionListener( new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e)
			{
				try
				{
					DatabaseMetaData metadata = connection.getMetaData();
					ResultSet resultSet = metadata.getColumns(null, null, (String) jl.getSelectedValue(), null);
					String columns= "";
					while (resultSet.next())
					{
						if( columns.equals("") )
						{
							columns = ""+resultSet.getString("COLUMN_NAME");
						}
						else
						{
							columns = columns+", "+resultSet.getString("COLUMN_NAME");
						}
					}

					txt.setValue( "select "+columns+" from "+ jl.getSelectedValue() );
				}
				catch(Exception ex)
				{
					txt.setValue( "select * from "+ jl.getSelectedValue() );
				}
			}
		} );

		// Get table names
		DatabaseMetaData meta = conn.getMetaData();
		ResultSet tables = meta.getTables(null, null, null, new String[] {"TABLE"});

		// Add table names to list
		while (tables.next())
		{
			lm.addElement(tables.getString("TABLE_NAME"));
		}
		tables.close();
	}

	public JPanel getPanel()
	{
		return panel;
	}

	public String getText()
	{
		return txt.getAsText();
	}
}
