// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.gui;
import fr.unicaen.openmap.dataStorage.SelectionMatrix;
import java.util.ArrayList;
import javax.swing.table.*;


/**
 * Link between graphical table display and the 'real' data.
 */
public class SelectionColumnMatrixModel extends AbstractTableModel {
    /**
     * Associated matrix.
     */
    private SelectionMatrix matrix = null;
    private TableModel datas = null;
    private int selection = -1;
    private ArrayList<Integer> sumColumns = null;
    private ArrayList<Integer> columnIndexToSum = null;
    
    public SelectionColumnMatrixModel(TableModel datas, SelectionMatrix matrix, int currentSubSelection, ArrayList<Integer> columnToSum) {
        this.matrix = matrix;
        this.datas = datas;
        this.selection = currentSubSelection;
        
        if (columnToSum!=null) this.columnIndexToSum=columnToSum;
        else columnIndexToSum=new ArrayList<Integer>();
        
        this.sumColumns=new ArrayList<Integer>();
        
        // add the sum for all the cols selected.
        for(int i=0;i<columnIndexToSum.size();i++){
            sumColumns.add(sommeCols(columnIndexToSum.get(i)));
        }
    }
    
    // the number of columns is numbre of data ...
    public int getColumnCount() { return datas.getColumnCount(); }
    public int getRowCount() { return matrix.getNbBoolRows(selection)+2; }
    public Object getValueAt(int row, int col) {
        if ( row >= matrix.getNbBoolRows(selection)){
            // une ligne vide
            if (row == matrix.getNbBoolRows(selection)) return null;
            //  we have to set the value.
            if (col==0) return "Total";
            int pos = columnIndexToSum.indexOf(col);
            if (pos!=-1) return sumColumns.get(pos);
            return null;
        } else  if (col<datas.getColumnCount()){
            int ligne = matrix.getBoolRows(selection,row);
            return datas.getValueAt(ligne, col);
        }
        return null;
    }
    public String getColumnName(int column) { return (String) datas.getColumnName(column); }
    public Class getColumnClass(int c) {
        if (c<datas.getColumnCount()) return datas.getValueAt(0,c).getClass();
        return null;
    }
    
    public boolean isCellEditable(int row, int col) { return false; }
    public void setValueAt(Object aValue, int row, int col) {
        if ( row >= matrix.getNbBoolRows(selection)){
            // nothing to do
        } else   if (col<datas.getColumnCount()){
            int ligne = matrix.getBoolRows(selection,row);
            datas.setValueAt(aValue,ligne,col);
            fireTableDataChanged();
        }
    }
    private int sommeCols(int col){
        int total=0;
        for (int i=0;i<getRealRowCount();i++) {
            if (getValueAt(i,col) !=null) {
                if (getValueAt(i,col) instanceof Double) total+= ((Double) getValueAt(i,col)).intValue();
                else if (getValueAt(i,col) instanceof String) total+= Integer.parseInt((String)getValueAt(i,col));
            }
        }
//            total+= (new Double ((String)getValueAt(i,col))).intValue();
        return total;
    }
    /**
     * Return the indexrow in the real TableModel.
     **/
    public int getRealRow(int row){
        return matrix.getBoolRows(selection,row);
    }
    
    public int getRealRowCount() { return matrix.getNbBoolRows(selection); }
    
    public void fireTableRowsInserted(int firstRow, int lastRow) {
        super.fireTableRowsInserted(firstRow,lastRow);        
        for (int i=firstRow;i<=lastRow;i++)
            // add the sum for all the cols selected.
            for(int j=0;j<columnIndexToSum.size();j++)
                sumColumns.set(j,sumColumns.get(j)+getIntValue(datas.getValueAt(i,columnIndexToSum.get(j))));        
    }
    
    private int getIntValue(Object obj){
	try {
        //    System.out.println("Try to get integer from "+obj);
        if (obj instanceof String) return  new Integer((String)obj);
        if (obj instanceof Double) return ((Double)obj).intValue();
        if (obj instanceof Integer) return (Integer) obj;
        return 0;
	} catch (Exception e) { return 0; }
    }
    
    public void fireTableRowsDeleted(int firstRow, int lastRow) {
        super.fireTableRowsDeleted(firstRow, lastRow);
        
        for (int i=firstRow;i<=lastRow;i++)
            // add the sum for all the cols selected.
            for(int j=0;j<columnIndexToSum.size();j++)
                sumColumns.set(j,sumColumns.get(j)-getIntValue(datas.getValueAt(i,columnIndexToSum.get(j))));        
    }
    
    public void fireTableStructureChanged() {
        super.fireTableStructureChanged();
        for(int j=0;j<columnIndexToSum.size();j++){
            sumColumns.set(j,sommeCols(columnIndexToSum.get(j)));
        }
        
    }
    
    public void setSubSelectionValue(int subSelectionValue){
        this.selection = subSelectionValue;
    }       
}
