// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.gui;
import fr.unicaen.openmap.dataStorage.SelectionMatrix;
import javax.swing.table.*;


/**
 * Link between graphical table display and the 'real' data.
 */
public class SelectionDataMatrixModel extends AbstractTableModel {
    /**
     * Associated matrix.
     */
    private SelectionMatrix matrix = null;
    private TableModel datas = null;
    private int selection = -1;
    
    public SelectionDataMatrixModel(TableModel datas, SelectionMatrix matrix, int currentSubSelection) {
        this.matrix = matrix;
        this.datas = datas;
        this.selection = currentSubSelection;
    }
    
    // the number of columns is numbre of data + one selection ...
    public int getColumnCount() { return datas.getColumnCount(); }
    public int getRowCount() { return matrix.getNbBoolRows(selection); }
    public Object getValueAt(int row, int col) {
        if (col<datas.getColumnCount()){
            int ligne = matrix.getBoolRows(selection,row);
            return datas.getValueAt(ligne, col);
        }       
        return null;
    }
    // this give us the ability to find the column name of the table 
    public String getColumnName(int column) { return (String) datas.getColumnName(column); }
    public Class getColumnClass(int c) { return getValueAt(0, c).getClass(); }
    public boolean isCellEditable(int row, int col) { return true; }
    public void setValueAt(Object aValue, int row, int col) {
        if (col<datas.getColumnCount()){
            int ligne = matrix.getBoolRows(selection,row);
            datas.setValueAt(aValue,ligne,col);
            fireTableDataChanged();
        }
    }
    
    /*
     * We can change the datas. So we notifie all listeners that Structure as change.
     */
    public void setMatrix(SelectionMatrix theMatrix) {
        this.matrix=theMatrix;
        fireTableStructureChanged();
    }
    
    public int getCurrentSubSelection(){
        return selection;
    }
    
    public void setCurrentSubSelection(int selection){
        if (selection<matrix.getNbCols()) this.selection=selection;
    }
}
