// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * SelectionMatrixPanel.java
 *
 * Created on 9 novembre 2005, 23:31
 *
 * @author Christophe.Turbout@info.unicaen.fr
 *
 */

package fr.unicaen.openmap.dataStorage.gui;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;
import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.dataStorage.SelectionDataStorage;
import fr.unicaen.openmap.dataStorage.SelectionDataStorageListener;
import fr.unicaen.openmap.layer.preLayer.UnicaenLayerListener;
import fr.unicaen.openmap.util.Util;
import java.awt.*;
import java.util.Locale;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


/**
 * This class is used to display the content of Vector<SelectionMatrix>.
 *
 */
public class SelectionMatrixPanel extends JPanel implements SelectionDataStorageListener, ChangeListener {
    private JTabbedPane tabs = null;
    private SelectionDataStorage selectionDataStorage = null;
    private UnicaenLayerListener unicaenLayerListener = null;
    // protected int selectedIndex = -1;
    
    /**
     * Use I18N mecanism to allow internationalization.
     * Use I18N.properties file or I18N_fr_FR.properties for french version
     **/
    protected I18n i18n = new BasicI18n(new Locale("fr","FR"));
    
    public  SelectionMatrixPanel(SelectionDataStorage selectionDataStorage){
        super();
        this.selectionDataStorage = selectionDataStorage;
        setDoubleBuffered(true);
        setLayout(new BorderLayout());
        // le JTabpanned
        tabs = new CustomTabbedPane(CustomTabbedPane.ALL_OPTIONS,selectionDataStorage);
        tabs.addChangeListener(this);
        add(tabs, BorderLayout.CENTER);
        // the components
        
        addComponents();
    }
    
    private void addComponents(){
        try {
            for (int i=0;i<selectionDataStorage.getSelectionsSize();i++){
                SelectionColumnMatrixPanel scmp = new SelectionColumnMatrixPanel(selectionDataStorage,i);
                tabs.addTab(selectionDataStorage.getSelectionName(i),scmp);
            }
        } catch (Exception e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
    }
 /*
  * We have to update panel, data structure has changed. This method is used when the structure is not the same which
  * means a column are a row has been added or suppressed.
  *
  **/
    public void fireStructureChanged(){
        // first we remove all column ...
        tabs.removeAll();
        // second we add all ...
        addComponents();
    }
    
    /**
     * Add a new column to selection Matrix => we need to update the good selection.
     **/
    
    public void fireColumnAdded(int selection, int subselection){
        ((SelectionColumnMatrixPanel) tabs.getComponentAt(selection)).fireColumnAdded(selection,subselection);
    }
    
    public void fireColumnRemoved(int selection, int subselection){
        ((SelectionColumnMatrixPanel) tabs.getComponentAt(selection)).fireColumnRemoved(selection,subselection);
    }
    
    public void fireDataChanged(int selection, int subselection, int row, boolean state){
        ((SelectionColumnMatrixPanel) tabs.getComponentAt(selection)).fireDataChanged(selection,subselection,row,state);
    }
    
    public void renameTab(int selection, String name){
        tabs.setTitleAt(selection,name);
    }
    
    
    /**
     * When state is changed, we need to update SelectionDataStorage to set the currentSelection.
     **/
    public void stateChanged(ChangeEvent e) {
        Debug.message(Util.className(this), " SelectionMatrixPanel init, current Index : " +selectionDataStorage.getCurrentSelection());
        JTabbedPane jpane = ((JTabbedPane)e.getSource());
        int selectedTab = jpane.getSelectedIndex();
        Debug.message(Util.className(this),  " SelectionMatrixPanel modif : "+selectedTab);
        if (selectionDataStorage!=null && selectedTab>-1 && selectionDataStorage.getCurrentSelection()!=selectedTab) {
            selectionDataStorage.setActiveSelection(selectedTab);
            JTabbedPane pane = (JTabbedPane) getParent();
            if (pane!=null) {
                SelectionStoragePanel parentPanel = (SelectionStoragePanel) pane.getParent();
                if (parentPanel!=null) parentPanel.setActiveSelection(selectedTab);
            }
            if (unicaenLayerListener!=null) unicaenLayerListener.refreshGraphics();
        }
    }
    /**
     *  Add UnicaenLayerListener to refresh layer();
     */
    public void addUnicaenListener(UnicaenLayerListener ull){
        unicaenLayerListener=ull;
    }
    
    /**
     *  Remove the UnicaenLayerListener;
     */
    public void removeUnicaenListener(){
        unicaenLayerListener=null;
    }
    
    /**
     * Selection the selection pane in JTabbedPane.
     */
    public void setActiveSelection(int selection){
        if (tabs.getSelectedIndex()!=selection) tabs.setSelectedIndex(selection);
    }
    
    public void fireUpdateColors(){
        for(int i=0; i< tabs.getTabCount(); i++){
            ((SelectionColumnMatrixPanel)   tabs.getComponentAt(i)).fireUpdateColors(i);
        }
    }

    public UnicaenLayerListener getUnicaenLayerListener() {
        return unicaenLayerListener;
    }
}
