// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.kml;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.util.Iterator;

import org.apache.batik.dom.util.DOMUtilities;
import org.w3c.dom.Element;

import com.bbn.openmap.Layer;
import com.bbn.openmap.layer.OMGraphicHandlerLayer;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.proj.coords.LatLonPoint;


import fr.unicaen.openmap.dataStorage.BoundingBox;


import fr.unicaen.openmap.util.AbstractTextFormatter;
import java.awt.Color;

public class KMLExporter extends AbstractTextFormatter {

	public static Projection CURRENT_PROJECTION;
	public static int NB_FIG = 0;
	
	public final static int EXPORT_DOM = 1;
	
	public final static int EXPORT_STAX = 2;
	
	private final String kmlNS = "http://earth.google.com/kml/2.1";

	private KMLDocument document;
	
	private KMLWriter kmlWriter;

	private int exportMode;

	public KMLExporter() {
		exportMode = EXPORT_STAX;
	}

	public String getFormatLabel() { return KMLConstants.IMAGEFORMAT_KML; }
	
	
	private void addList(Element parent, OMGraphicList omgl) {
		//System.out.println("taille: "+omgl.size());
	    Iterator it = omgl.iterator();
	    Object o = null;
		//Element g = document.createElementNS(kmlNS, "g");
	    while (it.hasNext()) {
		o = it.next();
		if (o instanceof OMGraphicList) {
		    addList(parent, (OMGraphicList) o);
		} else if (o instanceof OMGraphic)
		    addGraphic(parent, (OMGraphic) o);
	    }
	//parent.appendChild(parent);
	}

	private void addGraphic(Element parent, OMGraphic omg) {
		//KmlGraphics2D tmp = new KmlGraphics2D(document);
		//omg.setRenderType(OMGraphic.RENDERTYPE_LATLON);
		//omg.render(tmp);
             	Element element = KMLFactory.createKmlElement(omg,document);
		//omg.render(tmp);
		//Element e = tmp.getTopLevelGroup();
		//e.setAttributeNS(null, "id", null);
		//System.out.println(tmp.getTopLevelGroup().getAttributes());                                                                                     ;
		//parent.appendChild(tmp.getTopLevelGroup());
		if(element!=null)
                    document.getDocRoot().appendChild(element);
	}

	public byte[] getTextFromLayers(Layer[] layers, int w, int h, Color bgColor){
		/*DOMImplementation domImpl = GenericDOMImplementation.getDOMImplementation();
		document = domImpl.createDocument(kmlNS, "kml", null);
		kmlRoot = document.getDocumentElement();
		kmlRoot.setAttributeNS(null, "xmlns", kmlNS);
		docRoot = document.createElement("Document");
		kmlRoot.appendChild(docRoot);*/

		document = new KMLDocument();
		kmlWriter = new KMLWriter();
		if(exportMode == EXPORT_STAX)
			kmlWriter.startWrite();
			for (int i = layers.length -1 ; i >= 0 ; i--) {
			//for (int i = 0 ; i<layers.length ; i++) {
			Layer layer = layers[i];
			CURRENT_PROJECTION = layer.getProjection();
			if (layer instanceof OMGraphicHandlerLayer) {
				switch(exportMode){
				case EXPORT_DOM:
				addList(document.getKmlRoot(),
						((OMGraphicHandlerLayer) layer).getList());
				break;
				
				case EXPORT_STAX:
					OMGraphicList omgl = new OMGraphicList();
					//OMGraphicList omglTemp = ((OMGraphicHandlerLayer) layer).getList();
					BoundingBox bb = new BoundingBox(
									(LatLonPoint)(layer.getProjection().getUpperLeft()), 
									(LatLonPoint)(layer.getProjection().getLowerRight()));
					/*for(int j = 0; j < omglTemp.size(); j++){
						OMGraphic omg = omglTemp.getOMGraphicAt(j);
						//BoundingBox  tempBB = new BoundingBox(
								omg.,
								omg.getSh);
						if(bb.intersects())
						//System.out.println(bb.contains(omg.getShape());
					}*/
                                        //omgl = ((OMGraphicHandlerLayer) layer).getList();
                                        //omgl.re
					exportOMGraphicList(((OMGraphicHandlerLayer) layer).getList(), kmlWriter);

					break;
				}
			} 
		    

	    }
		switch(exportMode){
		case EXPORT_DOM: return endDOMWriting();
		case EXPORT_STAX: 
			kmlWriter.writeEndDocument();
			return endSTAXWriting();
		}
		return new byte[0];
	}
	
	private byte[] endDOMWriting(){
		 ByteArrayOutputStream stream = new ByteArrayOutputStream();
		   OutputStreamWriter writer;
			try {
			    writer = new OutputStreamWriter(stream, KMLConstants.ENCODING);
			    writer.write("<?xml version='1.0' encoding='utf-8'?>");
			    DOMUtilities.writeDocument(document.getDocument(),writer);
			    writer.flush();
			    //System.out.println("FIN DE l'EXPORT");
			    return stream.toByteArray();
			} 
			catch (UnsupportedEncodingException e) {e.printStackTrace();}
			catch(IOException ee){ee.printStackTrace();}
			return new byte[0];
	}
	
	private byte[] endSTAXWriting(){
		
		 ByteArrayOutputStream stream = new ByteArrayOutputStream();
		   OutputStreamWriter writer;
			try {
				writer = new OutputStreamWriter(stream, KMLConstants.ENCODING);
			    writer.write(kmlWriter.getText());
			    writer.flush();
			    return stream.toByteArray();
			} 
			catch (UnsupportedEncodingException e) {e.printStackTrace();}
			catch(IOException ee){ee.printStackTrace();}
			return new byte[0];
	}
	
	public void exportOMGraphicList(OMGraphicList omgl, KMLWriter writer){

		  //Iterator it = omgl.iterator();
		    Object o = null;
			//Element g = document.createElementNS(kmlNS, "g");
		    //while (it.hasNext()) {
                    for(int i = omgl.size() -1; i >= 0; i--){
			//o = it.next();
                        o = omgl.getOMGraphicAt(i);
			if (o instanceof OMGraphicList) {
				//System.out.println("------1-----");
				//System.out.println("ExportOmgraphicList: c'est un omgraphiclist");
			    exportOMGraphicList((OMGraphicList) o, writer);
			    } 
			else if (o instanceof OMGraphic){
				//System.out.println("------2-----");
				//System.out.println("ExportOmgraphicList: c'est un omgraphic");
			    exportOMGraphic((OMGraphic)o, writer);}
			
		    }
		 
	}
	
	public void exportOMGraphic(OMGraphic omg, KMLWriter writer){
                //System.out.println("EXPORT OMGRAPHIC " + omg.getClass());
		KMLFactory.exportOmgWithStax(omg,writer);		
	}

	public void createKmlDocument() {
	}

}
