// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.kml;

import java.io.StringWriter;
import java.util.ArrayList;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

public class KMLWriter {

	//private XMLStreamWriter mainWriter;

	private XMLStreamWriter styleWriter;

	StringWriter styleString, mainString;

	private String namespaceKML;

	//private String fileName;

	private ArrayList<String> colors;

        private ArrayList<XMLStreamWriter> writers = new ArrayList<XMLStreamWriter>();

//	public KMLWriter(/*String fileName*/) {
//		//this.fileName = fileName;
//		namespaceKML = "http://earth.google.com/kml/2.2";
//		mainString = new StringWriter();
//		styleString = new StringWriter();
//		colors = new ArrayList<String>();
//		try {
//			XMLOutputFactory output1 = XMLOutputFactory.newInstance();
//			XMLOutputFactory output2 = XMLOutputFactory.newInstance();
//			mainWriter = output1.createXMLStreamWriter(mainString);
//			styleWriter = output2.createXMLStreamWriter(styleString);
//		} catch (XMLStreamException e) {
//			e.printStackTrace();
//		}
//	}

	public KMLWriter() {
		namespaceKML = "http://earth.google.com/kml/2.2";
		mainString = new StringWriter();
		styleString = new StringWriter();
		colors = new ArrayList<String>();
		try {
			//XMLOutputFactory output1 = XMLOutputFactory.newInstance();
			XMLOutputFactory output2 = XMLOutputFactory.newInstance();
			//mainWriter = output1.createXMLStreamWriter(/*mainString*/System.out);
			styleWriter = output2.createXMLStreamWriter(styleString);
		} catch (XMLStreamException e) {
			e.printStackTrace();
		}
	}

	public void startWrite() {
		try {
			styleWriter.writeStartDocument();
			styleWriter.writeCharacters("\n");
			styleWriter.setPrefix("kml", namespaceKML);
			styleWriter.writeStartElement("kml");
			styleWriter.writeNamespace("kml", namespaceKML);
			styleWriter.writeCharacters("\n");
			styleWriter.writeStartElement("Document");
                        //styleWriter.flush();


		} catch (XMLStreamException e) {
			e.printStackTrace();
		}

	}

	public void writeStyle(String id, String color) {
		try {
			if (!colors.contains(color)) {
				colors.add(color);
				styleWriter.writeStartElement("Style");
				styleWriter.writeAttribute("id", id);
				styleWriter.writeCharacters("\n");
				styleWriter.writeStartElement("PolyStyle");
				styleWriter.writeCharacters("\n");
				styleWriter.writeStartElement("color");
				styleWriter.writeCharacters(color);
				styleWriter.writeEndElement();// fin color
				styleWriter.writeCharacters("\n");
				styleWriter.writeEndElement();// fin polystyle
				styleWriter.writeCharacters("\n");
				styleWriter.writeEndElement();// fin style
			}
		} catch (XMLStreamException e) {
			e.printStackTrace();
		}
	}

	public void writePlacemarkDescription(String name, String description,
			String coords) {
		try {
                    XMLOutputFactory output1 = XMLOutputFactory.newInstance();
                    XMLStreamWriter mainWriter = output1.createXMLStreamWriter(mainString);

			mainWriter.writeStartElement("Placemark");
			mainWriter.writeCharacters("\n");
			mainWriter.writeStartElement("name");
			mainWriter.writeCharacters(name);
			mainWriter.writeEndElement();// fin name
			mainWriter.writeCharacters("\n");

			// description
			mainWriter.writeStartElement("description");
			mainWriter.writeCharacters(description);
			mainWriter.writeEndElement();// fin description
			mainWriter.writeCharacters("\n");

			// point
			mainWriter.writeStartElement("Point");
			mainWriter.writeCharacters("\n");
			mainWriter.writeStartElement("coordinates");
			mainWriter.writeCharacters(coords);
			mainWriter.writeEndElement();// fin coords
			mainWriter.writeCharacters("\n");
			mainWriter.writeEndElement();// fin Point
			mainWriter.writeCharacters("\n");

			mainWriter.writeEndElement();// fin Placemark
			mainWriter.writeCharacters("\n");

                        mainWriter.flush();

		} catch (XMLStreamException e) {
			e.printStackTrace();
		}

	}

	public void writePolygon(String name, String refStyle, String Coords) {
		// System.out.println("WRITE POLY");
		try {
                    XMLOutputFactory output1 = XMLOutputFactory.newInstance();
                    XMLStreamWriter mainWriter = output1.createXMLStreamWriter(mainString);

			mainWriter.writeStartElement("Placemark");
			mainWriter.writeCharacters("\n");
			mainWriter.writeStartElement("name");
			mainWriter.writeCharacters(name);
			mainWriter.writeEndElement();// fin name
			mainWriter.writeCharacters("\n");

			// style
			mainWriter.writeStartElement("styleUrl");
			mainWriter.writeCharacters(refStyle);
			mainWriter.writeEndElement();// fin style
			mainWriter.writeCharacters("\n");

			// polygon
			mainWriter.writeStartElement("Polygon");
			mainWriter.writeCharacters("\n");
			mainWriter.writeStartElement("outerBoundaryIs");
			mainWriter.writeCharacters("\n");
			mainWriter.writeStartElement("LinearRing");
			mainWriter.writeCharacters("\n");
			mainWriter.writeStartElement("coordinates");
			mainWriter.writeCharacters(Coords);
			mainWriter.writeEndElement();// fin Coords
			mainWriter.writeCharacters("\n");
			mainWriter.writeEndElement();// fin linearring
			mainWriter.writeCharacters("\n");
			mainWriter.writeEndElement();// fin outerBound
			mainWriter.writeCharacters("\n");
			mainWriter.writeEndElement();// fin Poly
			mainWriter.writeCharacters("\n");

			mainWriter.writeEndElement();// fin Placemark
			mainWriter.writeCharacters("\n");

                        mainWriter.flush();

		} catch (XMLStreamException e) {
			e.printStackTrace();
		}

	}

	public void writeEndDocument() {

		try {
			styleWriter.flush();
			styleString.append(mainString.getBuffer());
			styleWriter.writeCharacters("\n");
			styleWriter.writeEndElement();// fin document
			styleWriter.writeCharacters("\n");
			styleWriter.writeEndElement();// fin kml
			styleWriter.writeEndDocument();
			styleWriter.flush();
			/*
			 * FileWriter fw = new FileWriter(fileName);
			 * fw.append(styleString.getBuffer());
			 * 
			 * mainWriter.close(); styleWriter.close();
			 */
		} catch (XMLStreamException e) {
			e.printStackTrace();
		}
		// catch(IOException eio){eio.printStackTrace();}
	}

	public String getText() {
		return styleString.getBuffer().toString();
	}

	public XMLStreamWriter getStyleWriter() {
		return styleWriter;
	}

//	public XMLStreamWriter getMainWriter() {
//		return mainWriter;
//	}

}
