// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.mif;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author jchauvea
 */
public class MidParser {

    //private String path;

    private String mifPath;

    private String midPath;

    private ArrayList<String> columnNames;

    private ArrayList<String> OMGraphicsNames;

    public final static String MID_EXTENSION = ".mid";

    public final static String MIF_EXTENSION = ".mif";

    public final static String MID_FIELD_SEPARATOR = ";";

    public final static String COLUMNS_PATTERN = "COLUMNS";

    public MidParser(String path) {
        //this.path = path;
        mifPath = path + MIF_EXTENSION;
        midPath = path + MID_EXTENSION;
        columnNames = new ArrayList<String>();
        OMGraphicsNames = new ArrayList<String>();
        OMGraphicsNames.add("the_geom");
    }

    public ArrayList<String> getColumnNames(){
        return columnNames;
    }

    public ArrayList<String> getOMGraphicsNames(){
        return OMGraphicsNames;
    }

    public ArrayList<ArrayList<Object>> parse() {
        parseColumnNames();
        ArrayList<ArrayList<Object>> midList = new ArrayList<ArrayList<Object>>();
        try {
            //System.out.println("MID PARSER -> " + midPath);
            BufferedReader in = new BufferedReader(new FileReader(midPath));
            String line;
            String[] tab;
            ArrayList<Object> tmpList;
            //boolean firstStep = true;
            while ((line = in.readLine()) != null) {
                tab = line.split(MID_FIELD_SEPARATOR);
                tmpList = new ArrayList<Object>();
                for(int i = 0 ; i < tab.length ; i++){
                    /*if(firstStep){
                        //String.valueOf(i)
                        columnNames.add(String.valueOf(i));
                    }*/
                    //System.out.println(tab[i]);
                    tmpList.add(tab[i]);
                }
                //firstStep = false;
                midList.add(tmpList);
            }
            in.close();
        } catch (IOException io) {
        }

        return midList;
    }

//    public ArrayList<ArrayList<Object>> parse() {
//
//    }
    private void parseColumnNames(){
        try {
            BufferedReader in = new BufferedReader(new FileReader(mifPath));
            String line;
            while ((line = in.readLine()) != null) {
                if(line.startsWith(COLUMNS_PATTERN))
                    break;
                else
                    continue;
            }
            int nbCols = Integer.parseInt(line.split(" ")[1]);
            int i = 0;
            while( i < nbCols){
                line = in.readLine();
                //System.out.println("line = " + line);
                 columnNames.add(line);
                 //System.out.println("col =" +line.split(" ")[1]);
                 i++;
            }
        } catch (Exception ex) {
            Logger.getLogger(MidParser.class.getName()).log(Level.SEVERE, null, ex);
        }

        

    }
//     public ArrayList<ArrayList<Object>> parseInMIF() {
//
//
//     }

}
