// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.mif;

import com.bbn.openmap.layer.mif.MIFLoader;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.dataStorage.DataObject;
import fr.unicaen.openmap.dataStorage.DataStorageListener;
import fr.unicaen.openmap.dataStorage.InformationDataStorage;
import fr.unicaen.openmap.dataStorage.OMDataLocation;
import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import fr.unicaen.openmap.util.propertyEditor.FileTypeDescription;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JComponent;
import javax.swing.JLabel;
/*import org.geotools.data.FeatureReader;
import org.geotools.data.mif.MIFFile;
import org.opengis.feature.Feature;
import org.opengis.feature.Property;
import org.opengis.feature.type.FeatureType;*/

/**
 *DataStorage for MIF (Mapinfo Interchange Format) files
 * @author jchauvea
 */
public class MifDataStorage extends InformationDataStorage {

    private String mifFile = "";
    public static final String MIF_BY_INFO = "mif.info";
    private static final String mifFilePropertyString = "mifFile";
    public static final java.lang.String PARAM_MIF = "mif";
    private MidParser midParser;

    public MifDataStorage(DataStorageListener dsl) {
        super(dsl);
    }


    public String getName() {
        return "".equals(mifFile) ? name : stripFileNameWithClassPathDirectories(mifFile);
    }

    /**
     * set the MidDataStorage properties
     * @param prefix
     * @param properties
     */
    @Override
    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        mifFile = properties.getProperty(prefix + mifFilePropertyString,
                properties.getProperty(prefix + MIF_BY_INFO));
        initialize();
    }

    @Override
    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);

        properties.put(prefix + mifFilePropertyString, stripFileNameWithClassPathDirectories(mifFile));

        // Used with getPropertyInfo
        properties.put(prefix + MIF_BY_INFO,
                new FileTypeDescription(FileTypeDescription.MIF_DESCRIPTION,
                mifFile, FileTypeDescription.MIF_PATTERN));

        return properties;
    }

    @Override
    public Properties getPropertyInfo(Properties properties) {
        properties = super.getPropertyInfo(properties);
        PropUtils.setI18NPropertyInfo(i18n,
                properties,
                MifDataStorage.class,
                MIF_BY_INFO,
                "File",
                "Select a MIF file.",
                "fr.unicaen.openmap.util.propertyEditor.FileTypePropertyEditor");

        properties.put(initPropertiesProperty,
                MIF_BY_INFO);
        return properties;
    }

    /**
     * initalize the dataStorage by loading data from mif file
     */
    protected void initialize() {

//        HashMap params = new HashMap();
//        ArrayList<DataObject> dataObjects = new ArrayList<DataObject>();
//        ArrayList<String> omGraphicNames = new ArrayList<String>();
//        System.out.println("-----------------initialize--------------------" + mifFile);
//        try {
//            MIFFile mf = new MIFFile(mifFile/*"/export/home/jchauvea/Bureau/popdens/Pop_Dens.MIF"*/, params);
//            FeatureType ft = mf.getSchema();
//            FeatureReader fr = mf.getFeatureReader();
//            System.out.println(params.size());
//            while (fr.hasNext()) {
//                System.out.println("***");
//                Feature in = fr.next();
//                Collection<Property> col = (Collection<Property>) in.getValue();
//                DataObject dataObject = new DataObject();
//                //System.out.println("");
//                for (Property prop : col) {
//                    //dataObject.addData(prop.getName().toString(), prop.getValue());
//                    System.out.println(prop.getName() + ";" + prop.getValue());
//                }
//                dataObjects.add(dataObject);
//                //System.out.println(in);
//            }
//            fr.close();
//
//        } catch (IOException ex) {
//            ex.printStackTrace();
//        }
//
        try {
            MIFLoader mifLoader = new MIFLoader(new BufferedReader(new FileReader(new File(mifFile))), true, 100000f, 100000f);
            OMGraphicList omgl = mifLoader.getList();
            //String midFile = mifFile.replaceAll(".mif", ".mid");
            midParser = new MidParser(mifFile.replaceAll(".mif", ""));
            ArrayList<DataObject> dataObjects = 
                    buildDataObjectsFromOMGraphics(omgl, midParser.parse());

            dataTable.setTable(dataObjects,
                    getDataAttributesNamesToDisplay(midParser.getColumnNames()),
                    midParser.getOMGraphicsNames(),
                    getDataAttributesNamesAliases());
            dataTableModel = new DataTableModel(dataTable);
            resetDataTableStoragePanel(dataTableModel);

            fireDataStorageEvent();
            //initialized = true;
        } catch (FileNotFoundException ex) {
            Logger.getLogger(MifDataStorage.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * Test if the dataStorage is ready(return true) or not
     * @return
     */
    @Override
    public boolean isReady() {
        return true;
    }

    public ArrayList<DataObject> buildDataObjectsFromOMGraphics(OMGraphicList omgl, ArrayList<ArrayList<Object>> midList){
        ArrayList<DataObject> dataObjects = new ArrayList<DataObject>();
        ArrayList<String> colNames = midParser.getColumnNames();
        int i = 0;
        for(OMGraphic omg : omgl){
            ArrayList<Object> midLine = midList.get(i);
            DataObject dataObject = new DataObject();
            for(int j = 0; j < midLine.size(); j++){
                dataObject.addData(colNames.get(j), midLine.get(j));
            }
            dataObject.addOMDataLocation("the_geom", new OMDataLocation(omg));
            i++;
            dataObjects.add(dataObject);
        }

        return dataObjects;
    }

    @Override
    public JComponent getGUI() {
        return new JLabel("MIF");
    }
}
