// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.out;

import java.io.FileWriter;
import java.io.BufferedWriter;
import java.io.PrintWriter;
import java.io.IOException;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.zip.GZIPOutputStream;
import java.util.regex.Pattern;

public class CSVWriter {

    private BufferedWriter writer;
    private String fileName;
    private String separator = ";";
    private Pattern spaceOrSeparator= Pattern.compile(".*[ \t"+separator+"].*");
    private boolean zip;
    private boolean previousCallIsNewLine = true;

    public CSVWriter(String fileName, boolean zip) {
        this.fileName = fileName;
        this.zip = zip;	
    }

    public void startWrite() {
        try {
            writer = zip
		? new BufferedWriter(new PrintWriter(new GZIPOutputStream(new FileOutputStream(fileName))))
		: new BufferedWriter(new FileWriter(fileName));
	} catch (IOException e) {
            e.printStackTrace();
	}
    }

    /** Write the data and add a separator in case previous call was not EndOfLine */ 
    public void writeData(Object data) {
        try {
	    if (previousCallIsNewLine)
		previousCallIsNewLine = false;
	    else
		writer.write(separator);
            if (data != null) {
		String d = data.toString();
		if (spaceOrSeparator.matcher(d).matches())
		    writer.write("\""+data.toString()+"\"");
		else
		    writer.write(data.toString());
	    }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void newLine() {
	previousCallIsNewLine = true;
	try {
	    writer.newLine();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void close() {
        try {
            writer.flush();
            writer.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
