// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.postgis;

import fr.unicaen.openmap.event.GUIChangeEvent;
import fr.unicaen.openmap.util.Util;

import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.dataStorage.gui.*;
import fr.unicaen.openmap.dataStorage.postgis.db.*;
import fr.unicaen.openmap.dataStorage.postgis.gui.*;
import java.util.Properties;
import javax.swing.*;
import com.bbn.openmap.util.*;
import fr.unicaen.openmap.event.GUIChangeListener;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.util.*;

/**
 * Example of a data storage : Postgis data access.
 * This class is part of our new ompg (&openmap) architecture.
 * It is mainly a copy/paste from my old PostgisLayer ...
 * TODO: complete rewrite ...
 *
 * This class implements GUIChangeListener but it's not used by its GUI.
 * In fact, underlying GUI components (query ONLY) already know
 * how to tell the PostgisDataStorage it must reload data.
 */
public class PostgisDataStorage extends InformationDataStorage implements DBListener, GUIChangeListener {


    public final static int CONNECTING = 0x02;
    public final static int CONNECT_OK = 0x04;
    public final static int CONNECT_ERROR = 0x08;
    public final static int QUERYING = 0x10;
    public final static int QUERY_OK = 0x11;
    public final static int QUERY_ERROR = 0x12;

        public enum DBState {

        CONNECTING(0x02, "connecting"),
        CONNECT_OK(2, "Connection ok"),
        QUERY_OK(2, "Query ok"),
        QUERYING(2, "Querying"),
        QUERY_ERROR(2, "Query error"),
        CONNECT_ERROR(3, "Connection error");

        protected int id;
        protected String label;

        DBState(int id, String label) {
            this.id = id;
            this.label = label;
        }

        public int getId() {
            return this.id;
        }

        public String getLabel() {
            return this.label;
        }
    }

    protected String name = "unnamed";

    public String getName() {
        return name;
    }

    public String toString() {
        return "[PostgisDataStorage:" + getName() + "]";
    }

    public PostgisDataStorage(DataStorageListener dsl) {
        super(dsl);
        Debug.message("PostgisDataStorage", this + " PostgisDataStorage(DataStorageListener dsl)");
    }
    protected boolean initialized = false;

    public boolean isReady() {
        /*try{
        throw new Exception();
        }
        catch(Exception e){e.printStackTrace();}*/
        //Debug.message("PostgisDataStorage", " Ready ? " + initialized);
        return initialized;
    }
    /**
     * Database Link.
     */
    protected DBConnection connection = new DBConnection();
    /**
     * Connection Thread.
     */
    protected DBConnectThread dbThread;
    /**
     * Thread to connect and query database.
     */
    protected DBQueryThread dbQueryThread;
    /**
     * Storage for data.
     */
    // protected DBTable table = new DBTable();
    protected ArrayList<DataObject> dataObjects = new ArrayList<DataObject>();
    /**
     * Parameters for database connection.
     */
    protected DBParams dbParams = new DBParams("", 0, "", "", "");
    protected String query = "";
    protected JComponent gui = null;
    /*
     * All information for database connectivity are provided through
     * properties interface.
     */
    public static final String serverHostPropertyString = "serverHost";
    public static final String serverPortPropertyString = "serverPort";
    public static final String databasePropertyString = "database";
    public static final String loginPropertyString = "login";
    public static final String passwordPropertyString = "password";
    public static final String queryPropertyString = "query";

    /**
     * This method must be called by the Layer enclosing this
     * DataStorage.
     */
    public void setProperties(String prefix, java.util.Properties properties) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        name = prefix;
        //	System.out.println("PostgisDataStorage name : "+name);
        //System.exit(0);
        Debug.message("PostgisDataStorage", this + " setProperties()");
        dbParams.addDBListerner(this);
        dbParams.setServer(properties.getProperty(prefix + serverHostPropertyString));
        dbParams.setPort(properties.getProperty(prefix + serverPortPropertyString));
        dbParams.setDatabase(properties.getProperty(prefix + databasePropertyString));
        dbParams.setLogin(properties.getProperty(prefix + loginPropertyString));
        dbParams.setPassword(properties.getProperty(prefix + passwordPropertyString));
        query = properties.getProperty(prefix + queryPropertyString);
        Debug.message("PostgisDataStorage", "Server name = " + dbParams.getServer());
        Debug.message("PostgisDataStorage", "Port number = " + dbParams.getPort());
        Debug.message("PostgisDataStorage", "DB login = " + dbParams.getLogin());
        Debug.message("PostgisDataStorage", "DB passwd = " + dbParams.getPassword());
        // Should we really do this here ?
        initialize();
    }

    //-------------------------------------------------------------------------To save
    public java.util.Properties getProperties(String prefix, Properties properties) {
        //Properties properties = new Properties();

        properties.put(prefix + serverHostPropertyString, dbParams.getServer());
        properties.put(prefix + serverPortPropertyString, dbParams.getPort());
        properties.put(prefix + databasePropertyString, dbParams.getDatabase());
        properties.put(prefix + loginPropertyString, dbParams.getLogin());
        properties.put(prefix + passwordPropertyString, dbParams.getPassword());
        properties.put(prefix + queryPropertyString, formatProperyString(query));

        return properties;
    }
    //-------------------------------------------------------------------------

    public Properties getPropertyInfo(Properties list) {

	list = super.getPropertyInfo(list);

	PropUtils.setI18NPropertyInfo(i18n,
				      list,
				      PostgisDataStorage.class,
				      queryPropertyString,
				      "Query", //label
				      "Query request", //tooltip
				      "fr.unicaen.openmap.util.propertyEditor.TextAreaPropertyEditor"
				      );
        list.put(initPropertiesProperty,
		 databasePropertyString + " " + loginPropertyString + " " +
		 passwordPropertyString + " " + serverHostPropertyString + " " +
                 serverPortPropertyString+ " " + queryPropertyString);

        return list;
    }

    private String formatProperyString(String s) {
        return s.replaceAll("\\n", "\\\\n");
    }

    public boolean refresh() {
        System.err.println("TODO: refresh() in " + Util.className(this));
        return true;
    }

    public int getCount() {
        return dataObjects.size();
    }

    public DataObject getElementAt(int i) {
        return dataObjects.get(i);
    }

    public JComponent getGUI() {
        if (gui == null) {
            gui = new PostgisDataStoragePanel(new QueryPanel(query, this), new DBParamsPanel(dbParams));
            gui.setBorder(BorderFactory.createLineBorder(Color.LIGHT_GRAY));
        
        }
        return gui;
    }

    /**
     *
     */
    public JComponent getDataGUI() {
        if (dtsp == null) {
            dtsp = new DataTableStoragePanel(this, dataTableModel);
        }
        return dtsp;
    }

    /**
     *
     */
    public void stateChanged(DBState state) {
        synchronized (this) {
            if (state  == DBState.QUERY_ERROR) {
                Debug.message("PostgisDataStorage", this + " Query Error ...");
                Debug.message("PostgisDataStorage", "Query = " + query);
                dbQueryThread = null;
            } else if (state == DBState.QUERY_OK) {
                Debug.message("PostgisDataStorage", this + " Query Ok ...");
                dataObjects = dbQueryThread.getDataObjects();
                ArrayList<String> columnNames = dbQueryThread.getColumnNames();
                ArrayList<String> omGraphicsNames = dbQueryThread.getOMGraphicsNames();
                dbQueryThread = null;
                initialized = true;
                Debug.message("PostgisDataStorage", this + " IS READY!!!!");
                dataTable.setTable(dataObjects, getDataAttributesNamesToDisplay(columnNames),
                        omGraphicsNames, getDataAttributesNamesAliases());

                // data are stored, we can create a dataTableModel and the panel to view them.
                // if they don't already exist
                if (dataTableModel == null) {
                    dataTableModel = new DataTableModel(dataTable);
                }
                resetDataTableStoragePanel(dataTableModel);

                // Lets advertise we are ready ...
                fireDataStorageEvent();
            } else if (state == DBState.CONNECT_OK) {
                Debug.message("PostgisDataStorage", this + " Connect Ok ...");
                connection.setConnection(dbThread.getConnection());
                connection.setParams(dbThread.getParams());
                dbThread = null;
                // retrieve data immediatly after a connection.
                tableRetrieveData();
            } else if (state == DBState.CONNECT_ERROR) {
                Debug.message("PostgisDataStorage", this + " Connect Error ...");
                connection.setConnection(null);
                connection.setParams(dbThread.getParams());
                dbThread = null;
            }
            if(gui != null)
                ((PostgisDataStoragePanel)gui).updateGUIWithState(state);
        }
    }
    /**
     *
     */
        public void dbParamsChanged() {
            connectDatabase();
            disconnectDatabase();
    }

    /**
     *
     */
    public void disconnectDatabase() {
        Debug.message("PostgisDataStorage", this + " disconnectDatabase()");
        try {
            connection.closeConnection();
        } catch (Exception e) {
        }
    }

    public void initialize() {
        connectDatabase();
    }

    /**
     *
     */
    public void connectDatabase() {
        connection.setParams(dbParams);
        if (dbQueryThread != null) {
            Debug.message("PostgisDataStorage", this + " A query is being processed ...");
            return;
        }
        if (dbThread != null) {
            Debug.message("PostgisDataStorage", this + " Already connecting ...");
            return;
        }
        Debug.message("PostgisDataStorage", this + " connectDatabase(" + dbParams + ")");
        dbThread = new DBConnectThread(dbParams, this);
        dbThread.start();
    }

    public void tableRetrieveData() {
        if (dbThread != null) {
            Debug.message("PostgisDataStorage", this + " Waiting for connection ...");
            return;
        }
        if (dbQueryThread != null) {
            Debug.message("PostgisDataStorage", this + " Already querying ...");
            return;
        }
        Debug.message("PostgisDataStorage", this + " Query starts ...");
        dbQueryThread = new DBQueryThread(connection, this, query);
        dbQueryThread.start();
    }

    //-------------------------------------------------------------------------------------------------------------------
    //try changeQuery...
    public String getQuery() {
        return query;
    }

    public void setQuery(String query) {
        this.query = query;
        initialize();
    }
    //...
    //-------------------------------------------------------------------------------------------------------------------

    /**
     * PostgisDataStorage is threaded, this method CAN return null !
     */
    public DataTableModel getDefaultDataTableModel() {
        return dataTableModel;
    }

    public void reloadFromGUI(GUIChangeEvent event) {
        Object source = event.getSource();
        if (source == gui) {
        } else {
            Debug.message("DataStorage", "[ShapeFileDataStorage] GUIChangeEvent not processed");
        }
        initialize();
    }




//    @Override
//    public JComponent getInitialisationGUI(){
//        return new PostgisDataStorageInitGUI(dbParams);
//    }



}
