// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.postgis.db;

import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.dataStorage.postgis.PostgisDataStorage;
import java.sql.*;
import java.util.ArrayList;

/**
 * Thread.
 * Todo: find the method addType in Postgres driver.
 */
public class DBConnectThread extends Thread {
    
    //----------------------------------------------------------------------
    // Private data
    //----------------------------------------------------------------------
    private DBParams params;
    private String urlDriverString = "jdbc:postgresql://";
    private String driverString = "org.postgresql.Driver";
    private Connection connection = null;
    private String errorState = "NO ERROR";
    private DBListener listener = null;

    //----------------------------------------------------------------------
    // Data access methods
    //----------------------------------------------------------------------
    public Connection getConnection() { return connection; }
    public DBParams getParams() { return params; }
    public String getError() { return errorState; }
    
    /**
     * Constructor.
     */
    public DBConnectThread (DBParams params, DBListener listener) {
	this.listener = listener;
	this.params = params;
    }
    
    /**
     * This launches the thread. This method will connect the
     * server using DBParams and the appropriate jdbc driver
     * for postgis.
     */
    public void run () {
	String databaseURL = urlDriverString + params.getServer() + ":" + params.getPort() + "/" + params.getDatabase();
	try {
	    Debug.message("DBConnectThread", "[DBConnectThread] Openning JDBC connection ...");
	    Class.forName("org.postgresql.Driver");
	    connection = DriverManager.getConnection(databaseURL, params.getLogin(), params.getPassword());
	    Debug.message("DBConnectThread", "[DBConnectThread] Adding geometric type entries ...");
	    
	    
	    // ((org.postgresql.Connection) connection).addDataType("geometry", "org.postgis.PGgeometry");
	    // ((org.postgresql.Connection) connection).addDataType("box3d", "org.postgis.PGbox3d");
	} catch (org.postgresql.util.PSQLException ex) {
            Debug.error(ex.getServerErrorMessage().toString());
            if (listener != null) listener.stateChanged(PostgisDataStorage.DBState.CONNECT_ERROR);
            return;
        } catch (Exception e) {
	    if (Debug.debugging("DBConnectThread")) {
		errorState = e.getMessage();
		Debug.message("DBConnectThread", "[DBConnectThread] connection failed");
		Debug.message("DBConnectThread", "[DBConnectThread] " + params);
		Debug.message("DBConnectThread", "[DBConnectThread] " + errorState);
		if (listener != null) listener.stateChanged(PostgisDataStorage.DBState.CONNECT_ERROR);
		e.printStackTrace();
	    }
	    return;
	}
	Debug.message("[DBConnectThread]", "[DBConnectThread] connection established");
	if (listener != null) listener.stateChanged(PostgisDataStorage.DBState.CONNECT_OK);
    }


}
