// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.postgis.db;

import java.sql.ResultSetMetaData;
import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.dataStorage.DataObject;
import java.util.Vector;

public class DBTable extends Vector<DBTableRow> {
    
    private ResultSetMetaData rsmd;
    public void setMetaData(ResultSetMetaData rsmd) { this.rsmd = rsmd; }
    public ResultSetMetaData getMetaData() { return rsmd; }
    
    private boolean systemTable = false;
    public boolean isSystemTable() { return systemTable; }
    
    /**
     */
    private String name = null;
    
    /**
     */
    private Vector columnNames = null;
    
    /**
     * Selected rows array.
     */
    private int selectedRows [] = null;
    
    /**
     * To display the DBTable object.
     */
    public String toString() {
        // if (isSystemTable()) return "<html><font size=\"-1\"><i>" + name + "</i></font></html>";
        // if (isRetrieved()) return "<html><font size=\"-1\"><b>" + name + "</b></font></html>";
        return name;
    }
    
    /**
     * In fact this class is not only for table
     * but for various data results.
     */
    public DBTable() { this("anonymous table"); }
    
    /**
     * Create a table, with a name.
     */
    public DBTable(String name) {
        super();
        this.name = name;
        systemTable = name.equals("geometry_columns") || name.equals("spatial_ref_sys");
        columnNames = new Vector<String>();
    }
    
    public void setColumnNames(Vector<String> v) { columnNames = v; }
    public Vector getColumnNames() { return columnNames; }
    
    /**
     * Provide a list of selected items in the table.
     *
     * @return a <code>Vector</code> of selected rows in the table.
     */
    public Vector<DBTableRow> getSelectedRows() {
        Vector<DBTableRow> v = new Vector<DBTableRow>();
        if (selectedRows==null) return v;
        for (int i = 0 ; i < selectedRows.length ; i++)
            v.addElement(elementAt(selectedRows[i]));
        return v;
    }
    
    /**
     * Provide a list of selected items in the table.
     *
     * @return a <code>Vector</code> of selected rows in the table.
     */
    public Vector<DBTableRow> getSelectedRows(int [] indexes) {
        Vector<DBTableRow> v = new Vector<DBTableRow>();
        for (int i = 0 ; i < indexes.length ; i++)
            v.addElement(elementAt(indexes[i]));
        return v;
    }
    
    /**
     * Changes selection state for all the rows if present in indexes array.
     */
    public void setSelectedRows(int [] indexes) {
        selectedRows = indexes;
    }
    
    /**
     *
     */
    public String getColumnName(int index) { return (String) columnNames.elementAt(index); }
    
    /**
     *
     */
    public int getColumnCount() { return (size() != 0) ? ((DBTableRow)elementAt(0)).size() : 0; }
    
    /**
     *
     */
    public int getRowCount() { return size(); }
    
    /**
     *
     */
    public Vector getColumn(String name) {
        return getColumn(columnNames.indexOf(name));
    }
    
    /**
     *
     */
    public DBTableRow getColumn(int index) {
        DBTableRow result = new DBTableRow();
        if (index >= getColumnCount()) return result;
        try {
            for (int i = 0 ; i < size() ; i++)
                result.add(elementAt(i).elementAt(index));
        } catch (Exception e) {
            Debug.message("DBTable", "[DBTable] " + e.getMessage());
            return new DBTableRow();
        }
        return result;
    }
    
    /**
     *
     */
    public boolean isRetrieved() {
        return (getColumnNames().size() != 0);
    }
    
    /**
     * get the value at (row,col);
     */
    public Object getValueAt(int row, int col){
        try { return ((DBTableRow) elementAt(row)).elementAt(col); } catch (Exception e) {
            e.printStackTrace();
            return ""; }
        
    }
    
    /**
     * set the value at (row,col);
     */
    public void setValueAt(Object aValue, int row, int column) {
        ((DBTableRow) elementAt(row)).removeElementAt(0);
        ((DBTableRow) elementAt(row)).insertElementAt((DataObject)aValue, 0);
    }
}
