// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.postgis.gui;

import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

public class PostgisDocument extends HighlightedDocument {

    private static final String[] keywords = { "select", "from", "where", "order", "by", "or", "and", "as"};
    private static final String[] functions = { "sin", "cos", "tan" };
    private static final String[] separators = { "(", ")", ";", ":", ",", "?", "[", "]", "{", "}"};
    private static final String[] operators = { "&", "|", "+", "-", "*", "/", ".", "=", ">", "<", "!" };
    
    private static final String[] openGISfunctions = {
	"AddGeometryColumn", "DropGeometryColumn", "AsBinary", "Dimension", "Envelope",
	"GeometryType", "X", "Y", "Z", "NumPoints", "PointN", "ExteriorRing", "NumInteriorRings",
	"InteriorRingN", "IsClosed", "NumGeometries", "GeometryN", "Distance", "AsText", "SRID",
	"GeometryFromText", "SetSRID", "EndPoint", "StartPoint", "Centroid" };
    private static final String[] postGISfunctions = {
	"area2d", "asbinary", "box3d", "extent", "find_srid", "force_collection", "force_2d",
	"force_3d", "length2d", "length3d", "length_spheroid", "length3d_spheroid", "mem_size",
	"npoints", "nrings", "numb_sub_objects", "perimeter2d", "perimeter3d", "point_inside_circle",
	"postgis_version", "summary", "translate", "truly_inside" };
    
    private static final String[] specialFunctions = { "<&", "&>", "<<", ">>", "~=", "~", "&&" };
    
    public PostgisDocument (JTextPane textPane) {
	super(textPane);
	int count = 2 * (keywords.length + functions.length + separators.length + operators.length
			 + openGISfunctions.length + postGISfunctions.length + specialFunctions.length);
	highlihtedItems = new Hashtable<String,Style>(count);
	for (int i = 0 ; i < keywords.length ; i++)
	    highlihtedItems.put(keywords[i], STYLE_KEYWORD);
	for (int i = 0 ; i < functions.length ; i++)
	    highlihtedItems.put(functions[i], STYLE_FUNCTION);

	for (int i = 0 ; i < openGISfunctions.length ; i++)
	    highlihtedItems.put(openGISfunctions[i], STYLE_FUNCTION);
	for (int i = 0 ; i < postGISfunctions.length ; i++)
	    highlihtedItems.put(postGISfunctions[i], STYLE_FUNCTION);
	for (int i = 0 ; i < specialFunctions.length ; i++)
	    highlihtedItems.put(specialFunctions[i], STYLE_FUNCTION);
	
	for (int i = 0 ; i < separators.length ; i++)
	    highlihtedItems.put(separators[i], STYLE_OPERATOR);
	for (int i = 0 ; i < operators.length ; i++)
	    highlihtedItems.put(operators[i], STYLE_SEPARATOR);
	highlihtedItems.put("\"", STYLE_STRING);
    }
}
