// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.postgis.gui;

import fr.unicaen.openmap.util.ContentFileLoader;
import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;
import fr.unicaen.openmap.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.print.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.tree.*;
import javax.swing.event.*;

//try...
import fr.unicaen.openmap.dataStorage.postgis.*;
import fr.unicaen.openmap.gui.ThemaMapFileChooserManager;
import fr.unicaen.openmap.util.JActionButton;
import java.awt.geom.Dimension2D;
import java.io.File;
import javax.swing.filechooser.FileNameExtensionFilter;
//...

/**
 * This class is used to display a database query.
 */
public class QueryPanel extends JPanel {
    //try...
    public PostgisDataStorage dataStorage;

    //...

    protected I18n i18n = Environment.getI18n();

    private QueryTextPane qtp = null;
    
    public QueryPanel() { this(""); }

    //a <<static>> panel which does nothing
    public QueryPanel(String query) {
	super();
        JScrollPane sp = new JScrollPane(qtp = new QueryTextPane(query));
        sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
	sp.setPreferredSize(new Dimension(350, 160));
	sp.setMinimumSize(new Dimension(10, 10));
        setLayout(new BorderLayout());
        add(sp, BorderLayout.CENTER);
    }

    //-------------------------------------------------------------------------------------------------------------------    
    //changeQuery... Ok
    public QueryPanel(String query, PostgisDataStorage dataStorage) {
	this(query);

	this.dataStorage = dataStorage;

	JActionButton submit = new JActionButton(i18n.get(this,"ok", "Run"))
	    {
		public void actionPerformed(ActionEvent e)
		{
		    if (!getPostgisDataStorage().getQuery().equals(getQuery()))
			{
			    //System.out.println("Query changes...");
			    getPostgisDataStorage().setQuery(getQuery()); //change in DataStorage
			    setQuery(getPostgisDataStorage().getQuery()); //after adjustment, change in the panel (eventually)
			}
		    else System.err.println("Warning: Same request, I don't work !");
		}
	    };
	submit.setToolTipText("validate request");

	JActionButton cancel = new JActionButton(i18n.get(this,"cancel", "Cancel"))
	    {
		public void actionPerformed(ActionEvent e)
		{
		    setQuery(getPostgisDataStorage().getQuery());
		}
	    };
	cancel.setToolTipText("write last request");

        JPanel pButtons = new JPanel();
        pButtons.add(submit);
        pButtons.add(cancel);
        add(pButtons, BorderLayout.SOUTH);
        JPanel pTop = new JPanel(new BorderLayout());
        pTop.add(new JLabel(i18n.get(this, "request","Request")), BorderLayout.WEST);
        JButton bImportSQL = new JButton(Util.loadIcon("Import16.png"));    
        bImportSQL.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                ThemaMapFileChooserManager.dataLoadDialog("", null);
                //JFileChooser jfc = new JFileChooser(".");
                FileNameExtensionFilter filter = new FileNameExtensionFilter(
		        "SQL files", "sql");
                File f = new File(ThemaMapFileChooserManager.dataLoadDialog("", filter));
                if(!f.exists())
                    return;
                ContentFileLoader.loadContent(f);
//                jfc.setFileFilter(filter);
//                if(jfc.showOpenDialog(QueryPanel.this) == JFileChooser.APPROVE_OPTION){
//                    qtp.setText(
//                            ContentFileLoader.loadContent(jfc.getSelectedFile()));
//                }
            }
        });
        JPanel pButton = new JPanel();
        pButton.add(bImportSQL);
        bImportSQL.setMaximumSize(new Dimension(20, 15));
        bImportSQL.setPreferredSize(new Dimension(20, 15));
        bImportSQL.setMinimumSize(new Dimension(20, 15));
        pButton.setBorder(BorderFactory.createEmptyBorder(0, 20, 0, 20));
        pTop.add(pButton, BorderLayout.EAST);
        add(pTop, BorderLayout.NORTH);
    }
   
    public PostgisDataStorage getPostgisDataStorage()
    {
	return dataStorage;
    }
    //...
    //-------------------------------------------------------------------------------------------------------------------

    public void setQuery(String query) {
	synchronized(this) {
	    qtp.setText(query);
	}
    }
    
    public String getQuery() {
	synchronized(this) {
	    return qtp.getText();
	}
    }
}
