package fr.unicaen.openmap.dataStorage.triangulation.paulchew;

import fr.unicaen.openmap.util.Util;

/**
 * This calls is a wrapper around Paul Chew's triangulation Pnt to hold data values.
 */
public class DataPoint extends Pnt {

    protected Object data;

    /**
     * Constructor 
     * @parameter lon, lat, data
     */
    public DataPoint (double x, double y, Object data) {
	super(x, y);
	this.data = data;
    }
    public DataPoint (double x, double y) {
	this(x, y, null);
    }

    public Object getData() {
	return data;
    }
    
    /**
     * Test if this DataPoint is outside of simplex.
     * @param simplex the simplex (an array of Pnts)
     * @return the simplex Pnt that "witnesses" outsideness (or null if not outside)
     */
    public DataPoint isOutside (DataPoint[] simplex) {
        int[] result = this.relation(simplex);
        for (int i = 0; i < result.length; i++) {
            if (result[i] > 0) return simplex[i];
        }
        return null;
    }

    public boolean equals(double xx, double yy) {
	return equals(xx, yy, 1E-10);
    }

    public boolean equals(double xx, double yy, double precision) {
	return Util.equals(coord(0), xx, precision) && Util.equals(coord(1), yy, precision); 
    }


    public String toString() {
	return super.toString() + " " + getData().toString();
    }
}
