// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.xmltable;

import fr.unicaen.openmap.dataStorage.*;
import java.util.Properties;
import com.bbn.openmap.util.*;
import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import fr.unicaen.openmap.dataStorage.gui.DataTableStoragePanel;
import fr.unicaen.openmap.util.propertyEditor.DataStoragePropertyEditor;
import fr.unicaen.openmap.util.propertyEditor.FileTypeDescription;

import javax.swing.*;
import java.awt.event.*;

/**
 * Sample implementation of an XML dataStorage.
 *
 * Example for openmap.properties :
 * <code>
 *   xmltest.class             = fr.unicaen.openmap.layer.dataDisplayLayer.GeometryDataDisplayLayer
 *   xmltest.dataStorageClass  = fr.unicaen.openmap.dataStorage.xmltable.XmlTableDataStorage
 *   xmltest.prettyName        = Test XML
 *   xmltest.lineColor         = FF000000
 *   xmltest.fillColor         = FFFFFFFF
 *   xmltest.selectedLineColor = FF990099
 *   xmltest.selectedFillColor = C900FF55
 *   xmltest.fill              = yes
 *   xmltest.xmlFile           = test.xml
 *   xmltest.geometryColumnName = prov
 * </code>
 */
public class XmlTableDataStorage extends InformationDataStorage {

    /**
     * Property name used in getPropertyInfo.
     */
    public static final String XML_BY_INFO = "xml.info";
    
    public String getName() { 
	return "".equals(xmlFile) ? name : stripFileNameWithClassPathDirectories(xmlFile); 
    }

    public String toString() { return "[XmlTableDataStorage:" + getName() + "]"; }
    
    public XmlTableDataStorage(DataStorageListener dsl) {
        super(dsl);
        Debug.message("DataStorage", this + " XmlTableDataStorage(DataStorageListener dsl)");
    }
    
    protected boolean initialized = false;
    
     public boolean isReady() { return initialized; }
    
    private static final String xmlFilePropertyString = "xmlFile";
    private static final String changeXmlFilePropertyString = "xmlFile.change";
    
    private String xmlFile = "";
    
    public void setProperties(String prefix, java.util.Properties properties) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
	// name = prefix;
        super.setProperties(prefix, properties);
  
        xmlFile = properties.getProperty(prefix + xmlFilePropertyString, 
					 properties.getProperty(prefix + XML_BY_INFO, ""));
        
        Debug.message("DataStorage" , "[XmlTableDataStorage] using file:" + xmlFile + ".");
        
        initialize();
    }
    
    public Properties getProperties(String prefix, Properties properties) {
    	properties = super.getProperties(prefix, properties);
	prefix = PropUtils.getScopedPropertyPrefix(prefix);

        properties.put(prefix + xmlFilePropertyString, stripFileNameWithClassPathDirectories(xmlFile));

	//Used with getPropertyInfo	    
	properties.put(prefix + XML_BY_INFO, //csvFilePropertyString,
		       new FileTypeDescription(FileTypeDescription.XML_DESCRIPTION, xmlFile, FileTypeDescription.XML_PATTERN));
        
        return properties;
    }

    public Properties getPropertyInfo(Properties properties) {
	properties = super.getPropertyInfo(properties);
	/*
	PropUtils.setI18NPropertyInfo(i18n,
				      properties,
				      XmlTableDataStorage.class,
				      xmlFilePropertyString,
				      xmlFilePropertyString,
				      xmlFilePropertyString,
				      "com.bbn.openmap.util.propertyEditor.FilePropertyEditor"
				      );
	*/
	PropUtils.setI18NPropertyInfo(i18n,
				      properties,
				      XmlTableDataStorage.class,
				      XML_BY_INFO,
				      "File",
				      "Select a XML file.",
				      "fr.unicaen.openmap.util.propertyEditor.FileTypePropertyEditor"
				      );

	properties.put(initPropertiesProperty, 
		       //		       xmlFilePropertyString
		       XML_BY_INFO
		       );

	return properties;
    }    
    
    public void initialize() {
        try {
            XmlTableParser parser = new XmlTableParser(xmlFile);
            dataTable.setTable(parser.getDataObjects(),
			       getDataAttributesNamesToDisplay(parser.getColumnNames()),
			       parser.getOMGraphicsNames(),
			       getDataAttributesNamesAliases()
			       );
            // data are stored, we can create dataTableModel and panel to view them.
            dataTableModel = new DataTableModel(dataTable);
            resetDataTableStoragePanel(dataTableModel);
            initialized = true;
        } catch (Exception e) {
            e.printStackTrace();
            initialized = false;
        }
        
	fireDataStorageEvent();
     }
    

    public JComponent getGUI() {
	JPanel panel = new JPanel();
	/*
	final JLabel xmlLabel = new JLabel(i18n.get(this, xmlFilePropertyString, "XML file: ")+xmlFile);
	JButton xmlButton = new JButton(i18n.get(this, changeXmlFilePropertyString, "Change XML File"));
	xmlButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    String _xmlFile = FileUtils.getFilePathToOpenFromUser("XML File");
		    if (_xmlFile == null || "".equals(_xmlFile)) {
			Debug.error("No XML file choosen");
			return;
		    }
		    xmlFile = _xmlFile;
		    xmlLabel.setText(i18n.get(this, xmlFilePropertyString, "XML file: ")+xmlFile);
		    initialize();
		}
	    });
	panel.add(xmlLabel); 
	panel.add(xmlButton);
	*/
	panel.add(DataStoragePropertyEditor.provideCustomPanel(this));

	return panel; 
    }
    

}
