// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.event;

import com.bbn.openmap.MapBean;
import com.bbn.openmap.event.CoordMouseMode;
import com.bbn.openmap.event.InfoDisplayEvent;
import com.bbn.openmap.geo.Geo;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMRect;
import com.bbn.openmap.proj.Length;
import com.bbn.openmap.proj.Planet;
import com.bbn.openmap.proj.Projection;
import fr.unicaen.openmap.InformationDelegator;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.text.DecimalFormat;
import java.util.Vector;

/**
 *
 * @author jchauvea
 */
public class AreaMouseMode extends CoordMouseMode {

    public final static transient String modeID = "Area".intern();
    public static int lineType = OMGraphic.LINETYPE_STRAIGHT;
    private boolean mousePressed;
    private Point2D startPoint;
    private Point2D endPoint;
    protected MapBean theMap;
    private String upperLeft;
    private String lowerRight;
    private double area;

    public AreaMouseMode() {
        this(true);
        // if you really want to change the cursor shape
        // setModeCursor(cursor.getPredefinedCursor(cursor.CROSSHAIR_CURSOR));
    }

    public AreaMouseMode(boolean consumeEvents) {
        super(modeID, consumeEvents);
        // if you really want to change the cursor shape
        // setModeCursor(cursor.getPredefinedCursor(cursor.CROSSHAIR_CURSOR));
    }

    public AreaMouseMode(String name, boolean consumeEvents) {
        super(name, consumeEvents);
        // if you really want to change the cursor shape
        // setModeCursor(cursor.getPredefinedCursor(cursor.CROSSHAIR_CURSOR));
    }

    public AreaMouseMode(boolean consumeEvents, InformationDelegator id,
            Length units) {
        super(modeID, consumeEvents);
        // if you really want to change the cursor shape
        // setModeCursor(cursor.getPredefinedCursor(cursor.CROSSHAIR_CURSOR));
        infoDelegator = id;
        //unit = units;
    }

    public AreaMouseMode(boolean consumeEvents, InformationDelegator id,
            Length units, int lType) {
        super(modeID, consumeEvents);
        // if you really want to change the cursor shape
        // setModeCursor(cursor.getPredefinedCursor(cursor.CROSSHAIR_CURSOR));
        infoDelegator = id;
        //unit = units;
        lineType = lType;
    }

    public AreaMouseMode(boolean consumeEvents, InformationDelegator id) {
        super(modeID, consumeEvents);
        // if you really want to change the cursor shape
        // setModeCursor(cursor.getPredefinedCursor(cursor.CROSSHAIR_CURSOR));
        infoDelegator = id;
    }

    public AreaMouseMode(String name, boolean consumeEvents,
            InformationDelegator id) {
        super(name, consumeEvents);
        // if you really want to change the cursor shape
        // setModeCursor(cursor.getPredefinedCursor(cursor.CROSSHAIR_CURSOR));
        infoDelegator = id;
    }

    @Override
    public void mousePressed(MouseEvent e) {
        mouseSupport.fireMapMousePressed(e);
        e.getComponent().requestFocus();

        if (e.getSource() instanceof MapBean) {
            // mouse has now been pressed
            mousePressed = true;
            // erase the old circle if any
            //eraseRectangle();

            if (theMap == null) {
                theMap = (MapBean) e.getSource();
            }

            // anchor the new first point of the line
            startPoint = theMap.getCoordinates(e);
            // ensure the second point is not yet set.
            endPoint = null;

        }
    }

    public void mouseClicked(MouseEvent e) {

        mouseSupport.fireMapMouseClicked(e);

        if (e.getSource() instanceof MapBean) {
            // if double (or more) mouse clicked
            if (e.getClickCount() >= 2) {
                // end of distance path
                mousePressed = false;
                startPoint = null;
                endPoint = null;
                paintRectangle(endPoint, endPoint);
            } else {
                ((MapBean) e.getSource()).repaint();
            }
            // cleanup
            //cleanUp();
        }
    }

    public void mouseMoved(MouseEvent e) {
        mouseSupport.fireMapMouseMoved(e);
        if (e.getSource() instanceof MapBean) {
            // only when the mouse has already been pressed
            if (mousePressed) {
                double lat1, lat2, long1, long2;
                // set the map bean
                theMap = (MapBean) (e.getSource());
                // erase the old line and circle first
                paintRectangle(startPoint, endPoint);
                // get the current mouse location in latlon
                endPoint = theMap.getCoordinates(e);
                // paint the new line and circle up to the current
                // mouse location
                paintRectangle(startPoint, endPoint);

                if (infoDelegator != null) {
//                    Debug.message("mousemodedetail",
//                            "DistanceMouseMode: firing mouse location");
                    // lat, lon of anchor point
                    lat1 = startPoint.getY();
                    long1 = startPoint.getX();
                    // lat, lon of current mouse position
                    lat2 = endPoint.getY();
                    long2 = endPoint.getX();
                    // calculate great circle distance in nm
                    // distance = getGreatCircleDist(lat1, long1,
                    // lat2, long2, Length.NM);
//                    distance = GreatCircle.sphericalDistance(ProjMath.degToRad(lat1),
//                            ProjMath.degToRad(long1),
//                            ProjMath.degToRad(lat2),
//                            ProjMath.degToRad(long2));

                    // calculate azimuth angle dec deg
//                    double azimuth = getSphericalAzimuth(lat1,
//                            long1,
//                            lat2,
//                            long2);
                    // convert total distance into all distance units
                    // String distNM =
                    // df.format(totalDistance+distance);
//                    double tmpDistance = totalDistance + distance;
                    String infoLine = createDistanceInformationLine(startPoint, endPoint);
                    // setup the info event
                    InfoDisplayEvent info = new InfoDisplayEvent(this, infoLine, InformationDelegator.COORDINATE_INFO_LINE);
                    // ask the infoDelegator to display the info
                    infoDelegator.requestInfoLine(info);
                }
            } else {
                fireMouseLocation(e);
            }
        }
    }

    public void paintRectangle(Point2D p1, Point2D p2) {
        if (theMap != null) {
            paintRectangle(p1, p2, theMap.getGraphics(true));
        }
    }

    public void paintRectangle(Point2D p1, Point2D p2, Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        g2d.setXORMode(java.awt.Color.lightGray);
        g2d.setColor(java.awt.Color.darkGray);
        if (p1 != null && p2 != null) {
            // the line connecting the segments
            OMRect rect = new OMRect(p1.getY(), p1.getX(), p2.getY(), p2.getX(), lineType);
            // get the map projection
            Projection proj = theMap.getProjection();
            // prepare the line for rendering
            rect.generate(proj);
            // render the line graphic
            rect.render(g2d);
        }
    }

    private String createDistanceInformationLine(Point2D p1, Point2D p2) {
        String line = "";
        line += "North West point: " + formatDouble(p1.getY()) + ";" + formatDouble(p1.getX());
        line += "   South East point: " + formatDouble(p2.getY()) + ";" + formatDouble(p2.getX());
        Vector<Geo> points = new Vector<Geo>();
        points.add(new Geo(p1.getY(), p1.getX()));
        points.add(new Geo(p2.getY(), p1.getX()));
        points.add(new Geo(p2.getY(), p2.getX()));
        points.add(new Geo(p1.getY(), p2.getX()));
        //r= E + l*((P-E)/90)
        double l = Math.abs((p1.getY() + p2.getY()) / 2);
        double r = (Planet.wgs84_earthEquatorialRadiusMeters_D +
                l * ((Planet.wgs84_earthPolarRadiusMeters_D - Planet.wgs84_earthEquatorialRadiusMeters_D) / 90)) / 1000;
        line += "  -   Area: " + String.valueOf(formatDouble(Geo.area(points.elements()) * r * r) + " km²");
        return line;
    }

    private String formatDouble(double d) {
        DecimalFormat df = new DecimalFormat("########.0000");
        return df.format(d);
    }
}
