// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui;

import com.bbn.openmap.Layer;
import com.bbn.openmap.LayerHandler;
import fr.unicaen.openmap.AtlasLayerHandler;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.util.Util;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import javax.swing.Icon;
import javax.swing.JCheckBox;
import javax.swing.JToggleButton;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

/**
 *
 * @author jchauvea
 */
public class AtlasLayerTreeNodeComponent extends LayerTreeNodeComponent {

    protected DefaultMutableTreeNode node;

    private Cursor waitCursor = new Cursor(Cursor.WAIT_CURSOR);
    private Cursor defaultCursor = new Cursor(Cursor.DEFAULT_CURSOR);

    public AtlasLayerTreeNodeComponent(AtlasLayersTree tree, Layer layer, DefaultMutableTreeNode node) {
        super(tree, layer);
        this.node = node;
    }

    @Override
    protected JToggleButton getOnOffButton() {
	boolean on = tree.getLayerHandler().hasLayer(layer);
        JCheckBox b = new JCheckBox(on ? Util.loadIcon("enabled.png") : Util.loadIcon("disabled.png"), on);
        b.setBorder(/*BorderFactory.createEmptyBorder(0,0,0,10)*/null);
        b.setBorderPainted(false);
        b.setFocusPainted(false);
        b.setBackground(Color.WHITE);
        b.setOpaque(false);
        b.setFocusable(true);

        b.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                if (AtlasLayerTreeNodeComponent.this.getBackground() != Color.LIGHT_GRAY) {
                    AtlasLayerTreeNodeComponent.this.setBackground(Color.LIGHT_GRAY);
                }
                JCheckBox bsource = (JCheckBox) e.getSource();
                AtlasLayerHandler lh = (AtlasLayerHandler) tree.getLayerHandler();
                setCursor(waitCursor);
                if (e.getStateChange() == ItemEvent.SELECTED) {
                    bsource.setIcon(Util.loadIcon("enabled.png"));

                    lh.loadLayer(layer);
                    tree.setSelectionPath(new TreePath(node.getPath()));
                } else {
                    bsource.setIcon(Util.loadIcon("disabled.png"));
                    lh.unloadLayer(layer);
                }
                setCursor(defaultCursor);
            }
        });
        return b;
    }
}
