// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui;

import com.bbn.openmap.Layer;
import com.bbn.openmap.LayerHandler;
import fr.unicaen.openmap.AtlasLayerHandler;
import fr.unicaen.openmap.AtlasLoader;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicDataDisplayLayerPopupMenu;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicPopupMenu;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Thematic;
import fr.unicaen.openmap.util.TooManyLegendException;
import fr.unicaen.openmap.util.Util;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Vector;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

/**
 *Atlas LayersTree - built according to a specified directory file system
 * @author jchauvea
 */
public class AtlasLayersTree extends LayersTree {

    protected String path;
    public static HashMap<String, Layer[]> layersByPath = new HashMap<String, Layer[]>();

    public AtlasLayersTree(DefaultMutableTreeNode root, String path,
            LayerHandler lh, JComponent comp) {

        super(root, lh, comp);
	removeMouseListener(this);

	// System.out.println("Atlas path: "+path + " -> "+this.path);
	
   	setPath(Util.locateFile(path));
    }

    public AtlasLayersTree(DefaultTreeModel model, LayerHandler lh, JComponent comp) {
        super(model, lh, comp);
    }

    private void clear() {
	root.removeAllChildren();
	layerHandler.removeAll();
	layersLoaded = false;
    }

    public void setPath(String path) {
	clear();
        this.path = path;
	buildFileSystemTree(root, new File(this.path));
        layersLoaded = true;
        setShowsRootHandles(true);
        expandTree(root);
        updateUI();
    }

    /** Add nodes from under "dir" into top. Highly recursive. */
    protected DefaultMutableTreeNode buildFileSystemTree(DefaultMutableTreeNode curTop, File dir) {
        String curPath = dir.getPath();
        DefaultMutableTreeNode curDir = null;
	String curName = curPath;
        if (curPath.contains(File.separator)) {
            String[] tab;
	    tab = curPath.split(File.separator.equals("\\") ?
				"\\" + File.separator :  File.separator);
            curName = tab[tab.length - 1];
	} 
	curDir = new DefaultMutableTreeNode(new DirectoryCell(curName));

        if (curTop != null) { // should only be null at root
            curTop.add(curDir);
        }
        Vector<String> ol = new Vector<String>();
        //System.out.println("DIR = " + dir.getAbsolutePath());
        String[] tmp = dir.list();
        if (tmp != null) {
            for (int i = 0; i < tmp.length; i++) {
                ol.addElement(tmp[i]);
            }
        }
        Collections.sort(ol, String.CASE_INSENSITIVE_ORDER);
        File f;
        Vector<String> files = new Vector<String>();
        // Make two passes, one for Dirs and one for Files. This is #1.
        for (int i = 0; i < ol.size(); i++) {
            String thisObject = (String) ol.elementAt(i);
            String newPath;
            if (curPath.equals(".")) {
                newPath = thisObject;
            } else {
                newPath = curPath + File.separator + thisObject;
            }
            if ((f = new File(newPath)).isDirectory()) {

                buildFileSystemTree(curDir, f);

            } else {
                files.addElement(thisObject);
            }
        }
        // Pass two: for properties files.
        for (int fnum = 0; fnum < files.size(); fnum++) {
            if (files.elementAt(fnum).endsWith(AtlasLoader.PROPERTIES_EXTENSION)) {
                DefaultMutableTreeNode pNode = 
		    new DefaultMutableTreeNode(
					       new PropertiesCell(
								  files.elementAt(fnum).replace(AtlasLoader.PROPERTIES_EXTENSION, "")));

                curDir.add(pNode);
                String tmpPath = curPath + File.separator + files.elementAt(fnum);
                if (!layersLoaded) {
                    try {
                        Layer[] ls = AtlasLoader.loadLayersFromPath(tmpPath, (AtlasLayerHandler) layerHandler);

                        if (ls != null) {
                            layersByPath.put(tmpPath, ls);
                            buildTreeForLayers(pNode, ls);
                        }
                    } catch (TooManyLegendException e) {
                        JOptionPane.showMessageDialog(this, e.getMessage(), "", JOptionPane.ERROR_MESSAGE);
                        continue;
                    }
                } else {
                    buildTreeForLayers(pNode, layersByPath.get(tmpPath));
                }
            }
        }

        return curDir;
    }

    @Override
    protected void buildTreeForLayers(DefaultMutableTreeNode pnode, Layer[] layers) {
        for (Layer l : layers) {
            DefaultMutableTreeNode node = new DefaultMutableTreeNode(l);
            pnode.add(node);
            nLayers++;
            if (l instanceof ThematicDataDisplayLayer) {
                popups.put(node, new ThematicDataDisplayLayerPopupMenu(this,
                        (ThematicDataDisplayLayer) l));
                ThematicDataDisplayLayer tddl = (ThematicDataDisplayLayer) l;
                tddl.addThematicListener(this);
                for (int i = 0; i < tddl.getThematicArray().size(); i++) {
                    DefaultMutableTreeNode nodeThema = new DefaultMutableTreeNode(
                            tddl.getThematicArray().get(i));
                    node.add(nodeThema);
                    popups.put(nodeThema, new ThematicPopupMenu(this, tddl.getThematicArray().get(i).isVisible()));

                }
            } else if (popups != null && l != null) {
                popups.put(node, new LayerPopupMenu(this, l.isVisible()));
            } else {
		System.err.println("?? null");
	    }
        }
    }

    @Override
    public AtlasLayerHandler getLayerHandler() {
        return (AtlasLayerHandler) super.getLayerHandler();
    }

    static public class Cell {
	private String name;
	public Cell(String name) {
	    this.name = name;
	}
	public String getName() { return name; }
	public String toString() { return name; }
    }
    static public class PropertiesCell extends Cell {
	public PropertiesCell(String name) {
	    super(name);
	}
    }
    static public class DirectoryCell  extends Cell {
	private String name;
	public DirectoryCell(String name) {
	    super(name);
	}
	public String getName() { return name; }
    }
}
