// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui;

import javax.swing.JSeparator;
import javax.swing.JMenuItem;

import com.bbn.openmap.Layer;
import com.bbn.openmap.LayerHandler;
import com.bbn.openmap.MapHandler;
import com.bbn.openmap.gui.LayerAddPanel;
import com.bbn.openmap.gui.LayersPanel;
import com.bbn.openmap.util.Debug;


public class LayersMenu extends com.bbn.openmap.gui.LayersMenu{

	
	protected transient String loadLayersButtonTitle = "Load Layers...";
	protected transient JMenuItem load = null;
	
    /**
     * Construct LayersMenu.
     */
    public LayersMenu() {
        this(null);
    }

    /**
     * Construct LayersMenu.
     * 
     * @param lHandler the handler for the layer
     */
    public LayersMenu(LayerHandler lHandler) {
        this(lHandler, "Layers", LAYERS_ON_OFF);
        setText(i18n.get(LayersMenu.class, "Layers","Layers"));
    }

    /**
     * Construct LayersMenu.
     * 
     * @param lHandler the handler for the layer
     * @param menuName the name of the menu
     * @param menuType either LAYERS_ON_OFF, or PALETTES_ON_OFF
     */
    public LayersMenu(LayerHandler lHandler, String menuName, int menuType) {
        super(lHandler, menuName, menuType);
        this.menuType = menuType;
        setMnemonic('h');
       // loadPanel = new LayerLoadPanel();

    }
    

    
    public void findAndInit(Object someObj) {
        if (someObj instanceof LayerHandler) {
            // do the initializing that need to be done here
            Debug.message("bc", "LayersMenu found a LayerHandler");
            setLayerHandler((LayerHandler) someObj);
        } else if (someObj instanceof LayersPanel) {
            setupEditLayersButton((LayersPanel) someObj);
        }
        else if(someObj instanceof LayerLoadPanel){
        	setupLayerLoadButton((LayerLoadPanel) someObj);
        }
        else if(someObj instanceof LayerAddPanel){
        	setupLayerAddButton((LayerAddPanel) someObj);
        }
        
    }
    
    /**
     * Constructs the menu item that will bring up the LayerLoadPanel.
     */
    public void setupLayerAddButton(final LayerAddPanel menu) {
    	//super.setupLayerAddButton(menu);
        final LayerAddPanel lap = menu;
        JMenuItem button = new JMenuItem(i18n.get(LayersMenu.class,
                "addLayersButtonTitle",
                addLayersButtonTitle));
       button.setActionCommand("add");
       button.addActionListener(lap);
       setAdd(button);
       add(button);
    }
    
    public void setupLayerLoadButton(final LayerLoadPanel menu) {
    	 final LayerLoadPanel llp = menu;
         JMenuItem button = new JMenuItem(i18n.get(LayersMenu.class,
                 "loadLayersButtonTitle",
                 loadLayersButtonTitle));
        button.setActionCommand("load");
        button.addActionListener(llp);
        setLoad(button);
        add(button);   	
    }
    
    
    /**
     * Set the load menu item.
     */
    public void setLoad(JMenuItem a) {
        load= a;
        if (getLayerHandler() != null) {
            setLayers(getLayerHandler().getLayers());
        }
    }
    
    public void setLayers(Layer[] inLayers) {
    	super.setLayers(inLayers);
    	if(load!=null){
    		add(new JSeparator());
    		add(load);
    	}
    		
    }

}
