// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui;

import java.awt.BorderLayout;

import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.tree.DefaultMutableTreeNode;

import com.bbn.openmap.Layer;
import com.bbn.openmap.util.Debug;

import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Thematic;
import java.beans.PropertyChangeEvent;
import java.util.Properties;
import javax.swing.JTabbedPane;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.TreePath;

public class LayersPanelTree extends com.bbn.openmap.gui.LayersPanel {

    protected LayersTree tree, treeNoDirectories;
    protected ThemaMapPanelsManager.themamap_mode mode;
    protected JComponent layerGUIPanel;
    public final static String ThematicSelectedCmd = "ThematicSelected";
    private boolean atlasBuilt = false;
    protected String atlasPath;
    protected ThemaMapPanel mapPanel;

    @Override
    public void createPanel(Layer[] inLayers) {
        
        if (ThemaMapPanelsManager.MODE == ThemaMapPanelsManager.themamap_mode.CLASSIC) {
            setLayout(new BorderLayout());
            createClassicPanel(inLayers);
        } else {
            if (tree == null) {
                setLayout(new BorderLayout());
                createAtlasPanel();
            }
        }
        //select first layer by default
        if(layerHandler.getLayers().length > 0){
            tree.setSelectionRow(0);
        }
    }

    private void createClassicPanel(Layer[] inLayers) {
        removeAll();
        add(controls, BorderLayout.NORTH);
        if (tree != null) {
            remove(tree);
        }
        tree = new ClassicLayersTree(
                new DefaultMutableTreeNode("Layers"), inLayers, layerHandler, layerGUIPanel);
        tree.setRootVisible(false);
        tree.addPropertyChangeListener(LayerSelectedCmd, this);
        tree.addPropertyChangeListener(ThematicSelectedCmd, this);
        tree.addPropertyChangeListener(LayerDeselectedCmd, this);
        tree.addPropertyChangeListener(mapPanel);
        add(new JScrollPane(tree), BorderLayout.CENTER);
        updateUI();
    }

    private void createAtlasPanel() {
        //System.out.println("==========================+CREATE ATLAS PANEL");
        remove(controls);
        JTabbedPane tabbedPane = null;
        if (!atlasBuilt) {
            atlasBuilt = true;
            tree = new AtlasLayersTree(new DefaultMutableTreeNode("Maps"), atlasPath, layerHandler, layerGUIPanel);
            tree.setRootVisible(false);
            tree.addPropertyChangeListener(LayerSelectedCmd, this);

            treeNoDirectories = new NoDirectoriesAtlasLayersTree(
                    new DefaultMutableTreeNode("Maps"), atlasPath, layerHandler, layerGUIPanel);
            treeNoDirectories.setRootVisible(false);
            treeNoDirectories.addPropertyChangeListener(LayerSelectedCmd, this);
            treeNoDirectories.addPropertyChangeListener(LayerSelectedCmd, this);
            treeNoDirectories.addPropertyChangeListener(LayerDeselectedCmd, this);
            tree.addPropertyChangeListener(mapPanel);
            tabbedPane = new JTabbedPane();
            //((AtlasLayersTree) tree).addSynchronizedTree((AtlasLayersTree) treeNoDirectories);
            //((AtlasLayersTree) treeNoDirectories).addSynchronizedTree((AtlasLayersTree) tree);

        } else {
            return;
        }
        tree.addPropertyChangeListener(ThematicSelectedCmd, this);
        add(tabbedPane, BorderLayout.CENTER);
        tabbedPane.addTab(i18n.get(this, "atlas", "Atlas"), new JScrollPane(treeNoDirectories));
        tabbedPane.addTab(i18n.get(this, "explorer", "Explorer"), new JScrollPane(tree));

        updateUI();

    }

    public void deleteLayers(Layer[] layers) {
        tree.removeLayers();
        layerGUIPanel.removeAll();
        layerGUIPanel.updateUI();

    }

    public void deleteLayer(Layer l) {
        tree.removeLayer(l);
    }

    public void setLayerGUIPanel(JComponent comp) {
        layerGUIPanel = comp;
        if (tree != null) {
            tree.setLayerGUIComp(comp);
        }
    }
    public static String atlasPathPropertyString = "atlaspath";

    @Override
    public void setProperties(String prefix, Properties props) {
        super.setProperties(prefix, props);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        atlasPath = props.getProperty(prefix + atlasPathPropertyString, "atlas");
    }

    /**
     * Change a layer's position.
     */
    @Override
    public void moveLayer(Layer layer, String command) {

        if (command == LayerDownCmd) {
            tree.moveDownSelectedNode();
            DefaultMutableTreeNode n = tree.getObjectNode(layer, tree.getRootNode());
            TreePath tp = new TreePath(n.getPath());
            //System.out.println(tp.toString());
            tree.setSelectionPath(tp);
            //tree.updateUI();
            return;
        }
        if (command == LayerUpCmd) {
            tree.moveUpSelectedNode();
            DefaultMutableTreeNode n = tree.getObjectNode(layer, tree.getRootNode());
            TreePath tp = new TreePath(n.getPath());
            //System.out.println(tp.toString());
            tree.setSelectionPath(tp);
            //tree.updateUI();
            return;
        }
    }

    @Override
    public void propertyChange(PropertyChangeEvent pce) {
        super.propertyChange(pce);
        String command = pce.getPropertyName();
        Object obj = pce.getNewValue();

        if (obj instanceof Thematic) {
            if (command == ThematicSelectedCmd) {
                firePropertyChange(command, null, ((Thematic) obj));
                return;
            }
            if (command == LayerUpCmd) {
                tree.moveUpSelectedNode();
                return;
            }
            if (command == LayerDownCmd) {
                tree.moveDownSelectedNode();
                return;
            }

        }
        if (obj instanceof Layer) {
            if (command == LayerRemoveCmd) {
                tree.removeSelectedNode();
                layerHandler.removeLayer((Layer) obj);
                return;
            }
        }
    }

    @Override
    public void findAndInit(Object someObj) {
        super.findAndInit(someObj);
        if (someObj instanceof ThemaMapPanel) {
            mapPanel = (ThemaMapPanel) someObj;
        }

    }

    public void setAtlasPath(String path) {
	if (ThemaMapPanelsManager.MODE == ThemaMapPanelsManager.themamap_mode.CLASSIC) {
	    Debug.error("Error panel is not in atlas mode");
	    return;
	}
	((AtlasLayersTree)tree).setPath(path);
	((AtlasLayersTree)treeNoDirectories).setPath(path);
    }
}

