// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Properties;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JPanel;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;
import com.bbn.openmap.PropertyConsumer;
import com.bbn.openmap.gui.OverviewMapHandler;
import com.bbn.openmap.gui.ScaleTextPanel;
import com.bbn.openmap.util.PropUtils;

import fr.unicaen.openmap.gui.dnd.MovableBox;

public class NavigationBox extends MovableBox{
	
	public static String className = "fr.unicaen.openmap.gui.NavigationBox";

	LinkedList<JComponent> children = new LinkedList<JComponent>();
	
	protected OverviewMapHandler overviewMap;
	protected NavigatePanel navPanel;
	protected ZoomPanel zoomPanel;
	protected ProjectionStackTool projStack;
	protected ScaleTextPanel scalePanel;
	
	public NavigationBox(){
		
		super(BoxLayout.Y_AXIS, "Nav", "Nav", ThemaMapPanel.transferHandler);
		type = BoxLayout.Y_AXIS;
		setName(i18n.get(this, "navigation", "Navigation"));
		setNameTab(i18n.get(this, "navigation", "Navigation"));
		preferredExternSize = new Dimension(220,300);
		//setBackground(Color.WHITE);
		
		overviewMap = new OverviewMapHandler();
        overviewMap.setUseAsTool(false);
        overviewMap.setBorder(BorderFactory.createRaisedBevelBorder());
        overviewMap.setPropertyPrefix("OverviewMapHandler");
        children.add(overviewMap);
		
        navPanel = new NavigatePanel();
        navPanel.setPropertyPrefix("NavigatePanel");
        zoomPanel = new ZoomPanel();
        zoomPanel.setPropertyPrefix("ZoomPanel");
        projStack = new ProjectionStackTool();
        projStack.setPropertyPrefix("ProjectionStackTool");
        scalePanel = new ScaleTextPanel();
        scalePanel.setPropertyPrefix("ScaleTextPanel");
		
        children.add(navPanel);
        children.add(zoomPanel);
        children.add(projStack);
        children.add(scalePanel);
		
        JPanel toolLeft = new JPanel();
        toolLeft.setOpaque(false);
        JPanel toolRight = new JPanel(new GridLayout(2,1));
        toolRight.setOpaque(false);
        JPanel toolRightTop = new JPanel();
       // toolRightTop
        JPanel toolRightBottom = new JPanel();
        
       // Box toolBox = Box.createHorizontalBox();
        JPanel toolBox = new JPanel();
        toolBox.setLayout(new BoxLayout(toolBox,BoxLayout.X_AXIS));
        toolBox.add(toolLeft);
        toolBox.add(toolRight);
        toolRight.add(toolRightTop);
        toolRight.add(toolRightBottom);
        toolLeft.add(navPanel);
        toolRightTop.add(zoomPanel);
        toolRightTop.add(projStack);
        toolRightBottom.add(scalePanel);
        overviewMap.setBorder(BorderFactory.createLineBorder(Color.GRAY));
        overviewMap.setMaximumSize(new Dimension(205,135));
        overviewMap.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
        add(overviewMap);
        toolBox.setMaximumSize(new Dimension(200,100));
        add(toolBox);
//        toolBox.setBorder(BorderFactory.createLineBorder(Color.BLACK));
//        toolLeft.setBorder(BorderFactory.createLineBorder(Color.RED));
//        toolRight.setBorder(BorderFactory.createLineBorder(Color.GREEN));
        toolBox.setPreferredSize(new Dimension(toolBox.getWidth(),80));
        //JPanel ptool = new JPanel();
        //ptool.add(toolBox);
        //add(ptool);
        //JPanel p = new JPanel();
        //p.add(scalePanel);
        //scalePanel.setMaximumSize(new Dimension(220,15));
        //ptool.setMaximumSize(new Dimension(220,50));
       // add(p);
        
        //p.setOpaque(false);
        //scalePanel.setOpaque(false);
       // ptool.setOpaque(false);
        toolBox.setOpaque(false);
        
        updateUI();
	}
	
	/*PROPERTIES*/

//	
//	 public void setProperties(String prefix, Properties props) {
//			
//			super.setProperties(prefix, props);
//
//			prefix = PropUtils.getScopedPropertyPrefix(prefix);
//			Iterator it = children.iterator();
//			while (it.hasNext()) {
//				Object obj = it.next();
//				if (obj instanceof PropertyConsumer) {
//					//String newPrefix = prefix
//					//		+ ((PropertyConsumer) obj).getPropertyPrefix();
//					((PropertyConsumer) obj).setProperties(prefix, props);
//				}
//			}
//		 }
//	 
//
//	    public Properties getProperties(Properties props) {
//	        props = super.getProperties(props);
//	        Iterator it = children.iterator();
//	        while (it.hasNext()) {
//	            Object obj = it.next();
//	            if (obj instanceof PropertyConsumer) {
//	                ((PropertyConsumer)obj).getProperties(props);
//	            }
//	        }
//	        return props;
//	    }
    
    /*public Properties getProperties(Properties props) {
    
        props = super.getProperties(props);
        
        Iterator it = children.iterator();
        while (it.hasNext()) {
            Object obj = it.next();
            if (obj instanceof PropertyConsumer) {
                ((PropertyConsumer)obj).getProperties(props);
            }
        }
        return props;
    }*/
    
/*	    public Properties getPropertyInfo(Properties props) {
	        props = super.getPropertyInfo(props);

	        Iterator it = children.iterator();
	        while (it.hasNext()) {
	            Object obj = it.next();
	            if (obj instanceof PropertyConsumer) {
	                ((PropertyConsumer)obj).getPropertyInfo(props);
	            }
	        }
	        return props;
	    }
    */
	
	/*public static String navboxPropertyString = "navigationbox";
    public Properties getProperties(Properties props) {
    	props.put(navboxPropertyString+".class", getClass());
        return props;
    }*/
	public void findAndInit(Object someObj) {
    	if(someObj instanceof DisplayMenu){
    		DisplayMenu menu = (DisplayMenu) someObj;
    		setMenuItem(menu.getNavigationMenuItem());
    		//menu.getConfigurationMenuItem().setCheckSelected(showConfigurationPanel);
    	}
	}

	public LinkedList<JComponent> getChildren() {
		return children;
	}
	
}
