// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui;

import com.bbn.openmap.Layer;
import com.bbn.openmap.LayerHandler;
import fr.unicaen.openmap.AtlasLoader;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Thematic;
import java.awt.Component;
import java.io.File;
import java.util.Collections;
import java.util.HashMap;
import java.util.Vector;
import javax.swing.JComponent;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

/**
 *
 * @author jchauvea
 */
public class NoDirectoriesAtlasLayersTree extends AtlasLayersTree {

    public NoDirectoriesAtlasLayersTree(DefaultMutableTreeNode root, String path,
            LayerHandler lh, JComponent comp) {
        super(root, path, lh, comp);
        this.setRootVisible(false);
    }

    public NoDirectoriesAtlasLayersTree(DefaultTreeModel model, LayerHandler lh, JComponent cp) {
        super(model, lh, cp);
        LayerCellRenderer renderer = new LayerCellRenderer();
        setCellRenderer(new LayerCellRenderer());
        setCellEditor(new LayerCellEditor(this, renderer));
    }

    /** Add nodes from under "dir" into top. Highly recursive. 
     * Two differences with the AtlasLayersTree method :
     * - don't insert directories execept at root
     * - don't try to load layers as the are already loaded by the other tree.
     */
    @Override
    protected DefaultMutableTreeNode buildFileSystemTree(DefaultMutableTreeNode curTop, File dir) {
        String curPath = dir.getPath();
        DefaultMutableTreeNode curDir = null;
  	String curName = curPath;
	if (curPath.contains(File.separator)) {
            String[] tab;
       	    tab = curPath.split(File.separator.equals("\\") ?
				"\\" + File.separator :  File.separator);
            curName = tab[tab.length - 1];
	}
	curDir = new DefaultMutableTreeNode(new DirectoryCell(curName));
       
        if (curTop != null) { // should only be null at root
            if (dir.isFile()) {
                curTop.add(curDir);
            }
        }
        Vector<String> ol = new Vector<String>();
        String[] tmp = dir.list();
        if (tmp != null) {
            for (int i = 0; i < tmp.length; i++) {
                ol.addElement(tmp[i]);
            }
        }
        Collections.sort(ol, String.CASE_INSENSITIVE_ORDER);
        File f;
        Vector<String> files = new Vector<String>();
        // Make two passes, one for Dirs and one for Files. This is #1.
        for (int i = 0; i < ol.size(); i++) {
            String thisObject = (String) ol.elementAt(i);
            String newPath;
            if (curPath.equals(".")) {
                newPath = thisObject;
            } else {
                newPath = curPath + File.separator + thisObject;
            }
            if ((f = new File(newPath)).isDirectory()) {
                buildFileSystemTree(curTop, f);
            } else {
                files.addElement(thisObject);
            }
        }
        // Pass two: for properties files.
        for (int fnum = 0; fnum < files.size(); fnum++) {
            if (files.elementAt(fnum).endsWith(AtlasLoader.PROPERTIES_EXTENSION)) {
		DefaultMutableTreeNode pNode = 
		    new DefaultMutableTreeNode(
					       new PropertiesCell(
								  files.elementAt(fnum).replace(AtlasLoader.PROPERTIES_EXTENSION, "")));
		
                curTop.add(pNode);

                String tmpPath = curPath + File.separator + files.elementAt(fnum);
		// layers should be loaded by the other tree
                Layer[] ls = AtlasLayersTree.layersByPath.get(tmpPath);
                if (ls != null) {
                    buildTreeForLayers(pNode, AtlasLayersTree.layersByPath.get(tmpPath));
                }
                // }
            }
        }

        return curDir;
    }
}
