// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui;
/*
 * SelectAndReorderFromList.java
 * 
 * @author Jacques.Madelaine@unicaen.fr
 */

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.DropMode;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel; 
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.TransferHandler;

/**
 *  Provide a JPanel with two lists.
 * The first list is the original list.
 * The second list's contain can be dragged from the original and reordered.
 */
public class SelectAndReorderFromList extends JPanel {
    /**
     * The listener is called whenener the ListModel to is modified.
     */
    public SelectAndReorderFromList(DefaultListModel from, DefaultListModel to, 
				    String fromLabel, String toLabel,
				    ActionListener listener,
				    boolean removeItemFromFromList) {

        JPanel p = new JPanel();
        p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));

        JList fromList = new JList(from);
        JLabel label = new JLabel(fromLabel);
        label.setAlignmentX(0f);
        p.add(label);
        JScrollPane sp = new JScrollPane(fromList);
        sp.setAlignmentX(0f);
        p.add(sp);
        add(p, BorderLayout.WEST);
       
        JList toList = new JList(to);	

	p = new JPanel();
	p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
	JButton button;
	Dimension bdim = new Dimension(55, 25);
	button = new JButton(">>");
	fixComponentSize(button, bdim);
	button.addActionListener(new TranferButtonActionListener(fromList, toList, listener, true,
								 removeItemFromFromList, !removeItemFromFromList));
	p.add(button);
	button = new JButton(">");
	fixComponentSize(button, bdim);
	button.addActionListener(new TranferButtonActionListener(fromList, toList, listener, false, 
								 removeItemFromFromList, !removeItemFromFromList));
	p.add(button);
	button = new JButton("<");
	fixComponentSize(button, bdim);
	button.addActionListener(new TranferButtonActionListener(toList, fromList, listener, false, 
								 true, !removeItemFromFromList));
	p.add(button);
	button = new JButton("<<");
	fixComponentSize(button, bdim);
	button.addActionListener(new TranferButtonActionListener(toList, fromList, listener, true, 
								 true, !removeItemFromFromList));
	p.add(button);
	add(p, BorderLayout.CENTER);
	
        p = new JPanel();
        p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
        label = new JLabel(toLabel);
        label.setAlignmentX(0f);
        p.add(label);
        sp = new JScrollPane(toList);
        sp.setAlignmentX(0f);
        p.add(sp);
        p.setBorder(BorderFactory.createEmptyBorder(0, 2, 0, 0));
        add(p, BorderLayout.EAST);
        
        setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));

	TransferHandler transferHandler = new FromToTransferHandler(listener, fromList, toList, removeItemFromFromList);

        fromList.setTransferHandler(transferHandler);
        fromList.setPrototypeCellValue("WWWWWWWWWW");
        fromList.setDropMode(DropMode.INSERT);
        fromList.setDragEnabled(true);
        fromList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        toList.setTransferHandler(transferHandler);
	toList.setPrototypeCellValue("WWWWWWWWWW");
        toList.setDropMode(DropMode.INSERT);
	toList.setDragEnabled(true);
        toList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

    }

    private static void fixComponentSize(Component cp, Dimension d){
        cp.setMaximumSize(d);
        cp.setPreferredSize(d);
        cp.setMaximumSize(d);
    }   
 
    /*
    public static void main(String[] args) {
	Box box = Box.createVerticalBox();
	{
	    DefaultListModel from = new DefaultListModel();
	    final DefaultListModel copy = new DefaultListModel();
	    
	    for (int i = 15; i >= 0; i--) {
		from.add(0, "Source item " + i);
	    }
	    
	    for (int i = 2; i >= 0; i--) {
		copy.add(0, "Target item " + i);
	    }
	    
	    
	    ActionListener actionListener = new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			System.out.println(java.util.Arrays.asList(copy.toArray()));
		    }
		};
	    box.add(new SelectAndReorderFromList(from, copy, "Original", "Selection", actionListener, true));
	}
	{
	    DefaultListModel from = new DefaultListModel();
	    final DefaultListModel copy = new DefaultListModel();
	    
	    for (int i = 15; i >= 0; i--) {
		from.add(0, "Source item " + i);
	    }
	    
	    for (int i = 2; i >= 0; i--) {
		copy.add(0, "Target item " + i);
	    }
	    
	    
	    ActionListener actionListener = new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			System.out.println(java.util.Arrays.asList(copy.toArray()));
		    }
		};
	    box.add(new SelectAndReorderFromList(from, copy, "Original", "Selection", actionListener, false));
	}
       
	JFrame frame = new JFrame("SelectAndReorderFromList Demo");
	frame.add(box);
	// frame.setPreferredSize(new Dimension(400, 350));
	frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	frame.pack();
	frame.setVisible(true);
    }
    */
}

class FromToTransferHandler extends TransferHandler {
    private ActionListener listener;
    private JList originalFromList, originalToList;
    private boolean removeItemFromFromList;

    public FromToTransferHandler(ActionListener listener, JList fromList, JList toList, 
				 boolean removeItemFromFromList) {
	this.listener = listener;
	originalFromList = fromList;
	originalToList = toList;
	this.removeItemFromFromList = removeItemFromFromList;
    }
    
    /** a StringSelection supporting a given DataFlavor */	
    static class MyStringSelection extends StringSelection {
	DataFlavor[] dataFlavors;
	public MyStringSelection(String data, DataFlavor df) {
	    super(data.toString());
	    this.dataFlavors = new DataFlavor[] {df};
	}
	public DataFlavor[] getTransferDataFlavors() {
	    return dataFlavors;
	}
    }

    public int getSourceActions(JComponent comp) {
	return MOVE;
    }
    
    private int fromIndex = -1;
    private int toIndex = -1;
    private JList fromList, toList;
    private DataFlavor df = new DataFlavor(String.class, "text/a-"+this.hashCode());
    public Transferable createTransferable(JComponent comp) {
	fromList = (JList)comp;
	fromIndex = fromList.getSelectedIndex();
	if (fromIndex < 0 || fromIndex >= fromList.getModel().getSize()) {
	    return null;
	}
	// System.out.println("FromTransferHandler createTransferable "+ fromIndex + " " + this.hashCode()); 
	return new MyStringSelection((String)fromList.getSelectedValue(), df);
    }

    public void exportDone(JComponent comp, Transferable trans, int action) {
	if (action != MOVE) {
	    return;
	}
	// System.out.println("Remove from copy element at " + fromIndex + " -- " + toIndex + " fromList " + (fromList == toList ? "==" : "!=") + " toList ");
	if (fromList == toList && toIndex < fromIndex) 
	    fromIndex++;
	// System.out.println("Remove from copy element at " + fromIndex + " -- " + toIndex + " from == comp " + (fromList == ((JList)comp)));
	if (removeItemFromFromList || ((JList)comp) == originalToList || toList == originalFromList)
	    ((DefaultListModel)((JList)comp).getModel()).removeElementAt(fromIndex);
	if (listener != null)
	    listener.actionPerformed(null);
    }

        
    public boolean canImport(TransferHandler.TransferSupport support) {		
	//we'll only support drops (not clipboard paste)
	if (!support.isDrop())
	    return false;

	//  System.out.println("Import flavors? "+ java.util.Arrays.asList(support.getTransferable().getTransferDataFlavors()));

	// we only import our own flavor
	boolean supportMyStringFlavor = false;
	for (DataFlavor df : support.getTransferable().getTransferDataFlavors())
	    if (df == this.df)
		supportMyStringFlavor = true;
	if (! supportMyStringFlavor)
	    return false;
	
	// and support only MOVE
	if (! (support.getSourceDropActions() == MOVE))
	    return false;	
	
	support.setDropAction(MOVE);
	return true;
    }

    public boolean importData(TransferHandler.TransferSupport support) {
	// System.out.println("FromTo ImportData "+ support); 
	// if we can't handle the import, say so
	if (!canImport(support)) {
	    return false;
	}
	// System.out.println("FromTo canImport true"); 


	// fetch the drop location
	JList.DropLocation dl = (JList.DropLocation)support.getDropLocation();

	toIndex = dl.getIndex();

	// fetch the data and bail if this fails
	String data;
	try {
	    data = (String)support.getTransferable().getTransferData(DataFlavor.stringFlavor);
	    // System.out.println(" Import Data " + data + " -> " + toIndex); 
	} catch (UnsupportedFlavorException e) {
	    return false;
	} catch (java.io.IOException e) {
	    return false;
	}

	toList = (JList)support.getComponent();
	DefaultListModel model = (DefaultListModel)toList.getModel();
	// System.out.println("-Import Data " + data + " -> " + toIndex); 
	boolean checkForDuplicate = !removeItemFromFromList && toList != fromList;
	if (! checkForDuplicate || ! model.contains(data))
	    model.insertElementAt(data, toIndex);

	Rectangle rect = toList.getCellBounds(toIndex, toIndex);
	toList.scrollRectToVisible(rect);
	toList.setSelectedIndex(toIndex);
	toList.requestFocusInWindow();

	return true;
    }  
}

class TranferButtonActionListener implements ActionListener {
    private JList fromList, toList;
    private ActionListener listener;
    private boolean all; // apply to all items
    // remove item from the fromList
    private boolean removeItemFromFromList;
    // check for duplicate when addind items
    private boolean checkForDuplicate;
    public TranferButtonActionListener(JList fromList, JList toList, ActionListener listener, boolean all, boolean removeItemFromFromList, boolean checkForDuplicate) {
	this.fromList = fromList;
	this.toList = toList;
	this.listener = listener;
	this.all = all;
	this.removeItemFromFromList = removeItemFromFromList;
	this.checkForDuplicate = checkForDuplicate;
    }

    public void actionPerformed(ActionEvent ae) {
	DefaultListModel from =  ((DefaultListModel)fromList.getModel());
	DefaultListModel to =  ((DefaultListModel)toList.getModel());
	if (all) {
	    for(java.util.Enumeration e = from.elements(); e.hasMoreElements(); ) {
		Object s = e.nextElement();
		if (! checkForDuplicate || ! to.contains(s))
		    to.addElement(s);
	    }
	    if (removeItemFromFromList)
		from.clear();
	} else {
	    int fromIndex = fromList.getSelectedIndex();
	    if (fromIndex < 0 || fromIndex >= from.getSize()) {
		return;
	    }
	    Object s = from.getElementAt(fromIndex);
	    if (! checkForDuplicate || ! to.contains(s))
		to.addElement(s);
	    if (removeItemFromFromList)
		from.removeElementAt(fromIndex);
	}
	if (listener != null) // call user's listener
	    listener.actionPerformed(null);
    }
}
