// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui;

import com.bbn.openmap.Environment;
import com.bbn.openmap.gui.DimensionQueryPanel;
import com.bbn.openmap.util.PaletteHelper;

import fr.unicaen.openmap.util.gui.ThemaMapJFileChooser;
import fr.unicaen.openmap.util.Util;

import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import java.awt.BorderLayout;

import javax.swing.filechooser.FileFilter;
import java.util.regex.Pattern;
import javax.swing.filechooser.FileNameExtensionFilter;


/**
 *
 * @author jchauvea
 */
public class ThemaMapFileChooserManager {

    public final static String PROPERTIES_LOAD_PATH = "properties_load_path";
    public final static String PROPERTIES_SAVE_PATH = "properties_save_path";
    public final static String PROJECT_SAVE_PATH = "project_save_path";
    public final static String DATA_LOAD_PATH = "data_load_path";
    public final static String DATA_SAVE_PATH = "data_save_path";
    public final static String IMG_SAVE_PATH = "img_save_path";

    public static FileChooser gmlSaveFileChooser = 
	new FileChooser(Environment.get(DATA_SAVE_PATH, "."), DATA_SAVE_PATH,
			Environment.getI18n().get(ThemaMapFileChooserManager.class, "gmlFiles", 
						  "GML files (compressed or not)"),			
			"gml", "gmz");
    public static String dataSaveDialog(String title) {
        gmlSaveFileChooser.setDialogTitle(title);
        return gmlSaveFileChooser.getSaveString();
    }

    public static FileChooser csvSaveFileChooser =
	new FileChooser(Environment.get(DATA_SAVE_PATH, "."), DATA_SAVE_PATH,
			Environment.getI18n().get(ThemaMapFileChooserManager.class, "csvFiles", 
						  "CSV files"),			
			"csv", "csv.gz");
    public static String csvSaveDialog(String title) {
        csvSaveFileChooser.setDialogTitle(title);
        return csvSaveFileChooser.getSaveString();
    }

    public static ThemaMapJFileChooser dataLoadFileChooser = 
	new ThemaMapJFileChooser(Environment.get(DATA_LOAD_PATH, "."), DATA_LOAD_PATH);

    public static String dataLoadDialog(String title, FileFilter ff){
	return dataLoadFileChooser.showOpenDialog(title, ff);
    }

    public static FileChooser propertiesSaveFileChooser = 
	new FileChooser(Environment.get(PROPERTIES_SAVE_PATH, "."), PROPERTIES_SAVE_PATH, 
			Environment.getI18n().get(ThemaMapFileChooserManager.class, "propertiesFiles", 
						  "Properties files"),			
			"properties", "props");

    public static String propertiesSaveDialog(String title) {
        propertiesSaveFileChooser.setDialogTitle(title);
        return propertiesSaveFileChooser.getSaveString();
    }

    public static FileChooser exportProjectFileChooser = 
	new FileChooser(Environment.get(PROJECT_SAVE_PATH, "."), PROJECT_SAVE_PATH,
			Environment.getI18n().get(ThemaMapFileChooserManager.class, "projectFiles", 
						  "Project files"),			
			"zip");
    public static String exportProjectDialog(String title) {
        exportProjectFileChooser.setDialogTitle(title);
        return exportProjectFileChooser.getSaveString();
    }

    public static FileChooser propertiesLoadFileChooser = 
	new FileChooser(Environment.get(PROPERTIES_LOAD_PATH, "."), PROPERTIES_LOAD_PATH,
			Environment.getI18n().get(ThemaMapFileChooserManager.class, "propertiesFiles", 
						  "Properties files"),			
			"properties", "props");

    public static String propertiesLoadDialog(String title){
        propertiesLoadFileChooser.setDialogTitle(title);
        return propertiesLoadFileChooser.getLoadString();
    }

    public static ImageFileChooser imgSaveFileChooser = 
	new ImageFileChooser(Environment.get(IMG_SAVE_PATH, "."), IMG_SAVE_PATH, 
			     Environment.getI18n().get(ThemaMapFileChooserManager.class, "imageFiles", 
						       "Image files"),			
			     0,0,
			     "png", "jpg", "jpeg", "gif");

    public static String imageSaveDialog(String title, int w, int h, String extension){
        return imageSaveDialog(title, w, h, extension, JFileChooser.FILES_ONLY);
    }
    public static String imageSaveDialog(String title, int w, int h, String extension, int fileSelection){
	imgSaveFileChooser.setDialogTitle(title);
        imgSaveFileChooser.setImageWidth(w);
        imgSaveFileChooser.setImageHeight(h);
        imgSaveFileChooser.setFileSelectionMode(fileSelection);
	imgSaveFileChooser.setExtensions(extension);
	return imgSaveFileChooser.getSaveString();
    }


    public static ImageFileChooser imgTextSaveFileChooser = 
	new ImageFileChooser(Environment.get(IMG_SAVE_PATH, "."), IMG_SAVE_PATH, 
			     Environment.getI18n().get(ThemaMapFileChooserManager.class, "svgFiles", "SVG files"),
			     0,0,
			     "svg", "svg.gz");

    public static String imageTextSaveDialog(String title, int w, int h, String extension){
	imgTextSaveFileChooser.setDialogTitle(title);
        imgTextSaveFileChooser.setImageWidth(w);
        imgTextSaveFileChooser.setImageHeight(h);
	imgTextSaveFileChooser.setExtensions(extension);
	imgTextSaveFileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);

	return imgTextSaveFileChooser.getSaveString();
    }

    public static class FileChooser extends JFileChooser {
    
	private String configPath;
	protected String description;
	protected String[] extensions;
	protected Pattern extensionsPattern;
	public FileChooser(String currentDirectory, String configPath, String description, String... extensions) {
	    super(currentDirectory);
	    this.configPath = configPath;
	    this.description = description;
	    setExtensions(extensions);
	}
	
	public void setConfigPath(String configPath) {
	    this.configPath = configPath;
	}

	public void setDescription(String description) {
	    this.description = description;
	    setFileFilter(new FileNameExtensionFilter(description, extensions));
	}
    
	public void setExtensions(String ...extensions) {
	    this.extensions = extensions;
	    FileNameExtensionFilter fileFilter = new FileNameExtensionFilter(description, extensions);
	    setFileFilter(fileFilter);
	    this.extensions = fileFilter.getExtensions();
	    String p = null;
	    for (String e : extensions)
		if (p == null)
		    p = ".*\\.("+e;
		else
		    p += "|"+e;
	    p += ")$";
	    extensionsPattern = Pattern.compile(p);
	}

	public String getSaveString() {
	    String saveString = null;
	    //System.out.println("getSaveString "+ getFileFilter());
	    int ret = showSaveDialog(null);
	    if (ret == JFileChooser.APPROVE_OPTION) {
		// get the path
		saveString = getSelectedFile().getAbsolutePath();
		// check extension add it if not present
		if (! extensionsPattern.matcher(saveString).matches())
		    saveString += "." + extensions[0];
		File f = new File(saveString);
		if(f.exists() && getFileSelectionMode() != JFileChooser.DIRECTORIES_ONLY) {
		    String question = 
			String.format(
				      Environment.getI18n().get(ThemaMapFileChooserManager.class, "overwrite", 
								"File %s exists. Do you want to overwrite it?"), 
				      saveString);
		    ret = JOptionPane.showConfirmDialog(null, question);
		    if(ret == JOptionPane.NO_OPTION)
			return getSaveString();
		    if(ret == JOptionPane.CANCEL_OPTION)
			return null;
		}
	
		// update the environment
		Environment.getInstance().put(configPath,
					      getSelectedFile().getParentFile().getAbsolutePath());
	    }
	    return saveString;
	}

	public String getLoadString() {
	    String loadString = null;
	    int ret = showOpenDialog(null);
	    if (ret == JFileChooser.APPROVE_OPTION) {
		// get the path
		loadString = getSelectedFile().getAbsolutePath();
		// update the environment
		Environment.getInstance().put(configPath,
					      getSelectedFile().getParentFile().getAbsolutePath());
	    }
	    return loadString;
	}
    
    }

    /** rewrite of SaveAsImageFileChooser */
    public static class ImageFileChooser extends FileChooser {
 
	DimensionQueryPanel dqp = new DimensionQueryPanel();

	/**
	 * Create file chooser with the image size fields filled in.
	 */
	public ImageFileChooser(String currentDirectory, String configPath, String description, int width, int height, String... extensions) {
	    super(currentDirectory, configPath, description, extensions);
	    dqp.setFieldHeight(height);
	    dqp.setFieldWidth(width);
	    JPanel imageSizePanel = 
		PaletteHelper.createPaletteJPanel(Environment.
						  getI18n().get(ThemaMapFileChooserManager.class, "setImageSize", " Set Image Size "));
	    imageSizePanel.setLayout(new BorderLayout());
	    imageSizePanel.add(dqp, BorderLayout.CENTER);
	    setAccessory(imageSizePanel);
	}

	/**
	 * Set the value of the image width setting from the GUI.
	 */
	public void setImageWidth(int w) {
	    dqp.setFieldWidth(w);
	}
	
	/**
	 * Get the value of the image width setting from the GUI.
	 */
	public int getImageWidth() {
	    return dqp.getFieldWidth();
	}
	
	/**
	 * Set the value of the image height setting from the GUI.
	 */
	public void setImageHeight(int h) {
	    dqp.setFieldHeight(h);
	}
	
	/**
	 * Get the value of the image height setting from the GUI.
	 */
	public int getImageHeight() {
	    return dqp.getFieldHeight();
	}
    }
}

