// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
// **********************************************************************
// 
// <copyright>
// 
//  BBN Technologies
//  10 Moulton Street
//  Cambridge, MA 02138
//  (617) 873-8000
// 
//  Copyright (C) BBNT Solutions LLC. All rights reserved.
// 
// </copyright>
// **********************************************************************
// 
// $Source: /cvs/distapps/openmap/src/openmap/com/bbn/openmap/gui/BasicMapPanel.java,v $
// $RCSfile: BasicMapPanel.java,v $
// $Revision: 1.11.2.7 $
// $Date: 2006/08/09 21:01:24 $
// $Author: dietrick $
// 
// **********************************************************************

//
// The code from openmap BasicMapPanel is copy/pasted as we can't simply inhered from it
//
package fr.unicaen.openmap.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collection;
import java.util.Properties;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JToggleButton;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;
import javax.swing.plaf.basic.BasicSplitPaneUI;

import com.bbn.openmap.BufferedLayerMapBean;
import com.bbn.openmap.Environment;
import com.bbn.openmap.MapBean;
import com.bbn.openmap.MapHandler;
import com.bbn.openmap.MultipleSoloMapComponentException;
import com.bbn.openmap.PropertyHandler;
import com.bbn.openmap.gui.MapPanel;
import com.bbn.openmap.gui.MapPanelChild;
import com.bbn.openmap.gui.OMComponentPanel;
import com.bbn.openmap.gui.ToolPanel;
import com.bbn.openmap.gui.menu.MenuList;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.proj.ProjectionFactory;
import com.bbn.openmap.util.Debug;

import fr.unicaen.openmap.InformationDelegator;

import fr.unicaen.openmap.gui.component.ThemaMapSplitPane;
import fr.unicaen.openmap.gui.dnd.DndTransferHandler;
import fr.unicaen.openmap.gui.dnd.MovableBox;
import fr.unicaen.openmap.gui.dnd.MovableBoxesContainer;
import fr.unicaen.openmap.layer.chartLayer.ChartLayer;
import fr.unicaen.openmap.util.Util;
import java.awt.Insets;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.beans.PropertyChangeListener;
import javax.swing.border.EmptyBorder;

/**
 * The BasicMapPanel is a MapPanel and OMComponentPanel that is the
 * heart of the OpenMap application framework. It can be used in a
 * application or applet. The Panel has a BorderLayout, and creates a
 * MapBean for its center area. It creates a MapHandler to use to hold
 * all of its OpenMap components, and uses the PropertyHandler given
 * to it in its constructor to create and configure all of the
 * application components. The best way to add components to the
 * MapPanel is to get the MapHandler from it and add components to
 * that. The BasicMapPanel also adds itself to its MapHandler, so when
 * the PropertyHandler adds MapPanelChildren components to the
 * MapHandler, the BasicMapPanel is able to find them via the
 * findAndInit method. By default, the BasicMapPanel looks for
 * MapPanelChildren and asks them for where they would prefer to be
 * located (BorderLayout.NORTH, BorderLayout.SOUTH, BorderLayout.EAST,
 * BorderLayout.WEST). If you extend this component, though, other
 * components could be found via that same findAndInit method.
 */
public class ThemaMapPanel extends OMComponentPanel implements MapPanel, PropertyChangeListener {

    public static DndTransferHandler transferHandler =
            new DndTransferHandler(/*"text"*/);

    //protected JSplitPane splitCenterAndSouth, splitWestAndMapEast, splitEastAndMap;
    protected ThemaMapSplitPane splitWestAndCenterEast,  splitCenterEast,  splitMapSouth;
    protected JPanel main,  center,  mapPan,  eastMap,  mapAndInfDelegator;
    protected MovableBoxesContainer west,  east,  south;
    protected ThemaMapPanelsManager panelsManager;
    protected MapHandler mapHandler;
    protected MapBean mapBean;
    protected PropertyHandler propertyHandler;
    protected MenuList menuList;
    protected ToolPanel toolPanel;
    protected InformationDelegator informationDelegator;
    protected JToggleButton scrollButton;
    protected JToggleButton externMap;
    protected JScrollPane scrollMap;
    protected JFrame mapFrame = null;

    /**
     * Creates an empty MapPanel that creates its own empty
     * PropertyHandler. The MapPanel will contain a MapBean, a
     * MapHandler, and a PropertyHandler with no properties. The
     * constructor to use to create a blank map framework to add
     * components to.
     */
    public ThemaMapPanel() {
        this(new PropertyHandler(new Properties()), false);
    }

    /**
     * Create a MapPanel with the option of delaying the search for
     * properties until the <code>create()</code> call is made.
     * 
     * @param delayCreation true to let the MapPanel know that the
     *        artful programmer will call <code>create()</code>
     */
    public ThemaMapPanel(boolean delayCreation) {
        this(null, delayCreation);
    }

    /**
     * Create a MapPanel that configures itself with the properties
     * contained in the PropertyHandler provided. If the
     * PropertyHandler is null, a new one will be created.
     */
    public ThemaMapPanel(PropertyHandler propertyHandler) {
        this(propertyHandler, false);
    }

    /** 
     * Create a MapPanel that configures itself with properties
     * contained in the PropertyHandler provided, and with the option
     * of delaying the search for properties until the
     * <code>create()</code> call is made.
     * 
     * @param delayCreation true to let the MapPanel know that the
     *        artful programmer will call <code>create()</code>
     */
    public ThemaMapPanel(PropertyHandler propertyHandler, boolean delayCreation) {
        setPropertyHandler(propertyHandler);

        if (!delayCreation) {
            create();
        }
    }

    /**
     * The method that triggers setLayout() and createComponents() to
     * be called. If you've told the BasicMapPanel to delay creation,
     * you should call this method to trigger the PropertyHandler to
     * create components based on the contents of its properties.
     */
    public void create() {
        removeAll();

        setLayout(new BorderLayout());
        main = new JPanel(new BorderLayout());
        main.setBorder(null);
        add(main, BorderLayout.CENTER);

        center = new JPanel(new BorderLayout());

        //south = new JPanel(new BorderLayout());

        mapPan = new JPanel(new BorderLayout());
        scrollMap = new JScrollPane(mapPan);
        mapAndInfDelegator = new JPanel(new BorderLayout());
        mapAndInfDelegator.add(scrollMap, BorderLayout.CENTER);

        eastMap = new JPanel(new BorderLayout());

        splitMapSouth = new ThemaMapSplitPane(JSplitPane.VERTICAL_SPLIT, mapAndInfDelegator, null, true);
        //flattenSplitPane(splitMapSouth);
        splitCenterEast = new ThemaMapSplitPane(JSplitPane.HORIZONTAL_SPLIT, splitMapSouth, /*east*/ null, true);
        //flattenSplitPane(splitCenterEast);
        splitWestAndCenterEast = new ThemaMapSplitPane(JSplitPane.HORIZONTAL_SPLIT, null, splitCenterEast, true);
        //flattenSplitPane(splitWestAndCenterEast);
        //east.setVisible(false);
        splitMapSouth.setDividerLocation(580);

        main.add(splitWestAndCenterEast);
        (((BasicSplitPaneUI) splitWestAndCenterEast.getUI()).getDivider()).addComponentListener(new ComponentAdapter() {
            public void componentMoved(ComponentEvent ce) {
                if(ThemaMapPanel.this.panelsManager!=null && !ThemaMapPanel.this.panelsManager.isExternMap()){
                    int i = splitWestAndCenterEast.getDividerLocation();
                    toolPanel.setMargin(new Insets(0,i, 0, 0));
                    toolPanel.updateUI();
                }
            }
        });
        
        scrollButton = new JToggleButton(Util.loadIcon("noscroll.png"));
        scrollButton.setBorder(null);
        scrollButton.setBackground(Color.LIGHT_GRAY);
        scrollButton.setOpaque(false);
        scrollButton.setToolTipText(i18n.get(this, "noscroll", "Remove map scrollbar"));
        scrollButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (scrollButton.isSelected()) {
                    scrollButton.setIcon(Util.loadIcon("scroll.png"));
                    scrollButton.setToolTipText(i18n.get(ThemaMapPanel.this, "scroll", "Add map scrollbar"));
                    mapAndInfDelegator.remove(scrollMap);
                    mapAndInfDelegator.add(mapPan, BorderLayout.CENTER);
                    splitMapSouth.updateUI();
                    mapAndInfDelegator.updateUI();
                    updateUI();
                } else {
                    scrollButton.setIcon(Util.loadIcon("noscroll.png"));
                    scrollButton.setToolTipText(i18n.get(ThemaMapPanel.this, "noscroll", "Remove map scrollbar"));
                    mapAndInfDelegator.remove(mapPan);
                    scrollMap = new JScrollPane(mapPan);
                    mapAndInfDelegator.add(scrollMap, BorderLayout.CENTER);
                    splitMapSouth.updateUI();
                    mapAndInfDelegator.updateUI();
                    updateUI();
                }
            }
        });

        externMap = new JToggleButton(Util.loadIcon("externmap.png"));
        externMap.setBorder(null);
        externMap.setBackground(Color.LIGHT_GRAY);
        externMap.setOpaque(false);
        externMap.setToolTipText(i18n.get(this, "externmap", "Externalize the map"));
        externMap.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (externMap.isSelected()) {
                    externalizeMap();
                    updateUI();
                } else {
                    externMap.setIcon(Util.loadIcon("externmap.png"));
                    externMap.setToolTipText(i18n.get(ThemaMapPanel.this, "externmap", "Externalize the map"));
                    internalizeMap();
                    updateUI();
                }
            }
        });

        createComponents();


    }

    private void externalizeMap() {
        panelsManager.setExternMap(true);
        externMap.setIcon(Util.loadIcon("internmap.png"));
        externMap.setToolTipText(i18n.get(ThemaMapPanel.this, "internmap", "Internalize the map"));
        externMap.setSelected(true);
        updateUI();
        mapFrame = new JFrame();
        splitMapSouth.setLeftComponent(null);
        mapFrame.getContentPane().setLayout(new BorderLayout());
        mapFrame.getContentPane().add(mapAndInfDelegator, BorderLayout.CENTER);
        remove(toolPanel);
        updateUI();
        mapFrame.getContentPane().add(toolPanel, BorderLayout.NORTH);
        int envWidth = Environment.getInteger(Environment.Width,
                MapBean.DEFAULT_WIDTH);
        int envHeight = Environment.getInteger(Environment.Height,
                MapBean.DEFAULT_HEIGHT);
        mapFrame.setSize(new Dimension(envWidth, envHeight));
        mapFrame.setTitle(i18n.get(this, "map", "Map"));
        mapFrame.setVisible(true);
        splitMapSouth.updateUI();
        mapFrame.addWindowListener(new WindowAdapter() {

            public void windowClosing(WindowEvent arg0) {
                internalizeMap();
                externMap.setIcon(Util.loadIcon("externmap.png"));
            }
        });
        mapFrame.setAlwaysOnTop(ThemaMapPanelsManager.BOXES_ON_TOP);
        toolPanel.setMargin(new Insets(0, 0, 0, 0));
        toolPanel.updateUI();
    }

    private void internalizeMap() {
        splitMapSouth.setDividerLocation(splitMapSouth.getMaximumDividerLocation() / 2);

        splitMapSouth.setLeftComponent(mapAndInfDelegator);
        mapFrame.dispose();
        splitMapSouth.updateUI();
        add(toolPanel, BorderLayout.NORTH);
        panelsManager.setExternMap(false);
        int i = splitWestAndCenterEast.getDividerLocation();
        toolPanel.setMargin(new Insets(0,i , 0, 0));
        toolPanel.updateUI();
        updateUI();
    }

    /**
     * The constructor calls this method that sets the LayoutManager
     * for this MapPanel. It returns a BorderLayout by default, but
     * this method can be overridden to change how the MapPanel places
     * components. If you change what this method returns, you should
     * also change how components are added in the findAndInit()
     * method.
     */
    /*protected LayoutManager createLayoutManager() {
    return new BorderLayout();
    }*/
    /**
     * Position the map bean in this panel according to the layout
     * manger. Defaults to BorderLayout.CENTER.
     */
    protected void addMapBeanToPanel(MapBean map) {

        mapPan.add(map, BorderLayout.CENTER);
    //panMap.add(map, BorderLayout.CENTER);
    //splitTop.add(panMap, BorderLayout.CENTER);
    }

    /**
     * The constructor calls this method that creates the MapHandler
     * and MapBean, and then tells the PropertyHandler to create the
     * components described in its properties. This method calls
     * getMapHandler() and getMapBean(). If the PropertyHandler is not
     * null, it will be called to created components based on its
     * properties, and those components will be added to the
     * MapHandler in this MapPanel.
     */
    protected void createComponents() {

        PropertyHandler ph = getPropertyHandler();
        // Make sure the MapBean is created and added to the
        // MapHandler.
        MapBean mb = getMapBean();
        MapHandler mh = getMapHandler();
        mh.add(this);
        ph.createComponents(getMapHandler());

        // At this point, check the MapHandler to see if a
        // ProjectionFactory has been added. If it hasn't, create one
        // with the default ProjectionLoaders. We might want to
        // remove this at some point, but not having it here will
        // catch some people by suprise when 4.6.1 comes out.
        Object obj = mh.get(com.bbn.openmap.proj.ProjectionFactory.class);
        if (obj == null) {
            Debug.message("basic",
                    "ThemaMapPanel adding ProjectionFactory and projections to MapHandler since there are none to be found.");
            mh.add(ProjectionFactory.loadDefaultProjections());
        }

        // Environment will only get loaded after the property file is
        // read.
        //mb.setProjection((new ProjectionFactory()).getDefaultProjectionFromEnvironment(Environment.getInstance()));
//        mb.setProjection((new ProjectionFactory()).makeProjection(Environment.get(Environment.Projection),
//        		new LatLonPoint.Double(Environment.getDouble(Environment.Latitude, 0),
//                Environment.getDouble(Environment.Longitude, 0)),
//                Environment.getFloat(Environment.Scale,
//                        Float.POSITIVE_INFINITY),
//                0,
//                0));
        mb.setProjection(mb.getProjectionFactory().getDefaultProjectionFromEnvironment(Environment.getInstance()));
        mb.setBckgrnd(Environment.getCustomBackgroundColor());
    }

    /**
     * MapPanel method. Get the MapBean used for the MapPanel. If the
     * MapBean is null, calls createMapBean() which will create a
     * BufferedLayerMapBean and add it to the MapHandler via a
     * setMapBean call. If you want something different, override this
     * method.
     */
    public MapBean getMapBean() {
        if (mapBean == null) {
            setMapBean(ThemaMapPanel.createMapBean());
        }
        return mapBean;
    }

    /**
     * Set the map bean used in this map panel, replace the map bean
     * in the MapHandler if there isn't already one, or if the policy
     * allows replacement. The MapHandler will be created if it
     * doesn't exist via a getMapHandler() method call.
     * 
     * @throws MultipleSoloMapComponentException if there is already a
     *         map bean in the map handler and the policy is to reject
     *         duplicates (since the MapBean is a SoloMapComponent).
     */
    public void setMapBean(MapBean bean) {
        if (bean == null && mapBean != null) {
            // remove the current MapBean from the application...
            getMapHandler().remove(mapBean);
        }

        mapBean = bean;

        if (mapBean != null) {
            getMapHandler().add(mapBean);
            addMapBeanToPanel(mapBean);
        }
    }

    /**
     * Get the PropertyHandler containing properties used to configure
     * the panel, creating it if it doesn't exist.
     */
    public PropertyHandler getPropertyHandler() {
        if (propertyHandler == null) {
            setPropertyHandler(new PropertyHandler());
        }
        return propertyHandler;
    }

    /**
     * Set the PropertyHandler containing the properties used to
     * configure this panel. Adds the PropertyHandler to the
     * MapHandler. If the MapHandler isn't set at this point, it will
     * be created via a getMapHandler() call.
     */
    public void setPropertyHandler(PropertyHandler handler) {
        propertyHandler = handler;
        if (handler != null) {
            getMapHandler().add(handler);
        }
    }

    /**
     * MapPanel method. Get the MapHandler used for the MapPanel.
     * Creates a standard MapHandler if it hasn't been created yet.
     */
    public MapHandler getMapHandler() {
        if (mapHandler == null) {
            mapHandler = new MapHandler();
        }
        return mapHandler;
    }

    /**
     * MapPanel method. Get a JMenuBar containing menus created from
     * properties.
     */
    public JMenuBar getMapMenuBar() {
        if (menuList != null) {
            return menuList.getMenuBar();
        } else {
            return null;
        }
    }

    /**
     * MapPanel method. Get a JMenu containing sub-menus created from
     * properties.
     */
    public JMenu getMapMenu() {
        if (menuList != null) {
            return menuList.getMenu();
        } else {
            return null;
        }
    }

    //Map Component Methods:
    ////////////////////////
    /**
     * Adds a component to the map bean context. This makes the
     * <code>mapComponent</code> available to the map layers and
     * other components.
     * 
     * @param mapComponent a component to be added to the map bean
     *        context
     * @throws MultipleSoloMapComponentException if mapComponent is a
     *         SoloMapComponent and another instance already exists
     *         and the policy is a reject policy.
     */
    public void addMapComponent(Object mapComponent) {
        if (mapComponent != null) {
            getMapHandler().add(mapComponent);
        }
    }

    /**
     * Remove a component from the map bean context.
     * 
     * @param mapComponent a component to be removed to the map bean
     *        context
     * @return true if the mapComponent was removed.
     */
    public boolean removeMapComponent(Object mapComponent) {
        if (mapComponent != null) {
            return getMapHandler().remove(mapComponent);
        }
        return true;
    }

    /**
     * Given a Class, find the object in the MapHandler. If the class
     * is not a SoloMapComponent and there are more than one of them
     * in the MapHandler, you will get the first one found.
     */
    public <T> T getMapComponentByType(Class<T> c) {
        return getMapHandler().get(c);
    }

    /**
     * Get all of the mapComponents that are of the given class type.
     */
    public <T> Collection<T> getMapComponentsByType(Class<T> c) {
        return getMapHandler().getAll(c);
    }

    /**
     * Find the object with the given prefix by looking it up in the
     * prefix librarian in the MapHandler.
     */
    public Object getMapComponent(String prefix) {
        return getPropertyHandler().get(prefix);
    }

    /**
     * The BasicMapPanel looks for MapPanelChild components, finds out
     * from them where they prefer to be placed, and adds them.
     */
    public void findAndInit(Object someObj) {
        if (someObj instanceof MapPanelChild && someObj instanceof Component) {
            if (Debug.debugging("basic")) {
                Debug.output("MapPanel: adding " + someObj.getClass().getName());
            }
            MapPanelChild mpc = (MapPanelChild) someObj;
            addMapPanelChild(mpc);
            invalidate();
        }

        if (someObj instanceof MenuList) {
            menuList = (MenuList) someObj;
        }

    }

    /**
     * Add a child to the MapPanel. 
     */
    protected void addMapPanelChild(MapPanelChild mpc) {
        if (mpc instanceof InformationDelegator) {
            informationDelegator = (InformationDelegator) mpc;
            informationDelegator.setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0, new Color(100, 100, 100)));
            informationDelegator.addScrollButton(scrollButton);
            informationDelegator.addExternMapButton(externMap);
            mapAndInfDelegator.add(informationDelegator, BorderLayout.SOUTH);
            return;
        }
        if (mpc instanceof ToolPanel) {
            toolPanel = (ToolPanel) mpc;
            add(toolPanel, BorderLayout.NORTH);
            return;
        }
        if (mpc instanceof ThemaMapPanelsManager) {
        	try{
            panelsManager = (ThemaMapPanelsManager) mpc;
            south = panelsManager.getSouthBox();
            splitMapSouth.setRightComponent(south);
            panelsManager.setLayerGUIPanel(panelsManager.getConfigPanel());
            west = panelsManager.getWestBox();
            splitWestAndCenterEast.setLeftComponent(west);
            east = panelsManager.getEastBox();
            splitCenterEast.setRightComponent(east);
            splitCenterEast.updateUI();
            panelsManager.getWestBox().updateGUI();
            panelsManager.getEastBox().updateGUI();
            panelsManager.getSouthBox().updateGUI();
            panelsManager.getWestBox().setMinimumSize(new Dimension(220,
                    panelsManager.getWestBox().getHeight()));
            panelsManager.getNavigationBox().addTarget(east);
            panelsManager.getLayersBox().addTarget(east);
            panelsManager.getNavigationBox().addTarget(west);
            panelsManager.getLayersBox().addTarget(west);
            panelsManager.getNavigationBox().addTarget(south);
            panelsManager.getLayersBox().addTarget(south);
            panelsManager.getConfigBox().addTarget(east);
            panelsManager.getConfigBox().addTarget(west);
            panelsManager.getConfigBox().addTarget(south);
        	}
        	catch(NullPointerException e){
        		System.err.println("One/several boxe(s) is/are missing");
        	}
            if (panelsManager.isExternMap()) {
                externalizeMap();
            }
            if (!panelsManager.isScrollableMap()) {
                scrollButton.doClick();
            }
            closeEmptySplitPanes();
            return;
        }

    }

    private void closeEmptySplitPanes() {
        if (east.getNbBox() == 0) {
            east.setVisible(false);
        }
        if (south.getNbBox() == 0) {
            south.setVisible(false);
        }
        if (west.getNbBox() == 0) {
            west.setVisible(false);
        }

    }

    public void groupBoxes() {
        west.removeAll();
        east.removeAll();
        south.removeAll();

        splitMapSouth.setRightComponent(south);
        splitWestAndCenterEast.setLeftComponent(west);
        splitCenterEast.setRightComponent(east);

        MovableBox layersBox = panelsManager.getLayersBox();
        MovableBox navigationBox = panelsManager.getNavigationBox();
        MovableBox configurationBox = panelsManager.getConfigBox();

        if (layersBox.isExternalized()) {
            layersBox.killParent();
        }
        panelsManager.getWestBox().addElement(layersBox);
        if (navigationBox.isExternalized()) {
            navigationBox.killParent();
        }
        panelsManager.getWestBox().addElement(layersBox);
        if (configurationBox.isExternalized()) {
            configurationBox.killParent();
        }
        panelsManager.getSouthBox().addElement(layersBox);

        west.add(panelsManager.getWestBox());
        south.add(panelsManager.getSouthBox());


    }

    /**
     * The MapPanel looks for MapPanelChild components and removes
     * them from iteself.
     */
    public void findAndUndo(Object someObj) {
        if (someObj instanceof MapPanelChild && someObj instanceof Component) {
            if (Debug.debugging("basic")) {
                Debug.output("MapPanel: removing " + someObj.getClass().getName());
            }
            remove((Component) someObj);
            invalidate();
        }

        if (someObj instanceof MenuList && menuList == someObj) {
            menuList = null;
        }
    }

    //MapBean Methods:
    //////////////////
    /**
     * A static method that creates a MapBean with it's projection set
     * to the values set in the Environment. Also creates a
     * BevelBorder.LOWERED border for the MapBean.
     */
    public static MapBean createMapBean() {
        int envWidth = Environment.getInteger(Environment.Width,
                MapBean.DEFAULT_WIDTH);
        int envHeight = Environment.getInteger(Environment.Height,
                MapBean.DEFAULT_HEIGHT);

        if (envWidth <= 0 || envHeight <= 0) {
            Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
            if (envWidth <= 0) {
                envWidth = (int) d.getWidth();
            }
            if (envHeight <= 0) {
                envHeight = (int) d.getHeight();
            }
        }

        Projection proj = (new ProjectionFactory()).getDefaultProjectionFromEnvironment(Environment.getInstance());

        if (Debug.debugging("mappanel")) {
            Debug.output("MapPanel: creating MapBean with initial projection " + proj);
        }

        return createMapBean(proj,BorderFactory.createEmptyBorder());
    }

    /**
     * A static method that creates a MapBean and sets its projection
     * and border to the values given.
     */
    public static MapBean createMapBean(Projection proj, Border border) {
        MapBean mapBeano = new BufferedLayerMapBean();
        mapBeano.setBorder(border);
        mapBeano.setProjection(proj);
        mapBeano.setPreferredSize(new Dimension(proj.getWidth(), proj.getHeight()));
        return mapBeano;
    }

    //Property Functions:
    /////////////////////
    /**
     * Get the current properties.
     */
    public Properties getProperties() {
        return getPropertyHandler().getProperties();
    }

    /**
     * Remove an existing property if it exists.
     * 
     * @return true if a property was actually removed.
     */
    public boolean removeProperty(String property) {
        return getPropertyHandler().removeProperty(property);
    }

    /**
     * Add (or overwrite) a property to the current properties
     */
    public void addProperty(String property, String value) {
        getPropertyHandler().addProperty(property, value);
    }

    /**
     * Add in the properties from the given URL. Any existing
     * properties will be overwritten except for openmap.components,
     * openmap.layers and openmap.startUpLayers which will be
     * appended.
     */
    public void addProperties(URL urlToProperties) {
        getPropertyHandler().addProperties(urlToProperties);
    }

    /**
     * Add in the properties from the given source, which can be a
     * resorce, file or URL. Any existing properties will be
     * overwritten except for openmap.components, openmap.layers and
     * openmap.startUpLayers which will be appended.
     * 
     * @throws MalformedURLException if propFile doesn't resolve
     *         properly.
     */
    public void addProperties(String propFile)
            throws java.net.MalformedURLException {
        getPropertyHandler().addProperties(propFile);
    }

    /**
     * remove a marker from a space delimated set of properties.
     */
    public void removeMarker(String property, String marker) {
        getPropertyHandler().removeMarker(property, marker);
    }

    /**
     * Add in the properties from the given Properties object. Any
     * existing properties will be overwritten except for
     * openmap.components, openmap.layers and openmap.startUpLayers
     * which will be appended.
     */
    public void addProperties(Properties p) {
        getPropertyHandler().addProperties(p);
    }

    /**
     * Append the given property into the current properties
     */
    public void appendProperty(String property, Properties src) {
        getPropertyHandler().appendProperty(property, src);
    }

    /**
     * Append the given property into the current properties
     */
    public void appendProperty(String property, String value) {
        getPropertyHandler().appendProperty(property, value);
    }

    /**
     * Prepend the given property into the current properties
     */
    public void prependProperty(String property, Properties src) {
        getPropertyHandler().prependProperty(property, src);
    }

    /**
     * Prepend the given property into the current properties
     */
    public void prependProperty(String property, String value) {
        getPropertyHandler().prependProperty(property, value);
    }

//    public static void flattenSplitPane(JSplitPane jSplitPane) {
//    	jSplitPane.setDividerSize(6);
//        jSplitPane.setUI(new BasicSplitPaneUI() {
//            public BasicSplitPaneDivider createDefaultDivider() {
//                return new BasicSplitPaneDivider(this) {
//                    public void setBorder(Border b) {
//                    }
//                };
//            }
//        });
//        jSplitPane.setBorder(null);
//        //jSplitPane.setOneTouchExpandable(true);
//        
//    }
    public ThemaMapPanelsManager getPanelsManager() {
        return panelsManager;
    }

    public void dispose() {
    }

    public void propertyChange(PropertyChangeEvent pce) {
        String action = pce.getPropertyName();
        if(action.equals(LayersPanelTree.LayerSelectedCmd) ||
                action.equals(LayersPanelTree.ThematicSelectedCmd)){
                Object ob = pce.getNewValue();
                if(ob instanceof ChartLayer){
                    displayChart((ChartLayer)ob);
                }
                else{
                    mapPan.removeAll();
                    addMapBeanToPanel(mapBean);
                }
                mapPan.updateUI();
       }
    }

    private void displayChart(ChartLayer chartLayer){
            mapPan.removeAll();
            mapPan.add(chartLayer.getChartGUI(), BorderLayout.CENTER);

    }

}
