// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>


package fr.unicaen.openmap.gui;

/**
 *
 * @author jchauvea
 */
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.File;
import javax.imageio.ImageIO;



public class TransparentBackground extends JComponent
		implements ComponentListener, WindowFocusListener{

	// constants ---------------------------------------------------------------
	// instance ----------------------------------------------------------------
	private JFrame frame;
	private BufferedImage background;
	//private long lastUpdate = 0;
	//private boolean refreshRequested = true;
	private Robot robot;
	private Rectangle screenRect;
	private ConvolveOp blurOp;

	// constructor -------------------------------------------------------------

	public TransparentBackground(JFrame frame) {

		frame = frame;
		try {
			robot = new Robot();
		} catch (AWTException e) {
			e.printStackTrace();
			return;
		}

		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		screenRect = new Rectangle(dim.width, dim.height);
		float[] my_kernel = {
				0.10f, 0.10f, 0.10f,
				0.10f, 0.20f, 0.10f,
				0.10f, 0.10f, 0.10f};
		blurOp = new ConvolveOp(new Kernel(3, 3, my_kernel));

		updateBackground();
		frame.addComponentListener(this);
		frame.addWindowFocusListener(this);
		//new Thread(this).start();
	}

	// protected ---------------------------------------------------------------

	protected void updateBackground() {
            background = robot.createScreenCapture(screenRect);
	}

    /*public int getBackgroundWidth(){
        return background.get
    }*/


	protected void refresh() {
		if (frame !=null && frame.isVisible() && this.isVisible()) {
			repaint();
			//refreshRequested = true;
			//lastUpdate = System.currentTimeMillis();
		}
	}


	// JComponent --------------------------------------------------------------

	protected void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		Point pos = this.getLocationOnScreen();
		BufferedImage buf = new BufferedImage(getWidth()+2, getHeight()+2, BufferedImage.TYPE_INT_RGB);
		buf.getGraphics().drawImage(background, -pos.x, -pos.y, null);
		Image img = blurOp.filter(buf, null);
		g2.drawImage(img,-1, -1, null);
		//g2.setColor(new Color(255, 255, 255, 0));
		//g2.drawRect(0, 0, getWidth(), getHeight());
	}

	// ComponentListener -------------------------------------------------------
	public void componentHidden(ComponentEvent e) {
	}

	public void componentMoved(ComponentEvent e) {
		repaint();
	}

	public void componentResized(ComponentEvent e) {
		repaint();

	}

	public void componentShown(ComponentEvent e) {
		repaint();
	}

	// WindowFocusListener -----------------------------------------------------
	public void windowGainedFocus(WindowEvent e) {
		refresh();
	}

	public void windowLostFocus(WindowEvent e) {
		refresh();
	}



	public static void main(String[] args) {
		JFrame frame = new JFrame();
        frame.setAlwaysOnTop(true);
        frame.setUndecorated(true);
        //frame.setDefaultLookAndFeelDecorated(false);
		TransparentBackground bg = new TransparentBackground(frame);
        bg.setBorder(null);
        SplashScreenPanel spc = new SplashScreenPanel("splashScreen.png");
        spc.setBorder(null);
        spc.setOpaque(false);
        bg.add(spc);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.getContentPane().add(bg);
		//frame.setLocation(500, 500);
        frame.pack();
        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
        frame.setLocation((screen.width - frame.getSize().width)/2,(screen.height - frame.getSize().height)/2);
        //frame.setLocationRelativeTo(null);
		frame.setVisible(true);
	}
}
