// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui.dnd;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;


import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.border.Border;


public class DropArea extends DndLabel{
	
	protected JFrame parentFrame;
	protected DndTransferHandler dndHandler;
	
	public final static Dimension DEFAULT_DIMENSION = new Dimension(100,5);
	public final static Dimension DEFAULT_MAX_DIMENSION = new Dimension(1000,5);
	public final static Color BORDER_COLOR = new Color(100,100,100);
	public final static Border DEFAULT_BORDER = BorderFactory.createRaisedBevelBorder();
	
	public DropArea(DndTransferHandler handler, MovableBoxesContainer boxesContainer){
		super("");
		//setPreferredSize(defaultDimension);
		setMinimumSize(DEFAULT_DIMENSION);
		setMaximumSize(DEFAULT_MAX_DIMENSION);
		this.boxesContainer = boxesContainer;
		dndHandler = handler;
		setAlignmentX(Component.CENTER_ALIGNMENT);
		setBorder(DEFAULT_BORDER);
		parentFrame = null;
		setTransferHandler(handler);
		handler.addDropArea(this);
	   /* MouseListener ml = new MouseAdapter(){

	        public void mousePressed(MouseEvent e){        
	          JComponent jc = (JComponent)e.getSource();
	          DndTransferHandler th = (DndTransferHandler)jc.getTransferHandler();
	          th.exportAsDrag(jc, e, DndTransferHandler.COPY);
	        }
	      };
	      
	    addMouseListener(ml);*/
	}
	
	public void hover(Dimension dim){
		//setBackground(Color.RED);
		setPreferredSize(dim);
		setMaximumSize(dim);
		setBorder(BorderFactory.createLineBorder(BORDER_COLOR));
		updateUI();
	}
	public void unhover(){
		//setBackground(Color.LIGHT_GRAY);
		setPreferredSize(DEFAULT_DIMENSION);
		setMaximumSize(DEFAULT_MAX_DIMENSION);
		setBorder(DEFAULT_BORDER);
		updateUI();
	}
	
	public DropArea (DndTransferHandler handler, MovableBoxesContainer boxesContainer, JFrame
			parentFrame){
		this(handler, boxesContainer);
		this.parentFrame = parentFrame;
	}
	
	public void receive(DragArea source){

		source.getBox().showDragLabel(true);
		if(source.getBox().isTabMode()){
			source.getBox().setTabMode(false);
			source.getBox().detachFromTabbed();
		}
		
		MovableBoxesContainer receiverContainer = getBoxesContainer();
		MovableBoxesContainer sourceContainer = source.getBoxesContainer();
		
		source.getBox().getParentBoxContainer().removeElement(source.getBox());
		
		if(source.getBox().isExternalized() && !source.getBoxesContainer().containsBoxes())
			source.getBox().killParent();
		
		source.getBox().setParentBoxContainer(receiverContainer);


		int indexReceiver = receiverContainer.getElementIndex(this);
		
		if(indexReceiver == 0){
			receiverContainer.addElement(source.getBox(), indexReceiver + 1);
		}
		else{
			receiverContainer.addElement(source.getBox(), indexReceiver);
		}

		receiverContainer.updateGUI();
		if(sourceContainer != receiverContainer){
			sourceContainer.updateGUI();
			source.getBox().setParentBoxContainer(receiverContainer);
		}

		
		if(source.getBox().getParentFrame() != null){
			String title = source.getBox().getParentFrame().getTitle();
			source.getBox().getParentFrame().setTitle(title.replaceAll(source.getText(), ""));
		}
		
		if(getParentFrame()!=null){
			getParentFrame().setTitle(getParentFrame().getTitle()+" "+source.getText());
			source.getBox().setParentFrame(getParentFrame());
		}
		
		
		//source.setBoxesContainer(receiverContainer);
		source.getBox().withPadding(true);
		source.getBox().bExternalize.setVisible(true);
		source.getBox().setExternalized(false);
		receiverContainer.updateUI();
		sourceContainer.updateUI();
		sourceContainer.updateGUI();
		
	}

	public JFrame getParentFrame() {
		return parentFrame;
	}

	public void setParentFrame(JFrame parentFrame) {
		this.parentFrame = parentFrame;
	}



}
