// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui.dnd;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.text.TabExpander;

import com.bbn.openmap.gui.OMComponentPanel;


import fr.unicaen.openmap.gui.DisplayMenuItem;
import fr.unicaen.openmap.gui.ThemaMapPanelsManager;
import fr.unicaen.openmap.layer.dataDisplayLayer.gui.SimplePalette;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.util.Util;

/**
 * 
 * @author jchauvea
 *
 */
public class MovableBox /*extends Box*/ extends OMComponentPanel {

    protected String name, nameTab;
    protected DragArea dragArea;
    protected MovableBoxesContainer parentBoxContainer, oldContainer;
    protected DndTransferHandler transferHandler;
    protected JFrame parentFrame;
    protected boolean externalized;
    protected DisplayMenuItem menuItem;
    protected JPanel panDragLabel;
    protected JButton bExternalize;
    protected int type;
    protected Dimension preferredExternSize;
    //tab mode
    protected DragTabArea dragTabArea;
    private int defaultPosition;
    private boolean tabMode;
    private boolean isTabbeded;
    private boolean hidden;
    protected JTabbedPane tabbed;
    protected ArrayList<MovableBoxesContainer> targets;
    public static Color dragBorderColor = new Color(100, 100, 100);
    public static int preferredSideSize = 220;

    public MovableBox() {
        super();
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        //type = BoxLayout.Y_AXIS;
    }

    public MovableBox(int type, String name, String nameTab,
            DndTransferHandler th) {
        super();
        setLayout(new BoxLayout(this, type));
        this.name = name;
        this.nameTab = nameTab;
        //this.parentBoxContainer = parent;
        this.transferHandler = th;
        tabMode = false;
        isTabbeded = true;
        targets = new ArrayList<MovableBoxesContainer>();
        preferredExternSize = new Dimension(800, 600);
        //defaultContainer = parent;
        createGUI();
    }

    public MovableBox(String name, String nameTab, /*MovableBoxesContainer parent,*/
            DndTransferHandler th, JTabbedPane tabbedPane) {
        this(BoxLayout.Y_AXIS, name, nameTab/*, parent*/, th);
        showDragLabel(false);
        //parent.addElement(this);
        tabbed = tabbedPane;
        defaultPosition = 0;
        tabMode = true;
    }

    protected void createGUI() {
        dragArea = new DragArea(name,
                this,
                transferHandler,
                parentBoxContainer);
        dragTabArea = new DragTabArea(nameTab,
                this,
                transferHandler,
                parentBoxContainer);
        panDragLabel = new JPanel(new BorderLayout());
        panDragLabel.add(dragArea, BorderLayout.CENTER);
        dragArea.setAlignmentX(JComponent.CENTER_ALIGNMENT);
        panDragLabel.setBorder(BorderFactory.createLineBorder(dragBorderColor, 1));
        //panDragLabel.setBackground(new Color(220,220,220));
        panDragLabel.setBackground(new Color(162, 181, 205));
        panDragLabel.setMaximumSize(new Dimension(2000, 25));
        // panDragLabel.setPreferredSize(new Dimension(2000,25));
        //panDrag.setPreferredSize(new Dimension(210,25));
        //panDragLabel = new JPanel();
        //panDragLabel.setMinimumSize(new Dimension(panDragLabel.getWidth(), 30));
        //panDragLabel.add(panDrag);
        bExternalize = new JButton(Util.loadIcon("externmap.png"));
        bExternalize.setBackground(Color.LIGHT_GRAY);
        bExternalize.setBorder(null);
        bExternalize.setOpaque(false);
        bExternalize.setAlignmentX(JButton.RIGHT_ALIGNMENT);
        panDragLabel.add(bExternalize, BorderLayout.EAST);
        bExternalize.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                externalize(new Point(0, 0));
            }
        });
        add(panDragLabel);
        externalized = false;
        setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        //setBackground(Color.WHITE);
    }

    public void showAt(int index) {
        if (tabbed != null) {
            defaultPosition = index;
            withPadding(false);
            tabbed.setTabComponentAt(index, dragTabArea);
            dragTabArea.setTabbed(tabbed);
            //dragTabArea.setBoxesContainer(parentBoxContainer);
            tabbed.updateUI();

        }
    }

    public void showDragLabel(boolean b) {
        panDragLabel.setVisible(b);
        updateUI();
    }

    public void withPadding(boolean b) {
        if (b) {
            setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        } else {
            setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
        }
    }

    public void externalize(Point location) {
        externalize(location, type);
    }

    public void externalize(Point location, int type) {
        try {
            if (externalized && parentBoxContainer.getNbBox() == 1) {
                return;
            }
            if (tabMode) {
                showDragLabel(true);
            }
            externalized = true;

            if (parentFrame != null) {
                String title = parentFrame.getTitle();
                parentFrame.setTitle(title.replaceAll(dragArea.getText(), ""));
            }
            bExternalize.setVisible(false);
            MovableBoxesContainer box = new MovableBoxesContainer(type);
            oldContainer = parentBoxContainer;
            JFrame f = buildScrollableFrame(box, this.getPreferredSize());
            f.setLocation(location);
            if (type == BoxLayout.Y_AXIS) {
                box.addElement(new DropArea(transferHandler, box, f));
            } else {
                box.addElement(new VerticalDropArea(transferHandler, box, f));
            }
            if (getParentBoxContainer() != null) {
                getParentBoxContainer().removeElement(this);
                getParentBoxContainer().updateGUI();
            }
            box.addElement(this);
            if (type == BoxLayout.Y_AXIS) {
                box.addElement(new DropArea(transferHandler, box, f));
            } else {
                box.addElement(new VerticalDropArea(transferHandler, box, f));
            }
            //dragArea.getBoxesContainer().removeElement(this);
            if (parentFrame != null && externalized && !parentBoxContainer.containsBoxes()) {
                parentFrame.dispose();
                parentFrame = null;
            }
            parentFrame = f;
            f.addWindowListener(new WindowAdapter() {

                public void windowClosing(WindowEvent arg0) {
                    parentBoxContainer.close();
                    parentFrame = null;
                    hidden = true;
                }
            });

            if (tabMode) {
                detachFromTabbed();
                tabMode = false;
            }
            box.updateGUI();
            f.setSize(preferredExternSize);
            f.setAlwaysOnTop(ThemaMapPanelsManager.BOXES_ON_TOP);
            f.setVisible(true);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /*public void freeze(){
    bExternalize.setVisible(false);
    dragArea.setTransferHandler(null);
    }

    public void unFreeze(){
    bExternalize.setVisible(true);
    dragArea.setTransferHandler(transferHandler);
    }*/
    public void detachFromTabbed() {
        showDragLabel(true);
        defaultPosition = dragTabArea.getTabbedIndex();
        tabbed.removeTabAt(defaultPosition);
        //tabbed.remove(defaultPosition);
//        tabbed.updateUI();
//        if (defaultPosition == tabbed.getTabCount() - 1) {
//            tabbed.setSelectedIndex(defaultPosition - 1);
//        }
        tabbed.updateUI();
        //SimplePalette.customTabbedPaneUI(tabbed, tabbed.getTabCount() - 1);
       
//        if(tabbed.getTabCount() - 1 > 1)//hack !?!
//            tabbed.removeTabAt(tabbed.getTabCount() - 1);
//        tabbed.updateUI();


    }

    protected JFrame buildScrollableFrame(Component c, Dimension dim) {
        JFrame f = new JFrame(name);
        JScrollPane p = new JScrollPane(c);
        f.getContentPane().add(p);
        f.setSize(dim);
        return f;
    }

    public void killParent() {
        if (parentFrame != null) {
            parentFrame.dispose();
            parentFrame = null;

        }
    }

    public void addTarget(MovableBoxesContainer comp) {
        targets.add(comp);
    }

    public void removeTarget(MovableBoxesContainer comp) {
        targets.remove(comp);
    }

    public boolean isTarget(JComponent comp) {
        return targets.contains(comp);
    }

    public void showTargets() {
        for (MovableBoxesContainer target : targets) {
            if (!target.isVisible()) {
                if (target.getParent() instanceof JSplitPane) {
                    JSplitPane p = (JSplitPane) target.getParent();
                    if (target == p.getRightComponent()) {
                        target.setVisible(true);
                        p.setDividerLocation(p.getMaximumDividerLocation() - getMinimumSize().width /*- preferredSideSize*/);
                        p.updateUI();
                    } else {//left
                        target.setVisible(true);
                        p.setDividerLocation(/*preferredSideSize*/getMinimumSize().width);
                        p.updateUI();
                    }
                }
            }
        }
    }

    public void hideEmptyTargets() {
        for (MovableBoxesContainer target : targets) {
            if (!target.containsBoxes()) {
                target.setVisible(false);
            }
        }
    }

    public void closeParent() {
        killParent();
    }

    public void close() {
        /*if (menuItem != null) {
        menuItem.setCheckSelected(false);
        } else {*/
        if (tabbed != null) {
            //setTabbeded(true);
            showDragLabel(false);
            tabbed.insertTab(name, null, this, "", defaultPosition);
            //tabbed.addTab(name, this);
            //setTabbed(tabbed);
            showAt(defaultPosition);
            //SimplePalette.customTabbedPaneUI(tabbed, tabbed.getTabCount() - 1);
            tabbed.setSelectedIndex(defaultPosition);
            tabbed.updateUI();
            dragArea.updateUI();
        } else {
            //  }
            if (oldContainer == null) { 
		if (menuItem != null) {
		    // deselect menu item menu command view config
		    menuItem.setCheckSelected(false);
		}
	    } else if (oldContainer.getComponentCount() > 0) {
                try {
                    DropArea da = (DropArea) oldContainer.getComponent(0);
                    da.receive(dragArea);
                } catch (ClassCastException exc) {
                    if (menuItem != null) {
                        menuItem.setCheckSelected(false);
                    }
                }
            }
        }
        externalized = false;
        parentFrame = null;
        //oldContainer.updateUI();
    }

    public boolean isExternalized() {
        return externalized;
    }

    public void setExternalized(boolean externalized) {
        this.externalized = externalized;
    }

    public JFrame getParentFrame() {
        return parentFrame;
    }

    public void setParentFrame(JFrame parentFrame) {
        this.parentFrame = parentFrame;
    }

    public void setMenuItem(DisplayMenuItem menuItem) {
        this.menuItem = menuItem;
    }

    public MovableBoxesContainer getParentBoxContainer() {
        return parentBoxContainer;
    }

    public void setParentBoxContainer(MovableBoxesContainer parentBoxContainer) {
        this.parentBoxContainer = parentBoxContainer;
        dragArea.setBoxesContainer(parentBoxContainer);
        dragTabArea.setBoxesContainer(parentBoxContainer);
    }

    public boolean isTabMode() {
        return tabMode;
    }

    public void setTabMode(boolean tabMode) {
        this.tabMode = tabMode;
    }

    public JTabbedPane getTabbed() {
        return tabbed;
    }

    public void setTabbed(JTabbedPane tabbed) {
        this.tabbed = tabbed;
    }

    public boolean isTabbeded() {
        return isTabbeded;
    }

    public void setTabbeded(boolean isTabbeded) {
        this.isTabbeded = isTabbeded;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
        dragArea.setText(name);
    }

    public String getNameTab() {
        return nameTab;
    }

    public void setNameTab(String nameTab) {
        this.nameTab = nameTab;
        dragTabArea.setText(nameTab);
    }

    public void setNames(String st, String s){
        setName(st);
        setNameTab(s);
    }

    public void setHidden(boolean b) {
        hidden = b;
    }

    public boolean isHidden() {
        return hidden;
    }
}
