// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui.menu;

import com.bbn.openmap.Environment;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.MapBean;
import com.bbn.openmap.image.AbstractImageFormatter;
import com.bbn.openmap.gui.menu.MapHandlerMenuItem;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JMenu;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import java.io.FileOutputStream;
import java.io.IOException;

import fr.unicaen.openmap.gui.ThemaMapFileChooserManager;
import fr.unicaen.openmap.util.*;

/**
 * SaveAsMenu based on openmap one in order to use ThemaMapFileChooserManager
 */
public class SaveAsImageMenuItem extends MapHandlerMenuItem implements ActionListener {

	AbstractImageFormatter formatter = null;

	public AbstractImageFormatter getFormatter() {
		return formatter;
	}

	public void setFormatter(AbstractImageFormatter formatter) {
		this.formatter = formatter;
	}

    private static String title = Environment.getI18n().get(SaveAsMenu.class,"SaveMapAs","Save map as");

    /**
     * @param display
     *            A String that will be displayed when this menuitem is shown in GUI
     * @param in_formatter
     *            A formatter that knows how to generate an image from MapBean.
     */
    public SaveAsImageMenuItem(String display, AbstractImageFormatter in_formatter) {
	super(display);
	formatter = in_formatter;
        addActionListener(this);
    }
    
    public void actionPerformed(ActionEvent ae) {
	Debug.message("saveimage", "SaveAsImageMenuItem: actionPerformed");

	if (mapHandler == null) {
	    Debug.output("SaveAsImageMenuItem: mapHandler = null, returning");
	    return;
	}

	MapBean mb = (MapBean) mapHandler.get("com.bbn.openmap.MapBean");

	if (mb != null) {
            Debug.message("savetext", "MapBean found, creating image " + formatter.getFormatLabel().toLowerCase());
	    String fileName = ThemaMapFileChooserManager.
		imageSaveDialog(title, mb.getWidth(), mb.getHeight(), formatter.getFormatLabel().toLowerCase());
	    if (fileName == null || fileName.length() == 0) {
		System.err.println("**Error SaveAsImageMenuItem: no filename given");
		return;
	    }                
	    int imageHeight = ThemaMapFileChooserManager.imgSaveFileChooser.getImageHeight();
	    int imageWidth = ThemaMapFileChooserManager.imgSaveFileChooser.getImageWidth();

	    saveAs(fileName, imageWidth, imageHeight, mb);

	    if (Debug.debugging("saveimage")) {
		com.bbn.openmap.proj.Projection proj = mb.getProjection();
		Debug.output("Created image at " + fileName + " where projection covers " + proj.getUpperLeft() + " to " + proj.getLowerRight());
	    }

        }
        return;
    }

    public void saveAs(String fileName, int w, int h, MapBean mb) {
        try {
	    byte[] imageBytes = getFormatter().getImageFromMapBean(mb, w, h);
	    FileOutputStream binFile = new FileOutputStream(fileName);
	    binFile.write(imageBytes);
	    binFile.close(); 
        } catch (IOException e) {	
	    Debug.error("SaveAsImageMenuItem: " + e);
        }

    }
}

