// **********************************************************************
// 
// <copyright>
// 
//  BBN Technologies
//  10 Moulton Street
//  Cambridge, MA 02138
//  (617) 873-8000
// 
//  Copyright (C) BBNT Solutions LLC. All rights reserved.
// 
// </copyright>
// **********************************************************************
// 
// $Source: /cvs/distapps/openmap/src/openmap/com/bbn/openmap/gui/menu/SaveAsImageMenuItem.java,v $
// $RCSfile: SaveAsImageMenuItem.java,v $
// $Revision: 1.4.2.1 $
// $Date: 2004/10/14 18:26:56 $
// $Author: dietrick $
// 
// **********************************************************************
//
// The only change from the original openmap SaveAsImageMenuItem is 
// for passing layers to svg renderer instead of MapBean. 
//
package fr.unicaen.openmap.gui.menu;

import java.awt.event.*;
import java.io.*;

import com.bbn.openmap.*;
import com.bbn.openmap.gui.menu.*;
import com.bbn.openmap.util.Debug;

import fr.unicaen.openmap.gui.ThemaMapFileChooserManager;
import fr.unicaen.openmap.util.*;

/**
 * A MenuItem that is capable of looking at MapBean and saving it as
 * a Text (svg by example).
 */
public class SaveAsTextMenuItem extends MapHandlerMenuItem implements
        ActionListener {

    AbstractTextFormatter formatter = null;
    private static String title = Environment.getI18n().get(SaveAsMenu.class,"SaveMapAs","Save map as");

    /**
     * @param display A String that will be displayed when this
     *        menuitem is shown in GUI
     * @param in_formatter A formatter that knows how to generate an
     *        image from MapBean.
     */
    public SaveAsTextMenuItem(String display,
            AbstractTextFormatter in_formatter) {
        super(display);
        formatter = in_formatter;
        addActionListener(this);
    }

    public void actionPerformed(ActionEvent ae) {
        Debug.message("savetext", "SaveAsTextMenuItem: actionPerformed");

        if (mapHandler == null) {
            Debug.output("SaveAsTextMenuItem: mapHandler = null, returning");
            return;
        }

        MapBean mb = (MapBean) mapHandler.get("com.bbn.openmap.MapBean");
        LayerHandler lh = (LayerHandler) mapHandler.get("com.bbn.openmap.LayerHandler");
        Layer[] layers = lh.getMapLayers();

        if (mb != null) {
            Debug.message("savetext", "MapBean found, creating image " + formatter.getFormatLabel().toLowerCase());
	    String fileName = ThemaMapFileChooserManager.
		imageTextSaveDialog(title, mb.getWidth(), mb.getHeight(), formatter.getFormatLabel().toLowerCase());
	    if (fileName == null || fileName.length() == 0) {
		System.err.println("**Error SaveAsTextMenuItem: no filename given");
		return;
	    }                
	    int imageHeight = ThemaMapFileChooserManager.imgTextSaveFileChooser.getImageHeight();
	    int imageWidth = ThemaMapFileChooserManager.imgTextSaveFileChooser.getImageWidth();

	    saveAs(fileName, layers, imageWidth, imageHeight, mb);

	    if (Debug.debugging("savetext")) {
		com.bbn.openmap.proj.Projection proj = mb.getProjection();
		Debug.output("Created image at " + fileName + " where projection covers " + proj.getUpperLeft() + " to " + proj.getLowerRight());
	    }

        }
        return;
    }

    public void saveAs(String fileName, Layer[] layers, int w, int h, MapBean mb) {
        try {	    
            byte[] imageBytes = formatter.getTextFromLayers(layers,
                    w,
                    h,
                    mb.getBackground());
            FileOutputStream binFile = new FileOutputStream(fileName);
            binFile.write(imageBytes);
            binFile.close();
        } catch (IOException e) {
        }

    }
}
