// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui.menu;

import com.bbn.openmap.MapBean;
import com.bbn.openmap.PropertyHandler;
import com.bbn.openmap.util.PropUtils;
import com.bbn.openmap.gui.menu.MapHandlerMenuItem;
import java.awt.event.*;
import java.io.*;
import java.util.Iterator;
import java.util.Properties;

import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.Environment;

import fr.unicaen.openmap.gui.ThemaMapFileChooserManager;
import fr.unicaen.openmap.util.UnicodePrintStream;


/**
 * A Menu item that generates a Properties object when it is clicked
 * upon. It needs BeanContext object. It iterates through objects in
 * BeanContext and looks for objects that implement PropertyConsumer
 * interface and generates Properties object from them.Wwhen it has
 * found all the Properties, it writes them to a file by providing the
 * user with a file dialog box.
 */
public class SavePropertiesMenuItem extends MapHandlerMenuItem implements
        ActionListener {

    private String titleString;
    private String saveProperties = "Save";

    public SavePropertiesMenuItem(String initString) {
        super(initString);
        titleString = initString;
        addActionListener(this);
    }

    public void actionPerformed(ActionEvent ae) {
        //Collect properties
        if (mapHandler == null) {
            if (Debug.debugging("menu")) {
                Debug.error("SavePropertiesMenuItem: no Map Handler to use");
            }
            return;
        }

        String saveString = ThemaMapFileChooserManager.propertiesSaveDialog(titleString);
        if (saveString==null ||saveString.length()==0) return;
        File file = new File(saveString);
        FileOutputStream fos;
        PrintStream ps = null;
        try {
            fos = new FileOutputStream(file);
        }
        catch (IOException fnfe) {
            Debug.error(fnfe.getMessage());
            return;
        }

        ps = new UnicodePrintStream(fos);

        Properties createdProperties = PropertyHandler.createOpenMapProperties(mapHandler, ps);
	ps.println("####");
	{ // patch Projection save
	    // warning...hackish...
	    
	    Iterator it = mapHandler.iterator();
	    Object o = null;
	    while (it.hasNext()) {
		o = it.next();
		if (o instanceof MapBean) 
		    break;
	    }
	    if (o != null) {
		com.bbn.openmap.proj.Projection proj = ((MapBean)o).getProjection();
		ps.println(Environment.Projection + "=" + proj.getClass().getName());
	    }
	}
	// save Debug and Title
	ps.println(Environment.DebugList + "=" + Environment.get(Environment.DebugList));
	ps.println(Environment.Title + "=" + Environment.get(Environment.Title));
    
	{ // patch: allways turn atlas mode off
	    // look for fr.unicaen.openmap.gui.ThemaMapPanelsManager component
	    for(String component : 
		    PropUtils.stringArrayFromProperties(createdProperties, PropertyHandler.componentProperty, " \t"))
		if ("fr.unicaen.openmap.gui.ThemaMapPanelsManager".equals(createdProperties.get(component+".class")))
		    ps.println("###\n"+ component+"."+fr.unicaen.openmap.gui.ThemaMapPanelsManager.atlasPropertyString+"=false\n");
	}
        ps.close();
    }
}


