// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.jts;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

import fr.unicaen.openmap.dataStorage.DataObject;
import fr.unicaen.openmap.dataStorage.OMDataLocation;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.util.Debug;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.operation.polygonize.Polygonizer;

/** 
 * Bridge to Java Topology Suite algorithms
 */
public class JTS {

    public static ArrayList<DataObject> polygonize(List<DataObject> dataObjects, String lineColumnName, String lineDataColumnName, String isoPolyColumnName, String isoPolyDataColumnName) {
 	// DataObject -> OMLine -> coords -> GeometryFactory -> add to Polygonizer
 	Polygonizer polygonizer = new Polygonizer();
 	for (DataObject data : dataObjects) {
 	    if (data == null) 
 		continue;
 	    OMDataLocation oml = data.getOMDataLocation(lineColumnName);
 	    if (oml == null)
 		continue;
	    Geometry geom = JTSFactory.OMGraphicToJTSGeometry(oml.getOMGraphic());
	    geom.setUserData(data.getData(lineDataColumnName));
	    if (Debug.debugging("JTS")) {
		System.out.print(geom.getUserData() + " \t");
		for (Coordinate coord : geom.getCoordinates())
		    System.out.print("("+coord.x+", "+coord.y+") ");
		System.out.println();
	    }
	    polygonizer.add(geom);
 	}
 
	ArrayList<DataObject> ret = new ArrayList<DataObject>();
 	for (Iterator i = polygonizer.getPolygons().iterator(); i.hasNext(); ) {
	    Geometry geom = (Geometry) i.next();	  
	    if (Debug.debugging("JTS")) {
		System.out.print(geom.getUserData() + "D \t");
		for (Coordinate coord : geom.getCoordinates())
		    System.out.print("("+coord.x+", "+coord.y+") ");
		System.out.println();
	    }
 	    OMDataLocation omdl = JTSFactory.JTSGeometryToOMDataLocation(geom);
	    DataObject data = new DataObject();
	    data.addOMDataLocation(isoPolyColumnName, omdl);
	    Object userData = geom.getUserData();
	    if (userData != null)
		data.addData(isoPolyDataColumnName, userData);
	    ret.add(data);
 	}
 	return ret;
     }
     
    public static double area(OMGraphic omg) {
	return JTSFactory.OMGraphicToJTSGeometry(omg).getArea();
    }

    public static OMGraphic buffer(OMGraphic omg, double d) {
	Geometry geom = JTSFactory.OMGraphicToJTSGeometry(omg);
	//System.out.println("*** Buffer distance "+d);
	return JTSFactory.JTSGeometryToOMGraphic(com.vividsolutions.jts.operation.buffer.BufferOp.bufferOp(geom, d));
    }
}
