// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
// **********************************************************************
//
// <copyright>
//
// BBN Technologies, a Verizon Company
// 10 Moulton Street
// Cambridge, MA 02138
// (617) 873-8000
//
// Copyright (C) BBNT Solutions LLC. All rights reserved.
//
// </copyright>
// **********************************************************************
//
// $Source: /cvs/distapps/openmap/src/openmap/com/bbn/openmap/layer/GraticuleLayer.java,v $
// $RCSfile: GraticuleLayer.java,v $
// $Revision: 1.17 $
// $Date: 2009/02/25 22:34:04 $
// $Author: dietrick $
//
// **********************************************************************

// Modified 28 September 2002 by David N. Allsopp to allow font size
// to be changed. See sections commented with 'DNA'.

// Rewrited by Jacques.Madelaine@unicaen.fr september 2011 

package fr.unicaen.openmap.layer;

import com.bbn.openmap.MapBean;
import com.bbn.openmap.MoreMath;
import com.bbn.openmap.event.ProjectionEvent;
import com.bbn.openmap.omGraphics.OMColorChooser;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.omGraphics.OMText;
import com.bbn.openmap.proj.Cylindrical;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.proj.coords.LatLonPoint;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.gui.ThemaMapColorChooser;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMLabel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableGraphicList;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Offseter;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;
import java.beans.beancontext.BeanContext;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 *
 * @author jchauvea
 */
public class GraticuleLayer extends com.bbn.openmap.layer.OMGraphicHandlerLayer
    implements ActionListener {

    public final static String equatorCmd = "equatorCmd";
    public final static String graticuleCmd = "graticuleCmd";
    
    private float latStep = 1; // number of degrees between 2 parallels
    private float lonStep = 1; // number of degrees between 2 meridians
    private int intervalLevel2Lat = 0; // number of parralels separating level 2 lines
    private int intervalLevel3Lat = 0; // number of parralels separating level 3 lines
    private int intervalLevel2Lon = 5; // number of meridians separating level 2 lines
    private int intervalLevel3Lon = 10; // number of meridians separating level 3 lines

    protected Font font = null;
    protected int fontSize = 10;

    protected boolean defaultShowRuler3 = false;
    protected boolean showRuler3 = defaultShowRuler3;
    protected boolean defaultShowRuler2 = false;
    protected boolean showRuler2 = defaultShowRuler2;
    protected boolean defaultShowRuler1 = true;
    protected boolean showRuler1 = defaultShowRuler1;

    protected int latLabelXoffset = 2;
    protected int latLabelYoffset = -2;
    protected int lonLabelXoffset = 2;
    protected int lonLabelYoffset = -5;


    // property text values
    public static final String LatStepProperty = "latStep";
    public static final String LonStepProperty = "lonStep";
    public static final String IntervalLevel2LatProperty = "intervalLevel2Lat";
    public static final String IntervalLevel3LatProperty = "intervalLevel3Lat";
    public static final String IntervalLevel2LonProperty = "intervalLevel2Lon";
    public static final String IntervalLevel3LonProperty = "intervalLevel3Lon";
    public static final String LatLabelXoffsetProperty = "latLabelXoffset" ;
    public static final String LatLabelYoffsetProperty = "latLabelYoffset";
    public static final String LonLabelXoffsetProperty = "lonLabelXoffset";
    public static final String LonLabelYoffsetProperty =  "lonLabelYoffset";
    public static final String ThresholdProperty = "threshold";
    public static final String FontSizeProperty = "fontSize"; 
    public static final String ShowRuler3Property = "showRuler3";
    public static final String ShowRuler2Property = "showRuler2";
    public static final String ShowRuler1Property = "showRuler1";

    // the symbol of Colors is used as the property string
    enum Colors {
	level1Color("b2ffffff"),
	    equatorColor("c1d68171"),
	    level2Color("94787878"),
	    dateLineColor("ff000099"),
	    level3Color("97323232"),
	    textColor("999999"),
	    //	    specialLineColor, // futur use for tropiques line ans polar circles
	    ;
	final String defaultValue;
	private Color color;
	Colors(String defaultValue) {
	    this.defaultValue = defaultValue;
	}
	public Color getColor() { return color; }
	public void setColor(Color color) { this.color = color; }
    }

    void setShowRuler1(boolean s) { showRuler1 = s; }
    void setShowRuler2(boolean s) { showRuler2 = s; }
    void setShowRuler3(boolean s) { showRuler3 = s; }
    
    void setFontSize(int s) { 
	fontSize = s; 
        font = new Font("Helvetica", java.awt.Font.PLAIN, fontSize);
    }

    /**
     * Flag for lineType - true is LINETYPE_STRAIGHT, false is
     * LINETYPE_GREATCIRCLE or LINETYPE_RHUMB
     */
    protected boolean boxy;

    /**
     * Construct the GraticuleLayer.
     */
    public GraticuleLayer() {
        // precalculate for boxy
        boxy = true;
        setName("Graticule");
    }

    /**
     * The properties and prefix are managed and decoded here, for the standard
     * uses of the GraticuleLayer.
     * 
     * @param prefix string prefix used in the properties file for this layer.
     * @param properties the properties set in the properties file.
     */
    public void setProperties(String prefix, java.util.Properties properties) {
        super.setProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
	latStep = PropUtils.floatFromProperties(properties, prefix +
						LatStepProperty,latStep);
	lonStep = PropUtils.floatFromProperties(properties, prefix +
						LonStepProperty,lonStep);
	intervalLevel2Lat = PropUtils.intFromProperties(properties, prefix +
							 IntervalLevel2LatProperty, intervalLevel2Lat);
	intervalLevel3Lat = PropUtils.intFromProperties(properties, prefix +
							 IntervalLevel3LatProperty, intervalLevel3Lat);
	intervalLevel2Lon = PropUtils.intFromProperties(properties, prefix +
							 IntervalLevel2LonProperty, intervalLevel2Lon);
	intervalLevel3Lon = PropUtils.intFromProperties(properties, prefix +
							 IntervalLevel3LonProperty, intervalLevel3Lon);
	latLabelXoffset = PropUtils.intFromProperties(properties, prefix +
						       LatLabelXoffsetProperty, latLabelXoffset);
	latLabelYoffset = PropUtils.intFromProperties(properties, prefix +
						       LatLabelYoffsetProperty, latLabelYoffset);
	lonLabelXoffset = PropUtils.intFromProperties(properties, prefix +
						       LonLabelXoffsetProperty, lonLabelXoffset);
	lonLabelYoffset = PropUtils.intFromProperties(properties, prefix +
						       LonLabelYoffsetProperty, lonLabelYoffset);

	for(Colors c : Colors.values())
	    c.setColor(PropUtils.parseColorFromProperties(properties, prefix +
							  c.toString(), c.defaultValue, true));

	setFontSize(PropUtils.intFromProperties(properties, prefix
						+ FontSizeProperty, fontSize));

        showRuler1 = PropUtils.booleanFromProperties(properties, prefix
						     + ShowRuler1Property, defaultShowRuler1);
	showRuler2 = PropUtils.booleanFromProperties(properties, prefix
						     + ShowRuler2Property, defaultShowRuler2);
        showRuler3 = PropUtils.booleanFromProperties(properties, prefix
						     + ShowRuler3Property, defaultShowRuler3);
    }

    /**
     * The properties and prefix are managed and decoded here, for the standard
     * uses of the GraticuleLayer.
     * 
     * @param properties the properties set in the properties file.
     */
    public Properties getProperties(Properties properties) {
        properties = super.getProperties(properties);

        String prefix = PropUtils.getScopedPropertyPrefix(this);

	properties.put(prefix + LatStepProperty, Float.toString(latStep));
	properties.put(prefix + LonStepProperty, Float.toString(lonStep));
	properties.put(prefix + IntervalLevel2LatProperty, Integer.toString( intervalLevel2Lat));
	properties.put(prefix + IntervalLevel3LatProperty, Integer.toString( intervalLevel3Lat));
	properties.put(prefix + IntervalLevel2LonProperty, Integer.toString( intervalLevel2Lon));
	properties.put(prefix + IntervalLevel3LonProperty, Integer.toString( intervalLevel3Lon));
	properties.put(prefix + LatLabelXoffsetProperty, Integer.toString( latLabelXoffset));
	properties.put(prefix + LatLabelYoffsetProperty, Integer.toString( latLabelYoffset));
	properties.put(prefix + LonLabelXoffsetProperty, Integer.toString( lonLabelXoffset));
	properties.put(prefix + LonLabelYoffsetProperty, Integer.toString( lonLabelYoffset));

	for (Colors c : Colors.values())
	    properties.put(prefix + c.toString(), c.getColor() == null 
			   ? c.defaultValue : Integer.toHexString(c.getColor().getRGB()));

        properties.put(prefix + FontSizeProperty, Integer.toString(fontSize)); 
	//font ??

        properties.put(prefix + ShowRuler1Property, Boolean.toString(showRuler1)); 
        properties.put(prefix + ShowRuler2Property, Boolean.toString(showRuler2)); 
        properties.put(prefix + ShowRuler3Property, Boolean.toString(showRuler3)); 

        return properties;
    }
    public Properties getPropertyInfo(Properties properties) {
        properties = super.getPropertyInfo(properties);
	properties.put(initPropertiesProperty, "");
        return properties;
    }

    public Component getGUI() {
        JPanel gui = new JPanel(new BorderLayout());
        Box boxConf = Box.createHorizontalBox();
        ThematicConfPanel tcpLatLine = ThematicConfPanel.createVerticalThematicConfPanel("Latitude Lines preferences");
        ThematicConfPanel tcpLonLine = ThematicConfPanel.createVerticalThematicConfPanel("Longitude Lines preferences");
	ThematicConfPanel tcpColors = ThematicConfPanel.createVerticalThematicConfPanel("Colors preferences");
	ThematicConfPanel tcpLabel = ThematicConfPanel.createVerticalThematicConfPanel("Labels preferences");
	
        JPanel spinnersLinePanel;
        JPanel wrap;
        Box boxLine;

	// lat spinners
        JSpinner latSpinner = new JSpinner(new SpinnerNumberModel(latStep, 0f, 90f, .1f));
        latSpinner.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent ce) {
		    JSpinner sp = (JSpinner) ce.getSource();
		    latStep = Float.parseFloat(sp.getValue().toString());
		    refreshGraphics();
		}
	    });
        JSpinner intervalLevel2LatSpinner = new JSpinner(new SpinnerNumberModel(intervalLevel2Lat, 0, 100, 1));
        intervalLevel2LatSpinner.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent ce) {
		    JSpinner sp = (JSpinner) ce.getSource();
		    intervalLevel2Lat = Integer.parseInt(sp.getValue().toString());
		    refreshGraphics();
		}
	    });
        JSpinner intervalLevel3LatSpinner = new JSpinner(new SpinnerNumberModel(intervalLevel3Lat, 0, 100, 1));
        intervalLevel3LatSpinner.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent ce) {
		    JSpinner sp = (JSpinner) ce.getSource();
		    intervalLevel3Lat = Integer.parseInt(sp.getValue().toString());
		    refreshGraphics();
		}
	    });
	
	Box spinnersLineBox = Box.createVerticalBox();

	wrap= new JPanel();
        wrap.add(new JLabel("Level 1 latitude step"));
        wrap.add(latSpinner);
	wrap.add(new JLabel(" degrees"));
        spinnersLineBox.add(wrap);

	wrap= new JPanel();
        wrap.add(new JLabel("Level 2 lines every "));
        wrap.add(intervalLevel2LatSpinner);
        wrap.add(new JLabel("lines"));
        spinnersLineBox.add(wrap);
	
	wrap= new JPanel();
        wrap.add(new JLabel("Level 3 lines every "));
        wrap.add(intervalLevel3LatSpinner);
        wrap.add(new JLabel("lines"));
        spinnersLineBox.add(wrap);

        tcpLatLine.setContent(spinnersLineBox);
        boxConf.add(tcpLatLine);

        //lon spinners
        JSpinner lonSpinner = new JSpinner(new SpinnerNumberModel(lonStep, 0f, 180f, .1f));
        lonSpinner.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent ce) {
		    JSpinner sp = (JSpinner) ce.getSource();
		    lonStep = Float.parseFloat(sp.getValue().toString());
		    refreshGraphics();
		}
	    });
        JSpinner intervalLevel2LonSpinner = new JSpinner(new SpinnerNumberModel(intervalLevel2Lon, 0, 100, 1));
        intervalLevel2LonSpinner.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent ce) {
		    JSpinner sp = (JSpinner) ce.getSource();
		    intervalLevel2Lon = Integer.parseInt(sp.getValue().toString());
		    refreshGraphics();
		}
	    });
        JSpinner intervalLevel3LonSpinner = new JSpinner(new SpinnerNumberModel(intervalLevel3Lon, 0, 100, 1));
        intervalLevel3LonSpinner.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent ce) {
		    JSpinner sp = (JSpinner) ce.getSource();
		    intervalLevel3Lon = Integer.parseInt(sp.getValue().toString());
		    refreshGraphics();
		}
	    });

	
	spinnersLineBox = Box.createVerticalBox();

	wrap= new JPanel();
        wrap.add(new JLabel("Level 1 longitude step"));
        wrap.add(lonSpinner);
	wrap.add(new JLabel(" degrees"));
        spinnersLineBox.add(wrap);

	wrap= new JPanel();
        wrap.add(new JLabel("Level 2 lines every "));
        wrap.add(intervalLevel2LonSpinner);
        wrap.add(new JLabel("lines"));
        spinnersLineBox.add(wrap);
	
	wrap= new JPanel();
        wrap.add(new JLabel("Level 3 lines every "));
        wrap.add(intervalLevel3LonSpinner);
        wrap.add(new JLabel("lines"));
        spinnersLineBox.add(wrap);

        tcpLonLine.setContent(spinnersLineBox);
        boxConf.add(tcpLonLine);

        //color buttons
        JPanel boxColorsButtons = new JPanel();
	boxColorsButtons.setLayout(new GridLayout(3,4));
	for(Colors c : Colors.values()) {
	    JPanel p = new JPanel();
	    p.add(new JLabel(c.toString()));
	    boxColorsButtons.add(p);
	    boxColorsButtons.add(new PropertyColorButton(c));
	}

        tcpColors.setContent(boxColorsButtons);
	tcpColors.collapse();
        boxConf.add(tcpColors);

        //labels
        Box boxLabel = Box.createVerticalBox();
	
	final JCheckBox cbShowRuler1 = new JCheckBox("1");
	cbShowRuler1.setSelected(showRuler1);
	final JCheckBox cbShowRuler2 = new JCheckBox("2");
	cbShowRuler2.setSelected(showRuler2);
	final JCheckBox cbShowRuler3 = new JCheckBox("3");
	cbShowRuler3.setSelected(showRuler3);
	ActionListener cbActionListener = new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    setShowRuler1(cbShowRuler1.isSelected());
		    setShowRuler2(cbShowRuler2.isSelected());
		    setShowRuler3(cbShowRuler3.isSelected());
		    refreshGraphics();
		}
	    };
	cbShowRuler1.addActionListener(cbActionListener);
	cbShowRuler2.addActionListener(cbActionListener);
	cbShowRuler3.addActionListener(cbActionListener);
        JSpinner fontSizeSpinner = new JSpinner(new SpinnerNumberModel(fontSize, 0, 40, 1));
        fontSizeSpinner.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent ce) {
		    JSpinner sp = (JSpinner) ce.getSource();
		    setFontSize(Integer.parseInt(sp.getValue().toString()));
		    refreshGraphics();
		}
	    });
	
        JSpinner latLabelXoffsetSpinner = new JSpinner(new SpinnerNumberModel(latLabelXoffset, -99, 99, 1));
        latLabelXoffsetSpinner.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent ce) {
		    JSpinner sp = (JSpinner) ce.getSource();
		    latLabelXoffset = Integer.parseInt(sp.getValue().toString());
		    refreshGraphics();
		}
	    });

        JSpinner latLabelYoffsetSpinner = new JSpinner(new SpinnerNumberModel(latLabelYoffset, -99, 99, 1));
        latLabelYoffsetSpinner.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent ce) {
		    JSpinner sp = (JSpinner) ce.getSource();
		    latLabelYoffset = Integer.parseInt(sp.getValue().toString());
		    refreshGraphics();
		}
	    });

        JSpinner lonLabelXoffsetSpinner = new JSpinner(new SpinnerNumberModel(lonLabelXoffset, -99, 99, 1));
        lonLabelXoffsetSpinner.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent ce) {
		    JSpinner sp = (JSpinner) ce.getSource();
		    lonLabelXoffset = Integer.parseInt(sp.getValue().toString());
		    refreshGraphics();
		}
	    });

        JSpinner lonLabelYoffsetSpinner = new JSpinner(new SpinnerNumberModel(lonLabelYoffset, -99, 99, 1));
        lonLabelYoffsetSpinner.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent ce) {
		    JSpinner sp = (JSpinner) ce.getSource();
		    lonLabelYoffset = Integer.parseInt(sp.getValue().toString());
		    refreshGraphics();
		}
	    });
	
	wrap = new JPanel();
	wrap.add(new JLabel("Label lines level"));
	wrap.add(cbShowRuler1);
	wrap.add(cbShowRuler2);
	wrap.add(cbShowRuler3);
	boxLabel.add(wrap);

	wrap = new JPanel();
	wrap.add(new JLabel("Lat label offset"));
	wrap.add(new JLabel("x"));
        wrap.add(latLabelXoffsetSpinner);
	wrap.add(new JLabel(" y "));
        wrap.add(latLabelYoffsetSpinner);
	boxLabel.add(wrap);

	wrap = new JPanel();
	wrap.add(new JLabel("Lon label offset"));
	wrap.add(new JLabel("x"));
        wrap.add(lonLabelXoffsetSpinner);
	wrap.add(new JLabel(" y "));
        wrap.add(lonLabelYoffsetSpinner);
	boxLabel.add(wrap);

	wrap = new JPanel();
	wrap.add(new JLabel("Font Size"));
	wrap.add(fontSizeSpinner);
	boxLabel.add(wrap);

        tcpLabel.setContent(boxLabel);
        boxConf.add(tcpLabel);

        gui.add(boxConf, BorderLayout.WEST);
        return gui;
    }

    /**
     * Implementing the ProjectionPainter interface.
     */
    public synchronized void renderDataForProjection(Projection proj,
                                                     java.awt.Graphics g) {
        if (proj == null) {
            Debug.error("GraticuleLayer.renderDataForProjection: null projection!");
            return;
        } else if (!proj.equals(getProjection())) {
            setProjection(proj.makeClone());
            // Figure out which line type to use
	    boxy = proj instanceof Cylindrical;

            setList(constructGraticuleLines());
        }
        paint(g);
    }

    /**
     * Invoked when the projection has changed or this Layer has been added to
     * the MapBean.
     * <p>
     * Perform some extra checks to see if reprojection of the graphics is
     * really necessary.
     * 
     * @param e ProjectionEvent
     * 
     */
    public void projectionChanged(ProjectionEvent e) {

        // extract the projection and check to see if it's really different.
        // if it isn't then we don't need to do all the work again, just repaint.
        Projection proj = setProjection(e);
        if (proj == null) {
            repaint();
            return;
        }

        // Figure out which line type to use
	boxy = proj instanceof Cylindrical;
       
        setList(null);
        doPrepare();
    }

    /**
     * Creates the OMGraphic list with graticule lines.
     */
    public synchronized OMGraphicList prepare() {
        return constructGraticuleLines();
    }

    protected OMGraphicList constructGraticuleLines() {

        OMGraphicList newgraphics = new OMGraphicList();
        // Lets figure out which lines should be painted...
        Projection projection = getProjection();
        if (projection == null)
            return newgraphics;
        

	Point2D ul = projection.getUpperLeft();
	Point2D lr = projection.getLowerRight();

	float left = (float) ul.getX();
	float right = (float) lr.getX();
	float up = (float) ul.getY();
	float down = (float) lr.getY();

	if (up > 80.0f)
	    up = 80.0f;
	if (down > 80.0f)
	    down = 80f; // unlikely
	if (up < -80.0f)
	    up = -80.0f; // unlikely
	if (down < -80)
	    down = -80.0f;

	// Find out whether we need to do one or two queries,
	// depending on if we're straddling the dateline.
	if (left > right || Math.abs(left - right) < 1) {
	    newgraphics.add(constructGraticuleLines(up, down, left, 180f));
	    newgraphics.add(constructGraticuleLines(up, down, -180f, right));
	} else {
	    newgraphics = constructGraticuleLines(up, down, left, right);
	}

        if (Debug.debugging("graticule")) {
            Debug.output("GraticuleLayer.constructGraticuleLines(): " + "constructed " + newgraphics.size() + " graticule lines");
        }

        return newgraphics;
    }

    private OMPoly constructParrallelLine(float left, float right, float lat, Color color) {

	// first pass to count the number of points we need
	int nbPoints = 1; // 1 for left 

	if (left < -90) {
	    if (right > -90) {
		nbPoints++; // one for -90f
	    }
	    if (right > 0) {
		nbPoints++; // one for 0f
	    }
	    if (right > 90) {
		nbPoints++; // one for 90f
	    }
	} else if (left < 0) {
	    if (right > 0) {
		nbPoints++; // one for 0f
	    }
	    if (right > 90) {
		nbPoints++; // one for 90f
	    }
	} else if (left < 90) {
	    if (right > 90) {
		nbPoints++; // one for 90f
	    }
	}  
	nbPoints++; // one for right

	// second pass, the real job
	double[] llp = new double[nbPoints*2];
	int nbp = 0; // index in llp
	llp[nbp++] = lat;
	llp[nbp++] = left;
	if (left < -90) {
	    if (right > -90) {
		llp[nbp++] = lat;
		llp[nbp++] = -90f;
	    }
	    if (right > 0) {
		llp[nbp++] = lat;
		llp[nbp++] = 0f;
	    }
	    if (right > 90) {
		llp[nbp++] = lat;
		llp[nbp++] = 90f;
	    }
	} else if (left < 0) {
	    if (right > 0){
		llp[nbp++] = lat;
		llp[nbp++] = 0f;
	    }
	    if (right > 90) {
		llp[nbp++] = lat;
		llp[nbp++] = 90f;
	    }
	} else if (left < 90) {
	    if (right > 90) {
		llp[nbp++] = lat;
		llp[nbp++] = 90f;
	    }
	}   
	llp[nbp++] = lat;
	llp[nbp++] = right;

	OMPoly line = new OMPoly(llp, OMGraphic.DECIMAL_DEGREES, boxy ? OMGraphic.LINETYPE_STRAIGHT
					: OMGraphic.LINETYPE_RHUMB);	
	line.setLinePaint(color);	
	return line;
    }

    private Point point = new Point(); // used to place labels

    protected OMPoly constructMeridianLine(float up, float down, float lon, Color color) {
	double llp[];
	if (up < 0 && down > 0) {
	    llp = new double[6];
	    llp[2] = 0f;
	    llp[3] = lon;
	    llp[4] = down;
	    llp[5] = lon;
	} else {
	    llp = new double[4];
	    llp[2] = down;
	    llp[3] = lon;
	}
	llp[0] = up;
	llp[1] = lon;
	
	OMPoly line = new OMPoly(llp, OMGraphic.DECIMAL_DEGREES, boxy ? OMGraphic.LINETYPE_STRAIGHT
				 : OMGraphic.LINETYPE_GREATCIRCLE);
	line.setLinePaint(color);
	return line;
    }


    protected OMText constructLabel(float lat, float lon, float t, int xoffset, int yoffset, boolean meridian, float stepSize, int justify) {  
	String text = Math.abs(t%1) <= 0.1 ? Integer.toString((int)t) : String.format("%.1f", t);
	Projection projection = getProjection();
	LatLonPoint llpoint;
	if (boxy) {
	    projection.forward(lat, lon, point);
	    if (meridian) 
		point.y = projection.getHeight();
	    else
		point.x = 0;
	    llpoint = projection.inverse(point.x,
					 point.y,
					 new LatLonPoint.Double());
	} else {
	    llpoint = new LatLonPoint.Double(lat, lon);
	    if (stepSize < 0.1)
		stepSize = 0.1f;
	    while (projection.forward(llpoint).getY() > projection.getHeight()) {
		llpoint.setLatitude(llpoint.getY() + stepSize);
	    }
	}
	
	OMText omText = new OMText(llpoint.getY(), llpoint.getX(),
				   xoffset, yoffset, text, font, justify);
	omText.setLinePaint(Colors.textColor.getColor());
	return omText;
    }

    /** 
     * @param up is always &lt;= 80
     * @param down is always &gt;= -80
     * @param left and right have values between -180 and 180
     * Thus lines never cross the dateline
     */
    protected OMGraphicList constructGraticuleLines(float up, float down, float left, float right) {
        OMGraphicList lines = new OMGraphicList();

        Projection projection = getProjection();

        // generate parallels of latitude by creating series of polylines

	// the equator
	if (up > 0 && 0 > down) {
	    lines.add(constructParrallelLine(left, right, 0, Colors.equatorColor.getColor()));
	    if (showRuler1 || showRuler2 || showRuler3)
		lines.add(constructLabel(0, left, 0f, latLabelXoffset, latLabelYoffset, false, latStep, OMText.JUSTIFY_LEFT));	
	}

	// to do: add tropiques lines and polar circles 

	// parralels for the north hemisphere
	int i = 1;
	for (float lat = latStep; lat < up; lat += latStep, i++) {
	    Color color = Colors.level1Color.getColor();
	    boolean showRuler = showRuler1;
	    if (intervalLevel2Lat > 0 && i % intervalLevel2Lat == 0) {
		color = Colors.level2Color.getColor();
		showRuler = showRuler || showRuler2;
	    }
	    if (intervalLevel3Lat > 0 && i % intervalLevel3Lat == 0) {
		color = Colors.level3Color.getColor();
		showRuler = showRuler || showRuler3;
	    }	    
	    lines.add(constructParrallelLine(left, right, lat, color));
	    if (showRuler)
		lines.add(constructLabel(lat, left, lat, latLabelXoffset, latLabelYoffset, false, latStep, OMText.JUSTIFY_LEFT));
	}
	// parralels for the south hemisphere
	i = 1;
	for (float lat = -latStep; lat > down; lat -= latStep, i++) {
	    Color color = Colors.level1Color.getColor();
	    boolean showRuler = showRuler1;
	    if (intervalLevel2Lat > 0 && i % intervalLevel2Lat == 0) {
		color = Colors.level2Color.getColor();
		showRuler = showRuler || showRuler2;
	    }
	    if (intervalLevel3Lat > 0 && i % intervalLevel3Lat == 0) {
		color = Colors.level3Color.getColor();
		showRuler = showRuler || showRuler3;
	    }	    
	    lines.add(constructParrallelLine(left, right, lat, color));
	    if (showRuler)
		lines.add(constructLabel(lat, left, lat, latLabelXoffset, latLabelYoffset, false, latStep, OMText.JUSTIFY_LEFT));	
	}

	// Prime meridian
	if (right > 0 && 0 > left) {
	    OMPoly meridian = constructMeridianLine(up, down, 0, Colors.dateLineColor.getColor());
	    lines.add(meridian);
	    if (showRuler1 || showRuler2 || showRuler3)
		lines.add(constructLabel(down, 0f, 0f, lonLabelXoffset, lonLabelYoffset, true, lonStep, OMText.JUSTIFY_LEFT));
	}

	// date line
	if (Math.abs(right-180) < 1) {
	    OMPoly meridian = constructMeridianLine(up, down, 180f,Colors.dateLineColor.getColor());
	    lines.add(meridian);
	    if (showRuler1 || showRuler2 || showRuler3)
		lines.add(constructLabel(down, 180f, 180f, lonLabelXoffset, lonLabelYoffset, true, lonStep, OMText.JUSTIFY_LEFT));
	}

	// meridians for the west hemisphere
	i = 1;
	for (float lon = lonStep; lon < right ; lon += lonStep, i++) {
	    Color color = Colors.level1Color.getColor();
	    boolean showRuler = showRuler1;
	    if (intervalLevel2Lon > 0 && i % intervalLevel2Lon == 0) {
		color = Colors.level2Color.getColor();
		showRuler = showRuler || showRuler2;
	    }
	    if (intervalLevel3Lon > 0 && i % intervalLevel3Lon == 0) {
		color = Colors.level3Color.getColor();
		showRuler = showRuler || showRuler3;
	    }	    
	    lines.add(constructMeridianLine(up, down, lon, color));
	    if (showRuler)
		lines.add(constructLabel(down, lon, lon, lonLabelXoffset, lonLabelYoffset, true, lonStep, OMText.JUSTIFY_LEFT));	
	}
	// meridians for the est hemisphere
	i=1;
	for (float lon = -lonStep; lon > left ; lon -= lonStep, i++) {
	    Color color = Colors.level1Color.getColor();
	    boolean showRuler = showRuler1;
	    if (intervalLevel2Lon > 0 && i % intervalLevel2Lon == 0) {
		color = Colors.level2Color.getColor();
		showRuler = showRuler || showRuler2;
	    }
	    if (intervalLevel3Lon > 0 && i % intervalLevel3Lon == 0) {
		color = Colors.level3Color.getColor();
		showRuler = showRuler || showRuler3;
	    }	    
	    lines.add(constructMeridianLine(up, down, lon, color));
	    if (showRuler)
		lines.add(constructLabel(down, lon, lon, lonLabelXoffset, lonLabelYoffset, true, lonStep, OMText.JUSTIFY_LEFT));	
	}

        if (Debug.debugging("graticule")) {
            Debug.output("GraticuleLayer.constructGraticuleLines("+left+","+right+"): " + "constructed " + lines.size() + " graticule lines");
        }
        lines.generate(projection);
        return lines;
    }

    public void refreshGraphics() {
        if (isVisible()) {
            doPrepare();
        }
    }

    public void actionPerformed(ActionEvent e) {
	Colors c = Colors.valueOf(e.getActionCommand()) ;
	JButton source = (JButton) e.getSource();
	c.setColor(ThemaMapColorChooser.showDialog(source, "Choose Color", c.getColor()));
	source.setBackground(c.getColor());
	refreshGraphics();
    }

    class PropertyColorButton extends JPanel {

        public PropertyColorButton(final Colors c) {
            super();
	    JButton b = new JButton();
            ComponentSizer.fixComponentSize(b, new Dimension(15, 15));
	    b.setActionCommand(c.toString());
	    b.addActionListener(GraticuleLayer.this);
            b.setBackground(c.getColor());
	    add(b);
        }
    }
}
