// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.chartLayer;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;
import fr.unicaen.openmap.dataStorage.DataStorageIterator;
import fr.unicaen.openmap.layer.chartLayer.chart.Chart;
import fr.unicaen.openmap.layer.chartLayer.chart.ThemaMapChartFactory;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.AttributesColorScalePanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ColorScalePanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.AttributesNames;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.AttributesPanel;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BoxAndWhiskerRenderer;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.general.DatasetUtilities;
import org.jfree.data.general.DefaultPieDataset;
import org.jfree.data.statistics.DefaultBoxAndWhiskerCategoryDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;


/**
 *
 * @author jchauvea
 */
public class ChartLayerPanel extends JPanel {

    protected I18n i18n = Environment.getI18n();


    protected ChartLayer chartLayer;
    private Chart.ChartType chartType = Chart.ChartType.XY;
    protected AttributesPanel attributesCategoriesPanel;
    protected AttributesPanel attributesSeriesPanel;
    //private JComboBox typeCombo;
       

    public ChartLayerPanel(final ChartLayer chartLayer) {
        this.chartLayer = chartLayer;
        final JPanel panType = new JPanel();
        Box boxLeft = Box.createVerticalBox();
        //ButtonGroup bg = new ButtonGroup();
        final JRadioButton radioXY = new JRadioButton("X Y");
        radioXY.setSelected(true);
        final JRadioButton radioPie = new JRadioButton("Camembert");
        final JRadioButton radioHisto = new JRadioButton("Histogramme");
        final JRadioButton radioHisto2 = new JRadioButton("Histogramme");
        final JRadioButton radioArea = new JRadioButton("Area");
        final JRadioButton radioStacked = new JRadioButton("Stacked bar");
        final JRadioButton radioBox = new JRadioButton("Box and Whisker");


        String[] categories = {"Catégorie 1"/*,"Catégorie 2"*/};
        final JComboBox comboChartCategory = new JComboBox(categories);
        //Box panCat1 = Box.createVerticalBox();
        Box panCat2 = Box.createVerticalBox();
        Box panCat3 = Box.createVerticalBox();
        final JPanel panCategories = new JPanel();
        final CardLayout cardCategories = new CardLayout();
        panCategories.setLayout(cardCategories);
        //panCategories.add("0", panCat1);
        panCategories.add("0", panCat2);
        panCategories.add("1", panCat3);
        //panCat1.add(new JLabel("<html>Un série se définie par<br/> deux colonnes de<br/> données (x/y)</html>"));
        //ButtonGroup groupCat1 = new ButtonGroup();
        ButtonGroup groupCat2 = new ButtonGroup();
        ButtonGroup groupCat3 = new ButtonGroup();
        groupCat2.add(radioXY);
        groupCat2.add(radioHisto);
        groupCat2.add(radioArea);
        groupCat2.add(radioStacked);
        groupCat3.add(radioBox);
        groupCat3.add(radioHisto2);
        groupCat3.add(radioPie);
        panCat2.add(radioXY);
        panCat2.add(radioHisto);
        panCat2.add(radioArea);
        panCat2.add(radioStacked);
        panCat3.add(radioBox);
        panCat3.add(radioHisto2);
        panCat3.add(radioPie);
        boxLeft.add(comboChartCategory);
        boxLeft.add(panCategories);
//        boxLeft.add(radioXY);
//        boxLeft.add(radioPie);
//        boxLeft.add(radioHisto);
//        boxLeft.add(radioArea);
//        boxLeft.add(radioStacked);
//        boxLeft.add(radioBox);
//        bg.add(radioHisto);
//        bg.add(radioXY);
//        bg.add(radioArea);
//        bg.add(radioStacked);
//        bg.add(radioBox);
//        bg.add(radioPie);
        final JPanel panOptions = new JPanel();
        //panOptions.setBorder(BorderFactory.createTitledBorder(""));
        JPanel panSize = new JPanel();
        //panSize.setBorder(BorderFactory.createTitledBorder(""));//i18n.get(this, "Dimensions","Dimensions")));
        /*JLabel labWidth = new JLabel(i18n.get(this, "width", "Width"));
        JSpinner spinWidth = new JSpinner();
        JLabel labHeight = new JLabel(i18n.get(this, "height", "Height"));
        JSpinner spinHeight = new JSpinner();*/
        //spinWidth.setPreferredSize(new Dimension(50, 20));
        //spinHeight.setPreferredSize(new Dimension(50, 20));
        //JPanel p1 = new JPanel();
        //p1.add(labWidth);
        //JPanel p2 = new JPanel();
        //p2.add(spinWidth);
        //JPanel p3 = new JPanel();
        //p3.add(labHeight);
        //JPanel p4 = new JPanel();
        //p4.add(spinHeight);
        // panSize.add(p1);
        //panSize.add(p2);
        //panSize.add(p3);
        // panSize.add(p4);
        JPanel panTitles = new JPanel(new GridLayout(2, 2));
        JPanel p5 = new JPanel();
        p5.add(new JLabel("Titre axe x"));
        JPanel p6 = new JPanel();
        JTextField fieldX = new JTextField("", 10);
        p6.add(fieldX);
        JPanel p7 = new JPanel();
        p7.add(new JLabel("Titre axe y"));
        JPanel p8 = new JPanel();
        JTextField fieldY = new JTextField("", 10);
        p8.add(fieldY);
        panTitles.add(p5);
        panTitles.add(p6);
        panTitles.add(p7);
        panTitles.add(p8);
        panOptions.add(panSize);
        //panOptions.add(panTitles);
        //final JLabel labImage = new JLabel(Util.loadIcon("xy.png"));
        final JPanel chartOverviewPanel = new JPanel();
        final ChartPanel xyPanel = buildXYSampleChart();
        final ChartPanel barChartPanel = buildBarSampleChart();
        final ChartPanel barChartPanel2 = buildBarSampleChart();
        final ChartPanel areaPanel = buildAreaSampleChart();
        final ChartPanel stackedBarPanel = buildStackedBarSampleChart();
        final ChartPanel boxAndWhiskerPanel = buildBoxAndWhiskerSampleChart();
        final ChartPanel piePanel = buildPieSampleChart();
        ComponentSizer.fixComponentSize(xyPanel, new Dimension(280, 180));
        ComponentSizer.fixComponentSize(barChartPanel, new Dimension(280, 180));
        ComponentSizer.fixComponentSize(barChartPanel2, new Dimension(280, 180));
        ComponentSizer.fixComponentSize(areaPanel, new Dimension(280, 180));
        ComponentSizer.fixComponentSize(stackedBarPanel, new Dimension(280, 180));
        ComponentSizer.fixComponentSize(boxAndWhiskerPanel, new Dimension(280, 180));
        ComponentSizer.fixComponentSize(piePanel, new Dimension(280, 180));
        panType.add(boxLeft);
        Box boxRight = Box.createVerticalBox();
        boxRight.add(chartOverviewPanel);
        boxRight.add(panTitles);
        panType.add(boxRight);

        final CardLayout sampleCardLayout = new CardLayout();
        chartOverviewPanel.setLayout(sampleCardLayout);
        final JPanel sampleCat1 = new JPanel();
        final JPanel sampleCat2 = new JPanel();
        final CardLayout cat1Card = new CardLayout();
        final CardLayout cat2Card = new CardLayout();
        sampleCat1.setLayout(cat1Card);
        sampleCat2.setLayout(cat2Card);
        chartOverviewPanel.add("0", sampleCat1);
        chartOverviewPanel.add("1", sampleCat2);
        sampleCat1.add("xy", xyPanel);
        sampleCat1.add("bar", barChartPanel);
        sampleCat1.add("area", areaPanel);
        sampleCat1.add("stacked", stackedBarPanel);
        sampleCat2.add("box", boxAndWhiskerPanel);
        sampleCat2.add("bar", barChartPanel2);
        sampleCat2.add("pie", piePanel);
        final JPanel panChartConf = new JPanel();
        panChartConf.setBorder(BorderFactory.createTitledBorder("Données à représenter"));
        JComponent panConfCat1 = buildFirstCategoryConfigurationPanel(chartLayer.getDataStorageIterator());
        JComponent panConfCat2 = buildSecondCategoryConfigurationPanel(chartLayer.getDataStorageIterator());
        final CardLayout cardConfiguration = new CardLayout();
        panChartConf.setLayout(cardConfiguration);
        panChartConf.add("0", panConfCat1);
        panChartConf.add("1", panConfCat2);
        comboChartCategory.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent arg0) {
                cardCategories.show(panCategories, String.valueOf(comboChartCategory.getSelectedIndex()));
                sampleCardLayout.show(chartOverviewPanel, String.valueOf(comboChartCategory.getSelectedIndex()));
                cardConfiguration.show(panChartConf, String.valueOf(comboChartCategory.getSelectedIndex()));
            }
        });
        //chartOverviewPanel.add(xyPanel);
        //final JPanel panData = new JPanel(new BorderLayout());
        //JLabel labHelp = new JLabel(i18n.get(this, "help_lists", ""));
        //Font f = labHelp.getFont();
        //labHelp.setFont(f.deriveFont(f.getStyle() ^ Font.BOLD));
        //final JPanel panLists = new JPanel(new GridLayout(1, 3));
        //final DefaultListModel columnListModel = new DefaultListModel();
        //for (String columnName : chartLayer.getDataStorageIterator().getDataColumnNames()) {
        //    columnListModel.addElement(columnName);
        // }
//        final JList columnList = new JList(columnListModel);
//        JPanel panButton = new JPanel();
//        panButton.setLayout(new GridLayout(4, 1));
//        panButton.add(new JPanel());
//        JPanel panAdd = new JPanel();
//        JButton bAdd = new JButton(">>");
//        panAdd.add(bAdd);
//        JPanel panRemove = new JPanel();
//        JButton bRemove = new JButton("<<");
//        bAdd.setBackground(Color.WHITE);
//        bRemove.setBackground(Color.WHITE);
//        bAdd.setOpaque(false);
//        bRemove.setOpaque(false);
//        panRemove.add(bRemove);
//        panButton.add(panAdd);
//        panButton.add(panRemove);
//        panButton.add(new JPanel());
        //final DefaultListModel selectedListModel = new DefaultListModel();
        //final JList selectedColumnList = new JList(selectedListModel);
        //JScrollPane scColList = new JScrollPane(columnList);
        //JScrollPane scSelectedColList = new JScrollPane(selectedColumnList);
        //panLists.add(scColList);
        //panLists.add(panButton);
        //panLists.add(scSelectedColList);
//        ComponentSizer.fixComponentSize(scColList, new Dimension(180,180));
//        ComponentSizer.fixComponentSize(scSelectedColList, new Dimension(180,180));
//        ComponentSizer.fixComponentSize(panLists, new Dimension(350,200));
        //panData.add(labHelp, BorderLayout.NORTH);
        //panData.add(panLists, BorderLayout.CENTER);
        ActionListener radioListener = new ActionListener() {

            public void actionPerformed(ActionEvent ae) {
                Object source = ae.getSource();
                if(source == radioXY){
                    cat1Card.show(sampleCat1, "xy");
                    chartType = Chart.ChartType.XY;
                    return;
                }
                if(source == radioHisto){
                    cat1Card.show(sampleCat1, "bar");
                    chartType = Chart.ChartType.BAR;
                    return;
                }
                if(source == radioArea){
                    cat1Card.show(sampleCat1, "area");
                    return;
                }
                if(source == radioStacked){
                    cat1Card.show(sampleCat1, "stacked");
                    return;
                }
                 if(source == radioBox){
                    cat2Card.show(sampleCat2, "box");
                    return;
                }
                if(source == radioHisto2){
                    cat2Card.show(sampleCat2, "bar");
                    return;
                }
                if(source == radioPie){
                    cat2Card.show(sampleCat2, "pie");
                    return;
                }
            }
        };
        radioHisto.addActionListener(radioListener);
        radioHisto2.addActionListener(radioListener);
        radioBox.addActionListener(radioListener);
        radioArea.addActionListener(radioListener);
        radioBox.addActionListener(radioListener);
        radioStacked.addActionListener(radioListener);
        radioXY.addActionListener(radioListener);
        radioPie.addActionListener(radioListener);
        radioStacked.addActionListener(radioListener);
        /*radioHisto.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (radioHisto.isSelected()) {
                    //chart.setChartType(Chart.ChartType.BAR);
                    chartOverviewPanel.removeAll();
                    chartOverviewPanel.add(barChartPanel);
                    chartOverviewPanel.updateUI();
                    //chart.getLayer().refresh();
                }
            }
        });
        radioXY.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (radioXY.isSelected()) {
                    //chart.setChartType(Chart.ChartType.XY);
                    chartOverviewPanel.removeAll();
                    chartOverviewPanel.add(xyPanel);
                    chartOverviewPanel.updateUI();
                    //chart.getLayer().refresh();
                }

            }
        });

        radioArea.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (radioArea.isSelected()) {
                    //chart.setChartType(Chart.ChartType.XY);
                    chartOverviewPanel.removeAll();
                    chartOverviewPanel.add(areaPanel);
                    chartOverviewPanel.updateUI();
                    //chart.getLayer().refresh();
                }

            }
        });

        radioStacked.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (radioStacked.isSelected()) {
                    //chart.setChartType(Chart.ChartType.XY);
                    chartOverviewPanel.removeAll();
                    chartOverviewPanel.add(stackedBarPanel);
                    chartOverviewPanel.updateUI();
                    //chart.getLayer().refresh();
                }

            }
        });

        radioBox.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (radioBox.isSelected()) {
                    //chart.setChartType(Chart.ChartType.XY);
                    chartOverviewPanel.removeAll();
                    chartOverviewPanel.add(boxAndWhiskerPanel);
                    chartOverviewPanel.updateUI();
                    //chart.getLayer().refresh();
                }

            }
        });

        radioPie.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (radioPie.isSelected()) {
                    //chart.setChartType(Chart.ChartType.XY);
                    chartOverviewPanel.removeAll();
                    chartOverviewPanel.add(piePanel);
                    chartOverviewPanel.updateUI();
                    //chart.getLayer().refresh();
                }

            }
        });*/
        //buttons add & remove listeners
//        bAdd.addActionListener(new ActionListener() {
//
//            public void actionPerformed(ActionEvent e) {
//                String value = (String) columnList.getSelectedValue();
//                selectedListModel.addElement(value);
//                columnListModel.removeElement(value);
//                chartLayer.addXName(value);
//                selectedColumnList.updateUI();
//                columnList.updateUI();
//            }
//        });
//        bRemove.addActionListener(new ActionListener() {
//
//            public void actionPerformed(ActionEvent e) {
//                String value = (String) selectedColumnList.getSelectedValue();
//                columnListModel.addElement(value);
//                selectedListModel.removeElement(value);
//                chartLayer.removeXName(value);
//                selectedColumnList.updateUI();
//                columnList.updateUI();
//            }
//        });
        //spiners listeners
//        spinWidth.addChangeListener(new ChangeListener() {
//
//            public void stateChanged(ChangeEvent event) {
//                String val = (((JSpinner) event.getSource()).getValue()).toString();
//                //chart.setChartWidth(Integer.parseInt(val));
//
//            }
//        });
        JPanel panLeft = new JPanel();
        Box panRight = Box.createVerticalBox();
        panLeft.add(panType);
        ComponentSizer.fixComponentSize(panLeft, new Dimension(450, 290));
        ComponentSizer.fixComponentSize(panRight, new Dimension(450, 290));
        panLeft.setBorder(BorderFactory.createTitledBorder("Type de graphique"));
        panRight.add(panChartConf);
//        JPanel panParam = new JPanel();
//        panParam.add(new JLabel("param"));
//        panParam.setBorder(BorderFactory.createTitledBorder("Paramètres du graphique"));
//        panRight.add(panParam);

        JPanel mainPanel = new JPanel(new BorderLayout());
        JPanel wrapLeftRight = new JPanel();
        wrapLeftRight.add(panLeft);
        wrapLeftRight.add(panRight);
        JButton bDraw = new JButton("Appliquer");
        bDraw.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {

                applyAndDraw();
                
                /*ChartLayerPanel.this.chartLayer.setChart(ThemaMapChartFactory.buildChart(
                        chartType,
                        ChartLayerPanel.this.chartLayer.getDataStorageIterator(),
                        "CHART NAME",
                        "X",
                        "Y",
                        ChartLayerPanel.this.chartLayer.getColorizer().getColorScale().getColorNumber(),
                        xCols,
                        TOOL_TIP_TEXT_KEY,
                        null));*/
            }
        });
        bDraw.setBackground(Color.WHITE);
        bDraw.setOpaque(false);
        JPanel pBottom = new JPanel();
        pBottom.add(bDraw);
        mainPanel.add(wrapLeftRight, BorderLayout.CENTER);
        mainPanel.add(panOptions, BorderLayout.NORTH);
        mainPanel.add(pBottom, BorderLayout.SOUTH);
        add(mainPanel);


        //temp
        //radioXY.setEnabled(false);
        radioArea.setEnabled(false);
        radioStacked.setEnabled(false);
    }

    private void applyAndDraw(){
         chartLayer.buildChartAndDrawFromParameters(chartType, "NAME",
                        "X",
                        "Y",
                        attributesCategoriesPanel.getSelected(),
                        attributesSeriesPanel.getSelected()
                        );
    }

    public void refreshGUI(){
        attributesCategoriesPanel.setAvailables(chartLayer.getDataStorageIterator().getDataColumnNames());

    }

    public static ChartPanel buildXYSampleChart() {
        XYSeries series = new XYSeries("Random Data");
        series.add(1.0, 500.2);
        series.add(5.0, 694.1);
        series.add(4.0, 100.0);
        series.add(12.5, 734.4);
        series.add(17.3, 453.2);
        series.add(21.2, 500.2);
        series.add(21.9, null);
        series.add(25.6, 734.4);
        series.add(30.0, 453.2);
        final XYSeriesCollection data = new XYSeriesCollection();
        data.addSeries(series);
        final JFreeChart chart = ChartFactory.createXYLineChart(
                "XY Demo",
                "X",
                "Y",
                data,
                PlotOrientation.VERTICAL,
                true,
                true,
                false);
        chart.setBackgroundPaint(new Color(255, 255, 255, 0));
        ChartPanel chartPanel = new ChartPanel(chart);
        chartPanel.setPreferredSize(new java.awt.Dimension(500, 270));
        return chartPanel;
    }

    public static ChartPanel buildBarSampleChart() {
        DefaultCategoryDataset dataset = new DefaultCategoryDataset();

        dataset.setValue(2, "A", "A");
        dataset.setValue(
                7, "A", "B");
        dataset.setValue(
                4, "A", "C");
        dataset.setValue(
                9, "A", "D");
        dataset.setValue(
                6, "A", "E");

        dataset.setValue(4, "X", "A");
        dataset.setValue(
                6, "X", "B");
        dataset.setValue(
                9, "X", "C");
        dataset.setValue(
                5, "X", "D");
        dataset.setValue(
                1, "X", "E");

        JFreeChart chart = ChartFactory.createBarChart("BarChart Demo", "Student", "Marks", dataset,
                PlotOrientation.VERTICAL, false, true, false);
        chart.setBackgroundPaint(new Color(255, 255, 255, 0));
        //CategoryPlot p = chart.getCategoryPlot();
        return new ChartPanel(chart);

    }

    public static ChartPanel buildAreaSampleChart() {

        double[][] data = new double[][]{
            {1.0, 4.0, 3.0, 5.0, 5.0, 7.0, 7.0, 8.0},
            {5.0, 7.0, 6.0, 8.0, 4.0, 4.0, 2.0, 1.0},
            {4.0, 3.0, 2.0, 3.0, 6.0, 3.0, 4.0, 3.0}
        };

        final CategoryDataset dataset = DatasetUtilities.createCategoryDataset(
                "Series ", "", data);

        JFreeChart chart = ChartFactory.createAreaChart(
                "Area Chart", // chart title
                "Category", // domain axis label
                "Value", // range axis label
                dataset, // data
                PlotOrientation.VERTICAL, // orientation
                true, // include legend
                true, // tooltips
                false // urls
                );
        CategoryPlot plot = chart.getCategoryPlot();
        plot.setForegroundAlpha(0.5f);
        chart.setBackgroundPaint(new Color(255, 255, 255, 0));
        return new ChartPanel(chart);
        //chartPanel.setPreferredSize(new Dimension(500, 270));
        //chartPanel.setEnforceFileExtensions(false);
    }

    public static ChartPanel buildStackedBarSampleChart() {
        double[][] data = new double[][]{
            {210, 300, 320, 265, 299, 200},
            {200, 304, 201, 201, 340, 300},};
        CategoryDataset dataset = DatasetUtilities.createCategoryDataset("Team ", "M", data);
        JFreeChart chart = ChartFactory.createStackedBarChart(
                "Stacked Bar Chart ", "", "Score",
                dataset, PlotOrientation.VERTICAL, true, true, false);

        CategoryPlot plot = chart.getCategoryPlot();
        plot.getRenderer().setSeriesPaint(0, new Color(255, 0, 0));
        plot.getRenderer().setSeriesPaint(1, new Color(0, 0, 255));
        chart.setBackgroundPaint(new Color(255, 255, 255, 0));
        return new ChartPanel(chart);


    }

    public static ChartPanel buildBoxAndWhiskerSampleChart() {
        DefaultBoxAndWhiskerCategoryDataset dataset = new DefaultBoxAndWhiskerCategoryDataset();
        final int seriesCount = 3;
        final int categoryCount = 4;
        final int entityCount = 22;
        for (int i = 0; i < seriesCount; i++) {
            for (int j = 0; j < categoryCount; j++) {
                final List<Double> list = new ArrayList<Double>();
                // add some values...
                for (int k = 0; k < entityCount; k++) {
                    final double value1 = 10.0 + Math.random() * 3;
                    list.add(new Double(value1));
                    final double value2 = 11.25 + Math.random(); // concentrate values in the middle
                    list.add(new Double(value2));
                }
                dataset.add(list, "Series " + i, " Type " + j);
            }

        }
        final CategoryAxis xAxis = new CategoryAxis("Type");
        final NumberAxis yAxis = new NumberAxis("Value");
        yAxis.setAutoRangeIncludesZero(false);
        final BoxAndWhiskerRenderer renderer = new BoxAndWhiskerRenderer();
        renderer.setFillBox(false);
        //renderer.setToolTipGenerator(new BoxAndWhiskerToolTipGenerator());
        final CategoryPlot plot = new CategoryPlot(dataset, xAxis, yAxis, renderer);

        final JFreeChart chart = new JFreeChart(
                "Box-and-Whisker Demo",
                new Font("SansSerif", Font.BOLD, 14),
                plot,
                true);
        chart.setBackgroundPaint(new Color(255, 255, 255, 0));
        return new ChartPanel(chart);
    }

    public static ChartPanel buildPieSampleChart() {
        DefaultPieDataset dataset = new DefaultPieDataset();
        /*for (String x : xNames) {
        dataset.setValue(x, );
        }*/
        dataset.setValue("S 1", 23.3);
        dataset.setValue("S 2", 56.5);
        dataset.setValue("S 3", 43.3);
        dataset.setValue("S 4", 11.1);
        JFreeChart chart = ChartFactory.createPieChart("Pie demo", dataset, false, false, true);
        chart.setBackgroundPaint(new Color(255, 255, 255, 0));
        return new ChartPanel(chart);

    }

    private JComponent buildFirstCategoryConfigurationPanel(final DataStorageIterator dsi){
        //JLabel labXCol = new JLabel("Abscisses");
        //ComponentSizer.fixComponentSize(labXCol, new Dimension(100,20));
        //JTextField fieldXCol = new JTextField("",15);
        /*CheckListItem[] xListItem = new CheckListItem[dsi.getDataColumnNames().length];
        int index = 0;
        for (String v : dsi.getDataColumnNames()) {
            xListItem[index] = new CheckListItem(v);
            //xListItem[index].setSelected(filterColumnValues.contains(v));
            index++;
        }
        JList xList = new JList(xListItem);
        xList.setCellRenderer(new ColoredCheckListRenderer());
        xList.setSelectionMode(
                ListSelectionModel.SINGLE_SELECTION);

        xList.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent event) {
                //filterColumnValues.clear();
                JList list = (JList) event.getSource();
                // Get index of item clicked
                int index = list.locationToIndex(event.getPoint());
                Object item = list.getModel().getElementAt(index);
                if(item instanceof CheckListItem){
                // Toggle selected state
                ((CheckListItem)item).setSelected(!((CheckListItem)item).isSelected());
                }
                // Repaint cell
                list.repaint(list.getCellBounds(index, index));

            }
        });*/
//        Box checks = Box.createVerticalBox();
//        for (String v : dsi.getDataColumnNames()) {
//            JPanel p = new JPanel(new BorderLayout());
//            JCheckBox check = new JCheckBox(v);
//            p.add(check, BorderLayout.WEST);
//            JButton bColor = new JButton();
//            ComponentSizer.fixComponentSize(bColor, new Dimension(10,10));
//            JPanel wrap = new JPanel();
//            wrap.add(bColor);
//            p.add(wrap, BorderLayout.EAST);
//            checks.add(p);
//        }

        //AttributesColorScalePanel colorScalePanel = new AttributesColorScalePanel(chartLayer.getColorizer().getColorScale(), (AttributesNames) chartLayer);
        //colorScalePanel.setNumberLabel("Nombre de séries");
        //colorScalePanel.setAvailableNames(chartLayer.getNoColorDataColumnNames());
        //JScrollPane sc = new JScrollPane(checks);
        //ComponentSizer.fixComponentSize(sc, new Dimension(200,100));
        //JButton addXColButton = new JButton("+");
        //JLabel labYCol = new JLabel("Ordonnées");
        //ComponentSizer.fixComponentSize(labYCol, new Dimension(100,20));
        //typeCombo = new JComboBox(dsi.getDataColumnNames());
        //ComponentSizer.fixComponentSize(typeCombo, new Dimension(200,20));
        Box panel = Box.createVerticalBox();
        JPanel panx = new JPanel();
       // panx.add(labXCol);
        //int n = chartLayer.getChart() == null ? 1 : chartLayer.getChart().getNBTypes();
        int n = 1;
        attributesSeriesPanel = new AttributesPanel(dsi.getDataColumnNames(), n);
        panx.add(attributesSeriesPanel/*new JScrollPane(ap, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,JScrollPane.HORIZONTAL_SCROLLBAR_NEVER)*/);
        panx.setBorder(BorderFactory.createTitledBorder("Séries / Statistiques"));
        JPanel pany = new JPanel();
        //pany.add(labYCol);
        attributesCategoriesPanel = new AttributesPanel( dsi.getDataColumnNames(), n);
        pany.add(attributesCategoriesPanel);
        pany.setBorder(BorderFactory.createTitledBorder("Catégories / Lignes"));
        panel.add(panx);
        panel.add(pany);
        return panel;
    }
    private JComponent buildSecondCategoryConfigurationPanel(DataStorageIterator dsi){
        return new JLabel("second category panel");}
}
