// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.dataDisplayLayer;

import com.bbn.openmap.MapBean;
import com.bbn.openmap.MouseDelegator;
import com.bbn.openmap.gui.DefaultHelpMenu;
import java.util.Properties;
import com.bbn.openmap.layer.OMGraphicHandlerLayer;
import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.proj.*;
import com.bbn.openmap.proj.coords.LatLonPoint;
import com.bbn.openmap.util.*;

import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.dataStorage.shapefile.ShapeFileDataStorage;
import fr.unicaen.openmap.layer.preLayer.UnicaenLayerListener;
import fr.unicaen.openmap.layer.AtlasLayer;
import fr.unicaen.openmap.util.AboutMenuItemBuildVersion;
import fr.unicaen.openmap.util.propertyEditor.DataStoragePropertyEditor;
import java.awt.Component;
import java.awt.Graphics;
import java.beans.beancontext.BeanContext;
import java.util.Iterator;
import java.util.Set;
import javax.swing.JOptionPane;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;
import com.bbn.openmap.layer.policy.BufferedImageRenderPolicy;

/**
 * Sample implementation of what we need in a DataDisplayLayer ...
 * This class is declared abstract.
 */
public abstract class SimpleDataDisplayLayer extends OMGraphicHandlerLayer implements DataStorageListener, UnicaenLayerListener, AnimationDataStorageListener, AtlasLayer {

    //Used to keep coords in properties
    public static final String scalePropertyString = "scale";
    public static final String latitudePropertyString = "latitude";
    public static final String longitudePropertyString = "longitude";
    //without specifics properties, hello world :
    protected double scale = 127306264f;
    protected double longitude = -20f;
    protected double latitude = 20f;
    private boolean initializedScale = false;
    /**
     * Use I18N mecanism to allow internationalization.
     * Use I18N.properties file or I18N_fr_FR.properties for french version
     **/
    protected I18n i18n = new BasicI18n();
    /**
     * Property string for the DataStorage class name.
     */
    static public String dataStorageClassPropertyString = "dataStorageClass";
    /**
     * Layer class name, found unsing java.lang.reflect.
     */
    protected String classNameForDebug = Util.className(this);
    /**
     * The iterator that makes the link with real data
     * encapsulated in a DataStorage.
     */
    protected DataStorageIterator dataStorageIterator;

    public DataStorageIterator getDataStorageIterator() {
        return dataStorageIterator;
    }

    public void setDataStorageIterator(DataStorageIterator dsi) {
        dataStorageIterator = dsi;
        refreshDSGUI();
    }
    /**
     *
     */
    private String createPalettePropertyString = "createPalette";
    /**
     *
     */
    protected boolean createPalette = true;
    /**
     * Property string for the visibility of the GUI DataStorage.
     */
    static public String showGUIDataStoragePropertyString = "showGUIDataStorage";
    protected boolean showGUIDataStorage = true;

    public SimpleDataDisplayLayer() {
        // please set policy in properties:

        //        setProjectionChangePolicy(new com.bbn.openmap.layer.policy.ListResetPCPolicy(this));
        BufferedImageRenderPolicy rp = new com.bbn.openmap.layer.policy.BufferedImageRenderPolicy(this);
        //rp.setUseImageBuffer(true);
        setRenderPolicy(rp);
        Debug.message(classNameForDebug, this + " new SimpleDataDisplayLayer()");
    }

    /**
     * Standard way to get the class name of the DataStorage needed to initialize
     * the DataStorageIterator. Do not forget to call this method in your
     * subclasses if you want to overrride it (e.g. <code>super.setProperties(prefix, properties);</code>
     * must be the first line of your overriding method).
     * @param prefix string prefix used in the properties file for this layer
     * @param properties the properties set in the properties file
     */
    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);

        Debug.message(classNameForDebug, this + " setProperties(...) : SimpleDisplayLayer");// from "+ properties + " ("+prefix+")");
        Debug.message("inspectordetail", DS_GLOBAL_CONFIGURATION_PROPERTY_STRING + "=> " + properties.getProperty(prefix + "." + DS_GLOBAL_CONFIGURATION_PROPERTY_STRING));
        //If a Layer is dynamiquely added, verify if this property is here... It's essential : we need to create *and* initialize a DataStorage !
        Properties special = PropUtils.parsePropertyList(properties.getProperty(prefix + "." + DS_GLOBAL_CONFIGURATION_PROPERTY_STRING), DataStoragePropertyEditor.propertySeparator, DataStoragePropertyEditor.keyValueSeparator);
        //	System.out.println("DS global configuration = " + special);

        //parse correctly
        Set<String> keys = special.stringPropertyNames();
        String key = "";
        for (Iterator<String> iterator = keys.iterator(); iterator.hasNext();) {
            key = iterator.next();
            properties.put(prefix + key, special.getProperty(key));
            properties.remove(key); //useless, but clean
        }
        //	System.out.println(properties);

        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        createPalette = Util.isTrue(properties.getProperty(prefix + createPalettePropertyString, "true"));
        showGUIDataStorage = Util.isTrue(properties.getProperty(prefix + showGUIDataStoragePropertyString, "true"));
        String className = properties.getProperty(prefix + dataStorageClassPropertyString);
        //System.out.println("SIMPLE LAYER DS CLASSNAME" + className);
	DataStorage ds = Util.loadDataStorage(className, prefix, properties, this);
	if (ds == null /*|| ! ds.isReady()*/) {
	    // Show message only if ds is null, otherwise an error message has been showned
	    if (ds == null) 
		JOptionPane.showMessageDialog(null, 
					      String.format(i18n.get(this, "errorDSConstruction", 
								     "<html><p>Impossible to create a %s for layer %s.<p> Create a default ShapeFileDataStorage.</html>"),
							    className, toString()), 
					      "Warning", 
					      JOptionPane.ERROR_MESSAGE);
	    ds = new ShapeFileDataStorage(this);
	    // remove wrong shapefile ou dbffile info from properties
	    properties.remove(prefix+ShapeFileDataStorage.PARAM_SHP);
	    properties.remove(prefix+ShapeFileDataStorage.PARAM_DBF);
	    ds.setProperties(prefix, properties);
	}
        dataStorageIterator = new DataStorageIteratorAdapter(ds);

        //restore specific coords properties, if any
        scale = PropUtils.doubleFromProperties(properties, prefix + scalePropertyString, scale);
        longitude = PropUtils.doubleFromProperties(properties, prefix + longitudePropertyString, longitude);
        latitude = PropUtils.doubleFromProperties(properties, prefix + latitudePropertyString, latitude);
    }

    /**
     * The properties and prefix are managed and decoded here, for the standard uses of SimpleDataDisplayLayer.
     * Notes this method allows to define what is present in file "save Map as properties file".
     * @param properties the properties set in  the properties file
     * @return Properties.
     */
    public Properties getProperties(Properties properties) {
        properties = super.getProperties(properties);
        Debug.message(classNameForDebug, this + " SimpleDataDisplay getProperties(...)");

        String prefix = PropUtils.getScopedPropertyPrefix(this);

        // we try to get revision stored in AboutMenuItemBuildVersion if this menu is loaded
        // AboutMenuItemBuildVersion is added to HelpMenuItem

        // helpMenu.class=com.bbn.openmap.gui.DefaultHelpMenu
        // helpMenu.items=helpUserMenuItem AboutMenuItem
        // helpUserMenuItem.class=com.bbn.openmap.gui.UserGuideMenuItems
        // helpUserMenuItem.class=com.bbn.openmap.gui.menu.WebSiteHelpMenuItem
        // AboutMenuItem.class=fr.unicaen.openmap.util.AboutMenuItemBuildVersion

        // get context
        BeanContext mapHandler = getBeanContext();

        AboutMenuItemBuildVersion aboutMenu = null;
        MapBean map = null;

        if (mapHandler != null) {
            Iterator it = mapHandler.iterator();
            Object someObj = null;
//            MapBean map = null;
            DefaultHelpMenu menu = null;
//	    MouseDelegator msd = null;
            while (it.hasNext()) {
                someObj = it.next();
                if (someObj instanceof DefaultHelpMenu) {
                    menu = (DefaultHelpMenu) someObj;
                } else if (someObj instanceof MapBean) {// used to get projection and center
                    map = (MapBean) someObj;
                }/* else if (someObj instanceof MouseDelegator){
                msd = (MouseDelegator) someObj;
                }*/
            }

            if (menu != null) {
                // we get help menu, let's get aboutMenu information
                int nb_menus = menu.getItemCount();
                for (int i = 0; i < nb_menus; i++) {
                    if (menu.getItem(i) instanceof AboutMenuItemBuildVersion) {
                        aboutMenu = (AboutMenuItemBuildVersion) menu.getItem(i);
                    }
                }
            }
        }
        // we get revision information, so we can tag properties.
        if (aboutMenu != null) {
            try {
                properties.put(prefix + "revision", aboutMenu.getRevision().toString());
            } catch (NullPointerException e) {
                System.err.println("Unknown Revision");
            }
        }
        properties.put(prefix + createPalettePropertyString, new Boolean(createPalette).toString());
        properties.put(prefix + showGUIDataStoragePropertyString, new Boolean(showGUIDataStorage).toString());
        if (dataStorageIterator != null && dataStorageIterator.getDataStorage() != null) {
            // System.out.println("properties " +properties);
            // System.out.println("dataStorageIterator.getDataStorage() "+dataStorageIterator.getDataStorage());
            properties.put(prefix + dataStorageClassPropertyString, dataStorageIterator.getDataStorage().getClass().getCanonicalName());
            properties = dataStorageIterator.getProperties(prefix, properties);
        }


        if (map != null) {
            Projection projection = map.getProjection();
            Proj p = (Proj) projection;

            //keep projection and center

            properties.put(prefix + scalePropertyString, Double.toString(p.getScale()));

            properties.put(prefix + longitudePropertyString, Double.toString(((LatLonPoint.Double) (p.getCenter())).getLongitude()));
            properties.put(prefix + latitudePropertyString, Double.toString(((LatLonPoint.Double) (p.getCenter())).getLatitude()));

        }

        return properties;
    }

    /**
     * A DataDisplayLayer needs a DataStorage, of any type, to run.
     * When a layer is added, we need to know, *before* using setProperties(), which dataStorage should be created. 
     * We can't exactly know how many properties a DataStorage can have, so, the following property contains the minimum set 
     * of information to create the DataStorage later (as className, file, ...).
     * @see #getPropertyInfo(Properties list)
     */
    public final String DS_GLOBAL_CONFIGURATION_PROPERTY_STRING = "DSGlobalConfiguration";

    /**
     * Method from PropertyConsumer. This Layer is addable now. 
     * <p> Don't forget : to be effective, modify following rows in openmap.properties, by example :
     * <pre>
     * l9.class=fr.unicaen.openmap.layer.dataDisplayLayer.SimpleDataDisplayLayer
     * l9.prettyName=Data Display Layer
     * openmap.addableLayers=l9
     * </pre>
     * Essential step : provide a dataStorage and keep configuration !
     * @return list of properties including a data storage configuration
     */
    public Properties getPropertyInfo(Properties list) {

        list = super.getPropertyInfo(list);

        PropUtils.setI18NPropertyInfo(i18n, //I18N
                list, //properties
                SimpleDataDisplayLayer.class, //class to set for
                DS_GLOBAL_CONFIGURATION_PROPERTY_STRING, //property Name
                DS_GLOBAL_CONFIGURATION_PROPERTY_STRING, //label
                DS_GLOBAL_CONFIGURATION_PROPERTY_STRING, //tooltip
                "fr.unicaen.openmap.util.propertyEditor.DataStoragePropertyEditor"); //warning : java.beans.PropertyEditor only ! ; if null : a default editor is provided (by getProperties...)

        list.put(initPropertiesProperty, DS_GLOBAL_CONFIGURATION_PROPERTY_STRING);

        return list;
    }

    /**
     * Set the center of the projection and scale.
     **/
    //from InteractiveDataDisplayLayer
    protected void setCenterAndProjection() {
        BeanContext mapHandler = getBeanContext();
        if (mapHandler == null) {
            Debug.error(classNameForDebug + " Error : mapHandler is null");
            return;
        }

        Iterator it = mapHandler.iterator();
        Object someObj = null;
        MapBean map = null;
        MouseDelegator msd = null;

        while (it.hasNext()) {
            someObj = it.next();
            if (someObj instanceof MapBean) {
                map = (MapBean) someObj;
            } else if (someObj instanceof MouseDelegator) {
                msd = (MouseDelegator) someObj;
            }
        }
        Projection projection = map.getProjection();
        Proj p = (Proj) projection;
        p.setScale((float) scale);
        p.setCenter((float) latitude, (float) longitude);
        map.setProjection(p);
        repaint();
    }

    ;

    public void paint(Graphics g) {
        if (!initializedScale) {
            initializedScale = true;
            //setCenterAndProjection();
        }
        super.paint(g);
        // Debug.message(classNameForDebug, this +" : paint(Graphics g)");

    }

    /**
     * Sample function to render the layer as a string. You don't need
     * to override this method to match the name of your subclass since
     * we use java.lang.reflect to find the class name.
     */
    public String toStringDebug() {
        return "[" + classNameForDebug + ":" + getName() + "]";
    }

    public String toString() {
        return getName() ;
    }
    /**
     * Basic sample implementation. You should override this
     * function to match your layer interfaces need.
     */
    public Component getGUI() {
        return showGUIDataStorage ? dataStorageIterator.getGUI() : null;
    }

    protected void refreshGUI() {
    }

    protected void refreshDSGUI() {
    }

    /**
     * Used to redraw all elements.
     */
    public void refreshGraphics() {
        Debug.message(classNameForDebug, this + " refresh()");
        // use Swing Worker ...
        doPrepare();

    }

    /**
     * Sample example for the DataStorageListener interface.
     * This is a work in progress, it may change ...
     */
    public void dataStorageEvent(DataStorage source) {
        Debug.message(classNameForDebug, this + " caught a dataStorage event.");
        refreshGraphics();
    }

    /**
     * Create the OMGraphics using DataStorage datas
     *
     * @return OMGraphicList
     */
    public synchronized OMGraphicList prepare() {

        if (getProjection() == null) {
            return null;
        }

        OMGraphicList omgl = null;
        setList(null);
        omgl = generateGraphics();
        setVisible(true);
        setList(omgl);
        return omgl;
    }

    public void removeTopAdaptorDataStorage() {
	DataStorage ds = getDataStorageIterator().getDataStorage();
	if (! (ds instanceof RemovableDataStorage)) {
	    System.err.println("Trying to remove a non removable Data Storage");
	    return;
	}
	DataStorage dsNew = ((RemovableDataStorage) ds).removeItSelf();
	setDataStorageIterator(new DataStorageIteratorAdapter(dsNew));
	dataStorageEvent(dsNew);
    }

    /*AnimationDSListener*/
    public void animationAdded(MonoValuedFilterDataStorage ads) {
    }

    public void animationRemoved(MonoValuedFilterDataStorage ads) {
    }

    public void animationChanged(MonoValuedFilterDataStorage ads) {
    }

    /**
     * Purge all storage used by the layer.
     * Used in atlas mode
     */
    public void purge() {
	setList(null);
	setDataStorageIterator(null);
    }
}

