// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.dataDisplayLayer.gui;

import fr.unicaen.openmap.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;
import fr.unicaen.openmap.gui.ThemaMapColorChooser;
import fr.unicaen.openmap.omGraphics.DrawingAttributes;
import fr.unicaen.openmap.util.gui.ComponentSizer;

/**
 * This Panel can be used to display all data concerning a certain
 * type of graphics. See sample use in @see GeometryDataDisplayLayer
 */
public class GraphicsPanel extends JPanel implements ActionListener {

    protected JPanel panel;
    protected JCheckBox fillModeCB;
    protected DrawingAttributes drawingAttributes;
    protected JPanel samplePanel;
    /**
     * Use I18N mecanism to allow internationalization.
     * Use I18N.properties file or I18N_fr_FR.properties for french version
     **/
    protected I18n i18n = new BasicI18n();
    Box leftBox, rightBox;

    public Dimension getMaximumSize() {
        return new Dimension(getPreferredSize().width, super.getMaximumSize().height);
    }

    public GraphicsPanel(DrawingAttributes da) {
        this(da, true);
    }

    public GraphicsPanel(DrawingAttributes da, boolean displayFillModeCB) {
	this(da, displayFillModeCB, true);
    }

    /**
     * This ctor may choose the buttons or checkbox that are to be displayed in the panel
     */
    public GraphicsPanel(DrawingAttributes da, boolean displayFillModeCB, boolean displaySamplePanel) {
        super();
        setLayout(new BorderLayout());
        this.drawingAttributes = da;
        Color daFillPaint = (Color)da.getFillPaint();
        leftBox = Box.createVerticalBox();
        rightBox = Box.createVerticalBox();
        panel = new JPanel();
        if (displayFillModeCB) {
            fillModeCB = new JCheckBox(i18n.get(GraphicsPanel.class, "Filled", "Filled"));
            boolean fill = false;
            if (drawingAttributes.getFillPaint() instanceof Color)
                fill = ((Color) (drawingAttributes.getFillPaint())).getAlpha() > 0;
            fillModeCB.setSelected(fill);
            fillModeCB.addActionListener(this);
        }
        toggleControlsDisplay(da, displayFillModeCB, displaySamplePanel);
    }

    public void toggleControlsDisplay(DrawingAttributes da, boolean displayFillModeCB) {
	toggleControlsDisplay(da, displayFillModeCB, true);
    }

    /**
     * This method tells if the corresponding buttons or checkbox are to be displayed in this panel
     */
    public void toggleControlsDisplay(DrawingAttributes da, boolean displayFillModeCB, boolean displaySamplePanel) {
        panel.removeAll();
        removeAll();
        leftBox.removeAll();
        rightBox.removeAll();

        if (displayFillModeCB) {
            JPanel wrapFillCB = new JPanel();
            wrapFillCB.add(fillModeCB);
            leftBox.add(wrapFillCB);
        }

        panel.add(new JLabel(i18n.get(GraphicsPanel.class, "lines_colors", "Lines and colors")));
        panel.add(da.getGUI());
        leftBox.add(panel);
        ComponentSizer.fixComponentSize(rightBox, new Dimension(150, 100));
        ComponentSizer.fixComponentSize(leftBox, new Dimension(150, 100));
	if (displaySamplePanel) {
	    samplePanel = new RawSamplePanel(180);
	    rightBox.add(samplePanel);
	}
        add(leftBox, BorderLayout.WEST);
        add(rightBox, BorderLayout.EAST);

	da.setWithFillPaintMenu(displayFillModeCB && da.isFilled());
    }

    protected ColorButton addButton(String name, Color color) {

        Box box = Box.createHorizontalBox();
        ColorButton button = new ColorButton(color);
        button.setPreferredSize(new Dimension(20, 20));
        button.setMaximumSize(new Dimension(20, 20));
        button.addActionListener(this);
        box.setAlignmentX(Component.RIGHT_ALIGNMENT);
        box.add(new JLabel(name), "label");
        box.add(button, "field");

        panel.add(box);

        return button;
    }

    public void actionPerformed(ActionEvent event) {
        if (event.getSource() == fillModeCB) {
            drawingAttributes.setFilled(fillModeCB.isSelected());
	    needToRefresh = true;
	    updateSamplePanel();
        }
    }

    public void updateSamplePanel() {	   
	if (samplePanel != null)
	    samplePanel.repaint();
    }

    boolean needToRefresh = true;

    public boolean needToRefresh() {
        return needToRefresh;
    }

    public void setRefreshed() {
        needToRefresh = false;
    }

    class RawSamplePanel extends JPanel {

        int size;
	java.awt.Polygon sample;	

        public RawSamplePanel(int s) {
            size = s;
            ComponentSizer.fixComponentSize(this, new Dimension(s, s));
	    initSample();
	    addMouseListener(new java.awt.event.MouseAdapter() {
		    
		    public void mouseClicked(MouseEvent e) {
			if (sample.contains(e.getX(), e.getY()))
			    drawingAttributes.showPopupLineAndColorMenu();
		    }
		}); 
        }

	private void initSample() {
	    int[] xs = {20, 80, 110, 30, 10};
            int[] ys = {10, 6, 50, 80, 60};
	    sample = new java.awt.Polygon(xs, ys, xs.length);
	}
		
        @Override
        public void paintComponent(Graphics g) {
            Graphics2D g2 = (Graphics2D) g;
            g.setColor(rightBox.getBackground());
            g.clearRect(0, 0, size, size);
            g.fillRect(0, 0, size, size);
            g2.setStroke(drawingAttributes.getStroke());
            g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);
            g.setColor((Color) drawingAttributes.getFillPaint());
            g.fillPolygon(sample);
            g.setColor((Color) drawingAttributes.getLinePaint());
            g.drawPolygon(sample);
        }
    }
}
