// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * IsochroneDataObject.java
 *
 * Created on 17 juillet 2006, 19:46
 *
 */

package fr.unicaen.openmap.layer.isochroneLayer;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 *
 * @author christof
 */
public class IsochroneDataObject {
    
    private int autoroute = 0;
    private int nationale = 0;
    private int departementale = 0;
    private int liaison = 0;
    
    private float origineLat = 0f;
    private float origineLon = 0f;
    private int[] zones = null;
    
    /** Creates a new instance of IsochroneDataObject */
    public IsochroneDataObject() {}

    /**
     * Save informations in XML Format.
     **/
    public String savetoXML(){
        StringBuffer buffer = new StringBuffer();
        buffer.append("<isochroneDataObject>");
        buffer.append("<autoroute>"+autoroute+"</autoroute>");
        buffer.append("<nationale>"+nationale+"</nationale>");
        buffer.append("<departementale>"+departementale+"</departementale>");
        buffer.append("<liaison>"+liaison+"</liaison>");
        buffer.append("<originelat>"+origineLat+"</originelat>");
        buffer.append("<originelon>"+origineLon+"</originelon>");
        buffer.append("<zones>");
        if (zones!=null) for(int i=0;i<zones.length;i++) buffer.append("<zone>"+zones[i]+"</zone>");
        buffer.append("</zones>");        
        buffer.append("</isochroneDataObject>");
        return buffer.toString();
        
    } 
    
    /**
     * Parse informations in XMl format and store them.
     **/
    public void loadFromXML(Element isochrone){
        NodeList autoroute = isochrone.getElementsByTagName("autoroute");
        if (autoroute.getLength()>0) setAutorouteSpeedCoef(Integer.parseInt(autoroute.item(0).getFirstChild().getNodeValue()));
        NodeList nationale = isochrone.getElementsByTagName("nationale");
        if (nationale.getLength()>0) setNationaleSpeedCoef(Integer.parseInt(nationale.item(0).getFirstChild().getNodeValue()));
        NodeList departementale = isochrone.getElementsByTagName("departementale");
        if (departementale.getLength()>0) setDepartementaleSpeedCoef(Integer.parseInt(departementale.item(0).getFirstChild().getNodeValue()));
        NodeList liaison = isochrone.getElementsByTagName("liaison");
        if (liaison.getLength()>0) setLiaisonSpeedCoef(Integer.parseInt(liaison.item(0).getFirstChild().getNodeValue()));
        NodeList originelat = isochrone.getElementsByTagName("originelat");
        NodeList originelon = isochrone.getElementsByTagName("originelon");
        if (originelat.getLength()>0 && originelon.getLength()>0) setOrigin((float)Double.parseDouble(originelat.item(0).getFirstChild().getNodeValue()),(float)Double.parseDouble(originelon.item(0).getFirstChild().getNodeValue()));
        
        NodeList zonesNodeList = isochrone.getElementsByTagName("zones");
        Element leszones = null;
        // on traite notre zone de données
        if (zonesNodeList.getLength()>0) leszones = (Element) zonesNodeList.item(0);
        
        if (leszones!=null) {
            NodeList leschoix = leszones.getElementsByTagName("zone");
            zones = new int[leschoix.getLength()];
            for (int i=0;i<leschoix.getLength();i++){
               zones[i]=Integer.parseInt(leschoix.item(i).getFirstChild().getNodeValue()); 
            }
        }
    }
    
    public void setAutorouteSpeedCoef(int speed){
        autoroute=speed;
    }

    public int getAutorouteSpeedCoef(){
        return autoroute;
    }

    public void setNationaleSpeedCoef(int speed){
        nationale=speed;
    }

    public int getNationaleSpeedCoef(){
        return nationale;
    }
 
    public void setDepartementaleSpeedCoef(int speed){
        departementale=speed;
    }

    public int getDepartementaleSpeedCoef(){
        return departementale;
    }
    
    public void setLiaisonSpeedCoef(int speed){
        liaison=speed;
    }

    public int getLiaisonSpeedCoef(){
        return liaison;
    }
    
    public void setOrigin(float latitude, float longitude){
        origineLat=latitude;
        origineLon=longitude;
    };
    
    public float getOrigineLatitude() { return origineLat; }
    
    public float getOrigineLongitude() {return origineLon; }  

    public void setZones(int[] zones){
       this.zones=zones; 
    }
    
    public int[] getZones(){ return zones;}
    
}
