// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.isochroneLayer;

import java.util.Iterator;


import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.proj.*;
import com.bbn.openmap.MoreMath;


public class IsochroneUtil {

    // The following code copied from DistanceMouseMode has been of great help to write this class 
    //
    //    dist = distRad * Planet.wgs84_earthEquatorialCircumferenceKM / MoreMath.TWO_PI;
    //    distance = (double)GreatCircle.spherical_distance(ProjMath.degToRad(lat1),
    //						      ProjMath.degToRad(long1),
    //						      ProjMath.degToRad(lat2),
    //						      ProjMath.degToRad(long2));

    /**
     * Compute the total length in km of the OMLines composing the OMGraphicList
     */
    public static float getLenthInKM(OMGraphicList omgl) {
	return getLengthInRad(omgl) * Planet.wgs84_earthEquatorialCircumferenceKM / MoreMath.TWO_PI;
    }

    /**
     * Compute the total length in radians of the OMLines composing the OMGraphicList
     */
    public static float getLengthInRad(OMGraphicList omgl) {
	float totalLength = 0.0f;
        for (int i=0;i<omgl.size();i++){
	    float length=0f;
	    try {
                OMGraphic omg = omgl.getOMGraphicAt(i);
                if (omg instanceof OMLine) length = getLengthInRad((OMLine)omg);
                if (omg instanceof OMPoly) length = getLengthInRad((OMPoly)omg);
	    } catch (ClassCastException e) {
		length = 0.0f;
		System.err.println("[IsochroneLayer] ### getLengthInRad: cast exception");
	    }
	    totalLength += length;
	}
	return totalLength;
    }

    /**
     * Compute the total length in radians of the OMLine
     */
    public static float getLengthInRad(OMLine oml) {
	double[] coords = oml.getLL();
	return (float)GreatCircle.sphericalDistance(ProjMath.degToRad(coords[0]),
						     ProjMath.degToRad(coords[1]),
						     ProjMath.degToRad(coords[2]),
						     ProjMath.degToRad(coords[3]));
    }  
    
    /**
     * Compute the total length in radians of the OMLine
     */
    public static float getLengthInRad(OMPoly oml) {
	double[] coords = oml.getRawllpts(); // already in Radians
	//        System.out.println(" getLengthInRad ompoly : "+coords.length);
        if (coords.length<4) return 0f;
        return (float)GreatCircle.sphericalDistance(coords[0],
						     coords[1],
						     coords[coords.length-2],
						     coords[coords.length-1]);
    }
}
