// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.legendLayer;

import com.bbn.openmap.Layer;
import com.bbn.openmap.LayerHandler;
import com.bbn.openmap.MapBean;
import com.bbn.openmap.PropertyConsumer;
import com.bbn.openmap.event.LayerEvent;
import com.bbn.openmap.event.MapMouseListener;
import com.bbn.openmap.event.ProjectionEvent;
import com.bbn.openmap.layer.OMGraphicHandlerLayer;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMText;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.animation.MetaAnimator;
import fr.unicaen.openmap.event.LayerListener;
import fr.unicaen.openmap.layer.legendLayer.gui.LegendLayerMouseListener;
import fr.unicaen.openmap.layer.legendLayer.gui.OMLegendFrame;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.MouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.LegendGUIManager;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMControlPoint;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.Refreshable;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableCompass;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableGraphicList;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableText;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableFrame;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableGraphic;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableMultiLineText;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableScale;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Offseter;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.OffseterListener;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.legendable.LegendableLayer;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.beancontext.BeanContext;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Properties;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;

/**
 *
 * @author jchauvea
 */
public class LegendLayer extends OMGraphicHandlerLayer
    implements LayerListener, Refreshable, PropertyConsumer, PropertyChangeListener, OffseterListener {

    protected ArrayList<LegendableLayer> legendableLayers = null;
    protected ArrayList<OMResizeableGraphicList> legends = new ArrayList<OMResizeableGraphicList>();
    protected String mapTitle = "";
    protected String legendTitle = "";
    protected OMGraphicList cachedOMGraphicList = null;
    private JComponent legendPanel;
    private JPanel updatablePanel;
    private JScrollPane scrollPanel;
    private LegendLayerMouseListener mouseControlPointListener;
    private boolean badCache = true;
    private LayerHandler layerHandler;
    private boolean explode;
    private OMResizeableScale omScale;
    private OMResizeableCompass omCompass;
    public final static int GAP = 15;
    private Properties props;
    private boolean dragging;
    private boolean controlPointsVisible;
    protected int xAnchor = 5;
    protected int yAnchor = 5;
    protected int width = 100;
    protected int height = 100;
    private String initMapTitle = "";
    public static final String ADDITIONAL_TEXT_PROPERTY_NAME = "additionalText";
    public static final String ADDITIONAL_COUNT_PROPERTY_NAME = "additionalTextsNumber";
    public final static String SCALE_PROPERTY_NAME = "scale";
    public final static String COMPASS_PROPERTY_NAME = "compass";
    public final static String DEFAULT_TEXT_PROPERTY_NAME = "defaultText";
    public final static String FRAME_PROPERTY_NAME = "legend.frame";
    public final static String MAP_FRAME_PROPERTY_NAME = "legendmap.frame";
    public static final String XAnchorPropertyString = "xAnchor";
    public static final String YAnchorPropertyString = "yAnchor";
    public static final String WidthPropertyString = "width";
    public static final String HeightPropertyString = "height";
    public final static String MapTitlePropertyString = "mapTitle";
    public static final String LegendTitlePropertyString = "legendTitle";
    public static final String AuthorPropertyString = "author";
    public static final String DatePropertyString = "date";
    public static final String CopyrightPropertyString = "copyright";
    public final static String ExplodePropertyString = "explode";
    public final static String AnimationFramePropertyString = "animation";
    public final static String AnimationTextsCountPropertyString = "animationTextsCount";
    public final static String AnimationTextPropertyString = "animationText";
    public static final int compassRadius = 20;
    protected OMResizeableGraphicList additionalTexts;
    protected LegendGUIManager legendGUIManager;
    private ArrayList<OMResizeableGraphic> interactiveGraphics;

    public LegendLayer() {
        //super();
        mouseControlPointListener = new LegendLayerMouseListener(this);
        setMouseModeIDsForEvents(new String[]{"Gestures"});
        setAddToBeanContext(true);
        legendGUIManager = new LegendGUIManager();
        interactiveGraphics = new ArrayList<OMResizeableGraphic>();
    }

    @Override
	public void projectionChanged(ProjectionEvent e) {
        // extract the projection and check to see if it's really different.
        // if it isn't then we don't need to do all the work again, just repaint.
        badCache = true;
        Projection proj = setProjection(e);
        if (proj == null) {
            repaint();
            return;
        }
        setList(null);
        refreshGraphicsAndGUI();
    }
    private OMResizeableText omMapTitle, omLegendTitle;
    private OMResizeableFrame frame;
    private OMLegendFrame mapFrame;

    private OMResizeableFrame animationFrame = null;

    public void layerUpdate() {
        badCache = true;
        /* // this must be done in the LegendLayer thread and not in the thread of the changed layer !!!
	   prepare();
	   repaint();
	*/
    }

    public void setLayers(LayerEvent evt) {
        //throw new UnsupportedOperationException("Not supported yet.");
    }

    /** Returns the width of the application window through the MapBean. */
    public int getAppWindowWidth() {
        MapBean mb = getMapBean();
        return mb == null ? -1 : mb.getWidth();
    }

    /** Returns the heightr of the application window through the MapBean. */
    public int getAppWindowHeight() {
        MapBean mb = getMapBean();
        return mb == null ? -1 : mb.getHeight();
    }

    private MapBean _mapBean = null;
    /*
     * Get the MapBean through the BeanContext. Set the layer property
     * "addToBeanContext" to true in order to be able to get it.
     */
    public MapBean getMapBean() {
	if (_mapBean != null)
	    return _mapBean;

        BeanContext mapHandler = getBeanContext();
        if (mapHandler == null) {
            return null;
        }

        Iterator it = mapHandler.iterator();
        while (it.hasNext()) {
            Object o = it.next();
            if (o instanceof MapBean) {
		_mapBean = (MapBean) o;
                return _mapBean;
            }
        }
        return null;
    }

    @Override
	public JComponent getGUI() {
        if (scrollPanel == null) {
            createGUI();
            updateGUI();
        }
        return scrollPanel;
    }
    private Collection<GraphicForGUI> graphicsForGUI;

    private void updateGUI() {
        // must use the same OMResizeableGraphics that are currently displayed
        // don't ask the layer to rebuild new ones!
        if (legendPanel == null) {
            return;
        }
        Debug.message("Legend", "[LegendLayer] - updateGUI <=");
        updateLegendableLayers();
        if (legendableLayers == null) {
            return;
        }
        updatablePanel.removeAll();

        if (omMapTitle == null) {
            return;
        }

        // ask each legend piece for its GUI
        // BUG: the legend piece should know their visu names !!!!
        // Beware: don't ask the layers to give back a legend piece,
        //         because it can be different and we will loose the synchro with the GUI.
        updatablePanel.add(buildLegendGUI());
        legendPanel.updateUI();
        updatablePanel.updateUI();
        Debug.message("Legend", "[LegendLayer] - updateGUI =>");
    }

    private void createGUI() {
        if (legendPanel != null) {
            return;
        }
	legendPanel = Box.createVerticalBox();
        legendPanel.setAlignmentY(Component.TOP_ALIGNMENT);
        

        Box topPanel = Box.createHorizontalBox();
        //topPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
        scrollPanel = new JScrollPane(legendPanel);
        updatablePanel = new JPanel();
        updatablePanel.setLayout(new BoxLayout(updatablePanel, BoxLayout.X_AXIS));
        JButton bRefresh = new JButton(i18n.get(this, "refresh", "Refresh Legend"), Util.loadIcon("testRefresh.png"));
        //bRefresh.setBackground(Color.WHITE);
        bRefresh.setOpaque(false);
        bRefresh.addActionListener(new ActionListener() {

		public void actionPerformed(ActionEvent arg0) {
		    badCache = true;
		    LegendLayer.this.refreshGraphicsAndGUI();
		}
	    });
        legendPanel.setBorder(BorderFactory.createLineBorder(Color.GRAY));
        JCheckBox cbMovable = new JCheckBox(i18n.get(LegendLayer.class, "movable", "Movable Legend Elements"));
        cbMovable.addActionListener(new ActionListener() {

		public void actionPerformed(ActionEvent evt) {
		    controlPointsVisible = ((JCheckBox) evt.getSource()).isSelected();

		    mouseControlPointListener.setOMControlPointsVisible(controlPointsVisible);
		    refreshGraphicsAndGUI();
		}
	    });



        topPanel.add(cbMovable);
        topPanel.add(bRefresh);
        //Box wrapTopPanel= Box.createHorizontalBox();
        //JPanel wrapTopPanel = new JPanel();
        //wrapTopPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
        //wrapTopPanel.
        //wrapTopPanel.add(topPanel);
        //wrapTopPanel.add(new JPanel());
        Box bWrap = Box.createHorizontalBox();
        bWrap.add(topPanel);
        bWrap.add(Box.createHorizontalGlue());
        //bWrap.setBackground(Color.WHITE);
        ComponentSizer.fixComponentSize(topPanel, new Dimension(500,50));
        legendPanel.add(bWrap/*, BorderLayout.NORTH*/);
        JPanel wrapUpdatables = new JPanel();
        wrapUpdatables.setLayout(new BorderLayout());
        wrapUpdatables.add(updatablePanel, BorderLayout.WEST);
        legendPanel.add(wrapUpdatables/*, BorderLayout.CENTER*/);
        
    }

    private JComponent buildLegendGUI() {
        Box gui = new TopBox();
        // Map & legend titles
        JComponent mapTitleGUI = omMapTitle.getGUI();
        JComponent legendTitleGUI = omLegendTitle.getGUI();
        String name = i18n.get(this, "titles", "Map & Legend Titles");
        ThematicConfPanel tcpTitle = ThematicConfPanel.createThematicConfPanel(name);
        tcpTitle.addCollapseListener(legendGUIManager);
        if (legendGUIManager.isOpened(name)) {
            tcpTitle.expand();
        } else {
            tcpTitle.collapse();
        }
        Box titlesBox = Box.createVerticalBox();
        JPanel wrapTitles = new JPanel(new BorderLayout());
        titlesBox.add(mapTitleGUI);
        titlesBox.add(legendTitleGUI);
        wrapTitles.add(frame.getToolGUI(), BorderLayout.NORTH);
        wrapTitles.add(titlesBox, BorderLayout.CENTER);
        tcpTitle.setContent(wrapTitles);
        gui.add(tcpTitle);

        /* Compass & scale */
        name = i18n.get(this, "layout", "Layout");
        ThematicConfPanel tcpCompass = ThematicConfPanel.createThematicConfPanel(name, false);
        tcpCompass.addCollapseListener(legendGUIManager);
        //System.out.println(n + " opened?" + legendGUIManager.isOpened(n));
        if (legendGUIManager.isOpened(name)) {
            tcpCompass.expand();
        } else {
            tcpCompass.collapse();
        }
        Box compassAndScaleBox = Box.createVerticalBox();
        compassAndScaleBox.add(omCompass.getGUI());
        compassAndScaleBox.add(omScale.getGUI());
        compassAndScaleBox.add(mapFrame.getGUI());
        tcpCompass.setContent(compassAndScaleBox);
        gui.add(tcpCompass);

	/* Thematic legends */
        for (OMResizeableGraphic g : legends) {
            gui.add(g.getGUI());
        }

	/* Animation */	
	if (animationFrame != null && animationFrame.getGUI() != null) {
	    name = i18n.get(this, "animation", "Animation");
	    ThematicConfPanel tcpAnimation = ThematicConfPanel.createThematicConfPanel(name, false);
	    tcpAnimation.addCollapseListener(legendGUIManager);
	    if (legendGUIManager.isOpened(name)) {
		tcpAnimation.expand();
	    } else {
		tcpAnimation.collapse();
	    }
	    Box animationBox = Box.createVerticalBox();
	    animationBox.add(animationFrame.getGUI());
	    tcpAnimation.setContent(animationBox);
	    gui.add(tcpAnimation);
	}
        /* Additionnal texts */
        JButton additionalTextsB = new JButton(i18n.get(LegendLayer.class, "addTextButton", "Add New Text"));
        JPanel wrapButton = new JPanel();
        name = i18n.get(LegendLayer.class, "addText", "Additionnal Texts");
        ThematicConfPanel tcpAddText = ThematicConfPanel.createThematicConfPanel(name, false);
        tcpAddText.addCollapseListener(legendGUIManager);
        //System.out.println(n + " opened?" + legendGUIManager.isOpened(n));
        if (legendGUIManager.isOpened(name)) {
            tcpAddText.expand();
        } else {
            tcpAddText.collapse();
        }
        Box textsBox = Box.createVerticalBox();
        tcpAddText.setContent(textsBox);
        wrapButton.add(additionalTextsB);
        textsBox.add(wrapButton);
        additionalTextsB.addActionListener(new ActionListener() {

		public void actionPerformed(ActionEvent event) {
		    addAdditionalText(" XXXX ");
		    badCache = true;
		    refreshGraphicsAndGUI();
		}
	    });
        if (additionalTexts != null) {
            for (OMResizeableGraphic t : additionalTexts.getResizeableGraphics()) {
                textsBox.add(t.getGUI());
            }
        }
        gui.add(tcpAddText);
        return gui;
    }

    protected OMResizeableText addAdditionalText(String text) {
        OMResizeableText omText =
	    new OMResizeableMultiLineText(this, getMouseControlPointsListener(),
				 getAppWindowWidth() / 3, 3 * GAP,
				 text, OMText.JUSTIFY_CENTER,
				 i18n.get(LegendLayer.class,
					  ADDITIONAL_TEXT_PROPERTY_NAME, "Additional text") + " " +
				 (additionalTexts.size() + 1));
        getMouseControlPointsListener().setOMControlPointsVisible(controlPointsVisible);

        additionalTexts.add(omText);

        return omText;
    }

    public MouseControlPointsListener getMouseControlPointsListener() {
        return mouseControlPointListener;
    }

    public synchronized MapMouseListener getMapMouseListener() {
        return mouseControlPointListener;
    }

    public void refreshGraphicsAndGUI() {
        setList(null);
        badCache = true;
        doPrepare();
    }

    public void refreshInteractiveGraphics() {
	// Debug.message("Legend", "[LegendLayer] - refreshInteractiveGraphics <=");
        repaint();
	// Debug.message("Legend", "[LegendLayer] - refreshInteractiveGraphics =>");
    }
    protected OMResizeableGraphicList legend = null;

    public synchronized OMGraphicList generateGraphics() {

        if (badCache && legendableLayers != null) {
            Debug.message("Legend", "[LegendLayer] - GenerateGraphics <=");
            String prefix = PropUtils.getScopedPropertyPrefix(this);

            if (mouseControlPointListener.getMapBean() == null) {
                mouseControlPointListener.setMapBean(getMapBean());
            }
            updateLegendableLayers();
            if (legend != null) {
                saveProperties();
            }

            mouseControlPointListener.removeAllOMControlPoints();
            legend = new OMResizeableGraphicList(this, mouseControlPointListener, "Legend");

            int x, y;

            /* Title */
            omMapTitle =
		new OMResizeableText(this, mouseControlPointListener, getAppWindowWidth() / 2, 2 * GAP,
				     mapTitle, OMText.JUSTIFY_CENTER,
				     i18n.get(LegendLayer.class, MapTitlePropertyString, MapTitlePropertyString));
            omMapTitle.setFontStyle(java.awt.Font.BOLD);
            omMapTitle.setFontSize(14.0F);

            legend.add(omMapTitle);

            /*Compass and scale*/
            x = compassRadius;
            y = getAppWindowHeight() - 5 * compassRadius / 3;

            omCompass = new OMResizeableCompass(this, mouseControlPointListener,
						x + compassRadius / 3, y + compassRadius / 3, compassRadius);
            legend.add(omCompass);

            // place the scale just at the right of the compass
            x += compassRadius * 2;
            //y += 5 * compassRadius / 3 - 4 * omScale.getHeight();
            omScale = new OMResizeableScale(this, mouseControlPointListener, x, y,
					    getProjection()/*, scaleWidth, scaleHeight*/);
            legend.add(omScale);

            /* The frame */
            x = xAnchor;
            y = yAnchor;
            frame = new OMResizeableFrame(this, mouseControlPointListener, xAnchor, yAnchor, x, y, "Legend", i18n.get(LegendLayer.class, "cartouche", "Legend Cartouche"), new FlowLayout());
            legends.clear();
            x += GAP;
            y += GAP;

            mapFrame = new OMLegendFrame(this);
            getMapBean().addPropertyChangeListener(this);//listen map resizing
            omLegendTitle = new OMResizeableText(this, mouseControlPointListener, x, y,
						 legendTitle, OMText.JUSTIFY_CENTER,
						 i18n.get(LegendLayer.class, LegendTitlePropertyString, LegendTitlePropertyString));
            omLegendTitle.setFontStyle(java.awt.Font.BOLD);
            width = omLegendTitle.getWidth();

            frame.add(omLegendTitle);

            y += omLegendTitle.getHeight() + GAP;

            /*Properties*/
            if (props != null) {
                // System.out.println("LegendLayer.generateGraphics "+ props);
                // frame.removeWidthHeightFromProperties(prefix + FRAME_PROPERTY_NAME, props);
                frame.setProperties(prefix + FRAME_PROPERTY_NAME, props);
                //System.out.println("set omMapTitle " + prefix + MapTitlePropertyString);
                omMapTitle.setProperties(prefix + MapTitlePropertyString, props);
                omScale.setProperties(prefix + SCALE_PROPERTY_NAME, props);
                omCompass.setProperties(prefix + COMPASS_PROPERTY_NAME, props);
                mapFrame.setProperties(prefix + MAP_FRAME_PROPERTY_NAME, props);
                //System.out.println("OMCOMPASSSSSSSSSSSSSSSSSSSSSs " + omCompass.getVisible());
                ////mapFrame.setProperties(prefix + MAP_FRAME_PROPERTY_NAME, props);
                // don't set legend title now
            }
	    
	    /* Animation */
	    if (getMetaAnimator() != null) {
		ArrayList<String> variablesValues = getMetaAnimator().getVariablesValuesAsArrayOfString();
		if (variablesValues != null && variablesValues.size() > 0) {
		    animationFrame = new OMResizeableFrame(this, mouseControlPointListener, x, y, 0, 0, "", i18n.get(LegendLayer.class, "animation", "Animation cartouche"), new FlowLayout());
		
		    y += 2 * GAP;
		
		    Debug.message("Legend", "[LegendLayer] Anim: "+getMetaAnimator().getVariablesValuesAsString());
		    int i = 0, w = 0, h = 0;
		    for (String varVal : variablesValues) {
			i++;
			OMResizeableText text =  new OMResizeableText(this, mouseControlPointListener, x, y-GAP,
								      varVal, OMText.JUSTIFY_CENTER,
								      i18n.get(LegendLayer.class, "animationVarValues", "Var/Value ")+i);
			animationFrame.add(text);

			w = text.getWidth();
			h = text.getHeight();
			width = Math.max(width, w);
			text.translate(w/2, 0); // do it after ?
			y += h + GAP;
		    }

		    animationFrame.extend(width, (h+GAP)*i+GAP);

		    for(OMResizeableGraphic omrg : animationFrame.getResizeableGraphics()) {
			omrg.addOMControlPointsToMouseListener();
		    }
		    // Beware: setProperties erase the modification of the texts, so we NEED to reset its content
		    if (props != null) {
			animationFrame.setProperties(prefix + AnimationFramePropertyString, props);
			int size = PropUtils.intFromProperties(props, prefix + AnimationTextsCountPropertyString, 0);
			for (i = 0; i < size; i++) {
			    // don't set the elements as it is done in OMResizeableGraphicList.setProperties()
			    // String otherPrefix = prefix + AnimationTextPropertyString + String.valueOf(i);
			    // animationFrame.getResizeableGraphics().get(i).setProperties(otherPrefix, props);
			    
			    // re-init the text:
			    ((OMResizeableText)animationFrame.getResizeableGraphics().get(i)).setText(variablesValues.get(i));
			}
		    }
		    frame.add(animationFrame);
		}
	    }

	    int colWidth = width; // current column width	    
	    width = 0;

            /* Other legends (thematics) */
            for (LegendableLayer legendableLayer : legendableLayers) {
                OMResizeableGraphicList l = legendableLayer.getLegend(this);
                if (l.getResizeableGraphics().size() <= 0) {
                    Debug.message("Legend", "[LegendLayer] ## no legend for " +
				  ((Layer) legendableLayer).getName());
                    continue;
                }
                legends.add(l);
                for (OMResizeableGraphic omrg : l.getResizeableGraphics()) {
                    int w = omrg.getWidth();
                    int h = omrg.getHeight();
                    colWidth = Math.max(colWidth, w);
                    if (explode) {
                        legend.add(omrg);
                    } else {
                        frame.add(omrg);
                    }
                    Debug.message("Legend2", "[LegendLayer] Translate " + omrg + " of " + legendableLayer.getName() + " to x=" + x + " y=" + y);
                    omrg.translate(x, y);
                    y += h;
		    if (y >= (getAppWindowHeight() - 2 * GAP)) {
			height = Math.max(height,  y - yAnchor);
			y = omLegendTitle.getHeight() + GAP;
			x += colWidth + 2 * GAP;
			width += colWidth + 2 * GAP;
			colWidth = 0;
		    }
		    y += GAP;
                }
                if (props != null) {
                    // Debug.message("Legend2", "[LegendLayer] --> setLegendProperties " + legendableLayer.getName()+"\n"+props);
                    legendableLayer.setLegendProperties(prefix, props);
                    // Debug.message("Legend2", "[LegendLayer] <-- setLegendProperties " + legendableLayer.getName());
                }
            }

	    width += colWidth + 2 * GAP;
            height =  Math.max(height,  y - yAnchor);
            // System.out.println("Frame.getWidth = " + frame.getWidth() +  " "+ frame);

            if (!explode && frame.getWidth() <= 5 && frame.getHeight() <= 5) {
                frame.extend(width, height);
                omLegendTitle.translate(-GAP + frame.getWidth() / 2, 0);
            }

            if (props != null) {
                omLegendTitle.setProperties(prefix + LegendTitlePropertyString, props);
            }

            Debug.message("Legend2", "[LegendLayer] " + frame + " extends: w=" + width + ", h=" + height);
            // map frame
            legend.add(frame);

            /* Additional texts */
            if (additionalTexts == null) {
                additionalTexts =
		    new OMResizeableGraphicList(this, mouseControlPointListener, "Additional texts");
            }
            legend.add(0, additionalTexts);
            // add the additional texts control points to the listener as they have all been removed
            for (OMResizeableGraphic omrg : additionalTexts.getResizeableGraphics()) {
                omrg.addOMControlPointsToMouseListener();
            }
            if (props != null) {
                int size = PropUtils.intFromProperties(props, prefix + ADDITIONAL_COUNT_PROPERTY_NAME, 0);
                for (int i = 0; i < size; i++) {
                    String otherPrefix = prefix + ADDITIONAL_TEXT_PROPERTY_NAME + String.valueOf(i);
                    additionalTexts.getResizeableGraphics().get(i).setProperties(otherPrefix, props);
                }
            }

            cachedOMGraphicList = legend.getGraphics();

            mouseControlPointListener.setOMControlPointsVisible(controlPointsVisible);

            cachedOMGraphicList.add(mapFrame);
            cachedOMGraphicList.generate(getProjection());

            badCache = false;
            SwingUtilities.invokeLater(new Runnable() {

		    public void run() {
			updateGUI();
		    }
		});
        }
        Debug.message("Legend", "[LegendLayer] - GenerateGraphics =>");

        return cachedOMGraphicList;
    }

    public void addInteractiveGraphics(OMResizeableGraphic g) {
        interactiveGraphics.add(g);
    }

    public void clearInteractiveGraphics() {
        interactiveGraphics.clear();
    }

    private OMGraphicList getOMControlPoints(OMGraphicList omgl) {
        OMGraphicList list = new OMGraphicList();
        Debug.message("Legend2", "List(omgl) size =" + omgl.size());
        for (int i = 0; i < omgl.size(); i++) {
            if (omgl.getOMGraphicAt(i) instanceof OMControlPoint) {
                list.add((OMControlPoint) (omgl.getOMGraphicAt(i)));
            }
        }

        Debug.message("Legend2", "List(list) size =" + list.size());
        return list;
    }

    private void saveProperties() {
        props = getProperties(getPropertyPrefix(), new Properties());
    }

    private void updateLegendableLayers() {
        if (layerHandler != null) {
            Debug.message("Legend", "[LegendLayer] -  update legendable layers  " + layerHandler.getLayers().length);
            if (legendableLayers != null) {
                legendableLayers.clear();
            } else {
                legendableLayers = new ArrayList<LegendableLayer>();
            }
            for (Layer layer : layerHandler.getLayers()) {
                if (layer.isVisible() && layer instanceof LegendableLayer) {
                    legendableLayers.add((LegendableLayer) layer);
                }

            }
        }
    }

    @Override
	public synchronized OMGraphicList prepare() {

	    Debug.message("Legend", "[LegendLayer] - prepare <=");
	    if (getProjection() == null || layerHandler == null) {
		return null;
	    }

	    OMGraphicList omgl = null;
	    setList(null);
	    omgl = generateGraphics();
	    setVisible(true);
	    setList(omgl);
	    Debug.message("Legend", "[LegendLayer] - prepare =>");
	    return omgl;
	}

    /**
     * Implementing the ProjectionPainter interface.
     */
    public synchronized void renderDataForProjection(Projection proj,
						     java.awt.Graphics g) {
        //System.out.println("RENDER DATA");
        if (getProjection() == null) {
            return;
        } else if (!proj.equals(getProjection())) {
            setProjection(proj.makeClone());
            setList(generateGraphics());
        }

        if (cachedOMGraphicList != null) {
            cachedOMGraphicList.generate(proj);
            cachedOMGraphicList.render(g);
        }

    }

    public void paint(java.awt.Graphics g) {
        Projection projection = getProjection();
        if (projection == null) {
            return;
        }
        super.paint(g);
        if (interactiveGraphics.size() > 0) {
            OMGraphicList omgl = new OMGraphicList();
            for (OMResizeableGraphic omrg : interactiveGraphics) {
                omgl.add(omrg.getGraphics());
            }
            omgl.setVisible(true);
            omgl.generate(projection);
            omgl.render(g);
        }
    }

    @Override
	public Properties getPropertyInfo(
					  Properties info) {
        info = super.getPropertyInfo(info);
        PropUtils.setI18NPropertyInfo(i18n, info, LegendLayer.class,
				      "mapTitle",
				      i18n.get(LegendLayer.class, "mapTitle", "Map Title"),
				      i18n.get(LegendLayer.class, "mapTitle", "Map Title"),
				      null);
        PropUtils.setI18NPropertyInfo(i18n,
				      info,
				      LegendLayer.class,
				      "legendTitle",
				      i18n.get(LegendLayer.class, "legendTitle", "Legend Title"),
				      i18n.get(LegendLayer.class, "legendTitle", "Legend Title"),
				      null);

        PropUtils.setI18NPropertyInfo(i18n,
				      info,
				      LegendLayer.class,
				      "xAnchor",
				      i18n.get(LegendLayer.class, "xAnchor", "X anchor"),
				      i18n.get(LegendLayer.class, "xAnchor", "X anchor"),
				      null);
        PropUtils.setI18NPropertyInfo(i18n,
				      info,
				      LegendLayer.class,
				      "yAnchor",
				      i18n.get(LegendLayer.class, "yAnchor", "Y anchor"),
				      i18n.get(LegendLayer.class, "yAnchor", "Y anchor"),
				      null);

        info.put(initPropertiesProperty,
		 MapTitlePropertyString + " " +
		 LegendTitlePropertyString + " " +
		 XAnchorPropertyString + " " +
		 YAnchorPropertyString);

        return info;
    }

    @Override
	public void findAndInit(Object someObj) {
        if (someObj instanceof LayerHandler) {
            // System.out.println("LAYER HANDLER =============================****");
            layerHandler = (LayerHandler) someObj;
            updateLegendableLayers();
            prepare();
        }
    }

    public String getMapTitle() {
        if (omMapTitle != null) {
            return omMapTitle.getText();
        }
        return mapTitle;
    }

    public String getLegendTitle() {
        if (omLegendTitle != null) {
            return omLegendTitle.getText();
        }
        return legendTitle;
    }

    /******************PROPERTIES*************************/
    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        setMouseModeIDsForEvents(new String[]{"Gestures"});
        setConsumeEvents(true);

        prefix = PropUtils.getScopedPropertyPrefix(prefix);

        mapTitle = properties.getProperty(prefix + MapTitlePropertyString, getMapTitle());
        legendTitle = properties.getProperty(prefix + LegendTitlePropertyString, getLegendTitle());
        xAnchor = Integer.parseInt(properties.getProperty(prefix + XAnchorPropertyString, "5"));
        yAnchor = Integer.parseInt(properties.getProperty(prefix + YAnchorPropertyString, "5"));
        width = Integer.parseInt(properties.getProperty(prefix + WidthPropertyString, "100"));
        height = Integer.parseInt(properties.getProperty(prefix + HeightPropertyString, "100"));
        explode = Boolean.parseBoolean(properties.getProperty(prefix + ExplodePropertyString, "false"));

        int size = PropUtils.intFromProperties(properties, prefix + ADDITIONAL_COUNT_PROPERTY_NAME, 0);
        if (size > 0) {
            additionalTexts = new OMResizeableGraphicList(this, mouseControlPointListener, "Additional texts");
        }
        for (int i = 0; i < size; i++) {
            String otherPrefix = prefix + ADDITIONAL_TEXT_PROPERTY_NAME + String.valueOf(i);
            OMResizeableText t = addAdditionalText("xxx " + i);
            if (t != null) {
                t.setProperties(otherPrefix, properties);
            }
        }
	
	size = PropUtils.intFromProperties(properties, prefix + AnimationTextsCountPropertyString, 0);
        if (size > 0) {
            animationFrame = new OMResizeableFrame(this, mouseControlPointListener, 0, 0, 0, 0, "", i18n.get(LegendLayer.class, "animation", "Animation cartouche"), new FlowLayout());
        }
        for (int i = 0; i < size; i++) {
            String otherPrefix = prefix + AnimationTextPropertyString + String.valueOf(i);
            OMResizeableText t = new OMResizeableText(this, mouseControlPointListener, 0, 0, 
						      "", OMText.JUSTIFY_CENTER,
						      AnimationTextPropertyString + String.valueOf(i));
	    animationFrame.add(t);
            if (t != null) {
                t.setProperties(otherPrefix, properties);
            }
        }
	

        // System.out.println("LegendLayer.generateGraphics "+ props);
        if (frame != null) {
            frame.setProperties(prefix + FRAME_PROPERTY_NAME, props);

            omMapTitle.setProperties(prefix + MapTitlePropertyString, props);
            omLegendTitle.setProperties(prefix + LegendTitlePropertyString, props);

            omScale.setProperties(prefix + SCALE_PROPERTY_NAME, props);
            omCompass.setProperties(prefix + COMPASS_PROPERTY_NAME, props);

            mapFrame.setProperties(prefix + MAP_FRAME_PROPERTY_NAME, props);

	    if (animationFrame != null) {
		animationFrame.setProperties(prefix + AnimationFramePropertyString, props);
	    }
        }
        if (legendableLayers != null) {
            for (LegendableLayer legendable : legendableLayers) {
                legendable.setLegendProperties(prefix, properties);
            }
        }

        // Update graphics, see generateGraphics
        props = properties;
    }

    public Properties getProperties(Properties properties) {
        properties = super.getProperties(properties);
        String prefix = PropUtils.getScopedPropertyPrefix(this);
        return getProperties(prefix, properties);
    }

    public Properties getProperties(String prefix, Properties properties) {
        // properties = super.getProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);

        properties.put(prefix + MapTitlePropertyString, getMapTitle());
        properties.put(prefix + LegendTitlePropertyString, getLegendTitle());
        properties.put(prefix + XAnchorPropertyString, String.valueOf(xAnchor));
        properties.put(prefix + YAnchorPropertyString, String.valueOf(yAnchor));
        properties.put(prefix + WidthPropertyString, String.valueOf(width));
        properties.put(prefix + HeightPropertyString, String.valueOf(height));
        properties.put(prefix + ExplodePropertyString, String.valueOf(explode));

        if (omScale != null) {
            properties = omScale.getProperties(prefix + SCALE_PROPERTY_NAME, properties);
        }

        if (omCompass != null) {
            properties = omCompass.getProperties(prefix + COMPASS_PROPERTY_NAME, properties);
        }

        if (frame != null) {
            properties = frame.getProperties(prefix + FRAME_PROPERTY_NAME, properties);
	    if (animationFrame != null) {
		properties = animationFrame.getProperties(prefix + AnimationFramePropertyString, properties);
	    }
        }
        if (mapFrame != null) {
            properties = mapFrame.getProperties(prefix + MAP_FRAME_PROPERTY_NAME, properties);
        }

        // Font settings
        if (omMapTitle != null) {
            properties = omMapTitle.getProperties(prefix + MapTitlePropertyString,
						  properties);
        }

        if (omLegendTitle != null) {
            properties = omLegendTitle.getProperties(prefix + LegendTitlePropertyString, properties);
        }

        if (additionalTexts != null) {
            int size = additionalTexts.getResizeableGraphics().size();
            properties.put(prefix + ADDITIONAL_COUNT_PROPERTY_NAME, String.valueOf(size));
            for (int i = 0; i < size; i++) {
                String otherPrefix = prefix + ADDITIONAL_TEXT_PROPERTY_NAME + String.valueOf(i);
                properties = additionalTexts.getResizeableGraphics().get(i).getProperties(otherPrefix, properties);
            }
        }

	if (animationFrame != null) {
	    int size = animationFrame.getResizeableGraphics().size();
            properties.put(prefix + AnimationTextsCountPropertyString, String.valueOf(size));
            for (int i = 0; i < size; i++) {
                String otherPrefix = prefix + AnimationTextPropertyString + String.valueOf(i);
                properties = animationFrame.getResizeableGraphics().get(i).getProperties(otherPrefix, properties);
            }
	}

        // Legend properties for each Thematic
        if (legendableLayers != null) {
            for (LegendableLayer legendable : legendableLayers) {
                properties = legendable.getLegendProperties(prefix, properties);
            }
        }

        return properties;
    }

    public void setDragging(boolean b) {
        dragging = b;
        if (!dragging) {
            doPrepare();
        }
    }

    public void titleUpdate(String title) {
        if (initMapTitle.equals("")) {
            initMapTitle = getMapTitle();
        }
        // System.out.println("************************" + initMapTitle + " - " + title);
        omMapTitle.setText(initMapTitle + " - " + title);
        refreshInteractiveGraphics();
    }

    public static void printOMGraphic(OMGraphic omg, int indent, java.io.PrintStream out) {
        if (omg instanceof OMGraphicList) {
            for (int i = 0; i < ((OMGraphicList) omg).size(); i++) {
                printOMGraphic(((OMGraphicList) omg).getOMGraphicAt(i), indent + 2, out);
            }
        } else {
            for (int n = indent; n > 0; n--) {
                out.print("-");
            }
            out.println(omg);
        }
    }

    public void propertyChange(PropertyChangeEvent pce) {
        //System.out.println(MapBean.ProjectionProperty);
        if (pce.getPropertyName().equals(MapBean.ProjectionProperty)) {
            badCache = true;
        }
    }

    public void offseterChanged(Offseter offseter) {
        badCache = true;
        refreshInteractiveGraphics();
    }

    public void layerRemoved(Layer l) {
    }

    private MetaAnimator _metaAnimator; 
    protected MetaAnimator getMetaAnimator() {
	if (_metaAnimator == null) {
 
	    BeanContext mapHandler = getBeanContext();
	    if (mapHandler == null)
		return null;

	    Iterator it = mapHandler.iterator();
	    while (it.hasNext()) {
		Object o = it.next();
		if (o instanceof MetaAnimator) {
		    _metaAnimator = (MetaAnimator) o;
		    break;
		}
	    }
	}
        return _metaAnimator;
    }
    public static class TopBox extends Box {

    public TopBox() {
        super(BoxLayout.X_AXIS);
    }

    @Override
    public Component add(Component c) {
        JPanel p = new JPanel(new BorderLayout());
        p.add(c, BorderLayout.NORTH);
        p.setBorder(BorderFactory.createEmptyBorder(0, 1, 0, 1));
        return super.add(p);
    }
}

}



/** Used to create GUI and save properties */
class GraphicForGUI {

    private String name, prefix;
    private OMResizeableGraphic graphic;

    public GraphicForGUI(String p, String n, OMResizeableGraphic g) {
        prefix = p;
        name = n;
        graphic = g;
    }

    public String getPrefix() {
        return prefix;
    }

    public String getName() {
        return name;
    }

    public OMResizeableGraphic getGraphic() {
        return graphic;
    }
}

