// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.legendLayer.gui;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;
import com.bbn.openmap.omGraphics.OMColorChooser;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.omGraphics.OMRect;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.gui.ThemaMapColorChooser;
import fr.unicaen.openmap.layer.legendLayer.LegendLayer;
import fr.unicaen.openmap.omGraphics.DrawingAttributes;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Properties;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 *
 * @author jchauvea
 */
public class OMLegendFrame extends OMGraphicList implements PropertyChangeListener {

    private OMRect outRect;
    private OMRect inRect;
    private OMPoly frame;
    private int thickness;
    protected LegendLayer legend;
    protected Box gui;
    protected DrawingAttributes daOut;
    protected DrawingAttributes daIn;
    protected DrawingAttributes daBg;
    protected I18n i18n = Environment.getI18n();
    public static String thicknessPropertyString = "thickness";
    public static String visiblePropertyString = "visible";
    public static String fillColorPropertyString = "fillColor";
    public static String inColorPropertyString = "inColor";
    public static String outColorPropertyString = "outColor";
    public static String DA_OUT_PROPERTY_NAME = "daout";
    public static String DA_IN_PROPERTY_NAME = "dain";
    public static String DA_BG_PROPERTY_NAME = "dabg";

    public OMLegendFrame(LegendLayer legend) {
        super();
        this.thickness = 10;
        this.legend = legend;
        daBg = new DrawingAttributes(false, false, false, false, false);
        daBg.setFilled(true);
        daIn = new DrawingAttributes(true, false, false, true, false); 
        daIn.setFillPaint(Color.WHITE);
        daOut = new DrawingAttributes(true, false, false, true, false);
        daOut.setFillPaint(Color.WHITE);
        daIn.getPropertyChangeSupport().addPropertyChangeListener(this);
        daOut.getPropertyChangeSupport().addPropertyChangeListener(this);
        visible = false;
        init();
    }

    private void init() {
        clear();
        if(!visible)
            return;
        int width = legend.getMapBean().getWidth() - 1;
        int height = legend.getMapBean().getHeight() - 1;
        //frame
        int[] xyPoints = {
            0, 0,
            width, 0,
            width, height,
            0, height,
            thickness, height - thickness,
            width - thickness, height - thickness,
            width - thickness, thickness,
            thickness, thickness,
            thickness, height - thickness,
            0, height,
            0, 0
        };
        frame = new OMPoly(xyPoints);
        frame.setFillPaint(daBg.getFillPaint());
        frame.setLinePaint(daBg.getFillPaint());

        //out rectangle
        outRect = new OMRect(0, 0, width, height);
        outRect.setLinePaint(daOut.getLinePaint());
        outRect.setStroke(daOut.getStroke());

        //in rectangle
        inRect = new OMRect(thickness, thickness,
                width - thickness, height - thickness);
        inRect.setLinePaint(daIn.getLinePaint());
        inRect.setStroke(daIn.getStroke());

        add(outRect);
        add(inRect);
        add(frame);


    }

    public Properties getProperties(String prefix, Properties props) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        //props.put(prefix + fillColorPropertyString, fillPaint);
        daOut.setPropertyPrefix(prefix + DA_OUT_PROPERTY_NAME);
        daIn.setPropertyPrefix(prefix + DA_IN_PROPERTY_NAME);
        daBg.setPropertyPrefix(prefix + DA_BG_PROPERTY_NAME);
        props.put(prefix + thicknessPropertyString, String.valueOf(thickness));
        props.put(prefix + visiblePropertyString, String.valueOf(isVisible()));
        props = daOut.getProperties(props);
        props = daIn.getProperties(props);
        props = daBg.getProperties(props);
        daBg.setFilled(true);
        return props;
    }

    public void setProperties(String prefix, Properties properties) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        if (properties == null) {
            return;
        }
        daIn.setProperties(prefix + DA_IN_PROPERTY_NAME, properties);
        daOut.setProperties(prefix + DA_OUT_PROPERTY_NAME, properties);
        daBg.setProperties(prefix + DA_BG_PROPERTY_NAME, properties);
        daBg.setFilled(true);
        thickness = PropUtils.intFromProperties(properties, prefix + thicknessPropertyString, thickness);
        setVisible(PropUtils.booleanFromProperties(properties, prefix + visiblePropertyString, isVisible()));
        init();

    }

    public JComponent getGUI() {
        if (gui == null) {
            gui = Box.createVerticalBox();
            //ComponentSizer.fixComponentSize(gui, new Dimension(250,50));
            //gui.setBorder(BorderFactory.createTitledBorder(i18n.get(this,"mapframe","Map Frame")));
            JButton bgColorButton = new JButton();
            bgColorButton.setBackground((Color) daBg.getFillPaint());
            ComponentSizer.fixComponentSize(bgColorButton, new Dimension(20, 20));
            bgColorButton.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent ae) {
                    JButton source = (JButton) ae.getSource();
                    Color col = ThemaMapColorChooser.showDialog(source, "", source.getBackground());
                    daBg.setFillPaint(col);
		    frame.setFillPaint(col);
                    source.setBackground(col);
                    legend.refreshGraphicsAndGUI();
                }
            });
	    bgColorButton.setEnabled(isVisible());
            final SpinnerNumberModel spm = new SpinnerNumberModel(thickness, 1, 50, 1);
            JSpinner spinThickness = new JSpinner(spm);
            spinThickness.addChangeListener(new ChangeListener() {

                public void stateChanged(ChangeEvent ce) {
                    thickness = (Integer) spm.getValue();
                    legend.refreshGraphicsAndGUI();
                }
            });
            JPanel panThickness = new JPanel();
            final JCheckBox visibleCB =
                    new JCheckBox(Util.loadIcon(!isVisible() ? "disabled.png" : "enabled.png"), isVisible());
            visibleCB.setBackground(Color.WHITE);
            visibleCB.setOpaque(false);
            visibleCB.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    setVisible(visibleCB.isSelected());                  
                    legend.refreshGraphicsAndGUI();
                }
            });
            panThickness.add(visibleCB);
            panThickness.add(new JLabel(i18n.get(this,"mapframe","Frame")));
	    panThickness.add(Box.createRigidArea(new Dimension(50,20)));
            panThickness.add(new JLabel(i18n.get(this, "thickness", "Thickness")));
            panThickness.add(spinThickness);
            Box panColors = Box.createVerticalBox();
            JPanel panBg = new JPanel();
            JPanel panLines = new JPanel();

            panBg.add(new JLabel(i18n.get(this, "color", "Color")));
            panBg.add(bgColorButton);

            panLines.add(new JLabel(i18n.get(this, "outlines", "Outlines")));
            panLines.add(new JLabel(i18n.get(this, "inside", "Inside")));
            panLines.add(daIn.getGUI());
	    //	    daIn.getGUI().setEnabled(isVisible());

            panLines.add(new JLabel(i18n.get(this, "outside", "Outside")));
            panLines.add(daOut.getGUI());
	    //	    daOut.getGUI().setEnabled(isVisible());

            panColors.add(panBg);
            panColors.add(panLines);
            panColors.setBorder(BorderFactory.createEmptyBorder());
            ComponentSizer.fixComponentSize(panColors, new Dimension(300,60));
            ComponentSizer.fixComponentSize(panThickness, new Dimension(300,30));
            //ComponentSizer.fixComponentSize(panThickness, new Dimension(250,40));
            //ComponentSizer.fixComponentSize(panColors, new Dimension(250,40));
            gui.add(panThickness);
            gui.add(panColors);
        }
        return gui;
    }

    public void propertyChange(PropertyChangeEvent pce) {
        legend.refreshGraphicsAndGUI();
    }

    public boolean isVisible() {
        return visible;
    }

    public void setVisible(boolean b) {
        visible = b;
    }
}
