// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.ThemaMapStrokeEditorMenu;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.AttributesNames;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ColorScale;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 *
 * @author jchauvea
 */
public class AttributesColorScalePanel extends ColorScalePanel {

    protected AttributesNames attributesNames;
    protected JComboBox colorCombos[];
    /**
     * Available data column names.
     */
    protected Vector<String> available;

    public AttributesColorScalePanel(ColorScale colorScale, AttributesNames attributesNames) {
        super(colorScale);
        setLayout(new BorderLayout());
        this.attributesNames = attributesNames;
        createColorScalePanel();
        createOptionPanel();
        add(panOption);
        JPanel pScale = new JPanel(new BorderLayout());
        pScale.add(panColorScale, BorderLayout.NORTH);
        JPanel pCenter = new JPanel();
        pCenter.add(panOption);
        pCenter.add(pScale);
        add(new JScrollPane(pCenter,
			    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
			    JScrollPane.HORIZONTAL_SCROLLBAR_NEVER), BorderLayout.CENTER);
    }

    @Override
	public void createOptionPanel(){
	super.createOptionPanel();
	colorNumber.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent arg0) {
		    int n = ((Integer) colorNumber.getValue()).intValue();
		    colorScale.setColorNumber(n);
		    refreshPanel();
		    return;
		}
	    });
    }

    @Override
	public void createColorScalePanel() {

        if (panColorScale == null) {
            panColorScale = new JPanel(new BorderLayout());
        }
        Box pColors = Box.createVerticalBox();
        colorCombos = new JComboBox[colorScale.getColorNumber()];
        colorButtons.clear();
        if (available == null) {
            available = new Vector<String>();
        }
        Dimension buttonsDimension = new Dimension(20, 20);
        JPanel pTop = new JPanel();
        pTop.add(new JLabel(i18n.get(AttributesColorScalePanel.class, "attributes","Attributes to represent")));
        panColorScale.add(pTop, BorderLayout.NORTH);
        //System.out.println(attributesNames.getName(0));
        for (int i = 0; i < colorScale.getColorNumber(); i++) {
	    // System.out.println(i);
	    // System.out.println("a-"+attributesNames.getName(i));
            ColorScaleButton b = new ColorScaleButton(this, i);
            colorButtons.add(b);
            ComponentSizer.fixComponentSize(b, buttonsDimension);
            b.setBackground(colorScale.getColor(i));
            colorCombos[i] = new JComboBox();

            for (int j = 0; j < available.size(); j++) {
                colorCombos[i].addItem(available.get(j));
            }

            if(!attributesNames.getName(i).equals("")){
                colorCombos[i].addItem(attributesNames.getName(i));
                colorCombos[i].setSelectedItem(attributesNames.getName(i));
            }

            ComponentSizer.fixComponentSize(colorCombos[i], new Dimension(100, 20));
            ItemListener itml = new ItemListener() {

		    public void itemStateChanged(ItemEvent ie) {
			refresh();
		    }
		};
            
            colorCombos[i].addItemListener(itml);
            JPanel p = new JPanel();
            p.add(b);
            p.add(colorCombos[i]);
            p.setBorder(BorderFactory.createEmptyBorder(-5, 0, -5, 0));
            pColors.add(p);
        }
        panColorScale.add(pColors, BorderLayout.CENTER);
        updateUI();
    }


    @Override
    public void updateColorScaleAt(int position, Color color) {
        super.updateColorScaleAt(position, color);
	attributesNames.refresh();
    }

    public void updateCombos(JComboBox unUpdateCombo, ItemListener itml) {
        for (JComboBox combo : colorCombos) {
            if (combo != unUpdateCombo) {
                combo.removeItemListener(itml);
                combo.removeAllItems();
                for (String s : available) {
                    combo.addItem(s);
                }
                combo.addItemListener(itml);
            }
        }
    }

    public void refresh() {
        Vector<String> v = new Vector<String>();
        for (int i = 0; i < colorCombos.length; i++) {
            if (colorCombos[i].getSelectedItem() != null && !colorCombos[i].getSelectedItem().toString().equals("")) {
                String sel = colorCombos[i].getSelectedItem().toString();
                v.add(sel);
                colorCombos[i].setToolTipText(sel);
            }
        }

        Enumeration<String> elements = v.elements();
        String[] names = new String[v.size()];
        String element;
        //reorganization
        for (int i = 0; i < colorScale.getColorNumber(); i++) {

            if (elements.hasMoreElements()) {
                element = elements.nextElement();
                names[i] = element;
            } else {
            }
        }
        attributesNames.setAttributesNames(names);
        fireColorScaleChanged();
    }

    /**
     * Sets the available names.
     * @param available new names
     */
    public void setAvailableNames(ArrayList<String> available) {
        this.available.clear();
        this.available.add("");
        for (String a : available) {
            this.available.addElement(a);
        }
        refreshPanel();
    }


}
