// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.ThemaMapStrokeEditorMenu;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.AttributesNames;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ColorScale;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Enumeration;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Fixed length AttributesNames management copy from AttributesColorScalePanel.
 *
 * @author Jacques.Madelaine@unicaen.fr
 */
public class AttributesPanel extends JPanel {

    protected AttributesNames attributesNames;
    protected JComboBox combos[];
    /**
     * Available data column names.
     */
    protected ArrayList<String> available;

    protected ItemListener itml; 
    protected int length;
    protected String label;

    public AttributesPanel(AttributesNames attributesNames, int length, String label) {
        setLayout(new BorderLayout());
        this.attributesNames = attributesNames;
	this.length = length;
	this.label = label;
	createAttributesPanel();
	add(panAttributes);
    }

    protected JPanel panAttributes;

    public void createAttributesPanel() {
	if (panAttributes == null) 
	    panAttributes = new JPanel(new BorderLayout());

	panAttributes.add(new JLabel(label), BorderLayout.WEST);

	Box box = Box.createVerticalBox();
        combos = new JComboBox[length];

        if (available == null) {
            available = new ArrayList<String>();
        }

	itml = new ItemListener() {	
		public void itemStateChanged(ItemEvent ie) {
		    refresh();
		}
	    };

        //System.out.println(attributesNames.getName(0));
        for (int i = 0; i < length; i++) {
            combos[i] = new JComboBox();

            for (int j = 0; j < available.size(); j++) {
                combos[i].addItem(available.get(j));
            }

            if(!attributesNames.getName(i).equals("")){
		combos[i].addItem(attributesNames.getName(i));
                combos[i].setSelectedItem(attributesNames.getName(i));
            }

            ComponentSizer.fixComponentSize(combos[i], new Dimension(150, 20));
            
            combos[i].addItemListener(itml);
            JPanel p = new JPanel();
            p.add(combos[i]);
	    // p.setBorder(BorderFactory.createEmptyBorder(-5, 0, -5, 0));
            box.add(p);
        }

	panAttributes.add(box, BorderLayout.CENTER);
        updateUI();

    }

//     public void updateCombos(JComboBox unUpdateCombo, ItemListener itml) {
//         for (JComboBox combo : combos) {
//             if (combo != unUpdateCombo) {
//                 combo.removeItemListener(itml);
//                 combo.removeAllItems();
//                 for (String s : available) {
//                     combo.addItem(s);
//                 }
//                 combo.addItemListener(itml);
//             }
//         }
//     }

    public void refresh() {
        String[] v = new String[combos.length];
        for (int i = 0; i < combos.length; i++) {
            if (combos[i].getSelectedItem() != null && !combos[i].getSelectedItem().toString().equals("")) {
                String sel = combos[i].getSelectedItem().toString();
                v[i]= sel;
                combos[i].setToolTipText(sel);
            }
        }

        attributesNames.setAttributesNames(v);
    }

    /**
     * Sets the available names.
     * @param available new names
     */
    public void setAvailableNames(String[ ]available) {
        this.available.clear();
        this.available.add("");
        for (String a : available) {
            this.available.add(a);
        }
	refreshPanel();
    }

    public void refreshPanel() {      
	panAttributes.removeAll();
	createAttributesPanel();
    }

}
