// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;
import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ColorScale;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.StandardColorScale;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ColorScaleListener;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.BorderLayout;
import javax.swing.BorderFactory;
import java.util.ArrayList;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 *
 * @author jchauvea
 */
public abstract class ColorScalePanel extends JPanel {

    public static I18n i18n = Environment.getI18n();
    /**
     * Default property name for color scale.
     */
    public static final String CUSTOM_PROPERTY_NAME = "custom";
    public static final String CRAYOLA_PROPERTY_NAME = "crayola";
    public static final String CRAYOLARED_PROPERTY_NAME = "crayolared";
    public static final String CRAYOLABLUE_PROPERTY_NAME = "crayolablue";
    public static final String PASTEL_PROPERTY_NAME = "pastel";
    public static final String WATER_PROPERTY_NAME = "water";
    public static final String SAND_PROPERTY_NAME = "sand";
    public static final String CAMOUFLAGE_PROPERTY_NAME = "camouflage";
    public static final String WARM_PROPERTY_NAME = "warm";
    public static final String DESERT1_PROPERTY_NAME = "desert1";
    public static final String DESERT2_PROPERTY_NAME = "desert2";
    public static final String FOREST_PROPERTY_NAME = "forest";
    public static final String COLD_PROPERTY_NAME = "cold";
    public static final String FRUIT_PROPERTY_NAME = "fruit";
    public static final String BLUE_GREEN_PROPERTY_NAME = "blueGreen";
    public static final String FLESH_PROPERTY_NAME = "flesh";
    public static final String MINERAL_PROPERTY_NAME = "mineral";


// Crayola
// #ddff0000 rouge
// #ddff2600 écarlate
// #ddff4800 orange-rouge
// #ddff8000 orange
// #ddff9900 jaune-orange
// #ddffff00  jaune
// #ddcefe00 jaune-vert
// #dd57d200 vert-jaune
// #dd30b800 pomme granny smith
// #dd008000 vert
// #dd0d99ba bleu-vert
// #dd0000ff bleu
// #dd400080 indigo
// #dd600080 bleu-violet
// #dd800080 violet (pourpre)
// #dd803300 brun
// #dd9a1a00 marron
// #dd000000 noir

// #ddba067e violet-rouge

// #ddffa3cc rose œillet
// 
// #ddffd9b0 abricot
// #dd0066ff céruléen
// #dd808080 gris
// #ddffcc00 pissenlit
// #ddf51780 rouge-violet
// 
// #ddb0b7c6 bleu cadet
// #dd00ccff bleu ciel
// #ddb469ff glycine
// #ddda9100 havane
// #ddc0c0c0 loup gris
// #ddff9999 melon
// #ddffcc99 pêche
// 
// #ddb00000 acajou
// #ddb38b6d amarante
// #dd7c80de bleuet
// #ddf0dc00 gerbe d’or
// #ddecafff lavande
// #ddff9d5b macaroni au fromage
// #dd6957c0 majestueuses montagnes pourpres
// #dde699ff mauveilleux
// #ddff7385 saumon
// #dd4d1a00 sépia
// #ddb5651d sienne brûlé
// #dd996633 sienne naturelle
// #dd99ffcc vert marin
// #dd608c00 vert olive
// 
// #ddccff99 vert printemps
// #dd999999 argent
// #dd5c7745 asperge
// #dd00cccc bleu œuf de merle
// #dd005cb8 bleu pacifique
// #dd11c1ff bleu turquoise
// #ddd64200 douce-amère
// #ddfc1464 fraise sauvage
// #dde600e6 magenta
// #dd808000 or
// #ddee8400 orange brûlé
// #ddcc00ff orchidée
// #dd99ccff pervenche
// #dd993366 prune
// #ddff71b8 rose chatouille
// #ddb61000 rouge brique
// #dd02863e vert forêt

    public static StandardColorScale getStandardShadedColorScale() {
	return predefinedColorMaps[6];
    }
    public static StandardColorScale getStandardPastelColorScale() {
	return predefinedColorMaps[7];
    }

    private static StandardColorScale predefinedColorMaps[] = {
	null,
	new StandardColorScale(i18n.get(ColorScalePanel.class, FLESH_PROPERTY_NAME, FLESH_PROPERTY_NAME),
			      new String[] {
				   "ff907b6f",
				   "ff907b6f ff5b443b",
				   "fff8e8d7 ff907b6f ff5b443b",
				   "fff8e8d7 ffd6c4b5 ff907b6f ff5b443b",
				   "fff8e8d7 ffd6c4b5 ffb3a092 ff907b6f ff5b443b",
				   "fff8e8d7 ffd6c4b5 ffb3a092 ff907b6f ff7e695e ff5b443b",
				   "fff8e8d7 ffd6c4b5 ffc4b2a3 ffb3a092 ff907b6f ff7e695e ff5b443b",
				   "fff8e8d7 ffd6c4b5 ffc4b2a3 ffb3a092 ffa18d81 ff907b6f ff7e695e ff5b443b",
				   "fff8e8d7 ffe7d6c6 ffd6c4b5 ffc4b2a3 ffb3a092 ffa18d81 ff907b6f ff7e695e ff5b443b",
				   "fff8e8d7 ffe7d6c6 ffd6c4b5 ffc4b2a3 ffb3a092 ffa18d81 ff907b6f ff7e695e ff6d574d ff5b443b",
			       }),
	new StandardColorScale(i18n.get(ColorScalePanel.class, COLD_PROPERTY_NAME, COLD_PROPERTY_NAME),
			      new String[] {
				   "ff004988",
				   "ffe5e9f4 ff004988",
				   "ffe5e9f4 ff80a2c4 ff004988",
				   "ffe5e9f4 ffb3c6dc ff80a2c4 ff004988",
				   "ffe5e9f4 ffb3c6dc ff80a2c4 ff336da0 ff004988",
				   "ffe5e9f4 ffb3c6dc ff80a2c4 ff6691b8 ff336da0 ff004988",
				   "ffe5e9f4 ffb3c6dc ff80a2c4 ff6691b8 ff4d7fac ff336da0 ff004988",
				   "ffe5e9f4 ffb3c6dc ff99b4d0 ff80a2c4 ff6691b8 ff4d7fac ff336da0 ff004988",
				   "ffe5e9f4 ffccd8e8 ffb3c6dc ff99b4d0 ff80a2c4 ff6691b8 ff4d7fac ff336da0 ff004988",
				   "ffe5e9f4 ffccd8e8 ffb3c6dc ff99b4d0 ff80a2c4 ff6691b8 ff4d7fac ff336da0 ff1a5b94 ff004988",
			       }),
	new StandardColorScale(i18n.get(ColorScalePanel.class, WARM_PROPERTY_NAME, WARM_PROPERTY_NAME),
			      new String[] {
				   "fffadb00",
				   "fffadb00 ff990000",
				   "fffadb00 ffc56200 ff990000",
				   "fffadb00 ffc56200 ffaf3100 ff990000",
				   "fffadb00 ffc56200 ffba4900 ffaf3100 ff990000",
				   "fffadb00 ffe5ab00 ffc56200 ffba4900 ffaf3100 ff990000",
				   "fffadb00 ffe5ab00 ffda9200 ffc56200 ffba4900 ffaf3100 ff990000",
				   "fffadb00 ffe5ab00 ffda9200 ffcf7a00 ffc56200 ffba4900 ffaf3100 ff990000",
				   "fffadb00 fff0c300 ffe5ab00 ffda9200 ffcf7a00 ffc56200 ffba4900 ffaf3100 ff990000",
				   "fffadb00 fff0c300 ffe5ab00 ffda9200 ffcf7a00 ffc56200 ffba4900 ffaf3100 ffa41900 ff990000",
			       }),
	new StandardColorScale(i18n.get(ColorScalePanel.class, BLUE_GREEN_PROPERTY_NAME, BLUE_GREEN_PROPERTY_NAME),
			      new String[] {
				   //"1E3789 007DBA 00A1D9 4DC0E6 0095A8 00883C 3DA231 BDCB1E B8C763 E8ECCB "
				   "E8ECCB B8C763 BDCB1E 3DA231 00883C 0095A8 4DC0E6 00A1D9 007DBA 1E3789",
				   "E8ECCB B8C763 BDCB1E 3DA231 00883C 0095A8 4DC0E6 00A1D9 007DBA 1E3789",
				   "E8ECCB B8C763 BDCB1E 3DA231 00883C 0095A8 4DC0E6 00A1D9 007DBA 1E3789",
				   "E8ECCB B8C763 BDCB1E 3DA231 00883C 0095A8 4DC0E6 00A1D9 007DBA 1E3789",
				   "E8ECCB B8C763 BDCB1E 3DA231 00883C 0095A8 4DC0E6 00A1D9 007DBA 1E3789",
				   "E8ECCB B8C763 BDCB1E 3DA231 00883C 0095A8 4DC0E6 00A1D9 007DBA 1E3789",
				   "E8ECCB B8C763 BDCB1E 3DA231 00883C 0095A8 4DC0E6 00A1D9 007DBA 1E3789",
				   "E8ECCB B8C763 BDCB1E 3DA231 00883C 0095A8 4DC0E6 00A1D9 007DBA 1E3789",
				   "E8ECCB B8C763 BDCB1E 3DA231 00883C 0095A8 4DC0E6 00A1D9 007DBA 1E3789",
				   "E8ECCB B8C763 BDCB1E 3DA231 00883C 0095A8 4DC0E6 00A1D9 007DBA 1E3789",
			       }),
	new StandardColorScale(i18n.get(ColorScalePanel.class, CRAYOLABLUE_PROPERTY_NAME, CRAYOLABLUE_PROPERTY_NAME),
			       new String[] {
				   "ddffdc00",
				   "ddffdc00 dd53006f",
				   "ddffdc00 dd004b45 dd53006f",
				   "ddffdc00 dd125e14 dd111171 dd53006f",
				   "ddffdc00 dd3e9300 dd004b45 dd111171 dd53006f",
				   "ddffdc00 dd3e9300 dd004b45 dd111171 dd270053 dd53006f",
				   "ddffdc00 dd3e9300 dd004b45 dd121298 dd111171 dd34006f dd53006f",
				   "ddffdc00 dd3e9300 dd125e14 dd004b45 dd121298 dd111171 dd34006f dd53006f",
				   "ddffdc00 dd3e9300 dd125e14 dd004b45 dd121298 dd111171 dd34006f dd270053 dd53006f",
				   "ddffdc00 ddbaff00 dd3e9300 dd125e14 dd004b45 dd121298 dd111171 dd34006f dd270053 dd53006f",
	    }),
	new StandardColorScale(i18n.get(ColorScalePanel.class, CRAYOLARED_PROPERTY_NAME, CRAYOLARED_PROPERTY_NAME),
			      new String[] {
				   "ddffdc00",
				   "ddffdc00 dd8b0052",
				   "ddffdc00 ddeb3200 dd8b0052",
				   "ddffdc00 ddeb5300 ddb00000 dd8b0052",
				   "ddffdc00 ddff9540 ddeb5300 ddb00000 dd8b0052",
				   "ddffdc00 ddff9540 ddeb5300 ddd00000 dd8b0038 dd8b0052",
				   "ddffdc00 ddff9540 ddeb5300 ddd00000 ddb00000 dd8b0038 dd8b0052",
				   "ddffdc00 ddffba00 ddff9540 ddeb5300 ddd00000 ddb00000 dd8b0038 dd8b0052",
				   "ddffdc00 ddffba00 ddff9540 ddeb5300 ddeb3200 ddd00000 ddb00000 dd8b0038 dd8b0052",
				   "ddffdc00 ddffba00 ddff9540 ddff8400 ddeb5300 ddeb3200 ddd00000 ddb00000 dd8b0038 dd8b0052",
			       }),
	new StandardColorScale(i18n.get(ColorScalePanel.class, PASTEL_PROPERTY_NAME, PASTEL_PROPERTY_NAME),
			      new String[] {
				   "ddffb300",
				   "ddffb300 dd74c476",
				   "ddffb300 ddedd3c1 dd74c476",
				   "ddffb300 ddedd3c1 dd74c476 dd80a6ce",
				   "ddffb300 ddedd3c1 dd74c476 ddbfd1e5 dd80a6ce",
				   "ddffb300 ddedd3c1 dda6dba0 dd74c476 ddbfd1e5 dd80a6ce",
				   "ddffff4d ddffb300 ddedd3c1 dda6dba0 dd74c476 ddbfd1e5 dd80a6ce",
				   "ddffff4d ddffb300 ddf4c6de ddedd3c1 dda6dba0 dd74c476 ddbfd1e5 dd80a6ce",
			       }),
	new StandardColorScale(i18n.get(ColorScalePanel.class, CRAYOLA_PROPERTY_NAME, CRAYOLA_PROPERTY_NAME),
			      new String[] {
				   "ddff0000",
				   "ddff0000 ddff8000 ",
				   "ddff0000 ddff8000 ddffff00",
				   "ddff0000 ddff8000 ddffff00 dd008000",
				   "ddff0000 ddff8000 ddffff00 dd008000 dd0000ff",
				   "ddff0000 ddff8000 ddffff00 dd008000 dd0000ff dd800080",
				   "ddff0000 ddff8000 ddffff00 dd008000 dd0000ff dd800080 dd803300",
				   "ddff0000 ddff8000 ddffff00 dd008000 dd0000ff dd800080 dd803300 dd000000",
				   "ddff0000 ddff8000 ddffff00 dd57d200 dd008000 dd0000ff dd800080 dd803300 dd000000",
				   "ddff0000 ddff8000 ddffff00 dd57d200 dd008000 dd0000ff dd400080 dd800080 dd803300 dd000000",
				   "ddff0000 ddff8000 ddffff00 dd57d200 dd008000 dd0000ff dd400080 dd600080 dd800080 dd803300 dd000000",
				   "ddff0000 ddff4800 ddff9900 ddffff00 dd57d200 dd008000 dd0000ff dd400080 dd600080 dd800080 dd803300 dd000000",
				   "ddff0000 ddff4800 ddff9900 ddffff00 ddcefe00 dd57d200 dd008000 dd0000ff dd400080 dd600080 dd800080 dd803300 dd000000",
				   "ddff0000 ddff4800 ddff8000 ddff9900 ddffff00 ddcefe00 dd57d200 dd008000 dd0000ff dd400080 dd600080 dd800080 dd803300 dd000000",
				   "ddff0000 ddff4800 ddff8000 ddff9900 ddffff00 ddcefe00 dd57d200 dd008000 dd0d99ba dd0000ff dd400080 dd600080 dd800080 dd803300 dd000000",
				   "ddff0000 ddff4800 ddff8000 ddff9900 ddffff00 ddcefe00 dd57d200 dd30b800 dd008000 dd0d99ba dd0000ff dd400080 dd600080 dd800080 dd803300 dd000000",
				   "ddff0000 ddff2600 ddff4800 ddff8000 ddff9900 ddffff00 ddcefe00 dd57d200 dd30b800 dd008000 dd0d99ba dd0000ff dd400080 dd600080 dd800080 dd803300 dd000000",
				   "ddff0000 ddff2600 ddff4800 ddff8000 ddff9900 ddffff00 ddcefe00 dd57d200 dd30b800 dd008000 dd0d99ba dd0000ff dd400080 dd600080 dd800080 dd803300 dd9a1a00 dd000000",
			       }),

	// html  ddc0c0c0 dd808080 dd000000 ddff0000 dd800000 ddffff00 dd808000 dd00ff00 dd008000 dd00ffff, dd008080 dd0000ff, dd000080 ddff00ff, dd800080
	new StandardColorScale(i18n.get(ColorScalePanel.class, CAMOUFLAGE_PROPERTY_NAME, CAMOUFLAGE_PROPERTY_NAME),
			      new String[] {
				   "523A1C 967E4D 76945F CABA89 F0EBE2 1C4761 4981A5 90B8D3 413A2C 224A1E",
				   "523A1C 967E4D 76945F CABA89 F0EBE2 1C4761 4981A5 90B8D3 413A2C 224A1E",
				   "523A1C 967E4D 76945F CABA89 F0EBE2 1C4761 4981A5 90B8D3 413A2C 224A1E",
				   "523A1C 967E4D 76945F CABA89 F0EBE2 1C4761 4981A5 90B8D3 413A2C 224A1E",
				   "523A1C 967E4D 76945F CABA89 F0EBE2 1C4761 4981A5 90B8D3 413A2C 224A1E",
				   "523A1C 967E4D 76945F CABA89 F0EBE2 1C4761 4981A5 90B8D3 413A2C 224A1E",
				   "523A1C 967E4D 76945F CABA89 F0EBE2 1C4761 4981A5 90B8D3 413A2C 224A1E",
				   "523A1C 967E4D 76945F CABA89 F0EBE2 1C4761 4981A5 90B8D3 413A2C 224A1E",
				   "523A1C 967E4D 76945F CABA89 F0EBE2 1C4761 4981A5 90B8D3 413A2C 224A1E",
				   "523A1C 967E4D 76945F CABA89 F0EBE2 1C4761 4981A5 90B8D3 413A2C 224A1E",
			       }),
	new StandardColorScale(i18n.get(ColorScalePanel.class, MINERAL_PROPERTY_NAME, MINERAL_PROPERTY_NAME),
			      new String[] {
				   "A12116 7A1114 7E6666 6C4A4D 544045 4F0B0A 889EA9 5D7784 3F5E65 173542",
				   "A12116 7A1114 7E6666 6C4A4D 544045 4F0B0A 889EA9 5D7784 3F5E65 173542",
				   "A12116 7A1114 7E6666 6C4A4D 544045 4F0B0A 889EA9 5D7784 3F5E65 173542",
				   "A12116 7A1114 7E6666 6C4A4D 544045 4F0B0A 889EA9 5D7784 3F5E65 173542",
				   "A12116 7A1114 7E6666 6C4A4D 544045 4F0B0A 889EA9 5D7784 3F5E65 173542",

				   "A12116 7A1114 7E6666 6C4A4D 544045 4F0B0A 889EA9 5D7784 3F5E65 173542",
				   "A12116 7A1114 7E6666 6C4A4D 544045 4F0B0A 889EA9 5D7784 3F5E65 173542",
				   "A12116 7A1114 7E6666 6C4A4D 544045 4F0B0A 889EA9 5D7784 3F5E65 173542",
				   "A12116 7A1114 7E6666 6C4A4D 544045 4F0B0A 889EA9 5D7784 3F5E65 173542",
				   "A12116 7A1114 7E6666 6C4A4D 544045 4F0B0A 889EA9 5D7784 3F5E65 173542",
			       }),
	new StandardColorScale(i18n.get(ColorScalePanel.class, FOREST_PROPERTY_NAME, FOREST_PROPERTY_NAME),
			      new String[] {
				   "94A653 6B8F24 48651C 2C4311 B39A12 8E7C14 665A10 4A420B 60410B 432D07",
				   "94A653 6B8F24 48651C 2C4311 B39A12 8E7C14 665A10 4A420B 60410B 432D07",
				   "94A653 6B8F24 48651C 2C4311 B39A12 8E7C14 665A10 4A420B 60410B 432D07",
				   "94A653 6B8F24 48651C 2C4311 B39A12 8E7C14 665A10 4A420B 60410B 432D07",
				   "94A653 6B8F24 48651C 2C4311 B39A12 8E7C14 665A10 4A420B 60410B 432D07",
				   "94A653 6B8F24 48651C 2C4311 B39A12 8E7C14 665A10 4A420B 60410B 432D07",
				   "94A653 6B8F24 48651C 2C4311 B39A12 8E7C14 665A10 4A420B 60410B 432D07",
				   "94A653 6B8F24 48651C 2C4311 B39A12 8E7C14 665A10 4A420B 60410B 432D07",
				   "94A653 6B8F24 48651C 2C4311 B39A12 8E7C14 665A10 4A420B 60410B 432D07",
				   "94A653 6B8F24 48651C 2C4311 B39A12 8E7C14 665A10 4A420B 60410B 432D07",
			       }),
	new StandardColorScale(i18n.get(ColorScalePanel.class, SAND_PROPERTY_NAME, SAND_PROPERTY_NAME),
			      new String[] {
				   "dd8b7d6b ddffe7ba dd008b8b dd006400 ddee7600 ddeec900 ddfff68f ddffa07a",
				   "dd8b7d6b ddffe7ba dd008b8b dd006400 ddee7600 ddeec900 ddfff68f ddffa07a",
				   "dd8b7d6b ddffe7ba dd008b8b dd006400 ddee7600 ddeec900 ddfff68f ddffa07a",
				   "dd8b7d6b ddffe7ba dd008b8b dd006400 ddee7600 ddeec900 ddfff68f ddffa07a",
				   "dd8b7d6b ddffe7ba dd008b8b dd006400 ddee7600 ddeec900 ddfff68f ddffa07a",
				   "dd8b7d6b ddffe7ba dd008b8b dd006400 ddee7600 ddeec900 ddfff68f ddffa07a",
				   "dd8b7d6b ddffe7ba dd008b8b dd006400 ddee7600 ddeec900 ddfff68f ddffa07a",
			       }),
	new StandardColorScale(i18n.get(ColorScalePanel.class, FRUIT_PROPERTY_NAME, FRUIT_PROPERTY_NAME),
			      new String[] {
				   "DC3628 A47E58 E2873F D4002E 8CB03A 1D7951 9E0C46 DBE9C8 621F62 FDE077",
				   "DC3628 A47E58 E2873F D4002E 8CB03A 1D7951 9E0C46 DBE9C8 621F62 FDE077",
				   "DC3628 A47E58 E2873F D4002E 8CB03A 1D7951 9E0C46 DBE9C8 621F62 FDE077",
				   "DC3628 A47E58 E2873F D4002E 8CB03A 1D7951 9E0C46 DBE9C8 621F62 FDE077",
				   "DC3628 A47E58 E2873F D4002E 8CB03A 1D7951 9E0C46 DBE9C8 621F62 FDE077",
				   "DC3628 A47E58 E2873F D4002E 8CB03A 1D7951 9E0C46 DBE9C8 621F62 FDE077",
				   "DC3628 A47E58 E2873F D4002E 8CB03A 1D7951 9E0C46 DBE9C8 621F62 FDE077",
				   "DC3628 A47E58 E2873F D4002E 8CB03A 1D7951 9E0C46 DBE9C8 621F62 FDE077",
				   "DC3628 A47E58 E2873F D4002E 8CB03A 1D7951 9E0C46 DBE9C8 621F62 FDE077",
				   "DC3628 A47E58 E2873F D4002E 8CB03A 1D7951 9E0C46 DBE9C8 621F62 FDE077",
			       }),
	/*
	new StandardColorScale(
			      new String[] {
			       }),
	*/
//        i18n.get(ColorScalePanel.class, DESERT1_PROPERTY_NAME, DESERT1_PROPERTY_NAME),
//        i18n.get(ColorScalePanel.class, DESERT2_PROPERTY_NAME, DESERT2_PROPERTY_NAME),

//    public static final String desert1ColorScaleInHex = "E3DCD3 C0B1A1 AA9886 967E69 795F48 563E29 AFBBA0 8D9D7A 5E7646 315B27";
//    public static final String desert2ColorScaleInHex = "EEE6D6 E1D4BC D3C09C C5A87B B68F62 A3774E 6F4919 EEEC00 FCC300 F29400";


    };

    protected JLabel numberLabel;
    protected JComboBox predefinedColorMapsCB;
    protected JSpinner colorNumber;
    protected JCheckBox shadedCheckBox;
    protected JCheckBox checkReverseColorMap;
    protected boolean shaded = false;
    protected ArrayList<ColorScaleButton> colorButtons = new ArrayList<ColorScaleButton>();
    protected ColorScale colorScale;
    protected JComponent panOption, panColorScale;
    protected JLabel colorLabel;

    public ColorScalePanel(ColorScale cs) {
        this.colorScale = cs;
    }

    public abstract void createColorScalePanel();

    public void createOptionPanel() {

        predefinedColorMapsCB = new JComboBox(predefinedColorMaps);
        ActionListener predefinedActionListener = new ActionListener() {
            // Predefined colormaps

            public void actionPerformed(ActionEvent ae) {
                int index = ((JComboBox) ae.getSource()).getSelectedIndex();
                if (index < 0) {
                    return;
                }
		colorScale.setModel(predefinedColorMaps[index]);
                shadedCheckBox.setSelected(shaded = false);
                refreshColor();
            }
        };

        predefinedColorMapsCB.addActionListener(predefinedActionListener);

        colorNumber = new JSpinner();
        /*colorNumber changeListener is set in derived classes */
        colorNumber.setValue(colorScale.getColorNumber());
        checkReverseColorMap = new JCheckBox(i18n.get(ColorScalePanel.class, "reverse", "reverse"));
        shadedCheckBox = new JCheckBox(i18n.get(ColorScalePanel.class, "shaded", "shaded"));
        shadedCheckBox.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                colorScale.setShaded(shadedCheckBox.isSelected());
                refreshPanel();
            }
        });
        checkReverseColorMap.addActionListener(new ActionListener () {
	        public void actionPerformed(ActionEvent e) {
		    colorScale.setReverse(checkReverseColorMap.isSelected());
		    refreshPanel();
		}
	    });
        panOption = Box.createVerticalBox();
        JPanel pNumber = new JPanel(new BorderLayout());
	pNumber.setBorder(BorderFactory.createEmptyBorder(5, 0, 10, 0));
        numberLabel = new JLabel(i18n.get(ColorScalePanel.class, "class_number", "Class number"));
        pNumber.add(numberLabel, BorderLayout.WEST);
        pNumber.add(colorNumber, BorderLayout.EAST);
        panOption.add(pNumber);
	JPanel pColorLabel = new JPanel(new BorderLayout());
        colorLabel = new JLabel(i18n.get(ColorScalePanel.class, "colors", "Colors"));	
	pColorLabel.add(colorLabel, BorderLayout.WEST);
	panOption.add(pColorLabel);
        JPanel pColors = new JPanel(new BorderLayout());
        pColors.add(predefinedColorMapsCB, BorderLayout.WEST);
        panOption.add(pColors);
        JPanel panCB = new JPanel();
        panCB.add(shadedCheckBox);
        panCB.add(checkReverseColorMap);
        panOption.add(panCB);


    }

    public void refreshPanel() {
	Object lock = listenerList.size() > 0 ? listenerList.get(0) : this;
	// System.out.println(Thread.currentThread() + " ColorScalePanel synchronized on " + lock);
	synchronized(lock) {
	    panColorScale.removeAll();
	    createColorScalePanel();
	    updateUI();
	    fireColorScaleChanged();
	}
	// System.out.println(Thread.currentThread() + " ColorScalePanel synchronized on " + lock + " <==");
    }

    public void updateColorScaleAt(int position, Color color) {
        colorScale.setColor(position, color);
    }

    /**
     * Just refresh color scale buttons.
     */
    public void refreshColor() {
        for (int i = 0; i < colorButtons.size(); i++) {
            colorButtons.get(i).setBackground(colorScale.getColor(i));
        }
        fireColorScaleChanged();
    }

    private ArrayList<ColorScaleListener> listenerList = new ArrayList<ColorScaleListener>();

    public void addColorScaleListener(ColorScaleListener l) {
        listenerList.add(l);
    }

    public void removeColorScaleListener(ColorScaleListener l) {
        listenerList.remove(l);
    }

    protected void fireColorScaleChanged() {
       int i = 0;
        for (ColorScaleListener l : listenerList) {
	    Debug.message("Colorizer", "##ColorScalePanel.fireColorScaleChanged " + (i++) + " => " + l);
            l.colorScaleChanged(this);
        }
    }

    public void setColorNumberModel(SpinnerModel spm) {
	try {
	    colorNumber.setModel(spm);
	    int nb = Integer.valueOf(spm.getValue().toString());
	    colorScale.setColorNumber(nb);
	    refreshPanel();
	} catch(Exception e) {
	    Debug.error("*** ColorScalePanel.setColorNumber: " + e.getMessage());
	}
    }

    public void updateColorNumberModel(SpinnerModel spm){
        colorNumber.setModel(spm);
        colorNumber.updateUI();
    }

    public JComponent getPanOption() {
        return panOption;
    }

    @Override
    public void setEnabled(boolean b){
        super.setEnabled(b);
        colorNumber.setEnabled(b);
        predefinedColorMapsCB.setEnabled(b);
        shadedCheckBox.setEnabled(b);
        checkReverseColorMap.setEnabled(b);
        for(JButton button : colorButtons)
            button.setEnabled(b);

    }

    public void setEnabledColorNumber(boolean b) {
	colorNumber.setEnabled(b);
    }

    public void setNumberLabel(String label){
        numberLabel.setText(label);
    }
}
