// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.font.FontRenderContext;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import javax.swing.event.*;
import java.util.*;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.util.*;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.util.classification.*;
import fr.unicaen.openmap.util.*;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;
import fr.unicaen.openmap.util.gui.ComponentSizer;

/**
 * 
 */
public class ColorizerPanel extends JPanel implements ClassificationListener
{
    protected Colorizer colorizer;

    //    protected HistogramPanel histo; no histogram available

    protected ThresholdColorScalePanel colorScalePanel;

    /**
     * Use I18N mecanism to allow internationalization.
     * Use I18N.properties file or I18N_fr_FR.properties for french version
     */
    protected static I18n i18n = new BasicI18n();

    /** Index of Custom in classifNameList */
    protected final int CUSTOM_INDEX = 0;
    /** Default classif index (here Quantile) */
    protected final int DEFAULT_CLASSIF_INDEX = 0;
    
    /** Classification name method to display in a combo box */
    public final static String classifNameList[] = {
	i18n.get(ColorizerPanel.class, "custom", "Custom"),
	i18n.get(ColorizerPanel.class, "quantile", "Quantile"), 
	i18n.get(ColorizerPanel.class, "nestedMeans", "Nested Means"), 
	i18n.get(ColorizerPanel.class, "fixedStep", "Fixed Step Classification"), 
	i18n.get(ColorizerPanel.class, "standardization", "Standardization"),
	i18n.get(ColorizerPanel.class, "boxplot", "Box Plot"),
	i18n.get(ColorizerPanel.class, "discrete", "Discrete"),
	i18n.get(ColorizerPanel.class, "discreteNumeric", "Discrete of numbers"),
	i18n.get(ColorizerPanel.class, "continuous", "Continuous"),
    };
    /** Classification class names. Must be strictly parrallel to classifNameList. */
    public final static String classifClassNameList[] = {
	"fr.unicaen.openmap.util.classification.Customized",
	"fr.unicaen.openmap.util.classification.Quantile", 
	"fr.unicaen.openmap.util.classification.NestedMeans",
	"fr.unicaen.openmap.util.classification.StepClassification",
	"fr.unicaen.openmap.util.classification.Standardization",
	"fr.unicaen.openmap.util.classification.BoxPlot",
	"fr.unicaen.openmap.util.classification.Discrete",
	"fr.unicaen.openmap.util.classification.DiscreteOfDouble",
	"fr.unicaen.openmap.util.classification.ContinuousClassification",
    };

    /** Customized classification class names used to select the index 0 "custom". */
    public final static String customizedClassifClassNameList[] = {
	"fr.unicaen.openmap.util.classification.CustomizedClassificationOfDouble",
	"fr.unicaen.openmap.util.classification.CustomizedClassificationOfString",
    };

    private JComboBox classifList = new JComboBox(classifNameList);    

    public ColorizerPanel(Colorizer _colorizer) {
	super();
        setLayout(new BorderLayout());
	this.colorizer = _colorizer;
	colorizer.addClassificationListener(this);
	JPanel classifPanel = new JPanel(new BorderLayout());
        ComponentSizer.fixComponentSize(classifList, new Dimension(120,20));
	classifList.addItemListener(new ItemListener() {
		public void itemStateChanged(ItemEvent e) {
		    if (classifListIsInhibited()) { // see classificationChanged()
			return;
		    }
		    int index = ((JComboBox)e.getSource()).getSelectedIndex();
		    // System.out.println("===== classifList select " + index);
		    colorizer.resetClassification(classifClassNameList[index]);
		    colorScalePanel.setColorNumberModel(colorizer.getClassification().getClassificationNumberModel());
		}
	    });

        colorScalePanel = new ThresholdColorScalePanel(colorizer.getColorScale(), colorizer);
	colorScalePanel.addColorScaleListener(colorizer);
        JPanel pClassif = new JPanel();

        pClassif.add(classifList);
	classifPanel.add(colorScalePanel);
	Debug.message("ThematicDataDisplayLayer", "[ColorizerPanel] ColorizerPanel: " + colorizer);
        JPanel p = new JPanel();
        p.setBorder(BorderFactory.createEmptyBorder(-10,0,0,0));
        p.add(classifPanel);
        add(pClassif, BorderLayout.NORTH);
        add(p,BorderLayout.CENTER);
	classificationChanged(this); 
    }

    public JComboBox getClassifList() {
        return classifList;
    }

    public void setEnabled(boolean b) {
        super.setEnabled(b);
        classifList.setEnabled(b);
        colorScalePanel.setEnabled(b);

    }

    private boolean _inhibitClassifListItemStateChanged = false;
    synchronized boolean classifListIsInhibited() {
	boolean ret = _inhibitClassifListItemStateChanged;
	// System.out.println("#### classifListIsInhibited " + ret);
	return ret;
    }
    synchronized void classifListInhibited(boolean b) {
	_inhibitClassifListItemStateChanged = b;
	// System.out.println("==== classifListIsInhibited " + b);
    }
    

    public void classificationChanged(Object source) {
	classifListInhibited(true); // in order to avoid an infinite loop...
	classifList.setSelectedIndex(getIndexClassifClassName());
	if (colorizer == null)
	    return;
	// reset ColorScale color number
	Classification classif = colorizer.getClassification();
	if (classif == null)
	    return;
	colorizer.getColorScale().setColorNumber(classif.getNumberOfClasses());
	if (colorScalePanel == null)
	    return;
	colorScalePanel.updateColorNumberModel(classif.getClassificationNumberModel());
	// disable color number spinner for Discrete classif and allow it always for Customized
	colorScalePanel.setEnabledColorNumber(! (classif instanceof DiscreteClassification) || classif instanceof CustomizedClassification);
	classifListInhibited(false);
    }

    protected int getIndexClassifClassName() {
	if (colorizer == null || colorizer.getClassification() == null){
	    return DEFAULT_CLASSIF_INDEX; //default : Quantile
	}
	String classifClassName = colorizer.getClassification().getClass().getName();
	for (int i = 0 ; i < classifClassNameList.length; i++)
	    if (classifClassNameList[i].equals(classifClassName))
		return i;
	
	for (String s : customizedClassifClassNameList)
	    if (s.equals(classifClassName))
		return 0;

	// fall back
	return -1;
    }
    
    public  boolean hasForColorScalePanel(ColorScalePanel colorScalePanel) {
	return colorScalePanel == colorScalePanel;
    }

}
