// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.awt.event.ItemEvent;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;

import fr.unicaen.openmap.dataStorage.DataStorageIterator;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.ArrowHead;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMFlow;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Colorizer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.MutableString;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Area;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Flow;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.event.ItemListener;
import javax.swing.JScrollPane;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;

/**
 * Flow visualization and management
 * 
 * @author Jerome Chauveau
 */
public class FlowPanel extends JPanel {

    private JPanel panFromTo, panStyle;
    private FlowOverviewPanel ovPanel;
    private JComboBox comboToCol, comboSort;
    private Flow flow;
    protected I18n i18n = new BasicI18n();
    static final int TYPE_START = 0;
    static final int TYPE_END = 1;
    private JCheckBox checkVisibility;
    private Box toBox;

    public FlowPanel(final Flow flow, final DataStorageIterator dsi,
		     int startModel, int endModel, int axis, ThematicGUI tgui) {
        this.flow = flow;
        setLayout(new BoxLayout(this, axis));

        JPanel panType = new JPanel();

        final SpinnerNumberModel model =
                new SpinnerNumberModel(flow.getNbStep(), //initial value
                0, //min
                100, //max
                1);//step

        JSpinner stepSpinner = new JSpinner(model);
        panType.add(new JLabel(i18n.get(this, "numberOfSteps", "Number of steps")));
        panType.add(stepSpinner);
        stepSpinner.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent arg0) {
                int val = model.getNumber().intValue();
                flow.setNbStep(val);
                refreshToBox(dsi);
                flow.setChosenType(val == 0 ? Flow.type.PATH : Flow.type.FLOW);
            }
        });

        //panel for FROM/TO Columns choice
        panFromTo = new JPanel();

        toBox = Box.createVerticalBox();
        refreshToBox(dsi);

        //panFromTo.setBorder(BorderFactory.createTitledBorder(i18n.get(this, "from_to", "From/To Columns")));
        panFromTo.add(toBox);

        // loops when to equals from
        final JCheckBox checkShowStartEqualsEnd = new JCheckBox(i18n.get(this, "showTheLoops", "Show the loops"));
        final JSpinner spinnerDepArrRadius = new JSpinner();
        checkShowStartEqualsEnd.setSelected(flow.isShowStartEqualsEnd());
        spinnerDepArrRadius.setEnabled(flow.isShowStartEqualsEnd());
        spinnerDepArrRadius.setValue(flow.getLoopRadius());
        checkShowStartEqualsEnd.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent arg0) {
                flow.setShowStartEqualsEnd(checkShowStartEqualsEnd.isSelected());
                spinnerDepArrRadius.setEnabled(checkShowStartEqualsEnd.isSelected());
		flow.refresh();
            }
        });
        spinnerDepArrRadius.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent arg0) {
                flow.setLoopRadius(Integer.parseInt(spinnerDepArrRadius.getValue().toString()));
            }
        });

        JPanel panSort = new JPanel();
        comboSort = new JComboBox(dsi.getDataColumnNames());
        panSort.add(comboSort);
        ComponentSizer.fixComponentSize(comboSort, new Dimension(150, 25));

        panStyle = new JPanel();
        Box leftStyleBox = Box.createVerticalBox();
        Box arrowBox = Box.createHorizontalBox();
        panStyle.setBorder(BorderFactory.createTitledBorder(""));
        JPanel pModels = new JPanel(new GridLayout(2, 1));
        Box startBox = Box.createHorizontalBox();
        Box endBox = Box.createHorizontalBox();
        pModels.add(startBox);
        pModels.add(endBox);
        arrowBox.add(pModels);
        startBox.add(new JLabel(i18n.get(this, "start_arrow", "Start")));
        startBox.add(comboBoxForArrowHeadModelChoice(startModel, TYPE_START));
        endBox.add(new JLabel(i18n.get(this, "end_arrow", "End")));
        endBox.add(comboBoxForArrowHeadModelChoice(endModel, TYPE_END));

        //overview panel
        ovPanel = new FlowOverviewPanel();
        ovPanel.setMaximumSize(new Dimension(120, 30));
        ovPanel.setPreferredSize(new Dimension(120, 30));
        ovPanel.setMinimumSize(new Dimension(120, 30));
        arrowBox.add(ovPanel);
        leftStyleBox.add(arrowBox);

        ButtonGroup bGroup = new ButtonGroup();

        final JRadioButton radioManual = new JRadioButton();
        bGroup.add(radioManual);
        radioManual.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (radioManual.isSelected()) {
                    flow.setControlPointsVisibility(true);
                    OMFlow.BENDED_MODE = OMFlow.BENDED_MANUAL_MODE;
                    flow.setBadCache(true);
                    checkVisibility.setEnabled(true);
                    checkVisibility.setSelected(true);
		    flow.refresh();
                }

            }
        });
        radioManual.setSelected(OMFlow.BENDED_MODE == OMFlow.BENDED_MANUAL_MODE);
        JPanel pl2 = new JPanel(new BorderLayout());
        pl2.add(radioManual, BorderLayout.WEST);
        pl2.add(new JLabel(i18n.get(this, "manual", "manual")));
        checkVisibility = new JCheckBox(i18n.get(this, "handles", "handles"));
        checkVisibility.setSelected(flow.isControlPointsVisibility());
        checkVisibility.addActionListener(new ActionListener() {
		
		public void actionPerformed(ActionEvent arg0) {
		    flow.setControlPointsVisibility(checkVisibility.isSelected());
		    flow.refresh();
		}
	    });
        pl2.add(checkVisibility, BorderLayout.EAST);
        leftStyleBox.add(pl2);
	
        JPanel pl1 = new JPanel();
        pl1.setLayout(new BorderLayout());
        final JRadioButton radioAuto = new JRadioButton();

        radioAuto.addActionListener(new ActionListener() {

		public void actionPerformed(ActionEvent e) {
		    if (radioAuto.isSelected()) {
			flow.setControlPointsVisibility(false);
			OMFlow.BENDED_MODE = OMFlow.BENDED_AUTO_MODE;
			flow.setBadCache(true);
			checkVisibility.setEnabled(false);
			flow.refresh();
		    }
		    
		}
	    });
        radioAuto.setSelected(OMFlow.BENDED_MODE == OMFlow.BENDED_AUTO_MODE);
        checkVisibility.setEnabled(OMFlow.BENDED_MODE == OMFlow.BENDED_MANUAL_MODE);
        bGroup.add(radioAuto);
        pl1.add(radioAuto, BorderLayout.WEST);
        pl1.add(new JLabel(i18n.get(this, "bended", "Bended size")));
        pl1.add(bendedSpinner(), BorderLayout.EAST);
        leftStyleBox.add(pl1);

        JPanel pl3 = new JPanel();
        pl3.add(new JLabel(i18n.get(this, "line_arc", "Arcs length")));
        pl3.add(nbLineSpinner());
        leftStyleBox.add(pl3);

        Box wrapType = Box.createVerticalBox();
        wrapType.add(panType);
        wrapType.add(panFromTo);
        JPanel panLoopsAndRadius = new JPanel();
        wrapType.add(panLoopsAndRadius);
        panLoopsAndRadius.add(checkShowStartEqualsEnd);
	panLoopsAndRadius.add(new JLabel(i18n.get(this, "radius", " - Radius")));
        panLoopsAndRadius.add(spinnerDepArrRadius);
        ThematicConfPanel confType = ThematicConfPanel.createThematicConfPanel(i18n.get(this, "type", "Representation type"), false);
        JPanel wrapPanType = new JPanel();
        wrapPanType.add(confType);
        confType.setContent(wrapType);
        tgui.addThematicConfigurationPanel(confType);
        add(wrapPanType);

        panStyle.add(leftStyleBox);
        wrapType.setBorder(BorderFactory.createEmptyBorder(0, 25, 0, 0));

        ThematicConfPanel vt = ThematicConfPanel.createThematicConfPanel(i18n.get(this, "arrow_style", "Arrows' Style"));
        vt.setContent(new JScrollPane(panStyle));
	vt.collapse();
        JPanel wrapStyle = new JPanel();
        wrapStyle.add(vt);
        add(wrapStyle);
        tgui.addThematicConfigurationPanel(vt);
        ThematicConfPanel panResizer = (ThematicConfPanel) flow.getResizer().getGUI(dsi);
        JPanel wrapSizer = new JPanel();
        wrapSizer.add(panResizer);
        add(wrapSizer);
        tgui.addThematicConfigurationPanel(panResizer);

	LabelizerGUI labelizerGUI = flow.getLabelizer().getLabelizerGUI(dsi);

	ThematicConfPanel labelConfPan;
	labelConfPan = ThematicConfPanel.createThematicConfPanel(
							    i18n.get(this, "labelTextAndFont", "Label Text & Font"),
							    labelizerGUI.getTextAndFontPanel(), tgui);
	labelConfPan.collapse();
	labelConfPan = ThematicConfPanel.createThematicConfPanel(
								 i18n.get(this, "LabelPositions", "Label Position"),
								 labelizerGUI.getOffseterPanel(), tgui);
 	labelConfPan.collapse();
   }

    private void refreshToBox(DataStorageIterator dsi) {
        toBox.removeAll();
        
        for (int i = 0; i < flow.getNbStep(); i++) {
            final int index = i+1;
            JComboBox cb = new JComboBox(dsi.getOMColumnNames());
            String stepCol = flow.getStepColumnAt(index);
            if(stepCol != null) {
                cb.setSelectedItem(stepCol);
	    } else {
		// flow.setStepColumn(index, cb.getSelectedItem().toString());	
		flow.setStepColumn(index, dsi.getOMColumnNames()[0]);
	    }
            
            toBox.add(cb);
            cb.addItemListener(new ItemListener() {
		    public void itemStateChanged(ItemEvent ie) {
			flow.setStepColumn(index,((JComboBox)ie.getSource()).getSelectedItem().toString());
			flow.refresh();
		    }
		});
	    // System.out.println("*** flow.getStepColumnAt("+index+") = <"+flow.getStepColumnAt(index)+">");
        }
        panFromTo.updateUI();
    } 

    public void refreshGUI(DataStorageIterator dsi) {
        if (!flow.isDragging()) {
            refreshToBox(dsi);
        }
    }

    /**
     * Build combo box for an arrow model choice
     *
     * @param selectedName
     * @param type
     * @param typeName
     * @return
     */
    private JComboBox comboBoxForArrowHeadModelChoice(
						      final int selectedIndex, final int type) {

        final JComboBox cb = new JComboBox();

        for (ArrowHead.ArrowHeadType arrowHeadType : ArrowHead.ArrowHeadType.values()) {
            cb.addItem(i18n.get(this, arrowHeadType.getName(), arrowHeadType.getName()));
        }
        final Flow myFlow = this.flow;
        ItemListener listener = new ItemListener() {

            public void itemStateChanged(ItemEvent arg0) {
                if (type == TYPE_START) {
                    myFlow.setStartArrowModel(cb.getSelectedIndex());
                } else {
                    myFlow.setEndArrowModel(cb.getSelectedIndex());
                }
                setOverviewIcon();

                flow.setBadCache(true);
            }
        };
        cb.setSelectedIndex(selectedIndex);
        cb.addItemListener(listener);
        return cb;
    }

    /**
     * build the bended value JSpiner
     *
     * @return
     */
    private JSpinner bendedSpinner() {
        // JPanel panSpinner = new JPanel();
        JSpinner spin = new JSpinner();
        //final Flow myFlow = flow;
        spin.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent event) {
                String val = (((JSpinner) event.getSource()).getValue()).toString();
                OMFlow.BENDED_DISTANCE = Integer.parseInt(val);
		//                flow.setBadCache(true);
		flow.refresh();
            }
        });
        JComponent editor = spin.getEditor();
        ((JSpinner.DefaultEditor) editor).getTextField().setColumns(5);
        spin.setValue(OMFlow.BENDED_DISTANCE);
        return spin;
    }

    /**
     * build the number of line per arc JSpiner
     *
     * @return
     */
    private JSpinner nbLineSpinner() {
        JSpinner spin = new JSpinner();
        spin.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent event) {
                String val = (((JSpinner) event.getSource()).getValue()).toString();
                OMFlow.ARC_LENGTH = Integer.parseInt(val);
                flow.setBadCache(true);
            }
        });
        JComponent editor = spin.getEditor();
        ((JSpinner.DefaultEditor) editor).getTextField().setColumns(5);
        spin.setValue(OMFlow.ARC_LENGTH);
        return spin;

    }

    /**
     * set the overview arrow model icon
     */
    private void setOverviewIcon() {
        ovPanel.repaint();
    }

    class FlowOverviewPanel extends JPanel {

        public void paintComponent(Graphics g) {
            g.clearRect(0, 0, getWidth(), getHeight());
            super.paintComponent(g);

            OMFlow sample = new OMFlow(flow,
                    new Point(getWidth() / 2 - 30, getHeight() / 2),
                    new Point(getWidth() / 2 + 30, getHeight() / 2),
                    ArrowHead.getModel(flow.getStartArrowModel()),
                    ArrowHead.getModel(flow.getEndArrowModel()),
                    5);
            if (flow.getDrawingAttributes().getFillPaint() != null) {
                g.setColor((Color) flow.getDrawingAttributes().getFillPaint());
                g.fillPolygon(sample.getXBorderAsInt(), sample.getYBorderAsInt(),
                        sample.getXBorder().length);
            }
            g.setColor((Color) flow.getDrawingAttributes().getLinePaint());
            g.drawPolygon(sample.getXBorderAsInt(), sample.getYBorderAsInt(),
                    sample.getXBorder().length);

        }
    }
}
