// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;


import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;

import com.bbn.openmap.I18n;
import com.bbn.openmap.Environment;
import com.bbn.openmap.Layer;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.util.PropUtils;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfigurationPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.FontSizer;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.FontDialog;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.GridLayout;
import javax.swing.JComponent;
import org.jdesktop.swingx.JXCollapsiblePane;

/**
 * Panel to manage a FontSizer.
 * <ul>
 * <li>a FontDialog to select font</li>
 * <li>a button to show the FontDialog and the selected font</li>
 * </ul>
 * Sample in fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Label.
 * @see fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.FontSizer
 * @see fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Label
 * Deprecated at the moment : should remove fontButton and leave only Size management.
 */
public class FontSizerPanel extends JPanel {
    /**
     * Internationalization of GUI.
     */
    public final static String TITLE_PROPERTY_NAME = "title";
    /**
     * Internationalization of GUI.
     */
    public final static String ADD_PROPERTY_NAME = "add";
    /**
     * Internationalization of GUI.
     */
    public final static String DELETE_PROPERTY_NAME = "delete";
    /**
     * Use I18N mecanism to allow internationalization.
     */
    protected I18n i18n = Environment.getI18n();
    /**
     * FontSizer associated with this panel.
     */
    protected FontSizer fontSizer;
    /**
     * Use to select a Font.
     */
    protected FontDialog fontDialog;

    /**
     * Able to show the FontDialog.
     */
    protected JButton fontButton;

    Box fontPanel;

    /**
     * contains font and add buttons
     */
    private JPanel buttonsPanel;

    private JButton addButton;

    /**
     * @param fontsizer FontSizer used for this FontSizerPanel
     */
    public FontSizerPanel(FontSizer fontSizer) {
	this(fontSizer, false);
    }
    /**
     * @param fontsizer FontSizer used for this FontSizerPanel
     */
    public FontSizerPanel(FontSizer fontSizer, boolean useFontDialog) {
        super();
        fontPanel = Box.createVerticalBox();
        this.fontSizer = fontSizer;

        buttonsPanel = new JPanel();

	if (useFontDialog) {
	    fontDialog = new FontDialog(getFontSizer().getFont(), null, 0, 0.0);
	    
	    fontButton = new JButton(i18n.get(FontSizerPanel.class,
					      FontSizer.FONT_PROPERTY_NAME,
					      "Select font"));
	    fontButton.setToolTipText(i18n.get(FontSizerPanel.class,
					       FontSizer.FONT_PROPERTY_NAME,
					       I18n.TOOLTIP,
					       "Select and configure a font. Current : {0}",
					       getFontSizer().getFont().getName()));
	    fontButton.addActionListener(new ActionListener() {
		    
		    public void actionPerformed(ActionEvent ae) {
			fontDialog.setVisible(true);
			
			if (!fontDialog.isCanceled()) {
			    getFontSizer().setFont(fontDialog.getSelectedFont());
			    refreshFontButton();
			    fireFontSizerChanged();
			}
		    }
		});
	    buttonsPanel.add(fontButton);
	}

        addButton = new JButton(i18n.get(FontSizerPanel.class, ADD_PROPERTY_NAME, "Add scale/size"));
        addButton.setToolTipText(i18n.get(FontSizerPanel.class, ADD_PROPERTY_NAME, I18n.TOOLTIP, "Add a new scale/size."));
        addButton.addActionListener(new ActionListener() {

		public void actionPerformed(ActionEvent event) {
		    getFontSizer().addScale();
		    initFontPanel();
		    updateUI();
		}
	    });

        buttonsPanel.add(addButton);

        // fontPanel.setBorder(BorderFactory.createTitledBorder(""));

        initFontPanel();
        add(fontPanel);
    }

    public JComponent getFontPanel() {
        return fontPanel;
    }

    /**
     * Refresh fontButton with values from fontSizer.
     */
    public void refreshFontButton() {
        fontDialog.selectFont(getFontSizer().getFont());

        //bonus :)
        fontButton.setFont(getFontSizer().getFont());
        fontButton.setText(getFontSizer().getFont().getName());
        fontButton.setToolTipText(i18n.get(FontSizerPanel.class,
					   FontSizer.FONT_PROPERTY_NAME,
					   I18n.TOOLTIP,
					   "Select and configure a font. Current : {0}",
					   getFontSizer().getFont().getName()));
    }

    /**
     * Get the FontSizer with an appropriate font size if any.
     * @return current FontSizer
     */
    public FontSizer getFontSizer() {
        return fontSizer;
    }

    /**
     * Create... options panel. <br/>
     * Button to choose the default font. <br/>
     * Spinners to manage scales and associated sizes <br/>
     */
    private void initFontPanel() {
	initFontPanel(true);
    }

    private void initFontPanel(boolean enabled) {
        fontPanel.removeAll(); //mouais

        fontPanel.add(buttonsPanel);
	int scalesSize = getFontSizer().getScalesSize();

        JPanel panTab = new JPanel(new GridLayout(scalesSize + 1, 3));
        panTab.add(new JLabel(i18n.get(FontSizerPanel.class,
				       FontSizer.SCALE_PROPERTY_NAME,
				       "Until scale : ")));
        panTab.add(new JLabel(i18n.get(FontSizerPanel.class,
				       FontSizer.FONT_SIZE_PROPERTY_NAME,
				       "Font size : ")));
        panTab.add(new JPanel());
        //Scales & sizes settings
        for (int i = 0; i < scalesSize; i++) {

            final int index = i;
            JPanel pan = new JPanel();
            Dimension dim = new Dimension(100, 20);
            final JSpinner scaleSpinner = new JSpinner(new SpinnerNumberModel(getFontSizer().getScale(i).getScale(), 1d, 200000000d, 50000d));
            scaleSpinner.setMaximumSize(dim);
            scaleSpinner.setPreferredSize(dim);
            scaleSpinner.addChangeListener(new ChangeListener() {

		    public void stateChanged(ChangeEvent ce) {
			getFontSizer().getScale(index).setScale(((Double) scaleSpinner.getValue()).floatValue());
			fireFontSizerChanged();
			initFontPanel();
			updateUI();
		    }
		});
            scaleSpinner.setToolTipText(i18n.get(FontSizerPanel.class,
						 FontSizer.SCALE_PROPERTY_NAME,
						 I18n.TOOLTIP,
						 "The scale used."));
	    scaleSpinner.setEnabled(enabled);

	     //note: allow size = 0 => No display :p
            final JSpinner sizeSpinner = new JSpinner(new SpinnerNumberModel(getFontSizer().getScale(i).getSize(), 0, 100, 1));
            sizeSpinner.setMaximumSize(dim);
            sizeSpinner.setPreferredSize(dim);
            sizeSpinner.addChangeListener(new ChangeListener() {

		    public void stateChanged(ChangeEvent ce) {
			getFontSizer().getScale(index).setSize(((Integer) sizeSpinner.getValue()).intValue());
			fireFontSizerChanged();
		    }
		});
            sizeSpinner.setToolTipText(i18n.get(FontSizerPanel.class,
						FontSizer.FONT_SIZE_PROPERTY_NAME,
						I18n.TOOLTIP,
						"The font size used."));
	    sizeSpinner.setEnabled(enabled);

            //Remove this ScaleAndSize
            JButton killer = new JButton(Util.loadIcon("cancel2.png"));
            killer.setBorder(null);
            killer.setBorderPainted(false);
            killer.setFocusPainted(false);
            killer.setBackground(Color.WHITE);
            killer.setOpaque(false);
	    ComponentSizer.fixComponentSize(killer, new Dimension(20, 20));
            killer.setToolTipText(i18n.get(FontSizerPanel.class,
					   DELETE_PROPERTY_NAME,
					   I18n.TOOLTIP,
					   "Delete scale/size."));
            killer.addActionListener(new ActionListener() {

		    public void actionPerformed(ActionEvent event) {
			getFontSizer().removeScale(index);
			fireFontSizerChanged();
			initFontPanel();
			updateUI();
		    }
		});
	    killer.setEnabled(enabled);

            panTab.add(scaleSpinner);
            panTab.add(sizeSpinner);
            panTab.add(killer);
        }

        JPanel panTabWrap = new JPanel();
        panTabWrap.add(panTab);
        fontPanel.add(panTabWrap);
    }

    public void fireFontSizerChanged() {
	getFontSizer().fireFontSizerChanged();
    }

    public void setEnabled(boolean b) {
	if (addButton != null)
	    addButton.setEnabled(b);
	initFontPanel(b);
    }

}
