// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.HistogramDrawingAttributes;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.AttributesNames;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ColorScale;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Histogram;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

/**
 *
 * @author jchauvea
 */
public class HistogramColorScalePanel extends AttributesColorScalePanel {

    private Histogram histogram;
    private JPanel axesPanel;

    public HistogramColorScalePanel(Histogram histo, ColorScale colorScale, AttributesNames attributesNames) {
        super(colorScale, attributesNames);
        this.histogram = histo;
        JPanel topPan = new JPanel();
        JRadioButton bBar = new JRadioButton(i18n.get(Histogram.class, "histogram", "Bars"));
        JRadioButton bCurve = new JRadioButton(i18n.get(Histogram.class, "curves", "Curves"));
        ButtonGroup bg = new ButtonGroup();
        bg.add(bBar);
        bg.add(bCurve);


        if (histo.isBars()) {
            bBar.setSelected(true);
        } else {
            bCurve.setSelected(true);
        }

        final JComboBox orientationBox = new JComboBox(Histogram.Orientation.values());
        orientationBox.setSelectedItem(histo.getOrientation());
        orientationBox.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent evt) {
                JComboBox cb = (JComboBox) evt.getSource();
                histogram.setOrientation((Histogram.Orientation) cb.getSelectedItem());
            }
        });

        bBar.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                histogram.setBars(((JRadioButton) e.getSource()).isSelected());
                orientationBox.setEnabled(histogram.isBars());
                createColorScalePanel();
                updateOptionPanel();
                histogram.refresh();
            }
        });

        topPan.add(bBar);
        topPan.add(bCurve);
        topPan.add(orientationBox);

        add(topPan, BorderLayout.NORTH);

        axesPanel = (JPanel)((HistogramDrawingAttributes)histogram.getDrawingAttributes()).getAxesPanel();
        add(axesPanel, BorderLayout.SOUTH);
        axesPanel.setVisible(!histo.isBars());

    }

    public void updateOptionPanel() {
        if (histogram != null) {
            boolean v = histogram.isBars();
            colorLabel.setVisible(v);
            predefinedColorMapsCB.setVisible(v);
            checkReverseColorMap.setVisible(v);
            shadedCheckBox.setVisible(v);
            axesPanel.setVisible(!v);
        }
    }

    @Override
    public void createColorScalePanel() {

        if (panColorScale == null) {
            panColorScale = new JPanel(new BorderLayout());
        }
        panColorScale.removeAll();
        Box pColors = Box.createVerticalBox();
        colorCombos = new JComboBox[colorScale.getColorNumber()];
        colorButtons.clear();
        if (available == null) {
            available = new Vector<String>();
        }
        Dimension buttonsDimension = new Dimension(20, 20);
        JPanel pTop = new JPanel();
        pTop.add(new JLabel(i18n.get(AttributesColorScalePanel.class, "attributes", "Attributes to represent")));
        panColorScale.add(pTop, BorderLayout.NORTH);
        //System.out.println(attributesNames.getName(0));
        for (int i = 0; i < colorScale.getColorNumber(); i++) {
            // System.out.println(i);
            // System.out.println("a-"+attributesNames.getName(i));
            ColorScaleButton b = new ColorScaleButton(this, i);
            colorButtons.add(b);
            ComponentSizer.fixComponentSize(b, buttonsDimension);
            b.setBackground(colorScale.getColor(i));
            colorCombos[i] = new JComboBox();

            for (int j = 0; j < available.size(); j++) {
                colorCombos[i].addItem(available.get(j));
            }

            if (!attributesNames.getName(i).equals("")) {
                colorCombos[i].addItem(attributesNames.getName(i));
                colorCombos[i].setSelectedItem(attributesNames.getName(i));
            }

            ComponentSizer.fixComponentSize(colorCombos[i], new Dimension(100, 20));
            ItemListener itml = new ItemListener() {

                public void itemStateChanged(ItemEvent ie) {
                    refresh();
                }
            };

            colorCombos[i].addItemListener(itml);
            JPanel p = new JPanel();
            if (histogram == null || histogram.isBars()) {
                p.add(b);
            }
            p.add(colorCombos[i]);
            p.setBorder(BorderFactory.createEmptyBorder(-5, 0, -5, 0));
            pColors.add(p);
        }
        panColorScale.add(pColors, BorderLayout.CENTER);
        updateUI();

    }
}
