// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package  fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Rectangle2D;
import javax.swing.*;
import javax.swing.border.*; 
import fr.unicaen.openmap.util.Colors;
import fr.unicaen.openmap.util.ColorTable;
import fr.unicaen.openmap.util.Colors.ColorType;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.classification.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import com.bbn.openmap.omGraphics.OMColorChooser;
import com.bbn.openmap.util.Debug;

import java.util.Locale;
import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;
import fr.unicaen.openmap.gui.ThemaMapColorChooser;

/**
 * Histogram visualization and management
 *
 * @author Yannick Lange Universite de Caen
 * adapted by Jacques Madelaine
 */
public class HistogramPanel extends JPanel implements ClassificationListener {
    public void classificationChanged(Object source) {
	repaint();
    }
    
    
    protected Colorizer colorizer; 
    protected int height; // = 500;
    protected int width;// = 450;
    protected int x0;// origin of the bars, also used in the MouseListener
    protected int y0;// origin of the bars
    protected int[] xticks = new int[0]; 	// xticks stores the x coodinates of the X lable ticks


    protected String stringNumberFormat = "%5.2f";

    /**
     * Use I18N mecanism to allow internationalization.
     * Use I18N.properties file or I18N_fr_FR.properties for french version
     */
    protected I18n i18n = new BasicI18n();

    /**
     * 
     */
    public HistogramPanel(Colorizer colorizer, int width, int height) {
	super();
	this.colorizer = colorizer;
	this.height = height;
	this.width = width;

	colorizer.addClassificationListener(this);

	//	setBorder(BorderFactory.createLineBorder(Color.red));
	//adjustSize();
    setPreferredSize(new Dimension(width, height));

	// // <mouselister>
	// MyMouseListener myMouseListener = new MyMouseListener();
	// addMouseListener(myMouseListener);
	// addMouseMotionListener(myMouseListener);
	// // </mouselistener>
    }

    /**
     * Draw the histogram
     */
    public void paintComponent(Graphics g) {
	
	Debug.message("Histogram", "###> HistogramPanel.paint() "+colorizer);
	
	// wipe the panel
	g.setColor(Color.black);
	g.clearRect(0, 0, width, height);

	// if no classif, no data => return
	if (colorizer.getClassification() == null || colorizer.getClassification().hasNoData()) {
	    Debug.message("Histogram", "###< HistogramPanel.paint() no data");
	    return;
	}

	if (colorizer.getClassification() instanceof ClassificationOfString) {
	    Debug.message("Histogram", "###< HistogramPanel.paint() no histo for ClassificationOfString");
	    g.drawString("No histo for this type of classification", 10, 10);
	    return; // no Histo yet for ClassificationOfString
	}
	if (! (colorizer.getClassification() instanceof ClassificationOfDouble))
	    return;
	ClassificationOfDouble classif = (ClassificationOfDouble) colorizer.getClassification();
	x0 = 0;
	y0 = 0;
	
	// this fixedClassif is used to draw the histogram
	ClassificationOfDouble fixedClassif = new StepClassification(colorizer.getClassification(), 60);
	// System.out.println("** "+fixedClassif.dump());
	int max = Util.getMax(fixedClassif.getCounts());

	// idea build a StepClassification to obtain a scale for the Y axis
	//	StepClassification(new Array<double>( ...

	int[] fixedCounts = fixedClassif.getCounts();
	double[] fixedBreaks = fixedClassif.getBreaks();
	int[] counts = classif.getCounts();
	double[] breaks = classif.getBreaks();
	ColorScale colors = colorizer.getColorScale();

	// compute text size, in order to compute the lower left histogram corner coordinates
	Rectangle2D r = Util.getStringBounds("X"+max+" ", g);
	x0 += r.getWidth(); // keep space for the Y axis labels
	y0 = height;
        y0 -= 4*r.getHeight(); // keep space for the x axis labels and room on top

	// *** x0 represents breaks[0] from the StepClassification ***
	
	// 2*r.getHeight() is kept blank on top
	double heightStep = (y0 - 2*r.getHeight()) / max;
	// keep room for the left most X axis label
	double widthStep = (width - x0 - Util.getStringBounds(""+breaks[breaks.length-1], g).getWidth()) 
	    / (fixedBreaks[fixedBreaks.length-1] - fixedBreaks[0]); 
	//	System.out.println("x0= " + x0 + " y0= " + y0 + " widthStep= " + widthStep + " heightStep= " + heightStep+ " max= " + max + " maxX " + fixedBreaks[fixedBreaks.length-1] +" minX "+ fixedBreaks[0]);
	// to find the x corresponding to value use :
	// x = x0+(int)(widthStep*(value-fixedBreaks[0]))

	// xticks stores the x coodinates of the X lable ticks
	xticks = new int[breaks.length];

	// Draw the X axis labels
	//
	// draw ticks and labels of the desired classif on the X axis 
	// on two rows hopping there will be no overlap
	//	       |            |              |
	//	   breaks[i-1]      |          breaks[i+1]
	//                       breaks[i]
	//
	// write the counts of ecah class in a class colored filled rectangle
	for (int i = 0; i < breaks.length; i++) {
	    int x = x0+(int)(widthStep*(breaks[i]-fixedBreaks[0]));
	    xticks[i] = x;
	    String s = String.format(stringNumberFormat, breaks[i]);
	    r = Util.getStringBounds(s, g);
	    g.setColor(Color.black);
	    // draw the mark
	    g.drawLine(x, y0, x, y0+(int)r.getHeight()*(1+i%2));
	    // draw the label centered on the mark
	    g.drawString(s,  x - (int)(r.getWidth()/2), y0 + (2+i%2)*(int)r.getHeight());
	    // System.out.println("HistogramPanel:draw tick for "+breaks[i]+" at x="+x);
	    // write the class count in a filled rectangle
	    if(i < breaks.length - 1) {
		int barWidth = (int)(widthStep * (breaks[i+1] - breaks[i]));
		s = ""+counts[i];
		r = Util.getStringBounds(s, g);
		g.setColor(colors.getColor(i));
		g.fillRect((int)(x+barWidth/2-r.getWidth()/2-2), (int)(1.5*r.getHeight()-r.getHeight()), (int)(r.getWidth()+4), (int)(1.5*r.getHeight()));
		g.setColor(Color.black);
		g.drawString(s, (int)(x+barWidth/2-r.getWidth()/2), (int)(1.5*r.getHeight()));
	    }
	}

	    
	// draw the 2 axis
	g.drawLine(x0, 0, x0, y0); // vertical
	g.drawLine(x0, y0, width, y0); // horizontal


	// Draw the basic histogram

	// Y axis labels
	// double graduationStep = Math.pow(10, Math.floor(Math.log10((double)max)));
	// System.out.println(" Graduation Step: " + graduationStep);
	g.setColor(Color.red);
	for (int i = 0; i < fixedCounts.length; i++) {	    
	    int total = fixedCounts[i];
	    r = Util.getStringBounds(" "+total, g);		    
		    
	    g.drawLine(x0, y0 - (int)(heightStep*total),
		       width, y0 - (int)(heightStep*total));
	    g.drawString(""+total,
			 x0 - (int)r.getWidth(), y0 - (int)(heightStep*total) + (int)(r.getHeight() /2));
	}

	// draw a bar per class
	for (int i = 0; i < fixedCounts.length; i++) {
	    // (x, y0) is the lower left point of the current bar
	    int x = x0+(int)(widthStep*(fixedBreaks[i]-fixedBreaks[0]));
	    int barWidth = (int)(widthStep * (fixedBreaks[i+1] - fixedBreaks[i]));
	    int barHeight = (int)(fixedCounts[i] * heightStep);
	    // we use the color of the corresponding class in classif
	    // g.setColor(colors.getColor(classif.getClass(fixedBreaks[i]+(fixedBreaks[i+1] - fixedBreaks[i])/2)));
	    // g.fillRect(x, y0 - barHeight, barWidth, barHeight);
	    int classNumi = classif.getClass(fixedBreaks[i]);
	    int classNumiplus1 = classif.getClass(fixedBreaks[i+1]);
	    if (classNumiplus1 == classNumi) { // draw a rectangle bar
		g.setColor(colors.getColor(classNumi));
		g.fillRect(x, y0 - barHeight, barWidth, barHeight);
	    } else { // draw 2 rectangle bars in case on a class boundary
		int barWidth1 =  (int)(widthStep * (breaks[classNumiplus1] - fixedBreaks[i]));
		g.setColor(colors.getColor(classNumi));
		g.fillRect(x, y0 - barHeight, barWidth1, barHeight);		
		g.setColor(colors.getColor(classNumiplus1));
		g.fillRect(x+barWidth1, y0 - barHeight, barWidth - barWidth1, barHeight);		
		// System.out.println("HistogramPanel:draw 2 bars  ("+fixedBreaks[i]+"-"+fixedBreaks[i+1]+"x"+fixedCounts[i] +") \t"+ barWidth1+"-"+(barWidth - barWidth1));
	    }
	    // draw the contour
	    g.setColor(Color.gray);
	    g.drawRect(x, y0 - barHeight, barWidth, barHeight);
	    // System.out.println("HistogramPanel:drawbar  ("+fixedBreaks[i]+"-"+fixedBreaks[i+1]+"x"+fixedCounts[i] +") \t"+classif.getClass(fixedBreaks[i]+(fixedBreaks[i+1] - fixedBreaks[i])/2)+" [ "+x+","+(y0 - barHeight)+"-"+ barWidth+"x"+ barHeight+"]");

	}		

    }

    private int wichTick(int x) {
	for (int i = 0; i < xticks.length; i++) {
	    if (x < xticks[i]-1)
		return -1;
	    if (x < xticks[i]+1)
		return i;
	}
	return -1;
    }

    public class MyMouseListener extends MouseAdapter implements MouseMotionListener {
 	public void mouseClicked(MouseEvent me)	{
	    int i = wichTick(me.getX());
	    //	    chooseColor(i);
 	}
    
	public void mouseEntered(MouseEvent me)	{
	    setCursor(new Cursor(Cursor.HAND_CURSOR));
	}
	
	public void mouseExited(MouseEvent me) {
	    setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	}

	// 	public void mousePressed(MouseEvent me)	{
	// 	    System.out.println("press");
	// 	}
	//
	// 	public void mouseReleased(MouseEvent me) {
	// 	    System.out.println("release");
	// 	}

	public void mouseMoved(MouseEvent me) {
	    // System.out.println("Mouse moved " + me);
	    int i = wichTick(me.getX());
	    if (i >= 0) 
		setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
	    else
		setCursor(new Cursor(Cursor.HAND_CURSOR));

	}
	
	public void mouseDragged(MouseEvent me) {
	}
    }

    protected void chooseColor(int i) {
	//Color c = OMColorChooser.showDialog(this, i18n.get(this,"colorChooser","Select color..."), colorizer.getColorScale().getColor(i));
	Color c = ThemaMapColorChooser.showDialog(
                this,
                i18n.get(this,"colorChooser","Select color..."),
                colorizer.getColorScale().getColor(i));
        if (c != null)
	    colorizer.getColorScale().setColor(i, c);
    }

}
