// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import fr.unicaen.openmap.dataStorage.DataStorageIterator;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.DataColumnNameList;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Labelizer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Area;
import fr.unicaen.openmap.util.FontDialog;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.gui.ComponentSizer;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;

/**
 * Gui panels for the Labelizer
 *
 * @author Jacques.Madelaine@unicaen.fr
 */
public class LabelizerGUI {

    private JComponent fontSizeScalePanel, textAndFontPanel, textAndFontConfPanel, offseterPanel;

    public JComponent getFontSizeScalePanel() { return fontSizeScalePanel; }
    public JComponent getTextAndFontPanel() { return textAndFontConfPanel; }
    public JComponent getOffseterPanel() { return offseterPanel; }

    private Labelizer labelizer;
    public Labelizer getLabelizer() { return labelizer; }

    public LabelizerGUI(Labelizer labelizer, DataStorageIterator dsi) {
	this.labelizer = labelizer;

	fontSizeScalePanel =  new FontSizerPanel(labelizer.getFontSizer());

	createTextAndFontPanel();

	offseterPanel = labelizer.getOffseter().getGUI(dsi);
    }

    public void setEnabled(boolean b) {
	if (dataList == null)
	    return;
	// textAndFontPanel.setEnabled(b);
	dataList.setEnabled(b);
	fontButton.setEnabled(b);
	previewTF.setEnabled(b);
	rotationSpinner.setEnabled(b);
	labelizer.getOffseter().setEnabled(b);
	fontSizeScalePanel.setEnabled(b);
    }
 
    protected  DataColumnNameList dataList;
    private void initDataList() {
	if (dataList == null)
	    dataList = new DataColumnNameList(labelizer.getVisu(), labelizer.getDataColumnName());
	else
	    dataList.init();
    }

    private JButton policeLabel;
    private JTextField previewTF;
    private FontDialog fontDialog;
    private JButton fontButton;
    private 	JSpinner rotationSpinner;

    private void createTextAndFontPanel() {
	I18n i18n = Environment.getI18n();
	textAndFontConfPanel = new JPanel(new BorderLayout());
	if (! getLabelizer().isAlwaysOn()) {
	    Box wrap = Box.createHorizontalBox();
	    JCheckBox onOffCB = new JCheckBox(i18n.get(this, "onOff", "Add a label"));
	    onOffCB.setSelected(getLabelizer().isOnOff());
	    onOffCB.addItemListener(new ItemListener() {
		    public void itemStateChanged(ItemEvent ie) {
			getLabelizer().setOnOff(((JCheckBox)ie.getSource()).isSelected());
		    }
		});
	    wrap.add(Box.createRigidArea(new Dimension(2, 2)));
	    wrap.add(onOffCB);   
	    textAndFontConfPanel.add(wrap, BorderLayout.NORTH);
	}


	textAndFontPanel = Box.createVerticalBox();
	textAndFontConfPanel.add(textAndFontPanel,  BorderLayout.CENTER);

	JPanel dataPanel = new JPanel();
	dataPanel.setLayout(new BoxLayout(dataPanel, BoxLayout.X_AXIS));
	dataPanel.add(new JLabel(i18n.get(Area.class, "colorDataColumnName", "Color Data Column Name: ")));
	dataList = new DataColumnNameList(labelizer.getVisu(), labelizer.getDataColumnName());
	dataPanel.add(dataList);
	textAndFontPanel.add(dataPanel);
	
	JPanel panel = new JPanel();
	panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));	
	panel.setBorder(new EmptyBorder(2, 6, 2, 6));

	policeLabel = new JButton();
	panel.add(policeLabel, BorderLayout.CENTER);
	fontDialog = new FontDialog(getLabelizer().getFont(), getLabelizer().getFontColor(), 
						     getLabelizer().getAlignment(), getLabelizer().getRotationAngle());
        fontButton = new JButton("", Util.loadIcon("visu/fonts.png"));
        ComponentSizer.fixComponentSize(fontButton, new Dimension(25,25));
        ActionListener actionListener = (new ActionListener() {

            public void actionPerformed(ActionEvent event) {
                fontDialog.setVisible(true);

                if (!fontDialog.isCanceled()) {
                    getLabelizer().setFont(fontDialog.getSelectedFont());
                    getLabelizer().setFontColor(fontDialog.getFontColor());
                    getLabelizer().setAlignment(fontDialog.getJustify());
                    getLabelizer().setRotationAngle(fontDialog.getRotationAngle());
		    getLabelizer().refresh();

		    refreshGUI();     
                }
            }
        });
        fontButton.setBackground(Color.WHITE);
        fontButton.setOpaque(false);
        fontButton.setBorder(null);
	fontButton.addActionListener(actionListener);
	policeLabel.setBackground(Color.WHITE);
        policeLabel.setOpaque(false);
        policeLabel.setBorder(null);
	policeLabel.addActionListener(actionListener);
        panel.add(fontButton, BorderLayout.EAST);
	textAndFontPanel.add(panel);

	JPanel rotPanel = new JPanel();
        rotPanel.add(new JLabel(i18n.get(FontDialog.class, "angle", "Rotation Angle (degrees): ")));    
	rotationSpinner = new JSpinner(new SpinnerNumberModel(getLabelizer().getRotationAngle(), -360d, 360d, 1d));
        rotationSpinner.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent ce) {
		    double angle = (Double)(((JSpinner)ce.getSource()).getValue());
		    getLabelizer().setRotationAngle(angle);
		    fontDialog.setRotationAngle(angle);
		    getLabelizer().refresh();
		}
	    });
        rotPanel.add(rotationSpinner);
	textAndFontPanel.add(rotPanel);

	JPanel previewPanel = new JPanel();
	previewPanel.setLayout(new BorderLayout());	
	previewPanel.setBorder(new TitledBorder(i18n.get(FontDialog.class,
						    FontDialog.PREVIEW_PROPERTY_STRING,
						  "Preview")));
	previewTF = new JTextField(i18n.get(LabelizerGUI.class,
					    FontDialog.PREVIEW_SAMPLE_PROPERTY_STRING,
					    "0123 abcd ABCD"));
	ComponentSizer.fixComponentSize(previewTF, new Dimension(300 , 25));
	previewPanel.add(previewTF, BorderLayout.CENTER);
	textAndFontPanel.add(previewPanel);

	refreshGUI();
    }

    public void refreshGUI() {     
	I18n i18n = Environment.getI18n();
	Font font = getLabelizer().getFont();
	policeLabel.setText(i18n.get(FontDialog.class, FontDialog.FONT_PROPERTY_STRING, "Font")
			    + "  " + font.getName() + " " + font.getSize() + "  ");	
	previewTF.setFont(font);
	previewTF.setForeground(getLabelizer().getFontColor());
	previewTF.setHorizontalAlignment(
					 FontDialog.getJTextFieldAlignmentFromOMTextAlignment(getLabelizer().getAlignment()));
	rotationSpinner.setValue(getLabelizer().getRotationAngle());
	fontDialog.setFontColor(getLabelizer().getFontColor());
	initDataList();
    }
}
