// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;


import javax.swing.*;
import javax.swing.event.*; 
import java.awt.*;
import java.awt.event.*;

import fr.unicaen.openmap.dataStorage.DataStorageIterator;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Resizer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.SymbolResizer;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;
import com.bbn.openmap.util.*;
import fr.unicaen.openmap.util.gui.ComponentSizer;

/** 
 * Panel for Resizer and SymbolResizer.<br/>
 * Able to modify values (maxSize, value, dataColumnName) of the associated Resizer.<br/>
 * Able to manage itself (enabled or disabled).<br/>
 * Extended version : 2 JSpinners ; ComboBox (dataList) ;<br/>
 * Possibility to invalidate spinners or dataList.
 * @see Resizer
 * @see SymbolResizer
 * @see fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Pie
 */
public class ResizerPanel extends JPanel {
    
    /**
     * Only for a Symbol Resizer.
     */
    protected JLabel dataListLabel = new JLabel();

    /**
     * Only for a SymbolResizer.
     * @see SymbolResizer#dataColumnName
     * @see SymbolResizer#dsi
     */
    protected ResizerComboBox dataList = new ResizerComboBox();

    //protected Box mainPan;

    /** panel fir the dataList combo box */
    protected JPanel dataListPanel = null;

    /**
     * Used to manage Resizer's maxSize.
     * @see Resizer#maxSize
     */
    protected JRadioButton maxSizeRB = new JRadioButton();

    /**
     * Used to manage Resizer's maxSize.
     * @see Resizer#maxSize
     */
    protected JSpinner maxSizeField = new JSpinner();
    
    /**
     * Used to manage Resizer's valuePerPixel.
     * @see Resizer#valuePerPixel
     */
    protected JRadioButton valuePerPixelRB = new JRadioButton();

    /**
     * Used to manage Resizer's valuePerPixel.
     * @see Resizer#valuePerPixel
     */
    protected JSpinner valuePerPixelField = new JSpinner();

    /**
     * Associated Resizer.
     */
    protected Resizer resizer;

    /**
     * Used to have a correct refresh for this GUI : what is active and when...
     */
    private boolean isEnabled = false;
    
    /**
     * Use I18N mecanism to allow internationalization.
     * Use I18N.properties file or I18N_fr_FR.properties for french version
     */
    protected I18n i18n = new BasicI18n();

    /**
     * Constructs a ResizerPanel associated with a Resizer.
     * Warning : only with no-fixed size process !
     * @param resizer The commands of ResizerPanel only affects this Resizer.
     * @param maxSizeWanted Ability to affect Resizer's maxSize.
     * @param valuePerPixelWanted Ability to affect Resizer's valuePerPixel.
     * @param dataListWanted Ability to affect Resizer's dataColumnName *only* for a SymbolResizer.
     */    
    public ResizerPanel(final Resizer resizer, 
			final boolean maxSizeWanted, 
			final boolean valuePerPixelWanted, 
			final boolean dataListWanted) {

	this.resizer = resizer;
	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	
	ButtonGroup sizeRBGroup = new ButtonGroup();
	ActionListener sizeRBGroupListener = new ActionListener() {
		
		public void actionPerformed(ActionEvent e) {
		    
		    if (maxSizeWanted) 
			maxSizeField.setEnabled(maxSizeRB.isSelected());
		    
		    if (valuePerPixelWanted)
			valuePerPixelField.setEnabled(valuePerPixelRB.isSelected());
		    resizer.setDeduceMaxSizeFromValuePerPixel(valuePerPixelRB.isSelected());
		}
	    };

	if (dataListWanted) {
	    
	    if (resizer instanceof SymbolResizer) { // bourrin	
		final SymbolResizer symbolResizer = (SymbolResizer)resizer;
		dataListPanel = new JPanel();

		dataListLabel = new JLabel(i18n.get(this, "dataColumnNameLabel", "Data Column Name")); 
		dataListPanel.add(dataListLabel);
		dataListPanel.add(dataList);
		setDataList(symbolResizer);
                add(dataListPanel);
	    }
	}

	//spinner 1
	if (valuePerPixelWanted) {
           
	    valuePerPixelField = new JSpinner(new SpinnerNumberModel(resizer.getValuePerPixel(), 0d, 1000000000000d, 1d));
            ComponentSizer.fixComponentSize(valuePerPixelField, new Dimension(100,20));
	    ChangeListener valuePerPixelFieldListener = new ChangeListener () {
		    
		    public void stateChanged(ChangeEvent e) {
			
			resizer.setValuePerPixel(((Double) valuePerPixelField.getValue()).doubleValue());
					
			if (maxSizeWanted)
			    maxSizeField.setValue(new Double(resizer.getMaxSize()));
		    }
		};
	    valuePerPixelField.addChangeListener(valuePerPixelFieldListener);
            //ComponentSizer.fixComponentSize(valuePerPixelField, new Dimension(50,20));
	    valuePerPixelRB = new JRadioButton(i18n.get(this,"valuePerPixelLabel","Value Per Pixel"));
	    valuePerPixelRB.addActionListener(sizeRBGroupListener);
	    sizeRBGroup.add(valuePerPixelRB);
	    JPanel box = new JPanel();
	    
	    if (maxSizeWanted){
		box.add(valuePerPixelRB);
            }
	    box.add(valuePerPixelField);
            add(box);
	}

	//spinner 2
	if (maxSizeWanted) {	
	    maxSizeField = new JSpinner(new SpinnerNumberModel(resizer.getMaxSize(), 0d, 1000000000000d, 1d));
            ComponentSizer.fixComponentSize(maxSizeField, new Dimension(100,20));
	    ChangeListener maxSizeFieldListener = new ChangeListener () {
		    public void stateChanged(ChangeEvent e) {
			resizer.setMaxSize(((Double) maxSizeField.getValue()).doubleValue());
			
			if (valuePerPixelWanted)
			    valuePerPixelField.setValue(new Double(resizer.getValuePerPixel()));
		    }
		};
	    maxSizeField.addChangeListener(maxSizeFieldListener);
            //ComponentSizer.fixComponentSize(maxSizeField, new Dimension(50,20));
	    maxSizeRB = new JRadioButton(i18n.get(this,"maxSizeLabel","Max Size"));
	    sizeRBGroup.add(maxSizeRB);
	    maxSizeRB.addActionListener(sizeRBGroupListener);
	    
	    JPanel box2 = new JPanel();
	    
	    if (valuePerPixelWanted){
		box2.add(maxSizeRB);
            }
	    box2.add(maxSizeField);
	    add(box2);
	}	
	maxSizeRB.setSelected(true);
	//ComponentSizer.fixComponentSize(valuePerPixelField, new Dimension(45,20));
        //ComponentSizer.fixComponentSize(maxSizeField, new Dimension(45,20));
	setEnabled(true);
    }
    
    /**
     * Constructs a ResizerPanel with all options.
     * @param resizer This resizer should be a SymbolResizer.
     */
    public ResizerPanel(Resizer resizer) {
	this(resizer,true,true,true);
    }



    /**
     * Returns the current status of ResizerPanel. True if is enabled, false otherwise.
     * @return if ResizerPanel is enabled.
     */
    public boolean isEnabled() {
	return isEnabled;
    }

    /**
     * ResizerPanel is able to manage itself.
     * @param b indicates if ResizerPanel is completely enabled or not.
     */
    public void setEnabled(boolean b) {
	isEnabled = b;

	dataListLabel.setEnabled(b);
	dataList.setEnabled(b);

	maxSizeRB.setEnabled(b);
	maxSizeField.setEnabled(b);
	
	valuePerPixelRB.setEnabled(b);
	valuePerPixelField.setEnabled(b);

	if (b) {
	    if (dataListPanel == null)
		return;
	    if (dataListPanel.isVisible() && dataList.getSelectedIndex() == -1 && dataList.getModel().getSize() > 0) {
		//no index selected in list
		maxSizeRB.setEnabled(false);
		maxSizeField.setEnabled(false);
		valuePerPixelRB.setEnabled(false);
		valuePerPixelField.setEnabled(false);
	    } else { //no list *or* index selected
		if (maxSizeRB.isSelected())  {
		    valuePerPixelField.setEnabled(!b);
		    maxSizeField.setEnabled(b);		
		} else {
		    maxSizeField.setEnabled(!b);
		    valuePerPixelField.setEnabled(b);
		}
	    }
	}
    }

    /**
     * Tests if the data list can change.
     * It <em>only</em> works for the SymbolResizer associated.
     * @param symbolResizer a SymbolResizer
     */
    public void setDataList(final SymbolResizer symbolResizer) {

	if (!(resizer instanceof SymbolResizer))
	    return;

	if (symbolResizer != (SymbolResizer)resizer) {
	    System.err.println("###==== setDataList with another resizer !!!! ===###");
	    return;
	}

	if (symbolResizer.getDataStorageIterator().getDataColumnNames() == null || 
	    symbolResizer.getDataStorageIterator().getDataColumnNames().length == 0)
	    return;

	try {
            for(ActionListener al : dataList.getActionListeners()){
                dataList.removeActionListener(al);
            }
            dataList.removeAllItems();
            for(String s : symbolResizer.getDataStorageIterator().getDataColumnNames())
                dataList.addItem(s);

	    dataListSetIndex(symbolResizer);
	    
	    dataList.addActionListener(new ActionListener() {
		    
		    public void actionPerformed(ActionEvent e) {
			int index = ((JComboBox)e.getSource()).getSelectedIndex();
			if (index < 0) 
			    return;
			
			if (symbolResizer.setDataColumnName(symbolResizer.getDataStorageIterator().getRealColumnName(index)))
			    symbolResizer.setMinMaxValue(symbolResizer.getDataStorageIterator());
		    }
		});

	} catch (NullPointerException ex) {
	    Debug.error("Resizer Data Column Name List is unavailable");
	    //dataList = new ResizerComboBox();
	} catch (java.lang.ArrayIndexOutOfBoundsException ofb) {
	    Debug.error("Empty Resizer Data Column Name List");
	    //dataList = new ResizerComboBox();
	}
    }

    public void refresh() {
	Debug.message("Resizer", "ResizerPanel.refresh() "+resizer.getMaxSize()+" "+resizer.getValuePerPixel());
	if (maxSizeField != null)
	    maxSizeField.setValue(new Double(resizer.getMaxSize()));
	
	if (valuePerPixelField != null)
	    valuePerPixelField.setValue(new Double(resizer.getValuePerPixel()));
    }

    public JComboBox getAndRemoveDataList(){
        dataListPanel.remove(dataList);
        dataListPanel.remove(dataListLabel);
        dataListPanel.updateUI();
        return dataList;
    }

    public void setDataListWanted(SymbolResizer symbolResizer, boolean b){
        dataListPanel.setVisible(b);
	dataListSetIndex(symbolResizer);
        updateUI();
    }

    protected void dataListSetIndex(SymbolResizer symbolResizer) {
	int index = -1;
	if(symbolResizer != null && symbolResizer.getDataColumnName()!=null && !"".equals(symbolResizer.getDataColumnName()))
	    index = symbolResizer.getDataStorageIterator().
		getIndexForColumnName(symbolResizer.getDataColumnName());
	// System.out.println("================== index "+index);
	if (index < 0 && symbolResizer != null)
	    symbolResizer.setDataColumnName("");
	dataList.setSelectedIndex(index);
    }



    /**
     * The only instanciation of this inner class is to setEnabled(true) ResizerPanel when the item selected changes. Thus, ResizerPanel is correctly refreshed when there is a item selected (or not).
     */
    public class ResizerComboBox extends JComboBox {

	public ResizerComboBox() {
	    super();
	}

	public ResizerComboBox(String[] names) {
	    super(names);
	}

	/**
	 * Update ResizerPanel when selected item changes.
	 */
	public void setSelectedItem(Object anObject) {
	    super.setSelectedItem(anObject);
	    ResizerPanel.this.setEnabled(true);
	}
    }//...end ResizerComboBox
}
