// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.util.ArrayList;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToggleButton;
import org.jdesktop.swingx.JXCollapsiblePane;

/**
 *
 * @author jchauvea
 */
public abstract class ThematicConfPanel extends JPanel {

    protected JToggleButton collapseButton;
    protected JXCollapsiblePane collapsiblePane;
    protected JComponent content;
    protected String name;
    //protected JPanel pButton;
    //protected JButton bFullSize;
    public final static int V_HEIGHT = 170;
    public final static int H_HEIGHT = 25;
    public final static Dimension CONTENT_DIMENSION = new Dimension(325, 145);
    public final static Dimension FULL_CONTENT_DIMENSION = null;
    protected ArrayList<CollapsablePanelsListener> collapsableListeners = new ArrayList<CollapsablePanelsListener>();
    protected boolean full = false;
    protected Dimension defaultCollapsibleDim;
    protected boolean maximizable = false;
    protected boolean fixedSize = true;
    //protected int hWidth;

    public ThematicConfPanel(String name, JComponent content, int axis) {
        this(name, axis);
        setContent(content);
    }

    public ThematicConfPanel(String name, int axis) {
        super(new BorderLayout());
        this.name = name;
        JXCollapsiblePane.Direction direction =
                axis == BoxLayout.X_AXIS ? JXCollapsiblePane.Direction.LEFT : JXCollapsiblePane.Direction.UP;
        collapsiblePane = new JXCollapsiblePane(direction);
        add(collapsiblePane, BorderLayout.CENTER);
    }

    public void setContent(JComponent content) {
        this.content = content;
        collapsiblePane.removeAll();
        if(fixedSize)
            ComponentSizer.fixComponentSize(content, CONTENT_DIMENSION);
        //content.setPreferredSize(CONTENT_DIMENSION);
        collapsiblePane.add(content);
        //collapsiblePane.setMinimumSize(CONTENT_DIMENSION);
        //defaultCollapsibleDim = collapsiblePane.getPreferredSize();
        if (content instanceof JScrollPane) {
            maximizable = true;
        } else {
            if (!full) {
                maximizable = false;
            }
        }
        //expand();
    }

    public abstract void expand();

    public abstract void collapse();

    public static VerticalThematicConfPanel createVerticalThematicConfPanel(String name) {
        return new VerticalThematicConfPanel(name);
    }

    public static HorizontalThematicConfPanel createHorizontalThematicConfPanel(String name) {
        return new HorizontalThematicConfPanel(name);
    }

    public static VerticalThematicConfPanel createVerticalThematicConfPanel(String name, boolean fixed) {
        return new VerticalThematicConfPanel(name, fixed);
    }

    public static HorizontalThematicConfPanel createHorizontalThematicConfPanel(String name, boolean fixed) {
        return new HorizontalThematicConfPanel(name);
    }

    public static ThematicConfPanel createThematicConfPanel(String name) {
        return createThematicConfPanel(name, true);
    }

    public static ThematicConfPanel createThematicConfPanel(String name, boolean fixed) {
        if (ThematicPalette.AXIS == BoxLayout.X_AXIS) {
            return createVerticalThematicConfPanel(name, fixed);
        } else {
            return createHorizontalThematicConfPanel(name, fixed);
        }
    }

    /**
     * Utility to create a ThematicConfPanel, set its content and add it to a ThematicGUI at the specified index
     */
    public static ThematicConfPanel createThematicConfPanel(String name, JComponent content,  ThematicGUI thematicGUI, boolean fixed, int index) {        
	ThematicConfPanel confPanel = createThematicConfPanel(name, fixed);
	confPanel.setContent(content);
	thematicGUI.addThematicConfigurationPanel(confPanel);	
	JPanel wrapConfPanel = new JPanel();
        wrapConfPanel.add(confPanel);
        thematicGUI.add(wrapConfPanel, index);
	return confPanel;
    }

    public static ThematicConfPanel createThematicConfPanel(String name, JComponent content,  ThematicGUI thematicGUI, boolean fixed) {        
	return createThematicConfPanel(name, content, thematicGUI, fixed, -1);
    }

    public static ThematicConfPanel createThematicConfPanel(String name, JComponent content,  ThematicGUI thematicGUI, int index) {        
	return createThematicConfPanel(name, content, thematicGUI, true, index);
    }

    public static ThematicConfPanel createThematicConfPanel(String name, JComponent content,  ThematicGUI thematicGUI) {        
	return createThematicConfPanel(name, content, thematicGUI, true, -1);
    }

    public JComponent getContent() {
        return content;
    }

    @Override
    public String getName() {
        return name;
    }

    public void addCollapseListener(CollapsablePanelsListener l) {
        collapsableListeners.add(l);
    }

    public void fireOpened() {
        for (CollapsablePanelsListener cpl : collapsableListeners) {
            cpl.panelOpened(name);
        }
    }

    public void fireClosed() {
        for (CollapsablePanelsListener cpl : collapsableListeners) {
            cpl.panelClosed(name);
        }
    }

    protected int computeContentPreferredSize() {
        int height = (int) content.getHeight();
        return height;

    }

    class ExpandButton extends JButton {

        @Override
        public void paint(Graphics g) {
            super.paint(g);
            setBackground(Color.gray);
            setOpaque(false);
            setBorder(null);
            ComponentSizer.fixComponentSize(this, new Dimension(20, 20));
            Graphics2D g2d = (Graphics2D) g;
            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);
            g2d.setColor(Color.GRAY);
            int[] xPoints = {0, 20, 20};
            int[] yPoints = {20, 0, 20};
            g2d.fillPolygon(xPoints, yPoints, 3);
            int[] xPoints2 = {8, 16, 16};
            int[] yPoints2 = {16, 8, 16};
            g2d.setColor(Color.WHITE);
            g2d.fillPolygon(xPoints2, yPoints2, 3);

        }
    }
}
