// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import com.bbn.openmap.layer.policy.BufferedImageRenderPolicy;
import fr.unicaen.openmap.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.utils.LayerExporter;
import fr.unicaen.openmap.util.UnicodePrintStream;
import java.awt.event.ActionEvent;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;

/**
 *
 * @author jchauvea
 */
public class ThematicDataDisplayLayerPopupMenu extends LayerPopupMenu {

    JMenu add;
    JMenuItem exportLayer;
    JMenuItem smoothLayer;
    JMenuItem zoomOn;
    ThematicDataDisplayLayer tddl;

    public ThematicDataDisplayLayerPopupMenu(LayersTree tree, ThematicDataDisplayLayer tddl) {
        super(tree, tddl.isVisible());
        this.tddl = tddl;
        add = new JMenu(
                i18n.get(LayerPopupMenu.class, "addThematic", "add"));
        add(new JSeparator());
        add(add);
        visuManager = new ThematicVisuManagement(add, tddl);
        add(new JSeparator());
        exportLayer = new JMenuItem(i18n.get(LayerPopupMenu.class, "exportLayer", "Export Layer"));
        add(exportLayer);
        exportLayer.addActionListener(this);
        smoothLayer = new JCheckBoxMenuItem(i18n.get(LayerPopupMenu.class, "smoothRender", "Smooth the render"));
        add(smoothLayer);
        smoothLayer.setSelected(!(tddl.getRenderPolicy() instanceof BufferedImageRenderPolicy));
        smoothLayer.addActionListener(this);
        zoomOn = new JMenuItem(i18n.get(this, "zoomOn", "Zoom On Layer"));
        add(zoomOn);
        zoomOn.addActionListener(this);
        //zoomOn.setEnabled(false);//zoomOn method ine TDDL not correct
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        super.actionPerformed(e);
        Object source = e.getSource();
        if (source == exportLayer) {
            String saveString = ThemaMapFileChooserManager.propertiesSaveDialog("");
            if (saveString==null ||saveString.length()==0) return;
            try {
                    LayerExporter.exportLayer(tddl, new UnicodePrintStream(new FileOutputStream(saveString)));
                } catch (FileNotFoundException ex) {
                    ex.printStackTrace();
                }
            return;
        }
        if (source == smoothLayer) {
            tddl.setSmooth(smoothLayer.isSelected());
            return;
        }
        if(source == zoomOn)
            tddl.zoomOn();
    }
}
