// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import fr.unicaen.openmap.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfigurationPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicPalette;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Thematic;
import java.awt.Component;
import java.awt.Dimension;
import java.util.ArrayList;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import org.jdesktop.swingx.JXCollapsiblePane;

/**
 *Wrapper panel for thematic GUI
 * Able to change its orientation(Horizontal/Vertical) according to its parents width
 * @author jchauvea
 */
public class ThematicGUI extends Box {

    protected ArrayList<ThematicConfPanel> configPanels = new ArrayList<ThematicConfPanel>();
    private int axis;
    private Thematic thematic;

    /**
     * Constructor
     * @param axis
     * @param tv
     */
    public ThematicGUI(int axis, Thematic tv) {
        super(axis);
        this.axis = axis;
        this.thematic = tv;
    }

   /**
    * Copy content in a new ThematicGUI
    * @param axis
    * @return
    */
    public ThematicGUI copyWithNewOrientation(int axis) {
        ThematicGUI tg = new ThematicGUI(axis, thematic);      
        for (ThematicConfPanel tcf : configPanels) {
            JComponent content = tcf.getContent();
            ThematicConfPanel confPanel = ThematicConfPanel.createThematicConfPanel(tcf.getName());
            confPanel.setContent(content);
            tg.add(confPanel);
            tg.addThematicConfigurationPanel(confPanel);
        }
        tg.updateUI();
        return tg;
    }

    public void addThematicConfigurationPanel(ThematicConfPanel panel) {
        configPanels.add(panel);
    }

    public Thematic getThematic() {
        return thematic;
    }

    public ArrayList<ThematicConfPanel> getConfigPanels(){
        return configPanels;
    }

}
