// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;


import fr.unicaen.openmap.layer.thematicDataDisplayLayer.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Thematic;

import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.dataStorage.DataStorageIterator;
import fr.unicaen.openmap.gui.LayersTree;

public class ThematicPalette extends JPanel implements /*AnimationDataStorageListener,*/ ComponentListener {

    private JPanel mainPanel;
    private JButton refreshButton;
    ThematicDataDisplayLayer layer;
    private DataStorageIterator dsi;
    private int indexNotLegend = 0;
    protected I18n i18n = Environment.getI18n();
    public static final int MIN_HORIZONTAL_WIDTH = 400;
    //public static final int MAX_THEMATIC_HEIGHT = 400;
    public static int AXIS = BoxLayout.X_AXIS;
    private ArrayList<ThematicGUI> thematicGUIList;
    private String currentCardName;
    private Component currentCard;
    protected HashMap<String, Component> cards = new HashMap<String, Component>();
    protected ThematicVisuManagement visuManager;

    public ThematicPalette(final ThematicDataDisplayLayer layer, DataStorageIterator dsi) {
        super();
        this.layer = layer;
        setDoubleBuffered(true);
        setLayout(new BorderLayout());
        thematicGUIList = new ArrayList<ThematicGUI>();
        addComponentListener(this);
        mainPanel = new JPanel(new CardLayout());
        JScrollPane sp = new JScrollPane(mainPanel);
        add(sp, BorderLayout.CENTER);
        this.dsi = dsi;
        refreshButton = new JButton(Util.loadIcon("testRefresh.png"));
        refreshButton.setToolTipText(i18n.get(ThematicPalette.class, "refreshButton", "Refresh the map"));
        refreshButton.setMaximumSize(new Dimension(35, 35));
        refreshButton.setPreferredSize(new Dimension(35, 35));
        final ThematicPalette me = this;
        refreshButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                // since layer.refresh() triggers a DataStorage event for the moment,
                // i will  temporarily user another function here
                if (layer.isReady()) {
                    layer.setNeedsToRegenerateGraphics(true);
                    layer.refreshGraphicsAndGUI();
                    layer.thematicChanged(null);
                }
            }
        });



        JMenuBar menuBar = new JMenuBar();
        JMenu menuVisu = new JMenu();
        menuBar.add(menuVisu);
        menuVisu.setIcon(Util.loadIcon("bottom2.png"));
        visuManager = new ThematicVisuManagement(menuVisu, layer);
        JButton nextVisuButton = new JButton(Util.loadIcon("right2.png"));;
        refreshButton.setBorder(BorderFactory.createEmptyBorder(-5, 0, 0, 0));
        refreshButton.setBorderPainted(false);
        refreshButton.setBackground(Color.white);
        refreshButton.setOpaque(false);
        refreshButton.setFocusPainted(false);
        nextVisuButton.setBorderPainted(false);
        nextVisuButton.setBackground(Color.white);
        nextVisuButton.setOpaque(false);
        nextVisuButton.setFocusPainted(false);
        nextVisuButton.setMaximumSize(new Dimension(15, 15));
        nextVisuButton.setPreferredSize(new Dimension(15, 15));
        menuBar.setBackground(Color.WHITE);
        menuBar.setBorder(BorderFactory.createEmptyBorder(-5, 0, 0, 0));
        menuBar.setOpaque(false);
        JPanel wrapEast = new JPanel();
        wrapEast.setBorder(null);
        wrapEast.setBackground(Color.WHITE);
        wrapEast.setOpaque(false);
        wrapEast.add(nextVisuButton);
        wrapEast.add(menuBar/*visuShortcutsButton*/);
        wrapEast.add(refreshButton);

        for (int i = 0; i < layer.getThematicArray().size(); i++) {
            addThematicPane(layer.getThematicArray().get(i), layer, null);
        }
    }



    //Used to refresh
    public void setDataStorageIterator(DataStorageIterator dsi) {
        this.dsi = dsi;

    }

    public void addThematicPane(Thematic tv, ThematicDataDisplayLayer layer, LayersTree ltree) {
        ThematicGUI tvGUI = (ThematicGUI) tv.getGUI(dsi);
        thematicGUIList.add(tvGUI);
        currentCard = buildThematicPane(tvGUI);
        cards.put(String.valueOf(tv.hashCode()), currentCard);
        mainPanel.add(currentCard, String.valueOf(tv.hashCode()));
        layer.getVisuNames().add(tv.getName());
        showCard(String.valueOf(tv.hashCode()));
    }

    private JPanel buildThematicPane(ThematicGUI tvGUI) {
        JPanel p1 = new JPanel(new BorderLayout());
        JPanel p2 = new JPanel(new BorderLayout());
        p1.add(p2, BorderLayout.WEST);
        p2.add(tvGUI, BorderLayout.NORTH);
        return p1;
    }


    @Deprecated
    public void removeThematicTab(Component component) {
        mainPanel.remove(component);
        setVisible(false);
        setVisible(true);
        updateUI();
    }

    public void removeCurrentThematicGUI() {
        mainPanel.remove(cards.get(currentCardName));
        cards.remove(currentCardName);
        mainPanel.updateUI();
        updateUI();
    }

    public void removeThematicGUI(String s) {
        mainPanel.remove(cards.get(s));
        cards.remove(s);
        mainPanel.updateUI();
        updateUI();
    }

    public int getIndexNotLegend() {
        return indexNotLegend;
    }

    public void showCard(String name) {
        CardLayout cl = (CardLayout) (mainPanel.getLayout());
        cl.show(mainPanel, name);
        setVisible(false);
        setVisible(true);
        currentCardName = name;
    }


    private int getNewOrientation() {
        if (AXIS == BoxLayout.X_AXIS && getWidth() < MIN_HORIZONTAL_WIDTH) {
            return BoxLayout.Y_AXIS;
        }
        if (AXIS == BoxLayout.Y_AXIS && getWidth() >= MIN_HORIZONTAL_WIDTH) {
            return BoxLayout.X_AXIS;
        } else {
            return AXIS;
        }
    }

    @SuppressWarnings("unchecked")
    public void componentResized(ComponentEvent ce) {
        int newAxis = getNewOrientation();
        //int i = 0;
        if (newAxis != AXIS) {
            ArrayList<ThematicGUI> tmpList = (ArrayList<ThematicGUI>) thematicGUIList.clone();
            thematicGUIList.clear();
            this.AXIS = newAxis;
            for (ThematicGUI cp : tmpList) {
                thematicGUIList.add(cp.copyWithNewOrientation(AXIS));

            }
            update();
        }
    }

    private void update() {
        mainPanel.removeAll();
        for (ThematicGUI tg : thematicGUIList) {
            mainPanel.add(buildThematicPane(tg), tg.getThematic().getName());
        }
        showCard(currentCardName);
        updateUI();
    }

    public void componentMoved(ComponentEvent arg0) {
    }

    public void componentShown(ComponentEvent arg0) {
    }

    public void componentHidden(ComponentEvent arg0) {
    }

    public ThematicVisuManagement getVisuManager(){ return visuManager; }

    public JButton getRefreshButton(){ return refreshButton; }

}
