// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicComboBoxUI;
import javax.swing.tree.DefaultMutableTreeNode;


import fr.unicaen.openmap.gui.LayersTree;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ClassDescription;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Thematic;



import fr.unicaen.openmap.util.*;


import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;

public class ThematicVisuManagement implements ListSelectionListener, MouseListener {

    /**
     * Use I18N mecanism to allow internationalization. Use I18N.properties file
     * or I18N_fr_FR.properties for french version
     */
    protected I18n i18n = new BasicI18n();
    public static int numAddedVisu = 0;
    private ClassDescription defaultThematics[] = {
        new ClassDescription(
        i18n.get(this, "raw", "Raw"),
        "fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Raw",
        "visu/Raw.png"),
        new ClassDescription(
        i18n.get(this, "coloredArea", "Colored Area"),
        "fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Area",
        "visu/Area.png"),
        new ClassDescription(
        i18n.get(this, "label", "Label"),
        "fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Label",
        "visu/Label.png"),
        new ClassDescription(
        i18n.get(this, "proportionalSymbol", "Proportional Symbol"),
        "fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.ProportionalSymbol",
        "visu/ProportionalSymbol.png"),
        new ClassDescription(
        i18n.get(this, "coloredProportionalSymbol",
        "Colored Proportional Symbol"),
        "fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.ColoredProportionalSymbol",
        "visu/ColoredProportionalSymbol.png"),
        new ClassDescription(
        i18n.get(this, "pie", "Pie"),
        "fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Pie",
        "visu/Pie.png"),
        new ClassDescription(
        i18n.get(this, "histogram", "Histogram"),
        "fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Histogram",
        "visu/Histogram.png"),
        new ClassDescription(
        i18n.get(this, "boxplot", "BoxPlot"),
        "fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.BoxPlot",
        "visu/BoxPlot.png"),
        new ClassDescription(
        i18n.get(this, "multiPolygons", "Multi Polygons"),
        "fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.MultiRegularPolygons",
        "visu/MultiRegularPolygons.png"),
        new ClassDescription(
        i18n.get(this, "flow", "Flow"),
        "fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Flow",
        "visu/Flow.png")
    /*new ClassDescription(
    i18n.get(this, "chart", "Chart"),
    "fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Chart",
    "visu/Chart.png"),*/
    };
    ThematicDataDisplayLayer layer;
    private ClassDescription thematics[];
    private JMenu menu;
    JList visuList;

    public ThematicVisuManagement(JMenu menu, ThematicDataDisplayLayer layer) {
        this.menu = menu;
        this.layer = layer;
        thematics = layer.getAddableVisuClassDescription();
        if (thematics == null)
            thematics = defaultThematics;
        init(layer);
        menu.setBorder(null);
    }

    public ThematicVisuManagement(ThematicDataDisplayLayer layer) {
        this.layer = layer;
        thematics = layer.getAddableVisuClassDescription();
        if (thematics == null)
            thematics = defaultThematics;
    }

    private void init(ThematicDataDisplayLayer layer) {
        //System.out.println("LAYER = "+layer);
        this.layer = layer;
        if (thematics == null)
            thematics = defaultThematics;
        menu.removeAll();
        for (int i = 0; i < thematics.length; i++) {
            JMenuItem item = new JMenuItem(thematics[i].getText(), thematics[i].getIcon());
            menu.add(item);
            final int index = i;
            item.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    String name = "visu_a" + (++ThematicVisuManagement.numAddedVisu);
                    while (ThematicVisuManagement.this.layer.getVisuNames().contains(name)) {

                        name = "visu_a" + (++ThematicVisuManagement.numAddedVisu);
                    }

                    Thematic tv = ThematicVisuManagement.this.layer.addNewThematicVisu(thematics[index].getClassName(), name);
                    ThematicVisuManagement.this.layer.fireNewThematicAdded(tv);

                }
            });
        }

    }

    /**
     * Custom display.
     */
    class ComboBoxRenderer extends JLabel implements ListCellRenderer {

        public ComboBoxRenderer() {
            setOpaque(true);
            setHorizontalAlignment(LEFT);
            setVerticalAlignment(CENTER);
        }

        public Component getListCellRendererComponent(JList list, Object value,
                int index, boolean isSelected, boolean cellHasFocus) {
            if (isSelected) {
                setBackground(list.getSelectionBackground());
                setForeground(list.getSelectionForeground());
            } else {
                setBackground(list.getBackground());
                setForeground(list.getForeground());
            }
            setIcon(((ClassDescription) value).getIcon());
            setText(((ClassDescription) value).getText());
            return this;
        }
    }

    static class ComboAddThematicUI extends BasicComboBoxUI {

        public static ComponentUI createUI(JComponent c) {
            return new ComboAddThematicUI();
        }

        protected JButton createArrowButton() {
            return new JButton(Util.loadIcon("addThema.png"));
        }
    }

    public void valueChanged(ListSelectionEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
        String name = "visu_a" + (++ThematicVisuManagement.numAddedVisu);
        while (layer.getVisuNames().contains(name)) {

            name = "visu_a" + (++ThematicVisuManagement.numAddedVisu);
        }
        layer.addNewThematicVisu(((ClassDescription) visuList.getSelectedValue()).getClassName(), name);
    }

    public void mouseReleased(MouseEvent e) {
    }

    public JComponent asTool() {

        JPanel pVisu = new JPanel();
        pVisu.setOpaque(false);
        for (int i = 0; i < thematics.length; i++) {
            JButton bVisu = new JButton(thematics[i].getIcon());
            final int a = i;
            bVisu.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent arg0) {
                    String name = "visu_a" + (++ThematicVisuManagement.numAddedVisu);
                    for (String s : layer.getVisuNames())
                        while (layer.getVisuNames().contains(name)) {
                            name = "visu_a" + (++ThematicVisuManagement.numAddedVisu);
                        }
                    Thematic tv = layer.addNewThematicVisu(thematics[a].getClassName(), name);
                    layer.fireNewThematicAdded(tv);
                }
            });
            bVisu.setToolTipText(i18n.get(this, "addVisu", "Add thematic") + " - " + thematics[a].getText());
            bVisu.setBackground(Color.WHITE);
            bVisu.setOpaque(false);
            bVisu.setPreferredSize(new Dimension(25, 25));
            pVisu.add(bVisu);
        }
        return pVisu;
    }
}
