// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ColorScale;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Colorizer;
import fr.unicaen.openmap.util.classification.Classification;
import fr.unicaen.openmap.util.classification.ClassificationListener;
import fr.unicaen.openmap.util.classification.ClassificationOfDouble;
import fr.unicaen.openmap.util.classification.ContinuousClassification;
import fr.unicaen.openmap.util.classification.Customized;
import fr.unicaen.openmap.util.classification.CustomizedClassification;
import fr.unicaen.openmap.util.classification.CustomizedClassificationOfDouble;
import fr.unicaen.openmap.util.classification.CustomizedClassificationOfString;
import fr.unicaen.openmap.util.classification.CustomizedDiscreteOfDouble;
import fr.unicaen.openmap.util.classification.DiscreteClassification;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import fr.unicaen.openmap.util.Util;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 *
 * @author jchauvea
 */
public class ThresholdColorScalePanel extends ColorScalePanel implements ClassificationListener {

    protected Colorizer colorizer = null;
    protected JSpinner[] spinners;

    public ThresholdColorScalePanel(ColorScale cs, Colorizer colorizer) {
        super(cs);
        this.colorizer = colorizer;
        colorizer.addClassificationListener(this);
        createColorScalePanel();
        createOptionPanel();
        setLayout(new BorderLayout());
        JPanel wrapOption = new JPanel(new BorderLayout());
        wrapOption.add(panOption, BorderLayout.NORTH);
        add(wrapOption, BorderLayout.WEST);
        add(panColorScale, BorderLayout.EAST);

    }

    @Override
    public void createOptionPanel() {
        super.createOptionPanel();
        remove(panOption);
        colorNumber.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent arg0) {
                int n = ((Integer) colorNumber.getValue()).intValue();
                colorScale.setColorNumber(n);
                if (colorizer.getClassification() != null) {
                    colorizer.getClassification().setNumberOfClasses(n);
                }
                refreshPanel();
                return;
            }
        });
        updateUI();
    }

    public static final int MAX_NUMBER_OF_LABELS = 300;

    @Override
    public void createColorScalePanel() {
        if (panColorScale == null) {
            panColorScale = new JPanel();
        }
	
	Box buttonsBox;
        Box spinnersOrLabelsBox;
	// System.out.println(Thread.currentThread() + " ThresholdColorScalePanel synchronized on " + this);
	synchronized(ThresholdColorScalePanel.this) {
	    panColorScale.removeAll();
	    buttonsBox = Box.createVerticalBox();
	    spinnersOrLabelsBox = initSpinnersOrLabelsBox();
	    panColorScale.add(buttonsBox);
	    panColorScale.add(spinnersOrLabelsBox);
	}
	// System.out.println(Thread.currentThread() + " ThresholdColorScalePanel synchronized on " + this + " <==");
	
        colorButtons.clear();

        ToolTipColorScaleButton b = new ToolTipColorScaleButton(this, 0);
        colorButtons.add(b);
        b.setBackground(colorScale.getColor(0));
        b.setText(getColorButtonText(0));
        buttonsBox.add(b);
	// set the minimum width of the 2 boxes
        buttonsBox.add(Box.createRigidArea(new Dimension(80, 2)));
	spinnersOrLabelsBox.add(Box.createRigidArea(new Dimension(80, 2)));

        ColorScaleButton lower = b;

	Classification classif = colorizer.getClassification();
	if (classif != null)
	    classif = classif.getRealClassif();
	   
        for (int i = 1; i < colorScale.getColorNumber(); i++) {
            b = new ToolTipColorScaleButton(this, i);
            b.setBackground(colorScale.getColor(i));
            buttonsBox.add(b);
            colorButtons.add(b);
            b.setText(getColorButtonText(i));
            if (classif == null) 
		continue;
	    if (classif instanceof DiscreteClassification) {
		if (i > MAX_NUMBER_OF_LABELS) // GUI truncated
		    break;
		JComponent component = i == MAX_NUMBER_OF_LABELS 
		    ? new JLabel("...") : createClassTextComponent((DiscreteClassification)classif, i);
		ComponentSizer.fixComponentSize(component, new Dimension(70, 22));
                spinnersOrLabelsBox.add(component);
            } else if (classif instanceof ContinuousClassification) {
		spinners[i - 1] = new CSJSpinner(b, b, ((ClassificationOfDouble)classif).getNthBreak(i));
		ComponentSizer.fixComponentSize(spinners[i - 1], new Dimension(100, 22));
                spinnersOrLabelsBox.add(spinners[i - 1]);
	    } else { // it is a ClassificationOfDouble
		if (spinners == null)
		    return;
                spinners[i - 1] = new CSJSpinner(lower, b, ((ClassificationOfDouble)classif).getNthBreak(i));
		if (spinners[i-1] == null) // prevent race condition 
		    return;
                spinnersOrLabelsBox.add(spinners[i - 1]);
            }
            lower = b;
        }

	if (classif != null 
	    && ! (classif instanceof DiscreteClassification) 
	    && ! (classif instanceof ContinuousClassification) 
	    && classif.getNumberOfClasses() > 1) 
	    spinnersOrLabelsBox.add(new NonEditableJSpinner(getSpinnerValue(colorScale.getColorNumber())));

        buttonsBox.setBorder(BorderFactory.createEmptyBorder(0, -30, 0, -20));
    }

    private JComponent createClassTextComponent(final DiscreteClassification dc, final int i) {
	if (i >= MAX_NUMBER_OF_LABELS)
	    return new JLabel("...");
	String text = dc.getClassText(i);
	if (dc instanceof CustomizedDiscreteOfDouble || dc instanceof CustomizedClassificationOfString) {
	    final JTextField textField = new JTextField(text);
	    textField.addKeyListener(new KeyListener() {
		    // only take key released into account
		    public void keyTyped(KeyEvent arg0) {
		    }
		    
		    public void keyPressed(KeyEvent arg0) {
		    }
		    
		    public void keyReleased(KeyEvent arg0) {
			if (dc instanceof CustomizedClassificationOfString)
			    ((CustomizedClassificationOfString)dc).setNthBreak(i, textField.getText());
			else
			    ((CustomizedDiscreteOfDouble)dc).setNthBreak(i, Util.parseDouble(textField.getText()));
		    }
		});
	    return textField;
	}
	return new JLabel(text);
    }
    
    @Override
    public void updateColorScaleAt(int position, Color color) {
        super.updateColorScaleAt(position, color);
        colorizer.setColorScale(colorScale);
    }

    private Box initSpinnersOrLabelsBox() {
        Box box = Box.createVerticalBox();
	Classification classif = colorizer.getClassification();
	if (classif == null)
	    return box;
	Classification realClassif = classif.getRealClassif();
	if (realClassif == null)
	    return box;
	    
	// System.out.println("ThresholdColorScalePanel.initSpinnersOrLabelsBox with: "+ realClassif);
	if (realClassif instanceof DiscreteClassification) {
	    JComponent component = createClassTextComponent((DiscreteClassification)realClassif, 0);
	    ComponentSizer.fixComponentSize(component, new Dimension(70, 22));
	    box.add(component);
	} else if (realClassif.getNumberOfClasses() > 1) {
		if (realClassif instanceof ContinuousClassification) {
		    spinners = new JSpinner[colorScale.getColorNumber()];
		    // System.out.println("## new spinners[] "+spinners.length);
		    JSpinner spin = new CSJSpinner(null, null, getSpinnerValue(0));
		    ComponentSizer.fixComponentSize(spin, new Dimension(100, 22));
		    box.add(spin);
		} else {
		    spinners = new JSpinner[colorScale.getColorNumber() - 1];
		    // System.out.println("** new spinners[] "+spinners.length);
		    box.add(new NonEditableJSpinner(getSpinnerValue(0)));
		}
	}

        return box;
    }

    protected String getColorButtonText(int i) {
        return i < colorButtons.size() && colorizer != null ? colorizer.getColorText(i) : "";
    }

    protected double getSpinnerValue(int i) {
        try { 
            if (colorizer == null)
		return 0;
	    Classification c = colorizer.getClassification();
	    if (c == null)
		return 0;
	    c = c.getRealClassif();
	    if (c == null)
		return 0;
	    
	    if (! (c instanceof ClassificationOfDouble))
		return 0;
	    return ((ClassificationOfDouble)c).getNthBreak(i);

        } catch (Exception e) {
            System.err.println("*** getSpinnerValue " + e.getMessage());
	    e.printStackTrace();
            return 0;
        }
    }

    public void classificationChanged(Object source) {
	if (source == this)
	    return;
        if (colorizer.getClassification() != null) {
            colorizer.getColorScale().setColorNumber(colorizer.getClassification().getNumberOfClasses());
        }
	if (panColorScale != null) 
	    refreshPanel();
	// updateUI();
    }

    private class CSJSpinner extends JSpinner implements ChangeListener {

        private JButton upper;
        private JButton lower;

        public CSJSpinner(ColorScaleButton lower, ColorScaleButton upper, double value) { // , double min, double max) {
            super(new SpinnerNumberModel(value, Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY, 1));
            ComponentSizer.fixComponentSize(this, new Dimension(100, 22));
            // setFont(new Font("Arial", Font.PLAIN, 8));
            this.upper = upper;
            this.lower = lower;
            addChangeListener(this);
        }

        //Enables CUSTOM CLASSIFICATION MODE -- change breaks in live
        public void stateChanged(ChangeEvent e) {
            double value = ((Double) this.getValue()).doubleValue();
	    Classification classif = colorizer.getClassification();
	    
	    if (classif instanceof ContinuousClassification) {
		// Continuous classif: just set nth break
		int position = upper == null ? 0 : ((ColorScaleButton) upper).getPosition();
		((ContinuousClassification)classif).setNthBreak(position, value);
		return;
	    }

            if (classif != null && (!(classif instanceof CustomizedClassification))) {
		// Classif is not already customized, so
		// we create a customClassif from current classif
		classif = new Customized(classif, colorScale.getColorNumber());
		colorizer.setClassification(classif);
	    }

            ((CustomizedClassificationOfDouble)(classif.getRealClassif())).setNthBreak(((ColorScaleButton) upper).getPosition(), value);
            int iLow = ((ColorScaleButton) upper).getPosition();
            lower.setText(getColorButtonText(((ColorScaleButton) lower).getPosition()));
            upper.setText(getColorButtonText(((ColorScaleButton) upper).getPosition()));
	    
	    fireColorScaleChanged();

            // System.out.println("####ColorizerColorScalePanel: " + colorizer.getClassification());
            return;
        }
    }

    @Override
    public void setEnabled(boolean b) {
        super.setEnabled(b);
	// should have a synchonized boolean to tell if SpinnersOrLabelsBox are initialized instead of testing for null
        if(spinners != null)
        for (JSpinner sp : spinners) {
            if (sp != null)
		sp.setEnabled(b);
        }
    }
}


class NonEditableJSpinner extends JSpinner { 
    public NonEditableJSpinner(double value) {
	super(new SpinnerNumberModel(value, Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY, 1.0));
	((JSpinner.DefaultEditor) getEditor()).getTextField().setEditable(false);
	for(ChangeListener cl : getChangeListeners())
	    removeChangeListener(cl);
	ComponentSizer.fixComponentSize(this, new Dimension(100, 22));
    }
}

class ToolTipColorScaleButton extends ColorScaleButton {

    private static Dimension buttonsDimension = new Dimension(22, 22);

    public ToolTipColorScaleButton(ColorScalePanel scalePanel, int position){
        super(scalePanel, position);
	setPreferredSize(buttonsDimension);
	setMaximumSize(buttonsDimension);
    }

    @Override
    public void setText(String text){
        setToolTipText(text);
    }

}
