// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToggleButton;

/**
 *
 * @author jchauvea
 */
public class VerticalThematicConfPanel extends ThematicConfPanel {

    private boolean vertical = true;

    public VerticalThematicConfPanel(String name, JComponent content) {
        this(name);
        setContent(content);
        collapseButton = new VerticalJButton(name, Util.loadIcon("expand.png"));
        //listener
        collapseButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                if (collapsiblePane.isCollapsed()) {
                    vertical = false;
                    expand();
                    collapsiblePane.setCollapsed(false);
                } else {
                    vertical = true;
                    collapse();
                    collapsiblePane.setCollapsed(true);
                }
            }
        });
        expand();
        collapsiblePane.setCollapsed(false);
    }

    public VerticalThematicConfPanel(String name) {
        super(name, BoxLayout.X_AXIS);
        collapsiblePane.setCollapsed(true);
        collapseButton = new VerticalJButton(name, Util.loadIcon("expand.png"));
        add(collapseButton, BorderLayout.WEST);
        //listener
        collapseButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                if (collapsiblePane.isCollapsed()) {
                    expand();
                } else {
                    collapse();
                }
            }
        });
        expand();
    }

    public VerticalThematicConfPanel(String name, boolean fixed) {
        this(name);
        fixedSize = fixed;
    }

    @Override
    public void expand() {
        vertical = false;
        fireOpened();
        ComponentSizer.fixComponentSize(collapseButton,
                new Dimension((int) CONTENT_DIMENSION.getWidth(), H_HEIGHT));
        if (content != null) {
            content.setVisible(true);
        }
        add(collapseButton, BorderLayout.NORTH);
        collapseButton.setText(name);
        collapsiblePane.setCollapsed(false);
    }

    @Override
    public void collapse() {
        vertical = true;
        fireClosed();
        add(collapseButton, BorderLayout.WEST);
        collapsiblePane.setCollapsed(true);
        if (content != null) {
            content.setVisible(false);
        }
        ComponentSizer.fixComponentSize(collapseButton, new Dimension(H_HEIGHT, V_HEIGHT));
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);
        Graphics2D g2d = (Graphics2D) g;
        //anti aliazing
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        g2d.setColor(new Color(0xDD, 0xDD, 0xDD, 0xFF));
        g2d.setColor(Color.GRAY);
        g2d.drawRoundRect(0, 0, getWidth() - 1, getHeight() - 1, 12, 12);
    }

    class VerticalJButton extends JToggleButton {

        public VerticalJButton(String name, ImageIcon icon) {
            super(name);
            setBorderPainted(false);
            setContentAreaFilled(false);
        }

        @Override
        public void paint(Graphics g) {
            setBackground(getParent().getBackground());
            setBorder(null);
            super.paint(g);
            Graphics2D g2d = (Graphics2D) g;
            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);
            if (vertical) {
                g2d.setColor(new Color(0xDD, 0xDD, 0xDD, 0xFF));
                g2d.fillRoundRect(0, 0, getWidth(), getHeight(), 12, 12);
                g2d.setColor(Color.GRAY);
                g2d.drawRoundRect(0, 0, getWidth() - 1, getHeight() - 1, 12, 12);
                g2d.setColor(Color.BLACK);
                collapseButton.setText("");
                super.paintComponent(g);
                g2d.translate(0, getHeight());
                g2d.rotate(-Math.PI / 2);
                g2d.drawString(name, 5, 15);
                Image image = Util.loadIcon("expand.png").getImage();
                g.drawImage(image, V_HEIGHT - 20, 5, this);
            } else {
                g2d.setColor(new Color(0xDD, 0xDD, 0xDD, 0xFF));
                g2d.fillRoundRect(0, 0, getWidth(), getHeight(), 12, 12);
                g2d.setColor(Color.GRAY);
                g2d.drawRoundRect(0, 0, getWidth() - 1, getHeight() - 1, 12, 12);
                collapseButton.setText(name);
                super.paintComponent(g);
                Image image = Util.loadIcon("collapse.png").getImage();
                g.drawImage(image, 5, 5, this);
            }
            g2d.dispose();

        }
    }
}
