// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import javax.swing.*;
import javax.swing.plaf.basic.BasicButtonUI;
import java.awt.*;
import java.awt.event.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.*;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;

/**
 * Menu for java 5.x compatibility. Same behavior than VisuTabComponent.
 * Ability to rename, move up, move down and close the Thematics.
 * @deprecated see ThematicPalette
 */ 
public class VisuTabPopup extends JPopupMenu implements ItemListener {
    private final JTabbedPane pane;
    private final ThematicDataDisplayLayer layer;
    private final Thematic tv;
    
    private Component current = null;
    private JCheckBoxMenuItem cb = null;
    
    /**
     * Use I18N mecanism to allow internationalization.
     * Use I18N.properties file or I18N_fr_FR.properties for french version
     */
    protected I18n i18n = new BasicI18n();

    public VisuTabPopup(final JTabbedPane pane, final ThematicDataDisplayLayer layer) {
	this.pane = pane;
        this.layer = layer;
	this.tv = null;
	
	JMenuItem menuItem;
	
	/*        menuItem = new JMenuItem("Rename");
        menuItem.addActionListener(new AbstractAction() {
		public void actionPerformed(ActionEvent e) {
		    int index = pane.getSelectedIndex();
		    String s = JOptionPane.showInputDialog("Please provide a new name", pane.getTitleAt(index));
		    if (s == null || s.trim().equals("")) return;
		    pane.setTitleAt(pane.getSelectedIndex(), s.trim());
		}
	    });
        add(menuItem);
	*/


	cb = new JCheckBoxMenuItem(i18n.get(this,"visibleItem","Visible"), true);
	cb.addItemListener(this);
	add(cb);
	
	addSeparator();
        menuItem = new JMenuItem(i18n.get(this,"upItem","Move Up"));
        menuItem.addActionListener(new AbstractAction() {
		public void actionPerformed(ActionEvent e) {
		    moveUpThematicVisu(pane.getSelectedIndex());
		}
	    });
        add(menuItem);
        menuItem = new JMenuItem(i18n.get(this,"downItem","Move Down"));
        menuItem.addActionListener(new AbstractAction() {
		public void actionPerformed(ActionEvent e) {
		    moveDownThematicVisu(pane.getSelectedIndex());
		}
	    });
        add(menuItem);
	addSeparator();
	
        menuItem = new JMenuItem(i18n.get(this,"closeItem","Close"));
        menuItem.addActionListener(new AbstractAction() {
		public void actionPerformed(ActionEvent e) {
		    int index = pane.getSelectedIndex();
		    pane.remove(index);
		    layer.deleteThematicVisu(index);
		    layer.refreshGraphicsAndGUI();
		}
	    });
        add(menuItem);
    }

    public void show(Component c, int x, int y) {
	cb.setSelected(layer.getThematicArray().get(pane.getSelectedIndex()).isVisible());
	super.show(c, x, y);
    }
    
    public void itemStateChanged(ItemEvent e) {
	if (e.getStateChange() == ItemEvent.SELECTED)
	    layer.getThematicArray().get(pane.getSelectedIndex()).setVisible(true);
	else if (e.getStateChange() == ItemEvent.DESELECTED)
	    layer.getThematicArray().get(pane.getSelectedIndex()).setVisible(false);
	layer.refreshGraphicsAndGUI();
    }
    
    /**
     * Move one tab from an index to another. No bound checking !
     */
    private synchronized void moveTab(int src, int dst) {
	Component comp = pane.getComponentAt(src);
	String title = pane.getTitleAt(src);
	Icon icon = pane.getIconAt(src);
	pane.remove(src);
	pane.insertTab(title, icon, comp, null, dst);
	pane.setSelectedIndex(dst);
    }
    
    /**
     * Move UP. index bounding is checked.
     */
    public void moveUpThematicVisu(int currentIndex) {
	if (currentIndex == 0) return;
	moveTab(currentIndex, currentIndex -1);
	layer.moveUpThematicVisu(currentIndex);
    }
    
    /**
     * Move DOWN. index bounding is checked.
     */
    public void moveDownThematicVisu(int currentIndex) {
	if (currentIndex == pane.getTabCount() -1) return;
	moveTab(currentIndex, currentIndex +1);
	layer.moveDownThematicVisu(currentIndex);
    }
}
