// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics;

import com.bbn.openmap.omGraphics.OMColorChooser;
import com.bbn.openmap.util.ColorFactory;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Thematic;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Paint;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Properties;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;

/**
 * Drawing attributes for Histogram plus axes color
 * @author jchauvea
 */
public class HistogramDrawingAttributes extends ThematicDrawingAttributes {

    private JPanel axesPanel;
    private Color axesColor = Color.WHITE;
    private JButton bAxesColor;

    public HistogramDrawingAttributes(Thematic thematic, boolean bColorMenu, boolean bCapMenu, boolean bJointMenu, boolean bLinePaint, boolean bFillPaint) {
        super(thematic, bColorMenu, bCapMenu, bJointMenu, bLinePaint, bFillPaint);

        axesPanel = new JPanel();
        axesPanel.add(new JLabel("Axes color"));
        bAxesColor = new JButton();
	bAxesColor.setBackground(getAxesPaint());
        ComponentSizer.fixComponentSize(bAxesColor, new Dimension(20, 20));
        bAxesColor.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent ae) {
                setAxesPaint(OMColorChooser.showDialog(null, "", getAxesPaint()));
		bAxesColor.setBackground(getAxesPaint());
            }
        });
        axesPanel.add(bAxesColor);
    }

    public JComponent getAxesPanel() {
        return axesPanel;
    }
    public static final String axesColorPropertyString = "axesColor";

    @Override
    public void setProperties(String prefix, Properties props) {
        super.setProperties(prefix, props);
	if (props == null) 
	    return;
	axesColor = ColorFactory.parseColor(props.getProperty(prefix + axesColorPropertyString, "ffffff"));
	if(bAxesColor != null)
	    bAxesColor.setBackground(axesColor);
    }

    @Override
    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);

        properties.put(prefix + axesColorPropertyString, ColorFactory.getHexColorString(axesColor));
        return properties;
    }

    public Color getAxesPaint() {
        return axesColor;
    }


   public void setAxesPaint(Color paint){
       axesColor = paint;
       thematic.setBadCache(true);
       thematic.getLayer().refreshGraphics();
   }
}
