// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics;

import com.bbn.openmap.omGraphics.OMCircle;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.proj.coords.LatLonPoint;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.OffsetableListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Offsetable;
import java.awt.Point;
import java.awt.geom.Point2D;

public class OMDisc extends OMOffsetableWithData implements Comparable<OMDisc> {

    private OMCircle omCircle;

    /** Creates an OMDisc with a Lat-lon center and a lat-lon axis. It has a RENDERTYPE_LATLON. 
     */
    public OMDisc(double latPoint, double lonPoint, double radius, Object data) {
        super(latPoint, lonPoint, data);
        omCircle = new OMCircle(latPoint, lonPoint, radius);
        omCircle.putAttribute(ThematicDataDisplayLayer.VALUE, data);
        renderType = OMDisc.RENDERTYPE_OFFSET;
        add(omCircle);
    }

    /** Create a OMDisc, positioned with a lat-lon center and x-y diameter. */
    public OMDisc(Offsetable off, String id, double latPoint, double lonPoint, int xOffset, int yOffset, int diameter, Object data, boolean v, boolean l) {
        super(off, id, latPoint, lonPoint, xOffset, yOffset, v, l, data);
        renderType = OMDisc.RENDERTYPE_OFFSET;
        omCircle = new OMCircle(latPoint, lonPoint, xOffset, yOffset, diameter, diameter);
        omCircle.putAttribute(ThematicDataDisplayLayer.VALUE, data);
        add(omCircle);
        init(xOffset, yOffset);
    }

    public OMDisc(double lat, double lon, int diameter, Object data) {
	this(null, "", lat, lon, 0, 0, diameter, data, false, false);
    }

    public OMCircle getOMCircle() {
        return omCircle;
    }

    public int compareTo(OMDisc omd) {
        int renderT = omCircle.getRenderType();
        if (renderT != omd.getRenderType()) {
            System.err.println("### Trying to compare to different rendered type discs: " + renderT +" - " + omd.getRenderType());
        }
        if (renderT == RENDERTYPE_LATLON || renderT == RENDERTYPE_OFFSET) {
            return (int) Math.signum(omCircle.getRadius() - omd.getOMCircle().getRadius());
        }
        return (int) Math.signum(omCircle.getWidth() - omd.getOMCircle().getWidth());
    }

    public int getWidth() {
        return omCircle.getWidth();
    }

    public int getHeight() {
	return omCircle.getWidth();
    }

    @Override
    public void translate(int x, int y) {
        super.translate(x, y);
        omCircle.setOffX(getX());
        omCircle.setOffY(getY());
    }
}
