// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics;

import java.awt.Point;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.util.ArrayList;


import com.bbn.openmap.proj.coords.LatLonPoint;
import com.bbn.openmap.omGraphics.OMAbstractLine;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMLine;
import com.bbn.openmap.omGraphics.OMSpline;
import com.bbn.openmap.proj.Projection;

import fr.unicaen.openmap.dataStorage.Centroid;

import fr.unicaen.openmap.layer.flowLayer.drawing.Spline;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Flow;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import java.awt.Color;
import java.awt.Graphics;
import javax.swing.JFrame;
import javax.swing.JPanel;

public class OMFlow extends OMAbstractLine implements HaveWidthAndData, Comparable<OMFlow>{
    // TODO: remove data members useful only at construction time
    private String id;
    private Centroid start, end;
    public static int ARC_LENGTH = 5;
    public static int BENDED_DISTANCE = 0;
    private float[] xBorder, yBorder;
    private int indexStartArrow, indexEndArrow;
    private ArrowHead.ArrowHeadType arrowStart, arrowEnd;
    private Point2D pStart, pRefCurved, pEnd;
    private int width;
    private Flow flow;
    private OMGraphicList controlPoints;
    private boolean legend;
    public static final int BENDED_AUTO_MODE = 1;
    public static final int BENDED_MANUAL_MODE = 2;
    public static int BENDED_MODE = BENDED_AUTO_MODE;
    private double[] xs;
    private double ys[];
    private OMGraphicList radials = new OMGraphicList();
    private ArrayList<Double> xSkeleton;
    private ArrayList<Double> ySkeleton;

    public OMFlow(String id, Flow flow, Centroid startPoint, Centroid endPoint,
            ArrowHead.ArrowHeadType arrowStart, ArrowHead.ArrowHeadType arrowEnd, int size, Object data) {
        //super(RENDERTYPE_LATLON, LINETYPE_STRAIGHT, DECLUTTERTYPE_NONE);
        super();
        this.id = id;
        this.flow = flow;
        this.start = startPoint;
        this.end = endPoint;
        this.arrowStart = arrowStart;
        this.arrowEnd = arrowEnd;
        this.width = size;
	putData(data);
        visible = true;
        legend = false;
        //System.out.println("FLOW  = depart: "+start.toString()+"- arrivée:"+end.toString() );
        controlPoints = new OMGraphicList();
        //System.out.println("OMFLOW D'ID "+id);
        //this.controlPoints = new OMGraphicList();
        generate(flow.getProjection());
    }

    /** used for GUI sample and in the legend */
     public OMFlow(Flow flow, Point p1, Point p2,
             ArrowHead.ArrowHeadType arrowStart, ArrowHead.ArrowHeadType arrowEnd, int size) {
         this.flow = flow;
         this.pStart = p1;
         this.pEnd = p2;
         this.arrowStart = arrowStart;
         this.arrowEnd = arrowEnd;
         this.width = size;
         visible = true;
         legend = true;
         generate(flow.getProjection());
     }

    private void initSkeletons(){
        xSkeleton = new ArrayList<Double>();
        ySkeleton = new ArrayList<Double>();
        xSkeleton.add(pStart.getX());
        xSkeleton.add(pRefCurved.getX());
        xSkeleton.add(pEnd.getX());
        ySkeleton.add(pStart.getY());
        ySkeleton.add(pRefCurved.getY());
        ySkeleton.add(pEnd.getY());

    }

    public void addSkeletonPoint(int index,double x, double y){
        //controlPoints.add(new OMFlowControlPoint(index, x, y, this));
        xSkeleton.add(index, x);
        ySkeleton.add(index, y);
    }


    public boolean generate(Projection proj) {
        shape = null;
        if (pStart == null) {
            // skeleton points must be in XY
            pStart = proj.forward(start.getLat(), start.getLon());
            pEnd = proj.forward(end.getLat(), end.getLon());
        }
        double[] xSkel;
        double[] ySkel;
        if (!legend) {
            if (isStartEqualsToEnd()) { // we have a loop
                if (flow.isShowStartEqualsEnd()) { // we show the loops
                    int radius = flow.getLoopRadius();
                    pRefCurved = new Point2D.Double(pStart.getX() + radius, pStart.getY() + radius);
                    Point2D pa = new Point2D.Double(pStart.getX(), pStart.getY() + radius);
                    Point2D pb = new Point2D.Double(pStart.getX() + radius, pStart.getY());
                    xSkel = new double[]{
                                (double)pStart.getX(),
                                (double) pa.getX(),
                                (double) pRefCurved.getX(),
                                (double) pb.getX(),
                                (double) pEnd.getX()
                            };
                    ySkel = new double[]{
                                (double) pStart.getY(),
                                (double) pa.getY(),
                                (double) pRefCurved.getY(),
                                (double) pb.getY(),
                                (double) pEnd.getY()
                            };
                } else {
                    return false;
                }
            } else { // it is not a loop
                pRefCurved = computeRefCurvedPoint(pStart, pEnd);
                xSkel = new double[]{(double) pStart.getX(), (double) pRefCurved.getX(), (double) pEnd.getX()};
                ySkel = new double[]{(double) pStart.getY(), (double) pRefCurved.getY(), (double) pEnd.getY()};
            }
        } else { // for legend or GUI sample
            pRefCurved = new Point((int) ((pStart.getX() + pEnd.getX()) / 2), (int) ((pStart.getY() + pEnd.getY()) / 2));
            initSkeletons();
            xSkel = new double[]{(int) pStart.getX(), (int) pRefCurved.getX(), (int) pEnd.getX()};
            ySkel = new double[]{(int) pStart.getY(), (int) pRefCurved.getY(), (int) pEnd.getY()};
        }
        //X & Y skeletons

        //spline computing
        //NatCubicSpline natSpline = new NatCubicSpline();
        int nbStep = computeNbPolylinePerArc();
        if (nbStep < 3) {
            nbStep = 3;
        }
        //Spline spline = new Spline();
        xs = new double[nbStep];
        ys = new double[nbStep];
        Spline.compute(xSkel.length, xSkel, ySkel, nbStep, xs, ys);
        //natSpline.setSteps(nbStep);
        //double[][] doubleSpline = natSpline.calc(xSkeleton, ySkeleton);
        //int[][] spline = new int[2][nbStep];
	//            for (int i = 0; i < nbStep; i++) {
	//                spline[0][i] = (int) doubleSpline[0][i];
	//                spline[1][i] = (int) doubleSpline[1][i];
	//            }
        double[][] sp = new double[2][nbStep];
        sp[0] = xs;
        sp[1] = ys;
        if (!legend) {
            sp = deleteDouble(sp);
        }
        Float[] border = computeBorder(sp, this.width);
        if (sp[0].length > 1 && border != null) {
            xBorder = new float[border.length / 2];
            yBorder = new float[border.length / 2];
            int iBorder;
            int iXY;
            for (iBorder = 0, iXY = 0; iBorder < border.length - 1; iBorder += 2, iXY++) {
                xBorder[iXY] = border[iBorder];
                yBorder[iXY] = border[iBorder + 1];
                //System.out.println("******XBORDER="+xBorder[iXY]+"|"+iBorder);
                //System.out.println("//////YBORDER="+yBorder[iXY]+"|"+(iBorder+1));
                }
        } else {
            //shape is just a point

            xBorder = new float[]{(float)sp[0][0]};
            yBorder = new float[]{(float)sp[1][0]};
            //System.out.println("******XBORDER="+xBorder[0]);
            //System.out.println("//////YBORDER="+yBorder[0]);
            }
	// if(spline[0].length != 1) {	
	// 	//compute flow border
	// 	computeBorder(spline,this.width);
	// } else{
	// 	//shape is just a point
	// 	xBorder = new int[]{spline[0][0]};
	// 	yBorder = new int[]{spline[1][0]};
	// }

        setNeedToRegenerate(false);
        createShape();

        return true;
    }

    @Override
    public GeneralPath getShape() {
        if (shape == null) {
            createShape();
        }
        return shape;

    }

    private void createShape() {
        shape = super.createShape(xBorder, yBorder, true);
    }


    private int computeNbPolylinePerArc() {
        int diffx = 0;
        int diffy = 0;
        if (isStartEqualsToEnd()) {
            diffx = 200;
            diffy = 200;
        } else {
            diffx = (int) pStart.getX() - (int) pEnd.getX();
            diffy = (int) pStart.getY() - (int) pEnd.getY();
        }
        int dist = (int) (Math.sqrt(diffx * diffx + diffy * diffy));
        return dist / ARC_LENGTH;
    }

    public static double[][] deleteDouble(double[][] aSpline) {
        ArrayList<Point.Double> listPoints = new ArrayList<Point.Double>();

        double[] xsTmp = aSpline[0];
        double[] ysTmp = aSpline[1];
        //create points to check doubles when adding into the list
        Point.Double pCurrent = null;
        Point.Double pBefore = null;
        for (int i = 0; i < xsTmp.length; i++) {
            pCurrent = new Point.Double(xsTmp[i], ysTmp[i]);
            if (i == 0) {
                listPoints.add(pCurrent);
                pBefore = pCurrent;
                continue;
            } else {
                if (!listPoints.contains(pCurrent) && !equalsXorY(pCurrent, pBefore)) {
                    listPoints.add(pCurrent);
                    pBefore = pCurrent;
                }

            }
        }

        //array with no doubles
        double[] newxs = new double[listPoints.size()];
        double[] newys = new double[listPoints.size()];
        int index = 0;

        for (Point.Double p : listPoints) {
            newxs[index] = p.getX();
            newys[index] = p.getY();
            index++;
        }

        return new double[][]{newxs, newys};
    }

    private static boolean equalsXorY(Point.Double p1, Point.Double p2) {
        return p1.getX() == p2.getX() ||  p1.getY() ==  p2.getY();
    }

    private Float[] computeBorder(double[][] spline, int width) {
        try {
            double[] xSkeletonPoints = spline[0];
            double[] ySkeletonPoints = spline[1];

            IndexForArrowHead indexStartArrowHead =
                    new IndexForArrowHead(xSkeletonPoints, ySkeletonPoints,
                    arrowStart.getLength() * width, arrowStart.getDepth() * width,
                    0, xSkeletonPoints.length, 1);
            IndexForArrowHead indexEndArrowHead =
                    new IndexForArrowHead(xSkeletonPoints, ySkeletonPoints,
                    arrowEnd.getLength() * width, arrowEnd.getDepth() * width,
                    xSkeletonPoints.length - 1, -1, -1);

            int bodyLength = Math.abs(indexEndArrowHead.getIDepth() - indexStartArrowHead.getIDepth() + 1);
            //the complete arrow border
            //int[] border = new int[2*(2*(bodyLength + 3)+1)];
            ArrayList<Float> border = new ArrayList<Float>();
            //lower edge
            //int[] lowerEdge = new int[2*(bodyLength+3)];
            ArrayList<Float> lowerEdge = new ArrayList<Float>();
            //the arrow body half width
            int bodyHalfWidth = width;
            int iBorder = 0; // index used to fill the border array (and the lowerEdge)
            // the real index is in fact 2*iBorder
            //insert the start point
            ////border[2*iBorder] = xSkeletonPoints[0];
            ////border[2*iBorder+1] = ySkeletonPoints[0];
            border.add(new Float(xSkeletonPoints[0]));
            border.add(new Float(ySkeletonPoints[0]));
            // that is also the end point of the border or the first point of lowerEdge
            ////lowerEdge[2*iBorder] = xSkeletonPoints[0];
            ////lowerEdge[2*iBorder+1] = ySkeletonPoints[0];
            lowerEdge.add(new Float(xSkeletonPoints[0]));
            lowerEdge.add(new Float(ySkeletonPoints[0]));
            iBorder++;

            //insert the start arrow head
            iBorder = compute2BorderPoints(xSkeletonPoints, ySkeletonPoints,
                    indexStartArrowHead.getILength(),
                    border, lowerEdge, iBorder, arrowStart.getWidth() * width);
            // insert the body
            for (int i = indexStartArrowHead.getIDepth(); i <= indexEndArrowHead.getIDepth(); i++) {
                iBorder = compute2BorderPoints(xSkeletonPoints, ySkeletonPoints,
                        i,
                        border, lowerEdge, iBorder, bodyHalfWidth);
            }

            // insert the end arrow head
            iBorder = compute2BorderPoints(xSkeletonPoints, ySkeletonPoints,
                    indexEndArrowHead.getILength(),
                    border, lowerEdge, iBorder, arrowEnd.getWidth() * width);

            // insert the end point
            //// border[2*iBorder] = (int)pEnd.getX();
            //// border[2*iBorder+1] = (int)pEnd.getY();
            border.add((float)pEnd.getX());
            border.add((float)pEnd.getY());
            //System.out.println((int)pEnd.getX());
            //System.out.println((int)pEnd.getY());
            iBorder++;


            // // add the lowerEdge to the border (in reverse order)
	    // for (int i = iBorder-1-1; i >= 0; i--, iBorder++) {
	    //     border[2*iBorder]   = lowerEdge[2*i];
	    //     border[2*iBorder+1] = lowerEdge[2*i+1];
	    //
	    // }

            for (int i = lowerEdge.size() - 2; i >= 0; i -= 2) {
                border.add(lowerEdge.get(i));
                border.add(lowerEdge.get(i + 1));
            }
            Float borderArray[] = new Float[border.size()];
            return border.toArray(borderArray);
            //return border.toArray(borderArray);
        } catch (ArrayIndexOutOfBoundsException exc) {
            //return new int []{spline[0][0], spline[1][0], spline[0][spline[0].length-1], spline[1][spline[1].length-1]};
            //exc.printStackTrace();
            return null;
        }
        //return null;
    }

    private int compute2BorderPoints(double[] xRealSkeleton, double[] yRealSkeleton,
            int iSkeleton,
            ArrayList<Float> upperPoints, ArrayList<Float> lowerPoints,
            int iPoints, double width) {

        double[] radialVector;

        if (iSkeleton == xRealSkeleton.length - 1) {
            radialVector = computeRadial(xRealSkeleton[iSkeleton - 1], yRealSkeleton[iSkeleton - 1],
                    xRealSkeleton[iSkeleton], yRealSkeleton[iSkeleton]);
        } else {
            radialVector = computeRadial(xRealSkeleton[iSkeleton], yRealSkeleton[iSkeleton],
                    xRealSkeleton[iSkeleton + 1], yRealSkeleton[iSkeleton + 1]);
        }

        upperPoints.add(new Float(xRealSkeleton[iSkeleton] + radialVector[0] * width));
        upperPoints.add(new Float(yRealSkeleton[iSkeleton] + radialVector[1] * width));
        lowerPoints.add(new Float(xRealSkeleton[iSkeleton] - radialVector[0] * width));
        lowerPoints.add(new Float(yRealSkeleton[iSkeleton] - radialVector[1] * width));
        radials.add(new OMLine(
                (int) (xRealSkeleton[iSkeleton] + radialVector[0] * width),
                (int) (yRealSkeleton[iSkeleton] + radialVector[1] * width),
                (int) (xRealSkeleton[iSkeleton] - radialVector[0] * width),
                (int) (yRealSkeleton[iSkeleton] - radialVector[1] * width)
                ));
        return iPoints + 1;
    }

    /**
     * Computes the norm of the 2 dimensions vector
     */
    protected double norm(double[] v) {
        return (double) Math.sqrt(v[0] * v[0] + v[1] * v[1]);
    }

    /**
     * Normalizes the vector, i.e. the result has a norm equal to 1
     */
    protected void normalizedouble(double[] v) {
        double n = norm(v);
        v[0] /= n;
        v[1] /= n;

    }

    /**
     * Computes a normalized othogonal vector to the given segment coordinates
     * given as arrays
     */
    protected double[] computeRadial(double[] from, double[] to) {
        double[] v = {to[1] - from[1], from[0] - to[0]};
        normalizedouble(v);
        return v;
    }

    /**
     * Computes a normalized othogonal vector to the given segment
     */
    protected double[] computeRadial(double from0, double from1, double to0, double to1) {
        double[] v = {to1 - from1, from0 - to0};
        normalizedouble(v);
        return v;
    }

    protected double[] defaultRadial() {
        double[] v = {1f, 1f};
        return v;
    }

    public OMGraphicList getControlPoints() {
        return controlPoints;
    }

    /**
     * Compute the mid ref point for curved arrows
     */
    protected Point2D computeRefCurvedPoint(Point2D a, Point2D b) {
        if (flow.existsCurvedPointForOMFlow(id) && BENDED_MODE == BENDED_MANUAL_MODE) {
            LatLonPoint pt = flow.getCurvedPointForOMFlow(id);
            if (flow.isControlPointsVisibility()) {
                controlPoints.add(new OMFlowControlPoint(0, pt, this));
            }
            return flow.getProjection().forward(pt);
        } else {
            double[] mid = {((double) a.getX() + (double) b.getX()) / 2, ((double) a.getY() + (double) b.getY()) / 2};
            double[] radial = computeRadial(new double[]{(double) a.getX(), (double) a.getY()}, mid);
            double[] ref;
            if (legend) {
                ref = new double[]{mid[0] + 0 * radial[0],
                            mid[1] + 0 * radial[1]};
            } else {
                ref = new double[]{mid[0] + BENDED_DISTANCE * radial[0],
                            mid[1] + BENDED_DISTANCE * radial[1]};
            }
            Point pRef = new Point((int) ref[0], (int) ref[1]);
            LatLonPoint llptRef = flow.getProjection().inverse(pRef);
            if (flow.isControlPointsVisibility()) {
                controlPoints.add(new OMFlowControlPoint(0, llptRef, this));
            }
            flow.setCurvedPointForOMFlow(id, llptRef);
            return pRef;
        }
    }

    public void updateCurvedPoint(int x, int y) {
        flow.setDragging(true);
        flow.setCurvedPointForOMFlow(id, (LatLonPoint) (flow.getProjection().inverse(new Point(x, y))));
        flow.setBadCache(true);
        flow.setDragging(false);
    }

    public void startDrag() {
        flow.addInteractiveGraphic(this);
    }

    public void endDrag() {
        flow.clearInteractiveGraphics();
        flow.setBadCache(true);
        flow.getLayer().refreshGraphicsAndGUI();
    }

    private boolean isStartEqualsToEnd() {
        return (int)pStart.getX() ==  (int)pEnd.getX()
	    &&  (int)pStart.getY() ==  (int)pEnd.getY();
    }

    public int getWidth() {
        return (int) (this.width + 0.5) * 2;
    }

    public int compareTo(OMFlow o) {
        return (int) Math.signum(this.getWidth() - o.getWidth());
    }

    public Point2D getPEnd() {
        return pEnd;
    }

    public void setPEnd(Point end) {
        pEnd = end;
    }

    public Point2D getPStart() {
        return pStart;
    }

    public void setPStart(Point start) {
        pStart = start;
    }

    public float[] getXBorder() {
        return xBorder;
    }

    public float[] getYBorder() {
        return yBorder;
    }

    public int[] getXBorderAsInt() {
        int[] tab = new int[xBorder.length];
        for (int i = 0; i < xBorder.length; i++) {
            tab[i] = (int) xBorder[i];
        }
        return tab;
    }

    public int[] getYBorderAsInt() {
        int[] tab = new int[yBorder.length];
        for (int i = 0; i < yBorder.length; i++) {
            tab[i] = (int) yBorder[i];
        }
        return tab;
    }

    public void updatePosition(int x, int y) {
        updateCurvedPoint(x, y);
        flow.getLayer().repaint();
    }

    private class IndexForArrowHead {
        // index of points in skeleton with distance to origin stricly lower than lenght or depth

        int iLength, iDepth;

        IndexForArrowHead(double[] xSkeleton, double[] ySkeleton, double length, double depth, int orig, int extr, int step) {
            // orig and extr are the index of resp. origin and extremity in the skeleton arrays
            // step can be +1 or -1

            iLength = iDepth = -1;

            if (length <= 0) {
                iLength = orig;
            }
            if (depth <= 0) {
                iDepth = orig;
            }

            if (iLength < 0 || iDepth < 0) {
                for (int i = orig + step; i != extr; i += step) {

                    double x = xSkeleton[i] - xSkeleton[orig];
                    double y = ySkeleton[i] - ySkeleton[orig];
                    double distance = (double) Math.sqrt(x * x + y * y);

                    if (iLength < 0 && distance >= length) {
                        iLength = i;
                        if (iDepth >= 0) {
                            break;
                        }
                    }

                    if (iDepth < 0 && distance >= depth) {
                        iDepth = i;
                        if (iLength >= 0) {
                            break;
                        }
                    }

                }
            }

        }

        int getILength() {
            return iLength;
        }

        int getIDepth() {
            return iDepth;
        }
    }

    public String getID() {
        return id;
    }

    private OMGraphic getOMSpline() {
        int[] dxs = new int[xs.length];
        int[] dys = new int[ys.length];
        for(int i = 0 ;i < xs.length; i++){
            dxs[i] = (int)xs[i];
            dys[i] = (int)ys[i];
        }
        return new OMSpline(dxs, dys);
    }

    public OMGraphicList getRadials(){
        return radials;
    }

    private void putData(Object data) {
        putAttribute(ThematicDataDisplayLayer.VALUE, data);
    }

    public Object getData() {
        return getAttribute(ThematicDataDisplayLayer.VALUE);
    }

    public Point2D getAnchorForLabel() {
	return null;
    }

}
