// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics;

import com.bbn.openmap.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.MultiColumnsRep.Orientation;
import java.awt.Color;
import java.awt.Paint;

/**
 * Graph visu of an histogram.
 */
public class OMGraph extends OMGraphicList implements Comparable<OMGraph> {

    public OMGraph(double latPoint, double lonPoint, double barsWitdh, Paint lineColor, Paint axesColor, double[] barsHeights, int max) {
	this(latPoint, lonPoint, 0, 0, barsWitdh, lineColor, axesColor, barsHeights, max, Orientation.UP);
    }
    
    public OMGraph(double latPoint, double lonPoint, int xOffset, int yOffset, double barsWitdh, Paint lineColor, Paint axesColor, double[] barsHeights, int max, Orientation orientation) {
        addGraphics(xOffset, yOffset, latPoint, lonPoint, barsWitdh, lineColor, axesColor, barsHeights, max, orientation);
    }

    protected void addGraphics(int xOffset, int yOffset, double latPoint, double lonPoint, double barsWitdh, Paint lineColor, Paint axesColor, double[] barsHeights, int max, Orientation orientation) {
        int px1 = (int) ((barsWitdh * barsHeights.length) * (-0.5));
        int px2 = px1 + (int) barsWitdh;
        int px0 = px1;
        for (int i = 0; i < barsHeights.length - 1; i++, px1 = px2, px2 += barsWitdh) {
	    OMLine omLine = null;
	    switch (orientation) {
	    case UP:
		omLine = new OMLine
		    (latPoint, lonPoint,
		     xOffset + (int) (px1 + barsWitdh / 2), yOffset + (int) -barsHeights[i],
		     xOffset + (int) (px1 + barsWitdh + barsWitdh / 2), yOffset + (int) -barsHeights[i + 1]);
		break;
	    case RIGHT:
		omLine = new OMLine
		    (latPoint, lonPoint,
		     xOffset +  (int) barsHeights[i], yOffset + (int) (px1 + barsWitdh / 2),
		     xOffset + (int) barsHeights[i + 1], yOffset + (int)(px1 + barsWitdh + barsWitdh / 2));
		break;
	    case LEFT:
		omLine = new OMLine
		    (latPoint, lonPoint,
		     xOffset +  (int) -barsHeights[i], yOffset + (int) (px1 + barsWitdh / 2),
		     xOffset + (int) -barsHeights[i + 1], yOffset + (int)(px1 + barsWitdh + barsWitdh / 2));
		break;
	    case DOWN:
		omLine = new OMLine
		    (latPoint, lonPoint,
		     xOffset + (int) (px1 + barsWitdh / 2), yOffset + (int) barsHeights[i],
		     xOffset + (int) (px1 + barsWitdh + barsWitdh / 2), yOffset + (int) barsHeights[i + 1]);
		break;
	    }
	    omLine.setLinePaint(lineColor);
            this.add(omLine);
        }
        // add the 2 axis
	OMLine axis1 = null, axis2 = null;
	switch (orientation) {
	case UP:
	    axis1 = new OMLine(latPoint, lonPoint, xOffset+px0, yOffset-max, xOffset+px0, yOffset);
	    axis2 = new OMLine(latPoint, lonPoint, xOffset+px0, yOffset, xOffset+px2, yOffset);
	    break;
	case RIGHT:
	    axis1 = new OMLine(latPoint, lonPoint, xOffset+max, yOffset+px2, xOffset, yOffset+px2);
	    axis2 = new OMLine(latPoint, lonPoint, xOffset, yOffset+px0, xOffset, yOffset+px2);
	    break;
	case LEFT:
	    axis1 = new OMLine(latPoint, lonPoint, xOffset-max, yOffset+px2, xOffset, yOffset+px2);
	    axis2 = new OMLine(latPoint, lonPoint, xOffset, yOffset+px0, xOffset, yOffset+px2);
	    break;
	case DOWN:
	    axis1 = new OMLine(latPoint, lonPoint, xOffset+px0, yOffset+max, xOffset+px0, yOffset);
	    axis2 = new OMLine(latPoint, lonPoint, xOffset+px0, yOffset, xOffset+px2, yOffset);
	    break;
	}
        axis1.setLinePaint(axesColor);
        this.add(axis1);
	axis2.setLinePaint(axesColor);
        this.add(axis2);
    }

    public int compareTo(OMGraph omg) {
	// return (int)Math.signum(this.max - omg.max); -> always 0!
        return 0;
    }
}
