// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics;

import com.bbn.openmap.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.MultiColumnsRep.Orientation;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Offsetable;
import java.awt.Paint;
import java.awt.Point;
import java.awt.geom.Point2D;

/**
 * An Histogram given the commun bar width, the bars heights and a color scale
 * centered on Point(latPoint, lonPoint).
 */
public class OMHistogram extends OMOffsetable implements Comparable<OMHistogram> {

    private double maxHeight = 0;
    private double barsWidth;
    private OMGraphicList bars;

    public OMHistogram(Offsetable off, String id, double latPoint, double lonPoint, int xOffset, int yOffset, double barsWitdh, Paint lineColor, double[] barsHeights, ColorScale colorScale, int max, Orientation orientation, boolean v, boolean l) {
	// compute maxHeight of this histogram, 
	// max is the max height of all the histograms build by this layer
        super(off, id, latPoint, lonPoint, xOffset, yOffset, v, l);
	maxHeight = 0;
        bars = new OMGraphicList();
        this.barsWidth = barsWitdh;
	for (int i = 0; i < barsHeights.length; i++) {
	    if (barsHeights[i] > maxHeight)
		maxHeight = barsHeights[i];
	}
	addGraphics(xOffset, yOffset, latPoint, lonPoint, barsWitdh, lineColor, barsHeights, colorScale, max, orientation);
        init(xOffset, yOffset);
    }

    protected void addGraphics(int xOffset, int yOffset, double latPoint, double lonPoint, double barsWitdh, Paint lineColor, double[] barsHeights, ColorScale colorScale, int max, Orientation orientation) {
	// p1, p2 are x-offsets of the first bar for UP orientation, y-offsets for LEFT and RIGHT.
	int p1 = (int)((barsWitdh * barsHeights.length) * (-0.5));
	int p2 = p1 + (int)barsWitdh;
	OMRect omBar = null;
	for (int i = 0; i < barsHeights.length; i++, p1 = p2, p2 += barsWitdh) {
	    switch (orientation) {
	    case UP:
		omBar = new OMRect(latPoint, lonPoint,
				   p1 + xOffset, yOffset + (int)-barsHeights[i],
				   p2 + xOffset, yOffset);
		break;
	    case RIGHT:
		omBar = new OMRect(latPoint, lonPoint,
				   xOffset + (int)barsHeights[i], p1 + yOffset, 
				   xOffset, p2 + yOffset);
		break;
	    case LEFT:
		omBar = new OMRect(latPoint, lonPoint,
				   xOffset + (int)-barsHeights[i], p1 + yOffset, 
				   xOffset, p2 + yOffset);
		break;
	    case DOWN:
		omBar = new OMRect(latPoint, lonPoint,
				   p1 + xOffset, yOffset + (int)barsHeights[i],
				   p2 + xOffset, yOffset);
		break;
	    }
	    omBar.setFillPaint(colorScale.getColor(i));
	    omBar.setLinePaint(lineColor);
            bars.add(omBar);
	    //this.add(omBar);
	}
        this.add(bars);
    }

    public void setFillPaint(ColorScale colorScale) {
	for(int i = 0; i < size(); i++)
	    getOMGraphicAt(i).setFillPaint(colorScale.getColor(i));
    }
    
    public int compareTo(OMHistogram omh) {
	return (int)Math.signum(this.maxHeight - omh.maxHeight);
    }

    public int getHeight() {
	return (int) maxHeight;
    }

    public int getWidth() {
	return (int) (size() * barsWidth);
    }

    //TODO
    public void setLocation(int x, int y){
        for(OMGraphic omg : bars){
            OMRect omr = (OMRect)omg;
            int height = Math.abs(omr.getBottom() - omr.getTop());
            omr.setLocation(x - (int)barsWidth / 2, y - height, x + (int)barsWidth, y);
        };
    }

    @Override
    public void translate(int x, int y) {
        super.translate(x, y);
        for(OMGraphic omg : bars){
            OMRect omr =(OMRect)omg;
            omr.setLocation(getLat(),
                    getLon(),
                    x + omr.getLeft(),
                    y + omr.getTop(),
                    x + omr.getRight(),
                    y + omr.getBottom());           
        }
    }

}
