// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.proj.coords.LatLonPoint;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Offsetable;
import java.awt.Point;
import java.awt.geom.Point2D;

/**
 *
 * @author jchauvea
 */
public class OMMultiRegularPolygon extends OMOffsetable {

    private int width;
    private OMGraphicList polygons;

    public OMMultiRegularPolygon(Offsetable off, String id, int nb, double lat, double lon,
            int nbEdges, double radius, int offX, int offY,
            double startAngle, int starCoef, int gapPercentage, boolean cpVisibility, boolean l) {
        super(off, id, lat, lon, offX, offY, cpVisibility, l);
        int gap = (int) (radius * gapPercentage / 100.0);
        int polyWidth = (int) (2 * radius + gap); // place needed for one poly
        polygons = new OMGraphicList();
        // nbSqrt is the edge of the biggest possible full square
        int nbSqrt = (int) Math.round(Math.sqrt(nb));
        width = polyWidth * nbSqrt;

        // try to center the group of polygons
	// (x0, y0) are the coodinates of the upper leftmost polygon
        int x0 = offX - polyWidth * (nbSqrt-1) / 2;
        int y0 = offY - polyWidth * (nbSqrt-1) / 2;

	
        // draw a square of regular polygons
	// do not pass the Offsetable to the little polygons as they are not individually positionnable
        for (int x = 0; x < nbSqrt; x++) {
            for (int y = 0; y < nbSqrt && nb > 0; y++, nb--) {
                polygons.add(new OMRegularPolygon(null, "", lat, lon, nbEdges, radius,
                        x0 + x * polyWidth,
                        y0 + y * polyWidth,
                        startAngle, starCoef, false, l));
            }
        }

        // we start by the rigth most vertical line
        for (int y = 0; nb > 0 && y < nbSqrt; nb--, y++) {
            polygons.add(new OMRegularPolygon(null, "", lat, lon, nbEdges, radius,
                    x0 + nbSqrt * polyWidth,
                    y0 + y * polyWidth,
                    startAngle, starCoef, false, l));
        }
        // we end by the bottom horizontal line
        for (int x = 0; nb > 0 && x < nbSqrt; nb--, x++) {
            polygons.add(new OMRegularPolygon(null,"", lat, lon, nbEdges, radius,
                    x0 + x * polyWidth,
                    y0 + nbSqrt * polyWidth,
                    startAngle, starCoef, false, l));

        }
        add(polygons);
        init(offX, offY);
    }


    public int getHeight() {
	return width;
    }

    public int getWidth() {
	return width;
    }

    @Override
    public void translate(int x, int y) {
        super.translate(x, y);
          for(OMGraphic om : polygons){
            OMRegularPolygon p = (OMRegularPolygon)om;
            p.translate(x, y);
        }
    }

}
