// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMLine;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Offseter;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.OffsetableListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Offsetable;
import java.util.ArrayList;

/**
 *
 * @author dje
 */
public abstract class OMOffsetable extends OMGraphicList {

    public static final int MARGIN = 2;
    protected Offsetable offsetable = null;
    protected OMOffsetableControlPoint controlPoint;
    private boolean startDrag = false;
    private int x0; // to be moved in the mouse controller
    private int y0;// to be moved in the mouse controller
    protected String id = null;
    private int xOffset, yOffset; // allready in the OMGraphic, but so convienent here also
    private double lat, lon;// same remark as for offsets

    public static enum Position {
        NORTH, WEST, SOUTH, EAST
    };

   public OMOffsetable() {
        id = "";
    }

    /**  Full Lat/Lon render type (no x/y offset is possible) */ 
   public OMOffsetable(double lat, double lon) {
       this.lat = lat;
       this.lon = lon;    
   } 

    /** XY RENDER TYPE attached to a latlon point */
    public OMOffsetable(Offsetable off, String id, double lat, double lon, int offX, int offY, boolean cpVisibility, boolean lineVisibility) {
        this.offsetable = off;
        this.id = id;
        this.xOffset = offX;
        this.yOffset = offY;
        this.lat = lat;
        this.lon = lon;
    }

    public void init(int offX, int offY) {
	setVague(true);
	if (offsetable == null)
	    return;
	if (offsetable.getOffseter().isVisible()) {
	    controlPoint = new OMOffsetableControlPoint(lat, lon, this, offX, offY);
	    add(0, controlPoint);
	    offsetable.getMouseControlPointsListener().addOMControlPoints(controlPoint);
	}
        if (offsetable.getOffseter().isLineVisible())
	    drawJoiningLine(offX, offY);
    }

    public void updatePosition(int x, int y) {
        if(startDrag){
            x0 = x - getX();
            y0 = y - getY();
            startDrag = false;
        }
        translate(x - x0 - getX(), y - y0 - getY());
	if (offsetable != null) {
	    //System.out.println(getClass() + " -> UPDATE POSITION!!!!!");
	    offsetable.getOffseter().setOffset(id, x - x0, y - y0);
	    offsetable.getLayer().repaint();
	}
    }

    public void startDrag() {
        if (offsetable != null) {
	    // System.out.println("<<< startDrag "+offsetableRaw);
            startDrag = true;
            offsetable.addInteractiveGraphic(this);
        }
    }

    public void endDrag() {
        if (offsetable != null) {
	    // System.out.println(">>> endDrag "+offsetableRaw);
            startDrag = false;
            offsetable.clearInteractiveGraphics();
	    offsetable.refresh();
        }
    }

    private Position getObjectPosition(int x, int y) {
	boolean ne = y >= x;
	boolean nw = y >= -x;
	if (ne)
	    return nw ? Position.NORTH : Position.EAST;
	return nw ? Position.WEST : Position.SOUTH;
    }

    /** draws a line connecting the lat/lon center with the object centered at x,y offset */ 
    private void drawJoiningLine(int x, int y) {
	Position pos = getObjectPosition(x, y);
	
	// correct x, y to be at oppposite position of the object from the lat/lon point
	switch (pos) {
	case NORTH:
	    y -= getHeight() / 2 + MARGIN;
	    break;
	case SOUTH:
	    y += getHeight() / 2 + MARGIN;
	    break;
	case WEST:
	    x -= getWidth() / 2 + MARGIN;
	    break;
	case EAST:
	    x += getWidth() / 2 + MARGIN;
	    break;
	}
	add(new OMLine(lat, lon, 0, 0, x, y));
    }

    public abstract int getHeight();
    public abstract int getWidth();

    public void translate(int x, int y){
        xOffset += x;
        yOffset += y;
    }
    public int getX(){return xOffset;}
    public int getY(){return yOffset;}
    public double getLat(){return lat;}
    public double getLon(){return lon;}

}
