// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics;

import java.awt.Image;

import javax.swing.ImageIcon;

import com.bbn.openmap.omGraphics.OMRaster;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Offsetable;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.util.Util;

public class OMPictogram extends OMOffsetableWithData implements Comparable<OMPictogram> {

    OMRaster pictogram;

    public int compareTo(OMPictogram o) {
	return getWidth() - o.getWidth(); 
    }

    public OMPictogram(Offsetable off, String id, double lat, double lon, int offx, int offy,  String pictoPath, int width, Object data, boolean cp, boolean l){
	this(off, id, lat, lon, offx, offy, Util.loadIcon(pictoPath), width, data, cp, l);
    }

    public OMPictogram(Offsetable off, String id, double lat, double lon, int offx, int offy,  ImageIcon pictoIcon, int width, Object data, boolean cp, boolean l){
	super(off, id, lat, lon, offx, offy, cp, l, data);
	pictogram = new OMRaster(lat, lon, 0, 0,
				 scaleImageIcon(pictoIcon, width));
	//translate(offx - pictogram.getWidth()/2, offy-pictogram.getHeight()/2);
        pictogram.setX(offx - pictogram.getWidth()/2);
        pictogram.setY(offy-pictogram.getHeight()/2);
	
	add(pictogram);
	init(offx, offy);
    }

    /**
     * Scale the ImageIcon
     */
    private static ImageIcon scaleImageIcon(ImageIcon im, int width){
	if(width<1)  width=1;
	Image scaled = im.getImage().getScaledInstance(width, -1, Image.SCALE_FAST);
	im.setImage(scaled);
	return im;
    }

    public int getWidth() {
        return pictogram.getWidth();
    }
    public int getHeight() {
        return pictogram.getHeight();
    }


    @Override
	public void translate(int x, int y) {
        super.translate(x, y);
        pictogram.setX(getX());
        pictogram.setY(getY());
    }

}
