// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics;

import java.awt.geom.Arc2D;

import com.bbn.openmap.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Offsetable;
import java.awt.Point;
import java.awt.geom.Point2D;

public class OMPie extends OMOffsetableWithData implements Comparable<OMPie> {

    private double width;
    private OMGraphicList arcs;
    /** Full Lat/Lon render type */ 
    public OMPie(Offsetable off, String id, double latPoint, double lonPoint, double radius, int nbParts, double[] angles, ColorScale colorScale, Object data) {
        super(latPoint, lonPoint, data);
        this.width = 2 * radius;
        arcs = new OMGraphicList();
        double startAngle = 0.0f;
        for (int i = 0; i < nbParts; i++) {
            double angle = angles[i];
            //// full lat/lon render type
            OMArc arc = new OMArc(latPoint, lonPoint, radius, startAngle, angle);
            arc.setArcType(Arc2D.PIE);
            arc.setFillPaint(colorScale.getColor(i));
            arcs.add(arc);
            startAngle += angle;
        }
        this.add(arcs);
    }

    /**
     * XY RENDER TYPE attached to a latlon point
     * @param latPoint
     * @param lonPoint
     * @param diameter
     * @param nbParts
     * @param angles
     * @param colorScale
     * @param data
     */
    public OMPie(Offsetable off, String id, double latPoint, double lonPoint, int xOffset, int yOffset, int diameter, int nbParts, double[] angles, ColorScale colorScale, Object data, boolean v, boolean l) {
        super(off, id, latPoint, lonPoint, xOffset, yOffset, v, l, data);
        this.width = diameter;
        arcs = new OMGraphicList();
        double startAngle = 0.0f;
        for (int i = 0; i < nbParts; i++) {
            double angle = (double) angles[i];
            // RENDERTYPE_XY
            OMArc arc = new OMArc(latPoint, lonPoint, xOffset, yOffset, diameter, diameter, startAngle, angle);
            arc.setArcType(Arc2D.PIE);
            arc.setFillPaint(colorScale.getColor(i));
            arcs.add(arc);
            startAngle += angle;
        }
        this.add(arcs);
        init(xOffset, yOffset);
    }

    /**
     * Full XY RENDER TYPE
     * @param x
     * @param y
     * @param diameter
     * @param nbParts
     * @param angles
     * @param colorScale
     * @param data
     */
    public OMPie(int x, int y, int diameter, int nbParts, double[] angles, ColorScale colorScale, Object data) {
	super(data);
	this.width = diameter;
        arcs = new OMGraphicList();
        double startAngle = 0.0f;
        for (int i = 0; i < nbParts; i++) {
            double angle = angles[i];
            // RENDERTYPE_XY
            OMArc arc = new OMArc(x, y, diameter, diameter, startAngle, angle);
            arc.setArcType(Arc2D.PIE);
            arc.setFillPaint(colorScale.getColor(i));
            arcs.add(arc);
            startAngle += angle;
        }
        this.add(arcs);
    }

    public void setFillPaint(ColorScale colorScale) {
        for (int i = 0; i < size(); i++) {
            getOMGraphicAt(i).setFillPaint(colorScale.getColor(i));
        }
    }

    public int compareTo(OMPie omd) {
        return (int) Math.signum(this.width - omd.width);
    }

    public int getWidth() {
        return (int) width;
    }
    public int getHeight() {
        return (int) width;
    }

    @Override
    public void translate(int x, int y) {
        super.translate(x, y);
        for (OMGraphic omg : arcs) {
            OMArc oma = (OMArc) omg;
            oma.setOffX(getX());
            oma.setOffY(getY());
        }
    }
}
