// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics;

import java.util.*;

import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMGraphic;

/**
 * A OMGraphicList that can be sorted by the size of their elements
 * <p> OMSizeable must extends OMGraphic and implements Comparable&lt;OMSizeable&gt;
 * <br>It should be possible to say it to the compiler, but ... !
 * <p>Added elements are actually added to the OMGraphicList only after sort() is called.
 */
public class OMSizeableGraphicList<OMSizeable extends OMGraphic> extends OMGraphicList {

    ArrayList<OMGraphic> sortedGraphics = new ArrayList<OMGraphic>();
    ArrayList<OMGraphic> noSizeGraphics = new ArrayList<OMGraphic>();

    /**
     * Add an OMSizeable to the list.
     */
    @Override
    public boolean add(OMGraphic g) {
	if (! (g instanceof HaveWidthAndData))
	    return noSizeGraphics.add(g);
	return sortedGraphics.add(g);
    }

    /**
     * Sort the OMSizeable graphics added and merge the one having no width and data.
     */
    @Override
    public void sort() {
        graphics.clear();	
	graphics.addAll(noSizeGraphics);
        Collections.sort(sortedGraphics, new Comparator<Object>() {
            public int compare(Object o1, Object o2) {
                if(((HaveWidthAndData)o1).getWidth() == ((HaveWidthAndData)o2).getWidth())
                    return 0;
                else
                   return  ((HaveWidthAndData)o1).getWidth() > ((HaveWidthAndData)o2).getWidth() ? 1 : -1;
            }
        });
	graphics.addAll(sortedGraphics);
    }

    @Override
    public String toString(){
        String s ="";
        for(OMGraphic omg : sortedGraphics){
            s+= ((HaveWidthAndData)omg).getWidth() + "\n";
        }
        return s;
    }
}
