// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics;

import com.bbn.openmap.Environment;
import com.bbn.openmap.omGraphics.BasicStrokeEditorMenu;
import com.bbn.openmap.omGraphics.OMColor;
import com.bbn.openmap.omGraphics.OMColorChooser;
import fr.unicaen.openmap.gui.ThemaMapColorChooser;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

/**
 *
 * @author jchauvea
 */
public class ThemaMapStrokeEditorMenu extends BasicStrokeEditorMenu {

    protected Color lineColor;

    public ThemaMapStrokeEditorMenu(BasicStroke bs, Color lineColor) {
        super(bs);
        this.lineColor = lineColor;
    }

    @Override
    public JPopupMenu setGUI(JPopupMenu popup) {
        JMenuItem colorMenu = new JMenuItem(Environment.getI18n().get(
                ThemaMapStrokeEditorMenu.class,
                "color", "Color"));

        ActionListener listener = new ActionListener() {

            public void actionPerformed(ActionEvent ae) {
                //lineColor = OMColorChooser.showDialog(ThemaMapStrokeEditorMenu.this, "", lineColor);
                lineColor = ThemaMapColorChooser.showDialog(
                        ThemaMapStrokeEditorMenu.this,
                        "", lineColor);
                resetStroke();

            }
        };
        colorMenu.addActionListener(listener);
        popup.add(colorMenu);
        return super.setGUI(popup);

    }

    @Override
    public void resetStroke() {
        
        setMiterLimit(miterLimit);
        basicStroke = new BasicStroke(width, endCaps, lineJoins, miterLimit, dash, dashPhase);
        if (launchButton != null) {
            launchButton.setIcon(createIcon(basicStroke, lineColor, 50, 20, true));
        }
        pcs.firePropertyChange("line", null, this);

    }

    public static ImageIcon createIcon(BasicStroke stroke, Color lineColor, int width,
            int height, boolean horizontalOrientation) {

        BufferedImage bigImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
        Graphics2D g = (Graphics2D) bigImage.getGraphics();

        g.setBackground(OMColor.clear);
        g.setPaint(OMColor.clear);
        g.fillRect(0, 0, width, height);
        g.setPaint(lineColor);
        g.setStroke(stroke);
        if (horizontalOrientation) {
            g.drawLine(0, height / 2, width, height / 2);
        } else {
            g.drawLine(width / 2, 0, width / 2, height);
        }

        return new ImageIcon(bigImage);
    }

    public BasicStroke getStroke(){return basicStroke;}

    public Color getLineColor(){return lineColor;}
}
