// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import java.util.ArrayList;
import java.util.Properties;
import java.util.regex.Pattern;

public class OMResizeableColorScaleLegend extends OMResizeableFrame {

    public  int rectWidth = 40, rectHeight = 15;
    private ColorScale colorScale;
    private String[] colorNames;
    protected ArrayList<OMResizeableText> texts = new ArrayList<OMResizeableText>();

    public OMResizeableColorScaleLegend(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, ColorScale colorScale, String[] colorNames, String title, String nameForGUI) {
        super(r, ml, xAnchor, yAnchor, 0, 0, title, nameForGUI, false); // we'll set width and height at the end of the ctor.
        init(colorScale, colorNames);
    }

    protected void init() {
        init(colorScale, colorNames);
    }

    public void init(ColorScale colorScale, String[] colorNames) {
        Pattern tagHTML = Pattern.compile("<[^>]*>");
        this.colorScale = colorScale;
        this.colorNames = colorNames;
        int x0 = getXAnchor() + (int) (0.5 * rectHeight), y0 = getYAnchor() + (int) (0.5 * rectHeight);

        y0 += clearAndAddTitle();

        int maxTextWidth = getOMTitleWidth();
	int offset = getOMTitleHeight();

        y0 += offset;

        for (int i = 0; i < colorNames.length; i++, y0 += 1.5 * rectHeight) {
	    // System.out.println("OMResizeableColorScaleLegend x0="+x0+" y0="+y0+" "+rectWidth+"x"+rectHeight);

            OMResizeableGroup group =
                    new OMResizeableGroup(refreshable, ml,
                    x0 - OMControlPointRadius, y0 - OMControlPointRadius, 0, 0, "Elem " + i);
            if (colorScale != null) {
                OMResizeableRect rect =
                        new OMResizeableRect(refreshable, ml,
                        x0, y0, rectWidth, rectHeight, colorScale.getColor(i));
                group.add(rect);
            }

            OMResizeableText text;
	    if (texts.size() > i)
		text = texts.get(i);
	    else {
		text = new OMResizeableText(refreshable, ml, 0, 0,
					    tagHTML.matcher(colorNames[i]).replaceAll(""), OMText.JUSTIFY_LEFT,
					    String.format("%s", colorNames[i]), true, true);
		// text.addKeyListener(i, colorNames);
		texts.add(text);
	    }

	    text.moveTo(x0 + rectWidth + rectHeight, y0 + (rectHeight + text.getHeight())/2);

            if (maxTextWidth < text.getWidth())
                maxTextWidth = text.getWidth();
            
            group.add(text);
            this.add(group);
        }

        int newWidth = (int) (0.5 * rectHeight) + rectWidth + rectHeight + maxTextWidth;
        int newHeight = (int) ((1.5 + colorNames.length * 1.5) * rectHeight) + getOMTitleHeight();

        this.extendBorder(newWidth - getWidth(), newHeight - getHeight());
     	recenterTitle(getWidth());
    }

    public void extend(int dw, int dh) {
	if (dw == 0 && dh == 0)
	    return;

        // % of growth ?
        double pw = 1;
        if (getWidth() != 0) {
            pw = (double) (getWidth() + dw) / getWidth();
        }
        double ph = 1;
        if (getHeight() != 0) {
            ph = (double) (getHeight() + dh) / getHeight();
        }

        if (getWidth() == 0 || getHeight() == 0) {
            return;
        }

        rectWidth *= pw;
        if (rectWidth < 2)
            rectWidth = 2; // can't shrink it to much!
        
        rectHeight *= ph;
        if (rectHeight < 2)
            rectHeight = 2; // can't shrink it to much!

	init();
	refreshGUI();

	//// the folowing should work, but I choose to not rebuild the texts.
	//for(OMResizeableGraphic g: getResizeableGraphics()) {
	//    if(g instanceof OMResizeableGroup)
	//	for (OMResizeableGraphic r: ((OMResizeableGroup)g).getResizeableGraphics()) {
	//	    if (r instanceof OMResizeableRect)
	//		r.extend(rectWidth-oldRectWidth, rectHeight-oldRectHeight);
	//	    else if (r instanceof OMResizeableText)
	//		r.translate(rectWidth-oldRectWidth, 0);
	//	}
	//}
    }

    //=====================================================
    // PROPERTIES
    //=====================================================
    public static final String SCALE_LEGEND_PROPERTY_NAME = "scaleLegend";
    public static final String RECTANGLES_WIDTH_PROPERTY_NAME = "rectangles.width";
    public static final String RECTANGLES_HEIGHT_PROPERTY_NAME = "rectangles.height";

    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
	String tmpPrefix = PropUtils.getScopedPropertyPrefix(prefix) + SCALE_LEGEND_PROPERTY_NAME + ".";
        rectWidth = PropUtils.intFromProperties(properties, tmpPrefix + RECTANGLES_WIDTH_PROPERTY_NAME, rectWidth);
        rectHeight = PropUtils.intFromProperties(properties, tmpPrefix + RECTANGLES_HEIGHT_PROPERTY_NAME, rectHeight);
        init();
        super.setProperties(prefix, properties);
    }

    public Properties getProperties(String prefix, Properties properties) {
	properties = super.getProperties(prefix, properties);
	prefix = PropUtils.getScopedPropertyPrefix(prefix) + SCALE_LEGEND_PROPERTY_NAME + ".";
	properties.put(prefix + RECTANGLES_WIDTH_PROPERTY_NAME, Integer.toString(rectWidth));
        properties.put(prefix + RECTANGLES_HEIGHT_PROPERTY_NAME, Integer.toString(rectHeight));
	return properties;
    }
}
