// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.MinMeanMaxValuesAndSizes;
import java.awt.Paint;
import java.awt.Color;
import java.util.Properties;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/** Disc for DiscLegend */
public class OMResizeableDisc extends OMResizeableGraphic {

    private OMCircle omDisc;
    private OMLine omLine;

    public OMResizeableDisc(Refreshable r, MouseControlPointsListener ml, int px1, int py1, int radius, int lineLength) {
        this(r, ml, px1, py1, radius, lineLength, null, null);
    }

    public OMResizeableDisc(Refreshable r, MouseControlPointsListener ml, int px1, int py1, int radius, int lineLength, Paint lineColor, Paint fillColor) {
        super(r, ml, ""); // empty name as we don't want a GUI for it.
        omDisc = new OMCircle(px1, py1, 2 * radius, 2 * radius);
        if (lineColor != null)
            omDisc.setLinePaint(lineColor);
        if (fillColor != null)
            omDisc.setFillPaint(fillColor);
        add(omDisc);
	
	if (lineLength > 0) {
	    omLine = new OMLine(px1, py1 - radius, px1 + lineLength, py1 - radius);
	    // if (lineColor != null) omLine.setLinePaint(lineColor);
	    add(omLine);
	}      
  
	if (ml != null)
	    addOMControlPoint(new OMControlNPoint(px1, py1, 
						  OMResizeableGraphic.OMControlPointRadius, 
						  Color.yellow, Color.red, refreshable.getMapBean()));
    }

    public void translate(int dx, int dy) {
        // System.out.println("DiscTranslate: " + dx + ";" + dy);
        omDisc.setX(omDisc.getX() + dx);
        omDisc.setY(omDisc.getY() + dy);

	if (omLine != null) {
	    int[] linePts = omLine.getPts();
	    linePts[0] += dx;
	    linePts[2] += dx;
	    linePts[1] += dy;
	    linePts[3] += dy;
	}

	for(int i = 0; i < omControlPoints.size(); i++) 
	    ((OMControlPoint)omControlPoints.getOMGraphicAt(i)).translate(dx, dy);

    }

    public void extend(int dw, int dh) { }

    public int getXAnchor() {
        return omDisc.getX();
    }

    public int getYAnchor() {
        return omDisc.getY();
    }

    public int getWidth() {
        return omDisc.getWidth();
    }

    public int getHeight() {
        return omDisc.getHeight();
    }

    public void setRadius(int r) {
        omDisc.setWidth(r * 2);
        omDisc.setHeight(r * 2);
    }

}
