// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.Environment;
import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.util.PropUtils;
import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Paint;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Properties;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;

/** 
 * OMResizeableDiscLegend construct a legend for proportionnal discs.
 * <pre>
 *         ..------------value
 *       ......
 *     ..........
 *     ..........
 *       ......
 *         ..
 * </pre>
 * The asciiart is copyrighted Yannick Lange ;-)
 *
 * @author Jacques.Madelaine@info.unicaen.fr
 */
public class OMResizeableDiscLegend extends OMResizeableRegularPolygonLegend {

    /** if columnName is null, draw only 1 disc (because they are of fixed size. */
    public OMResizeableDiscLegend(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, Resizer resizer, MinMaxValues minMaxValues, String columnName, Paint lineColor, Paint fillColor, String name) {
        super(r, ml, xAnchor, yAnchor, resizer, minMaxValues, 0, 0, 0, columnName, lineColor, fillColor, name); 
    }

    protected OMResizeableGraphic newOMResizeableFeature(int x0, int y0, int offset, int radius, int biggestRadius, double trueRadiusFactor, int nbEdges, double startAngle, int starCoef, Paint lineColor, Paint fillColor) {
	// mouseListener is null as you can't move or extend it individually, but with its group
	return new OMResizeableDisc(refreshable, null, 
				    x0 + biggestRadius, 
				    (int)Math.round(y0 + 2 * biggestRadius - radius * trueRadiusFactor), 
				    radius, biggestRadius + offset,
				    lineColor, fillColor);
    }

}
