// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import java.awt.Paint;
import java.awt.Point;
import java.util.Properties;
import java.util.ArrayList;

import com.bbn.openmap.Environment;
import com.bbn.openmap.omGraphics.OMText;
import com.bbn.openmap.util.PropUtils;


import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.MouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.ArrowHead;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.Refreshable;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.MinMaxValues;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.MinMeanMaxValuesAndSizes;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Resizer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Flow;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ColorScale;
import fr.unicaen.openmap.util.Util;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;

public class OMResizeableFlowLegend extends OMResizeableFrame {

    public static int FLOW_SIZE = 120;
    Flow flow;
    protected OMResizeableColorScaleLegend scl;
    private MinMeanMaxValuesAndSizes mmmValuesAndSizes;

    public OMResizeableFlowLegend(Flow flow, Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, Resizer resizer, MinMaxValues minMaxValues, String columnName, Paint lineColor, Paint fillColor, ColorScale colorScale, String[] colorNames, String colorDataColumnName, String name) {
	super(r, ml, xAnchor, yAnchor, 0, 0, columnName, name, false);
	this.flow = flow;
	this.ml = ml;
        mmmValuesAndSizes =
	    new MinMeanMaxValuesAndSizes(resizer, minMaxValues);
	init(minMaxValues, columnName, lineColor, fillColor, colorScale, colorNames, colorDataColumnName);
    }
    
    public void init(MinMaxValues minMaxValues,  String columnName, Paint  lineColor, Paint fillColor, ColorScale colorScale, String[] colorNames, String colorDataColumnName) {

   	int deltaY0 = clearAndAddTitle();

	final int offset =  getOMTitleHeight();

        int x0 = getXAnchor() + offset;
        int y0 = getYAnchor() + deltaY0 + offset;

	y0 += 2 * offset;

	int polygonIndex = mmmValuesAndSizes.getSize() - 1;
	if (columnName == null && polygonIndex > 1)// only 1 polygon if columnName is null
	    polygonIndex = 0; 

	int maxTextWidth = 0;
	// in reverse order so that the smallest polygon is on top.
        int h = 0;
	for(int i = polygonIndex; i >= 0; i--) {
	    h = mmmValuesAndSizes.getSize(i);
	    y0 += (i == polygonIndex ? 0 : 2*h + offset);
	    OMResizeableGroup group = 
		new OMResizeableGroup(refreshable, ml, 
				      x0 - 2 * OMControlPointRadius, y0 - 2 * OMControlPointRadius, 0, 0, "Elem "+i);
		  
	    OMResizeableFlow poly =
		new OMResizeableFlow(refreshable, ml, flow, new Point(x0,y0),
				     new Point(x0 + FLOW_SIZE, y0),
				     /*ArrowHead.getModel(flow.getStartArrowModel())*/ArrowHead.ArrowHeadType.NOHEAD,
				     /*ArrowHead.getModel(flow.getEndArrowModel())*/ArrowHead.ArrowHeadType.NOHEAD,
				     h, fillColor, lineColor);
		    
	    OMResizeableText text =
		new OMResizeableText(refreshable, ml,
				     x0 + FLOW_SIZE + offset, y0,
				     mmmValuesAndSizes.getValue(i),
				     OMText.JUSTIFY_LEFT, "flow value "+i, true, true);
	    if (text.getWidth() > maxTextWidth)
		maxTextWidth = text.getWidth();
	    group.add(poly);
	    group.add(text);
	    this.add(group);
	}

	int width = FLOW_SIZE + 2 * offset + maxTextWidth;
	y0 += 2 * h + offset;
	int height = y0 - getYAnchor();

	if (colorScale != null) {	    
	    scl = new OMResizeableColorScaleLegend(refreshable, ml, getXAnchor(), y0, 
						   colorScale, colorNames, colorDataColumnName, 
						   colorDataColumnName);
	    scl.showBorder(false);
	    width = Math.max(width, scl.getWidth());
	    height += scl.getHeight();
	    this.add(scl);
	}   
	this.extendBorder(width, height);
	recenterTitle(getWidth());
    }

    protected boolean hasAutoAlign() { return true; }
   
}
