// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.Environment;
import com.bbn.openmap.omGraphics.BasicStrokeEditorMenu;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.omGraphics.OMText;
import com.bbn.openmap.util.PropUtils;
import com.bbn.openmap.I18n;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.LegendGUIManager;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.MouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.omGraphics.DrawingAttributes;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import fr.unicaen.openmap.util.legendable.LegendableLayer;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.LayoutManager;
import java.awt.Paint;
import java.awt.Stroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Properties;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * A group of OMResizeableGraphic inside a frame that can be used to translate its content.
 */
public class OMResizeableFrame extends OMResizeableGroup implements PropertyChangeListener {

    protected OMPoly border;
    protected boolean visibleBorder;
    private JButton daButton;
    protected LayoutManager layout;
    private DrawingAttributes da;
    private boolean visuFrame = false;
    private OMResizeableText omTitle;
    private String title;

    private boolean autoAlignement;
    public static final String autoAlignementPropertyString = "autoAlignement";

    private void setAutoAlignement(boolean b) {
	autoAlignement = b;
    }

    private boolean getAutoAlignement() {
	return autoAlignement;
    }

    protected boolean isVisuFrame() {
        return visuFrame;
    }

    public void setVisuFrame(boolean v) {
        visuFrame = v;
    }

    public OMResizeableFrame(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, int width, int height, String title, String nameForGUI) {
        this(r, ml, xAnchor, yAnchor, width, height, title, true, nameForGUI, true);
    }
    
    public OMResizeableFrame(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, int width, int height, String title, String nameForGUI, boolean ra) {
        this(r, ml, xAnchor, yAnchor, width, height, title, true, nameForGUI, ra);
    }

    public OMResizeableFrame(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, int width, int height, String title, String nameForGUI, LayoutManager l) {
        this(r, ml, xAnchor, yAnchor, width, height, title, true, nameForGUI, true);
        this.layout = l;
        da = new DrawingAttributes();
        da.getPropertyChangeSupport().addPropertyChangeListener(this);
    }
    protected OMResizeableFrame(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, int width, int height, String title, boolean withControlAngles, String nameForGUI){
        this(r, ml, xAnchor, yAnchor, width, height, title, withControlAngles, nameForGUI, true);
    }


    protected OMResizeableFrame(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, int width, int height, String title, boolean withControlAngles, String nameForGUI, boolean ra) {
        super(r, ml, xAnchor, yAnchor, width, height, withControlAngles, nameForGUI, ra);
	this.title = title;
        da = new DrawingAttributes();
        da.getPropertyChangeSupport().addPropertyChangeListener(this);
        border = new OMPoly(new int[]{});
        visibleBorder = true;
        setXYsBorder();
        add(border);
    }

    /** remove all OMResizeableGraphic it contains except omControlPoints. Add the title. */
    public void clear() {
        resizeableGraphics.clear();
        graphics.clear();
        add(border);
        add(omControlPoints);
	
        refreshGUI();
    }

    public int clearAndAddTitle() {
	clear();
	omTitle = new OMResizeableText(refreshable, ml, 
				       xAnchor, yAnchor, 
				       title, OMText.JUSTIFY_CENTER, "title");
	omTitle.setFontStyle(java.awt.Font.BOLD);
	int h = omTitle.getHeight();
	omTitle.translate(0, h);
	this.add(omTitle);

	return h;
    }

    public void recenterTitle(int dw) {
	// System.out.println(" -- OMResizeableFrame.recenterTitle("+dw+")");
	if (omTitle != null)
	    omTitle.translate(dw/2, 0);
    }

    public int getOMTitleWidth() {
	return omTitle.getWidth();
    }

    public int getOMTitleHeight() {
	return omTitle.getHeight();
    }

    /** The OMResizeableGraphic will be also availaible with getGraphics() */
    public void add(OMResizeableGraphic g) {
        resizeableGraphics.add(g);
        // beware border is first
        graphics.remove(border);
        graphics.add(g.getGraphics());
        graphics.add(border);
    }

    /** extend the border but don't forward this action to the embedded graphics */
    public void extendBorder(int dw, int dh) {

        width += dw;
        height += dh;

        for (int i = 0; i < omControlPoints.size(); i++)
            ((OMControlPoint) omControlPoints.getOMGraphicAt(i)).extend(dw, dh);

        setXYsBorder();
        // Debug.message("Legend2", "OMResizeableFrame.extendBorder("+dw+","+dh+") ->"+this);
    }

    public String toString() {
        return "Frame@" + hashCode() + " " + width + "x" + height + "@" + xAnchor + "," + yAnchor +
                (getVisible() ? " v" : " ") +
                " border " + (border == null ? "null" : border.isVisible());
    }

    protected void setXYsBorder() {
        border.setXs(new int[]{
                    xAnchor,
                    xAnchor + width,
                    xAnchor + width,
                    xAnchor,
                    xAnchor
                });
        border.setYs(new int[]{
                    yAnchor,
                    yAnchor,
                    yAnchor + height,
                    yAnchor + height,
                    yAnchor
                });

    }

    public void translate(int dx, int dy) {
        // Debug.message("Legend2", this+".translate("+dx+", "+dy+") \t<=");
        super.translate(dx, dy);
        setXYsBorder();
        // Debug.message("Legend2", this+".translate("+dx+", "+dy+") \t=>");
    }

    public void extend(int dw, int dh) {
        // Debug.message("Legend2", "<- "+this+".extend("+dw+", "+dh+") \t<=");
        super.extend(dw, dh);
	recenterTitle(dw);
        //// don't forward the extend by default. See OMResizeableExtensibleFrame
    }

    /** extend only the border */
    public void extendOnlyItSelf(int dw, int dh) {
        extendBorder(dw, dh);
    }

    public int getXAnchor() {
        return xAnchor;
    }

    public int getYAnchor() {
        return yAnchor;
    }

    public void setXAnchor(int x) {
        xAnchor = x;
    }

    public void setYAnchor(int y) {
        yAnchor = y;
    }

    public int getWidth() {
        return width;
    }

    public int getHeight() {
        return height;
    }

    /** show border only if the graphic is visible */
    public void showBorder(boolean b) {
        visibleBorder = b;
	// if (daButton != null) daButton.setVisible(b);
        border.setVisible(b && getVisible());
    }

    public boolean isVisibleBorder() { return visibleBorder; }

    public void setBorderFillPaint(java.awt.Paint color) {
        border.setFillPaint(color);
        refreshable.refreshInteractiveGraphics();
    }

    public void setBorderLinePaint(Paint p) {
        border.setLinePaint(p);
        refreshable.refreshInteractiveGraphics();
    }

    public void setBorderLineStroke(Stroke s) {
        border.setStroke(s);
        refreshable.refreshInteractiveGraphics();
    }

    /** provides a visible/non visible toggle button, plus a border visible/non visible toggle button and optional align check box */
    public JComponent getToolGUI() {
        if (toolGUI != null) {
            return toolGUI;
        }
        JComponent p = super.getToolGUI();
	ComponentSizer.fixComponentSize(p, new Dimension(ThematicConfPanel.CONTENT_DIMENSION.width, 50));
        p.setBackground(Color.WHITE);
	JLabel label = new JLabel(getName(), 0);
	ComponentSizer.fixComponentSize(label, new Dimension(100,20));
        p.add(label);
        p.setBorder(BorderFactory.createMatteBorder(1, 0, 1, 0, Color.BLACK));
        JComponent daGUI = (JComponent) da.getGUI();
        daGUI.setBackground(new Color(0, 0, 0, 0));
        daGUI.setBorder(null);
        daButton = (JButton) da.getGUI();
        if(!isVisibleBorder())
	    daButton.setIcon(Util.loadIcon("empty.png"));
        daButton.setToolTipText(Environment.getI18n().get(this,
                "frameBorderDrawingAttributesButton", I18n.TOOLTIP,
                "Modify Frame Border Drawing Parameters"));

	JCheckBox borderVisibleCB = new JCheckBox(); 
	borderVisibleCB.setBackground(Color.WHITE);
	borderVisibleCB.setSelected(isVisibleBorder());
	borderVisibleCB.setIcon(Util.loadIcon(!isVisibleBorder() ? "disabled.png" : "enabled.png"));
	borderVisibleCB.setToolTipText("Frame border visible");
	borderVisibleCB.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent ce) {
		    JCheckBox borderVisibleCB = (JCheckBox)(ce.getSource());

		    // save paints to set to transparent temporarely
		    // in order to trigger automatic redraw
		    Paint linePaint = da.getLinePaint();
		    da.setLinePaint(DrawingAttributes.transparentColor);

		    showBorder(borderVisibleCB.isSelected());
		    
		    da.setLinePaint(linePaint);

		    borderVisibleCB.setIcon(Util.loadIcon(!isVisibleBorder() ? "disabled.png" : "enabled.png"));
		    if (borderVisibleCB.isSelected())
			da.resetGUI();	
		    else			
			daButton.setIcon(Util.loadIcon("empty.png"));
		}
	    });
	JPanel pp = new JPanel();
	pp.add(daButton);
	pp.add(borderVisibleCB);
	pp.setBackground(Color.WHITE);
	pp.setBorder(BorderFactory.createLineBorder(Color.GRAY));
	p.add(pp);

	if (hasAutoAlign()) {
	    final JButton autoAlignCB = new JButton(Environment.getI18n().get(OMResizeableFrame.class, "align", "align"));
	    autoAlignCB.setToolTipText(Environment.getI18n().get(OMResizeableFrame.class,  "align", I18n.TOOLTIP,  "Automatic frame content alignment"));
	    // autoAlignCB.setBorder(BorderFactory.createEmptyBorder(0, 25, 0, 0));
	    // autoAlignCB.setBackground(Color.WHITE);
	    autoAlignCB.setOpaque(false);
	    ComponentSizer.fixComponentSize(autoAlignCB, new Dimension(100, 20));
	    autoAlignCB.setSelected(getAutoAlignement());

	    autoAlignCB.addActionListener(new ActionListener() {
		    
		    public void actionPerformed(ActionEvent arg0) {
			setAutoAlignement(true);
			refreshGraphics();
		    }
		});
	    p.add(autoAlignCB);
	}

        return p;
    }

    protected boolean hasAutoAlign() { return false; }

    @Override
    public JComponent getGUI() {
        JComponent c = super.getGUI();

        if (isVisuFrame()) {
            LegendGUIManager guiManager = ThematicDataDisplayLayer.legendManager;
            ThematicConfPanel tcp = ThematicConfPanel.createThematicConfPanel(getName(), false);
            tcp.addCollapseListener(guiManager);
            JPanel wrap = new JPanel();
            wrap.add(c);
            tcp.setContent(wrap);
            if (guiManager.isOpened(getName())) {
                tcp.expand();
            } else {
                tcp.collapse();
            }
            return tcp;
        } else {
            return c;
        }
    }

    @Override
    protected void refreshGUI() {
        super.refreshGUI();
        if (guiPanel != null) {
            guiPanel.setBorder(null);
            visibleCB.setBackground(Color.WHITE);
            visibleCB.setText("");
        }
    }
    protected Box guiContent;

    public Component getGuiContent() {
        return guiContent;
    }

    public static final String BORDER_PROPERTY_NAME = "border";
    public static final String BORDER_LINE_COLOR_NAME = "border_line_color";
    public static final String BORDER_FILL_COLOR_NAME = "border_fill_color";

    public void setProperties(String prefix, Properties properties) {
        // System.out.println(this+" props "+properties);
        super.setProperties(prefix, properties);
        if (properties == null) {
            return;
        }
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        da.setProperties(prefix, properties);
        showBorder(PropUtils.booleanFromProperties(properties, prefix + BORDER_PROPERTY_NAME, visibleBorder));
        setBorderFillPaint(da.getFillPaint());
        setBorderLinePaint(da.getLinePaint());
        setBorderLineStroke(da.getStroke());
        setAutoAlignement(PropUtils.booleanFromProperties(properties, prefix + autoAlignementPropertyString, false));
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        properties = da.getProperties(properties);        
        properties.put(prefix + BORDER_PROPERTY_NAME, Boolean.toString(visibleBorder));
        if (getAutoAlignement()) {
            properties = removeAnchorFromProperties(prefix, properties);
            properties = removeSizeFromProperties(prefix, properties);
        }
        properties.put(prefix + autoAlignementPropertyString, false);
        return properties;
    }

    /** Set the visibility of the item of the graphic list according to their own visibility */
    protected void setGraphicsVisible(boolean v) {
        super.setGraphicsVisible(v);
        showBorder(visibleBorder);
    }

    public Component createStrokeEditor() {
        return da.getGUI();
    }

    public void propertyChange(PropertyChangeEvent pce) {
        String propertyName = pce.getPropertyName();
        //System.out.println(propertyName + "; " + pce.getSource());
        if (propertyName.equals("stroke")) {
            setBorderLineStroke((Stroke) pce.getNewValue());
            return;
        }
        if (propertyName.equals("fillPaint")) {
            setBorderFillPaint((Paint) pce.getNewValue());
            return;
        }
        if (propertyName.equals("linePaint")) {
            setBorderLinePaint((Paint) pce.getNewValue());
            return;
        }
    }
    
}

