// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.Environment;
import com.bbn.openmap.omGraphics.OMText;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.MouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.Refreshable;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ColorScale;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.MinMaxValues;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Resizer;
import java.awt.Paint;
import java.util.Properties;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JScrollPane;

/**
 *OMResizeableGlobalPieFrame 
 *
 * The "general pie" is constructed with angles that are the mean values of the angle among all the pies.
 *
 *  It is invisible by default.
 * @author Jacques.Madelaine@unicaen.fr
 */
public class OMResizeableGlobalPieFrame extends OMResizeableFrame {

    protected OMResizeablePie generalPie;

    public OMResizeableGlobalPieFrame(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, Resizer resizer, double[] generalAngles, Paint lineColor, Paint fillColor, ColorScale colorScale, String[] colorNames, String name) {
        super(r, ml, xAnchor, yAnchor, 0, 0, Environment.getI18n().get(OMResizeableGlobalPieFrame.class, "global", "Average Distribution"), true, name);
        init(resizer, generalAngles, lineColor, fillColor, colorScale, colorNames);
    }

    public void init(Resizer resizer, double[] generalAngles, Paint lineColor, Paint fillColor, ColorScale colorScale, String[] colorNames) {

	int deltaY0 = clearAndAddTitle();

        final int offset = getOMTitleHeight();

        int x0 = getXAnchor(), y0 = getYAnchor() + deltaY0 + offset;

	int generalPieRadius = 3 * offset;

	generalPie = new OMResizeablePie(refreshable, ml, x0, y0, generalPieRadius, colorNames.length, generalAngles, colorScale, getName());
	
	this.add(generalPie);
	
	int width = Math.max(generalPie.getHeight(), getOMTitleWidth());

	generalPie.translate(width/2, generalPieRadius);
	this.extendBorder(width, y0 - getYAnchor() + 2 * generalPieRadius + offset);
	recenterTitle(getWidth());
    }

    public void setProperties(String prefix, Properties props) {
        if (props == null) {
            return;
        }
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
	super.setProperties(prefix, props);
	
	// Default visibility false
	if (props.get(prefix + VISIBILITY_PROPERTY_NAME) == null)
	    setVisible(false);
    }
}

