// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.I18n;

import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.legendLayer.LegendLayer.TopBox;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;

import javax.swing.*;
import java.util.*;

/**
 * Just a list. The methods getXAnchor, getYAnchor, getWidth or getHeight all return 0!
 */
public class OMResizeableGraphicList extends OMResizeableGraphic {

    private int guiOrientation = BoxLayout.Y_AXIS;

    public OMResizeableGraphicList(Refreshable r, MouseControlPointsListener ml, String nameForGUI) {
        super(r, ml, nameForGUI);
    }

    public OMResizeableGraphicList(Refreshable r, MouseControlPointsListener ml, String nameForGUI, boolean ra) {
        super(r, ml, nameForGUI, ra);
    }

    protected ArrayList<OMResizeableGraphic> resizeableGraphics = new ArrayList<OMResizeableGraphic>();

    public ArrayList<OMResizeableGraphic> getResizeableGraphics() {
        return resizeableGraphics;
    }

    /** The OMResizeableGraphic will be also availaible with getGraphics() */
    public void add(OMResizeableGraphic g) {
        resizeableGraphics.add(g);
        graphics.add(g.getGraphics());
    }

    public void add(int index, OMResizeableGraphic g) {
        resizeableGraphics.add(index, g);
        graphics.add(index, g.getGraphics());
    }

    public void remove(OMResizeableGraphic g) {
        resizeableGraphics.remove(g);
        graphics.remove(g.getGraphics());
        refreshGUI();
    }

    /**
     * Test if an OMResizeableGraphic is already in the list
     */
    public boolean contains(OMResizeableGraphic omg) {
        return resizeableGraphics.contains(omg);
    }

    public int size() {
        return resizeableGraphics.size();
    }

    /** Set the visibility of the item of the graphic list according to their own visibility */
    protected void setGraphicsVisible(boolean v) {
        super.setGraphicsVisible(v);
        for (OMResizeableGraphic omrg : getResizeableGraphics()) {
            omrg.setGraphicsVisible(v);
        }
    }

    public void translate(int dx, int dy) {
    }

    public void extend(int dw, int dh) {
    }

    public int getXAnchor() {
        return 0;
    }

    public int getYAnchor() {
        return 0;
    }

    public int getWidth() {
        return 0;
    }

    public int getHeight() {
        return 0;
    }

    protected JComponent guiPanel = null;

    /** Presents a control for each of the OMResizeableGraphics it contains. */
    public JComponent getGUI() {
        if (guiPanel == null) {
            if(guiOrientation == BoxLayout.Y_AXIS)
                guiPanel = Box.createVerticalBox();
            else
                guiPanel = new TopBox();
            //guiPanel = new JPanel();
            //guiPanel.setLayout(new BoxLayout(guiPanel, guiOrientation));
        }
       
        refreshGUI();
        return guiPanel;
    }

    public void setGuiOrientation(int o) {
        guiOrientation = o;
    }

    protected void refreshGUI() {
        if (guiPanel == null) {
            return;
        }
        guiPanel.removeAll();

        for (OMResizeableGraphic omrg : getResizeableGraphics()) {
            if (omrg != null) { // real self defense programming !-)
                JComponent gui = omrg.getGUI();
                if (gui != null) {
                    guiPanel.add(gui);
                }
            }
        }
        guiPanel.updateUI();
    }

    protected void updateFromGUI() {
        for (OMResizeableGraphic om : resizeableGraphics) {
            om.updateFromGUI();
        }
    }

    public String toString() {
        StringBuffer ret = new StringBuffer(super.toString());
        for (OMResizeableGraphic omrg : getResizeableGraphics()) {
            ret.append(" ").append(omrg.toString());
        }
        return ret.toString();
    }

    @Override
    public void resetPositionProperties(String prefix, Properties props) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        for (OMResizeableGraphic omrg : resizeableGraphics) {
            omrg.resetPositionProperties(prefix, props);
        }
    }

    /**
     * Recursively set properties of its elements.
     */
    @Override
    public void setProperties(String prefix, Properties props) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        super.setProperties(prefix, props);
        for (int i = 0; i < resizeableGraphics.size(); i++) {
            resizeableGraphics.get(i).setProperties(prefix + i, props);
        }
    }

  /**
     * Recursively get properties of its elements.
     */
    @Override
    public Properties getProperties(String prefix, Properties props) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        props = super.getProperties(prefix, props);
        for (int i = 0; i < resizeableGraphics.size(); i++) {
            props = resizeableGraphics.get(i).getProperties(prefix + i, props);
        }
        return props;
    }

    /**
     * Remove children anchors from properties and itselves when removeableAnchors is true
     * Used before redrawing legend elements when their sizes have changed. This occurs when
     * thematic size parameters have been modified.
     * @param prefix
     * @param props
     * @return
     */
    public Properties removeAnchorFromProperties(String prefix, Properties props) {
        super.removeAnchorFromProperties(prefix, props);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        for (int i = 0; i < resizeableGraphics.size(); i++) {
            props = resizeableGraphics.get(i).removeAnchorFromProperties(prefix + i, props);
        }
        return props;
    }

    public Properties removeSizeFromProperties(String prefix, Properties props) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        super.removeSizeFromProperties(prefix, props);
        for (int i = 0; i < resizeableGraphics.size(); i++) {
            props = resizeableGraphics.get(i).removeSizeFromProperties(prefix + i, props);
        }
        return props;

    }

}
