// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import java.awt.Color;
import java.util.Properties;
import com.bbn.openmap.MapBean;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.gui.component.ComponentTitledBorder;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.MouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import javax.swing.BorderFactory;

/**
 * A group of OMResizeableGraphic inside a non extensible frame.
 */
public class OMResizeableGroup extends OMResizeableGraphicList {

    protected int xAnchor, yAnchor;
    protected int width, height;

    /** Note: setting width, height to 0 leads to a single visible control point to drag the group. 
     *        In fact, there are points on the same position.
     */
    public OMResizeableGroup(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, int width, int height, String name, boolean ra) {
        this(r, ml, xAnchor, yAnchor, width, height, false, name, ra);
    }

    protected OMResizeableGroup(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, int width, int height, String name){
        this(r, ml, xAnchor, yAnchor, width, height, false, name, true);
    }

     protected OMResizeableGroup(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, int width, int height, boolean withControlAngles, String name){
        this(r, ml, xAnchor, yAnchor, width, height, false, name, true);
     }

    protected OMResizeableGroup(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, int width, int height, boolean withControlAngles, String name, boolean ra) {
        super(r, ml, name, ra);

        this.xAnchor = xAnchor;
        this.yAnchor = yAnchor;
        this.width = width;
        this.height = height;

        MapBean mb = refreshable.getMapBean();
        addOMControlPoint(new OMControlNPoint(xAnchor + width / 2, yAnchor,
                OMControlPointRadius, Color.yellow, Color.red, mb));
        addOMControlPoint(new OMControlEPoint(xAnchor + width, yAnchor + height / 2,
                OMControlPointRadius, Color.yellow, Color.red, mb));
        addOMControlPoint(new OMControlSPoint(xAnchor + width / 2, yAnchor + height,
                OMControlPointRadius, Color.yellow, Color.red, mb));
        addOMControlPoint(new OMControlWPoint(xAnchor, yAnchor + height / 2,
                OMControlPointRadius, Color.yellow, Color.red, mb));
        if (withControlAngles) {
            addOMControlPoint(new OMControlNWAngle(xAnchor, yAnchor,
                    OMControlPointRadius, Color.yellow, Color.red, mb));
            addOMControlPoint(new OMControlNEAngle(xAnchor + width, yAnchor,
                    OMControlPointRadius, Color.yellow, Color.red, mb));
            addOMControlPoint(new OMControlSEAngle(xAnchor + width, yAnchor + height,
                    OMControlPointRadius, Color.yellow, Color.red, mb));
            addOMControlPoint(new OMControlSWAngle(xAnchor, yAnchor + height,
                    OMControlPointRadius, Color.yellow, Color.red, mb));
        }

    }

    /** translate forwards the translation to the graphics of the group */
    public void translate(int dx, int dy) {
        translateOnlyItSelf(dx, dy);

        for (OMResizeableGraphic omrg : resizeableGraphics) {
            omrg.translate(dx, dy);
        }
    }

    public void translateOnlyItSelf(int dx, int dy) {
        for (int i = 0; i < omControlPoints.size(); i++) {
            ((OMControlPoint) omControlPoints.getOMGraphicAt(i)).translate(dx, dy);
        }

        xAnchor += dx;
        yAnchor += dy;

    }

    /** The extend is not forwarded to the group. See @OMResizeableFrame. */
    public void extend(int dw, int dh) {
        extendOnlyItSelf(dw, dh);
        //// don't forward the extend by default
    }

    public void forwardExtend(int dw, int dh){
	for (OMResizeableGraphic omrg : resizeableGraphics) omrg.extend(dw, dh);
    }

    public void extendOnlyItSelf(int dw, int dh) {
        // nothing to do
    }

    public int getXAnchor() {
        return xAnchor;
    }

    public int getYAnchor() {
        return yAnchor;
    }

    public int getWidth() {
        return width;
    }

    public int getHeight() {
        return height;
    }

    protected void refreshGUI() {
        super.refreshGUI();
        if (guiPanel == null) {
            return;
        }
        guiPanel.add(getToolGUI(), 0);
        //guiPanel.setBorder(BorderFactory.createTitledBorder(getName()));
        visibleCB.setText(getName());
        guiPanel.setBorder(new ComponentTitledBorder(visibleCB, guiPanel,BorderFactory.createLineBorder(Color.gray)));
        visibleCB.updateUI();
        // guiPanel.setBorder(new ComponentTitledBorder(new, guiPanel, null))
        guiPanel.updateUI();
    }

    public void setProperties(String prefix, Properties properties) {
        if (properties == null) {
            return;
        }
        super.setProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        //Position
        // translate only itself as the components have been set.
        translateOnlyItSelf(
                PropUtils.intFromProperties(properties, prefix + X_ANCHOR_PROPERTY_NAME, getXAnchor()) - getXAnchor(),
                PropUtils.intFromProperties(properties, prefix + Y_ANCHOR_PROPERTY_NAME, getYAnchor()) - getYAnchor());

        //Size
        // don't extend as it is un-extensible
    }

}
