// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.Environment;
import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import java.awt.Color;
import java.awt.Paint;
import java.awt.event.*;
import javax.swing.*;
import java.util.Properties;

public class OMResizeableHistogramLegend extends OMResizeableFrame {

    protected OMResizeableColorScaleLegend scl;
    public static final String histoLegendPropertyString = "histoLegend";
    private MinMeanMaxValuesAndSizes mmmValuesAndSizes;

    public OMResizeableHistogramLegend(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor, Resizer resizer, MinMaxValues minMaxValues, double barsWidth, Paint lineColor, Paint fillColor, ColorScale colorScale, String[] colorNames, String name) {
        super(r, ml, xAnchor, yAnchor, 0, 0, "", name, false); // we'll set width and height at the end of the ctor.
        mmmValuesAndSizes = new MinMeanMaxValuesAndSizes(resizer, minMaxValues);
        init(resizer, minMaxValues, barsWidth, lineColor, fillColor, colorScale, colorNames);
    }

    public void init(Resizer resizer, MinMaxValues minMaxValues, double barsWidth, Paint lineColor, Paint fillColor, ColorScale colorScale, String[] colorNames) {

	int deltaY0 = clearAndAddTitle();

	final int offset =  getOMTitleHeight();

        int x0 = getXAnchor() + offset / 2, y0 = getYAnchor() + deltaY0 + offset;

        int polygonIndex = mmmValuesAndSizes.getSize() - 1;	
	
	int maxTextWidth = 0;
        int maxHeight = mmmValuesAndSizes.getSize(0);
        y0 += maxHeight;
        for (int i = 0; i <= polygonIndex; i++) {
            int height = mmmValuesAndSizes.getSize(i);
            // System.out.println("Histogram Legend " + i + " [" + x0 + ", " + y0 + "] " + height + " -- " + (mmmValuesAndSizes.getValue(i)));

            OMResizeableGroup group = new OMResizeableGroup(refreshable, ml, x0, y0, 0, 0, "Elem " + i);

            OMResizeableRect rect = new OMResizeableRect(refreshable, ml, x0, y0 - height, (int) barsWidth, height, fillColor, lineColor);

            OMResizeableText text =
                    new OMResizeableText(refreshable, ml,
                    (int) (x0 + barsWidth + offset), y0 - height,
                    mmmValuesAndSizes.getValue(i),
                    OMText.JUSTIFY_LEFT, "bar value " + i, true, true);
            x0 += barsWidth * 3 / 2;
	    if (maxTextWidth < text.getWidth())
		maxTextWidth = text.getWidth();
	    
            group.add(rect);
            group.add(text);

            add(group);
        }


        scl = new OMResizeableColorScaleLegend(refreshable, ml, getXAnchor(), y0 + offset,
                colorScale, colorNames,
                "", "");
        scl.showBorder(false);
        int width =  (int) barsWidth + offset + maxTextWidth;
	if (width < scl.getWidth())
	    width = scl.getWidth();
        int height = y0 + offset + scl.getHeight();
        this.add(scl);
        this.extendBorder(width, height + 20);
	recenterTitle(getWidth());
    }

    protected boolean hasAutoAlign() { return true; }
   
}
