// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ColorScale;
import java.awt.Paint;
import java.util.Properties;

public class OMResizeableMultiRegularPolygonLegend extends OMResizeableFrame {

    protected OMResizeableColorScaleLegend scl;

    public OMResizeableMultiRegularPolygonLegend(Refreshable r, MouseControlPointsListener ml, int x0, int y0, int nbEdges, double radius, double startAngle, int starCoef, Paint lineColor, Paint fillColor, String dataColumnName, double valuePerUnit, ColorScale colorScale, String[] colorNames, String colorDataColumnName, String name) {
	super(r, ml, x0, y0, 0, 0, dataColumnName, name); // we'll set width and height at the end of the ctor.

	init(r, ml, x0, y0, nbEdges, radius, startAngle, starCoef, lineColor, fillColor, dataColumnName, valuePerUnit, colorScale, colorNames, colorDataColumnName);
    }

    protected void init(Refreshable r, MouseControlPointsListener ml, int x0, int y0, int nbEdges, double radius, double startAngle, int starCoef, Paint lineColor, Paint fillColor, String dataColumnName, double valuePerUnit, ColorScale colorScale, String[] colorNames, String colorDataColumnName) {

	y0 += clearAndAddTitle();
	final int offset = getOMTitleHeight();
	x0 += offset + 1.1 * radius;
	y0 += 1.5*radius;
	
	OMResizeableRegularPolygon omPoly = 
	    new OMResizeableRegularPolygon(r, ml, x0, y0, nbEdges, 
					   radius, startAngle, starCoef, 0, lineColor, fillColor);
	this.add(omPoly);

	x0 += offset + 1.2*radius;
	
	OMResizeableText omText = 
	    new OMResizeableText(r, ml, x0, y0,
				 String.format("%3g", valuePerUnit),
				 OMText.JUSTIFY_LEFT, dataColumnName);
	y0 += omText.getHeight()/2;
	omText.translate(0, omText.getHeight()/2);
	
	int gap = omText.getHeight() - (y0 - getYAnchor()); 
	if (gap > 0) { // the text will be outside of the top frame 
	    omPoly.translate(0, gap); 
	    omText.translate(0, gap); 
	    y0 += gap;
	}
	this.add(omText);

	int width = x0 - getXAnchor() + omText.getWidth() + (int)(2.4 * radius);
	y0 += 1.1 * radius;
	int height = y0 - getYAnchor();

	if (colorScale != null) {	    
	    scl = new OMResizeableColorScaleLegend(refreshable, ml, getXAnchor(), y0, 
						   colorScale, colorNames, colorDataColumnName, 
						   colorDataColumnName);
	    scl.showBorder(false);
	    width = Math.max(width, scl.getWidth());
	    height += scl.getHeight();
	    this.add(scl);
	}   

	this.extendBorder((int)Math.max(getOMTitleWidth(), width), height);
	recenterTitle(getWidth());
    }

}
