// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.Environment;
import com.bbn.openmap.MapBean;
import com.bbn.openmap.omGraphics.OMArc;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMLine;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.MouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMControlNPoint;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMControlPoint;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMDisc;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMPie;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.Refreshable;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ColorScale;
import fr.unicaen.openmap.util.Util;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Arc2D;
import javax.swing.JCheckBox;
import javax.swing.JComponent;

/**
 * Used for the global pie drawn in the legend.
 * @author jchauvea
 */
public class OMResizeablePie extends OMResizeableGraphic {

    private OMPie omPie;
    int x, y;
    double radius;

    public OMResizeablePie(Refreshable r, MouseControlPointsListener ml, int px1, int py1, int radius, int nbParts, double[] angles, ColorScale cs, String name) {
        super(r, ml, name);
        this.x = px1;
        this.y = py1;
	this.radius = radius;
        omPie = new OMPie(px1, py1, radius*2, nbParts, angles, cs, "");
        addOMControlPoint(new OMControlNPoint(px1, py1,
					      OMResizeableGraphic.OMControlPointRadius,
					      Color.yellow, Color.red, refreshable.getMapBean()));

        for (int i = 0; i < omPie.size(); i++) {
            add(omPie.getOMGraphicAt(i));
        }

    }

    public void translate(int dx, int dy) {

	for (OMGraphic omg  : omPie) {
	    if ((omg instanceof OMGraphicList)) {
		for (OMGraphic omg2 : (OMGraphicList)omg) {
		    
		    OMArc arc = (OMArc) omg2;
		    
		    arc.setX(arc.getX() + dx);
		    arc.setY(arc.getY() + dy);
		}
	    }
	}
	
        x = getXAnchor() + dx;
        y = getYAnchor() + dy;
	
        for (int i = 0; i < omControlPoints.size(); i++) {
            ((OMControlPoint) omControlPoints.getOMGraphicAt(i)).translate(dx, dy);
        }
    }

    public void extend(int dw, int dh) {
    }

    public int getXAnchor() {
        return x;
    }

    public int getYAnchor() {
        return y;
    }

    public int getWidth() {
        return (int) radius * 2;
    }

    public int getHeight() {
        return (int) radius * 2;
    }

    @Override
    public JComponent getGUI(){
	return null;
    }
}
