// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.Environment;
import com.bbn.openmap.omGraphics.OMText;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.MouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.Refreshable;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ColorScale;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.MinMaxValues;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Resizer;
import java.awt.Paint;
import java.util.Properties;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JScrollPane;

/**
 *OMResizeablePieLegend 
 *
 * The "general pie" is constructed with angles that the mean values of the angle among all the pies.
 *
 * @author jchauvea
 */
public class OMResizeablePieLegend extends OMResizeableFrame {

    protected OMResizeableDiscLegend discLegend = null;

    public OMResizeablePieLegend(Refreshable r, MouseControlPointsListener ml, int xAnchor, int yAnchor,Resizer resizer, double[] generalAngles, Paint lineColor, Paint fillColor, ColorScale colorScale, String[] colorNames, OMResizeableDiscLegend discLegend, String name) {
        super(r, ml, xAnchor, yAnchor, 0, 0, "", true, name);
        this.discLegend = discLegend;
        init(resizer, generalAngles, lineColor, fillColor, colorScale, colorNames);
    }

    public void init(Resizer resizer, double[] generalAngles, Paint lineColor, Paint fillColor, ColorScale colorScale, String[] colorNames) {
	int deltaY0 = clearAndAddTitle();

        final int offset = getOMTitleHeight();

        int x0 = getXAnchor(), y0 = getYAnchor() + deltaY0 + offset;
        if(discLegend != null) {
            add(discLegend);
            discLegend.translate(0, offset);
            y0 += discLegend.getHeight() + offset;
        }
        int generalPieRadius = 60;
        OMResizeableColorScaleLegend scl = 
	    new OMResizeableColorScaleLegend(refreshable, ml, x0, y0, colorScale, colorNames, "", "");
        add(scl);
        
        y0 += scl.getHeight() + offset;
	String generalPieName = Environment.getI18n().get(this, "globalPie", "Global Pie");

	OMResizeableGlobalPieFrame generalPieFrame = 
	    new OMResizeableGlobalPieFrame(refreshable, ml, x0 + offset, y0, resizer, generalAngles, lineColor, fillColor, colorScale, colorNames, generalPieName);
	y0 += offset;
	add(generalPieFrame);
 	// generalPieFrame.setVisible(false); is useless see OMResizeableGlobalPieFrame.setProperties

        int w = Math.max(scl.getWidth(),
			 discLegend == null ? 0 : discLegend.getWidth())
	    + 2*offset;
        int h = offset + scl.getHeight() + (discLegend == null ? 0 : discLegend.getHeight() + offset)
	    + (generalPieFrame == null || !generalPieFrame.getVisible() ? 0 : generalPieFrame.getHeight() + offset) + offset;

        if(discLegend != null)
            discLegend.translate((w - discLegend.getWidth()) / 2 , 0);
        scl.translate((w - scl.getWidth()) / 2 , 0);
	
        this.extendBorder(w, h);
	recenterTitle(getWidth());
    }

    protected boolean hasAutoAlign() { return true; }

}
