// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.event.*;
import com.bbn.openmap.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import java.awt.Paint;
import java.awt.Color;
import java.awt.event.*;
import javax.swing.*;
import java.awt.Dimension;

public class OMResizeableRegularPolygon extends OMResizeableGraphic {
    private OMPoly omPoly;
    private OMLine omLine;
    private int xOffset, yOffset;
    private int nbEdges;
    private double startAngle = 0.0F;
    private double radius;
    private int starCoef;

    public OMResizeableRegularPolygon(Refreshable refreshable, MouseControlPointsListener ml, int px1, int py1, int nbEdges, double radius, double startAngle, int starCoef, int lineLength) {
	this(refreshable, ml, px1, py1, nbEdges, radius, startAngle, starCoef, lineLength, null, null);
    }

    public OMResizeableRegularPolygon(Refreshable refreshable, MouseControlPointsListener ml, int px1, int py1, int nbEdges, double radius, double startAngle, int starCoef, int lineLength, Paint lineColor, Paint fillColor) {
	super(refreshable, ml, ""); // empty name as we don't want a GUI for it.
	this.radius = radius;
	this.nbEdges = nbEdges;
	this.starCoef = starCoef;
	xOffset = px1; yOffset = py1;
	this.startAngle = startAngle;

	omPoly = new OMPoly(new int[] {}); // create a RENDER_XY type polygon
	setXYs();
	if (lineColor != null)
            omPoly.setLinePaint(lineColor);
        if (fillColor != null)
            omPoly.setFillPaint(fillColor);

	if (lineLength > 0) {
	    omLine = new OMLine(px1, (int)(py1 - radius), px1 + lineLength, (int)(py1 - radius));
	    // if (lineColor != null) omLine.setLinePaint(lineColor);
	    omLine.setLinePaint(Color.red);
	    add(omLine);
	}        

	if (ml != null)
	    addOMControlPoint(new OMControlNPoint(px1, py1, 
						  OMResizeableGraphic.OMControlPointRadius, 
						  Color.yellow, Color.red, refreshable.getMapBean()));
	add(omPoly);
    }

    // a polygon centered on the point (xOffset, yOffset)
    private void setXYs() {
	double angle = startAngle; 
	int[] Xs, Ys;

	if (starCoef == 0) {
	    // we build a nbEdges edges regular polygon
	    Xs = new int[nbEdges+1];
	    Ys = new int[nbEdges+1];
	    double angleIncr = 2*Math.PI/nbEdges;
	    for(int i = 0; i < nbEdges; i++, angle += angleIncr) { 
		Xs[i] = xOffset + (int)(radius*Math.cos(angle));
		Ys[i] = yOffset + (int)(radius*Math.sin(angle));
	    }	       
	    //the last point is equals to the first one in order to have a closed polygon
	    Xs[nbEdges] = Xs[0];
	    Ys[nbEdges] = Ys[0];
	} else { // a star
	    // Note that this code leeds to the same shape when starCoef = 0, but with 2*nbEdges edges. 
	    Xs = new int[2*nbEdges+1];
	    Ys = new int[2*nbEdges+1];
	    double angleIncr = Math.PI/nbEdges;
	    double radiusForStar = Math.cos(Math.PI/nbEdges) * (1 - starCoef/100.0) * radius;
	    for(int i = 0; i < 2*nbEdges; i++, angle += angleIncr) { 
		Xs[i] = xOffset + (int)(radius*Math.cos(angle));
		Ys[i] = yOffset + (int)(radius*Math.sin(angle));
		i++; 
		angle += angleIncr;
		Xs[i] = xOffset + (int)(radiusForStar*Math.cos(angle));
		Ys[i] = yOffset + (int)(radiusForStar*Math.sin(angle));
	    }
	    //the last point is equals to the first one in order to have a closed polygon
            Xs[2*nbEdges] = Xs[0];
            Ys[2*nbEdges] = Ys[0];
	}
	omPoly.setXs(Xs); 
	omPoly.setYs(Ys);
    }

    public void translate(int dx, int dy) {      
	// System.out.println("PolyTranslate: " + dx + ";" + dy);
	xOffset += dx;
	yOffset += dy;
	setXYs();

	if (omLine != null) {
	    int[] linePts = omLine.getPts();
	    linePts[0] += dx;
	    linePts[2] += dx;
	    linePts[1] += dy;
	    linePts[3] += dy;
	}

	for(int i = 0; i < omControlPoints.size(); i++) 
	    ((OMControlPoint)omControlPoints.getOMGraphicAt(i)).translate(dx, dy);
    }

    public void extend(int dw, int dh) {}

    public int getXAnchor() { return xOffset; }
    public int getYAnchor() { return yOffset; }
    public int getWidth() { return (int)(2*radius); }
    public int getHeight() { return (int)(2*radius); }


}
