// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.util.Colors;
import fr.unicaen.openmap.util.FontDialog;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.text.JTextComponent;
import javax.swing.plaf.basic.BasicButtonUI;
import java.awt.*;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Properties;

public class OMResizeableText extends OMResizeableGraphic {

    private OMText omText;

    public OMResizeableText(Refreshable r, MouseControlPointsListener ml, int px1, int py1, String stuff) {
        this(r, ml, px1, py1, stuff, OMText.JUSTIFY_LEFT);
    }

    public OMResizeableText(Refreshable r, MouseControlPointsListener ml, int px1, int py1, String stuff, int just) {
        this(r, ml, px1, py1, stuff, just, null);
    }

    public OMResizeableText(Refreshable r, MouseControlPointsListener ml, int px1, int py1, String stuff, int just, String nameForGUI) {
        this(r, ml, px1, py1, stuff, just, nameForGUI, true);
    }

    public OMResizeableText(Refreshable r, MouseControlPointsListener ml, int px1, int py1, String stuff, int just, String nameForGUI, boolean withControlPoints){
        this(r, ml, px1, py1, stuff, just, nameForGUI, withControlPoints, false);
    }

    public OMResizeableText(Refreshable r, MouseControlPointsListener ml, int px1, int py1, String stuff, int just, String nameForGUI, boolean withControlPoints, boolean ra) {
	super(r, ml, nameForGUI, ra);
	omText = new OMText(px1, py1, stuff, just);
	setJustify(just);
	if (withControlPoints)
	    addOMControlPoint(new OMControlNPoint
			      (px1, py1, 
			       OMResizeableGraphic.OMControlPointRadius, 
			       Color.yellow, Color.red, refreshable.getMapBean()));
	add(omText);
	// System.out.println("### new "+Util.className(this)+" : "+ml+" name:"+getName()+" "+stuff);
    }

    public void translate(int dx, int dy) {
        // System.out.println("TRANSLATE TEXT "+dx+","+dy + " for " + this);
        omText.setX(omText.getX() + dx);
        omText.setY(omText.getY() + dy);
        for (int i = 0; i < omControlPoints.size(); i++) {
            ((OMControlPoint) omControlPoints.getOMGraphicAt(i)).translate(dx, dy);
        }
    }

    /** 
     * 
     */
    //nothing to do as we don't have angle control point. 
    // * <br/> We could imagine a sub class that will change its font size 
    public void extend(int dw, int dh) {
        //// An example of false good idea :
        //// Extend message comes from a frame (no OMControlAngle) => we translate the text
        // translate(0, dh);
        //// It works fine for text below an embedded frame, but not for the one above !
        //// Must solve this problem in OMResizeableFrame.extend
    }

    public int getXAnchor() {
        return omText.getX();
    }

    public int getYAnchor() {
        return omText.getY();
    }
    protected int width = -1;
    protected int height = -1;

    protected void initWidthHeight() {
        Rectangle2D rect = Util.getStringBounds(getText(), getFont());
        width = (int) (rect.getWidth() * 1.1);
        height = (int) rect.getHeight();
    }

    public int getWidth() {
        if (width < 0) {
            initWidthHeight();
        }
        return width;
    }

    public int getHeight() {
        if (height < 0) {
            initWidthHeight();
        }
        return height;

    }

    protected void updateFromGUI() {
        if (textComponent != null) {
            setText(textComponent.getText());
        }
    }

    public String getText() {
        return omText.getData() == null ? "" : omText.getData();
    }

    public void setText(String text) {
        omText.setData(text);
        width = height = -1;
    }

    public Font getFont() {
        return omText.getFont();
    }

    public void setFont(Font font) {
        omText.setFont(font);
    }

    public void setLinePaint(Paint p) {
        omText.setLinePaint(p);
    }

    public void setJustify(int just) {
        omText.setJustify(just);
    }

    public int getJustify() {
        return omText.getJustify();
    }

    public void setRotationAngle(double a) {
	omText.setRotationAngle(Math.toRadians(a));
    }

    public double getRotationAngle() {
	return Math.toDegrees(omText.getRotationAngle());
    }

    public Color getFontColor() {
	return (Color)(omText.getLinePaint());
    }

    public void setFontColor(Color color) {
	omText.setLinePaint(color);
    }

    public void updateText(String text) {
        setText(text);
        refreshable.refreshInteractiveGraphics();
    }
    protected JTextComponent textComponent;

    public JComponent getGUI() {
	return getGUI(false);
    }

    protected JComponent getGUI(boolean multiline) {
        JPanel p = new JPanel();
        JComponent tg = getToolGUI();
        p.add(tg);
        JLabel lab = new JLabel(getName());
	lab.setToolTipText(getName());
        ComponentSizer.fixComponentSize(lab, 
					new Dimension(multiline ? 50 : 100,25));
        p.add(lab);
	
	KeyListener keyListener = new KeyListener() {
		// only take key released into account
		public void keyPressed(KeyEvent e) {
		}
		
		public void keyReleased(KeyEvent e) {
		    setText(textComponent.getText());
		    refreshable.refreshInteractiveGraphics();
		}
		
		public void keyTyped(KeyEvent e) {
		}
	    };

	if (multiline) {
	    textComponent = new JTextArea(getText());
	    ((JTextArea) textComponent).addKeyListener(keyListener);
	     JScrollPane areaScrollPane = new JScrollPane(textComponent);  
	     ComponentSizer.fixComponentSize(areaScrollPane, new Dimension(200 , 75));
	     areaScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
	     p.add(areaScrollPane);
	} else {
	    textComponent = new JTextField(getText());
	    ((JTextField) textComponent).addKeyListener(keyListener);
	    ComponentSizer.fixComponentSize(textComponent, new Dimension(100 , 25));
	    p.add(textComponent);
	    ((JTextField)textComponent).setHorizontalAlignment(FontDialog.getJTextFieldAlignmentFromOMTextAlignment(getJustify()));
	}
	textComponent.setFont(getFont());
	textComponent.setForeground(getFontColor());

        final FontDialog fontDialog = new FontDialog(omText.getFont(), (Color)(omText.getLinePaint()), getJustify(), getRotationAngle());
        JButton fontButton = new JButton("", Util.loadIcon("visu/fonts.png"));
        ComponentSizer.fixComponentSize(fontButton, new Dimension(25,25));
        fontButton.setPreferredSize(new Dimension(20, 20));
        fontButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent event) {
                fontDialog.setVisible(true);

                if (!fontDialog.isCanceled()) {
                    omText.setFont(fontDialog.getSelectedFont());
                    omText.setLinePaint(fontDialog.getFontColor());
                    omText.setJustify(fontDialog.getJustify());
		    omText.setRotationAngle(Math.toRadians(fontDialog.getRotationAngle()));
                    refreshGraphics();
                }
            }
        });
        fontButton.setBackground(Color.WHITE);
        fontButton.setOpaque(false);
        fontButton.setBorder(null);
        p.add(fontButton);

        return p;
    }
    /**
     * Property name for font name.
     */
    public final static String FONT_PROPERTY_NAME = "font";
    /**
     * Property name for font color
     */
    public final static String FONT_COLOR_PROPERTY_NAME = "font.color";
    /**
     * Property name for font size.
     */
    public final static String FONT_SIZE_PROPERTY_NAME = "font.size";
    /**
     * Property name for font style.
     */
    public final static String FONT_STYLE_PROPERTY_NAME = "font.style";
    public final static String JUSTIFY_PROPERTY_NAME = "justify";
    public final static String ROTATION_ANGLE_PROPERTY_NAME = "rotationAngle";
    /**
     * Property name for data used in OMText.
     */
    public final static String DATA_PROPERTY_NAME = "data";

    public void setFontStyle(int s) {
	omText.setFont(omText.getFont().deriveFont(s));
    }

    public void setFontSize(float s) {
	omText.setFont(omText.getFont().deriveFont(s));
    }

    public int getFontSize() {
	return omText.getFont().getSize();
    }

    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);

        //Font settings
        omText.setFont(new Font(properties.getProperty(prefix + FONT_PROPERTY_NAME, omText.getFont().getName()),
                PropUtils.intFromProperties(properties, prefix + FONT_STYLE_PROPERTY_NAME, omText.getFont().getStyle()),
                PropUtils.intFromProperties(properties, prefix + FONT_SIZE_PROPERTY_NAME, omText.getFont().getSize())));
        setFontColor(Colors.parseColor(properties.getProperty(prefix + FONT_COLOR_PROPERTY_NAME, "000000")));
	//Justify
        setJustify(PropUtils.intFromProperties(properties, prefix + JUSTIFY_PROPERTY_NAME, omText.getJustify()));
	//Angle
        setRotationAngle(PropUtils.doubleFromProperties(properties, prefix + ROTATION_ANGLE_PROPERTY_NAME, getRotationAngle()));

	//Text
	setText(properties.getProperty(prefix + DATA_PROPERTY_NAME, getText()));

        //System.out.println("Resizeable text setProps " + omText.getData() + " - visible = " + getVisible());
    }

    public Properties getProperties(String prefix, Properties properties) {
        super.getProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        //System.out.println("SET TEXT PROPERTIES 2" + prefix);
        //Font settings
        properties.put(prefix + FONT_PROPERTY_NAME, getFont().getName());
        properties.put(prefix + FONT_COLOR_PROPERTY_NAME, Integer.toHexString(getFontColor().getRGB()));
        properties.put(prefix + FONT_STYLE_PROPERTY_NAME, String.valueOf(getFont().getStyle()));
        properties.put(prefix + FONT_SIZE_PROPERTY_NAME, String.valueOf(getFont().getSize()));
        //Justify
        properties.put(prefix + JUSTIFY_PROPERTY_NAME, Integer.toString(getJustify()));
	//Angle
	properties.put(prefix + ROTATION_ANGLE_PROPERTY_NAME, Double.toString(getRotationAngle()));
        //Text
        properties.put(prefix + DATA_PROPERTY_NAME, getText());

        //System.out.println("Resizeable text getProps" + omText.getData() + " - visible = " + visible);
        return properties;
    }

    public String toString() {
	return super.toString() + " name: " + getName() + " " + getText();
    }

}
