// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.util;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Properties;

import com.bbn.openmap.util.ColorFactory;


/**
 * 
 */
abstract public class AbstractColorScale {

    public static final String colorScalePropertyString = "colorScale";
    public static final String defaultColorScaleInHex = "ffffb300 ffec93c1 ff74c476 ff80a6ce ffffff4d fff4c6de ffa6dba0 ffbfd1e5";


    public void setProperties(String prefix, Properties properties) {
        String colorScaleInHex =
                properties.getProperty(prefix + colorScalePropertyString, defaultColorScaleInHex);
        initFromColorScaleInHex(-1, colorScaleInHex);
    }

    abstract protected void initFromColorScaleInHex(int colorNumber, String colorScaleInHex) ;

    public Properties getProperties(String prefix, Properties properties) {
	properties.put(prefix + colorScalePropertyString, toHexString());
  	return properties;
    }

    abstract public Color getColor(int index);
    
    abstract public void setColor(int index, Color color);

    protected int colorNumber;
    public int getColorNumber() {
        return colorNumber; 
    }

    public void setColorNumber(int nb) {	
        for (int i = colorNumber; i < nb; i++) {
	    // add random colors
            setColor(i, new Color((int) (Math.random() * 255), (int) (Math.random() * 255), (int) (Math.random() * 255)));
        }
        colorNumber = nb;
    }

     /**
      * Used to display in a JComboBox.
      */
     public String toString() {
         return getName();
     }

    public String getName() {
	return "unknown";
    }
    
    public String toHexString() {
        String colorScaleInHex = "";
        for (int i = 0; i < colorNumber; i++)
            colorScaleInHex += " " + ColorFactory.getHexColorString(getColor(i));
	return colorScaleInHex;
    }

    /**
     * Use this method for tracing instead of toString
     */
    public String debug() {
        String s = "ColorScale (" + getColorNumber() + ") ";
        for (int i = 0; i < getColorNumber(); i++) {
	    if (i % 5 == 0)
		s += "\n";
	    Color c = getColor(i);
            s += "[r="+c.getRed()+",g="+c.getGreen()+",b="+c.getBlue()+"] ";
        }
        return s;
    }

}
