// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.util;

import com.bbn.openmap.util.Debug;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.HaveWidthAndData;
import fr.unicaen.openmap.util.Util;

public class MinMaxValues {
    /** positive min and max */
    double pmin, pmax; 
    /** negative min and max */
    double nmin, nmax;
    public MinMaxValues(double pmin, double pmax) {
	this(pmin, pmax, 0, 0);
    }

    public MinMaxValues(double pmin, double pmax, double nmin, double nmax) {
	this.pmin = pmin;
	this.pmax = pmax;
	this.nmin = nmin;
	this.nmax = nmax;
	Debug.message("Legend2", "new MinMaxValues("+nmin+", "+nmax+" - "+pmin+", "+pmax+")");
    }
    public double getPMin() { return pmin; }
    public double getPMax() { return pmax; }
    public double getNMin() { return nmin; }
    public double getNMax() { return nmax; }
    
    static public MinMaxValues createMinMaxValues(OMGraphicList omgl) {
	return createMinMaxValues(omgl, true);
    }
    static public MinMaxValues createMinMaxValues(OMGraphicList omgl, boolean sorted) {
	if (omgl == null || omgl.size() == 0)
	    return new MinMaxValues(0, 0, 0, 0);
	
	Debug.message("Legend", "createMinMaxValues "+omgl.getOMGraphicAt(0));

	HaveWidthAndData min;
	// skip first non HaveWidthAndData member (useful for Flow)
	int i = 0;
        for(; i < omgl.size() && !(omgl.getOMGraphicAt(i) instanceof HaveWidthAndData); i++);

	if (i >= omgl.size())
	    return new MinMaxValues(0, 0, 0, 0);

	min = (HaveWidthAndData) (omgl.getOMGraphicAt(i));
        HaveWidthAndData max = min;
	if (sorted) 
	    max = (HaveWidthAndData) omgl.getOMGraphicAt(omgl.size() - 1);
	else
	    for (i++; i < omgl.size(); i++) {
		if (omgl instanceof HaveWidthAndData) {
		    HaveWidthAndData hwad = (HaveWidthAndData) omgl.getOMGraphicAt(i);
		    int size = hwad.getWidth();
		    if (size > max.getWidth())
			max = hwad;
		    else if (size < min.getWidth())
			min = hwad;
		}
	    }
	
	double dmin = 0, dmax = 0;
	try {
	    dmin = Util.parseDouble(min.getData());
	    dmax = Util.parseDouble(max.getData());
	} catch (java.lang.ClassCastException e) {
	}
	return new MinMaxValues(dmin, dmax);
    }

}
