// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.util;

import java.awt.event.ActionEvent;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.Component;
import java.awt.Frame;
import java.awt.Window;
import java.util.ArrayList;
import java.util.Properties;
import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.gui.RefreshableStringComboBox;
import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import fr.unicaen.openmap.util.gui.WholeNumberField;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;

import java.awt.event.ActionListener;

import java.util.HashMap;
import java.util.concurrent.ExecutionException;

/**
 * Utility class to set an (x, y) offset
 * 3 cases:
 *  - an offset is defined for all graphics
 *  - each graphics has it own offset (stored in an hashmap)
 *  - the offset in given by the value of 2 columns.
 */
public class Offseter {

    private HashMap<String, OffsetLocation> offsets = new HashMap<String, OffsetLocation>();
    private String[] identificationCols = new String[0];
    private DataStorageIterator dsi;
    protected static I18n i18n = new BasicI18n();
    private int x = 0, y = 0;
    private JComponent gui;
    private boolean cpVisible = false; // controlPoints visibility
    private boolean lineVisible = false;
    private JCheckBox boxManual;
    private JButton bKeyManual;
    private JCheckBox boxLine;
    // private JButton bPal; // <autoplacement> 
    private OffsetType type = OffsetType.NO;
    private String xColumnName = "X", yColumnName = "Y";

    public Offseter(int x, int y) {
        this.x = x;
        this.y = y;
    }

    public Offseter() {
        this(0, 0);
    }

    /** Offset types */
    public static enum OffsetType {
	NO(i18n.get(Offseter.class, "no", "no offset")),
	    GLOBAL(i18n.get(Offseter.class, "global", "global offset")), 
	    INDIVIDUAL(i18n.get(Offseter.class, "individual", "individual offsets")),
	    COLUMN_DEP(i18n.get(Offseter.class, "column_dep", "given with 2 columns")),
	    ;
	private String string;
	OffsetType(String string) { this.string = string; }
	public String toString() { return string; }
	public String toSymbol() { return super.toString(); }
    }
  
    public boolean isOn() {
	return type != OffsetType.NO;
    }
      
    public OffsetType getType() {
	return type;
    }

    private void setType(OffsetType type) {
	this.type = type;
	fireOffseterChanged();
    }

    /** set individual offset */
    public void setX(int x, String id) {
        offsets.get(id).x = x;
    }

    /** set individual offset */
    public void setY(int y, String id) {
        offsets.get(id).y = y;
    }
 
    /**
     * Stores offset x and y positions plus key.
     */
    static public class OffsetLocationAndKey {

        private int x;
        private int y;
	private String key;
        public OffsetLocationAndKey(int x, int y, String key) {
            this.x = x;
            this.y = y;
	    this.key = key;
        }

        public int getX() {
            return x;
        }

        public int getY() {
            return y;
        }
	public String getKey() {
	    return key;
	}
    } 
   
    /** nullOffset is returned in case of NO type Offseter */
    private static OffsetLocationAndKey nullOffset = new OffsetLocationAndKey(0, 0, null);

    /** globalOffset is returned in case of GLOBAL Offset type */
    private OffsetLocationAndKey globalOffset = new OffsetLocationAndKey(x, y, null);
    public OffsetLocationAndKey getOffsetLocation(DataObject data) { 
	switch(getType()) {
	case NO:
	    return nullOffset;
	case GLOBAL:
	    return globalOffset;
	case INDIVIDUAL:
	    {
		String key = buildKey(data);
		return new OffsetLocationAndKey(getX(key), getY(key), key);
	    }
	case COLUMN_DEP:
		return new OffsetLocationAndKey(
						Util.parseDouble(data.getData(xColumnName)).intValue(), 
						Util.parseDouble(data.getData(yColumnName)).intValue(),
						null);
	}
	return null;
    }


    private int getX(String id) {
        if (offsets.get(id) == null) {
            return getX();
        }
        return offsets.get(id).getX();

    }

    private int getY(String id) {
        if (offsets.get(id) == null) {
            return getY();
        }
        return offsets.get(id).getY();
    }

    public void setXY(String id, int x, int y) {
        setX(x, id);
        setY(y, id);
    }

    public void setX(int x) {
        this.x = x;
	globalOffset = new OffsetLocationAndKey(x, y, null);
	fireOffseterChanged();
    }

    public void setY(int y) {
        this.y = y;
	globalOffset = new OffsetLocationAndKey(x, y, null);
        fireOffseterChanged();
    }

    public int getX() {
        return x;
    }

    public int getY() {
        return y;
    }

    protected void insert(String key) {
        offsets.put(key, new OffsetLocation(0, 0));

    }

    public static final String offsetTypePropertyString = "offsetType";
    public static final String xOffsetPropertyString = "Xoffset";
    public static final String yOffsetPropertyString = "Yoffset";
    public static final String visiblePropertyString = "cpvisibility";
    public static final String lineVisiblePropertyString = "linevisibility";
    public static final String nbOffsetPropertyString = "nb";
    public static final String idPropertyString = "id";
    public static final String nbIdColsPropertyString = "nbCols";
    public static final String colNamePropertyString = "colName";
    public static final String xColumnNamePropertyString = "Xcolumn";
    public static final String yColumnNamePropertyString = "Ycolumn";

    /**
     * used to set the object with properties ressources
     */
    public void setProperties(String prefix, Properties properties) {
	type = OffsetType.valueOf(properties.getProperty(prefix + offsetTypePropertyString, "NO"));
	switch (type) {
	case NO:
	    break;
	case GLOBAL:
	case INDIVIDUAL:
	    x = PropUtils.intFromProperties(properties, prefix + xOffsetPropertyString, x);
	    y = PropUtils.intFromProperties(properties, prefix + yOffsetPropertyString, y);
	    globalOffset = new OffsetLocationAndKey(x, y, null);
	    cpVisible = Boolean.parseBoolean(properties.getProperty(prefix + visiblePropertyString, "false"));
	    lineVisible = Boolean.parseBoolean(properties.getProperty(prefix + lineVisiblePropertyString, "false"));
	    if (type == OffsetType.INDIVIDUAL) {
		//identification columns
		int nbCols = Integer.parseInt(properties.getProperty(prefix + nbIdColsPropertyString, "0"));
		identificationCols = new String[nbCols];
		for (int i = 0; i < nbCols; i++)
		    identificationCols[i] =
			properties.getProperty(prefix + i + "." + colNamePropertyString, "");
		//offsets
		int nb = Integer.parseInt(properties.getProperty(prefix + nbOffsetPropertyString, "0"));
		for (int i = 0; i < nb; i++) {
		    String id = properties.getProperty(prefix + i + "." + idPropertyString, "");
		    offsets.put(id,
				new OffsetLocation(
						   Integer.parseInt(properties.getProperty(prefix + id + "." 
											   + xOffsetPropertyString, "0")), 
						   Integer.parseInt(properties.getProperty(prefix + id + "." 
											   + yOffsetPropertyString, "0"))));
		}
	    }
	    break;
	case COLUMN_DEP:
	    xColumnName = properties.getProperty(prefix + xColumnNamePropertyString);
	    yColumnName = properties.getProperty(prefix + yColumnNamePropertyString);
	    lineVisible = Boolean.parseBoolean(properties.getProperty(prefix + lineVisiblePropertyString, "false"));
	    break;				 
	}
    }

    public Properties getProperties(String prefix, Properties properties) {
	properties.put(prefix + offsetTypePropertyString, type.toSymbol());
	switch (type) {
	case NO:
	    break;
	case GLOBAL:
	case INDIVIDUAL:
	    properties.put(prefix + xOffsetPropertyString, String.valueOf(x));
	    properties.put(prefix + yOffsetPropertyString, String.valueOf(y));
	    properties.put(prefix + visiblePropertyString, String.valueOf(cpVisible));
	    properties.put(prefix + lineVisiblePropertyString, String.valueOf(lineVisible));
	    if (type == OffsetType.INDIVIDUAL) {
		//identification columns
		properties.put(prefix + nbIdColsPropertyString, String.valueOf(identificationCols.length));
		int indexCol = 0;
		for (String colName : identificationCols) {
		    properties.put(prefix + indexCol + "." + colNamePropertyString, colName);
		    indexCol++;
		}
		//offsets
		properties.put(prefix + nbOffsetPropertyString, String.valueOf(offsets.size()));
		int index = 0;
		for (String key : offsets.keySet()) {
		    properties.put(prefix + index + "." + idPropertyString, key);
		    properties.put(prefix + key + "." + xOffsetPropertyString, String.valueOf(offsets.get(key).getX()));
		    properties.put(prefix + key + "." + yOffsetPropertyString, String.valueOf(offsets.get(key).getY()));
		    index++;
		}
	    }
	    break;
	case COLUMN_DEP:
	    properties.put(prefix + xColumnNamePropertyString, xColumnName);
	    properties.put(prefix + yColumnNamePropertyString, yColumnName);
	    properties.put(prefix + lineVisiblePropertyString, String.valueOf(lineVisible));
	    break;
	}
        return properties;
    }

    public void setXColumnIndex(int index) {
	xColumnName = dsi.getRealColumnName(index);
	fireOffseterChanged();
    }

    public void setYColumnIndex(int index) {
	yColumnName = dsi.getRealColumnName(index);
	fireOffseterChanged();
    }

    private Box bottomGUI = Box.createVerticalBox();
    JComboBox typeBox; 
    public JComponent getGUI(DataStorageIterator dsi) {
        if (gui != null) {
	    refreshGUI(dsi);
            return gui;
        }
	setDSI(dsi);
        gui = Box.createVerticalBox();
	
	typeBox = new JComboBox(OffsetType.values());
        typeBox.setSelectedItem(getType());
	typeBox.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent evt) {
                JComboBox cb = (JComboBox) evt.getSource();
                setType((OffsetType) cb.getSelectedItem());
		refreshGUI();
            }
        });

	JPanel wrapTypeBox = new JPanel();
	wrapTypeBox.add(typeBox);
	gui.add(wrapTypeBox);
	gui.add(bottomGUI);
	refreshGUI(dsi);
	
	// <autoplacement> 
        // JPanel panAutoPlacement = new JPanel();
        // panAutoPlacement.add(new JLabel(i18n.get(Offseter.class, "pal", "Automatic placement")));
        // bPal = new JButton(i18n.get(Offseter.class, "apply", "Apply"));
        // bPal.setBackground(Color.WHITE);
        // bPal.setOpaque(false);
        // panAutoPlacement.add(bPal);
        // ComponentSizer.fixComponentSize(bPal, new Dimension(140, 20));
        // gui.add(panAutoPlacement);
	// </autoplacement> 
   
        return gui;
    }

    public void refreshGUI(DataStorageIterator dsi) {
	setDSI(dsi);
	refreshGUI();
    }

    private void refreshGUI() {
	bottomGUI.removeAll();
	switch (type) {
	case NO:
	    break;
	case GLOBAL:
	    bottomGUI.add(getGlobalGUI());
	    bottomGUI.add(getShowLinesGUI());
	    break;
	case INDIVIDUAL:
	    bottomGUI.add(getIndividualGUI());
	    bottomGUI.add(getShowLinesGUI());
	    break;
	case COLUMN_DEP:
	    bottomGUI.add(getColumnDepGUI());
	    bottomGUI.add(getShowLinesGUI());
	    break;
	}
        if(gui != null && gui.getParent() != null){
            gui.getParent().setVisible(false);
            gui.getParent().setVisible(true);
	    gui.updateUI();
        }
    }
    
    private JComponent showLinesGUI = null;
    private JComponent getShowLinesGUI() {
	if (showLinesGUI == null) {
	    showLinesGUI = new JPanel();
	    boxLine = new JCheckBox(i18n.get(Offseter.class, "showLines", "Show the joining lines"));
	    boxLine.addActionListener(new ActionListener() {
		    public void  actionPerformed(ActionEvent event) {
			setLineVisible(boxLine.isSelected());
		    }
		});
	    
	    boxLine.setSelected(lineVisible);
	    showLinesGUI.add(boxLine);
	}
	return showLinesGUI;
    }

    private JComponent individualGUI = null;
    private JComponent getIndividualGUI() {
	if (individualGUI == null) {
	    individualGUI = new JPanel();
	    Box box = Box.createVerticalBox();
	    JPanel panKey = new JPanel();
	    bKeyManual = new JButton(getChooseKeyTitle());
	    bKeyManual.setBackground(Color.WHITE);
	    bKeyManual.setOpaque(false);
	    ComponentSizer.fixComponentSize(bKeyManual, new Dimension(140, 20));
	    panKey.add(bKeyManual);
	    box.add(panKey);
	    bKeyManual.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent arg0) {
			if (identificationCols.length > 0) {
			    int yesno = JOptionPane.showConfirmDialog(
								      individualGUI,
								      "<html><p>"
								      + i18n.get(Offseter.class, "currentKey", "Current key: ") 
								      + identificationColsToString() + "<p>"
								      + i18n.get(Offseter.class, "changeKeyDialog",
										 "If you continue, all the current individual offsets will be all reset. Do you wish to continue ?")
								      + "</html>",
								      i18n.get(Offseter.class, "warning", "Warning"),
								      JOptionPane.YES_NO_OPTION);
			    if (yesno != JOptionPane.YES_OPTION)
				return;
			}

			// get the frame parent
			Frame frameParent = null;
			for(Component parent = individualGUI.getParent(); parent != null; parent = parent.getParent())
			    if (parent instanceof Frame) {
				frameParent = (Frame)parent;
				break;
			    }
			// but it apprears to be useless for the placemnt of the JDialog
			KeyDialog keyDialog = new KeyDialog(frameParent);
			keyDialog.setVisible(true);
		    }
		});

	    boxManual = new JCheckBox(i18n.get(Offseter.class, "showControlPoints", "Show the control points"));
	    boxManual.addActionListener(new ActionListener() {
		    public void  actionPerformed(ActionEvent event) {
			setVisible(boxManual.isSelected());
		    }
		});
	    boxManual.setSelected(cpVisible);
	    enableControlPoints();
	    box.add(boxManual);
	    individualGUI.add(box);
	}
	return individualGUI;
    }

    private JComponent globalGUI = null;
    JSpinner xOffsetField;
    JSpinner yOffsetField;
    private JComponent getGlobalGUI() {
	if (globalGUI == null) {
	    globalGUI = new JPanel();
	    JLabel labelX = new JLabel(i18n.get(Offseter.class, xOffsetPropertyString, "x: "));
	    JLabel labelY = new JLabel(i18n.get(Offseter.class, yOffsetPropertyString, "y: "));

	    xOffsetField = new JSpinner(new SpinnerNumberModel(x, -10000, 10000, 1));
	    xOffsetField.addChangeListener(new ChangeListener () {
		    public void stateChanged(ChangeEvent e) {
			setX((Integer)(xOffsetField.getValue()));
		    }
		});
	    yOffsetField = new JSpinner(new SpinnerNumberModel(y, -10000, 10000, 1));
	    yOffsetField.addChangeListener(new ChangeListener () {
		    public void stateChanged(ChangeEvent e) {
			setY((Integer)(yOffsetField.getValue()));
		    }
		});
	    globalGUI.add(labelX);
	    globalGUI.add(xOffsetField);
	    globalGUI.add(labelY);
	    globalGUI.add(yOffsetField);
	}

        return globalGUI;
    }

    private JComponent columnDepGUI = null;
    protected RefreshableStringComboBox xColumnList, yColumnList;
    protected JComponent getColumnDepGUI() {
	if (columnDepGUI != null) {
	    // update xColumnList yColumnList
	    xColumnList.init(dsi.getDataColumnNames(), xColumnName);
	    yColumnList.init(dsi.getDataColumnNames(), yColumnName);
	} else {
	    Box box = Box.createVerticalBox();
	    ActionListener xListerner = new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			int index = ((JComboBox)e.getSource()).getSelectedIndex();
			if (index < 0) 
			    return;
			setXColumnIndex(index);
		    }
		};
	    ActionListener yListerner = new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			int index = ((JComboBox)e.getSource()).getSelectedIndex();
			if (index < 0) 
			    return;
			setYColumnIndex(index);
		    }
		};
	    Box boxX = Box.createHorizontalBox();
	    boxX.add(new JLabel(i18n.get(Offseter.class, xOffsetPropertyString, "x: ")));
	    xColumnList = new RefreshableStringComboBox(xListerner, dsi.getDataColumnNames(), xColumnName);
	    boxX.add(xColumnList);
	    Box boxY = Box.createHorizontalBox();
	    boxY.add(new JLabel(i18n.get(Offseter.class, yOffsetPropertyString, "y: ")));
	    yColumnList = new RefreshableStringComboBox(yListerner, dsi.getDataColumnNames(), yColumnName);
	    boxY.add(yColumnList);   
	    box.add(boxX);
	    box.add(boxY);
	    columnDepGUI = new JPanel();
	    columnDepGUI.add(box);
	}
	return columnDepGUI;
    }

    private ArrayList<OffseterListener> listenerList = new ArrayList<OffseterListener>();

    public void addOffseterListener(OffseterListener l) {
        if (!listenerList.contains(l)) {
            listenerList.add(l);
        }
    }

    public void removeOffseterListener(OffseterListener l) {
        listenerList.remove(l);
    }

    protected void fireOffseterChanged() {
        for (int i = 0; i < listenerList.size(); i++) {
            listenerList.get(i).offseterChanged(this);
        }
    }

    private void setDSI(DataStorageIterator dsi) {
        this.dsi = dsi;
    }

    public String[] getIdentificationCols() {
        return identificationCols;
    }

    public void setOffset(String id, int xOffset, int yOffset) {
        if (!contains(id)) {
            insert(id);
        }
        setXY(id, xOffset, yOffset);
    }

    protected boolean contains(String key) {
        return offsets.containsKey(key);
    }

    private String buildKey(DataObject dataObject) {
        if (identificationCols == null || identificationCols.length == 0) {
            return null;
        }
        String key = "";
        for (int i = 0; i < identificationCols.length; i++) {
	    // we should use a separator in order to desambiguitate
            key += dataObject.getData(identificationCols[i].toString());
        }
        return key;
    }

    /**
     * Stores offset x and y positions
     */
    static public class OffsetLocation {

        private int x;
        private int y;

        public OffsetLocation(int x, int y) {
            this.x = x;
            this.y = y;
        }

        public int getX() {
            return x;
        }

        public int getY() {
            return y;
        }

        public void setX(int x) {
            this.x = x;
        }

        public void setY(int y) {
            this.y = y;
        }
        public static final String xPropertyString = "x";
        public static final String yPropertyString = "y";

        public void setProperties(String prefix, Properties properties) {
            prefix = PropUtils.getScopedPropertyPrefix(prefix);
            x = Integer.parseInt(properties.getProperty(prefix + xPropertyString, "0"));
            y = Integer.parseInt(properties.getProperty(prefix + yPropertyString, "0"));
        }

        public Properties getProperties(String prefix, Properties properties) {
            prefix = PropUtils.getScopedPropertyPrefix(prefix);
            properties.put(prefix + xPropertyString, String.valueOf(x));
            properties.put(prefix + yPropertyString, String.valueOf(y));

            return properties;
        }
    }

    public boolean isVisible() {
        return type == OffsetType.INDIVIDUAL ? cpVisible : false;
    }

    public void setVisible(boolean b) {
        cpVisible = b;
	fireOffseterChanged();
    }

    public void setLineVisible(boolean b) {
        lineVisible = b;
	fireOffseterChanged();
    }

    public boolean isLineVisible() {
        return isOn() ? lineVisible : false;
    }

    protected void enableControlPoints() {
	boxManual.setEnabled(identificationCols.length > 0);
	bKeyManual.setText(getChooseKeyTitle());
	bKeyManual.setToolTipText(
				  identificationCols.length > 0 
				  ? i18n.get(Offseter.class, "currentKey", "Current key: ") 
				  + identificationColsToString()
				  : i18n.get(Offseter.class, "choosekey.tooltip", "You must choose a key to identify individual only one data row") );
    }

    protected String getChooseKeyTitle() {
	return identificationCols.length > 0 
	    ? i18n.get(Offseter.class, "changekey","Change the key")
	    : i18n.get(Offseter.class, "choosekey","Choose a key");
    }

    protected void setIdentificationCols(Object[] cols) {
	offsets.clear(); // we should warned the user *TBD*
	identificationCols = new String[cols.length];
	for (int i = 0; i < cols.length; i++) {
	    identificationCols[i] = cols[i].toString();
	}
	fireOffseterChanged();
    }

    protected String identificationColsToString() {
	StringBuilder key = new StringBuilder();
	for (String s : identificationCols) {
	    key.append(s);
	    key.append(" ");
	}
	return key.toString();
    }

    class KeyDialog extends JDialog {

        JList keyList;

        public KeyDialog(Frame owner) {
            super(owner);
            setModal(true);
            setSize(350, 280);
            JPanel main = new JPanel(new BorderLayout());
            add(main);
            setTitle(getChooseKeyTitle());
            keyList = new JList(dsi.getDataColumnNames());
            JButton okList = new JButton("ok");
            if(dsi.getDataColumnNames() != null){
                int [] indices = indicesToSelect();
                if(indices.length > 0)
                    keyList.setSelectedIndices(indices);
            }
            okList.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			setIdentificationCols(keyList.getSelectedValues());
			dispose();
			enableControlPoints();
		    }
		});
            main.add(new JLabel(i18n.get(Offseter.class, "keyText", "Choose a or several column(s) as key")), BorderLayout.NORTH);
            main.add(new JScrollPane(keyList), BorderLayout.CENTER);
            JPanel panSouth = new JPanel();
            panSouth.add(okList);
            main.add(panSouth, BorderLayout.SOUTH);

        }

        private boolean isSelectedColumnName(String s){
            //System.out.println(s + " isSelected?");
            for(String sTemp : identificationCols){
                if(sTemp.equals(s))
                    return true;
            }
            return false;
        }

        private int[] indicesToSelect(){
            int[] indices = new int[identificationCols.length];
            int index = 0;
            //System.out.println("Column names = " + dsi.getColumnNames());
            for(int i = 0; i < dsi.getDataColumnNames().length; i++){
                if(isSelectedColumnName(dsi.getDataColumnNames()[i])){
                    indices[index] = i;
                    index ++;
                }
            }
            return indices;

        }
    }

    public void setEnabled(boolean b) {
	if (typeBox != null)
	    typeBox.setEnabled(b);
	if (boxManual != null)
	    boxManual.setEnabled(b);
	if (bKeyManual != null)
	    bKeyManual.setEnabled(b);
	if (boxLine != null)
	    boxLine.setEnabled(b);
	if (xOffsetField != null)
	    xOffsetField.setEnabled(b);
	if (yOffsetField != null)
	    yOffsetField.setEnabled(b);
	if (xColumnList != null)
	    xColumnList.setEnabled(b);
	if (yColumnList != null)
	    yColumnList.setEnabled(b);
    }
}

