// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.util;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Properties;

import com.bbn.openmap.util.ColorFactory;


/**
 */
public class SimpleColorScale extends AbstractColorScale {

    private ArrayList<Color> colorScale;

    /**
     * Creates a color scale from the colors representation in hexadecimal. 
     */
    public SimpleColorScale(int colorNumber, String colorScaleinHex) {
	initFromColorScaleInHex(colorNumber, colorScaleinHex);
    }	
    
    public SimpleColorScale(int colorNumber) {
	this.colorNumber = colorNumber;
	colorScale = new ArrayList<Color>();
    }

    protected void initFromColorScaleInHex(int colorNumber, String colorScaleInHex) {	
        String colorScaleTabInHex[] = colorScaleInHex.split("[ \t]+");
        colorScale = new ArrayList<Color>(colorScaleTabInHex.length);
        for (int i = 0; i < colorScaleTabInHex.length; i++) {
            if ("".equals(colorScaleTabInHex[i])) {
                continue;
            }
            colorScale.add(ColorFactory.parseColor(colorScaleTabInHex[i]));
            // displayName += ColorFactory.parseColor(colorScaleTabInHex[i]) + " ";
        }
	this.colorNumber = colorNumber > 0 ? colorNumber : colorScaleTabInHex.length;
    }

    public Color getColor(int index) {
	if (index < 0 || index >= colorScale.size()) {
	    return new Color(0, 0, 0, 0); // transparent
	}	    
	return colorScale.get(index);
    }

    public void setColor(int index, Color color) {
	while (index >= colorScale.size())
	    colorScale.add(color);
        colorScale.set(index, color);
	// com.bbn.openmap.util.Debug.message("Colorizer", "ColorScale setcolor "+index +" to " + this);
    }

    public String colorScaleToString() {
	return " " + colorScale + " " +  this.hashCode() + " "  + colorScale.hashCode();
    }

}
