// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.util;

import fr.unicaen.openmap.event.RefreshableGUI;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.Container;
import java.text.*;
import java.util.Properties;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ResizerPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfigurationPanel;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;
import fr.unicaen.openmap.gui.component.ComponentTitledBorder;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;
import java.awt.CardLayout;
import java.awt.Dimension;
import org.jdesktop.swingx.JXCollapsiblePane;

/**
 * Utility class to resize a graphical object proportionally to its data value
 * <em>or</em> a fixed size.
 * 
 * @see Resizer
 */
public class SymbolResizer extends Resizer implements RefreshableGUI {

    public static final String dataColumnNamePropertyString = "dataColumnName";
    protected String dataColumnName = "";
    private boolean withRadius;
    //private JCheckBox checkShowNullAsPoint;
    private JPanel mainPanel;
    /**
     * Use I18N mecanism to allow internationalization. Use I18N.properties file
     * or I18N_fr_FR.properties for french version
     */
    protected I18n i18n = new BasicI18n();

    public String getDataColumnName() {
        return getIsFixedSize() ? null : dataColumnName;
    }

    public boolean setDataColumnName(String s) {
        if (dataColumnName != null && dataColumnName.equals(s)) {
            Debug.message("Resizer", "SymbolResizer.setDataColumnName() skipped for " + s);
            return false;
        }

        dataColumnName = s;
        if (dc != null && dc instanceof DataColumnValueComputer) {
            ((DataColumnValueComputer) dc).setDataColumnName(dataColumnName);
        }
        Debug.message("Resizer", "SymbolResizer.setDataColumnName() changed to " + s);
        if (getDataStorageIterator() != null) {
            setMinMaxValue(getDataStorageIterator());
            refreshGUI();
        }
        fireResizerChanged();
        return true;
    }
    public static final String isFixedSizePropertyString = "isFixedSize";
    protected boolean isFixedSize = true;

    public boolean getIsFixedSize() {
        return isFixedSize;
    }

    public void setIsFixedSize(boolean b) {
	if (isFixedSize == b)
	    return;
        isFixedSize = b;
        fireResizerChanged();
    }
    public static final String fixedSizePropertyString = "fixedSize";
    int fixedSize = 10;

    protected int getFixedSize() {
        return fixedSize;
    }

    public void setFixedSize(int s) {
	if (fixedSize == s)
	    return;
        fixedSize = s;
        fireResizerChanged();
    }

    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);

        setDataColumnName(properties.getProperty(prefix + dataColumnNamePropertyString, dataColumnName));
        setIsFixedSize(Util.isTrue(properties.getProperty(prefix + isFixedSizePropertyString, "true")));
        setFixedSize(PropUtils.intFromProperties(properties, prefix + fixedSizePropertyString, 25));
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
	if (dataColumnName != null)
	    properties.put(prefix + dataColumnNamePropertyString, dataColumnName);
        properties.put(prefix + isFixedSizePropertyString, new Boolean(
                getIsFixedSize()).toString());
        properties.put(prefix + fixedSizePropertyString, String.valueOf(getFixedSize()));
        return properties;
    }

    /**
     * Returns element's size (width, radius or anything determinating its size)
     * proportionnal to value parameter.
     */
    public float getSize(double value) {
        if (getIsFixedSize()) {
            return getFixedSize();
        }
        return super.getSize(value);
    }

    public float getSize(DataObject data) {
        if (isFixedSize) {
            return getFixedSize();
        }
        return super.getSize(data);
    }

    public String getString(DataObject data) {
        if (getIsFixedSize()) {
            return String.valueOf(getFixedSize());
        }
        return dc.getString(data);
    }

    public SymbolResizer(DataValueComputer dc) {
        super(dc);
    }

    public void setMinMaxValue(DataStorageIterator dsi) {
        if (!getIsFixedSize()) {  // min and max value are not needed for fixed size
            super.setMinMaxValue(dsi);
        }
    }
    private DataStorageIterator dsi;

    public DataStorageIterator getDataStorageIterator() {
        return dsi;
    }

    protected void setDataStorageIterator(DataStorageIterator dsi) {
	// System.out.println("###### change dsi to "+dsi.getDataStorage());
        this.dsi = dsi;
    }

    public void refreshGUI(DataStorageIterator dsi) {
        setDataStorageIterator(dsi);
        if (proportionalGUI != null) {
            proportionalGUI.setDataList(this);
        }
    }

    public void refreshGUI() {
        if (proportionalGUI != null) {
            proportionalGUI.refresh();
        }
    }

    protected ResizerPanel proportionalGUI;
    protected JComboBox comboSize;
    protected ItemListener comboSizeListener;
    private final int FIXED_INDEX = 1;
    private final int PROPORTIONAL_INDEX = 0;

    public ItemListener getFixedSizeRBGroupListener() {
        return comboSizeListener;
    }

    /** The GUI palette needs all the data column names */
    public JComponent getGUI(DataStorageIterator dsi) {
        setDataStorageIterator(dsi);
        if (maxSize == NO_MAX_SIZE) {
            setMaxSize(getMaxSizeAccordingToNbRecords(dsi.getDataStorage().getCount()));
        }
        if (confPanel != null) {
            return confPanel;
        }

        comboSize = new JComboBox(new String[]{
		i18n.get(ResizerPanel.class, "proportional", "Proportional"), 
		i18n.get(ResizerPanel.class, "unique", "Unique"), 
	    });
        JPanel fixedGUI = new JPanel();
        if (proportionalGUI == null) {
            proportionalGUI = new ResizerPanel(this);
        }

        final JSpinner fixedSizeField = new JSpinner(new SpinnerNumberModel(
                getFixedSize(), 0, 1000, 1));
        fixedSizeField.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                setFixedSize(((Integer) fixedSizeField.getValue()).intValue());
            }
        });

        fixedGUI.add(new JLabel(i18n.get(ResizerPanel.class, "sizeInPixels", "Size in pixels")));
        fixedGUI.add(fixedSizeField);

        fixedSizeField.setEnabled(true); //getIsFixedSize()); // always enabled as it is hidden if not

        final JPanel panel = new JPanel();
        final CardLayout cl = new CardLayout();
        panel.setLayout(cl);
        panel.add("prop", proportionalGUI);
        panel.add("fix", fixedGUI);
        //panel.add("sumprop", new JPanel());
        comboSizeListener = new ItemListener() {

            public void itemStateChanged(ItemEvent arg0) {
                setIsFixedSize(comboSize.getSelectedIndex() == FIXED_INDEX);
                String card = "";
                switch(comboSize.getSelectedIndex()){
                    case FIXED_INDEX:
                        card = "fix";
                        break;
                    case PROPORTIONAL_INDEX:
                        card = "prop";
                        proportionalGUI.setDataListWanted(SymbolResizer.this, true);
                        break;
                    default:
                        card = "prop";
                        proportionalGUI.setDataListWanted(SymbolResizer.this, false);
			proportionalGUI.setEnabled(true); // always enable the radio buttons
                }
                cl.show(panel, card);
            }
        };

        mainPanel = new JPanel(new BorderLayout());
        comboSize.addItemListener(comboSizeListener);
        JPanel wrapNorth = new JPanel();
        wrapNorth.add(new JLabel(i18n.get(ResizerPanel.class, "size", "Size")));
        wrapNorth.add(comboSize);
        mainPanel.add(wrapNorth, BorderLayout.NORTH);
        mainPanel.add(panel, BorderLayout.CENTER);
//        if(checkShowNullAsPoint != null)
//            mainPanel.add(checkShowNullAsPoint, BorderLayout.SOUTH);
        proportionalGUI.setEnabled(true);
        comboSize.setSelectedIndex(isFixedSize ? FIXED_INDEX : PROPORTIONAL_INDEX);

        confPanel = ThematicConfPanel.createThematicConfPanel(i18n.get(ResizerPanel.class, "size", "Size"), false);
        //comboSize
        ((ThematicConfPanel) confPanel).setContent(mainPanel);

        return confPanel;
    }

    public boolean isWithRadius() {
        return withRadius;
    }

    public void setWithRadius(boolean withRadius) {
        this.withRadius = withRadius;
    }

    public ResizerPanel getProportionalGUI() {
        return proportionalGUI;
    }

    public void setShowNullAsPoint(JCheckBox cb){
        JPanel wrapSouth = new JPanel();
        wrapSouth.add(cb);
        mainPanel.add(wrapSouth, BorderLayout.SOUTH);
        mainPanel.updateUI();
    }

    public JComboBox getComboSize(){return comboSize;}
}
