// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMPoint;

import java.util.*;
import java.awt.event.*;
import java.awt.BorderLayout;

import javax.swing.*;


import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.classification.ClassificationListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;

import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.omGraphics.DrawingAttributes;
import fr.unicaen.openmap.util.classification.Quantile;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.Dimension;

/**
 * Choropleth visualization mode.
 *
 */
public class Area extends Colorized {

    public Area(ThematicDataDisplayLayer layer, String name) {
        this(layer, name, i18n.get(Area.class, "coloredArea", "Colored Area"));
    }

    public Area(ThematicDataDisplayLayer layer, String name, String typeName) {
        super(layer, name, typeName);
    }

    protected void buildDrawingAttributes() {
        drawingAttributes = new ThematicDrawingAttributes(this, true, false, false, true, false);
    }

    public boolean isAlwaysColorized() { return true; }

    public OMGraphicList generateGraphics(DataStorageIterator dsi) {
        try {
            super.generateGraphics(dsi);
            OMGraphicList omgl = new OMGraphicList();

            while (dsi.hasNext()) {
                DataObject data = dsi.next();
                OMDataLocation omdl = data.getOMDataLocation(geometryColumnName);
                if (omdl == null)
                    continue;

                OMGraphic geom = omdl.getOMGraphic();

                if (geom == null && omdl.getCentroid() != null)
                    geom = new OMPoint(omdl.getCentroid().getLat(), omdl.getCentroid().getLon());

                if (geom == null)
                    continue;

                Object value = data.getData(colorDataColumnName.get());

                // clones may consume resource. TODO: a property ?
		//	    geom = cloneOMGraphic(geom);

                colorizeGraphic(geom, value);
  
                omgl.add(geom);
            }
            return omgl;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public OMResizeableGraphic getLegend(Refreshable r, MouseControlPointsListener ml) {
        OMResizeableFrame frame = 
	    new OMResizeableColorScaleLegend(r, ml, 0, 0,
					     colorizer.getColorScale(), colorizer.getColorsTexts(),
					     colorDataColumnName.get(), getFullName());
        frame.setVisuFrame(true);
        return frame;
    }

    public boolean useSharedGraphics() {
        return this instanceof Area;
    }

    public JComponent getGUI(DataStorageIterator dsi) {
        thematicGUI = (ThematicGUI) super.getGUI(dsi);
	// expand classif panel
	thematicGUI.getConfigPanels().get(1).expand();

	return thematicGUI;
    }

//    /**
//     * Clone an OMGraphicList in an other one - only OMPoly are cloned
//     */
//    private void cloneOMPolyInList(OMGraphicList lstFrom, OMGraphicList lstTo){
// 
//    	for(int i = 0; i < lstFrom.size(); i++){
//    		if(lstFrom.getOMGraphicAt(i) instanceof OMGraphicList){
//    			cloneOMPolyInList((OMGraphicList)(lstFrom.getOMGraphicAt(i)), lstTo);
//    		}else{
//    			if(lstFrom.getOMGraphicAt(i) instanceof OMPoly){
//    				lstTo.add(cloneOMPoly((OMPoly)(lstFrom.getOMGraphicAt(i))));
//    			}
//    		}   		
//    	}
//    }
//    
//    /**
//     * OMPoly shape clone
//     * @param source
//     * @return
//     */
//    private  OMPoly cloneOMPoly(OMPoly source){
//    	return  new OMPoly(
//				source.getLatLonArray(),
//				source.getUnits(), source.getLineType());
//    }
}
