// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.util.Debug;

import java.util.*;
import java.awt.event.*;

import javax.swing.event.*;
import javax.swing.*;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;

import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Offseter;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.AttributesPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.MouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicGUI;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.*;

import com.bbn.openmap.util.*;
import fr.unicaen.openmap.omGraphics.DrawingAttributes;
import java.awt.CardLayout;
import java.awt.Stroke;

/**
 * Histogram visualization mode.
 */
public class BoxPlot extends MultiColumnsRep {


    public BoxPlot(ThematicDataDisplayLayer layer, String name) {
        this(layer, name, i18n.get(BoxPlot.class, "boxplot", "BoxPlot"));
    }

    public BoxPlot(ThematicDataDisplayLayer layer, String name, String typeName) {
        super(layer, name, typeName);
	resizer.setMaxSize(20);    }

    @Override
	public OMGraphicList generateGraphics(DataStorageIterator dsi) {
   	
	super.generateGraphics(dsi);
	
	mouseControlPointsListener.clearControlPoints();
 
       OMSizeableGraphicList omgl = new OMSizeableGraphicList<OMBoxPlot>();

	if (colorDataColumnNames.length <= 0)
	    return omgl;
	for (int i = 1; i < 4; i++)
	    if (colorDataColumnNames[i] == null || "".equals(colorDataColumnNames[i])) {
		Debug.error("*** BoxPlot must set Q_"+(i+1));
		return omgl;
	    }
	Debug.message("BoxPlot", "Boxplot usng data columns: "+java.util.Arrays.asList(colorDataColumnNames));
        resizer.setMinMaxValue();
	if (colorDataColumnNames[0] != null && ! "".equals(colorDataColumnNames[0]))
	    resizer.adjustMinMaxValue(dsi, colorDataColumnNames[0]);
	else
	    resizer.adjustMinMaxValue(dsi, colorDataColumnNames[1]);
	if (colorDataColumnNames[colorDataColumnNames.length-1] != null && ! "".equals(colorDataColumnNames[colorDataColumnNames.length-1]))
	    resizer.adjustMinMaxValue(dsi, colorDataColumnNames[colorDataColumnNames.length-1]);
	else
	    resizer.adjustMinMaxValue(dsi, colorDataColumnNames[colorDataColumnNames.length-2]);

        while (dsi.hasNext()) {
            DataObject data = dsi.next();
            OMDataLocation omdl = data.getOMDataLocation(geometryColumnName);
            if (omdl == null) {
                continue;
            }

            Centroid centroid = omdl.getCentroid();
            if (centroid == null) {
                continue;
            }
	    
            double values[] = new double[colorDataColumnNames.length];
            String labels[] = new String[colorDataColumnNames.length];
            boolean valuesAreAllNull = true;
            for (int i = 0; i < colorDataColumnNames.length; i++) {
		if (colorDataColumnNames[i] == null || "".equals(colorDataColumnNames[i])) {
		    values[i] = Double.NaN;
		    labels[i] = "";
		    continue;
		}
                Object data_i = data.getData(colorDataColumnNames[i]);
                if (data_i != null && !"".equals(data_i.toString())) {
                    valuesAreAllNull = false;
                }
		double v = Util.parseDouble(data.getData(colorDataColumnNames[i]));
                values[i] = resizer.getSize(v);
		labels[i] = ""+v;
	    }
	    
            if (valuesAreAllNull) {
                Debug.message("ThematicDataDisplayLayer", "no BoxPlot because all values are nul for: " + data);
                continue;
            }

	    Offseter.OffsetLocationAndKey loc = offseter.getOffsetLocation(data);
	    OMBoxPlot boxplot = new OMBoxPlot(this, loc.getKey(), 
					      centroid.getLat(), centroid.getLon(),
					      loc.getX(), loc.getY(),
					      getBarsWidth(), values, labels, 
					      drawingAttributes.getLinePaint(), drawingAttributes.getFillPaint(),
					      orientation, offseter.isVisible(), offseter.isLineVisible());
	    if (getColorize()) {
		Object value = data.getData(colorDataColumnName.get());
		boxplot.setFillPaint(colorizer.getColor(value));
	    }
	    boxplot.setStroke(getDrawingAttributes().getStroke());
	    boxplot.setMatted(getDrawingAttributes().isMatted());
	    boxplot.setMattingPaint(getDrawingAttributes().getMattingPaint());
	    
	    omgl.add(boxplot); 
        }

        omgl.sort();
        initMinMaxValues();
        return omgl;
    }

    protected AttributesPanel attributesPanel;
    public JComponent getGUI(final DataStorageIterator dsi) {
	thematicGUI = (ThematicGUI) super.getGUI(dsi);
	
	Box box = Box.createVerticalBox();
	
	JComboBox orientationBox = new JComboBox(Histogram.Orientation.values());
	orientationBox.setSelectedItem(getOrientation());
	orientationBox.addItemListener(new ItemListener() {
		 
		public void itemStateChanged(ItemEvent evt) {
		    JComboBox cb = (JComboBox) evt.getSource();
		    setOrientation((Histogram.Orientation) cb.getSelectedItem());
		}
	    });
	JPanel panOrientation = new JPanel();
	panOrientation.add(new JLabel(i18n.get(this, "orientation", "Orientation")));
	panOrientation.add(orientationBox);
	box.add(panOrientation);

	attributesPanel = new AttributesPanel(this, 5, i18n.get(this, "attributes", "Attributes"));
	attributesPanel.setAvailableNames(dsi.getDataColumnNames());
	box.add(attributesPanel);
	ThematicConfPanel histoConfPan =
	    ThematicConfPanel.createThematicConfPanel(i18n.get(Histogram.class, "attributes_choice", "Attributes choice"), false);
	histoConfPan.setContent(box);
        thematicGUI.addThematicConfigurationPanel(histoConfPan);
        JPanel wrapHisto = new JPanel();
        wrapHisto.add(histoConfPan);
        thematicGUI.add(wrapHisto, 3);
	 
	return thematicGUI;
    }
    
    @Override
	public void refreshGUI(DataStorageIterator dsi) {
        super.refreshGUI(dsi);
        if (attributesPanel != null) {
            attributesPanel.setAvailableNames(dsi.getDataColumnNames());
            actualiseAttributesNames(dsi);
            attributesPanel.refreshPanel(); // redraws all
        }
    }

    public OMResizeableGraphic getLegend(Refreshable r, MouseControlPointsListener ml) {
        if (minMaxValues == null) {
            return null;
        }
        OMResizeableFrame frame =
	    new OMResizeableFrame(r, ml, 0, 0, 10, 10, colorDataColumnNames[0], getFullName());
	frame.clearAndAddTitle();        
	int width = frame.getOMTitleWidth();
	int height =  frame.getOMTitleHeight()*2;
	if (getColorize()) {
	    OMResizeableFrame scl = new OMResizeableColorScaleLegend(r, ml, 0,  frame.getOMTitleHeight()*2, 
								     colorizer.getColorScale(), colorizer.getColorsTexts(), 
								     colorDataColumnName.get(), colorDataColumnName.get());
	    scl.showBorder(false);
	    width = Math.max(width, scl.getWidth());
	    height += scl.getHeight();
	    frame.add(scl);
	}
	frame.extendBorder(width, height);
     	frame.recenterTitle(frame.getWidth());
        frame.setVisuFrame(true);
        return frame;
    }

}
