// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Properties;

import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMPoint;

import fr.unicaen.openmap.dataStorage.Centroid;
import fr.unicaen.openmap.dataStorage.DataObject;
import fr.unicaen.openmap.dataStorage.DataStorageIterator;
import fr.unicaen.openmap.dataStorage.OMDataLocation;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ColorizerPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ColorScalePanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.DataColumnNameList;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.MouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicGUI;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMDisc;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMRegularPolygon;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMSizeableGraphicList;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.Refreshable;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.ThematicDrawingAttributes;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableColoredProportionalSymbolLegend;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableGraphic;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableFrame;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Colorizer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ColorScale;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ColorScaleListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Offseter;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.MutableString;
import fr.unicaen.openmap.util.classification.ClassificationListener;
import fr.unicaen.openmap.util.classification.Quantile;
import fr.unicaen.openmap.util.gui.ComponentSizer;

public class ColoredProportionalSymbol extends ProportionalSymbol implements ClassificationListener, ColorScaleListener {

    public static final String colorDataColumnNamePropertyString = "colorDataColumnName";
    protected MutableString colorDataColumnName = new MutableString("");
    protected Colorizer colorizer = new Colorizer();
    //private JComponent panelGUI;
    DataColumnNameList dataList;

    public ColoredProportionalSymbol(ThematicDataDisplayLayer layer, String name) {
        this(layer, name, i18n.get(ColoredProportionalSymbol.class,
                "coloredProportionalSymbol", "Colored Proportional Symbol"));
    }

    public ColoredProportionalSymbol(ThematicDataDisplayLayer layer,
            String name, String typeName) {
        super(layer, name, typeName);
        colorizer.addClassificationListener(this);
	colorizer.addColorScaleListener(this);
    }

    /** drawing attributes whithout fill color menu */
    protected void buildDrawingAttributes() {
        drawingAttributes = new ThematicDrawingAttributes(this, true, false, false, true, false);
    }

    @Override
    public JComponent getGUI(DataStorageIterator dsi) {
        thematicGUI = (ThematicGUI) super.getGUI(dsi);
        JPanel mainPan = new JPanel();
        mainPan.setLayout(new BoxLayout(mainPan, BoxLayout.X_AXIS));

        final JPanel coloredRegularpolygonPanel = new JPanel();

        try {
            graphicsPanelToggleControlsDisplay(false);

            coloredRegularpolygonPanel.setLayout(new BoxLayout(
                    coloredRegularpolygonPanel, BoxLayout.Y_AXIS));

            ColorizerPanel colorizerPanel = colorizer.getGUI();
 
	    initDataList();

            JPanel pData = new JPanel();
            pData.add(new JLabel(i18n.get(this, "colorDataColumnName",
                    "Color Data Column Name: ")));
            pData.add(dataList);
            pData.add(colorizerPanel.getClassifList());
            JPanel wrapColorizerPanel = new JPanel(new BorderLayout());
            wrapColorizerPanel.add(pData, BorderLayout.NORTH);
            wrapColorizerPanel.add(colorizerPanel, BorderLayout.CENTER);
            coloredRegularpolygonPanel.add(wrapColorizerPanel);
            ThematicConfPanel classifConfPanel =
                    ThematicConfPanel.createThematicConfPanel(i18n.get(this, "classification_color", "Classification"), false);
            classifConfPanel.setContent(coloredRegularpolygonPanel);
            JPanel wrapClassConf = new JPanel();
            wrapClassConf.add(classifConfPanel);
            thematicGUI.add(wrapClassConf);
            classifConfPanel.collapse();
            thematicGUI.addThematicConfigurationPanel(classifConfPanel);
            if (colorizer.getClassification() == null) {
                Quantile quantile = new Quantile(null, 4);
                quantile.reInit(dsi, colorDataColumnName.get());
                colorizer.setClassification(quantile);
            }
        } catch (Exception e) {
        }

        return thematicGUI;
    }

    private void initDataList() {
	if (dataList == null)
	    dataList = new DataColumnNameList(this, colorDataColumnName);
	else
	    dataList.init();
    }

    public void refreshGUI(DataStorageIterator dsi) {
        super.refreshGUI(dsi);
	initDataList();
    }

    @Override
    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        //minMedianMaxValuesAndSizes.setProperties(prefix, properties);
        colorizer.setProperties(prefix, properties);
        colorDataColumnName.set(properties.getProperty(prefix + colorDataColumnNamePropertyString, "color"));
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        //properties = minMedianMaxValuesAndSizes.getProperties(prefix, properties);
        properties = colorizer.getProperties(prefix, properties);
        properties.put(prefix + colorDataColumnNamePropertyString,
		       colorDataColumnName.get());
        return properties;
    }

    @Override
    public OMGraphicList generateGraphics(DataStorageIterator dsi) {
	mouseControlPointsListener.clearControlPoints();

        resizer.setMinMaxValue(dsi);
        colorizer.setData(dsi, colorDataColumnName.get());

        OMSizeableGraphicList omglPoly = new OMSizeableGraphicList<OMDisc>();
        OMGraphicList ompl = new OMGraphicList(); // for the OMPoint (Discs of null sizes)
	double startAngleInRadians = getStartAngleInRadians();

        boolean offVis = offseter.isVisible();
        boolean offLineVis = offseter.isLineVisible();

        while (dsi.hasNext()) {
            DataObject data = dsi.next();
            OMDataLocation omdl = data.getOMDataLocation(geometryColumnName);
            if (omdl == null) {
                continue;
            }

            Centroid centroid = omdl.getCentroid();
            if (centroid == null) {
                continue;
            }

            double size = resizer.getSize(data);
            double lat = centroid.getLat();
            double lon = centroid.getLon();
            int offX = 0;
            int offY = 0;
            String key = "";
            if (offseter.isOn()) {
		Offseter.OffsetLocationAndKey loc = offseter.getOffsetLocation(data);
		offX = loc.getX();
		offY = loc.getY();
		key = loc.getKey();
            }
            OMGraphic p = null;

            if (getShowNullAsPoint() && ((int) Math.abs(2 * size)) < 2) { // null sized disc => OMPoint
                p = new OMPoint(centroid.getLat(), centroid.getLon(), 1);
		ompl.add(p);
	    } else {
		switch(getMode()) {
		case PICTO:
		    // no colored picto
		    break;
		case POLY:
		    if (nbEdges < 21 || starCoef > 0) { //draw poly
			p = new OMRegularPolygon(this, key, lat, lon,
						 nbEdges, Math.abs(size),
						 offX, offY, startAngleInRadians,
						 starCoef, resizer.getString(data), offVis, offLineVis);
			break;
		    }
		    // fall back to DISC in case nbEdges > 21 and starCoef == 0
		case DISC:
		    p = new OMDisc(this, key, lat, lon,
				   offX, offY,
				   (int) Math.abs(Math.round(size * 2)),
				   resizer.getString(data), offVis, offLineVis);
		}
		omglPoly.add(p);
	    }
	    p.setLinePaint(size >= 0 ? drawingAttributes.getLinePaint() 
			   : ColorScale.negative((Color) drawingAttributes.getLinePaint()));
            p.setFillPaint(colorizer.getColor(data.getData(colorDataColumnName.get())));
            p.setStroke(drawingAttributes.getStroke());
	    p.setMatted(getDrawingAttributes().isMatted());
	    p.setMattingPaint(getDrawingAttributes().getMattingPaint());
        }

	if (ompl.size() > 0) // add null sized discs
	    omglPoly.add(ompl);        
	omglPoly.sort();
        initMinMaxValues(omglPoly); // used for the legend

        return omglPoly;
    }

    public void classificationChanged(Object source) {
        if (source != colorizer) {
            colorizer.setData(getLayer().getDataStorageIterator(), colorDataColumnName.get());
        }
        refresh();
    }

    public void colorScaleChanged(ColorScalePanel colorScalePanel) {
	if (colorizer != null && colorizer.hasForColorScalePanel(colorScalePanel))
	    refresh();
    }

    public void setColorDataColumnName(String name) {
        colorDataColumnName.set(name);
	classificationChanged(this);
    }

    public boolean isReady() {
        return super.isReady() && !colorDataColumnName.equals("");
    }

    public String getMissingInfos() {
        String mess = super.getMissingInfos();
        if (colorDataColumnName.equals("")) {
            mess += i18n.get(this, "colorDataColumnName",
                    "Color data column name") + "\n";
        }
        return mess;
    }

    public OMResizeableGraphic getLegend(Refreshable r,
            MouseControlPointsListener ml) {
        if (minMaxValues == null) {
            return null;
        }
        OMResizeableFrame frame = 
	    new OMResizeableColoredProportionalSymbolLegend(r, ml, 0, 0,
							    resizer, minMaxValues, getMode(),
							    nbEdges, getStartAngleInRadians(),
							    starCoef, getDataColumnName(), drawingAttributes.getLinePaint(), new Color(0, 0, 0, 0),
							    colorizer.getColorScale(), colorizer.getColorsTexts(), colorDataColumnName.get(), getFullName());
        frame.setVisuFrame(true);
        return frame;
    }
}
