// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMLine;
import com.bbn.openmap.omGraphics.OMPoint;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.omGraphics.OMRect;
import com.bbn.openmap.omGraphics.OMText;

import java.util.*;
import java.awt.event.*;
import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.*;


import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.classification.ClassificationListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;

import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.omGraphics.DrawingAttributes;
import fr.unicaen.openmap.util.classification.Quantile;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.Dimension;

/**
 * Base class for visualizations using (or not) a colorizer. 
 * 
 */
abstract public class Colorized extends Raw implements ClassificationListener, ColorScaleListener {

    public static final String colorDataColumnNamePropertyString = "colorDataColumnName";
    public static final String classifiedPropertyString = "classified";
    protected MutableString colorDataColumnName = new MutableString("");
    protected Colorizer colorizer = new Colorizer();

    public Colorized(ThematicDataDisplayLayer layer, String name, String typeName) {
        super(layer, name, typeName);
        colorizer.addClassificationListener(this);
        colorizer.addColorScaleListener(this);	
	// System.out.println("*** new Colorized ");

    }

    public void setProperties(String prefix, Properties properties) {
        //System.out.println("[ThematicDataDisplayLayer.Colorized] setProperties");
        super.setProperties(prefix, properties);
        colorizer.addClassificationListener(this);
        colorizer.setProperties(prefix, properties);
        colorDataColumnName.set(properties.getProperty(prefix + colorDataColumnNamePropertyString, "data"));
	isColorized = Boolean.parseBoolean(properties.getProperty(prefix + classifiedPropertyString, "false"));
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        properties = colorizer.getProperties(prefix, properties);
        properties.put(prefix + colorDataColumnNamePropertyString, colorDataColumnName.get());
        properties.put(prefix + classifiedPropertyString, String.valueOf(getColorize()));

        return properties;
    }

    /** colorization of area */
    protected void colorizeGraphic(OMGraphic omg, Object value) {
        if (omg instanceof OMGraphicList) {
            for (OMGraphic o : (OMGraphicList) omg)
                colorizeGraphic(o, value);
            return;
        }

	boolean fill = true; // default is to fill
	if (omg instanceof OMLine || omg instanceof OMText)
	    fill = false;
	else {
	     if (omg instanceof OMPoly) {
		 OMPoly poly = (OMPoly) omg;
		 if (! poly.isPolygon()) {
		     double points[] = poly.getLatLonArray();
		     if (points[0] != points[points.length - 2] ||
			 points[1] != points[points.length - 1])
			 fill = false;
		 }
	     }
	}    

        if (fill) {
            omg.setLinePaint(drawingAttributes.getLinePaint());
            omg.setFillPaint(colorizer.getColor(value));
        } else {
            omg.setLinePaint(colorizer.getColor(value));
            omg.setFillPaint(drawingAttributes.getFillPaint());
        }

	omg.setStroke(drawingAttributes.getStroke());
	omg.setMatted(getDrawingAttributes().isMatted());
	omg.setMattingPaint(getDrawingAttributes().getMattingPaint());

	omg.putAttribute(ThematicDataDisplayLayer.VALUE, value);  
    }

    /** colorizotion of feature taking size (width or nb) into account */
    protected void colorizeGraphic(OMGraphic omf, DataObject data, int size) {
	if (getColorize()) {
	    Object value = data.getData(colorDataColumnName.get());
	    omf.setFillPaint(colorizer.getColor(value));
	} else
	    omf.setFillPaint(size >= 0 ? drawingAttributes.getFillPaint()
			     : ColorScale.negative((Color) drawingAttributes.getFillPaint()));
	
	omf.setLinePaint(size >= 0 ? drawingAttributes.getLinePaint()
			 : ColorScale.negative((Color) drawingAttributes.getLinePaint()));
	omf.setStroke(drawingAttributes.getStroke());
	omf.setMatted(getDrawingAttributes().isMatted());
	omf.setMattingPaint(getDrawingAttributes().getMattingPaint());
    }

    public OMGraphicList generateGraphics(DataStorageIterator dsi) {
	if (getColorize()) 
                colorizer.setData(dsi, colorDataColumnName.get());
	return null;
    }

    protected ThematicConfPanel colorizerConfPanel; 
    private ColorizerPanel colorizerPanel;

    public boolean isAlwaysColorized() { return false; }

    protected boolean isColorized = false;
    public void setColorize(boolean b) { 
	if (isAlwaysColorized())
	    return;
	isColorized = b;

	enableConfPanel();
	refresh();
    }

    protected void enableConfPanel() {
	boolean b = getColorize();
	if (dataList != null) {
	    dataList.setEnabled(b);
	    colorizerPanel.setEnabled(b);
	    colorizerConfPanel.updateUI();
	    graphicsPanelToggleControlsDisplay(!b);
	}
    }

    public boolean getColorize() {
	return isAlwaysColorized() || isColorized;
    }

    public JComponent getGUI(DataStorageIterator dsi) {
        thematicGUI = (ThematicGUI) super.getGUI(dsi);
	// System.out.println("*** Colorizered getGUI");
        try {
            graphicsPanelToggleControlsDisplay(false);

            initDataList();

	    colorizerPanel = colorizer.getGUI();
	    JPanel dataPanel = new JPanel(); // will contain column names list and classif list
	    dataPanel.setLayout(new BoxLayout(dataPanel, BoxLayout.X_AXIS));
	    dataPanel.add(new JLabel(i18n.get(Area.class, "colorDataColumnName", "Color Data Column Name: ")));
	    dataPanel.add(dataList);
	    dataPanel.add(Box.createRigidArea(new Dimension(4, 2)));
	    dataPanel.add(new JLabel(i18n.get(Colorized.class, "classification", "Classif ")));
	    dataPanel.add(colorizerPanel.getClassifList()); // classifList will not appear in colorizerPanel anymore
	    ComponentSizer.fixComponentSize(dataList, new Dimension(120, 20));

	    Box wrapColorizeCB = null;
	    if (! isAlwaysColorized()) {
		wrapColorizeCB = Box.createHorizontalBox();
		final JCheckBox classifCB = new JCheckBox(i18n.get(Colorized.class, "classified", "Use the colorization"));
		classifCB.setSelected(getColorize());
		classifCB.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent ie) {
			    setColorize(classifCB.isSelected());
			}
		    });
		wrapColorizeCB.add(Box.createRigidArea(new Dimension(2, 2)));
		wrapColorizeCB.add(classifCB);   
	    }
	    JPanel wrap = new JPanel(new BorderLayout());
	    if (wrapColorizeCB != null)
		wrap.add(wrapColorizeCB, BorderLayout.NORTH);
	    JPanel wrapCenter = new JPanel(new BorderLayout());
	    wrapCenter.add(dataPanel, BorderLayout.NORTH);
	    wrapCenter.add(colorizerPanel, BorderLayout.CENTER);
	    wrap.add(wrapCenter, BorderLayout.CENTER);

            colorizerConfPanel = ThematicConfPanel.createThematicConfPanel(i18n.get(Colorized.class, "colorization", "Colorization"), false);
            colorizerConfPanel.setContent(wrap);
            JPanel wrapConf = new JPanel();
            wrapConf.add(colorizerConfPanel);
            thematicGUI.add(wrapConf);
            thematicGUI.addThematicConfigurationPanel(colorizerConfPanel);
            colorizerConfPanel.collapse();
            if (colorizer.getClassification() == null) {
                Quantile quantile = new Quantile(null, 4);
                quantile.reInit(dsi, colorDataColumnName.get());
                colorizer.setClassification(quantile);
            }

	    enableConfPanel();
        } catch (Exception e) {
            e.printStackTrace();
        }
	// System.out.println("<== Colorizered getGUI");
        return thematicGUI;
    }

    protected  DataColumnNameList dataList;
    private void initDataList() {
	if (dataList == null)
	    dataList = new DataColumnNameList(this, colorDataColumnName);
	else
	    dataList.init();
    }

    public String getColorDataColumnName() {
        return colorDataColumnName.get();
    }

    public void setColorDataColumnName(String name) {
	Debug.message("ThematicDataDisplayLayer", this + " ** setDataColumnName: \"" + colorDataColumnName+ "\" to \"" + name + "\"");
        if (name != null && !name.equals(colorDataColumnName.get())) {
	    Debug.message("ThematicDataDisplayLayer", this + " **** setDataColumnName: \"" + colorDataColumnName+ "\" to \"" + name + "\"");

            colorDataColumnName.set(name);
	    if (getColorize())
		colorizer.setData(getLayer().getDataStorageIterator(), colorDataColumnName.get());
	    Debug.message("ThematicDataDisplayLayer", this + " **** setDataColumnName: \"" + colorDataColumnName+ "\" to \"" + name + "\" " + colorizer);
        }
        refresh();
    }

    public void refreshGUI(DataStorageIterator dsi) {
        super.refreshGUI(dsi);
        initDataList();
    }

    public void classificationChanged(Object source) {
	if (! getColorize()) {
	    Debug.message("Colorizer", this + " => classificationChanged(@" + source.hashCode() + "  " + source + ")" + " **discarded**: no colorization");
	    return;
	}
 
	Debug.message("Colorizer", this + (source != colorizer ? " != " : " == ") + " classificationChanged(" + source + ")" + "colorDataColumn: " + colorDataColumnName);
	if (source != colorizer)
	    colorizer.setData(getLayer().getDataStorageIterator(), colorDataColumnName.get());
        
        // no refresh() colorizer will do it
        Debug.message("ThematicDataDisplayLayer", this + " <= classificationChanged(" + source + ")");
    }

    public void colorScaleChanged(ColorScalePanel colorScalePanel) {
	if (getColorize() && colorizer != null && colorizer.hasForColorScalePanel(colorScalePanel))
	    refresh();
    }

}
