// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import java.awt.Color;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Properties;
import java.util.Set;

import javax.swing.JComponent;

import com.bbn.openmap.proj.coords.LatLonPoint;
import com.bbn.openmap.event.MapMouseListener;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;

import fr.unicaen.openmap.dataStorage.Centroid;
import fr.unicaen.openmap.dataStorage.DataObject;
import fr.unicaen.openmap.dataStorage.DataStorageIterator;
import fr.unicaen.openmap.dataStorage.OMDataLocation;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ColorScalePanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.FlowPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.MouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ResizerPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicGUI;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicMouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.ArrowHead;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.HaveWidthAndData;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMFlow;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMLabel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.Refreshable;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableFlowLegend;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableFrame;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableGraphic;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ColorScale;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ColorScaleListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Colorizer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.DataColumnValueComputer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Labelizer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.MinMaxValues;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.MutableString;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Resizer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.ResizerListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.SymbolResizer;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.classification.Classification;
import fr.unicaen.openmap.util.classification.ClassificationListener;
import fr.unicaen.openmap.util.classification.Quantile;

/**
 * Flow visualization mode
 *  
 * Bug: There are many reasons for BENDED_MODE, BENDED_DISTANCE and ARC_LENGTH to be non static in this class !!!
 * Curved points coords should be put in properties only if bendedMode is equal to BENDED_MANUEL_MODE ???
 * @author Jerome Chauveau
 */
public class Flow extends Colorized implements ResizerListener, VisuWithMouseListener {

    public enum type {
        FLOW, PATH
    };
    private type chosenType = type.PATH;
    private boolean dragging;
    protected String valueColumnName = "";
    private int startArrowModel = 1;//noHead
    private int endArrowModel = 1;
    private Projection projection;
    private FlowPanel flowPanel;
    private Object[] primaryKeyArray = {};
    private HashMap<String, LatLonPoint> curvedPoints;
    private boolean controlPointsVisibility = false;
    private int flowNumber;
    private OMGraphicList controlPoints;
    ThematicMouseControlPointsListener mouseControlPointsListener;
    protected SymbolResizer resizer = new SymbolResizer(new DataColumnValueComputer(""));
    private boolean showStartEqualsEnd = false;
    private int loopRadius = 25;
    private ArrayList<String> stepColumns;
    private int nbStep;
    protected Labelizer labelizer;

    public Flow(ThematicDataDisplayLayer layer, String name) {
        this(layer, name, i18n.get(Flow.class, "flow", "Flow"));	
	labelizer = new Labelizer(this, false); // 2nd arg is false, because the labelizer not always on	
	labelizer.setFontColor(Color.BLACK); 
    }

    public Flow(ThematicDataDisplayLayer layer, String name, String typeName) {
        super(layer, name, typeName);
        interactive = true;
        mouseControlPointsListener = new ThematicMouseControlPointsListener(this);
        resizer.addResizerListener(this);
        resizer.setMaxSize(10);
        ((SymbolResizer) resizer).setFixedSize(4);
        curvedPoints = new HashMap<String, LatLonPoint>();
        stepColumns = new ArrayList<String>();
    }
    //PROPERTIES
    // fromColumn is the geometryColumn of the Raw visu
    static String valueColumnNamePropertyString = "valueColumn";
    static String bendedModePropertyString = "bendedMode";
    static String arcLengthPropertyString = "arcLength";
    static String bendedDistancePropertyString = "bendedDistance";
    static String curvedPointPropertyString = "curvedPoint";
    static String flowNumberPropertyString = "flowNumber";
    static String flowNamePropertyString = "name";
    static String flowPrimaryKeyPropertyString = "primary_keys";
    static String flowStartArrowPropertyString = "start_arrow";
    static String flowEndArrowPropertyString = "end_arrow";
    static String flowModePropertyString = "mode";
    static String controlPointsVisibilityString = "controlPointsVisibility";
    static String showStartEqualsEndString = "showStartEqualsEnd";
    static String loopRadiusString = "showStartEqualsEnd";
    static String stepColumnsPropertyString = "stepColumns";
    static String nbStepPropertyString = "nbStep";
    static String labelPropertyString = "label";

    @Override
    public void setProperties(String prefix, Properties properties) {

        super.setProperties(prefix, properties);
        resizer.setProperties(prefix, properties);
        nbStep = PropUtils.intFromProperties(properties, prefix + nbStepPropertyString, 0);
        setChosenType(nbStep == 0 ? Flow.type.PATH : Flow.type.FLOW);
        String[] stepTab = PropUtils.stringArrayFromProperties(properties, prefix + stepColumnsPropertyString, " ");
        if (stepTab != null) {
            for (String s : stepTab) {
                stepColumns.add(s);
            }
        }
        valueColumnName = properties.getProperty(
                prefix + valueColumnNamePropertyString, null);
        OMFlow.BENDED_MODE = Integer.parseInt(
                properties.getProperty(prefix + bendedModePropertyString, "1"));
        OMFlow.BENDED_DISTANCE = Integer.parseInt(
                properties.getProperty(prefix + bendedDistancePropertyString, "0"));
        OMFlow.ARC_LENGTH = Integer.parseInt(
                properties.getProperty(prefix + arcLengthPropertyString, "5"));
        flowNumber = Integer.parseInt(
                properties.getProperty(prefix + flowNumberPropertyString, "0"));
        showStartEqualsEnd = Boolean.parseBoolean(properties.getProperty(prefix + showStartEqualsEndString, "false"));
        controlPointsVisibility = Boolean.parseBoolean(properties.getProperty(prefix + controlPointsVisibilityString, "true"));
        startArrowModel = Integer.parseInt(properties.getProperty(prefix + flowStartArrowPropertyString, "1"));
        endArrowModel = Integer.parseInt(properties.getProperty(prefix + flowEndArrowPropertyString, "1"));
        setLoopRadius(Integer.parseInt(properties.getProperty(prefix + loopRadiusString, "25")));
        String t = properties.getProperty(prefix + flowModePropertyString, "FLOW");
        if (t.equals("PATH")) {
            chosenType = type.PATH;
        } else {
            chosenType = type.FLOW;
        }

        //primary keys array
        buildPrimaryKeyArray(properties.getProperty(prefix + flowPrimaryKeyPropertyString, ""));

        //curved points
        String key;
        double x, y;
        for (int i = 0; i < flowNumber; i++) {
            key = properties.getProperty(
                    prefix + i + "." + flowNamePropertyString, "");
            x = Double.parseDouble(
                    properties.getProperty(prefix + key + "." + "x", "0"));
            y = Double.parseDouble(
                    properties.getProperty(prefix + key + "." + "y", "0"));
            curvedPoints.put(key, new LatLonPoint.Double(x, y));
        }
        setStepColumn(0, geometryColumnName);

	labelizer.setProperties(prefix+ labelPropertyString + ".", properties);
    }

    @Override
    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        properties = resizer.getProperties(prefix, properties);
        String stepString = "";
        for (String s : stepColumns) {
            stepString += " " + s;
        }
        properties.put(prefix + stepColumnsPropertyString, stepString);
        if (valueColumnName != null) {
            properties.put(prefix + valueColumnNamePropertyString, valueColumnName);
        }
        properties.put(prefix + nbStepPropertyString, String.valueOf(nbStep));
        properties.put(prefix + bendedModePropertyString, String.valueOf(OMFlow.BENDED_MODE));
        properties.put(prefix + bendedDistancePropertyString, String.valueOf(OMFlow.BENDED_DISTANCE));
        properties.put(prefix + arcLengthPropertyString, String.valueOf(OMFlow.ARC_LENGTH));
        properties.put(prefix + flowPrimaryKeyPropertyString, primaryKeysToString());
        properties.put(prefix + flowNumberPropertyString, String.valueOf(curvedPoints.size()));
        properties.put(prefix + flowStartArrowPropertyString, String.valueOf(startArrowModel));
        properties.put(prefix + flowEndArrowPropertyString, String.valueOf(endArrowModel));
        properties.put(prefix + flowModePropertyString, chosenType.toString());
        properties.put(prefix + isShowStartEqualsEnd(), String.valueOf(isShowStartEqualsEnd()));
        properties.put(prefix + controlPointsVisibilityString, String.valueOf(controlPointsVisibility));
        properties.put(prefix + loopRadiusString, String.valueOf(getLoopRadius()));
        { // should be put in properties only if bendedMode is equal to BENDED_MANUEL_MODE ???
            int i = 0;
            Set<String> keys = curvedPoints.keySet();
            for (String key : keys) {
                properties.put(prefix + i + "." + flowNamePropertyString, key);
                properties.put(prefix + key + "." + "x", String.valueOf(curvedPoints.get(key).getLatitude()));
                properties.put(prefix + key + "." + "y", String.valueOf(curvedPoints.get(key).getLongitude()));
                i++;
            }
        }
        properties.put(prefix + colorDataColumnNamePropertyString, colorDataColumnName.get());
        properties = labelizer.getProperties(prefix+ labelPropertyString + ".", properties);

        return properties;
    }

    private void createGraphic(String id, DataObject data, Centroid from, Centroid to) {
	if (isInteractiveOMFlow(id))
	    return;

	String stringData = !resizer.getIsFixedSize() ? data.getData(valueColumnName).toString() : "";
	int width = (int) resizer.getSize(data);
	OMFlow omf = new OMFlow(id, this, from, to,
				ArrowHead.getModel(startArrowModel),
				ArrowHead.getModel(endArrowModel),
				width, stringData);

	if (getLabelizer().isOnOff()) {
	    // add label on top of flow
	    LatLonPoint llpt = getCurvedPointForOMFlow(id);
	    if (llpt != null) {
		OMLabel oml = getLabelizer().createOMLabel
		    (new Centroid(llpt.getLongitude(), llpt.getLatitude()), data, null);
		if (oml != null)
			cachedOMGraphicList.add(oml);
	    }
	}
	
	colorizeGraphic(omf, data, width);
	
	cachedOMGraphicList.add(omf);
	//splines.add(omf.getOMSpline());
	//radials.add(omf.getRadials());
	mouseControlPointsListener.addOMControlPoints(omf.getControlPoints());
	controlPoints.add(omf.getControlPoints());
    }
    
    public OMGraphicList generateGraphics(DataStorageIterator dsi) {
        //OMGraphicList splines = new OMGraphicList();
        // OMGraphicList radials = new OMGraphicList();
        try {
            super.generateGraphics(dsi);
            setPrimaryKeyArray(dsi.getDataColumnNames());
            resizer.setMinMaxValue(dsi);
            projection = getLayer().getProjection();
            cachedOMGraphicList = new OMGraphicList();
            controlPoints = new OMGraphicList();
            mouseControlPointsListener.clearControlPoints();

            if (chosenType == type.FLOW) {
                int id = 0;
                while (dsi.hasNext()) {
                    DataObject data = dsi.next();
                    for (int i = 0; i < nbStep; i++) {
                        OMDataLocation omdl_from = data.getOMDataLocation(stepColumns.get(i));
                        OMDataLocation omdl_to = data.getOMDataLocation(stepColumns.get(i + 1));
                        if (omdl_from == null || omdl_to == null) {
                            continue;
                        }
                        Centroid centroid_from = omdl_from.getCentroid();
                        Centroid centroid_to = omdl_to.getCentroid();
                        if (centroid_from == null || centroid_to == null) {
                            continue;
                        }

			createGraphic(String.valueOf(id), data,  centroid_from, centroid_to);

			id++;
                    }
                }
            } else { // Flow.type.PATH
                int id = 0;
                Centroid current = null;
                Centroid last = null;
                dsi.reset();
                while (dsi.hasNext()) {
                    DataObject data = dsi.next();
                    //System.out.println("id = " + id);
                    OMDataLocation omdlFrom = data.getOMDataLocation(geometryColumnName);
                    if (omdlFrom == null) {
                        current = null;
                        last = null;
                        id++;
                        continue;
                    }
                    current = omdlFrom.getCentroid();
                    if (id == 0 || last == null) {
                        id++;
                        last = current;
                        continue;
                    }
		    			
		    createGraphic(String.valueOf(id), data,  last, current);

                    last = current;
                    id++;
                }

            }

	    // should add control points only if necessary (i.e. custom positionning is on)
            cachedOMGraphicList.add(0, controlPoints);

            //// cachedOMGraphicList.add(splines);
            //// splines.setLinePaint(Color.RED);
            //// cachedOMGraphicList.add(radials);
            //// radials.setLinePaint(Color.GREEN);

            return cachedOMGraphicList;
        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }

    private boolean isInteractiveOMFlow(String id) {
        for (int i = 0; i < interactiveGraphics.size(); i++) {
            OMGraphic omg = interactiveGraphics.getOMGraphicAt(i);
            if (omg instanceof OMFlow && ((OMFlow) omg).getID().equals(id))
                return true;
        }
        return false;
    }

    public void removeDraggedOMFlow(OMFlow omflow) {
        addInteractiveGraphic(omflow);
    }

    public void addDraggedOMFlow(OMFlow omflow) {
        clearInteractiveGraphics();
    }

    @Override
    public JComponent getGUI(final DataStorageIterator dsi) {
        thematicGUI = (ThematicGUI) super.getGUI(dsi);

        if (valueColumnName != null) {
            resizer.setDataColumnName(valueColumnName);
        }
        flowPanel = new FlowPanel(this, dsi, startArrowModel, endArrowModel, 
				  guiOrientation, thematicGUI);
        thematicGUI.add(flowPanel, 1);

        return thematicGUI;
    }

    public void refreshGUI(DataStorageIterator dsi) {
        super.refreshGUI(dsi);
        // System.out.println("REFRESH GUI FLOW");
        resizer.refreshGUI(dsi);
        if (flowPanel != null) {
            flowPanel.refreshGUI(dsi);
        }
	labelizer.refreshGUI(dsi);
    }

    /**
     * gives curved point for the given id
     *
     * @param id
     * @return
     */
    public LatLonPoint getCurvedPointForOMFlow(String id) {
        return curvedPoints.get(id);
    }

    public void setCurvedPointForOMFlow(String id, LatLonPoint p) {
        curvedPoints.put(id, p);
    }

    public boolean existsCurvedPointForOMFlow(String id) {
        return curvedPoints.get(id) != null;
    }

    public boolean isReady() {
        return true; // return super.isReady() 
    }

    public String getMissingInfos() {
        String mess = getName() + ":\n";

        if ("".equals(valueColumnName) && !resizer.getIsFixedSize()) {
            mess += i18n.get(ResizerPanel.class, "dataColumnNameLabel", "Data Column Name") + "\n";
        }

        if (primaryKeyArray == null || primaryKeyArray.length == 0) {
            mess += "identification!" + "\n";
        }

        return mess;
    }

    public OMResizeableGraphic getLegend(Refreshable r, MouseControlPointsListener ml) {
	MinMaxValues minMaxValues = 
	    MinMaxValues.createMinMaxValues(getList(), false);
        if (minMaxValues == null) {
            return null;
        }
        int bended_tmp = OMFlow.BENDED_DISTANCE;
        OMFlow.BENDED_DISTANCE = 0;
        OMResizeableFrame frame =
	    new OMResizeableFlowLegend(this, r, ml, 0, 0,
				       resizer, minMaxValues, valueColumnName,
				       drawingAttributes.getLinePaint(), drawingAttributes.getFillPaint(),
				       getColorize() ? colorizer.getColorScale() : null,
				       getColorize() ? colorizer.getColorsTexts() : null,
				       colorDataColumnName.get(), getFullName());
        frame.setVisuFrame(true);
        OMFlow.BENDED_DISTANCE = bended_tmp;
        return frame;
    }

    public void setOMColumnName(String name) {
        super.setOMColumnName(name);
        setStepColumn(0, geometryColumnName);
    }

    public int getEndArrowModel() {
        return endArrowModel;
    }

    public void setEndArrowModel(int endArrowModel) {
        this.endArrowModel = endArrowModel;
    }

    public int getStartArrowModel() {
        return startArrowModel;
    }

    public void setStartArrowModel(int startArrowModel) {
        this.startArrowModel = startArrowModel;
    }

    public Projection getProjection() {
        return projection;
    }

    public String getValueColumnName() {
        return valueColumnName;
    }

    public void setControlPointsVisibility(boolean b) {
        controlPointsVisibility = b;
    }

    public boolean isControlPointsVisibility() {
        return controlPointsVisibility;
    }

    public SymbolResizer getResizer() {
        return resizer;
    }

    public void resizerChanged(Resizer resizer) {
        if (this.resizer == resizer) {
            valueColumnName = ((SymbolResizer) resizer).getDataColumnName();
            refresh(); 
        }
    }

    public type getChosenType() {
        return chosenType;
    }

    public void setChosenType(type chosenType) {
        this.chosenType = chosenType;
    }

    public void setPrimaryKeyArray(Object[] primaryKeyArray) {
        this.primaryKeyArray = primaryKeyArray;
    }

    private String buildKey(DataObject dataObject) {
        String key = "";
        for (int i = 0; i < primaryKeyArray.length; i++) {
            key += dataObject.getData(primaryKeyArray[i].toString());
        }
        return key;
    }

    private String primaryKeysToString() {
        String keys = "";
        for (int i = 0; i < primaryKeyArray.length; i++) {
            keys += primaryKeyArray[i] + " ";
        }
        return keys;

    }

    private void buildPrimaryKeyArray(String st) {
        primaryKeyArray = st.split(" ");
    }

    public boolean isEmptyPrimaryKey() {
        return primaryKeyArray == null ||
                primaryKeyArray.length == 0;
    }

    public boolean isDragging() {
        return dragging;
    }

    public void setDragging(boolean dragging) {
        this.dragging = dragging;
    }

    @Override
    public boolean isInteractiveAndBadCache() {
        return (getBadCache() && interactive);
    }

    public ThematicMouseControlPointsListener getMouseControlPointsListener() {
        return mouseControlPointsListener;
    }

    public MapMouseListener getMapMouseListener() {
        super.getMapMouseListener();
        return mouseControlPointsListener;
    }

    /**
     * @return the showStartEqualsEnd
     */
    public boolean isShowStartEqualsEnd() {
        return showStartEqualsEnd;
    }

    public void setShowStartEqualsEnd(boolean b) {
        showStartEqualsEnd = b;
    }

    /**
     * @return the loopRadius
     */
    public int getLoopRadius() {
        return loopRadius;
    }

    /**
     * @param loopRadius the loopRadius to set
     */
    public void setLoopRadius(int loopRadius) {
        this.loopRadius = loopRadius;
    }

    public void setNbStep(int nb) {
        this.nbStep = nb;
    }

    public int getNbStep() {
        return nbStep;
    }

    public void setStepColumn(int index, String colName) {
        while (stepColumns.size() <= index)
            stepColumns.add(colName);
      
	stepColumns.set(index, colName);
	// System.out.println(index+ " @@ "+stepColumns);
    }

    public String getStepColumnAt(int index) {
        try {
            return stepColumns.get(index);
        } catch (IndexOutOfBoundsException e) {
            return null;
        }
    }

    public void clearCurvedPoints() { 
	curvedPoints.clear(); 
    }

    public Labelizer getLabelizer() {
	return labelizer;
    }
}
