// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import java.util.*;
import java.awt.event.*;
import java.awt.Paint;

import javax.swing.event.*;
import javax.swing.*;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;

import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.*;

import com.bbn.openmap.util.*;
import fr.unicaen.openmap.omGraphics.DrawingAttributes;
import java.awt.BasicStroke;
import java.awt.CardLayout;
import java.awt.Stroke;

/**
 * Histogram visualization mode.
 */
public class Histogram extends MultiColumnsRep implements ColorScaleListener {

    protected ColorScale colorScale = new ColorScale();
    protected Boolean bars = true;

    public Histogram(ThematicDataDisplayLayer layer, String name) {
        this(layer, name, i18n.get(Histogram.class, "histogram", "Histogram"));
    }

    public Histogram(ThematicDataDisplayLayer layer, String name, String typeName) {
        super(layer, name, typeName);
	resizer.setMaxSize(20);    }

    @Override
    protected void buildDrawingAttributes() {
        drawingAttributes = new HistogramDrawingAttributes(this, true, false, false, true, false);
    }

    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        try {
            bars = PropUtils.booleanFromProperties(properties, prefix + barsPropertyString, true);
            colorScale.setProperties(prefix, properties);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        properties.put(prefix + barsPropertyString, String.valueOf(bars));
        properties = colorScale.getProperties(prefix, properties);
        return properties;
    }

    @Override
    public OMGraphicList generateGraphics(DataStorageIterator dsi) {
	super.generateGraphics(dsi);
        mouseControlPointsListener.clearControlPoints();

	// length = max(colorScale length, bars number)
	int length = colorDataColumnNames.length;
	if (length > colorScale.getColorNumber())
	    length = colorScale.getColorNumber();

        // may be we should use DataValueComputer? But like this, it works fine!
        resizer.setMinMaxValue();
        for (int i = 0; i < length; i++) {
            resizer.adjustMinMaxValue(dsi, colorDataColumnNames[i]);
        }

        OMSizeableGraphicList omgl = new OMSizeableGraphicList<OMHistogram>();

        while (dsi.hasNext()) {
            DataObject data = dsi.next();
            OMDataLocation omdl = data.getOMDataLocation(geometryColumnName);
            if (omdl == null) {
                continue;
            }

            Centroid centroid = omdl.getCentroid();
            if (centroid == null) {
                continue;
            }
            double values[] = new double[length];
            boolean valuesAreAllNull = true;
            for (int i = 0; i < length; i++) {
                Object data_i = data.getData(colorDataColumnNames[i]);
                if (data_i != null && !"".equals(data_i.toString())) {
                    valuesAreAllNull = false;
                }
                values[i] = resizer.getSize(Util.parseDouble(data.getData(colorDataColumnNames[i])));
            }

            if (valuesAreAllNull) {
                Debug.message("ThematicDataDisplayLayer", "no Histo because all values are nul for: " + data);
                continue;
            }

	    Paint linePaint = getColorize() 
		? colorizer.getColor(data.getData(colorDataColumnName.get()))
		: drawingAttributes.getLinePaint();
		
	    Offseter.OffsetLocationAndKey loc = offseter.getOffsetLocation(data);
	    OMGraphic omg = bars
		? new OMHistogram(this, loc.getKey(), 
				centroid.getLat(), centroid.getLon(),
				loc.getX(), loc.getY(),
				getBarsWidth(), linePaint,
				values, colorScale, (int) Math.ceil(resizer.getMaxSize()),
				orientation, offseter.isVisible(), offseter.isLineVisible())
		: new OMGraph(centroid.getLat(), centroid.getLon(),
			      offseter.getX(), offseter.getY(),
			      getBarsWidth(), linePaint,
			      ((HistogramDrawingAttributes)drawingAttributes).getAxesPaint(),
			      values, (int) Math.ceil(resizer.getMaxSize()),
			      orientation);
	    omg.setStroke(getDrawingAttributes().getStroke());
	    omg.setMatted(getDrawingAttributes().isMatted());
	    omg.setMattingPaint(getDrawingAttributes().getMattingPaint());

	    omgl.add(omg);
        }
        omgl.sort();
        initMinMaxValues();
        return omgl;
    }
    
    public JComponent getGUI(final DataStorageIterator dsi) {
        thematicGUI = (ThematicGUI) super.getGUI(dsi);

        graphicsPanelToggleControlsDisplay(false);

        colorScalePanel = new HistogramColorScalePanel(this, colorScale, (AttributesNames)this);
        colorScalePanel.setAvailableNames(getNoColorDataColumnNames(dsi));
        colorScalePanel.refreshPanel();
        colorScalePanel.addColorScaleListener(this);

        final JPanel scalesPanel = new JPanel(/*new CardLayout()*/);
        JPanel mainPanel = new JPanel(new BorderLayout());

        scalesPanel.add(colorScalePanel);
        mainPanel.add(scalesPanel, BorderLayout.CENTER);

        ThematicConfPanel histoConfPan =
	    ThematicConfPanel.createThematicConfPanel(i18n.get(this, "attributes_choice", "Attributes choice"), mainPanel, thematicGUI, false, 3);

        return thematicGUI;
    }

    public OMResizeableGraphic getLegend(Refreshable r, MouseControlPointsListener ml) {
        if (minMaxValues == null) {
            return null;
        }
        OMResizeableFrame frame =
	    new OMResizeableHistogramLegend(r, ml, 0, 0, resizer, minMaxValues, 
					    bars ? getBarsWidth() : 2, drawingAttributes.getLinePaint(), new Color(0, 0, 0, 0),
					    bars ? colorScale : null, colorDataColumnNames, getFullName());
        frame.setVisuFrame(true);
        return frame;
    }

    protected HistogramColorScalePanel colorScalePanel;

    //when we change dsi, data change so... refresh list *and* colorscale
    @Override
    public void refreshGUI(DataStorageIterator dsi) {
        super.refreshGUI(dsi);
        if (colorScalePanel != null) {
            colorScalePanel.setAvailableNames(getNoColorDataColumnNames(dsi));
            actualiseAttributesNames(dsi);
            colorScalePanel.refreshPanel(); // redraws all
        }
    }

    public void colorScaleChanged(ColorScalePanel colorScalePanel) {
	super.colorScaleChanged(colorScalePanel);
        if (this.colorScalePanel == colorScalePanel) {
            refresh();
        }
    }

    public boolean isBars(){
        return bars;
    }

    public void setBars(boolean b){
        bars = b;
    }

}
