// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import com.bbn.openmap.layer.DeclutterMatrix;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMText;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.util.PropUtils;

import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.LabelizerGUI;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.MouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicGUI;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMDisc;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMLabel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMRegularPolygon;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.Refreshable;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableGraphic;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.ThematicDrawingAttributes;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.util.*;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.*;

import java.util.*;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.plaf.basic.BasicButtonUI;

/**
 * Label visualization mode : draws a text string near by a geographical centroid
 * <p>The text is found in data column: <code>dataColumnName</code>. 
 * <p> TODO: enable setting of x and y offset of the label!
 */
public class Label extends ColorizedAndWithMouseListener implements VisuWithMouseListener {
    // // <declut>
    // protected DeclutterMatrix declutterMatrix = new DeclutterMatrix();
    // private boolean declutt = false;
    // // </declut>

    protected Labelizer labelizer;
    private Color fontColor; // use to sync Labelizer fontColor and DrawingAttributes lineColor
    public Label(ThematicDataDisplayLayer layer, String name) {
        this(layer, name, i18n.get(Label.class, "label", "Label"));
    }

    public Label(ThematicDataDisplayLayer layer, String name, String typeName) {
        super(layer, name, typeName);
	labelizer = new Labelizer(this);
	// labelizer.setFontColor(null); // no color, use raw lineColor
	labelizer.setFontColor(fontColor = drawingAttributes == null ? Color.BLACK : (Color)drawingAttributes.getLinePaint());
        interactive = true;
    }

    @Override
    protected void buildDrawingAttributes() {
        drawingAttributes = new ThematicDrawingAttributes(this, true, false, false, true, false);
    }


    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        labelizer.setProperties(prefix, properties);
	// fontColor is the drawingAttributes line paint.
	fontColor = (Color)drawingAttributes.getLinePaint(); 
	labelizer.setFontColor((Color)drawingAttributes.getLinePaint());	
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        properties = labelizer.getProperties(prefix, properties);
        return properties;
    }

    @Override
    public OMGraphicList generateGraphics(DataStorageIterator dsi) {
	
	super.generateGraphics(dsi);

        mouseControlPointsListener.clearControlPoints();

        OMGraphicList omgl = new OMGraphicList();
	// we don't use font sizer: Projection projection = getLayer().getProjection();
	Projection projection = null;
	// //<declut>
	// if (declutt) {
	//     declutterMatrix.setWidth(projection.getWidth());
	//     declutterMatrix.setHeight(projection.getHeight());
	//     declutterMatrix.create();
	// }
	// //</declut>

        for (int i = 0; dsi.hasNext(); i++) {
            DataObject data = dsi.next();
            OMDataLocation omdl = data.getOMDataLocation(geometryColumnName);
            // System.out.println("Label "+ i);

            if (omdl == null) {
                continue;
            }

            // <clipping>
            if (!getLayer().overlapsCurrentProjBB(omdl.getBoundingBox())) {
                continue;
            }
            // </clipping>

            Centroid centroid = omdl.getCentroid();
	    if (centroid == null) {
		System.err.println("* no centroid for "+omdl);
		continue;
	    }

	    OMLabel omt = labelizer.createOMLabel(centroid, data, projection);
	    if (omt == null)
		continue;
	    omt.setLinePaint(getColorize() 
			     ?	colorizer.getColor(data.getData(colorDataColumnName.get()))
			     : labelizer.getFontColor());
	    // //<declut>
            //BasicLocation bl = new BasicLocation((double) centroid.getLat(), (double) centroid.getLon(), "", omt);
            // bl.generate(proj, declutterMatrix);
	    // //</declut>

            omgl.add(omt);
        }
        return omgl;
    }

    @Override
    public JComponent getGUI(final DataStorageIterator dsi) {

        if (thematicGUI != null) {
            return thematicGUI;
        }
        thematicGUI = (ThematicGUI) super.getGUI(dsi);

        graphicsPanelToggleControlsDisplay(false);

	LabelizerGUI labelizerGUI = labelizer.getLabelizerGUI(dsi);

	ThematicConfPanel.createThematicConfPanel(
						  i18n.get(Offsetable.class, "positions", "positions"),
						  labelizerGUI.getOffseterPanel(), thematicGUI, 1);

	ThematicConfPanel.createThematicConfPanel(
						  i18n.get(this, "textAndFont", "Text & Font"),
						   labelizerGUI.getTextAndFontPanel(), thematicGUI, 2);

	// Use of font sizer to be upgraded... (see projection = null in generateGraphics)
	//  ThematicConfPanel.createThematicConfPanel(
	// 					  i18n.get(this, "fontsizePerScale", "Font size per Scale"), 
	// 					  labelizerGUI.getFontSizeScalePanel(), thematicGUI, 3);
	
        return thematicGUI;
    }

    @Override
    public void drawingAttributesChanged() {
	labelizer.setFontColor((Color)drawingAttributes.getLinePaint());	
	labelizer.refreshLabelizerGUI();
	super.drawingAttributesChanged();
    }

    @Override    
	public void refresh() {
	// update drawingAttributes in case labelizer font color has changed
	if (drawingAttributes.getLinePaint() != labelizer.getFontColor())
	    drawingAttributes.setLinePaint(labelizer.getFontColor());
	super.refresh();
    }

    public void refreshGUI(final DataStorageIterator dsi) {
        super.refreshGUI(dsi);
	labelizer.refreshGUI(dsi);
    }

    public boolean needsToRegenerateGraphics() {
        return false; 
    }

    public OMResizeableGraphic getLegend(Refreshable r, MouseControlPointsListener ml) {
	return null;
    }

}

