// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import com.bbn.openmap.omGraphics.OMGraphicList;
import java.util.*;
import java.awt.event.*;

import javax.swing.event.*;
import javax.swing.*;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;

import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.*;

import com.bbn.openmap.util.*;
import fr.unicaen.openmap.omGraphics.DrawingAttributes;
import java.awt.BasicStroke;
import java.awt.CardLayout;
import java.awt.Stroke;

/**
 * Visualisations using several columns and an orientable feature.
 * Base class for Histogram and BoxPlot (should also include Pie, but not for the moment) 
 */
abstract public class MultiColumnsRep extends OffsetableColorized implements AttributesNames, ResizerListener {


    public enum Orientation {

        LEFT(i18n.get(Histogram.class, "left", "left")),
        UP(i18n.get(Histogram.class, "up", "up")),
        RIGHT(i18n.get(Histogram.class, "right", "right")),
        DOWN(i18n.get(Histogram.class, "down", "down"));
        private final String text;

        Orientation(String text) {
            this.text = text;
        }

        @Override
        public String toString() {
            return this.text;
        }
    }
    public static final String colorDataColumnNamesPropertyString = "colorDataColumnNames";
    public static final String barsWidthPropertyString = "barsWidth";
    public static final String barsPropertyString = "bars";
    public static final String orientationPropertyString = "orientation";
    /** names of the column used to draw the histogram bars */
    protected String[] colorDataColumnNames = new String[0];
    protected double barsWidth = 10d;
    protected Orientation orientation = Orientation.UP;
    protected Resizer resizer = new Resizer();
    protected ThemaMapStrokeEditorMenu tse;
    protected MinMaxValues minMaxValues;

    public MultiColumnsRep(ThematicDataDisplayLayer layer, String name, String typeName) {
        super(layer, name, typeName);
        resizer.addResizerListener(this);
	resizer.setMaxSize(20);
        interactive = true;
    }

    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        barsWidth = PropUtils.doubleFromProperties(properties, prefix + barsWidthPropertyString, 0.1);
        try {
	    resizer.setProperties(prefix, properties);
            colorDataColumnNames = properties.getProperty(prefix + colorDataColumnNamesPropertyString, "").split("[ \t]+");
            orientation = Orientation.valueOf(properties.getProperty(prefix + orientationPropertyString, "UP"));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        properties.put(prefix + barsWidthPropertyString, String.valueOf(barsWidth));
        properties = resizer.getProperties(prefix, properties);
        StringBuffer colorDataColumnNamesString = new StringBuffer();
        for (int i = 0; i < colorDataColumnNames.length; i++) {
            colorDataColumnNamesString.append(colorDataColumnNames[i]);
            colorDataColumnNamesString.append(" ");
        }
        properties.put(prefix + colorDataColumnNamesPropertyString, String.valueOf(colorDataColumnNamesString));
        properties.put(prefix + orientationPropertyString, orientation.name());
        return properties;
    }

    protected void initMinMaxValues() {
        minMaxValues =
                new MinMaxValues(0, resizer.getValuePerPixel() * (int) Math.ceil(resizer.getMaxSize()));
    }

    public JComponent getGUI(final DataStorageIterator dsi) {
        thematicGUI = (ThematicGUI) super.getGUI(dsi);

        final JSpinner barsWidthSpinner =
                new JSpinner(new SpinnerNumberModel((double) barsWidth, 0d, 10000d, 0.1d));
        barsWidthSpinner.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                setBarsWidth(((Double) barsWidthSpinner.getValue()).doubleValue());
            }
        });
        final JPanel sizePanel = new JPanel();
        sizePanel.setBorder(BorderFactory.createTitledBorder(""));
        sizePanel.add(new JLabel(i18n.get(this, "barsWidthLabel", "Bars Width")), "label");
        sizePanel.add(barsWidthSpinner, "field");

        sizePanel.add(resizer.getGUI(dsi));

        ThematicConfPanel sizeConfPan =
                ThematicConfPanel.createThematicConfPanel(i18n.get(this, "size", "Size"));
        sizeConfPan.setContent(sizePanel);
        thematicGUI.addThematicConfigurationPanel(sizeConfPan);
        JPanel wrapSize = new JPanel();
        wrapSize.add(sizeConfPan);
        thematicGUI.add(wrapSize, 2);

        return thematicGUI;
    }

    private boolean isAColorDataColumnNames(String columnName) {
        for (int i = 0; i < colorDataColumnNames.length; i++) {
            if (colorDataColumnNames[i].equals(columnName)) {
                return true;
            }
        }
        return false;
    }

    protected void setColorDataColumnNames(String[] colorDataColumnNames) {
        this.colorDataColumnNames = colorDataColumnNames;
    }

    /** gives colorDataColumnName[i]. @see Names interface */
    public String getName(int i) {
        return i >= 0 && i < colorDataColumnNames.length ? colorDataColumnNames[i] : "";
    }

    /** @see AttributesNames interface */
    public void setAttributesNames(String[] names) {
        setColorDataColumnNames(names);
    }

    /** remove names from AttributesNames that are no more attributes */
    protected void actualiseAttributesNames(DataStorageIterator dsi) {
         setAttributesNames(colorDataColumnNames);
    }

    protected ArrayList<String> getNoColorDataColumnNames(DataStorageIterator dsi) {
        ArrayList<String> names = new ArrayList<String>();
        for (String name : dsi.getDataColumnNames()) {
            if (!isAColorDataColumnNames(name)) {
                names.add(name);
            }
        }
        return names;
    }

    public void resizerChanged(Resizer resizer) {
        if (this.resizer == resizer) {
            refresh();
        }
    }

    protected double getBarsWidth() { 
	return barsWidth; 
    }

    protected void setBarsWidth(double d) {
        barsWidth = d;
        refresh();
    }

    public Orientation getOrientation() {
        return orientation;
    }
    
    public void setOrientation(Orientation o) {
	orientation = o;
	refresh();
    }

}
