// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import java.util.*;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import fr.unicaen.openmap.dataStorage.*;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.util.*;

/**
 * Multi regular polygons visualization mode.
 */
public class MultiRegularPolygons extends RegularPolygon {

    public static final String elemRadiusPropertyString = "elemRadius";
    public static final String gapPercentagePropertyString = "gapPercentage";
    protected int elemRadius = 5;
    protected int gapPercentage = -10;

    public MultiRegularPolygons(ThematicDataDisplayLayer layer, String name) {
        this(layer, name, i18n.get(MultiRegularPolygons.class, "multiRegPol", "Multi Reg. Polygons"));
    }

    public MultiRegularPolygons(ThematicDataDisplayLayer layer, String name, String typeName) {
        super(layer, name, typeName);
        resizer = new SymbolResizer(new DataColumnValueComputer(""));
        resizer.addResizerListener(this);
        interactive = true;
    }

    // JComboBox dataList;
    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);

        setElemRadius(PropUtils.intFromProperties(properties, prefix + elemRadiusPropertyString, getElemRadius()));
        setGapPercentage(PropUtils.intFromProperties(properties, prefix + gapPercentagePropertyString, getGapPercentage()));

    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);

        properties.put(prefix + elemRadiusPropertyString, Integer.toString(getElemRadius()));
        properties.put(prefix + gapPercentagePropertyString, Integer.toString(getGapPercentage()));
        return properties;
    }

    public OMGraphicList generateGraphics(DataStorageIterator dsi) {

	super.generateGraphics(dsi);

        mouseControlPointsListener.clearControlPoints();

	resizer.setMinMaxValue(dsi);
	OMGraphicList omgl = new OMGraphicList();
        double startAngleInRadians = startAngle * Math.PI / 180.0;
        boolean offVis = offseter.isVisible();
        boolean offLineVis = offseter.isLineVisible();
        while (dsi.hasNext()) {
            DataObject data = dsi.next();
            OMDataLocation omdl = data.getOMDataLocation(geometryColumnName);
            if (omdl == null) {
                continue;
            }

            Centroid centroid = omdl.getCentroid();
            if (centroid == null) {
                continue;
            }

            int nb = (int) resizer.getSize(data);
            double lat = centroid.getLat();
            double lon = centroid.getLon();
            int offX = 0;
            int offY = 0;
            String key = "";
            if (offseter.isOn()) {
		Offseter.OffsetLocationAndKey loc = offseter.getOffsetLocation(data);
		offX = loc.getX();
		offY = loc.getY();
		key = loc.getKey();
            }

            OMMultiRegularPolygon p = new OMMultiRegularPolygon(this, key, 
								nb > 0 ? nb : -nb, lat, lon,
								nbEdges, elemRadius, offX, offY,
								startAngleInRadians, starCoef, gapPercentage, 
								offVis, offLineVis);
	    colorizeGraphic(p, data, nb);
            // Debug.message("ThematicDataDisplayLayer", "MultiRegPolygon "+nb+" ["+centroid.getLat()+", "+centroid.getLon()+"]");
            omgl.add(p);
	}
        omgl.sort();
        return omgl;
    }


    public JComponent getGUI(final DataStorageIterator dsi) {

        thematicGUI = (ThematicGUI) super.getGUI(dsi);
        thematicGUI.remove(regpolysPanel);
        ThematicConfPanel resizerGUI = (ThematicConfPanel) resizer.getGUI(dsi);

        thematicGUI.addThematicConfigurationPanel(resizerGUI);
        JPanel wrapSize = new JPanel();
        wrapSize.add(resizerGUI);
        thematicGUI.add(wrapSize, 2);

        final RegularPolygonPanel regPan = getRegularPolygonGUI();
        JPanel wrapRegPan = new JPanel();
        wrapRegPan.add(regPan);
        ThematicConfPanel regConfPan =
                ThematicConfPanel.createThematicConfPanel(i18n.get(this, "draw", "Draw"));
        regConfPan.setContent(wrapRegPan);
        thematicGUI.addThematicConfigurationPanel(regConfPan);
        JPanel wrapReg = new JPanel();
        wrapReg.add(regConfPan);
        thematicGUI.add(wrapReg, 3);

        Box sbox = Box.createHorizontalBox();
        SpinnerModel model =
                new SpinnerNumberModel(getElemRadius() * 2, //initial value
                0, //min
                150, //max
                5);//step
        final JSpinner elemDiameterField = new JSpinner(model);
        elemDiameterField.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                setElemRadius(((Integer) elemDiameterField.getValue()).intValue());
                getRegularPolygonGUI().repaintSamplePanel();
            }
        });
        sbox.add(elemDiameterField, "field");
        regPan.addToSliderBox(new JLabel(i18n.get(this, "diameterLabel", "Diameter")), sbox);

        sbox = Box.createHorizontalBox();
        model =
                new SpinnerNumberModel(getGapPercentage(), //initial value
                -100, //min
                100, //max
                5);//step
        final JSpinner gapPercentageField = new JSpinner(model);
        gapPercentageField.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                setGapPercentage(((Integer) gapPercentageField.getValue()).intValue());
                //getRegularPolygonGUI().setGap(gapPercentage);
                getRegularPolygonGUI().repaintSamplePanel();
            }
        });
        sbox.add(gapPercentageField, "field");
        regPan.addToSliderBox(new JLabel(i18n.get(this, "gapLabel", "Gap")), sbox);

        return thematicGUI;
    }

    public void setElemRadius(int n) {
        elemRadius = n;
        refresh();
    }

    public int getElemRadius() {
        return elemRadius;
    }

    public int getGapPercentage() {
        return gapPercentage;
    }

    public void setGapPercentage(int g) {
        gapPercentage = g;
        refresh();
    }

    public OMResizeableGraphic getLegend(Refreshable r, MouseControlPointsListener ml) {
        double valuePerUnit = resizer.getValuePerPixel();

        OMResizeableFrame frame = new OMResizeableMultiRegularPolygonLegend(r, ml, 0, 0,
									    nbEdges, elemRadius,
									    startAngle * Math.PI / 180.0, starCoef,
									    drawingAttributes.getLinePaint(), drawingAttributes.getFillPaint(),
									    getDataColumnName(), valuePerUnit, 
									    getColorize() ? colorizer.getColorScale() : null,
									    getColorize() ? colorizer.getColorsTexts() : null,
									    colorDataColumnName.get(),
									    getFullName());
        frame.setVisuFrame(true);
        return frame;
    }

    public void refreshGUI(DataStorageIterator dsi) {
        super.refreshGUI(dsi);
    }

}
