// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import com.bbn.openmap.event.MapMouseListener;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.dataStorage.DataStorageIterator;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicGUI;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicMouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Offseter;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.OffseterListener;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.util.Properties;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JPanel;

/**
 * Offsetable thematic
 * @author jchauvea
 */
public class OffsetableRaw extends Raw implements Offsetable {

    protected Offseter offseter;
    protected ThematicMouseControlPointsListener mouseControlPointsListener;
    protected Box box;

    public OffsetableRaw(ThematicDataDisplayLayer layer, String name) {
        this(layer, name, "");

    }

    public OffsetableRaw(ThematicDataDisplayLayer layer, String name, String typeName) {
        super(layer, name, typeName);
        offseter = new Offseter();
	offseter.addOffseterListener(this);
        mouseControlPointsListener = new ThematicMouseControlPointsListener(this);
    }

    public void offseterChanged(Offseter offseter) {
        if (this.offseter == offseter) {
            refresh();
        }
    }

    @Override
    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        properties = offseter.getProperties(prefix, properties);
        return properties;
    }

    @Override
    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
	offseter.setProperties(prefix, properties);
    }

    @Override
    public JComponent getGUI(final DataStorageIterator dsi) {
        thematicGUI = (ThematicGUI) super.getGUI(dsi);
        ThematicConfPanel confPan =
                ThematicConfPanel.createThematicConfPanel(i18n.get(Offsetable.class, "positions", "Positions"));
        confPan.collapse();
	confPan.setContent(offseter.getGUI(dsi));
        thematicGUI.addThematicConfigurationPanel(confPan);
        JPanel wrap = new JPanel();
        wrap.add(confPan);
        thematicGUI.add(wrap);
        
        return thematicGUI;
    }

    public void refreshGUI(DataStorageIterator dsi) {
        super.refreshGUI(dsi);
        offseter.refreshGUI(dsi);
    }

    public ThematicMouseControlPointsListener getMouseControlPointsListener() {
        return mouseControlPointsListener;
    }

    @Override
    public MapMouseListener getMapMouseListener() {
        return mouseControlPointsListener;
    }

    public Offseter getOffseter() {
        return offseter;
    }

}
