// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
//-*-tab-width:4;-*-
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMPoint;
import com.bbn.openmap.util.Debug;
import java.util.*;
import java.awt.Color;
import java.awt.event.*;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import javax.swing.*;

import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.*;
import fr.unicaen.openmap.layer.dataDisplayLayer.gui.GraphicsPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicGUI;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.ThematicDrawingAttributes;
import fr.unicaen.openmap.omGraphics.DrawingAttributes;

/**
 * Raw visualization mode : draws one geometry
 */
public class Raw extends Thematic implements PropertyChangeListener {

    // property definition string
    public static final String geometryColumnNamePropertyString = "geometryColumnName";
    protected String geometryColumnName = "";
    protected GraphicsPanel graphicsPanel = null;
    protected final static int icon_width = 20;
    protected final static int icon_height = 20;
    protected Dimension guiDimension = new Dimension(200, 170);
    protected DrawingAttributes drawingAttributes;
   
    /**
     * Any additional JMenu items that should be added to the line menu.
     */
    protected JMenu[] lineMenuAdditions = null;

    /**
     * A hook to add to the line menu brought up in the GUI for the
     * DrawingAttributes.
     */
    public void setLineMenuAdditions(JMenu[] lma) {
        lineMenuAdditions = lma;
    }

    public JMenu[] getLineMenuAdditions() {
        return lineMenuAdditions;
    }

    public Raw(ThematicDataDisplayLayer layer, String name) {
        this(layer, name, i18n.get(Raw.class, "raw", "Raw"));
    }

    public Raw(ThematicDataDisplayLayer layer, String name, String typeName) {
        super(layer, name, typeName);
        buildDrawingAttributes();
	if (drawingAttributes != null)
	    drawingAttributes.getPropertyChangeSupport().addPropertyChangeListener(this);
    }

    protected void buildDrawingAttributes() {
        drawingAttributes = new ThematicDrawingAttributes(this, true, false, false, true, true);
    }

    @Override
    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        geometryColumnName = properties.getProperty(prefix + geometryColumnNamePropertyString, "");
	if (drawingAttributes != null)
	    drawingAttributes.setProperties(prefix, properties);
    }

    @Override
    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
	if (drawingAttributes != null) {
	    drawingAttributes.setPropertyPrefix(prefix);
	    properties = drawingAttributes.getProperties(prefix, properties);
	}
        properties.put(prefix + geometryColumnNamePropertyString,
                geometryColumnName);
        return properties;
    }

    public OMGraphicList generateGraphics(DataStorageIterator dsi) {

        OMGraphicList omgl = new OMGraphicList();

        if ("".equals(geometryColumnName) || geometryColumnName == null) {
            if (dsi.getOMColumnNames().length == 0) // no geometry
                return omgl;
            geometryColumnName = dsi.getOMColumnNames()[0];
        }

        for (int i = 0; dsi.hasNext(); i++) {
            DataObject data = dsi.next();
            OMDataLocation omdl = data.getOMDataLocation(geometryColumnName);

            if (omdl == null)
                continue;

            OMGraphic geom = omdl.getOMGraphic();

            // Hack : maybe, there's no shape in OMDataLocation, however if a
            // Centroid exists, we draw an OMPoint
            if (geom == null && omdl.getCentroid() != null)
                geom = new OMPoint(omdl.getCentroid().getLat(), omdl.getCentroid().getLon());
           
            if (geom == null)
                continue;

            // <clipping>
            if (!getLayer().overlapsCurrentProjBB(omdl.getBoundingBox()))
                continue;
            // </clipping>

            // to clone or not to clone ? // omgl.add(geom);
            //OMGraphic omg = clone(geom);
            if (geom instanceof OMGraphicList) {
                OMGraphicList l = (OMGraphicList) geom;
                for (OMGraphic g : l)
                    g.putAttribute(ThematicDataDisplayLayer.INDEX_IN_DS, new Integer(i));
            } else
                geom.putAttribute(ThematicDataDisplayLayer.INDEX_IN_DS, new Integer(
                        i));
            omgl.add(geom);
            //System.out.println(geom.getClass() + " - " + geom.hashCode());
        }

        omgl.setLinePaint(drawingAttributes.getLinePaint());
        omgl.setFillPaint(drawingAttributes.getFillPaint());
        omgl.setStroke(drawingAttributes.getStroke());
        omgl.setMatted(drawingAttributes.isMatted());
        omgl.setMattingPaint(drawingAttributes.getMattingPaint());
        return omgl;
    }

    public DrawingAttributes getDrawingAttributes() {
        return drawingAttributes;
    }

    /**
     * Support object to notify listeners when something has changed.
     */
    protected PropertyChangeSupport propertyChangeSupport = null;

    /**
     * Get the PropertyChangeSupport object to register anything that is
     * interested in finding out when some parameter has changed.
     */
    public PropertyChangeSupport getPropertyChangeSupport() {
        return propertyChangeSupport;
    }

    public void setPropertyChangeSupport(PropertyChangeSupport support) {
        propertyChangeSupport = support;
    }

    public void propertyChange(PropertyChangeEvent pce) {
        if (graphicsPanel != null)
            graphicsPanel.updateSamplePanel();

    }

    public JComponent getGUI(final DataStorageIterator dsi) {
        if (thematicGUI != null) {
            return thematicGUI;
        }
        thematicGUI = new ThematicGUI(guiOrientation, this);
        thematicGUI.setBackground(Color.LIGHT_GRAY);

	JPanel rawPanel = new JPanel(new BorderLayout());
        JPanel geometryBox = new JPanel();

        try {
            geometryList = new JComboBox(dsi.getOMColumnNames());
            if (!"".equals(geometryColumnName) && geometryColumnName != null) {
                //System.out.println(geometryColumnName + " ->SELECTION DE lINDEX =" + dsi.getIndexForOMColumnName(geometryColumnName));
                geometryList.setSelectedIndex(dsi.getIndexForOMColumnName(geometryColumnName));
            } else {
                geometryColumnName = dsi.getOMColumnNames()[0];
                geometryList.setSelectedIndex(dsi.getIndexForOMColumnName(geometryColumnName));
            }
        } catch (NullPointerException ex) {
            Debug.error("Geometry Column Name List is unavailable");
            geometryList = new JComboBox();
        } catch (java.lang.ArrayIndexOutOfBoundsException ofb) {
            Debug.error("Empty Geometry Column Name List");
            geometryList = new JComboBox();
        }

        geometryList.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                try {
                    int index = ((JComboBox) e.getSource()).getSelectedIndex();
                    setOMColumnName(dsi.getRealOMColumnName(index));
                } catch (ArrayIndexOutOfBoundsException ex) {
                }
            }
        });
        geometryBox.add(geometryList);
        JPanel wrapGeo = new JPanel(new BorderLayout());
        wrapGeo.add(geometryBox, BorderLayout.NORTH);
        JPanel colorLinePanel = new JPanel(new BorderLayout());
	if (drawingAttributes != null) {
	    colorLinePanel.add(graphicsPanel = new GraphicsPanel(drawingAttributes, true), 
			       BorderLayout.CENTER);
	    colorLinePanel.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 10));
	}
        Box panStyle = Box.createHorizontalBox();

        rawPanel.add(wrapGeo, BorderLayout.NORTH);
        JPanel wrapCLP = new JPanel(new BorderLayout());
        wrapCLP.add(colorLinePanel, BorderLayout.WEST);
        rawPanel.add(wrapCLP, BorderLayout.CENTER);
        panStyle.setBorder(BorderFactory.createEmptyBorder(0, 40, 5, 0));
        rawPanel.add(panStyle, BorderLayout.SOUTH);
        ThematicConfPanel tcpGeo = ThematicConfPanel.createThematicConfPanel(i18n.get(Raw.class, "colors_line",
                "Geographic entity"));
        JPanel wrap = new JPanel();
        wrap.add(tcpGeo);
        tcpGeo.setContent(rawPanel);
        thematicGUI.add(wrap);
        thematicGUI.addThematicConfigurationPanel(tcpGeo);
        return thematicGUI;
    }

    public void drawingAttributesChanged() {
	refresh();
    }

    protected JComboBox geometryList;

    protected void graphicsPanelToggleControlsDisplay(boolean displayFillModeCB) {
        if (graphicsPanel != null) {
            graphicsPanel.toggleControlsDisplay(drawingAttributes, displayFillModeCB);
        }
    }

    public void setOMColumnName(String name) {
	if (name == null) {
	    geometryColumnName = name;
	    return;
	}
	if (! name.equals(geometryColumnName)) {
	    geometryColumnName = name;
	    refresh();
	}
    }

    public void refreshGUI(final DataStorageIterator dsi) {
        if (geometryList == null) {
            return;
        }

	if (dsi.getOMColumnNames() == null || dsi.getOMColumnNames().length == 0)
	    return;
        ActionListener[] als = geometryList.getActionListeners();
        for (int i = 0; i < als.length; i++) {
            geometryList.removeActionListener(als[i]);
        }
        geometryList.removeAllItems();
        //System.out.println("---------"+dsi.getOMColumnNames().length+"---------------------");
        for (int i = 0; i < dsi.getOMColumnNames().length; i++) {
            geometryList.addItem(dsi.getOMColumnNames()[i]);
        }
        //System.out.println("------------------------------");
        int index = dsi.getIndexForOMColumnName(geometryColumnName);
        if (index == -1 && dsi.getOMColumnNames().length > 0) {
            index = 0;
        }

        geometryList.setSelectedIndex(index); // do this before adding the actionListener !!!!
        // refresh the column name to the default set by the gui :
        //setOMColumnName(dsi.getRealOMColumnName(geometryList.getSelectedIndex()));
        if (geometryList.getSelectedItem() != null) {
            geometryColumnName = geometryList.getSelectedItem().toString();
        }
        //geometryList.updateUI();
        geometryList.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                try {
                    int index = ((JComboBox) e.getSource()).getSelectedIndex();
                    setOMColumnName(dsi.getRealOMColumnName(index));
                    setBadCache(true);
                    getLayer().refreshGraphics();
                } catch (ArrayIndexOutOfBoundsException ex) {
                }
            }
        });
    }

    public boolean isReady() {
        return geometryColumnName != null && !"".equals(geometryColumnName);
    }

    public String getOMColumnName() {
        return geometryColumnName;

    }

    @Override
    public void setVisible(boolean visible) {
        super.setVisible(visible);
        if (useSharedGraphics()) {
            getLayer().aSharedGraphicsThematicHasChanged();
        }
    }
}
