// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.utils;

import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.util.Util;
import java.io.File;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.TransformerException;

import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;



/**
 * Provides transformation of an xml file to another
 * file format using a xsl stylsheet.
 * @version 1.0
 * @author Arnaud Daret, arnaud.daret@info.unicaen.fr
 */
public class XMLTransformer {
    TransformerFactory transformerFactory = null;
    StreamSource stylesource = null;
    File stylesheet = null;
    Transformer transformer = null;
    Boolean styleChanged = true;
    
    /** Empty Constructor */
    public XMLTransformer() {
        transformerFactory = TransformerFactory.newInstance();
        
    }
    
    
    /**
     * Tranform method
     * @param in the xml file you want to transform
     * @param stylesheet the stylesheet you will apply for the transformation
     * @param out the result file
     */
    public void transform(File in, File stylesheet, File out) {        
        long currentTime = System.currentTimeMillis();
        long startTime = currentTime;
        long oldTime = currentTime;
        try {
            // TransformerFactory transformerFactory = TransformerFactory.newInstance();
            if (this.stylesheet==null || stylesource==null || !this.stylesheet.equals(stylesheet)) {
                this.stylesheet=stylesheet;
                stylesource = new StreamSource(stylesheet);
                styleChanged = true;
            } else styleChanged = false;
            
            currentTime = System.currentTimeMillis();
            Debug.message(Util.className(this), this + "Stream_Time = "+(currentTime-oldTime)+" ms");
            oldTime = currentTime;
            
            if (transformer==null || styleChanged) transformer = transformerFactory.newTransformer(stylesource);
            currentTime = System.currentTimeMillis();
            Debug.message(Util.className(this), this + "FactoryTime = "+(currentTime-oldTime)+" ms");
            oldTime = currentTime;
            
            transformer.setOutputProperty(OutputKeys.VERSION,"1.0");
            StreamSource source = new StreamSource(in);
            StreamResult result = new StreamResult(out);
            transformer.transform(source, result);
            currentTime = System.currentTimeMillis();
            Debug.message(Util.className(this), this + "TransfoTime = "+(currentTime-oldTime)+" ms");
            
        } catch(TransformerException e) {
            e.printStackTrace();
        } finally {
            currentTime = System.currentTimeMillis();
            Debug.message(Util.className(this), this + "TOTAL__TIME = "+(currentTime-startTime)+" ms\n");
        }
    }
}
