// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.omGraphics.event;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.event.StandardMapMouseInterpreter;
import com.bbn.openmap.proj.coords.LatLonPoint;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import java.awt.event.MouseEvent;

/**
 *
 * @author jchauvea
 */
public class TMMapMouseInterpreter extends StandardMapMouseInterpreter {
    
    public TMMapMouseInterpreter(ThematicDataDisplayLayer layer){
        super(layer);
    }

    /** allow to freeze highlightning with left click */
    private boolean _freezeHighlight = false; 
    public void freezeHighlight() { _freezeHighlight = true; }
    public void unfreezeHighlight() { _freezeHighlight = false; }
    public boolean highlightIsFrozen() { return _freezeHighlight; }
    
    /**
     * Notification that an OMGraphic was right-clicked on.
     *
     * @return true
     */
    public boolean rightClick(OMGraphic omg, MouseEvent me) {
        if (!(omg instanceof OMGraphicList))
            return displayPopup(grp.getItemsForOMGraphicMenu(omg), me);
        else {
            for (OMGraphic om : (OMGraphicList) omg) {
                if(om.contains(me.getX(), me.getY())){
		    om.setAttributes(omg.getAttributes());
		    return displayPopup(grp.getItemsForOMGraphicMenu(om), me);
                }
            }
        }
        return false;
    }

    /**
     * Notification that the mouse is over an OMGraphic. Makes all the highlight
     * calls.
     * 
     * @return true
     */
    public boolean mouseOver(OMGraphic omg, MouseEvent me) {
	if (highlightIsFrozen())
	    return true;
	return super.mouseOver(omg, me);
    }


    /**
     * Handle a left-click on an OMGraphic. 
     * 
     * @return true
     */
    public boolean leftClick(OMGraphic omg, MouseEvent me) {
	unfreezeHighlight();
	mouseOver(omg, me); //-> select(omg);
	freezeHighlight();
	return false;
    }

    /**
     * Notification that the user clicked on something else other than the
     * provided OMGraphic that was previously left-clicked on. Calls
     * deselect(omg).
     * 
     * @return false
     */
    public boolean leftClickOff(OMGraphic omg, MouseEvent me) {
	unfreezeHighlight();
	return super.leftClickOff(omg, me);
    }

}
