// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.tools.drawing;

import com.bbn.openmap.I18n;
import com.bbn.openmap.omGraphics.EditableOMGraphic;
import com.bbn.openmap.omGraphics.EditableOMPoly;
import com.bbn.openmap.omGraphics.GraphicAttributes;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMPoly;
import com.bbn.openmap.tools.drawing.AbstractToolLoader;
import com.bbn.openmap.tools.drawing.EditToolLoader;
import com.bbn.openmap.tools.drawing.OMPolyLoader;
import com.bbn.openmap.tools.drawing.EditClassWrapper;
import com.bbn.openmap.util.Debug;

/**
 * Loader that knows how to create/edit OMPoly objects.
 * Uses the class fr.unicaen.openmap.tools.drawing.OMPolygon 
 * which is an OMPOly with a different name, in order to recognize it.
 * 
 * @author Jacques.Madelaine@unicaen.fr
 */
public class TMPolygonLoader extends AbstractToolLoader implements EditToolLoader {

    protected String polygonClassName = "fr.unicaen.openmap.tools.drawing.OMPolygon";

    public TMPolygonLoader() {
        init();
    }

    public void init() {
	addEditClassWrapper(new EditClassWrapper(polygonClassName, 
						 "com.bbn.openmap.omGraphics.EditableOMPoly", 
						 "editablepoly.gif", 
						 i18n.get(TMPolygonLoader.class, "polygon", "Polygon")));
    }

    /**
     * Give the classname of a graphic to create, returning an
     * EditableOMGraphic for that graphic. The GraphicAttributes
     * object lets you set some of the initial parameters of the poly,
     * like poly type and rendertype.
     */
    public EditableOMGraphic getEditableGraphic(String classname,
                                                GraphicAttributes ga) {
        if (classname.intern() == polygonClassName) {
            EditableOMPoly eomp = new EditableOMPolygon(ga);
	    return eomp;
        }
        return null;
    }

    /**
     * Give an OMGraphic to the EditToolLoader, which will create an
     * EditableOMGraphic for it.
     */
    public EditableOMGraphic getEditableGraphic(OMGraphic graphic) {
        if (graphic instanceof OMPolygon) {
            return new EditableOMPolygon((OMPoly) graphic);
        }
        return null;
    }
    
}

