// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util;

import java.awt.Color;
import java.util.Properties;
import com.bbn.openmap.util.ColorFactory;
import com.bbn.openmap.omGraphics.OMColor;

/**
 * The purpose for this class is to exchange colors
 * between parts of the software in a single object.
 */
public class Colors {
    public enum ColorType { FILL_COLOR, LINE_COLOR, SELECTED_FILL_COLOR, SELECTED_LINE_COLOR, AXES_LINE_COLOR };
    public static final String lineColorPropertyString = "lineColor";
    public static final String fillColorPropertyString = "fillColor";
    public static final String selectedFillColorPropertyString = "selectedFillColor";
    public static final String selectedLineColorPropertyString = "selectedLineColor";
    public static final String axesLineColorPropertyString = "axesLineColor";
    public static final String fillPropertyString = "fill";

    private Color lineColor = new Color(0 ,0 ,0); //black;
    private Color fillColor = new Color(254, 247, 198); //Color.green;
    private Color selectedLineColor = Color.white;
    private Color selectedFillColor = Color.blue;
    private Color axesLineColor = Color.BLACK;
    private boolean fill = false;
    
    public Color getLineColor() { return lineColor; }
    public void setLineColor(Color lineColor) { this.lineColor = lineColor; }
    public void setLineColor(String s) { lineColor = parseColor(s); }

    public Color getAxesLineColor() { return axesLineColor; }
    public void setAxesLineColor(Color c){ axesLineColor = c; }
    public void setAxesLineColor(String s){ axesLineColor = parseColor(s);}
    
    public Color getRealFillColor() { return fillColor; }
    public Color getFillColor() { return fill ? fillColor : OMColor.clear; }
    public void setFillColor(Color fillColor) { this.fillColor = fillColor; }
    public void setFillColor(String s) { fillColor = parseColor(s); }
    
    public Color getSelectedLineColor() { return selectedLineColor; }
    public void setSelectedLineColor(Color selectedLineColor) { this.selectedLineColor = selectedLineColor; }
    public void setSelectedLineColor(String s) { selectedLineColor = parseColor(s); }
    
    public Color getSelectedFillColor() { return fill ? selectedFillColor : null; }
    public void setSelectedFillColor(Color selectedFillColor) { this.selectedFillColor = selectedFillColor; }
    public void setSelectedFillColor(String s) { selectedFillColor = parseColor(s); }
    
    public boolean getFillMode() { return fill; }
    public void setFillMode(boolean fill) { this.fill = fill; }
    
    public static Color parseColor(String s) {
	return ColorFactory.parseColor(s, !(s.length() < 7));
    }
    /**
     * This function is provided for debugging purpose.
     */
    public String toString() {
	return "Colors:\n  " +
	    "lineColor: " + ColorFactory.getHexColorString(lineColor) + "\n  " +
	    "fillColor: " + ColorFactory.getHexColorString(fillColor) + "\n  " +
	    "selectedLineColor: " + ColorFactory.getHexColorString(selectedLineColor) + "\n  " +
	    "selectedFillColor: " + ColorFactory.getHexColorString(selectedFillColor) + "\n  " +
	    "fill: " + fill + "\n";
    }
    
    /**
     * used to set the object with properties ressources
     */
    public void setProperties(String prefix, Properties properties) {
	setLineColor(properties.getProperty(prefix + lineColorPropertyString, "ff8822d0"/*"ffacc5e3"*/));
	setFillColor(properties.getProperty(prefix + fillColorPropertyString, "fffef7c6"));
	setSelectedLineColor(properties.getProperty(prefix + selectedLineColorPropertyString, "FF990099"));
	setSelectedFillColor(properties.getProperty(prefix + selectedFillColorPropertyString, "C900FF55"));
	setFillMode(Util.isTrue(properties.getProperty(prefix + fillPropertyString, "true")));
        setAxesLineColor(properties.getProperty(prefix + axesLineColorPropertyString, "ff000000"));
    }
    
    public Properties getProperties(String prefix, Properties properties) {
	if (getLineColor() != null)
	    properties.put(prefix + lineColorPropertyString, ColorFactory.getHexColorString(getLineColor()));
	if (getFillColor() != null)
	    properties.put(prefix + fillColorPropertyString, ColorFactory.getHexColorString(getFillColor()));
	if (getSelectedLineColor() != null)
	    properties.put(prefix + selectedLineColorPropertyString, ColorFactory.getHexColorString(getSelectedLineColor()));
	if (getSelectedFillColor() != null)
	    properties.put(prefix + selectedFillColorPropertyString, ColorFactory.getHexColorString(getSelectedFillColor()));
        if (getAxesLineColor() != null)
	    properties.put(prefix + axesLineColorPropertyString, ColorFactory.getHexColorString(getAxesLineColor()));
        properties.put(prefix + fillPropertyString, new Boolean(getFillMode()).toString());
        return properties;
    }

}
