// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util;


import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;
import fr.unicaen.openmap.dataStorage.DataStorage;
import fr.unicaen.openmap.dataStorage.DataStorageListener;
import fr.unicaen.openmap.util.propertyEditor.DataStoragePropertyEditor;
import fr.unicaen.openmap.util.propertyEditor.Inspector;
import java.awt.*;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.ImageIcon;
import java.io.File;
import java.io.InputStream;
import java.lang.reflect.Constructor;
import java.util.Properties;
import java.util.regex.Pattern;
import java.util.zip.GZIPInputStream;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JOptionPane;
import java.util.HashMap;
import java.util.Set;

public final class Util {

    /**
     * Property string for the OMGraphic Index in 
     */
    static public final String OMGraphicIndex = "index";
    static Pattern spaces = Pattern.compile("\u00a0");

    static public String className(Object o) {
        String s = o.getClass().getName();
        return s.substring(s.lastIndexOf('.') + 1, s.length());
    }

    static public ImageIcon loadIcon(String file) {
	try {
            URL url = Util.class.getResource("/fr/unicaen/openmap/data/images/" + file);
	    if (url != null)
                return new ImageIcon(url, file);
	    String name = locateFile(file);
	    if (name != null)
		return new ImageIcon(name, file);
            return null;
	} catch (Exception e) { e.printStackTrace();return null; }
    }

    static public String vectorToString (String[] cols){
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < cols.length; i++) {
            sb.append(cols[i]);
            if (i+1!=cols.length) sb.append(",");
        }
        return sb.toString();
    }

    static public Vector<String> textToVector(String text) {
        Vector<String> vector = new Vector<String>();
        StringTokenizer st = new StringTokenizer(text, "\n");
        for (; st.hasMoreTokens();) {
            String s = st.nextToken().trim();
            if (!vector.contains(s)) {
                vector.addElement(s);
            }
        }
        return vector;
    }

    static public Color getDefaultTextColor(Color c) {
        int r = c.getRed();
        int g = c.getGreen();
        int b = c.getBlue();
        if (r > 240 || g > 240) {
            return Color.black;
        } else {
            return Color.white;
        }
    }

    static public String splitString(String message, int max) {
        if (message.length() <= max) {
            return message;
        }
        int index = 0;
        int tmp = 0;
        while (tmp != -1 && tmp < max) {
            index = tmp;
            tmp = message.indexOf(' ', index + 1);
        }
        return message.substring(0, index) + "\n" +
	    splitString(message.substring(index + 1, message.length()), max);
    }

    /**
     * Returns true if string is equals (case insensitive) to on, yes, true.
     */
    public static boolean isTrue(String bool) {
        return ((bool.equalsIgnoreCase("on")) ||
                (bool.equalsIgnoreCase("yes")) ||
                (bool.equalsIgnoreCase("true")));
    }

    /**
     *
     */
    public static int getMax(int[] values) {
        int max = Integer.MIN_VALUE;

        for (int i = 0; i < values.length; i++) {
            if (values[i] > max) {
                max = values[i];
            }
        }
        return max;
    }

    /**
     * Returns a Rectangle2D which is the bounding box of the string s in the default font of the graphics
     */
    public static Rectangle2D getStringBounds(String s, Graphics g) {
        return getStringBounds(s, g.getFont());
    }

    /**
     * Returns a Rectangle2D which is the bounding box of the string s in the given font
     */
    public static Rectangle2D getStringBounds(String s, Font f) {
        return f.getStringBounds(s, new FontRenderContext(null, true, true));
    }

    /**
     * Locate file 'fileName' in classpath, if it is not an absolute file name.
     * @return absolute name of the file as a string if found, null otherwise.
     * borrowed from ShapeLayer
     */
    public static String locateFile(String name) {
        //System.out.println("NAME = " + name);
        File file = new File(name);
        if (file.exists()) {
            return name;
        } else {
            URL url = ClassLoader.getSystemResource(name);
            //System.out.println("URL = " + url);
            //OK, now we want to look around for the file, in the
            //classpaths, and as a resource.  It may be a file in
            //a classpath, available for direct access.
            if (url != null) {
                String newname = url.getFile();
                //System.out.println("New Name = " + newname);
                file = new File(newname);
                if (file.exists()) {
                    return newname;
                }
            }
        }
        return name;
    }

    /**
     * Locate file 'fileName' in classpath, or as an absolute file name.
     * @return URL of the file if found, null otherwise.
     */
    public static URL getURLforFileName(String name) {
        File file = new File(name);
        try {
            if (file.exists()) {
                return file.toURI().toURL();
            }
        } catch (java.net.MalformedURLException e) {
            e.printStackTrace();
            return null;
        }

        return ClassLoader.getSystemResource(name);
    }

    static public DataStorage loadDataStorage(String className, String prefix, Properties properties, DataStorageListener dsl) {

        try {
            DataStorage ds = createDataStorage(className, dsl);


	    // try{throw new Exception("LOAD DS");} catch(Exception e){e.printStackTrace();}
            if (ds == null) {
                Debug.error("DataStorage is null !!! " + className);
                return null;
            }
	    // System.out.println("## Set Properties for: " + ds);
            ds.setProperties(prefix, properties);
            return ds;
        } catch (Exception e) {
            Debug.error("Error loading DataStorage !!! " + className + " encountred " + e);
            e.printStackTrace();
        }
        return null;
    }

    static public DataStorage createDataStorage(String className, DataStorageListener dsl) {
        try {
	    // try{throw new Exception("CREATE DS");} catch(Exception e){e.printStackTrace();}
            Class cls = Class.forName(className);
            Constructor[] constructors = cls.getConstructors();
            for (int i = 0; i < constructors.length; i++) {
                Class params[] = constructors[i].getParameterTypes();
                if (params.length == 1 && params[0].getName().equals("fr.unicaen.openmap.dataStorage.DataStorageListener")) {
                    Object obj = constructors[i].newInstance(new Object[]{dsl}); //
		    // System.out.println("## New DataStorage constructed: " + obj);
                    return ((DataStorage) obj);
                }
            }
            Debug.error("Util : no constructor found for " + className);
            return null;
        } catch (Exception e) {
            Debug.error("Util : createDataStorage failure for class \"" + className + "\" raised " + e.getClass().getName());
            // e.printStackTrace();
            return null;
        }
    }

    public static boolean equals(double x1, double x2, double precision) {
	return (Math.abs(x1) < precision*precision && Math.abs(x2) < precision*precision)
	    || Math.abs(x1-x2) < Math.abs(x1*precision);
    }

    private static int maxNumberFormatExceptionOutput = 20;
    public static void resetMaxNumberFormatExceptionOutput() {
	setMaxNumberFormatExceptionOutput(20);
    }
    public static void setMaxNumberFormatExceptionOutput(int m) {
	maxNumberFormatExceptionOutput = m;
    }

    /** Robust, crude and expensive parseDouble method. 
     */
    public static Double parseDouble(Object o) {
        if (o == null) {
            //System.out.println(Double.MIN_VALUE);
            return 0.0;
        }
        if (o instanceof Double) {
            return (Double) o;
        }
        String number = o.toString();
        if ("".equals(number)) {
            return 0.0;
        }

        try {
            number = spaces.matcher(number).replaceAll("");
            return Double.valueOf(number);
        } catch (NumberFormatException e) {
	    if (maxNumberFormatExceptionOutput-- > 0)
		System.err.println("NumberFormatException for " + o);
        }
        return 0.0;
    }

    /** Compare */
    public static int compare(Object o1, Object o2) {
        try {
            if (o1 instanceof Double) {
                return ((Double) o1).compareTo((Double) o2);
            }
        } catch (ClassCastException e) {
        }
        return o1.toString().compareTo(o2.toString());
    }

    public static InputStream open(String fileName) {
	return open(fileName, false);
    }

    public static File openFile(String fileName){
        try {
            URL url = PropUtils.getResourceOrFileOrURL(fileName);
            return new File(url.toURI());
        } catch (URISyntaxException ex) {
            Logger.getLogger(Util.class.getName()).log(Level.SEVERE, null, ex);
        } catch (MalformedURLException ex) {
            Logger.getLogger(Util.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    public static InputStream open(String fileName, boolean tryToUnzip) {
        InputStream in = null;
        URL url = null;
        try {
            url = PropUtils.getResourceOrFileOrURL(fileName);
        } catch (java.net.MalformedURLException e) {
        }

        if (url != null) {
            try {
                in = url.openStream();
            } catch (java.io.IOException e) {
                Debug.error("IOException for file " + fileName);
                in = null;
            }
        }

        if (in != null && tryToUnzip) { // try to uncompress...
            try {
                in = new GZIPInputStream(in);
            } catch (Exception notGZIPexception) {
                // It's not a zip file, then we re-open the url
                try {
                    in = url.openStream();
                } catch (java.io.IOException e) {
                    Debug.error("IOException for file " + fileName);
                    in = null;
                }
            }
        }
	
        return in;
    }

    public static InputStream openWithFailOver(String fileName, boolean tryToUnzip, DataStorage ds) {

	InputStream in = Util.open(fileName, tryToUnzip);

        if (in != null)
	    return in;

	Inspector inspector = new Inspector();
	I18n i18n = Environment.getI18n();
	do {
	    JPanel p = new JPanel(new BorderLayout());
	    String f = i18n.get(ds, "file", "The file") + " ";
	    String nf = " " + i18n.get(ds, "not_found", "was not found - please browse to select a new one");
	    JLabel lab = new JLabel("<html><b>" + f + "'" + fileName + "'" + nf + "</b></html>");
	    lab.setForeground(Color.RED);
	    p.add(lab, BorderLayout.NORTH);
	    fileName = "#";
	    p.add(DataStoragePropertyEditor.provideSimpleCustomPanel(ds, inspector), BorderLayout.CENTER);
	    int choice = JOptionPane.showConfirmDialog(null, p, "Please Select a CSV File",
						       JOptionPane.OK_CANCEL_OPTION,
						       JOptionPane.INFORMATION_MESSAGE);
	    if (choice == JOptionPane.CANCEL_OPTION) {
		return null;
	    }
	    ds.setProperties(inspector.collectProperties());
	} while (!ds.isReady());
	return null;
    }

    public static final HashMap<String, String> escapeMapIn = new HashMap<String, String>()
    {
	{
	    //    in   out
	    put("=", "\u241F");   // ␟ e2 90 9f SYMBOL FOR UNIT SEPARATOR
	    put(" ", "\u2420");    // ␠ e2 90 a0 SYMBOL FOR SPACE
	    put("\t", "\u2409");   // ␉ e2 90 89 SYMBOL FOR HORIZONTAL TABULATION 
	    put("\"", "\u241D");   // ␝ e2 90 9d SYMBOL FOR GROUP SEPARATOR
	    put(";", "\u241E");    // ␞ e2 90 9e SYMBOL FOR RECORD SEPARATOR          
	}
    };

    public static final HashMap<String, String> escapeMapOut = new HashMap<String, String>()
    {
	{
	    //    in            out
	    put("\u241F", "=");   // ␟ e2 90 9f SYMBOL FOR UNIT SEPARATOR      
	    put("\u2420", " ");    // ␠ e2 90 a0 SYMBOL FOR SPACE 	     
	    put("\u2409", "\t");   // ␉ e2 90 89 SYMBOL FOR HORIZONTAL TABULATION       
	    put("\u241D", "\"");   // ␝ e2 90 9d SYMBOL FOR GROUP SEPARATOR        
	    put("\u241E", ";");    // ␞ e2 90 9e SYMBOL FOR RECORD SEPARATOR           
	}
    };

    public static String escapePropertyString( String in )
    {
	if( in == null )
	    {
		return null;
	    }

	Set<String> keys = escapeMapIn.keySet();
	String out = "";
	for( char c : in.toCharArray() )
	    {
		if( keys.contains(""+c) )
		    {
			out = out + escapeMapIn.get(""+c);
		    }
		else
		    {
			out = out + c;
		    }
	    }

	return out;
    }

    public static String unEscapePropertyString( String in )
    {
	if( in == null )
	    {
		return null;
	    }

	Set<String> keys = escapeMapOut.keySet();
	String out = "";
	for( char c : in.toCharArray() )
	    {
		if( keys.contains(""+c) )
		    {
			out = out + escapeMapOut.get(""+c);
		    }
		else
		    {
			out = out + c;
		    }
	    }

	return out;
    }

}
