// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util;

import fr.unicaen.openmap.dataStorage.SortDataStorage;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import fr.unicaen.openmap.util.Util;
/**
 * Factory for various Comparator specified by SortDataStorage.Format
 */
public class ValueComparatorFactory {

    public static Comparator<String> createComparator(SortDataStorage.Format format, final boolean ascending) {
	switch (format) {
	case NUMERIC:
	    return new Comparator<String>() {
		public int compare(String o1, String o2) {
		    double d1 = Util.parseDouble(o1);
		    double d2 = Util.parseDouble(o2);
		    if (d1 < d2) 
			return ascending ? -1 : +1;
		    if (d1 > d2)
			return ascending ? +1 : -1;
		    return 0;
		}
	    };
	case STRING:
	    return new Comparator<String>() {
		public int compare(String o1, String o2) {
		    return (ascending ? 1 : -1) * o1.compareTo(o2);
		}
	    };
	default: // date
	    return new DateComparator(format, ascending);
	}

    }
    
}

class DateComparator implements Comparator<String> {
    private SimpleDateFormat dateFormat;
    private boolean ascending;
    public DateComparator(SortDataStorage.Format format, boolean ascending) {
	dateFormat = new SimpleDateFormat(format.getInnerFormat());
	this.ascending = ascending;
    }
    
    public int compare(String o1, String o2) {
	try {
	    Date date1 = dateFormat.parse(o1);
	    Date date2 = dateFormat.parse(o2);
	    Long time1 = date1.getTime();
	    Long time2 = date2.getTime();
	    //Logger.getLogger("ValueComparator").log(Level.INFO, o1 + " compare to " + o2 + " = " + time1.compareTo(time2));
	    return (ascending ? 1 : -1) * time1.compareTo(time2);
	} catch (ParseException ex) {
	    //Logger.getLogger(ValueComparator.class.getName()).log(Level.SEVERE, null, ex);
	}
	return 0;
    }
}

