// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.classification;

import com.bbn.openmap.util.Debug;

import java.util.ArrayList;
import java.util.Collections;
import fr.unicaen.openmap.dataStorage.DataObject;
import fr.unicaen.openmap.util.Util;

/**
 * Abstract base class for String classification.
 */
abstract public class ClassificationOfString extends Classification {
    protected ArrayList<String> data;
    public ClassificationOfString(ArrayList<String> data, int nbClasses) {
	super(nbClasses);
	reInit(data, nbClasses);
    }

    public void reInit(fr.unicaen.openmap.dataStorage.DataStorageIterator dsi,java.lang.String dataColumnName) {
	data = new ArrayList<String>();
	while (dsi.hasNext()) {
	    DataObject dataObject = dsi.next();
	    Object d = dataObject.getData(dataColumnName);
	    data.add(d != null ? d.toString() : "");
	}
	dsi.reset();
	// System.out.println("### ClassificationOfString.reInit( "+ dataColumnName + " ) nbClasses = "+ nbClasses);
	init();
    }

    public void reInit(ArrayList<String> data, int nbClasses) {
	this.data = data;
	this.nbClasses = nbClasses;
	if (data != null)
	    init();
    }

    public boolean hasNoData() { return data == null || data.size() == 0; }
    
    public void unsetData() { data = null; } 


    // breaks ???

    protected String[] breaks = null;
    /**
     * breaks.size() == nbClasses + 1
     * <br>min == breaks[0] and max == breaks[nbClasses]
     */ 
    public String[] getBreaks() { return breaks; }

    protected void initCountsFromBreaks() {
	counts = new int[breaks.length-1];		
	for(int i = 0; i < counts.length; i++)
	    counts[i] = 0;
		
	for(String s : data)
	    for(int j = 0; j < breaks.length-1; j++)
		if (s.compareTo(breaks[j+1]) < 0) {
		    counts[j]++;
		    break;
		}
    }
    /**
     * return the class rank of the value between 1 and nbClasses
     * <p>Not really used
     */
    public int getClass(Object o) {	
	if (breaks == null)
	    return 0;
	String v = o.toString();
	int i = 1;
	for (; i <= nbClasses; i++)
	    if (breaks[i].equals(v))
		return i-1;
	return i-1;
    }

    public String getBreaksAsString() {
	if (breaks == null) 
	    return "";
	StringBuilder s = new StringBuilder();
	for (String b : breaks) {
	    s.append("\t");
	    s.append(b);
	}
	return s.toString();
    }

    public void reInit(Classification c, int nbClasses) {
	ArrayList<String> data;
	c = c.getRealClassif();
	if (c instanceof ClassificationOfDouble) {
	    ClassificationOfDouble cod = (ClassificationOfDouble)c;
	    if (cod.data == null)
		data = null;
	    else {
		data = new ArrayList<String>(cod.data.size());
		for (Double d : cod.data)
		    data.add(d.toString());
	    }
	} else 	if (c instanceof ClassificationOfString) {
	    data = ((ClassificationOfString)c).data;
	} else {
	    Debug.error("### ClassificationOfString.reInit with an unknown Classification! ### "+c.getClass());
	    return;
	}
	reInit(data, nbClasses);
    }

    public String toString() {
	StringBuilder ret = new StringBuilder(super.toString());
	int totalcounts = 0;
	ret.append("#"+nbClasses + " [");
	if (breaks != null) {
	    for (int i = 0; i < breaks.length-1; i++) { 
		ret.append(breaks[i]);
		if (counts != null) {
		    ret.append("("+ counts[i] + ")");
		    totalcounts += counts[i];
		}
		ret.append(", ");
	    }  
	    ret.append(breaks[breaks.length-1]);
	} 
	ret.append(" ] ");
	if (counts != null)
	    ret.append("(" + totalcounts + ") ");
	if (breaks != null)
	    ret.append(breaks.length);
	return ret.toString();
    }

    public String dump() {
	Collections.sort(data);
	String ret = "Data ("+data.size()+") [";
	for(String d : data) ret += String.format("%s; ", d);
	return ret + "]";
    }

    /** returns a text describing the ith class */
    public String getClassText(int i) {
	if (breaks == null || i+1 >= breaks.length)
	    return "";
	return ((i == 0 ? "[ " : "] ") 
		+ breaks[i]
		+ " - " 
		+ breaks[i+1] 
		+"]");
    }

    /** returns texts describing the classes */
    public String[] getClassesTexts() {
	int nbTexts = 0;
	if (breaks != null) //avoid null pointer
	    nbTexts = breaks.length - 1;
	String[] texts = new String[nbTexts];
	for (int i = 0; i < nbTexts; i++)
	    texts[i] = getClassText(i);
	return texts;
    }

}
