// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.classification;

import java.util.ArrayList;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerListModel;


/**
 * Reset automatically nbClasses when its value is not admissible.
 * Admissible values are given by a vector.
 */
abstract public class ClassificationWithContrainedNbOfClasses extends ClassificationOfDouble {

    public ClassificationWithContrainedNbOfClasses(ArrayList<Double> data, int nbClasses) {
	super(data, nbClasses);
	nbClasses = getClosestAdmissibleNumberOfClasses(nbClasses);
    }

    protected int getClosestAdmissibleNumberOfClasses(int n) {
	Integer admissibleNbClasses[] = getAdmissibleNbClasses();
	for (int i = 0; i < admissibleNbClasses.length; i++) {
	    if (n == admissibleNbClasses[i])
		return n;
	    if (n < admissibleNbClasses[i])
		return admissibleNbClasses[i];
	}
	return  n >  admissibleNbClasses[admissibleNbClasses.length-1] ?
	    admissibleNbClasses[admissibleNbClasses.length-1] : n;
    }

    public SpinnerModel getClassificationNumberModel() {
	nbClasses = getClosestAdmissibleNumberOfClasses(nbClasses);
	SpinnerModel m = new SpinnerListModel (getAdmissibleNbClasses());
	m.setValue(nbClasses);
	return m;
    }

    abstract protected Integer[] getAdmissibleNbClasses();
}
