// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.classification;

import java.util.ArrayList;
import java.util.Collections;
import java.lang.reflect.Constructor;
import java.text.NumberFormat;
import javax.swing.*;
import fr.unicaen.openmap.dataStorage.DataObject;
//import fr.unicaen.openmap.util.classification.gui.ClassificationPanel;
import fr.unicaen.openmap.util.Util;
/**
 * Not really a classification. Also not really continuous : 
 * in fact, the colorizer will use its colorscale "continuously" if its classificaiton is a ContinuousClassification
 * 
 */
public class ContinuousClassification extends CustomizedClassificationOfDouble {
    
    /**
     * @param data 
     * @param nbClasses the number of wished classes; some methods may decide to change its value.
     */
    public ContinuousClassification(ArrayList<Double> data, int nbClasses) {
	super(data, nbClasses);
    }


    protected void init() {
	if (data == null) return;
	if (initDone) // the classif has been customized
	    return;

	Collections.sort(data);

	// set breaks
	breaks = new double[nbClasses+1];
	breaks[0] = data.get(0); // first break is the min value;
	breaks[nbClasses] = data.get(data.size()-1); // last break is the max value

	initBreaks();

	initCountsFromBreaks();
    }

    private boolean initDone = false; // see Colorizer.setProperties
   public void setBreaks(String[] s) {
       initDone = true;
       super.setBreaks(s);
   }

    protected void initBreaks() {
	// System.out.println("> initBreaks  "+this);   
	double N = data.size();
	for (int i = 1; i < nbClasses-1; i++) {
	    int d = (int)Math.floor(i*N/nbClasses);
	    // the mean value of data at d and d+1
	    breaks[i] = (d+1) >= N ?  data.get(data.size()-1)
		: (data.get(d)+data.get(d+1))/2;
	}
	breaks[nbClasses-1]=breaks[nbClasses];
	// System.out.println("< initBreaks  "+this);
    }


    public void setNthBreak(int n, double v) {
	initDone = true;
	// should check index validity
        breaks[n] = v;
	// System.out.println("setNthBreak " + n + " " + v + " "+this);
    }


    static public class ClassRankAndRatio {
	private int rank;
	private double ratio;
	ClassRankAndRatio(int rank, double ratio) {
	    this.rank = rank;
	    this.ratio = ratio;
	}
	public int getRank() { return rank; }
	public double getRatio() { return ratio; }
    }

    /**
     * return the class rank of the value between 1 and nbClasses
     */
    public ClassRankAndRatio getClassRankAndRatio(Object o) {
	if (breaks == null)
	    return null;
	double d = Util.parseDouble(o);
	if (Double.isNaN(d))
	    return null;
	int c = getClass(d);
	if (c < 0)
	    return null;
	double ratio =  (d - breaks[c]) / (breaks[c+1] - breaks[c]);
	if (ratio < 0)
	    ratio = 0;
	else if (ratio >= 1)
	    ratio = 1.0;
	return new ClassRankAndRatio(c, ratio);
    }

}
