// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.classification;

import com.bbn.openmap.util.Debug;

import java.util.ArrayList;
import java.util.Collections;
import fr.unicaen.openmap.dataStorage.DataObject;
import java.util.HashMap;
import java.util.TreeSet;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;


/**
 * CustomizedClassification allows the user to specify the breaks.
 * Uses internally a CustomizedClassificationOfDouble or a CustomizedClassificationOfString
 * depending on the type of the classification it was initialized from
 */
public class Customized extends Classification implements CustomizedClassification {

    /** 
     * realClassif is instanceof CustomizedClassificationOfString or CustomizedClassificationOfDouble
     * depending on the type of the classif it is reinit from
     */
    private Classification realClassif;

    public Customized(ArrayList<Object> data, int nbClasses) {
	super(nbClasses);
    }

    public Customized(Classification classif, int nbClasses) {
	super(nbClasses);
	reInit(classif, nbClasses);
    }

    public SpinnerModel getClassificationNumberModel() {
	return realClassif == null 
	    ? new SpinnerNumberModel (nbClasses, nbClasses, nbClasses, 0)
	    : realClassif.getClassificationNumberModel();
    }

    public Classification getRealClassif() { 
	return realClassif; 
    }

    public void reInit(fr.unicaen.openmap.dataStorage.DataStorageIterator dsi, java.lang.String dataColumnName) {
	// System.out.println("#### Customized classif reInit(dsi, " + dataColumnName + ")");
    }
    
    public void reInit(Classification c, int nbClasses) {
	if (c == null)
	    return;
	if (c.getRealClassif() instanceof ClassificationOfString)
	    realClassif = new CustomizedClassificationOfString(null, nbClasses);
	else // c instanceof ClassificationOfDouble
	    realClassif = c instanceof DiscreteOfDouble
		? new CustomizedDiscreteOfDouble(null, nbClasses)
		: new CustomizedClassificationOfDouble(null, nbClasses);
	realClassif.reInit(c, nbClasses);
  } 
    
    public boolean hasNoData() {
	return realClassif == null || realClassif.hasNoData();
    }

    public void unsetData() {
	if (realClassif != null)
	    realClassif.unsetData();
    }

    protected void initCountsFromBreaks() {
	if (realClassif != null) {
	    realClassif.initCountsFromBreaks();
	    counts = realClassif.getCounts();
	} else 
	    counts = null;
    }
	
    public int getClass(Object d) {
	return realClassif == null ? -1 : realClassif.getClass(d);
    }

    public String toString() {
	return super.toString() + (realClassif == null ? " null" : realClassif.toString());
    }
    public String dump() {
	return realClassif == null ? "null" : realClassif.dump();
    }

    public String getClassText(int i) {
	return realClassif == null ? "" : realClassif.getClassText(i);	
    }
    public String[] getClassesTexts() {
	return realClassif == null ? null : realClassif.getClassesTexts();	
    }
    
    public String getBreaksAsString() {
	return realClassif == null ? "" : realClassif.getBreaksAsString();	
    }

    public void setNumberOfClasses(int nbClasses) {
	this.nbClasses = nbClasses;
	if (realClassif != null) 
	    realClassif.setNumberOfClasses(nbClasses);
    }

    protected void init() {
	if (realClassif != null) {
	    realClassif.init();
	    nbClasses = realClassif.nbClasses;
	}
    }

    public void setNthBreak(int n, double v) {
	if (!(realClassif instanceof CustomizedClassificationOfDouble)) {
	    Debug.error("Customized: realClassif is either null or not a ClassificationOfDouble");
	}
	((CustomizedClassificationOfDouble)realClassif).setNthBreak(n, v);
    }

    public String getClassName() {
	return realClassif == null ? "" : realClassif.getClassName();
    }
    
    public void setBreaks(String[] s) {
	if (realClassif == null)
	    realClassif = new CustomizedClassificationOfString(null, s.length-1);
	realClassif.setBreaks(s);
    }
}

