// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.classification;

import java.util.ArrayList;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;

/**
 * 
 */
public class CustomizedDiscreteOfDouble extends DiscreteOfDouble implements CustomizedClassification {

    public CustomizedDiscreteOfDouble(ArrayList<Double> data, int nbClasses) {
        super(data, nbClasses);
    }

    public CustomizedDiscreteOfDouble(ClassificationOfDouble c) {
        super(c.data, c.getNumberOfClasses());
        setBreaks(c.getBreaks());
    }

   public SpinnerModel getClassificationNumberModel() {
       return new SpinnerNumberModel (nbClasses, 1, Math.max(100, 2*nbClasses), 1);
    }

    public void reInit(ClassificationOfDouble c) {
        reInit(c.data, c.nbClasses);
        setBreaks(c.getBreaks());
    }
 
   public void reInit(fr.unicaen.openmap.dataStorage.DataStorageIterator dsi,java.lang.String dataColumnName) {
   }

    protected void init() {
	// init breaks, try to keep previous values
        if (breaks == null) {
            breaks = new double[getNumberOfClasses() + 1];
            for (double d : breaks) {
                d = 0.0;
            }
        } else {
	    double[] newBreaks = new double[getNumberOfClasses() + 1];
	    int i = 0;
	    for (; i < breaks.length && i < newBreaks.length -1 ; i++)
		newBreaks[i] = breaks[i];
	    for (; i < newBreaks.length; i++)
		newBreaks[i] = 0.0;
	    breaks = newBreaks;
	}
	// initCountsFromBreaks(); // no use of counts
    }

    protected void initBreaks() {
	try{ throw(new Exception("Should not be called")); } catch(Exception e) { e.printStackTrace(); }
    }

    public void setBreaks(double[] breaks) {
	if (breaks == null) {
	    return;
	}
	this.breaks = breaks;
        if (breaks.length == getNumberOfClasses() + 1)
	    return;
	// we need to complete the breaks array
        init();	
    }

    public void setNthBreak(int n, double v) {
        if (breaks == null || n >= breaks.length) {
            init();
        }
        // we try init only once!
        if (breaks == null || n >= breaks.length) {
            return;
        }
        breaks[n] = v;
        init();
    }

    @Override
    public void reInit(Classification c, int nbClasses) {
	super.reInit(c, nbClasses);
	c = c.getRealClassif();
	if (c instanceof DiscreteOfDouble)
	    setBreaks(((DiscreteOfDouble) c).getBreaks());		
    }
}
