// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.classification;

import java.util.ArrayList;
import java.util.Collections;
import fr.unicaen.openmap.dataStorage.DataObject;
import java.util.HashMap;
import java.util.TreeSet;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;

/**
 * Set a class per different values
 * Uses internally a DiscreteOfDouble or a DiscreteOfString classification
 * depending on the type of the first datat value.
 */
public class Discrete extends Classification implements DiscreteClassification {

    public Discrete(ArrayList<Object> data, int nbClasses) {
	super(nbClasses);
    }

    public SpinnerModel getClassificationNumberModel() {
	// System.out.println("### Discrete nb Classes = " + nbClasses);
	return new SpinnerNumberModel (nbClasses, nbClasses, nbClasses, 0);
    }

    /** 
     * realClassif is instanceof DiscreteOfDouble or DiscreteOfString depending on the type of the first data value
     */
    private Classification realClassif;

    public Classification getRealClassif() { 
	return realClassif; 
    }

    public void reInit(fr.unicaen.openmap.dataStorage.DataStorageIterator dsi, java.lang.String dataColumnName) {
	dsi.reset();
	if (! dsi.hasNext())
	    return;
	DataObject dataObject = dsi.next();
	Object value = dataObject.getData(dataColumnName);
	realClassif = value instanceof Double 
	    ? new DiscreteOfDouble(null, nbClasses)
	    :  new DiscreteOfString(null, nbClasses);
	dsi.reset();
	realClassif.reInit(dsi, dataColumnName);
	nbClasses = realClassif.nbClasses; 
    }

    public void reInit(Classification c, int nbClasses) {
	if (c instanceof ClassificationOfDouble) {
	} else {
	}
    } 
    
    public boolean hasNoData() {
	return realClassif == null || realClassif.hasNoData();
    }

    public void unsetData() {
	if (realClassif != null)
	    realClassif.unsetData();
    }

    protected void initCountsFromBreaks() {
	if (realClassif != null) {
	    realClassif.initCountsFromBreaks();
	    counts = realClassif.getCounts();
	} else 
	    counts = null;
    }
	
    public int getClass(Object d) {
	return realClassif == null ? 0 : realClassif.getClass(d);
    }

    public String toString() {
	return realClassif == null ? "null" : realClassif.toString();
    }
    public String dump() {
	return realClassif == null ? "null" : realClassif.dump();
    }

    public String getClassText(int i) {
	return realClassif == null ? "" : realClassif.getClassText(i);	
    }
    public String[] getClassesTexts() {
	return realClassif == null ? null : realClassif.getClassesTexts();	
    }
    
    public String getBreaksAsString() {
	return realClassif == null ? "" : realClassif.getBreaksAsString();	
    }

    protected void init() {
	if (realClassif != null) {
	    realClassif.init();
	    nbClasses = realClassif.nbClasses;
	}
    }
}
