// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.classification;

import java.util.ArrayList;
import java.util.Collections;
import fr.unicaen.openmap.dataStorage.DataObject;
import fr.unicaen.openmap.util.Util;
import java.util.HashMap;
import java.util.TreeSet;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;

/**
 * Set a class per different values
 */
public class DiscreteOfDouble extends ClassificationOfDouble implements DiscreteClassification {

    public DiscreteOfDouble(ArrayList<Double> data, int nbClasses) {
	super(data, nbClasses);
    }

    public SpinnerModel getClassificationNumberModel() {
	// System.out.println("### DiscreteOfDouble nb Classes = " + nbClasses);
	return new SpinnerNumberModel (nbClasses, 1, nbClasses, 1);
    }

    protected void initBreaks() {
	HashMap<Double, Integer> counts = new HashMap<Double, Integer>();
	Double value = null;
	int count = 0;
	for (Double s : data) {
	    if (! s.equals(value)) { // new value
		if (value != null) counts.put(value, count);
		value = s;
		count = 1;
	    } else
		count++;
	}
	if (value != null) counts.put(value, count);

	TreeSet<Double> keys = new TreeSet<Double>(counts.keySet());
	breaks = new double[keys.size()+1];
	int i = 0;
	for (Double s : keys) {
	    breaks[i++] = s;
	}
	this.counts = new int[keys.size()];
	i = 0;
	for (Double s : keys)
	    this.counts[i++] = counts.get(s);
	nbClasses = keys.size();
	// System.out.println("<== DiscreteOfDouble.init() => "+ this);
    }

    protected void initCountsFromBreaks() {
	// nothing to do, already done in initBreaks()
    }

    public int getClass(Object o) {
	if (breaks == null)
	    return 0;
	double d = Util.parseDouble(o);

	return getClass(d);
    }

    public int getClass(double d) {
	// may be we should use a dichotomy ?
	int i = 0;
	for (; i < nbClasses; i++)
	    if (d == breaks[i])
		return i;
	return i;
    }


    
   /** returns a text describing the ith class */
    public String getClassText(int i) {
	if (breaks != null && i < breaks.length)
	    return String.valueOf(breaks[i]);
	return "";
    }
}
