// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.classification;

import com.bbn.openmap.util.Debug;

import java.util.ArrayList;
import java.util.Collections;
import fr.unicaen.openmap.dataStorage.DataObject;
import java.util.HashMap;
import java.util.TreeSet;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;

/**
 * Set a class per different values
 */
public class DiscreteOfString extends ClassificationOfString implements DiscreteClassification {

    public DiscreteOfString(ArrayList<String> data, int nbClasses) {
	super(data, nbClasses);
    }

    public SpinnerModel getClassificationNumberModel() {
	// System.out.println("### DiscreteOfString nb Classes = " + nbClasses);
	return new SpinnerNumberModel (nbClasses, 1, nbClasses, 1);
    }

    public void reInit(ArrayList<String> data) {
	this.data = data;
	if (data != null)
	    init();
    }

    protected void init() {
	// System.out.println("==> DiscreteOfString.init " + data);
	if (data == null) return;
	Collections.sort(data);
	int N = data.size();
	if (N == 0) {
	    // Debug.error("[Classification] *** Error *** empty data");
	    return;
	}
	HashMap<String, Integer> counts = new HashMap<String, Integer>();
	String value = null;
	int count = 0;
	for (String s : data) {
	    if (! s.equals(value)) { // new value
		if (value != null) counts.put(value, count);
		value = s;
		count = 1;
	    } else
		count++;
	}
	if (value != null) counts.put(value, count);

	TreeSet<String> keys = new TreeSet<String>(counts.keySet());
	breaks = new String[keys.size()+1];
	int i = 0;
	for (String s : keys) {
	    breaks[i++] = s;
	}
	this.counts = new int[keys.size()];
	i = 0;
	for (String s : keys)
	    this.counts[i++] = counts.get(s);
	nbClasses = keys.size();
	// System.out.println("<== DiscreteOfString.init() => "+ this);
    }
    

   /** returns a text describing the ith class */
    public String getClassText(int i) {
	if (breaks != null && i < breaks.length)
	    return breaks[i];
	return "";
    }

    public int getClass(Object o) {
	if (breaks == null)
	    return 0;
	// in ClassificationOfString.reInit null strings are replaced by empty strings ("").
	String v = o != null ? o.toString() : "";
	int i = 0;
	for (; i < nbClasses; i++)
	    if (breaks[i].equals(v))
		return i;
	return -1;
    }

}
