// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.classification;

import java.util.ArrayList;
import java.util.Collections;

import javax.swing.SpinnerModel;
import javax.swing.SpinnerListModel;


/**
 * Nested means classif
 * This leed to a power of 2 number of classes 
 */
public class NestedMeans extends ClassificationWithContrainedNbOfClasses {

    private static Integer _admissibleNbClasses[] = {2, 4, 8, 16, 32};
    protected Integer[]  getAdmissibleNbClasses() {
	return _admissibleNbClasses;
    }

    public NestedMeans(ArrayList<Double> data, int nbClasses) {
	super(data, nbClasses);
    }
    
    private static double getMean(int imin, int imax, ArrayList<Double> data) {
	double sum = 0.0;
	for(int i = imin; i < imax; i++)
	    sum += data.get(i);	
	return imax > imin ? sum/(imax-imin) : sum;
    }

    private static int getIndexInSortedArray(double v, int imin, int imax, ArrayList<Double> data) {
	// dichotomic search
	int low = imin;
	int hight = imax - 1;
	while (low < hight) {
	    int m = (low + hight) / 2;
	    if (data.get(m) < v)
		low = m + 1;
	    else 
		hight = m;
	}
	return hight;
    }
    
    protected void initBreaks() {
	computeNestedMeans(0, nbClasses, 0, data.size());
    }

    /**
     * params: 
     *    lowbreak, hightbreak are indexes in the breaks array
     *    imin, imax are indexes in the data array
     * each call computes breaks[(hightbreak+lowbreak)/2] as the mean value of data[imin]..data[imax]
     * and do it recursively for the two half subsets of data
     */
    private void computeNestedMeans(int lowbreak, int hightbreak, int imin, int imax) {
	if (hightbreak - lowbreak <= 1)
	    return;
	double mean = getMean(imin, imax, data);
	int i = getIndexInSortedArray(mean, imin, imax, data);
	int b = (hightbreak + lowbreak)/2;
	breaks[b] = mean;
	computeNestedMeans(lowbreak, b, imin, i);
	computeNestedMeans(b, hightbreak, i, imax);
    }

}

