// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.classification;

import java.util.ArrayList;
import java.util.Collections;

import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;

/**
 * breaks are equal to (mean + (2p+1)standard_deviation/2).
 * This leed to an odd number of classes
 */
public class Standardization extends ClassificationWithContrainedNbOfClasses {
    
    private static Integer _admissibleNbClasses[] = {1, 3, 5, 7, 9, 11, 13, 15};
    protected Integer[]  getAdmissibleNbClasses() {
	return _admissibleNbClasses;
    }


    public Standardization(ArrayList<Double> data, int nbClasses) {
	super(data, nbClasses);
    }
     
    protected void initBreaks() {

	// compute mean and standart deviation
	// should use a more numerically robust algorithm 
	double sum = 0, sum2 = 0;
	for(double d : data) {
	    sum += d;
	    sum2 += d*d;
	}
	int N = data.size();
	double mean = sum / N;
	double deviation = Math.sqrt(sum2/N-mean);
	//	System.out.println("Standardization: mean="+mean+" standard deviation="+deviation);

	// breaks are equal to (mean + (2p+1)deviation/2)
	for (int i = 1; i <= nbClasses-1; i++) {
	    breaks[i] = mean + (2*(i-1-(nbClasses-1)/2)+1)*deviation/2;
	    // ###Problem: min and max may resp. greater or lower than mean breaks[0] or breaks[nbClasses]
	    if (breaks[i] > breaks[nbClasses])
		breaks[i] = breaks[nbClasses];
	    else if (breaks[i] < breaks[0])
		breaks[i] = breaks[0];
	}
    }
    
}

