// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.find;

import java.awt.Point;
import java.awt.geom.Point2D;

import com.bbn.openmap.proj.coords.LatLonPoint;
import com.bbn.openmap.proj.Projection;


//This class should provide any methods to transform window distance in lat/lon or vice-versa
//author : Yannick Lange, ylange@etu.info.unicaen.fr, DESS NAPI, juin-septembre 2003.
public class Distance
{
    /**
     * Compute pixel distance between two points given in lat/lon behind given projection.
     * @param pt1 first point
     * @param pt2 second point
     * @param proj Projection to use to compute distance
     * @return distance in pixels.
     */
    public static int computeDistanceInPixel(LatLonPoint pt1, LatLonPoint pt2, Projection proj)
    {
	Point2D _pt1 = proj.forward(pt1);
	Point2D _pt2 = proj.forward(pt2);
	double[] p1 = {_pt1.getX(), _pt1.getY()};
	double[] p2 = {_pt2.getX(), _pt2.getY()};	

	double dx = p2[0] - p1[0];
	double dy = p2[1] - p1[1];
	int pix = (int)(Math.sqrt(dx*dx+dy*dy) * 3/2 );//x3/2 : it seems this ratio is necessary to keep proportions with any projection, but I don't know why !?

	return pix;
    }



}
