// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.gui;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;
import fr.unicaen.openmap.gui.ThemaMapFileChooserManager;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

/**
 *
 * @author jchauvea
 */
public class ThemaMapJFileChooser extends JDialog {

    private JFileChooser fileChooser;
    private JTextField urlField;
    private boolean local = true;
    private JButton bOk, bCancel;
    private boolean ok = false;
    private String data_load_path;

    protected I18n i18n = Environment.getI18n();

    public ThemaMapJFileChooser(String path, String data_load_path) {
        super();
	this.data_load_path = data_load_path;
        fileChooser = new JFileChooser(path);
        JPanel pLocal = new JPanel(new BorderLayout());
        Box localBox = Box.createVerticalBox();
        JPanel pUrl = new JPanel();
        JPanel pRadios = new JPanel();
        final JRadioButton rb1 = new JRadioButton(i18n.get(this,"local","Local file"));
        final JRadioButton rb2 = new JRadioButton(i18n.get(this,"remote","Remote file (URL)"));
        ButtonGroup bg = new ButtonGroup();
        bg.add(rb1);
        bg.add(rb2);
        pRadios.add(rb1);
        pRadios.add(rb2);
        final JPanel pMain = new JPanel();
        pMain.setLayout(new CardLayout());
        Component cps[] = fileChooser.getComponents();
        JPanel pTop = new JPanel();
        pTop.add(cps[0]);
        pLocal.add(pTop, BorderLayout.NORTH);
        pLocal.add(localBox, BorderLayout.CENTER);
        localBox.add(cps[1]);
        localBox.add(cps[2]);
        localBox.add(cps[3]);
        JComponent cp3 = (JComponent)cps[3];
	// Hack: remove <open, cancel> panel if it exists (doestn't exist on MacOS)
	if (cp3.getComponents().length > 3) 
	    cp3.remove(cp3.getComponent(3));
//        for(Component c : cp3.getComponents()){
//            JComponent cc = (JComponent)c;
//            System.out.println(cc.getComponents().length);
//        }
        JPanel panBottom = new JPanel(new BorderLayout());
        JPanel panButtons = new JPanel();
        panBottom.add(panButtons, BorderLayout.EAST);
        bOk = new JButton(i18n.get(this,"load","Load"));
        bCancel = new JButton(i18n.get(this,"cancel","Cancel"));
        panButtons.add(bOk);
        panButtons.add(bCancel);        

        pUrl.add(new JLabel(i18n.get(this,"resource_url","Resource file URL")));
        urlField = new JTextField("http://");
        ComponentSizer.fixComponentSize(urlField, new Dimension(200, 20));
        pUrl.add(urlField);

        pMain.add(pLocal, "local");
        pMain.add(pUrl, "url");
        JPanel p = new JPanel(new BorderLayout());
        p.add(pRadios, BorderLayout.NORTH);
        p.add(pMain, BorderLayout.CENTER);
        p.add(panBottom, BorderLayout.SOUTH);
        getContentPane().add(p);

        rb1.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent arg0) {
                CardLayout cl = (CardLayout) (pMain.getLayout());
                if (rb1.isSelected()) {
                    cl.show(pMain, "local");
                    local = true;
                } else {
                    cl.show(pMain, "url");
                    local = false;
                }                
            }
        });

        bOk.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                ok = true;
                setVisible(false);
            }
        });
        bCancel.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                ok = false;
                setVisible(false);
            }
        });
        rb1.setSelected(true);

        p.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        setSize(500, 350);
        setModal(true);
    }

    public void setFileFilter(FileFilter ff){
        fileChooser.setFileFilter(ff);
    }

    public String getResourceString() {
        if (local) {
             Environment.getInstance().put(data_load_path,
                    fileChooser.getSelectedFile().getParentFile().getAbsolutePath());
             return fileChooser.getSelectedFile().getPath();
        } else {
            return urlField.getText();
        }
    }
    
    public String showOpenDialog(String title, FileFilter ff) {
        setTitle(title);
        setFileFilter(ff);
        setVisible(true);
        if (ok) {
            //System.out.println(getResourceString());
            return getResourceString();
        }
	return null;
    }

    public void setFileSelectionMode(int mode){
        fileChooser.setFileSelectionMode(mode);
    }
}
