// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.propertyEditor;

import com.bbn.openmap.Environment;

/**
 * Can be used with FileTypePropertyEditor.
 * @see FileTypePropertyEditor
 * @see fr.unicaen.openmap.dataStorage.CSVFileDataStorage#getPropertyInfo(java.util.Properties list)
 * @see fr.unicaen.openmap.dataStorage.CSVFileDataStorage#getProperties(java.lang.String prefix, java.util.Properties properties)
 * @see fr.unicaen.openmap.dataStorage.CSVFileDataStorage#setProperties(java.lang.String prefix, java.util.Properties properties) 
 */
public class FileTypeDescription {



    public static final String CSV_PATTERN = "(.*)\\.(csv|csv.gz|csv.zip)";
    public static final String DBF_PATTERN = ".*\\.dbf$";
    public static final String SHP_PATTERN = ".*\\.shp$";
    public static final String XML_PATTERN = ".*\\.xml$";
    public static final String KML_PATTERN = ".*\\.kml$";
    public static final String XSD_PATTERN = ".*\\.xsd$";
    public static final String SQL_PATTERN = ".*\\.sql$";
    public static final String MIF_PATTERN = ".(.*)\\.(MIF|mif)";
    public static final String GML_PATTERN = "(.*)\\.(gml|gmz)";
    public static final String IMG_PATTERN = "(.*)\\.(png|jpg|gif|bmp)";
	public static final String  NC_PATTERN = "(.*)\\.nc";
    public static final String PROPERTIES_PATTERN = ".*\\.properties$";
    
    public static final String CSV_DESCRIPTION = Environment.getI18n().get(FileTypeDescription.class, "csv", "CSV File");
    public static final String DBF_DESCRIPTION = Environment.getI18n().get(FileTypeDescription.class, "dbf", "DBF File");
    public static final String SHP_DESCRIPTION = Environment.getI18n().get(FileTypeDescription.class, "shp", "SHP File");
    public static final String XML_DESCRIPTION = Environment.getI18n().get(FileTypeDescription.class, "xml", "XML File");
    public static final String GML_DESCRIPTION = Environment.getI18n().get(FileTypeDescription.class, "gml", "GML File");
    public static final String KML_DESCRIPTION = Environment.getI18n().get(FileTypeDescription.class, "kml", "KML File");
    public static final String XSD_DESCRIPTION = Environment.getI18n().get(FileTypeDescription.class, "xsd", "XSD File");
    public static final String MIF_DESCRIPTION = Environment.getI18n().get(FileTypeDescription.class, "mif", "MIF File");
    public static final String IMG_DESCRIPTION = Environment.getI18n().get(FileTypeDescription.class, "img", "Image File");
	public static final String  NC_DESCRIPTION = Environment.getI18n().get(FileTypeDescription.class, "nc",  "NetCDF File");
    public static final String SQL_DESCRIPTION = Environment.getI18n().get(FileTypeDescription.class, "sql", "SQL File");
    public static final String PROPERTIES_DESCRIPTION = Environment.getI18n().get(FileTypeDescription.class, "prop", "Properties File");
    

    
    /**
     * Default CSV FileTypeDescription.
     */
    public static final FileTypeDescription CSV = new FileTypeDescription(CSV_DESCRIPTION, "", CSV_PATTERN);

    /**
     * Default DBF FileTypeDescription.
     */
    public static final FileTypeDescription DBF = new FileTypeDescription(DBF_DESCRIPTION, "", DBF_PATTERN);

    /**
     * Default SHP FileTypeDescription.
     */
    public static final FileTypeDescription SHP = new FileTypeDescription(SHP_DESCRIPTION, "", SHP_PATTERN);

    /**
     * Default MIF FileTypeDescription.
     */
    public static final FileTypeDescription MIF = new FileTypeDescription(MIF_DESCRIPTION, "", MIF_PATTERN);

    /**
     * Default XML FileTypeDescription.
     */
    public static final FileTypeDescription XML = new FileTypeDescription(XML_DESCRIPTION, "", XML_PATTERN);

    /**
     * Default SQL FileTypeDescription.
     */
    public static final FileTypeDescription SQL = new FileTypeDescription(SQL_DESCRIPTION, "", SQL_PATTERN);
 
    private String description, path, pattern;

    public FileTypeDescription(String description, String path, String pattern) {
	this.description = description;
	this.path = path;
	this.pattern = pattern;
    }

    public String getDescription() { return description; }
    public String getPath() { return path; }
    public String getPattern() { return pattern; }

    /*    public String toString() {
	return "description : " + description + "\npath : " + path + "\npattern : " + pattern;
	}*/
}
