// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.propertyEditor;

import java.awt.Component;
import java.beans.PropertyEditorSupport;

import com.bbn.openmap.util.PropUtils;

import fr.unicaen.openmap.event.GUIChangeEvent;
import fr.unicaen.openmap.event.GUIChangeListener;
import fr.unicaen.openmap.gui.FileNamePanel;

/**
 * Common class for a FilePropertyEditor using FileNamePanel abilities.
 */
public class FileTypePropertyEditor extends PropertyEditorSupport implements GUIChangeListener {
    private FileNamePanel panel;



    public FileTypePropertyEditor() {
	panel = new FileNamePanel("nothing", "", "", this);
    }

    /*
    public FileTypePropertyEditor(GUIChangeListener changeListener) {
	panel = new FileNamePanel("nothing", "", "", changeListener);
    }
    */



    public boolean supportsCustomEditor() {
	return true;
    }

    public Component getCustomEditor() {
	//	if (panel != null)
	//	return new JPanel();

	return panel;
    }

    /**
     * Returns current file name.
     * @return String representation for file name
     */
    public String getAsText() {
	//filename
	return panel.getFilePath();
    }
 
    /**
     * Set (or change) the object that is to be edited. If the object is a FileTypeDescription, the FileNamePanel can be refreshed.
     * @param value The new target object to be edited.
     */
    public void setValue(Object value) {

	if (value == null)
	    return;

	if (value instanceof FileTypeDescription) {
	    FileTypeDescription fileDescription = (FileTypeDescription) value;
	    panel.setDescription(fileDescription.getDescription());
	    panel.setPattern(fileDescription.getPattern());
	    panel.setFilePath(fileDescription.getPath());
	}
    }

    //should really implements GUIChangeListener ?
    public void reloadFromGUI(GUIChangeEvent event) {
	//used as PropertyEditor, I don't see now...
    }
}
