// <header>
/**
 *  ThemaMap : The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008-2012 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.propertyEditor;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyEditorSupport;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Thematic;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.dataStorage.SharedDataStorage;

public class LayerAndThematicNamePropertyEditor extends PropertyEditorSupport
{
	private JComboBox layer = new JComboBox();
	private JComboBox thematic = new JComboBox();

	public boolean supportsCustomEditor()
	{
		return true;
	}

	public Component getCustomEditor()
	{
		Box box = Box.createHorizontalBox();

		// Fill layer ComboBox with layers names
		for( ThematicDataDisplayLayer t : SharedDataStorage.getThematicDataDisplayLayers() )
		{
			layer.addItem( t.getName().replaceAll("[ \\t]", "_") );
		}

		layer.addActionListener( new ActionListener()
				{
					public void	actionPerformed(ActionEvent e)
					{
						thematic.removeAllItems();
						ThematicDataDisplayLayer l = SharedDataStorage.getLayerByName(""+layer.getSelectedItem());

						for( Thematic t : l.getThematicArray() )
						{
							thematic.addItem(t.getName().replaceAll("[ \\t]", "_"));
						}
					}
				}
		);

		
		// Fill thematic ComboBox with thematic names
		ThematicDataDisplayLayer l = SharedDataStorage.getLayerByName(""+layer.getSelectedItem());

		for( Thematic t : l.getThematicArray() )
		{
			thematic.addItem(t.getName().replaceAll("[ \\t]", "_"));
		}

		box.add(layer);
		box.add(thematic);
		return box;
	}

	public void setValue(Object string)
	{
		if (!(string instanceof String))
		{
			return;
		}

		String layerAndThematic[] = ((String) string).split(",");

		if( layerAndThematic.length == 2 )
		{
			layer.setSelectedItem(layerAndThematic[0]);
			thematic.setSelectedItem(layerAndThematic[1]);
		}
	}

	public String getAsText()
	{
		return ""+layer.getSelectedItem()+","+thematic.getSelectedItem();
	}
}
